@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: conlay.c
@@ -511,8 +512,8 @@ BOOLEAN cla_modifynodeinst(NODEINST *ni, INTBIG deltalx, INTBIG deltaly, INTBIG
 #ifdef CLA_DEBUG
 	if (cla_conlaydebug)
 		ttyputmsg(M_("Change node %s by X(%s %s) Y(%s %s) r=%ld t=%ld"),
-			describenodeinst(ni), latoa(deltalx), latoa(deltahx), latoa(deltaly),
-				latoa(deltahy), dangle, dtrans);
+			describenodeinst(ni), latoa(deltalx, 0), latoa(deltahx, 0), latoa(deltaly, 0),
+				latoa(deltahy, 0), dangle, dtrans);
 #endif
 
 	/* mark that this nodeinst has changed */
@@ -622,7 +623,7 @@ void cla_modwithin(NODEINST *ni, INTBIG dangle, INTBIG dtrans)
 #ifdef	CLA_DEBUG
 		if (cla_conlaydebug)
 			ttyputmsg(M_("Internal arc %s moves 0:%s,%s  1:%s,%s"),
-				describearcinst(ai), latoa(nox), latoa(noy), latoa(onox), latoa(onoy));
+				describearcinst(ai), latoa(nox, 0), latoa(noy, 0), latoa(onox, 0), latoa(onoy, 0));
 #endif
 		/* move the arcinst */
 		cla_domovearcinst(ai, nox, noy, onox, onoy, 0);
@@ -725,19 +726,24 @@ BOOLEAN cla_modrigid(NODEINST *ni, INTBIG dangle, INTBIG dtrans)
 #ifdef	CLA_DEBUG
 		if (cla_conlaydebug)
 			ttyputmsg(M_("Other end of arc moves to (%s,%s)"),
-				latoa(ax[thatend]), latoa(ay[thatend]));
+				latoa(ax[thatend], 0), latoa(ay[thatend], 0));
 #endif
 
 		/* see if other nodeinst has changed */
 		locked = FALSE;
-		if (ono->changed == cla_changeclock) locked = TRUE;
-		if (ono->proto->primindex == 0)
+		if (ono->changed == cla_changeclock) locked = TRUE; else
 		{
-			if ((ono->parent->userbits&NILOCKED) != 0) locked = TRUE;
-		} else
-		{
-			if ((us_useroptions&NOPRIMCHANGES) != 0 &&
-				(ono->proto->userbits&LOCKEDPRIM) != 0) locked = TRUE;
+			if ((ono->userbits&NILOCKED) != 0) locked = TRUE; else
+			{
+				if (ono->proto->primindex == 0)
+				{
+					if ((ono->parent->userbits&NPILOCKED) != 0) locked = TRUE;
+				} else
+				{
+					if ((us_useroptions&NOPRIMCHANGES) != 0 &&
+						(ono->proto->userbits&LOCKEDPRIM) != 0) locked = TRUE;
+				}
+			}
 		}
 		if (!locked)
 		{
@@ -771,7 +777,7 @@ BOOLEAN cla_modrigid(NODEINST *ni, INTBIG dangle, INTBIG dtrans)
 #ifdef	CLA_DEBUG
 		if (cla_conlaydebug)
 			ttyputmsg(M_("Now arc runs from (%s,%s) to (%s,%s)"),
-				latoa(ax[0]), latoa(ay[0]), latoa(ax[1]), latoa(ay[1]));
+				latoa(ax[0], 0), latoa(ay[0], 0), latoa(ax[1], 0), latoa(ay[1], 0));
 #endif
 		cla_domovearcinst(ai, ax[0],ay[0], ax[1],ay[1], 0);
 	}
@@ -928,14 +934,19 @@ BOOLEAN cla_modflex(NODEINST *ni, INTBIG dangle, INTBIG dtrans)
 
 		/* see if other nodeinst has changed */
 		mangle = 1;
-		if ((ai->userbits&FIXANG) == 0) mangle = 0;
-		if (ono->proto->primindex == 0)
+		if ((ai->userbits&FIXANG) == 0) mangle = 0; else
 		{
-			if ((ono->parent->userbits&NILOCKED) != 0) mangle = 0;
-		} else
-		{
-			if ((us_useroptions&NOPRIMCHANGES) != 0 &&
-				(ono->proto->userbits&LOCKEDPRIM) != 0) mangle = 0;
+			if ((ono->userbits&NILOCKED) != 0) mangle = 0; else
+			{
+				if (ono->proto->primindex == 0)
+				{
+					if ((ono->parent->userbits&NPILOCKED) != 0) mangle = 0;
+				} else
+				{
+					if ((us_useroptions&NOPRIMCHANGES) != 0 &&
+						(ono->proto->userbits&LOCKEDPRIM) != 0) mangle = 0;
+				}
+			}
 		}
 		if (mangle != 0)
 		{
@@ -968,7 +979,7 @@ BOOLEAN cla_modflex(NODEINST *ni, INTBIG dangle, INTBIG dtrans)
 #ifdef	CLA_DEBUG
 					if (cla_conlaydebug)
 						ttyputmsg(M_("  Moving other end of arc to (%s,%s), other node by (%s,0)"),
-							latoa(ax[thatend]), latoa(ay[thatend]), latoa(dx-odx));
+							latoa(ax[thatend], 0), latoa(ay[thatend], 0), latoa(dx-odx, 0));
 #endif
 					if (dx != odx)
 					{
@@ -1000,7 +1011,7 @@ BOOLEAN cla_modflex(NODEINST *ni, INTBIG dangle, INTBIG dtrans)
 #ifdef	CLA_DEBUG
 				if (cla_conlaydebug)
 					ttyputmsg(M_("  Moving other end of arc to (%s,%s), other node by (0,%s)"),
-						latoa(ax[thatend]), latoa(ay[thatend]), latoa(dy-ody));
+						latoa(ax[thatend], 0), latoa(ay[thatend], 0), latoa(dy-ody, 0));
 #endif
 				if (dy != ody)
 				{
@@ -1038,7 +1049,7 @@ BOOLEAN cla_modflex(NODEINST *ni, INTBIG dangle, INTBIG dtrans)
 #ifdef	CLA_DEBUG
 		if (cla_conlaydebug)
 			ttyputmsg(M_("  ! Moving arc %s to (%s,%s)-(%s,%s)"), describearcinst(ai),
-				latoa(ax[0]), latoa(ay[0]), latoa(ax[1]), latoa(ay[1]));
+				latoa(ax[0], 0), latoa(ay[0], 0), latoa(ax[1], 0), latoa(ay[1], 0));
 #endif
 		cla_domovearcinst(ai, ax[0],ay[0], ax[1],ay[1], 1);
 	}
@@ -1187,9 +1198,9 @@ void cla_updatearc(ARCINST *ai, INTBIG fx, INTBIG fy, INTBIG tx, INTBIG ty, INTB
 #ifdef CLA_DEBUG
 	if (cla_conlaydebug)
 		ttyputmsg(M_("Change arc %s from (%s,%s)-(%s,%s) to (%s,%s)-(%s,%s)"),
-			describearcinst(ai), latoa(ai->end[0].xpos), latoa(ai->end[0].ypos),
-				latoa(ai->end[1].xpos), latoa(ai->end[1].ypos),
-					latoa(fx), latoa(fy), latoa(tx), latoa(ty));
+			describearcinst(ai), latoa(ai->end[0].xpos, 0), latoa(ai->end[0].ypos, 0),
+				latoa(ai->end[1].xpos, 0), latoa(ai->end[1].ypos, 0),
+					latoa(fx, 0), latoa(fy, 0), latoa(tx, 0), latoa(ty, 0));
 #endif
 	ai->end[0].xpos = fx;
 	ai->end[0].ypos = fy;
@@ -1243,7 +1254,7 @@ void cla_domovearcinst(ARCINST *ai, INTBIG fx, INTBIG fy, INTBIG tx, INTBIG ty,
 	if (cla_conlaydebug)
 	{
 		ttyputmsg(M_("Jogging arc %s (0%o) to run from (%s,%s) to (%s,%s)"),
-			describearcinst(ai), ai, latoa(fx), latoa(fy), latoa(tx), latoa(ty));
+			describearcinst(ai), ai, latoa(fx, 0), latoa(fy, 0), latoa(tx, 0), latoa(ty, 0));
 	}
 #endif
 
@@ -1354,8 +1365,8 @@ void cla_adjustmatrix(NODEINST *ni, PORTPROTO *pp, XARRAY trans)
 		if (cla_conlaydebug)
 		{
 			ttyputmsg(M_("Node %s port %s was at (%s,%s), is at (%s,%s)"), describenodeinst(ni),
-				pp->protoname, latoa(onox), latoa(onoy), latoa(dx), latoa(dy));
-			ttyputmsg(M_("    Node moved (%s,%s)"), latoa(ox), latoa(oy));
+				pp->protoname, latoa(onox, 0), latoa(onoy, 0), latoa(dx, 0), latoa(dy, 0));
+			ttyputmsg(M_("    Node moved (%s,%s)"), latoa(ox, 0), latoa(oy, 0));
 		}
 #endif
 		trans[2][0] = dx - onox + ox;   trans[2][1] = dy - onoy + oy;
