@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: conlingtt.c
@@ -882,7 +883,7 @@ void cli_replacename(char *oldname, char *newname)
 				{
 					if ((dec->flag&LAYERVALID) != 0) addtoinfstr(infstr, ','); else
 						addtoinfstr(infstr, '[');
-					(void)sprintf(line, "width=%s", latoa(dec->width));
+					(void)sprintf(line, "width=%s", latoa(dec->width, 0));
 					addstringtoinfstr(infstr, line);
 				}
 				if ((dec->flag&ATTRVALID) != 0)
@@ -918,7 +919,7 @@ void cli_replacename(char *oldname, char *newname)
 			/* decide whether to print the arc offset */
 			if ((dec->flag&OFFSETVALID) != 0)
 			{
-				(void)sprintf(line, " [%s,%s]", latoa(dec->xoff), latoa(dec->yoff));
+				(void)sprintf(line, " [%s,%s]", latoa(dec->xoff, 0), latoa(dec->yoff, 0));
 				addstringtoinfstr(infstr, line);
 			}
 			addstringtoinfstr(infstr, " to ");
@@ -1019,7 +1020,7 @@ char *cli_makearcline(ARCINST *ai)
 	{
 		if (!added) addtoinfstr(infstr, '['); else addtoinfstr(infstr, ',');
 		addstringtoinfstr(infstr, "width=");
-		addstringtoinfstr(infstr, latoa(ai->width));
+		addstringtoinfstr(infstr, latoa(ai->width, 0));
 		added = TRUE;
 	}
 	if (ai->numvar != 0) for(i=0; i<ai->numvar; i++)
@@ -1057,22 +1058,22 @@ char *cli_makearcline(ARCINST *ai)
 				case CLLEFT:
 					xcons++;
 					conx = -conptr->value * lambda / WHOLE;
-					(void)sprintf(line, " left %s", latoa(-conx));
+					(void)sprintf(line, " left %s", latoa(-conx, 0));
 					break;
 				case CLRIGHT:
 					xcons++;
 					conx = conptr->value * lambda / WHOLE;
-					(void)sprintf(line, " right %s", latoa(conx));
+					(void)sprintf(line, " right %s", latoa(conx, 0));
 					break;
 				case CLDOWN:
 					ycons++;
 					cony = -conptr->value * lambda / WHOLE;
-					(void)sprintf(line, " down %s", latoa(-cony));
+					(void)sprintf(line, " down %s", latoa(-cony, 0));
 					break;
 				case CLUP:
 					ycons++;
 					cony = conptr->value * lambda / WHOLE;
-					(void)sprintf(line, " up %s", latoa(cony));
+					(void)sprintf(line, " up %s", latoa(cony, 0));
 					break;
 			}
 			addstringtoinfstr(infstr, line);
@@ -1089,8 +1090,8 @@ char *cli_makearcline(ARCINST *ai)
 		doit = TRUE;
 	if (doit)
 	{
-		(void)sprintf(line, " [%s,%s]", latoa(ai->end[1].xpos - ai->end[0].xpos),
-			latoa(ai->end[1].ypos - ai->end[0].ypos));
+		(void)sprintf(line, " [%s,%s]", latoa(ai->end[1].xpos - ai->end[0].xpos, 0),
+			latoa(ai->end[1].ypos - ai->end[0].ypos, 0));
 		addstringtoinfstr(infstr, line);
 	}
 	addstringtoinfstr(infstr, " to ");
@@ -1248,8 +1249,8 @@ INTBIG cli_addcomponentinfo(void *infstr, NODEINST *ni)
 		ni->highy-ni->lowy != ni->proto->highy-ni->proto->lowy)
 	{
 		nodesizeoffset(ni, &plx, &ply, &phx, &phy);
-		(void)sprintf(line, "[size=(%s,%s)", latoa(ni->highx-ni->lowx-plx-phx),
-			latoa(ni->highy-ni->lowy-ply-phy));
+		(void)sprintf(line, "[size=(%s,%s)", latoa(ni->highx-ni->lowx-plx-phx, 0),
+			latoa(ni->highy-ni->lowy-ply-phy, 0));
 		addstringtoinfstr(infstr, line);
 		added += strlen(line);
 	}
@@ -1272,8 +1273,8 @@ INTBIG cli_addcomponentinfo(void *infstr, NODEINST *ni)
 	if (ni->firstportarcinst == NOPORTARCINST)
 	{
 		if (added != 0) addtoinfstr(infstr, ','); else addtoinfstr(infstr, '[');
-		(void)sprintf(line, "location=(%s,%s)", latoa((ni->highx+ni->lowx) / 2),
-			latoa((ni->highy+ni->lowy) / 2));
+		(void)sprintf(line, "location=(%s,%s)", latoa((ni->highx+ni->lowx) / 2, 0),
+			latoa((ni->highy+ni->lowy) / 2, 0));
 		addstringtoinfstr(infstr, line);
 		added += strlen(line) + 1;
 	}
@@ -1313,7 +1314,7 @@ void cli_addcomponentdata(void *infstr, COMPONENT *compo)
 		addtoinfstr(infstr, '[');
 		if ((compo->flag&COMPSIZE) != 0)
 		{
-			(void)sprintf(line, "size=(%s,%s)", latoa(compo->sizex), latoa(compo->sizey));
+			(void)sprintf(line, "size=(%s,%s)", latoa(compo->sizex, 0), latoa(compo->sizey, 0));
 			addstringtoinfstr(infstr, line);
 		}
 		if ((compo->flag&COMPROT) != 0)
@@ -1326,7 +1327,7 @@ void cli_addcomponentdata(void *infstr, COMPONENT *compo)
 		if ((compo->flag&COMPLOC) != 0)
 		{
 			if ((compo->flag&(COMPSIZE|COMPROT)) != 0) addtoinfstr(infstr, ',');
-			(void)sprintf(line, "location=(%s,%s)", latoa(compo->locx), latoa(compo->locy));
+			(void)sprintf(line, "location=(%s,%s)", latoa(compo->locx, 0), latoa(compo->locy, 0));
 			addstringtoinfstr(infstr, line);
 		}
 		if ((compo->flag&COMPATTR) != 0)
