@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: conlinprs.c
@@ -375,7 +376,7 @@ BOOLEAN cli_doparseconn(char *str, BOOLEAN quiet, CONNECTION *dcl)
 					if (!quiet) ttyputerr(M_("Missing width in '%s'"), str);
 					return(TRUE);
 				}
-				dcl->width = atola(st);
+				dcl->width = atola(st, 0);
 				dcl->flag |= WIDTHVALID;
 			} else
 			{
@@ -443,7 +444,7 @@ BOOLEAN cli_doparseconn(char *str, BOOLEAN quiet, CONNECTION *dcl)
 			}
 			st = getkeyword(&pt, " \t,");
 			if (st == NOSTRING) return(TRUE);
-			dcl->xoff = atola(st);
+			dcl->xoff = atola(st, 0);
 			dcl->flag |= OFFSETVALID;
 			if (tonextchar(&pt) != ',')
 			{
@@ -451,7 +452,7 @@ BOOLEAN cli_doparseconn(char *str, BOOLEAN quiet, CONNECTION *dcl)
 				return(TRUE);
 			}
 			st = getkeyword(&pt, " \t]");
-			dcl->yoff = atola(st);
+			dcl->yoff = atola(st, 0);
 			if (tonextchar(&pt) != ']')
 			{
 				if (!quiet) ttyputerr(M_("Missing ']' in arc extent '%s'"), str);
@@ -739,7 +740,7 @@ BOOLEAN cli_parsetwovalue(INTBIG *x, INTBIG *y, char *name, char **pt, BOOLEAN q
 	}
 	st = getkeyword(pt, " \t,)]");
 	if (st == NOSTRING) return(TRUE);
-	*x = atola(st);
+	*x = atola(st, 0);
 	if (tonextchar(pt) != ',')
 	{
 		if (!quiet) ttyputerr(M_("Missing ',' in %s option"), name);
@@ -747,7 +748,7 @@ BOOLEAN cli_parsetwovalue(INTBIG *x, INTBIG *y, char *name, char **pt, BOOLEAN q
 	}
 	st = getkeyword(pt, " \t)]");
 	if (st == NOSTRING) return(TRUE);
-	*y = atola(st);
+	*y = atola(st, 0);
 	if (tonextchar(pt) != ')')
 	{
 		if (!quiet) ttyputerr(M_("Missing ')' in size option"));
