@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: aidtable.c
@@ -257,7 +258,7 @@ TOOL el_tools[] =
 	{"routing", TOOLINCREMENTAL | TOOLSYNTHESIS, 0, &ro_routerp, NOCLUSTER,
 	ro_init,               ro_done,              ro_set,				/* init/done/set */
 	0,                     0,                    ro_slice,				/* request/examine/slice */
-	ro_startbatch,         0,											/* startbatch/endbatch */
+	ro_startbatch,         ro_endbatch,									/* startbatch/endbatch */
 	0,                     0,											/* startobject/endobject */
 	ro_modifynodeinst,     0,                    ro_modifyarcinst,		/* modnode/modnodes/modarc */
 	ro_modifyportproto,    0,                    0,						/* modport/modnodeproto/moddescript */
@@ -366,7 +367,7 @@ TOOL el_tools[] =
 	/* design-rule checker */
 	{"drc", TOOLON | TOOLINCREMENTAL | TOOLANALYSIS, 0, &dr_drcp, NOCLUSTER,
 	dr_init,               dr_done,              dr_set,				/* init/done/set */
-	0,                     dr_examinenodeproto,  dr_slice,				/* request/examine/slice */
+	dr_request,            dr_examinenodeproto,  dr_slice,				/* request/examine/slice */
 	0,                     0,											/* startbatch/endbatch */
 	0,                     0,											/* startobject/endobject */
 	dr_modifynodeinst,     0,                    dr_modifyarcinst,		/* modnode/modnodes/modarc */
