@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: data.c
@@ -44,6 +45,7 @@ INTBIG       el_facet_message_key;		/* key for "FACET_message" */
 INTBIG       el_schematic_page_size_key;/* key for "FACET_schematic_page_size" */
 INTBIG       el_transistor_width_key;	/* key for "transistor_width" */
 INTBIG       el_prototype_center_key;	/* key for "prototype_center" */
+INTBIG       el_essential_bounds_key;	/* key for "FACET_essentialbounds" */
 INTBIG       el_node_size_default_key;	/* key for "NODE_size_default" */
 INTBIG       el_arc_width_default_key;	/* key for "ARC_width_default" */
 INTBIG       el_attrkey_width;			/* key for "ATTR_width" */
