@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: dbchange.c
@@ -60,11 +61,12 @@ static void         db_killbatch(CHANGEBATCH*);
 static CHANGEBATCH *db_preparenewbatch(void);
 static void         db_erasebatch(CHANGEBATCH*);
 static void         db_freechangefacet(CHANGEFACET*);
-static void         db_loadhistorylist(INTBIG);
+static void         db_loadhistorylist(INTBIG, void*);
 static void         db_broadcastchange(CHANGE*, BOOLEAN, BOOLEAN);
 static BOOLEAN      db_reversechange(CHANGE*);
 static char        *db_describechange(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
 static BOOLEAN      db_majorvariable(INTBIG type, INTBIG key);
+static void         db_setdirty(INTBIG change, INTBIG obj, INTBIG a1, INTBIG a2, INTBIG a3, INTBIG a4);
 
 /*
  * Routine to free all memory associated with this module.
@@ -123,13 +125,7 @@ CHANGE *db_change(INTBIG obj, INTBIG change, INTBIG a1, INTBIG a2, INTBIG a3, IN
 	INTBIG a5, INTBIG a6)
 {
 	REGISTER BOOLEAN firstchange;
-	REGISTER INTBIG i, major;
-	REGISTER PORTPROTO *pp;
-	REGISTER NODEPROTO *np;
-	REGISTER NODEINST *ni;
-	REGISTER ARCINST *ai;
 	REGISTER CHANGE *c;
-	REGISTER LIBRARY *lib;
 	static char *broadcasttype[] = {M_("nodeinstnew"), M_("nodeinstkill"), M_("nodeinstmod"),
 		M_("arcinstnew"), M_("arcinstkill"), M_("arcinstmod"), M_("portprotonew"),
 		M_("portprotokill"), M_("portprotomod"), M_("nodeprotonew"), M_("nodeprotokill"),
@@ -147,7 +143,57 @@ CHANGE *db_change(INTBIG obj, INTBIG change, INTBIG a1, INTBIG a2, INTBIG a3, IN
 		ttyputmsg("%s", db_describechange(change, obj, a1, a2, a3, a4, a5, a6));
 	}
 
-	/* determine what needs to be updated */
+	/* determine what needs to be marked as changed */
+	db_setdirty(change, obj, a1, a2, a3, a4);
+
+	/* get change module */
+	c = db_allocchange();
+	if (c == NOCHANGE)
+	{
+		ttyputnomemory();
+		return(NOCHANGE);
+	}
+
+	/* insert new change module into linked list */
+	firstchange = FALSE;
+	if (db_currentbatch == NOCHANGEBATCH)
+	{
+		db_currentbatch = db_preparenewbatch();
+		if (db_currentbatch == NOCHANGEBATCH)
+		{
+			ttyputnomemory();
+			return(NOCHANGE);
+		}
+		firstchange = TRUE;
+		db_currentbatch->changehead = c;
+		c->prevchange = NOCHANGE;
+	} else
+	{
+		c->prevchange = db_currentbatch->changetail;
+		db_currentbatch->changetail->nextchange = c;
+	}
+	db_currentbatch->changetail = c;
+	c->nextchange = NOCHANGE;
+
+	/* save the change information */
+	c->changetype = change;     c->entryaddr = obj;
+	c->p1 = a1;   c->p2 = a2;   c->p3 = a3;
+	c->p4 = a4;   c->p5 = a5;   c->p6 = a6;
+
+	/* broadcast the change */
+	db_broadcastchange(c, firstchange, FALSE);
+	return(c);
+}
+
+void db_setdirty(INTBIG change, INTBIG obj, INTBIG a1, INTBIG a2, INTBIG a3, INTBIG a4)
+{
+	REGISTER INTBIG i, major;
+	REGISTER PORTPROTO *pp;
+	REGISTER NODEPROTO *np;
+	REGISTER NODEINST *ni;
+	REGISTER ARCINST *ai;
+	REGISTER LIBRARY *lib;
+
 	np = NONODEPROTO;
 	lib = NOLIBRARY;
 	major = 0;
@@ -228,7 +274,7 @@ CHANGE *db_change(INTBIG obj, INTBIG change, INTBIG a1, INTBIG a2, INTBIG a3, IN
 	/* set "changed" and "dirty" bits */
 	if (np != NONODEPROTO)
 	{
-		if (major != 0) np->revisiondate = getcurrenttime();
+		if (major != 0) np->revisiondate = (UINTBIG)getcurrenttime();
 		np->adirty = 0;
 		for(i=0; i<el_maxtools; i++)
 			if ((el_tools[i].toolstate & TOOLON) == 0) np->adirty |= 1 << i;
@@ -238,44 +284,6 @@ CHANGE *db_change(INTBIG obj, INTBIG change, INTBIG a1, INTBIG a2, INTBIG a3, IN
 		if (major != 0) lib->userbits |= LIBCHANGEDMAJOR; else
 			lib->userbits |= LIBCHANGEDMINOR;
 	}
-
-	/* get change module */
-	c = db_allocchange();
-	if (c == NOCHANGE)
-	{
-		ttyputnomemory();
-		return(NOCHANGE);
-	}
-
-	/* insert new change module into linked list */
-	firstchange = FALSE;
-	if (db_currentbatch == NOCHANGEBATCH)
-	{
-		db_currentbatch = db_preparenewbatch();
-		if (db_currentbatch == NOCHANGEBATCH)
-		{
-			ttyputnomemory();
-			return(NOCHANGE);
-		}
-		firstchange = TRUE;
-		db_currentbatch->changehead = c;
-		c->prevchange = NOCHANGE;
-	} else
-	{
-		c->prevchange = db_currentbatch->changetail;
-		db_currentbatch->changetail->nextchange = c;
-	}
-	db_currentbatch->changetail = c;
-	c->nextchange = NOCHANGE;
-
-	/* save the change information */
-	c->changetype = change;     c->entryaddr = obj;
-	c->p1 = a1;   c->p2 = a2;   c->p3 = a3;
-	c->p4 = a4;   c->p5 = a5;   c->p6 = a6;
-
-	/* broadcast the change */
-	db_broadcastchange(c, firstchange, FALSE);
-	return(c);
 }
 
 BOOLEAN db_majorvariable(INTBIG type, INTBIG key)
@@ -928,6 +936,9 @@ BOOLEAN db_reversechange(CHANGE *c)
 	INTBIG oldval;
 	REGISTER INTSML oldshort;
 
+	/* determine what needs to be marked as changed */
+	db_setdirty(c->changetype, c->entryaddr, c->p1, c->p2, c->p3, c->p4);
+
 	switch (c->changetype)
 	{
 		case NODEINSTNEW:
@@ -1411,6 +1422,7 @@ char *db_describechange(INTBIG changetype, INTBIG entryaddr, INTBIG p1, INTBIG p
 	REGISTER ARCINST *ai;
 	REGISTER PORTPROTO *pp;
 	REGISTER NODEPROTO *np;
+	REGISTER INTBIG lambda;
 	VARIABLE myvar;
 	REGISTER void *infstr;
 
@@ -1430,10 +1442,11 @@ char *db_describechange(INTBIG changetype, INTBIG entryaddr, INTBIG p1, INTBIG p
 			return(returninfstr(infstr));
 		case NODEINSTMOD:
 			ni = (NODEINST *)entryaddr;
+			lambda = lambdaofnode(ni);
 			infstr = initinfstr();
 			formatinfstr(infstr, M_(" Node '%s' changed in facet %s [was %sx%s, center (%s,%s), rotated %ld"),
-				describenodeinst(ni), describenodeproto(ni->parent), latoa(p3-p1), latoa(p4-p2),
-				latoa((p1+p3)/2), latoa((p2+p4)/2), p5);
+				describenodeinst(ni), describenodeproto(ni->parent), latoa(p3-p1, lambda),
+				latoa(p4-p2, lambda), latoa((p1+p3)/2, lambda), latoa((p2+p4)/2, lambda), p5);
 			if (p6 != 0) addstringtoinfstr(infstr, M_(", transposed"));
 			addstringtoinfstr(infstr, "]");
 			return(returninfstr(infstr));
@@ -1451,10 +1464,11 @@ char *db_describechange(INTBIG changetype, INTBIG entryaddr, INTBIG p1, INTBIG p
 			return(returninfstr(infstr));
 		case ARCINSTMOD:
 			ai = (ARCINST *)entryaddr;
+			lambda = lambdaofarc(ai);
 			infstr = initinfstr();
 			formatinfstr(infstr, M_(" Arc '%s' modified in facet %s [was %s wide %s long from (%s,%s) to (%s,%s)]"),
-				describearcinst(ai), describenodeproto(ai->parent), latoa(p5), latoa(p6), latoa(p1), latoa(p2),
-				latoa(p3), latoa(p4));
+				describearcinst(ai), describenodeproto(ai->parent), latoa(p5, lambda), latoa(p6, lambda),
+					latoa(p1, lambda), latoa(p2, lambda), latoa(p3, lambda), latoa(p4, lambda));
 			return(returninfstr(infstr));
 		case PORTPROTONEW:
 			pp = (PORTPROTO *)entryaddr;
@@ -1487,9 +1501,11 @@ char *db_describechange(INTBIG changetype, INTBIG entryaddr, INTBIG p1, INTBIG p
 			return(returninfstr(infstr));
 		case NODEPROTOMOD:
 			np = (NODEPROTO *)entryaddr;
+			lambda = lambdaoffacet(np);
 			infstr = initinfstr();
 			formatinfstr(infstr, M_(" Facet '%s' modified [was from %s<=X<=%s %s<=Y<=%s]"),
-				describenodeproto(np), latoa(p1), latoa(p2), latoa(p3), latoa(p4));
+				describenodeproto(np), latoa(p1, lambda), latoa(p2, lambda),
+					latoa(p3, lambda), latoa(p4, lambda));
 			return(returninfstr(infstr));
 		case OBJECTSTART:
 			/* args: addr, type */
@@ -1567,7 +1583,7 @@ static DIALOGITEM db_undodialogitems[] =
  /*  5 */ {0, {8,344,24,456}, CHECK, N_("Show details")},
  /*  6 */ {0, {468,108,492,180}, BUTTON, N_("Redo")}
 };
-static DIALOG db_undodialog = {{50,75,555,774}, N_("Change History"), 0, 6, db_undodialogitems};
+static DIALOG db_undodialog = {{50,75,555,774}, N_("Change History"), 0, 6, db_undodialogitems, 0, 0};
 
 /* special items in the "undo" dialog: */
 #define DUND_UNDO      2		/* Undo (button) */
@@ -1579,50 +1595,52 @@ void db_undodlog(void)
 {
 	INTBIG itemHit, details;
 	TOOL *tool;
+	REGISTER void *dia;
 
 	/* display the undo dialog box */
-	if (DiaInitDialog(&db_undodialog)) return;
-	DiaInitTextDialog(DUND_CHGLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone,
+	dia = DiaInitDialog(&db_undodialog);
+	if (dia == 0) return;
+	DiaInitTextDialog(dia, DUND_CHGLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone,
 		-1, SCHORIZBAR|SCSMALLFONT);
 	details = 0;
-	db_loadhistorylist(details);
+	db_loadhistorylist(details, dia);
 
 	/* default number of changes to undo */
-	DiaSelectLine(DUND_CHGLIST, 1000);
+	DiaSelectLine(dia, DUND_CHGLIST, 1000);
 
 	/* loop until done */
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == OK) break;
 		if (itemHit == DUND_DETAILS)
 		{
 			details = 1 - details;
-			DiaSetControl(DUND_DETAILS, details);
-			db_loadhistorylist(details);
+			DiaSetControl(dia, DUND_DETAILS, details);
+			db_loadhistorylist(details, dia);
 			continue;
 		}
 		if (itemHit == DUND_UNDO)
 		{
 			(void)undoabatch(&tool);
-			db_loadhistorylist(details);
+			db_loadhistorylist(details, dia);
 			continue;
 		}
 		if (itemHit == DUND_REDO)
 		{
 			(void)redoabatch(&tool);
-			db_loadhistorylist(details);
+			db_loadhistorylist(details, dia);
 			continue;
 		}
 	}
 
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 }
 
 /*
  * helper routines for undo dialog
  */
-void db_loadhistorylist(INTBIG details)
+void db_loadhistorylist(INTBIG details, void *dia)
 {
 	REGISTER INTBIG node, arcinst, portproto, nproto, object, variable, lineno, curline;
 	REGISTER CHANGE *c;
@@ -1630,7 +1648,7 @@ void db_loadhistorylist(INTBIG details)
 	CHANGEBATCH *batch;
 	REGISTER void *infstr;
 
-	DiaLoadTextDialog(DUND_CHGLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1);
+	DiaLoadTextDialog(dia, DUND_CHGLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1);
 	lineno = 0;
 	curline = -1;
 	for(batch = db_tailbatch; batch != NOCHANGEBATCH; batch = batch->nextchangebatch)
@@ -1710,7 +1728,7 @@ void db_loadhistorylist(INTBIG details)
 			(void)sprintf(line, " %ld %s", variable, makeplural(M_("variable"), variable));
 			addstringtoinfstr(infstr, line);
 		}
-		DiaStuffLine(DUND_CHGLIST, returninfstr(infstr));
+		DiaStuffLine(dia, DUND_CHGLIST, returninfstr(infstr));
 		if (batch == db_donebatch) curline = lineno;
 		lineno++;
 
@@ -1718,13 +1736,13 @@ void db_loadhistorylist(INTBIG details)
 		{
 			for(c = batch->changehead; c != NOCHANGE; c = c->nextchange)
 			{
-				DiaStuffLine(DUND_CHGLIST, db_describechange(c->changetype, c->entryaddr, c->p1, c->p2,
+				DiaStuffLine(dia, DUND_CHGLIST, db_describechange(c->changetype, c->entryaddr, c->p1, c->p2,
 					c->p3, c->p4, c->p5, c->p6));
 				lineno++;
 			}
 		}
 	}
-	DiaSelectLine(DUND_CHGLIST, curline);
+	DiaSelectLine(dia, DUND_CHGLIST, curline);
 }
 
 /************************* MEMORY ALLOCATION *************************/
