@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: dbcontrol.c
@@ -56,7 +57,7 @@
  *    version "6.03"   has major=6, minor=3, and detail=1000
  * The routine "io_getversion()" parses these version strings.
  */
-char *el_version = "6.06";
+char *el_version = "6.07";
 
        BOOLEAN db_multiprocessing;	/* true if multiprocessing */
 
@@ -207,8 +208,10 @@ void osprimaryosinit(void)
 	lib = newlibrary(libname, libfile);
 	selectlibrary(lib);
 
+#if 0	/* moved this to "usr.c:us_do2init()" */
 	/* establish the library location */
 	setupenvironment();
+#endif
 }
 
 /*
@@ -309,19 +312,14 @@ INTBIG asktool(TOOL *tool, char *command, ...)
 }
 
 /*
- * routine to turn tool "tool" back on.  It will also inform the tool of each
- * facet that changed while the tool was off (via "examinenodeproto").  If
- * "nocatchup" is true, this informing will not occur.
+ * routine to turn tool "tool" back on.  The user interface will notice this change
+ * and inform the tool of each facet that changed while it was off (via "examinenodeproto").
  */
-void toolturnon(TOOL *tool, BOOLEAN nocatchup)
+void toolturnon(TOOL *tool)
 {
-	REGISTER INTBIG bit;
-	REGISTER NODEPROTO *np;
-	REGISTER LIBRARY *lib;
-
 	/* turn on the tool */
 	(void)setval((INTBIG)tool, VTOOL, "toolstate", tool->toolstate|TOOLON, VINTEGER);
-
+#if 0
 	/* if the tool is not incremental, don't inform it of changes while off */
 	if ((tool->toolstate & TOOLINCREMENTAL) == 0) return;
 
@@ -340,6 +338,7 @@ void toolturnon(TOOL *tool, BOOLEAN nocatchup)
 		}
 		np->adirty &= ~bit;
 	}
+#endif
 }
 
 /*
@@ -435,13 +434,13 @@ void db_freeallmemory(void)
 	/* free all database memory */
 	db_freetechnologymemory();
 	db_freechangememory();
-	db_freetextmemory();
 	db_freevariablememory();
 	db_freenoprotomemory();
 	db_freemathmemory();
 	db_freemergememory();
 	db_freegeomemory();
 	db_freeerrormemory();
+	db_freetextmemory();
 	initerrorlogging("");
 #  if LANGTCL
 	db_freetclmemory();
@@ -471,9 +470,16 @@ BOOLEAN stopping(INTBIG reason)
  */
 void setmultiprocesslocks(BOOLEAN on)
 {
+	static INTBIG threadcapability = 0;		/* 1: can  -1: can't */
+
 	if (on)
 	{
-		if (!graphicshas(CANDOTHREADS)) return;
+		if (threadcapability == 0)
+		{
+			if (graphicshas(CANDOTHREADS)) threadcapability = 1; else
+				threadcapability = -1;
+		}
+		if (threadcapability < 0) return;
 	}
 	db_multiprocessing = on;
 }
