@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: dbcreate.c
@@ -604,7 +605,7 @@ ARCINST *newarcinst(ARCPROTO *typ, INTBIG wid, INTBIG initialbits, NODEINST *nA,
 	PORTPROTO *pA, INTBIG xA, INTBIG yA, NODEINST *nB, PORTPROTO *pB, INTBIG xB, INTBIG yB,
 	NODEPROTO *parnt)
 {
-	REGISTER INTBIG i;
+	REGISTER INTBIG i, lambda;
 	REGISTER PORTPROTO *pp;
 	REGISTER ARCINST *ai;
 	REGISTER POLYGON *poly;
@@ -695,8 +696,11 @@ ARCINST *newarcinst(ARCPROTO *typ, INTBIG wid, INTBIG initialbits, NODEINST *nA,
 	{
 		(void)db_error(DBBADENDAP|DBNEWARCINST);
 		if (db_printerrors)
-			ttyputmsg(_("Point (%s,%s) not inside port %s of node %s in facet %s"), latoa(xA),
-				latoa(yA), pA->protoname, describenodeinst(nA), describenodeproto(parnt));
+		{
+			lambda = lambdaoffacet(parnt);
+			ttyputmsg(_("Point (%s,%s) not inside port %s of node %s in facet %s"), latoa(xA, lambda),
+				latoa(yA, lambda), pA->protoname, describenodeinst(nA), describenodeproto(parnt));
+		}
 		db_donextchangequietly = FALSE;
 		freepolygon(poly);
 		return(NOARCINST);
@@ -706,8 +710,11 @@ ARCINST *newarcinst(ARCPROTO *typ, INTBIG wid, INTBIG initialbits, NODEINST *nA,
 	{
 		(void)db_error(DBBADENDBP|DBNEWARCINST);
 		if (db_printerrors)
-			ttyputmsg(_("Point (%s,%s) not inside port %s of node %s in facet %s"), latoa(xB),
-				latoa(yB), pB->protoname, describenodeinst(nB), describenodeproto(parnt));
+		{
+			lambda = lambdaoffacet(parnt);
+			ttyputmsg(_("Point (%s,%s) not inside port %s of node %s in facet %s"), latoa(xB, lambda),
+				latoa(yB, lambda), pB->protoname, describenodeinst(nB), describenodeproto(parnt));
+		}
 		db_donextchangequietly = FALSE;
 		freepolygon(poly);
 		return(NOARCINST);
@@ -1375,8 +1382,8 @@ PORTPROTO *newportproto(NODEPROTO *np, NODEINST *nodeinst, PORTPROTO *portaddres
 	}
 
 	/* look for duplicate names */
-	for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-		if (namesame(pp->protoname, name) == 0)
+	pp = getportproto(np, name);
+	if (pp != NOPORTPROTO)
 	{
 		db_donextchangequietly = FALSE;
 		return((PORTPROTO *)db_error(DBDUPLICATE|DBNEWPORTPROTO));
@@ -1439,28 +1446,51 @@ PORTPROTO *db_newportproto(NODEPROTO *np, NODEINST *nodeinst, PORTPROTO *portadd
 
 void db_enterportproto(PORTPROTO *pp)
 {
+	REGISTER INTBIG i, j;
 	REGISTER PORTPROTO *p, *last;
+	REGISTER NODEPROTO *facet;
 	REGISTER PORTEXPINST *pe;
 
-	for(last = NOPORTPROTO, p = pp->parent->firstportproto; p != NOPORTPROTO; p = p->nextportproto)
+	facet = pp->parent;
+	for(last = NOPORTPROTO, p = facet->firstportproto; p != NOPORTPROTO; p = p->nextportproto)
 		last = p;
-	if (last == NOPORTPROTO) pp->parent->firstportproto = pp; else
+	if (last == NOPORTPROTO) facet->firstportproto = pp; else
 		last->nextportproto = pp;
 	pp->nextportproto = NOPORTPROTO;
 
 	/* load up this portexpinst description */
-	pe = allocportexpinst(pp->parent->cell->cluster);
+	pe = allocportexpinst(facet->cell->cluster);
 	pe->proto = pp->subportproto;
 	db_addportexpinst(pp->subnodeinst, pe);
 	pe->exportproto = pp;
 	pp->subportexpinst = pe;
 
+	/* insert into hash table of portproto names */
+	facet->numportprotos++;
+	if (facet->numportprotos < facet->portprotohashtablesize/2)
+	{
+		i = db_namehash(pp->protoname) % facet->portprotohashtablesize;
+		for(j=1; j<=facet->portprotohashtablesize; j += 2)
+		{
+			if (facet->portprotohashtable[i] == NOPORTPROTO)
+			{
+				facet->portprotohashtable[i] = pp;
+				break;
+			}
+			i += j;
+			if (i >= facet->portprotohashtablesize) i -= facet->portprotohashtablesize;
+		}
+	} else
+	{
+		db_buildportprotohashtable(facet);
+	}
+
 	/* special case: if port is on node that changes size with connectivity, update its geometry */
 	if ((pp->subnodeinst->proto->userbits&WIPEON1OR2) != 0)
-		updategeom(pp->subnodeinst->geom, pp->parent);
+		updategeom(pp->subnodeinst->geom, facet);
 
 	/* clear cache of port associations in this cell */
-	db_clearportcache(pp->parent);
+	db_clearportcache(facet);
 
 	/* mark a change to the database */
 	db_changetimestamp++;
@@ -1668,6 +1698,10 @@ void db_retractportproto(PORTPROTO *pp)
 		lpo = ppo;
 	}
 
+	/* rebuild hash table of cell names */
+	np->numportprotos--;
+	db_buildportprotohashtable(np);
+
 	/* special case: if port is on node that changes size with connectivity, update its geometry */
 	if ((pp->subnodeinst->proto->userbits&WIPEON1OR2) != 0)
 		updategeom(pp->subnodeinst->geom, np);
@@ -1701,3 +1735,53 @@ void db_removeportexpinst(PORTPROTO *pp)
 		lpe = pe;
 	}
 }
+
+/*
+ * Routine to rebuild the hash table of cell names in library "lib".
+ */
+void db_buildportprotohashtable(NODEPROTO *facet)
+{
+	REGISTER INTBIG i, j, numportprotos, hashtablesize;
+	REGISTER PORTPROTO *pp;
+
+	/* free the former table */
+	if (facet->portprotohashtablesize > 0)
+	{
+		facet->portprotohashtablesize = 0;
+		efree((char *)facet->portprotohashtable);
+	}
+
+	/* determine the size of the hash table */
+	numportprotos = 0;
+	for(pp = facet->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto) numportprotos++;
+	if (numportprotos != facet->numportprotos)
+	{
+		ttyputmsg(_("Warning: number of ports in facet %s corrected"), describenodeproto(facet));
+		facet->numportprotos = numportprotos;
+	}
+	hashtablesize = pickprime(numportprotos * 4);
+
+	/* create the hash table and clear it */
+	facet->portprotohashtable = (PORTPROTO **)emalloc(hashtablesize * (sizeof (PORTPROTO *)),
+		facet->cell->cluster);
+	if (facet->portprotohashtable == 0) return;
+	for(i=0; i<hashtablesize; i++)
+		facet->portprotohashtable[i] = NOPORTPROTO;
+	facet->portprotohashtablesize = hashtablesize;
+
+	/* insert all cells into the table */
+	for(pp = facet->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+	{
+		i = db_namehash(pp->protoname) % hashtablesize;
+		for(j=1; j<=hashtablesize; j += 2)
+		{
+			if (facet->portprotohashtable[i] == NOPORTPROTO)
+			{
+				facet->portprotohashtable[i] = pp;
+				break;
+			}
+			i += j;
+			if (i >= hashtablesize) i -= hashtablesize;
+		}
+	}
+}
