@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: dberror.c
@@ -168,12 +169,23 @@ void telldatabaseerror(void)
  *   addlinetoerror(void *e, x1, y1, x2, y2)     add line to error "e"
  *   addpolytoerror(void *e, POLYGON *p)         add polygon to error "e"
  *   addpointtoerror(void *e, x, y)              add point to error "e"
+ *
  * To report errors, call:
  *   termerrorlogging(explain)                   complete error accumulation
  *   sorterrors()                                sort errors by key
  *   n = numerrors()                             returns number of errors
  *   s = reportnexterror(show, &g1, &g2)         report next error
  *   s = reportpreverror()                       report previous error
+ *
+ * To obtain errors internally, call:
+ *   e = getnexterror(e, msg)                    gets error after "e" (0 is first error)
+ *   msg = describeerror(e)                      return description of error "e"
+ *   reporterror(e)                              reports error "e"
+ *   t = getnumerrorgeom(e)                      get number of geoms on error "e"
+ *   eg = geterrorgeom(e, i)                     get geom "i" on error "e"
+ *   msg = describeerrorgeom(eg)                 return description of geom "eg"
+ *   showerrorgeom(eg)                           highlight error geom "eg"
+ *   adviseofchanges(routine)                    call "routine" when errors change
  */
 
 typedef enum {ERRORTYPEGEOM, ERRORTYPEEXPORT, ERRORTYPELINE, ERRORTYPEPOINT} ERRORHIGHLIGHTTYPE;
@@ -218,11 +230,11 @@ static INTBIG     db_numerrors;
 static INTBIG     db_truenumerrors;
 static INTBIG     db_maxerrors = 0;
 static void      *db_errorreportmutex = 0;			/* mutex for error reporting modules */
-
+static void     (*db_errorchangedroutine)(void) = 0;
 
 static BOOLEAN db_addtoerrorlist(ERRORLIST *el, ERRORHIGHLIGHT *eh);
 static int     db_errorlistascending(const void *e1, const void *e2);
-char          *db_reportcurrenterror(INTBIG showhigh, GEOM **g1, GEOM **g2);
+static char   *db_reportcurrenterror(INTBIG showhigh, GEOM **g1, GEOM **g2);
 static void    db_clearerrorlist(void);
 
 /*
@@ -468,30 +480,13 @@ void addpointtoerror(void *errorlist, INTBIG x, INTBIG y)
 	(void)db_addtoerrorlist(el, eh);	
 }
 
-BOOLEAN db_addtoerrorlist(ERRORLIST *el, ERRORHIGHLIGHT *eh)
-{
-	REGISTER ERRORHIGHLIGHT **neweh;
-	REGISTER INTBIG i;
-
-	neweh = (ERRORHIGHLIGHT **)emalloc((sizeof (ERRORHIGHLIGHT *)) * (el->numhighlights+1),
-		db_cluster);
-	if (neweh == 0) return(TRUE);
-	for(i=0; i<el->numhighlights; i++)
-		neweh[i] = el->highlights[i];
-	neweh[el->numhighlights] = eh;
-	if (el->numhighlights > 0) efree((char *)el->highlights);
-	el->highlights = neweh;
-	el->numhighlights++;
-	return(FALSE);
-}
-
 /*
  * Routine called when all errors are logged.  Initializes pointers for replay of errors.
  */
 void termerrorlogging(BOOLEAN explain)
 {
 	REGISTER INTBIG errs;
-	REGISTER ERRORLIST *ep;
+	REGISTER ERRORLIST *el;
 
 	/* code cannot be called by multiple procesors: uses globals */
 	NOT_REENTRANT;
@@ -502,8 +497,11 @@ void termerrorlogging(BOOLEAN explain)
 
 	/* enumerate the errors */
 	errs = 0;
-	for(ep = db_firsterrorlist; ep != NOERRORLIST; ep = ep->nexterrorlist)
-		ep->index = ++errs;
+	for(el = db_firsterrorlist; el != NOERRORLIST; el = el->nexterrorlist)
+		el->index = ++errs;
+
+	if (db_errorchangedroutine != 0)
+		(*db_errorchangedroutine)();
 
 	if (errs > 0 && explain)
 	{
@@ -513,9 +511,13 @@ void termerrorlogging(BOOLEAN explain)
 	}
 }
 
+/*
+ * Routine to sort the errors by their "key" (a value provided to "logerror()").
+ * Obviously, this should be called after all errors have been reported.
+ */
 void sorterrors(void)
 {
-	REGISTER ERRORLIST *ep, **sortedlist;
+	REGISTER ERRORLIST *el, **sortedlist;
 	REGISTER INTBIG i;
 
 	if (db_truenumerrors == 0) return;
@@ -526,34 +528,90 @@ void sorterrors(void)
 	sortedlist = (ERRORLIST **)emalloc(db_truenumerrors * (sizeof (ERRORLIST *)), db_cluster);
 	if (sortedlist == 0) return;
 	i = 0;
-	for(ep = db_firsterrorlist; ep != NOERRORLIST; ep = ep->nexterrorlist)
-		sortedlist[i++] = ep;
+	for(el = db_firsterrorlist; el != NOERRORLIST; el = el->nexterrorlist)
+		sortedlist[i++] = el;
 	esort(sortedlist, db_truenumerrors, sizeof (ERRORLIST *), db_errorlistascending);
 	for(i=0; i<db_truenumerrors; i++)
 	{
-		ep = sortedlist[i];
-		ep->index = i+1;
-		if (i == 0) ep->preverrorlist = NOERRORLIST; else
-			ep->preverrorlist = sortedlist[i-1];
-		if (i < db_truenumerrors-1) ep->nexterrorlist = sortedlist[i+1]; else
-			ep->nexterrorlist = NOERRORLIST;
+		el = sortedlist[i];
+		el->index = i+1;
+		if (i == 0) el->preverrorlist = NOERRORLIST; else
+			el->preverrorlist = sortedlist[i-1];
+		if (i < db_truenumerrors-1) el->nexterrorlist = sortedlist[i+1]; else
+			el->nexterrorlist = NOERRORLIST;
 	}
 	db_firsterrorlist = sortedlist[0];
 	efree((char *)sortedlist);
 }
 
-/*
- * Helper routine for "sorterrors()" to sort errors by their key
- */
-int db_errorlistascending(const void *e1, const void *e2)
-{
-	REGISTER ERRORLIST *er1, *er2;
+#if 0
+static void    db_describenodetoerrorfile(FILE *io, NODEINST *ni);
 
-	er1 = *((ERRORLIST **)e1);
-	er2 = *((ERRORLIST **)e2);
-	return(er1->sortkey - er2->sortkey);
+/*
+ * Routine to save the current errors to disk.
+ */
+void saveerrors(void)
+{
+	REGISTER ERRORLIST *el;
+	REGISTER ERRORHIGHLIGHT *eh;
+	FILE *io;
+
+	io = fopen("errors.txt", "w");
+	if (io == 0) return;
+	for(el = db_firsterrorlist; el != NOERRORLIST; el = el->nexterrorlist)
+	{
+		fprintf(io, "ERROR %ld %s %s\n", el->numhighlights, describenodeproto(el->facet), el->message);
+		for(i=0; i<el->numhighlights; i++)
+		{
+			eh = el->highlights[i];
+			switch (eh->type)
+			{
+				case ERRORTYPEGEOM:
+					fprintf(io, "  GEOM");
+					if (eh->geom->entryisnode)
+					{
+						fprintf(io, " NODE");
+						ni = eh->geom->entryaddr.ni;
+						db_describenodetoerrorfile(io, ni);
+					} else
+					{
+						ai = eh->geom->entryaddr.ai;
+						var = getvalkey((INTBIG)ai, VARCINST, -1, el_arc_name_key);
+						if (var != NOVARIABLE) fprintf(io, "ARC NAME %s", (char *)var->addr); else
+						{
+							fprintf(io, "ARC TYPE %s WIDTH %s AT %s %s %s %s", ai->proto->protoname,
+								ai->width, ai->end[0].xpos, ai->end[0].ypos, ai->end[1].xpos,
+									ai->end[1].ypos);
+						}
+					}
+					if (eh->showgeom) fprintf(io, " SHOW"); else fprintf(io, " NOSHOW");
+					fprintf(io, " PATH %ld", eh->pathlen);
+					for(j=0; j<eh->pathlen; j++)
+					{
+						ni = el->path[j];
+						fprintf(io, " %s", describenodeproto(ni->parent));
+						db_describenodetoerrorfile(io, ni);
+					}
+					fprintf(io, "\n");
+					break;
+			}
+		}
+	}
 }
 
+void db_describenodetoerrorfile(FILE *io, NODEINST *ni)
+{
+	REGISTER VARIABLE *var;
+
+	var = getvalkey((INTBIG)ni, VNODEINST, -1, el_node_name_key);
+	if (var != NOVARIABLE) fprintf(io, " NAME %s", (char *)var->addr); else
+	{
+		fprintf(io, " TYPE %s AT %ld %ld %ld %ld", describenodeproto(ni->proto),
+			ni->lowx, ni->highx, ni->lowy, ni->highy);
+	}
+}
+#endif
+
 /*
  * Routine to return the number of logged errors.
  */
@@ -588,7 +646,7 @@ char *reportnexterror(INTBIG showhigh, GEOM **g1, GEOM **g2)
  */
 char *reportpreverror(void)
 {
-	REGISTER ERRORLIST *ep;
+	REGISTER ERRORLIST *el;
 
 	if (db_preverrorlist != NOERRORLIST)
 	{
@@ -599,15 +657,208 @@ char *reportpreverror(void)
 	{
 		/* at start: go to end of list */
 		db_preverrorlist = db_curerrorlist = db_nexterrorlist = NOERRORLIST;
-		for(ep = db_firsterrorlist; ep != NOERRORLIST; ep = ep->nexterrorlist)
+		for(el = db_firsterrorlist; el != NOERRORLIST; el = el->nexterrorlist)
 		{
 			db_preverrorlist = db_curerrorlist;
-			db_curerrorlist = ep;
+			db_curerrorlist = el;
 		}
 	}
 	return(db_reportcurrenterror(1, 0, 0));
 }
 
+/*
+ * Routine to return a list of errors.  On the first call, set "e" to zero.
+ * Returns the next in the list (zero when done).
+ */
+void *getnexterror(void *elv)
+{
+	REGISTER ERRORLIST *el;
+
+	if (elv == 0) el = db_firsterrorlist; else
+	{
+		el = (ERRORLIST *)elv;
+		if (el == NOERRORLIST) return(0);
+		el = el->nexterrorlist;
+	}
+	if (el == NOERRORLIST) return(0);
+	return((void *)el);
+}
+
+/*
+ * Routine to return the number of objects associated with error "e".  Only
+ * returns "geom" objects
+ */
+INTBIG getnumerrorgeom(void *elv)
+{
+	REGISTER ERRORLIST *el;
+	REGISTER ERRORHIGHLIGHT *eh;
+	REGISTER INTBIG i, total;
+
+	if (elv == 0) return(0);
+	el = (ERRORLIST *)elv;
+	total = 0;
+	for(i=0; i<el->numhighlights; i++)
+	{
+		eh = el->highlights[i];
+		if (eh->type == ERRORTYPEGEOM) total++;
+	}
+	return(total);
+}
+
+/*
+ * Routine to return the next object in a list of objects on an error.  The previous
+ * object is "eg".  Returns zero at the end of the list.
+ */
+void *geterrorgeom(void *elv, INTBIG index)
+{
+	REGISTER ERRORLIST *el;
+	REGISTER ERRORHIGHLIGHT *eh;
+	REGISTER INTBIG i, total;
+
+	if (elv == 0) return(0);
+	el = (ERRORLIST *)elv;
+	total = 0;
+	for(i=0; i<el->numhighlights; i++)
+	{
+		eh = el->highlights[i];
+		if (eh->type != ERRORTYPEGEOM) continue;
+		if (total == index) break;
+		total++;
+	}
+	if (i >= el->numhighlights) return(0);
+	return(eh);
+}
+
+/*
+ * Routine to describe an object on an error.  The object is "ehv".
+ */
+char *describeerrorgeom(void *ehv)
+{
+	REGISTER ERRORHIGHLIGHT *eh;
+	REGISTER GEOM *geom;
+	REGISTER void *infstr;
+	REGISTER INTBIG i;
+	REGISTER NODEINST *ni;
+
+	if (ehv == 0) return("");
+	eh = (ERRORHIGHLIGHT *)ehv;
+	geom = eh->geom;
+	infstr = initinfstr();
+	if (geom->entryisnode) formatinfstr(infstr, _("Node %s"), describenodeinst(geom->entryaddr.ni)); else
+		formatinfstr(infstr, _("Arc %s"), describearcinst(geom->entryaddr.ai));
+	for(i=0; i<eh->pathlen; i++)
+	{
+		ni = eh->path[i];
+		formatinfstr(infstr, _(" in %s:%s:%s"), ni->parent->cell->lib->libname,
+			nldescribenodeproto(ni->parent), describenodeinst(ni));
+	}
+	return(returninfstr(infstr));
+}
+
+/*
+ * Routine to highlight an object on an error.  The object is "ehv".
+ */
+void showerrorgeom(void *ehv)
+{
+	REGISTER ERRORHIGHLIGHT *eh;
+	REGISTER GEOM *geom;
+	INTBIG lx, hx, ly, hy;
+	REGISTER INTBIG i;
+	REGISTER WINDOWPART *w;
+	REGISTER NODEPROTO *np;
+
+	if (ehv == 0) return;
+	eh = (ERRORHIGHLIGHT *)ehv;
+	geom = eh->geom;
+	np = geomparent(geom);
+	for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
+		if (w->curnodeproto == np) break;
+	if (w == NOWINDOWPART)
+	{
+		el_curwindowpart = us_wantnewwindow(0);
+		us_fullview(np, &lx, &hx, &ly, &hy);
+		us_switchtofacet(np, lx, hx, ly, hy, NONODEINST, NOPORTPROTO, FALSE, 0);
+	}
+	(void)asktool(us_tool, "show-object", (INTBIG)geom);
+
+	for(i=eh->pathlen-1; i>=0; i--)
+	{
+		sethierarchicalparent(np, eh->path[i]);
+		np = eh->path[i]->parent;
+	}
+}
+
+/*
+ * Routine to describe error "elv".
+ */
+char *describeerror(void *elv)
+{
+	REGISTER ERRORLIST *el;
+
+	if (elv == 0) return("");
+	el = (ERRORLIST *)elv;
+	return(el->message);
+}
+
+/*
+ * Routine to highlight and report error "elv".
+ */
+void reporterror(void *elv)
+{
+	REGISTER ERRORLIST *el;
+
+	if (elv == 0) return;
+	el = (ERRORLIST *)elv;
+	db_curerrorlist = el;
+	db_nexterrorlist = db_curerrorlist->nexterrorlist;
+	db_preverrorlist = db_curerrorlist->preverrorlist;
+	(void)db_reportcurrenterror(1, 0, 0);
+}
+
+/*
+ * Routine to request that "routine" be called whenever any changes are made to the list of
+ * errors.
+ */
+void adviseofchanges(void (*routine)(void))
+{
+	db_errorchangedroutine = routine;
+}
+
+/******************** ERROR REPORTING SUPPORT ROUTINES ********************/
+
+/*
+ * Internal routine to add detail to an error.  The error is "el" and the detail is "eh".
+ * Returns TRUE on error.
+ */
+BOOLEAN db_addtoerrorlist(ERRORLIST *el, ERRORHIGHLIGHT *eh)
+{
+	REGISTER ERRORHIGHLIGHT **neweh;
+	REGISTER INTBIG i;
+
+	neweh = (ERRORHIGHLIGHT **)emalloc((sizeof (ERRORHIGHLIGHT *)) * (el->numhighlights+1),
+		db_cluster);
+	if (neweh == 0) return(TRUE);
+	for(i=0; i<el->numhighlights; i++)
+		neweh[i] = el->highlights[i];
+	neweh[el->numhighlights] = eh;
+	if (el->numhighlights > 0) efree((char *)el->highlights);
+	el->highlights = neweh;
+	el->numhighlights++;
+	return(FALSE);
+}
+
+/*
+ * Helper routine for "sorterrors()" to sort errors by their key
+ */
+int db_errorlistascending(const void *e1, const void *e2)
+{
+	REGISTER ERRORLIST *el1, *el2;
+
+	el1 = *((ERRORLIST **)e1);
+	el2 = *((ERRORLIST **)e2);
+	return(el1->sortkey - el2->sortkey);
+}
+
 /*
  * Routine to return the error message associated with the current error.
  * Highlights associated graphics if "showhigh" is nonzero.  Fills "g1" and "g2"
@@ -681,8 +932,7 @@ char *db_reportcurrenterror(INTBIG showhigh, GEOM **g1, GEOM **g2)
 			np = facet;
 			for(j=hierpathcount-1; j>=0; j--)
 			{
-				(void)setvalkey((INTBIG)np, VNODEPROTO, us_descent_path_key, (INTBIG)hierpath[j],
-					VNODEINST|VDONTSAVE);
+				sethierarchicalparent(np, hierpath[j]);
 				np = hierpath[j]->parent;
 			}
 			numfacets++;
@@ -712,19 +962,19 @@ char *db_reportcurrenterror(INTBIG showhigh, GEOM **g1, GEOM **g2)
 			if (newwindows > 0)
 			{
 				el_curwindowpart = neww[0];
-				neww[1] = us_splitcurrentwindow(2, FALSE, &neww[0]);
+				neww[1] = us_splitcurrentwindow(2, FALSE, &neww[0], 50);
 				newwindows--;
 			}
 			if (newwindows > 0)
 			{
 				el_curwindowpart = neww[0];
-				neww[2] = us_splitcurrentwindow(1, FALSE, &neww[0]);
+				neww[2] = us_splitcurrentwindow(1, FALSE, &neww[0], 50);
 				newwindows--;
 			}
 			if (newwindows > 0)
 			{
 				el_curwindowpart = neww[1];
-				neww[3] = us_splitcurrentwindow(1, FALSE, &neww[1]);
+				neww[3] = us_splitcurrentwindow(1, FALSE, &neww[1], 50);
 				newwindows--;
 			}
 			count = 0;
