@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: dbgeom.c
@@ -784,7 +785,7 @@ void db_printrtree(RTNODE *expectedparent, RTNODE *rtn, INTBIG indent)
 	infstr = initinfstr();
 	for(i=0; i<indent; i++) addtoinfstr(infstr, ' ');
 	(void)sprintf(line, M_("Node X(%s-%s) Y(%s-%s) has %d children:"),
-		latoa(rtn->lowx), latoa(rtn->highx), latoa(rtn->lowy), latoa(rtn->highy), rtn->total);
+		latoa(rtn->lowx, 0), latoa(rtn->highx, 0), latoa(rtn->lowy, 0), latoa(rtn->highy, 0), rtn->total);
 	addstringtoinfstr(infstr, line);
 
 	/* sensibility check of this R-tree node */
@@ -821,7 +822,7 @@ void db_printrtree(RTNODE *expectedparent, RTNODE *rtn, INTBIG indent)
 			for(i=0; i<indent+3; i++) addtoinfstr(infstr, ' ');
 			addstringtoinfstr(infstr, geomname((GEOM *)rtn->pointers[j]));
 			db_rtnbbox(rtn, j, &lx, &hx, &ly, &hy);
-			(void)sprintf(line, " X(%s-%s) Y(%s-%s)", latoa(lx), latoa(hx), latoa(ly), latoa(hy));
+			(void)sprintf(line, " X(%s-%s) Y(%s-%s)", latoa(lx, 0), latoa(hx, 0), latoa(ly, 0), latoa(hy, 0));
 			addstringtoinfstr(infstr, line);
 			ttyputmsg("%s", returninfstr(infstr));
 		} else db_printrtree(rtn, (RTNODE *)rtn->pointers[j], indent+3);
@@ -1101,8 +1102,8 @@ void db_boundfacet(NODEPROTO *facet, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG
 	first = TRUE;
 	for(ni = facet->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
-		/* special case: do not include "facet center" primitive from Generic */
-		if (ni->proto == gen_facetcenterprim) continue;
+		/* special case: do not include "facet center" or "essential-bounds" primitives from Generic */
+		if (ni->proto == gen_facetcenterprim || ni->proto == gen_essentialprim) continue;
 
 		/* special case for invisible pins: do not include if inheritable or interior-only */
 		if (ni->proto == gen_invispinprim)
@@ -1164,7 +1165,7 @@ INTBIG initsearch(INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy, NODEPROTO *facet)
 	REGISTER RSEARCH *rs;
 
 	rs = db_allocrsearch();
-	if (rs == 0) return(-1);
+	if (rs == NORSEARCH) return(-1);
 	rs->depth = 0;
 	rs->rtn[0] = facet->rtree;
 	rs->position[0] = 0;
