@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: dblang.c
@@ -32,31 +33,41 @@
 #include "database.h"
 #include "dblang.h"
 
-#define EVALUATIONCACHESIZE 10
+#define EVALUATIONCACHESIZE 20
 typedef struct
 {
-	UINTBIG   changetimestamp;
-	UINTBIG   traversaltimestamp;
-	char     *code;
-	UINTBIG   type;
-	INTBIG    value;
-	VARIABLE  var;
+	char     *code;					/* the interpreted code that is being cached */
+	INTBIG    oaddr, otype;			/* address/type of object this code sits on */
+	UINTBIG   type;					/* type of the result of the code */
+	UINTBIG   reqtype;				/* requested type of the result of the code */
+	INTBIG    usagestamp;			/* timestamp for when this cache entry was last used */
+	UINTBIG   changetimestamp;		/* indication of changes to the database that may affect evaluation */
+	UINTBIG   traversaltimestamp;	/* indication of hierarchy traversal that may affect evaluation */
+	INTBIG    value;				/* evaluated value of the code */
+	VARIABLE  var;					/* variable with the evaluation of the code */
 } EVALUATIONCACHE;
 
 static EVALUATIONCACHE db_evaluationcache[EVALUATIONCACHESIZE];
-static INTBIG db_evaluationcachesize = 0;
-static INTBIG db_evaluationcachelast = 0;
+static INTBIG db_evaluationusagetime = 0;
 static void   *db_languagecachemutex = 0;		/* mutex for caching language evaluation */
 
 /* prototypes for local routines */
 #if LANGLISP || LANGTCL || LANGJAVA || LANGMM
-static VARIABLE *db_enterqueryincache(char *code, INTBIG type, INTBIG value);
+static VARIABLE *db_enterqueryincache(char *code, INTBIG type, INTBIG reqtype, INTBIG value, INTBIG oaddr, INTBIG otype);
 #endif
+static BOOLEAN db_codehasvariables(char *code, INTBIG language);
 
 /****************************** LANGUAGE INTERFACE ******************************/
 
 void db_initlanguages(void)
 {
+	REGISTER INTBIG i;
+
+	for(i=0; i<EVALUATIONCACHESIZE; i++)
+	{
+		db_evaluationcache[i].code = 0;
+		db_evaluationcache[i].usagestamp = 0;
+	}
 	(void)ensurevalidmutex(&db_languagecachemutex, TRUE);
 }
 
@@ -120,6 +131,52 @@ BOOLEAN loadcode(char *program, INTBIG language)
 	return(TRUE);
 }
 
+/*
+ * Routine to return TRUE if "code" (in language "language") has variables in it (and so may evaluate
+ * differently with different contexts).
+ */
+BOOLEAN db_codehasvariables(char *code, INTBIG language)
+{
+#if LANGJAVA
+	REGISTER INTBIG i;
+#endif
+
+	switch (language)
+	{
+#if LANGLISP
+		case VLISP:
+			break;
+#endif
+
+#if LANGTCL
+		case VTCL:
+			break;
+#endif
+
+#if LANGJAVA
+		case VJAVA:
+			/* if there is an "@" or a "(" with letter or digit in front, this has variables */
+			for(i=0; code[i] != 0; i++)
+			{
+				if (code[i] == '@') return(TRUE);
+				if (code[i] == '(')
+				{
+					while (i > 0 && code[i-1] == ' ') i--;
+					if (i <= 0) continue;
+					if (isalnum(code[i-1])) return(TRUE);
+				}
+			}
+			return(FALSE);
+#endif
+
+#if LANGMM
+		case VMATHEMATICA:
+			break;
+#endif
+	}
+	return(TRUE);
+}
+
 /*
  * routine to evaluate string "code" in the specified "language", trying to match
  * "type" as a return type.  Returns a variable that describes the result (NOVARIABLE on error).
@@ -134,7 +191,7 @@ VARIABLE *doquerry(char *code, INTBIG language, UINTBIG type)
 #endif
 #if LANGJAVA
 	char *str;
-	INTBIG methodreturntype;
+	INTBIG methodreturntype, reqtype;
 #endif
 #if	LANGMM
 	char *str;
@@ -143,28 +200,42 @@ VARIABLE *doquerry(char *code, INTBIG language, UINTBIG type)
 	double fval;
 	REGISTER void *infstr;
 #endif
-	REGISTER INTBIG i, index;
-	INTBIG addr;
+	REGISTER INTBIG index;
+	INTBIG addr, oaddr, otype;
 	REGISTER VARIABLE *retvar;
+	REGISTER BOOLEAN hasvariables;
+	REGISTER EVALUATIONCACHE *ec;
+
+	/* see if this code has variables in it (if not, caching is simpler) */
+	hasvariables = db_codehasvariables(code, language);
+	db_evaluationusagetime++;
 
 	/* search cache of evaluations */
 	if (db_multiprocessing) emutexlock(db_languagecachemutex);
-	for(i=0; i<db_evaluationcachesize; i++)
+	if (!getcurrentvariableobject(&oaddr, &otype)) oaddr = otype = 0;
+	for(index=0; index<EVALUATIONCACHESIZE; index++)
 	{
-		index = db_evaluationcachelast - i - 1;
-		if (index < 0) index += EVALUATIONCACHESIZE;
-		if (db_evaluationcache[index].changetimestamp != db_changetimestamp) continue;
-		if (db_evaluationcache[index].traversaltimestamp != db_traversaltimestamp) continue;
-		if (strcmp(db_evaluationcache[index].code, code) != 0) continue;
-		if (db_evaluationcache[index].type != type) continue;
+		ec = &db_evaluationcache[index];
+		if (ec->code == 0) continue;
+		if (strcmp(ec->code, code) != 0) continue;
+		if (ec->reqtype != type) continue;
+		if (hasvariables)
+		{
+			if (ec->changetimestamp != db_changetimestamp) continue;
+			if (ec->traversaltimestamp != db_traversaltimestamp) continue;
+			if (ec->oaddr != oaddr) continue;
+			if (ec->otype != otype) continue;
+		}
 
 		/* found in cache: return value */
-		retvar = &db_evaluationcache[index].var;
+		retvar = &ec->var;
+		ec->usagestamp = db_evaluationusagetime;
 		if (db_multiprocessing) emutexunlock(db_languagecachemutex);
 		return(retvar);
 	}
-	if (db_multiprocessing) emutexunlock(db_languagecachemutex);
 
+	/* do the evaluation */
+	if (db_multiprocessing) emutexunlock(db_languagecachemutex);
 	switch (language)
 	{
 #if LANGLISP
@@ -181,7 +252,7 @@ VARIABLE *doquerry(char *code, INTBIG language, UINTBIG type)
 
 			/* convert the evaluation to a string */
 			if (lsp_describeobject(obj, type, &addr)) return(NOVARIABLE);
-			retvar = db_enterqueryincache(code, type, addr);
+			retvar = db_enterqueryincache(code, type, type, addr, oaddr, otype);
 			return(retvar);
 #endif
 
@@ -197,7 +268,7 @@ VARIABLE *doquerry(char *code, INTBIG language, UINTBIG type)
 
 			/* convert the result to the desired type */
 			addr = tcl_converttoelectric(tcl_interp->result, type);
-			retvar = db_enterqueryincache(code, type, addr);
+			retvar = db_enterqueryincache(code, type, type, addr, oaddr, otype);
 			return(retvar);
 #endif
 
@@ -208,8 +279,9 @@ VARIABLE *doquerry(char *code, INTBIG language, UINTBIG type)
 			str = java_query(code, &methodreturntype);
 			if (str == 0) return(NOVARIABLE);
 			getsimpletype(str, &methodreturntype, &addr, 0);
-			type = ((type) & ~VTYPE) | (methodreturntype & VTYPE);
-			retvar = db_enterqueryincache(code, type, addr);
+			reqtype = type;
+			type = (type & ~VTYPE) | (methodreturntype & VTYPE);
+			retvar = db_enterqueryincache(code, type, reqtype, addr, oaddr, otype);
 			return(retvar);
 #endif
 
@@ -316,7 +388,7 @@ VARIABLE *doquerry(char *code, INTBIG language, UINTBIG type)
 				retarray[0] = addr;
 				addr = (INTBIG)retarray;
 			}
-			retvar = db_enterqueryincache(code, *type, addr);
+			retvar = db_enterqueryincache(code, *type, *type, addr, oaddr, otype);
 			return(retvar);
 #endif
 	}
@@ -480,49 +552,57 @@ BOOLEAN languageconverse(INTBIG language)
 #if LANGLISP || LANGTCL || LANGJAVA || LANGMM
 /*
  * Routine to enter code "code" which returns "addr" of type "type" into the
- * cache of evaluations.
+ * cache of evaluations.  The code sits on object "oaddr" of type "otype".
  */
-VARIABLE *db_enterqueryincache(char *code, INTBIG type, INTBIG addr)
+VARIABLE *db_enterqueryincache(char *code, INTBIG type, INTBIG reqtype, INTBIG addr, INTBIG oaddr, INTBIG otype)
 {
 	REGISTER VARIABLE *var;
+	REGISTER INTBIG oldeststamp, i;
+	REGISTER EVALUATIONCACHE *ec, *oldestec;
 
 	/* lock the shared resource (cache) */
 	if (db_multiprocessing) emutexlock(db_languagecachemutex);
 
 	/* if the cache is full, clear the oldest */
-	if (db_evaluationcachelast < db_evaluationcachesize)
+	oldestec = &db_evaluationcache[0];
+	oldeststamp = oldestec->usagestamp;
+	for(i=0; i<EVALUATIONCACHESIZE; i++)
 	{
-		efree((char *)db_evaluationcache[db_evaluationcachelast].code);
-		if ((db_evaluationcache[db_evaluationcachelast].type&VTYPE) == VSTRING)
-			efree((char *)db_evaluationcache[db_evaluationcachelast].value);
+		ec = &db_evaluationcache[i];
+		if (ec->usagestamp >= oldeststamp) continue;
+		oldestec = ec;
+		oldeststamp = ec->usagestamp;
+	}
+	if (oldestec->code != 0)
+	{
+		efree((char *)oldestec->code);
+		if ((oldestec->type&VTYPE) == VSTRING)
+			efree((char *)oldestec->value);
 	}
 
 	/* load the cache entry */
-	db_evaluationcache[db_evaluationcachelast].changetimestamp = db_changetimestamp;
-	db_evaluationcache[db_evaluationcachelast].traversaltimestamp = db_traversaltimestamp;
-	(void)allocstring(&db_evaluationcache[db_evaluationcachelast].code, code, db_cluster);
-	db_evaluationcache[db_evaluationcachelast].type = type;
+	oldestec->changetimestamp = db_changetimestamp;
+	oldestec->traversaltimestamp = db_traversaltimestamp;
+	oldestec->oaddr = oaddr;
+	oldestec->otype = otype;
+	(void)allocstring(&oldestec->code, code, db_cluster);
+	oldestec->type = type;
+	oldestec->reqtype = reqtype;
+	oldestec->usagestamp = db_evaluationusagetime;
+
 	if ((type&VTYPE) == VSTRING)
 	{
-		(void)allocstring((char **)&db_evaluationcache[db_evaluationcachelast].value,
-			(char *)addr, db_cluster);
+		(void)allocstring((char **)&oldestec->value, (char *)addr, db_cluster);
 	} else
 	{
-		db_evaluationcache[db_evaluationcachelast].value = addr;
+		oldestec->value = addr;
 	}
 
 	/* get the variable that describes it */
-	var = &db_evaluationcache[db_evaluationcachelast].var;
-	var->addr = db_evaluationcache[db_evaluationcachelast].value;
+	var = &oldestec->var;
+	var->addr = oldestec->value;
 	var->type = type;
 
-	/* advance the cache pointer */
-	db_evaluationcachelast++;
-	if (db_evaluationcachelast > db_evaluationcachesize)
-		db_evaluationcachesize = db_evaluationcachelast;
-	if (db_evaluationcachelast >= EVALUATIONCACHESIZE)
-		db_evaluationcachelast = 0;
-
 	/* unlock the shared resource (cache) */
 	if (db_multiprocessing) emutexunlock(db_languagecachemutex);
 
@@ -537,8 +617,9 @@ void db_termlanguage(void)
 {
 	REGISTER INTBIG i;
 
-	for(i=0; i<db_evaluationcachesize; i++)
+	for(i=0; i<EVALUATIONCACHESIZE; i++)
 	{
+		if (db_evaluationcache[i].code == 0) continue;
 		efree((char *)db_evaluationcache[i].code);
 		if ((db_evaluationcache[i].type&VTYPE) == VSTRING)
 			efree((char *)db_evaluationcache[i].value);
