@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: dblangelk.c
@@ -103,7 +104,7 @@ static ELKObject lsp_nextobject(ELKObject);
 static ELKObject lsp_gettool(ELKObject);
 static ELKObject lsp_maxtool(void);
 static ELKObject lsp_indextool(ELKObject);
-static ELKObject lsp_toolturnon(ELKObject, ELKObject);
+static ELKObject lsp_toolturnon(ELKObject);
 static ELKObject lsp_toolturnoff(ELKObject);
 static ELKObject lsp_getlibrary(ELKObject);
 static ELKObject lsp_newlibrary(ELKObject, ELKObject);
@@ -1114,18 +1115,14 @@ ELKObject lsp_indextool(ELKObject oindex)
 	return(Make_EElectric((INTBIG)&el_tools[aindex], T_ETool));
 }
 
-ELKObject lsp_toolturnon(ELKObject otool, ELKObject onocatchup)
+ELKObject lsp_toolturnon(ELKObject otool)
 {
 	REGISTER TOOL *tool;
-	INTBIG nocatchupn;
-	BOOLEAN nocatchup;
 
 	/* get inputs from LISP */
 	Ensure_Type(otool, T_ETool);   tool = ETOOL(otool)->handle;
-	if (lsp_getnumericobject(onocatchup, &nocatchupn)) return(Null);
-	nocatchup = (nocatchupn != 0 ? TRUE : FALSE);
 
-	toolturnon(tool, nocatchup);
+	toolturnon(tool);
 	return(Null);
 }
 
@@ -1761,7 +1758,7 @@ void init_lib_electric(void)
 	Define_Primitive((ELKObject(*)(ELLIPSIS))lsp_gettool, "gettool", 1, 1, EVAL);
 	Define_Primitive((ELKObject(*)(ELLIPSIS))lsp_maxtool, "maxtool", 0, 0, EVAL);
 	Define_Primitive((ELKObject(*)(ELLIPSIS))lsp_indextool, "indextool", 1, 1, EVAL);
-	Define_Primitive((ELKObject(*)(ELLIPSIS))lsp_toolturnon, "toolturnon", 2, 2, EVAL);
+	Define_Primitive((ELKObject(*)(ELLIPSIS))lsp_toolturnon, "toolturnon", 1, 1, EVAL);
 	Define_Primitive((ELKObject(*)(ELLIPSIS))lsp_toolturnoff, "toolturnoff", 1, 1, EVAL);
 
 	/* define the library predicates */
