@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: dblangtcl.c
@@ -930,22 +931,18 @@ int tcl_indextool(ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
 int tcl_toolturnon(ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
 {
 	REGISTER TOOL *tool;
-	REGISTER INTBIG nocatchupn;
-	REGISTER BOOLEAN nocatchup;
 
 	/* get input */
-	if (argc != 3)
+	if (argc != 2)
 	{
-		Tcl_AppendResult(interp, "Usage: toolturnon TOOL NOCATCHUP", (char *)NULL);
+		Tcl_AppendResult(interp, "Usage: toolturnon TOOL", (char *)NULL);
 		return TCL_ERROR;
 	}
 	tool = (TOOL *)tcl_converttowarnedelectric(argv[1], VTOOL, "first");
 	if (tool == NOTOOL) return TCL_ERROR;
-	nocatchupn = myatoi(argv[2]);
-	nocatchup = nocatchupn != 0 ? TRUE : FALSE;
 
 	/* call Electric */
-	toolturnon(tool, nocatchup);
+	toolturnon(tool);
 	return TCL_OK;
 }
 
