@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: dblibrary.c
@@ -415,6 +416,9 @@ void changelambda(INTBIG count, TECHNOLOGY **techarray, INTBIG *newlam,
 		lib->temp2 = 0;
 		if (how == 1 && whichlib != lib) continue;
 
+		/* announce start of changes to this library */
+		startobjectchange((INTBIG)lib, VLIBRARY);
+
 		/* preserve lambda for this library */
 		oldlam = (INTBIG *)emalloc(count * SIZEOFINTBIG, el_tempcluster);
 		if (oldlam == 0) return;
@@ -426,6 +430,7 @@ void changelambda(INTBIG count, TECHNOLOGY **techarray, INTBIG *newlam,
 		}
 
 		lib->temp2 = (INTBIG)oldlam;
+
 	}
 
 	/* mark all facets in the library for scaling */
@@ -448,7 +453,11 @@ void changelambda(INTBIG count, TECHNOLOGY **techarray, INTBIG *newlam,
 	{
 		if (how == 1 && whichlib != lib) continue;
 		efree((char *)lib->temp2);
+
+		/* announce end of changes to this library */
+		endobjectchange((INTBIG)lib, VLIBRARY);
 	}
+
 }
 
 /*
@@ -489,8 +498,10 @@ void db_scalefacet(NODEPROTO *np, INTBIG count, TECHNOLOGY **techarray, INTBIG *
 		for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 			db_scalearcinst(ai, newlam, oldlam);
 
-		/* scale the facet and facet-center */
+		/* scale the facet */
 		db_boundfacet(np, &np->lowx, &np->highx, &np->lowy, &np->highy);
+
+		/* scale variables such as facet-center and characteristic spacing */
 		var = getvalkey((INTBIG)np, VNODEPROTO, VINTEGER|VISARRAY, el_prototype_center_key);
 		if (var != NOVARIABLE)
 		{
@@ -498,6 +509,13 @@ void db_scalefacet(NODEPROTO *np, INTBIG count, TECHNOLOGY **techarray, INTBIG *
 			position[0] = muldiv(position[0], newlam, oldlam);
 			position[1] = muldiv(position[1], newlam, oldlam);
 		}
+		var = getval((INTBIG)np, VNODEPROTO, VINTEGER|VISARRAY, "FACET_characteristic_spacing");
+		if (var != NOVARIABLE)
+		{
+			position = (INTBIG *)var->addr;
+			position[0] = muldiv(position[0], newlam, oldlam);
+			position[1] = muldiv(position[1], newlam, oldlam);
+		}
 	}
 }
 
