@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: dbmemory.c
@@ -35,7 +36,7 @@
 #  include <unistd.h>
 #endif
 
-#ifdef	MACOS
+#if defined(MACOS) && !defined(MACOSX)
 #  include <Memory.h>
 #  define MUSTALLOW 32768L				/* leave this much unallocated for system uses */
 #  define LOWLEVELALLOC NewPtr			/* system-routine to allocate memory */
@@ -45,7 +46,7 @@
 #  define LOWLEVELFREE  free			/* system-routine to deallocate memory */
 #endif
 
-#if defined(MACOS) || defined(DEBUGMEMORY)	/* memory clustering only on MAC (or if debugging memory) */
+#if (defined(MACOS) && !defined(MACOSX)) || defined(DEBUGMEMORY)	/* memory clustering only on MAC (or if debugging memory) */
 #  define USECLUSTERS  1
 #endif
 
@@ -335,7 +336,7 @@ INTBIG *emalloc(INTBIG amount, CLUSTER *cluster)
 	if (cluster == NOCLUSTER) ttyputerr(_("Warning: allocated %ld bytes from null cluster!"), amount);
 
 	/* block must be nonzero length */
-	if (amount == 0) amount = 1;
+	if (amount <= 0) amount = 1;
 
 	/* block must be a word amount */
 	amount = (amount + SIZEOFINTBIG - 1) & ~(SIZEOFINTBIG - 1);
@@ -743,7 +744,7 @@ char *db_getpages(INTBIG size)
 		if (db_sparepagecount >= SPAREPAGELIMIT)
 		{
 			/* no room in table, look for smallest entry and overwrite it */
-			smallest = db_sparepages[0].pagecount + 1;
+			smallest = db_sparepages[newcount = 0].pagecount + 1;
 			for(i=1; i<db_sparepagecount; i++)
 				if (db_sparepages[i].pagecount < smallest)
 					smallest = db_sparepages[newcount = i].pagecount;
@@ -930,7 +931,7 @@ INTBIG getpagesize(void)
  */
 char *valloc(INTBIG nbytes)
 {
-#  ifdef MACOS
+#  if defined(MACOS) && !defined(MACOSX)
 	char *ptr;
 	INTBIG left, dummy;
 
