@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: dbnoproto.c
@@ -35,14 +36,38 @@
 
 #define PRINTFRAMEQLAMBDA 60		/* size of text in frames (in quarter lambda units) */
 
+/*
+ * The new traversal scheme stores on first-class objects in the facet.
+ * If NEWTRAVERSAL is defined, then NODEPROTO must have these fields:
+ *	NODEINST           *parentinst;
+ *	INTBIG              parentinstindex;
+ *	INTBIG              parentinsttimestamp;
+ *	NODEINST           *parentinstuser;
+ */
+/* #define NEWTRAVERSAL 1 */
+
 /* hierarchical traversal data structures */
-static NODEINST **db_hierpath;
-static INTBIG    *db_hierindex;
-static INTBIG     db_hierdepth = 0;
+typedef struct
+{
+	NODEINST **path;
+	INTBIG    *index;
+	INTBIG     depth;
+	INTBIG     limit;
+} HTRAVERSAL;
+
+static HTRAVERSAL db_hiertraverse = {0, 0, 0, 0};
+
 static BOOLEAN    db_hiertraversing = FALSE;
 static INTBIG     db_hiertempclimb = 0;
        UINTBIG    db_traversaltimestamp = 0;
-static INTBIG     db_hierlimit = 0;
+
+#ifdef NEWTRAVERSAL
+static INTBIG     db_hiercalldepth;
+static INTBIG     db_htraversaltimestamp = 1;
+static NODEPROTO *db_hierbottom = NONODEPROTO;
+#else
+static INTBIG     db_descent_path_key = 0;	/* variable key for "USER_descent_path" */
+#endif
 
 /* prototypes for local routines */
 static INTBIG  db_schematic_pages(NODEPROTO*);
@@ -54,17 +79,17 @@ static void    db_changeallports(PORTPROTO*);
 static BOOLEAN db_addtohierpath(NODEINST *ni, INTBIG index, BOOLEAN bottom);
 static void    db_insertcell(CELL *cell);
 static void    db_retractcell(CELL *cell);
-static INTBIG  db_namehash(char *name);
+static void    db_copyhierarchicaltraversal(HTRAVERSAL *src, HTRAVERSAL *dst);
 
 /*
  * Routine to free all memory associated with this module.
  */
 void db_freenoprotomemory(void)
 {
-	if (db_hierlimit > 0)
+	if (db_hiertraverse.limit > 0)
 	{
-		efree((char *)db_hierpath);
-		efree((char *)db_hierindex);
+		efree((char *)db_hiertraverse.path);
+		efree((char *)db_hiertraverse.index);
 	}		
 }
 
@@ -142,6 +167,8 @@ NODEPROTO *allocnodeproto(CLUSTER *cluster)
 	np->firstarcinst = NOARCINST;
 	np->firstportproto = NOPORTPROTO;
 	np->cachedequivfacet = NONODEPROTO;
+	np->numportprotos = 0;
+	np->portprotohashtablesize = 0;
 	np->tech = NOTECHNOLOGY;
 	np->nextnodeproto = NONODEPROTO;
 	np->cell = NOCELL;
@@ -178,6 +205,7 @@ void freenodeproto(NODEPROTO *np)
 		efree((char *)np->globalnetworks);
 		efree((char *)np->globalnetnames);
 	}
+	if (np->portprotohashtablesize > 0) efree((char *)np->portprotohashtable);
 	if (np->numvar != 0) db_freevars(&np->firstvar, &np->numvar);
 	efree((char *)np);
 }
@@ -322,8 +350,8 @@ NODEPROTO *newnodeproto(char *fname, LIBRARY *lib)
 	np->cell = c;
 	np->cellview = view;
 	np->tech = whattech(np);
-	np->creationdate = getcurrenttime();
-	np->revisiondate = getcurrenttime();
+	np->creationdate = (UINTBIG)getcurrenttime();
+	np->revisiondate = (UINTBIG)getcurrenttime();
 
 	/* determine version number of this facet */
 	if (explicitversion > 0)
@@ -801,7 +829,7 @@ CELL *db_findcellname(char *name, LIBRARY *lib)
 #define	HASH_MSK	((1 << HASH_SH) - 1)
 
 /*
- * Routine to encode a cell name for the hash table.
+ * Routine to encode a cell or portproto name for the hash table.
  */
 INTBIG db_namehash(char *name)
 {
@@ -836,22 +864,88 @@ NODEPROTO *getcurfacet(void)
  * is the top of the hierarchy, and the last entry is the instance in whose definition
  * the traversal currently resides.
  */
-void gettraversalpath(NODEPROTO *here, NODEINST ***nilist, INTBIG **indexlist, INTBIG *depth)
+void gettraversalpath(NODEPROTO *here, NODEINST ***nilist, INTBIG **indexlist, INTBIG *depth, INTBIG maxdepth)
 {
+#ifdef NEWTRAVERSAL
+	REGISTER NODEPROTO *np;
+	REGISTER NODEINST *ni;
+	INTBIG index;
+	REGISTER INTBIG i, in, otheri, upperlimit;
+
+	db_hiertraverse.depth = 0;
+	if (maxdepth > 0) maxdepth += db_hiertempclimb; else
+		maxdepth = MAXINTBIG;
+	if (db_hiertraversing)
+	{
+		if (db_hiercalldepth > 0)
+		{
+			if (db_hierbottom != NONODEPROTO) np = db_hierbottom; else
+				np = here;
+			for(i=0; i<db_hiercalldepth; i++)
+			{
+				ni = np->parentinst;
+				index = np->parentinstindex;
+				if (db_addtohierpath(ni, index, TRUE)) break;
+				np = ni->parent;
+				if (db_hiertraverse.depth >= maxdepth) break;
+			}
+		} else np = here;
+
+		db_hiertraversing = FALSE;
+		for(;;)
+		{
+			if (db_hiertraverse.depth >= maxdepth) break;
+			if (np == NONODEPROTO) break;
+			ni = descentparent(np, &index);
+			if (ni == NONODEINST) break;
+			if (db_addtohierpath(ni, index, TRUE)) break;
+			np = ni->parent;
+		}
+		db_hiertraversing = TRUE;
+	} else
+	{
+		np = here;
+		for(;;)
+		{
+			if (db_hiertraverse.depth >= maxdepth) break;
+			if (np == NONODEPROTO) break;
+			ni = descentparent(np, &index);
+			if (ni == NONODEINST) break;
+			if (db_addtohierpath(ni, index, TRUE)) break;
+			np = ni->parent;
+		}
+	}
+
+	/* reverse the list */
+	if (db_hiertraverse.depth > 1)
+	{
+		upperlimit = db_hiertraverse.depth / 2;
+		for(i=0; i<upperlimit; i++)
+		{
+			otheri = db_hiertraverse.depth - i - 1;
+			ni = db_hiertraverse.path[i];   db_hiertraverse.path[i] = db_hiertraverse.path[otheri];    db_hiertraverse.path[otheri] = ni;
+			in = db_hiertraverse.index[i];  db_hiertraverse.index[i] = db_hiertraverse.index[otheri];  db_hiertraverse.index[otheri] = in;
+		}
+	}
+	*depth = db_hiertraverse.depth - db_hiertempclimb;
+	*indexlist = db_hiertraverse.index;
+	*nilist = db_hiertraverse.path;
+#else
 	REGISTER NODEPROTO *np, *cnp;
 	REGISTER NODEINST *ni;
+	INTBIG index;
 
 	if (db_hiertraversing)
 	{
 		/* see if any additional hierarchy can be found at the top of the list */
-		if (db_hierdepth > 0) np = db_hierpath[0]->parent; else
+		if (db_hiertraverse.depth > 0) np = db_hiertraverse.path[0]->parent; else
 			np = here;
 		if (np != NONODEPROTO)
 		{
 			for(;;)
 			{
 				if (np == NONODEPROTO) break;
-				ni = descentparent(np);
+				ni = descentparent(np, &index);
 				if (ni == NONODEINST) break;
 				if (db_addtohierpath(ni, 0, FALSE)) break;
 				np = ni->parent;
@@ -859,10 +953,9 @@ void gettraversalpath(NODEPROTO *here, NODEINST ***nilist, INTBIG **indexlist, I
 				if (cnp != NONODEPROTO) np = cnp;
 			}
 		}
-
-		*depth = db_hierdepth - db_hiertempclimb;
-		*indexlist = db_hierindex;
-		*nilist = db_hierpath;
+		*depth = db_hiertraverse.depth - db_hiertempclimb;
+		*indexlist = db_hiertraverse.index;
+		*nilist = db_hiertraverse.path;
 	} else
 	{
 		/* not in an active traversal, see if one exists from edit sequences */
@@ -870,18 +963,19 @@ void gettraversalpath(NODEPROTO *here, NODEINST ***nilist, INTBIG **indexlist, I
 		for(;;)
 		{
 			if (np == NONODEPROTO) break;
-			ni = descentparent(np);
+			ni = descentparent(np, &index);
 			if (ni == NONODEINST) break;
 			if (db_addtohierpath(ni, 0, FALSE)) break;
 			np = ni->parent;
 			cnp = contentsview(np);
 			if (cnp != NONODEPROTO) np = cnp;
 		}
-		*depth = db_hierdepth - db_hiertempclimb;
-		*indexlist = db_hierindex;
-		*nilist = db_hierpath;
-		db_hierdepth = 0;
+		*depth = db_hiertraverse.depth - db_hiertempclimb;
+		*indexlist = db_hiertraverse.index;
+		*nilist = db_hiertraverse.path;
+		db_hiertraverse.depth = 0;
 	}
+#endif
 }
 
 /*
@@ -891,10 +985,16 @@ void gettraversalpath(NODEPROTO *here, NODEINST ***nilist, INTBIG **indexlist, I
  * hierarchy, and the last entry is the instance in whose definition the traversal
  * currently resides.
  */
-void db_gettraversalpath(NODEINST ***nilist, INTBIG *depth)
+void db_gettraversalpath(NODEPROTO *here, NODEINST ***nilist, INTBIG *depth)
 {
-	*nilist = db_hierpath;
-	*depth = db_hierdepth - db_hiertempclimb;
+#ifdef NEWTRAVERSAL
+	INTBIG *indexlist;
+
+	gettraversalpath(here, nilist, &indexlist, depth, 0);
+#else
+	*nilist = db_hiertraverse.path;
+	*depth = db_hiertraverse.depth - db_hiertempclimb;
+#endif
 }
 
 /*
@@ -903,10 +1003,18 @@ void db_gettraversalpath(NODEINST ***nilist, INTBIG *depth)
  */
 void begintraversehierarchy(void)
 {
+#ifdef NEWTRAVERSAL
 	db_hiertraversing = TRUE;
-	db_hierdepth = 0;
+	db_hiercalldepth = 0;
 	db_hiertempclimb = 0;
 	db_traversaltimestamp++;
+	db_htraversaltimestamp++;
+#else
+	db_hiertraversing = TRUE;
+	db_hiertraverse.depth = 0;
+	db_hiertempclimb = 0;
+	db_traversaltimestamp++;
+#endif
 }
 
 /*
@@ -914,18 +1022,32 @@ void begintraversehierarchy(void)
  */
 void endtraversehierarchy(void)
 {
+#ifdef NEWTRAVERSAL
+	db_htraversaltimestamp++;
 	db_hiertraversing = FALSE;
+#else
+	db_hiertraversing = FALSE;
+#endif
 }
 
 /*
  * routine to traversal down the hierarchy, into nodeinst "ni", index "index" of
  * the array (if the node is arrayed).
  */
-void downhierarchy(NODEINST *ni, INTBIG index)
+void downhierarchy(NODEINST *ni, NODEPROTO *np, INTBIG index)
 {
+#ifdef NEWTRAVERSAL
+	np->parentinst = ni;
+	np->parentinstindex = index;
+	np->parentinsttimestamp = db_htraversaltimestamp;
+	db_traversaltimestamp++;
+	db_hierbottom = np;
+	db_hiercalldepth++;
+#else
 	/* add this node to the traversal stack */
 	if (db_addtohierpath(ni, index, TRUE)) return;
 	db_traversaltimestamp++;
+#endif
 }
 
 /*
@@ -950,43 +1072,43 @@ BOOLEAN db_addtohierpath(NODEINST *ni, INTBIG index, BOOLEAN bottom)
 	REGISTER NODEINST **nilist;
 
 	/* make sure there is room in the list of nodes traversed */
-	if (db_hierdepth >= db_hierlimit)
+	if (db_hiertraverse.depth >= db_hiertraverse.limit)
 	{
-		newlimit = db_hierlimit+10;
-		if (newlimit < db_hierdepth) newlimit = db_hierdepth;
+		newlimit = db_hiertraverse.limit+10;
+		if (newlimit < db_hiertraverse.depth) newlimit = db_hiertraverse.depth;
 		nilist = (NODEINST **)emalloc(newlimit * (sizeof (NODEINST *)), db_cluster);
 		if (nilist == 0) return(TRUE);
 		newindex = (INTBIG *)emalloc(newlimit * SIZEOFINTBIG, db_cluster);
 		if (newindex == 0) return(TRUE);
-		for(i=0; i<db_hierdepth; i++)
+		for(i=0; i<db_hiertraverse.depth; i++)
 		{
-			nilist[i] = db_hierpath[i];
-			newindex[i] = db_hierindex[i];
+			nilist[i] = db_hiertraverse.path[i];
+			newindex[i] = db_hiertraverse.index[i];
 		}
-		if (db_hierlimit > 0)
+		if (db_hiertraverse.limit > 0)
 		{
-			efree((char *)db_hierpath);
-			efree((char *)db_hierindex);
+			efree((char *)db_hiertraverse.path);
+			efree((char *)db_hiertraverse.index);
 		}
-		db_hierpath = nilist;
-		db_hierindex = newindex;
-		db_hierlimit = newlimit;
+		db_hiertraverse.path = nilist;
+		db_hiertraverse.index = newindex;
+		db_hiertraverse.limit = newlimit;
 	}
 	if (bottom)
 	{
-		db_hierpath[db_hierdepth] = ni;
-		db_hierindex[db_hierdepth] = index;
+		db_hiertraverse.path[db_hiertraverse.depth] = ni;
+		db_hiertraverse.index[db_hiertraverse.depth] = index;
 	} else
 	{
-		for(i=db_hierdepth; i>0; i--)
+		for(i=db_hiertraverse.depth; i>0; i--)
 		{
-			db_hierpath[i] = db_hierpath[i-1];
-			db_hierindex[i] = db_hierindex[i-1];
+			db_hiertraverse.path[i] = db_hiertraverse.path[i-1];
+			db_hiertraverse.index[i] = db_hiertraverse.index[i-1];
 		}
-		db_hierpath[0] = ni;
-		db_hierindex[0] = index;
+		db_hiertraverse.path[0] = ni;
+		db_hiertraverse.index[0] = index;
 	}
-	db_hierdepth++;
+	db_hiertraverse.depth++;
 	return(FALSE);
 }
 
@@ -995,11 +1117,18 @@ BOOLEAN db_addtohierpath(NODEINST *ni, INTBIG index, BOOLEAN bottom)
  */
 void uphierarchy(void)
 {
-	if (db_hierdepth <= 0) return;
+#ifdef NEWTRAVERSAL
+	db_traversaltimestamp++;
+	if (db_hiercalldepth > 0) db_hiercalldepth--;
+	if (db_hiercalldepth == 0 || db_hierbottom == NONODEPROTO) db_hierbottom = NONODEPROTO; else
+		db_hierbottom = db_hierbottom->parentinst->parent;
+#else
+	if (db_hiertraverse.depth <= 0) return;
 
 	/* pop the node off the traversal stack */
-	db_hierdepth--;
+	db_hiertraverse.depth--;
 	db_traversaltimestamp++;
+#endif
 }
 
 /*
@@ -1009,17 +1138,53 @@ void uphierarchy(void)
  * not only extracts this information, but it verifies that the instance still
  * exists.  Returns NONODEINST if there is no parent.
  */
-NODEINST *descentparent(NODEPROTO *np)
+NODEINST *descentparent(NODEPROTO *np, INTBIG *index)
 {
+#ifdef NEWTRAVERSAL
+	REGISTER NODEINST *ni, *vni;
+	REGISTER NODEPROTO *inp;
+
+	if (db_hiertraversing)
+	{
+		ni = np->parentinst;
+		*index = np->parentinstindex;
+	} else
+	{
+		ni = np->parentinstuser;
+		*index = 0;
+	}
+	if (ni == NONODEINST) return(NONODEINST);
+
+	/* quick validity check */
+	if (ni->parent == np) return(NONODEINST);
+
+	/* if the timestamps match, no need to validate */
+	if (np->parentinsttimestamp == db_htraversaltimestamp) return(ni);
+
+	/* validate this pointer: node must be a current instance of this facet */
+	for(vni = np->firstinst; vni != NONODEINST; vni = vni->nextinst)
+		if (vni == ni) return(ni);
+
+	/* might be an icon */
+	for(inp = iconview(np); inp != NONODEPROTO; inp = inp->prevversion)
+	{
+		for(vni = inp->firstinst; vni != NONODEINST; vni = vni->nextinst)
+			if (vni == ni) return(ni);
+	}
+	return(NONODEINST);
+#else
 	REGISTER VARIABLE *var;
 	REGISTER NODEINST *ni, *vni;
 	REGISTER NODEPROTO *inp;
 
 	/* find this level in the stack */
-	var = getvalkey((INTBIG)np, VNODEPROTO, VNODEINST, us_descent_path_key);
+	if (db_descent_path_key == 0)
+		db_descent_path_key = makekey("USER_descent_path");
+	var = getvalkey((INTBIG)np, VNODEPROTO, VNODEINST, db_descent_path_key);
 	if (var == NOVARIABLE) return(NONODEINST);
 	ni = (NODEINST *)var->addr;
 	if (ni->parent == np) return(NONODEINST);
+	*index = 0;
 
 	/* validate this pointer: node must be a current instance of this facet */
 	for(vni = np->firstinst; vni != NONODEINST; vni = vni->nextinst)
@@ -1033,6 +1198,105 @@ NODEINST *descentparent(NODEPROTO *np)
 	}
 
 	return(NONODEINST);
+#endif
+}
+
+
+void sethierarchicalparent(NODEPROTO *np, NODEINST *parentni)
+{
+#ifdef NEWTRAVERSAL
+	np->parentinstuser = parentni;
+#else
+	if (db_descent_path_key == 0)
+		db_descent_path_key = makekey("USER_descent_path");
+	(void)setvalkey((INTBIG)np, VNODEPROTO, db_descent_path_key, (INTBIG)parentni,
+		VNODEINST|VDONTSAVE);
+#endif
+}
+
+/*
+ * Routine to create a hierarchical traversal snapshot object and return it.
+ * Returns zero on error.
+ */
+void *newhierarchicaltraversal(void)
+{
+	REGISTER HTRAVERSAL *ht;
+
+	ht = (HTRAVERSAL *)emalloc(sizeof (HTRAVERSAL), db_cluster);
+	if (ht == 0) return(0);
+	ht->limit = 0;
+	return((void *)ht);
+}
+
+/*
+ * Routine to take a snapshot of the current hierarchical traversal and
+ * save it in "snapshot" (an object returned by "newhierarchicaltraversal()").
+ */
+void gethierarchicaltraversal(void *snapshot)
+{
+	REGISTER HTRAVERSAL *ht;
+
+	ht = (HTRAVERSAL *)snapshot;
+	db_copyhierarchicaltraversal(&db_hiertraverse, ht);
+}
+
+/*
+ * Routine to set saved hierarchical traversal "snapshot" as the current hierarchical traversal.
+ */
+void sethierarchicaltraversal(void *snapshot)
+{
+	REGISTER HTRAVERSAL *ht;
+
+	ht = (HTRAVERSAL *)snapshot;
+	db_copyhierarchicaltraversal(ht, &db_hiertraverse);
+}
+
+/*
+ * Routine to delete a saved hierarchy traversal snapshot.
+ */
+void killhierarchicaltraversal(void *snapshot)
+{
+	REGISTER HTRAVERSAL *ht;
+
+	ht = (HTRAVERSAL *)snapshot;
+	if (ht->limit > 0)
+	{
+		efree((char *)ht->path);
+		efree((char *)ht->index);
+	}
+	efree((char *)ht);
+}
+
+/*
+ * Internal routine to copy hierarchical traversal stacks.
+ */
+void db_copyhierarchicaltraversal(HTRAVERSAL *src, HTRAVERSAL *dst)
+{
+	REGISTER INTBIG i, *newindex;
+	REGISTER NODEINST **newpath;
+
+	if (src->depth > dst->limit)
+	{
+		newpath = (NODEINST **)emalloc(src->depth * (sizeof (NODEINST *)), db_cluster);
+		if (newpath == 0) return;
+		newindex = (INTBIG *)emalloc(src->depth * SIZEOFINTBIG, db_cluster);
+		if (newindex == 0) return;
+
+		if (dst->limit > 0)
+		{
+			efree((char *)dst->path);
+			efree((char *)dst->index);
+		}
+		dst->path = newpath;
+		dst->index = newindex;
+		dst->limit = src->depth;
+	}
+	for(i=0; i<src->depth; i++)
+	{
+		dst->path[i] = src->path[i];
+		dst->index[i] = src->index[i];
+	}
+	dst->depth = src->depth;
 }
 
 /************************* VIEWS *************************/
@@ -1374,12 +1638,8 @@ PORTPROTO *equivalentport(NODEPROTO *np, PORTPROTO *pp, NODEPROTO *enp)
 			opp->cachedequivport = NOPORTPROTO;
 		for(opp = np->firstportproto; opp != NOPORTPROTO; opp = opp->nextportproto)
 		{
-			for(epp = enp->firstportproto; epp != NOPORTPROTO; epp = epp->nextportproto)
-			{
-				if (namesame(epp->protoname, opp->protoname) != 0) continue;
-				opp->cachedequivport = epp;
-				break;
-			}
+			epp = getportproto(enp, opp->protoname);
+			if (epp != NOPORTPROTO) opp->cachedequivport = epp;
 		}
 		np->cachedequivfacet = enp;
 	}
@@ -1590,7 +1850,7 @@ void framepoly(INTBIG boxnum, POLYGON *poly, NODEPROTO *np)
 		poly->string = line;
 	} else if (boxnum < xsections*2+ysections*2-2+xsections*2+ysections*2)
 	{
-		/* section numbers along top and bottom */
+		/* section numbers along left and right */
 		pindex = (boxnum-2-(xsections-1)*2-(ysections-1)*2-xsections*2) % ysections;
 		poly->yv[0] = poly->yv[1] = pindex * ysecsize - (schysize/2 - framewid);
 		poly->yv[2] = poly->yv[3] = poly->yv[0] + ysecsize;
@@ -1672,7 +1932,7 @@ void framepoly(INTBIG boxnum, POLYGON *poly, NODEPROTO *np)
 				break;
 			case 5:			/* creation date */
 				if (np == NONODEPROTO) break;
-				(void)sprintf(ndstring, _("Created: %s"), timetostring(np->creationdate));
+				(void)sprintf(ndstring, _("Created: %s"), timetostring((time_t)np->creationdate));
 				poly->xv[0] = poly->xv[3] = schxsize/2 - framewid - xlogobox;
 				poly->xv[1] = poly->xv[2] = schxsize/2 - framewid;
 				poly->yv[0] = poly->yv[1] = -schysize/2 + framewid + ylogobox / 6;
@@ -1694,7 +1954,7 @@ void framepoly(INTBIG boxnum, POLYGON *poly, NODEPROTO *np)
 				break;
 			case 7:			/* revision date */
 				if (np == NONODEPROTO) break;
-				(void)sprintf(ndstring, _("Revised: %s"), timetostring(np->revisiondate));
+				(void)sprintf(ndstring, _("Revised: %s"), timetostring((time_t)np->revisiondate));
 				poly->xv[0] = poly->xv[3] = schxsize/2 - framewid - xlogobox;
 				poly->xv[1] = poly->xv[2] = schxsize/2 - framewid;
 				poly->yv[0] = poly->yv[1] = -schysize/2 + framewid;
@@ -1857,9 +2117,12 @@ NODEPROTO *copynodeproto(NODEPROTO *fromnp, LIBRARY *tolib, char *toname)
 		for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 		{
 			pp = pi->proto;
-			for(ppt = lnt->firstportproto; ppt != NOPORTPROTO; ppt = ppt->nextportproto)
-				if (namesame(pp->protoname, ppt->protoname) == 0 && pp->connects == ppt->connects)
-					break;
+			ppt = getportproto(lnt, pp->protoname);
+			if (ppt != NOPORTPROTO)
+			{
+				/* the connections must match, too */
+				if (pp->connects != ppt->connects) ppt = NOPORTPROTO;
+			}
 			if (ppt == NOPORTPROTO)
 			{
 				ttyputerr(_("Cannot use subfacet %s in library %s: exports don't match"),
@@ -1871,9 +2134,12 @@ NODEPROTO *copynodeproto(NODEPROTO *fromnp, LIBRARY *tolib, char *toname)
 		for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
 		{
 			pp = pe->proto;
-			for(ppt = lnt->firstportproto; ppt != NOPORTPROTO; ppt = ppt->nextportproto)
-				if (namesame(pp->protoname, ppt->protoname) == 0 && pp->connects == ppt->connects)
-					break;
+			ppt = getportproto(lnt, pp->protoname);
+			if (ppt != NOPORTPROTO)
+			{
+				/* the connections must match, too */
+				if (pp->connects != ppt->connects) ppt = NOPORTPROTO;
+			}
 			if (ppt == NOPORTPROTO)
 			{
 				ttyputerr(_("Cannot use subfacet %s in library %s: exports don't match"),
@@ -1959,12 +2225,8 @@ NODEPROTO *copynodeproto(NODEPROTO *fromnp, LIBRARY *tolib, char *toname)
 			{
 				/* facets associate ports by name */
 				pp = ai->end[i].portarcinst->proto;
-				for(ppt = ono[i]->proto->firstportproto; ppt != NOPORTPROTO; ppt = ppt->nextportproto)
-					if (namesame(ppt->protoname, pp->protoname) == 0)
-				{
-					opt[i] = ppt;
-					break;
-				}
+				ppt = getportproto(ono[i]->proto, pp->protoname);
+				if (ppt != NOPORTPROTO) opt[i] = ppt;
 			}
 			if (opt[i] == NOPORTPROTO)
 				ttyputerr(_("Error: no port for %s arc on %s node"), describearcproto(ai->proto),
@@ -2180,7 +2442,7 @@ NODEINST *replacenodeinst(NODEINST *ni, NODEPROTO *np, BOOLEAN ignoreportnames,
 	REGISTER PORTPROTO *pp, *opt;
 	INTBIG endx[2], endy[2], bx, by, cx, cy, lxo, hxo, lyo, hyo, lx, hx, ly, hy, psx, psy,
 		arcdx, arcdy, arccount, thisend, xp, yp, other;
-	REGISTER INTBIG i, zigzag, ang, fixedalignment, dx, dy, alignment;
+	REGISTER INTBIG i, zigzag, ang, fixedalignment, dx, dy, alignment, lambda;
 	XARRAY trans;
 	REGISTER VARIABLE *varold, *varnew;
 	NODEINST *endnodeinst[2];
@@ -2238,7 +2500,8 @@ NODEINST *replacenodeinst(NODEINST *ni, NODEPROTO *np, BOOLEAN ignoreportnames,
 			if (ni->highx-ni->lowx != np->highx-np->lowx ||
 				ni->highy-ni->lowy != np->highy-np->lowy)
 			{
-				alignment = muldiv(us_alignment_ratio, el_curlib->lambda[el_curtech->techindex], WHOLE);
+				lambda = lambdaoffacet(np);
+				alignment = muldiv(us_alignment_ratio, lambda, WHOLE);
 				dx = lx - us_alignvalue(lx, alignment, &other);
 				lx -= dx;   hx -= dx;
 				dy = ly - us_alignvalue(ly, alignment, &other);
@@ -2318,7 +2581,8 @@ NODEINST *replacenodeinst(NODEINST *ni, NODEPROTO *np, BOOLEAN ignoreportnames,
 		if (arccount > 0)
 		{
 			arcdx /= arccount;   arcdy /= arccount;
-			alignment = muldiv(us_alignment_ratio, el_curlib->lambda[el_curtech->techindex], WHOLE);
+			lambda = lambdaoffacet(np);
+			alignment = muldiv(us_alignment_ratio, lambda, WHOLE);
 			arcdx = us_alignvalue(arcdx, alignment, &other);
 			arcdy = us_alignvalue(arcdy, alignment, &other);
 			if (arcdx != 0 || arcdy != 0)
