@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: dbtech.c
@@ -35,6 +36,7 @@
 #include "tecgen.h"
 #include "tecart.h"
 #include "tecschem.h"
+#include "tecmocmos.h"
 #include "drc.h"
 #include "efunction.h"
 #include <math.h>
@@ -89,7 +91,11 @@ static char     ***tech_drcuncondistancewrule = 0;	/* cache for "drcmindistance"
 static char     ***tech_drccondistancemrule = 0;	/* cache for "drcmindistance" */
 static char     ***tech_drcuncondistancemrule = 0;	/* cache for "drcmindistance" */
 static char     ***tech_drcedgedistancerule = 0;	/* cache for "drcmindistance" */
-
+#ifdef SURROUNDRULES
+static INTBIG    **tech_drcsurroundlayerpairs = 0;	/* cache for "drcsurroundrules" */
+static INTBIG    **tech_drcsurrounddistances = 0;	/* cache for "drcsurroundrules" */
+static char     ***tech_drcsurroundrule = 0;		/* cache for "drcsurroundrules" */
+#endif
 
 #define NOROUTINELIST ((ROUTINELIST *)-1)
 
@@ -108,8 +114,6 @@ void   tech_initmaxdrcsurround(void);
 /* prototypes for local routines */
 static INTBIG tech_getdrcmindistance(TECHNOLOGY*, INTBIG, INTBIG, BOOLEAN, INTBIG, BOOLEAN, INTBIG*, char **);
 static void   tech_3ddefaultlayerheight(TECHNOLOGY *tech);
-static void   tech_nodeprotosizeoffset(NODEPROTO *np, INTBIG *lx, INTBIG *ly, INTBIG *hx, INTBIG *hy,
-				INTBIG lambda);
 static INTBIG tech_arcprotowidthoffset(ARCPROTO *ap, INTBIG lambda);
 static void   tech_initnodesizeoffset(void);
 static void   tech_initlayerfunction(void);
@@ -157,6 +161,11 @@ void db_freetechnologymemory(void)
 	if (tech_drccondistancemrule != 0) efree((char *)tech_drccondistancemrule);
 	if (tech_drcuncondistancemrule != 0) efree((char *)tech_drcuncondistancemrule);
 	if (tech_drcedgedistancerule != 0) efree((char *)tech_drcedgedistancerule);
+#ifdef SURROUNDRULES
+	if (tech_drcsurroundlayerpairs != 0) efree((char *)tech_drcsurroundlayerpairs);
+	if (tech_drcsurrounddistances != 0) efree((char *)tech_drcsurrounddistances);
+	if (tech_drcsurroundrule != 0) efree((char *)tech_drcsurroundrule);
+#endif
 
 	if (db_3darraylength > 0)
 	{
@@ -525,6 +534,11 @@ void tech_initmaxdrcsurround(void)
 	if (tech_drccondistancemrule != 0) efree((char *)tech_drccondistancemrule);
 	if (tech_drcuncondistancemrule != 0) efree((char *)tech_drcuncondistancemrule);
 	if (tech_drcedgedistancerule != 0) efree((char *)tech_drcedgedistancerule);
+#ifdef SURROUNDRULES
+	if (tech_drcsurroundrule != 0) efree((char *)tech_drcsurroundrule);
+	if (tech_drcsurroundlayerpairs != 0) efree((char *)tech_drcsurroundlayerpairs);
+	if (tech_drcsurrounddistances != 0) efree((char *)tech_drcsurrounddistances);
+#endif
 
 	tech_drcmaxdistances = (INTBIG **)emalloc((el_maxtech * (sizeof (INTBIG *))), db_cluster);
 	if (tech_drcmaxdistances == 0) return;
@@ -566,6 +580,14 @@ void tech_initmaxdrcsurround(void)
 	if (tech_drcuncondistancemrule == 0) return;
 	tech_drcedgedistancerule = (char ***)emalloc((el_maxtech * (sizeof (char **))), db_cluster);
 	if (tech_drcedgedistancerule == 0) return;
+#ifdef SURROUNDRULES
+	tech_drcsurroundlayerpairs = (INTBIG **)emalloc((el_maxtech * (sizeof (INTBIG *))), db_cluster);
+	if (tech_drcsurroundlayerpairs == 0) return;
+	tech_drcsurrounddistances = (INTBIG **)emalloc((el_maxtech * (sizeof (INTBIG *))), db_cluster);
+	if (tech_drcsurrounddistances == 0) return;
+	tech_drcsurroundrule = (char ***)emalloc((el_maxtech * (sizeof (char **))), db_cluster);
+	if (tech_drcsurroundrule == 0) return;
+#endif
 
 	for(t = el_technologies; t != NOTECHNOLOGY; t = t->nexttechnology)
 	{
@@ -589,6 +611,11 @@ void tech_initmaxdrcsurround(void)
 		tech_drccondistancemrule[t->techindex] = 0;
 		tech_drcuncondistancemrule[t->techindex] = 0;
 		tech_drcedgedistancerule[t->techindex] = 0;
+#ifdef SURROUNDRULES
+		tech_drcsurroundrule[t->techindex] = 0;
+		tech_drcsurroundlayerpairs[t->techindex] = 0;
+		tech_drcsurrounddistances[t->techindex] = 0;
+#endif
 
 		var = getvalkey((INTBIG)t, VTECHNOLOGY, VFRACT, dr_wide_limitkey);
 		if (var != NOVARIABLE) tech_drcwidelimit[t->techindex] = var->addr;
@@ -614,7 +641,6 @@ void tech_initmaxdrcsurround(void)
 		if (var != NOVARIABLE) tech_drcuncondistancem[t->techindex] = (INTBIG *)var->addr;
 		var = getvalkey((INTBIG)t, VTECHNOLOGY, VFRACT|VISARRAY, dr_edge_distanceskey);
 		if (var != NOVARIABLE) tech_drcedgedistance[t->techindex] = (INTBIG *)var->addr;
-
 		var = getvalkey((INTBIG)t, VTECHNOLOGY, VSTRING|VISARRAY, dr_connected_distances_rulekey);
 		if (var != NOVARIABLE) tech_drccondistancerule[t->techindex] = (char **)var->addr;
 		var = getvalkey((INTBIG)t, VTECHNOLOGY, VSTRING|VISARRAY, dr_unconnected_distances_rulekey);
@@ -629,6 +655,14 @@ void tech_initmaxdrcsurround(void)
 		if (var != NOVARIABLE) tech_drcuncondistancemrule[t->techindex] = (char **)var->addr;
 		var = getvalkey((INTBIG)t, VTECHNOLOGY, VSTRING|VISARRAY, dr_edge_distances_rulekey);
 		if (var != NOVARIABLE) tech_drcedgedistancerule[t->techindex] = (char **)var->addr;
+#ifdef SURROUNDRULES
+		var = getvalkey((INTBIG)t, VTECHNOLOGY, VINTEGER|VISARRAY, dr_surround_layer_pairskey);
+		if (var != NOVARIABLE) tech_drcsurroundlayerpairs[t->techindex] = (INTBIG *)var->addr;
+		var = getvalkey((INTBIG)t, VTECHNOLOGY, VFRACT|VISARRAY, dr_surround_distanceskey);
+		if (var != NOVARIABLE) tech_drcsurrounddistances[t->techindex] = (INTBIG *)var->addr;
+		var = getvalkey((INTBIG)t, VTECHNOLOGY, VSTRING|VISARRAY, dr_surround_rulekey);
+		if (var != NOVARIABLE) tech_drcsurroundrule[t->techindex] = (char **)var->addr;
+#endif
 
 		/* compute max distances if there are any there */
 		if (tech_drcuncondistance[t->techindex] == 0) continue;
@@ -1536,6 +1570,21 @@ void transistorsize(NODEINST *ni, INTBIG *length, INTBIG *width)
 				nodesizeoffset(ni, &lx, &ly, &hx, &hy);
 				*length = ni->highy-hy - (ni->lowy+ly);
 				*width = ni->highx-hx - (ni->lowx+lx);
+
+				/* special case if scalable transistors */
+				if (ni->proto == mocmos_scalablentransprim || ni->proto == mocmos_scalableptransprim)
+				{
+					varw = getvalkey((INTBIG)ni, VNODEINST, -1, el_attrkey_width);
+					if (varw != NOVARIABLE)
+					{
+						lambda = lambdaofnode(ni);
+						pt = describevariable(varw, -1, -1);
+						if (*pt == '-' || *pt == '+' || isdigit(*pt))
+						{
+							*width = muldiv(atofr(pt), lambda, WHOLE);
+						}
+					}
+				}
 			}
 			break;
 
@@ -1554,7 +1603,9 @@ void transistorsize(NODEINST *ni, INTBIG *length, INTBIG *width)
 						*length = muldiv(atofr(pt), lambda, WHOLE);
 					}
 				}
-				if (varw == NOVARIABLE) *length = lambda * 2; else
+
+				/* why did JG change "width" to "length" below!!! */
+				if (varw == NOVARIABLE) *width = lambda * 3; else
 				{
 					pt = describevariable(varw, -1, -1);
 					if (*pt == '-' || *pt == '+' || isdigit(*pt))
@@ -1803,7 +1854,7 @@ void shapeportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, BOOLEAN purpose)
 			break;
 
 		default:
-			tech_fillportpoly(ni, pp, poly, *t1, thistn, CLOSED);
+			tech_fillportpoly(ni, pp, poly, *t1, thistn, CLOSED, lambdaofnode(ni));
 			break;
 	}
 }
@@ -1834,7 +1885,7 @@ void shapetransportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans
 			break;
 
 		default:
-			tech_fillportpoly(ni, pp, poly, trans, thistn, CLOSED);
+			tech_fillportpoly(ni, pp, poly, trans, thistn, CLOSED, lambdaofnode(ni));
 			break;
 	}
 }
@@ -2180,7 +2231,7 @@ INTBIG arcwidthoffset(ARCINST *ai)
 
 	/* if the technology has its own routine, use it */
 	ap = ai->proto;
-	if (ap->tech->nodesizeoffset != 0)
+	if (ap->tech->arcwidthoffset != 0)
 		return((*(ap->tech->arcwidthoffset))(ai));
 
 	return(tech_arcprotowidthoffset(ap, lambdaofarc(ai)));
@@ -2281,71 +2332,6 @@ ARCPROTO *getarconlayer(INTBIG layer, TECHNOLOGY *tech)
 
 /******************** LAYER DESCRIPTION ********************/
 
-/*
- * routine to accumulate a list of polygons at least "tot" long in the
- * polygon structure "list".  The routine returns true if there is no
- * more memory.
- */
-BOOLEAN ensurepolylist(POLYLIST *list, INTBIG tot, CLUSTER *cluster)
-{
-	REGISTER POLYGON **newpolylist;
-	REGISTER INTBIG j, *newlx, *newhx, *newly, *newhy;
-
-	/* make sure the list is the right size */
-	if (tot > list->polylisttotal)
-	{
-		newpolylist = (POLYGON **)emalloc(tot * (sizeof (POLYGON *)), dr_tool->cluster);
-		newlx = (INTBIG *)emalloc(tot * SIZEOFINTBIG, dr_tool->cluster);
-		newhx = (INTBIG *)emalloc(tot * SIZEOFINTBIG, dr_tool->cluster);
-		newly = (INTBIG *)emalloc(tot * SIZEOFINTBIG, dr_tool->cluster);
-		newhy = (INTBIG *)emalloc(tot * SIZEOFINTBIG, dr_tool->cluster);
-		if (newpolylist == 0 || newlx == 0 || newhx == 0 || newly == 0 || newhy == 0)
-		{
-			ttyputerr(_("DRC error allocating memory for polygons"));
-			return(TRUE);
-		}
-		for(j=0; j<tot; j++) newpolylist[j] = NOPOLYGON;
-		if (list->polylisttotal != 0)
-		{
-			for(j=0; j<list->polylisttotal; j++)
-				newpolylist[j] = list->polygons[j];
-			efree((char *)list->polygons);
-			efree((char *)list->lx);
-			efree((char *)list->hx);
-			efree((char *)list->ly);
-			efree((char *)list->hy);
-		}
-		list->polygons = newpolylist;
-		list->lx = newlx;
-		list->hx = newhx;
-		list->ly = newly;
-		list->hy = newhy;
-		list->polylisttotal = tot;
-	}
-
-	/* make sure there is a polygon for each entry in the list */
-	for(j=0; j<tot; j++) if (list->polygons[j] == NOPOLYGON)
-		list->polygons[j] = allocpolygon(4, cluster);
-	return(FALSE);
-}
-
-void freepolylist(POLYLIST *list)
-{
-	REGISTER INTBIG i;
-
-	for(i=0; i<list->polylisttotal; i++)
-		freepolygon(list->polygons[i]);
-	if (list->polylisttotal > 0)
-	{
-		efree((char *)list->polygons);
-		efree((char *)list->lx);
-		efree((char *)list->hx);
-		efree((char *)list->ly);
-		efree((char *)list->hy);
-	}
-	efree((char *)list);
-}
-
 /*
  * routine to return the full name of layer "layer" in technology "tech".
  * This routine accesses the "layer_names" variable on the technology objects.
@@ -2409,6 +2395,14 @@ BOOLEAN layerispoly(INTBIG fun)
 	return(FALSE);
 }
 
+/* Routine to return nonzero if the layer function "fun" is polysilicon  */
+BOOLEAN layerisgatepoly(INTBIG fun)
+{
+	fun &= LFINTRANS;
+	if (fun == LFINTRANS) return(TRUE);
+	return(FALSE);
+}
+
 /* Routine to return nonzero if the layer function "fun" is a contact/via  */
 BOOLEAN layeriscontact(INTBIG fun)
 {
@@ -2893,6 +2887,49 @@ void drcminnodesize(NODEPROTO *np, LIBRARY *lib, INTBIG *sizex, INTBIG *sizey, c
 	}
 }
 
+#ifdef SURROUNDRULES
+#define MAXSURROUNDRULES 20
+
+/*
+ * Routine to return surround rules for layer "layer" of technology "tech".
+ * Returns the number of surround rules, and for each one sets an entry in the
+ * arrays "layers" (the layer that must surround this), "dist" (the distance of the surround)
+ * and "rules" (the rule that describes this).
+ */
+INTBIG drcsurroundrules(TECHNOLOGY *tech, INTBIG layer, INTBIG **layers, INTBIG **dist, char ***rules)
+{
+	REGISTER INTBIG *pairs, index, count, i;
+	static INTBIG mylayer[MAXSURROUNDRULES];
+	static INTBIG mydist[MAXSURROUNDRULES];
+	static char *myrule[MAXSURROUNDRULES];
+
+	/* make sure cache of information is valid */
+	if (tech_drcsurroundlayerpairs == 0)
+	{
+		tech_initmaxdrcsurround();
+		if (tech_drcsurroundlayerpairs == 0) return(0);
+	}
+
+	index = tech->techindex;
+	pairs = tech_drcsurroundlayerpairs[index];
+
+	count = 0;
+	for(i=0; pairs[i] != 0; i++)
+	{
+		if ((pairs[i]&0xFFFF) != layer) break;
+		if (count >= MAXSURROUNDRULES) break;
+		mylayer[count] = (pairs[i] >> 16) & 0xFFFF;
+		mydist[count] = tech_drcsurrounddistances[index][i];
+		myrule[count] = tech_drcsurroundrule[index][i];
+		count++;
+	}
+	*layers = mylayer;
+	*dist = mydist;
+	*rules = myrule;
+	return(count);
+}
+#endif
+
 /******************** MATHEMATICS ********************/
 
 /*
