@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: dbtechi.c
@@ -537,7 +538,7 @@ BOOLEAN tech_pinusecount(NODEINST *ni, WINDOWPART *win)
 	/* if the next level up the hierarchy is visible, consider arcs connected there */
 	if (win != NOWINDOWPART && ni->firstportexpinst != NOPORTEXPINST)
 	{
-		db_gettraversalpath(&nilist, &depth);
+		db_gettraversalpath(ni->parent, &nilist, &depth);
 		if (depth == 1)
 		{
 			upni = nilist[0];
@@ -837,9 +838,10 @@ INTBIG tech_inittrans(INTBIG count, NODEINST *ni, POLYLOOP *pl)
 /*
  * routine to describe box "box" of transistor "ni" that may be part of a
  * serpentine path.  If the variable "trace" exists on the node, get that
- * x/y/x/y information as the centerline of the serpentine path.  The path
- * extends "lwid" to the left, "rwid" to the right, and "extendt/extendb" on the ends
- * (this information is found in the "laylist" structure).  The outline is
+ * x/y/x/y information as the centerline of the serpentine path.  The "laylist"
+ * structure describes the transistor: "lwidth" and "rwidth" are the extension
+ * of the active layer to the left and right; "extendt/extendb" are the extension
+ * of the poly layer on the top and bottom ends.  The outline is
  * placed in the polygon "poly".  Layer information for this polygon is in
  * "lay", port information is in "portstruct", and the value of lambda is in
  * "lambda".  In all manhattan cases, nonoverlapping polygons are
@@ -946,10 +948,17 @@ void tech_filltrans(POLYGON *poly, TECH_POLYGON **lay, TECH_SERPENT *laylist,
 				yr = othery + mult(sine(ang), rwid);
 				switch (angle)
 				{
+#if 0	/* make nonoverlapping: */
 					case 0:    thisxr = thisxl = maxi(xl,xr);   break;
 					case 1800: thisxr = thisxl = mini(xl,xr);   break;
 					case 900:  thisyr = thisyl = maxi(yl,yr);   break;
 					case 2700: thisyr = thisyl = mini(yl,yr);   break;
+#else	/* make overlapping: */
+					case 0:    thisxr = thisxl = mini(xl,xr);   break;
+					case 1800: thisxr = thisxl = maxi(xl,xr);   break;
+					case 900:  thisyr = thisyl = mini(yl,yr);   break;
+					case 2700: thisyr = thisyl = maxi(yl,yr);   break;
+#endif
 				}
 			} else	      /* NonManhattan */
 			{
@@ -1080,9 +1089,10 @@ void tech_filltransport(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans
 	}
 
 	/* nonserpentine transtors fill in the normal way */
+	lambda = lambdaofnode(ni);
 	if (var == NOVARIABLE)
 	{
-		tech_fillportpoly(ni, pp, poly, trans, nodedata, -1);
+		tech_fillportpoly(ni, pp, poly, trans, nodedata, -1, lambda);
 		return;
 	}
 
@@ -1094,7 +1104,6 @@ void tech_filltransport(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans
 	total /= 2;
 
 	/* see if nonstandard width is specified */
-	lambda = lambdaofnode(ni);
 	defwid = lambda * defwid / WHOLE;
 	diffinset = lambda * diffinset / WHOLE;   diffextend = lambda * diffextend / WHOLE;
 	polyinset = lambda * polyinset / WHOLE;   polyextend = lambda * polyextend / WHOLE;
@@ -1231,8 +1240,21 @@ VARIABLE *tech_filldisplayablenvar(NODEINST *ni, POLYGON *poly, WINDOWPART *win,
 	VARIABLE **varnoeval, POLYLOOP *pl)
 {
 	REGISTER NODEPROTO *facet;
+	REGISTER VARIABLE *var;
 	REGISTER INTBIG lx, hx, ly, hy;
 	INTBIG olx, ohx, oly, ohy;
+	REGISTER BOOLEAN oldonobjectstate;
+
+	/* track which object we are on */
+	oldonobjectstate = db_onanobject;
+	if (!db_onanobject)
+	{
+		db_onanobject = TRUE;
+		db_onobjectaddr = (INTBIG)ni;
+		db_onobjecttype = VNODEINST;
+	}
+	db_lastonobjectaddr = (INTBIG)ni;
+	db_lastonobjecttype = VNODEINST;
 
 	facet = ni->parent;
 	pl->numvar = ni->numvar;
@@ -1244,8 +1266,10 @@ VARIABLE *tech_filldisplayablenvar(NODEINST *ni, POLYGON *poly, WINDOWPART *win,
 	nodesizeoffset(ni, &olx, &oly, &ohx, &ohy);
 	lx += olx;   hx -= ohx;
 	ly += oly;   hy -= ohy;
-	return(tech_filldisplayableanyvar(poly, lx, hx, ly, hy, (INTBIG)ni, VNODEINST,
-		facet->tech, win, facet, varnoeval, pl));
+	var = tech_filldisplayableanyvar(poly, lx, hx, ly, hy, (INTBIG)ni, VNODEINST,
+		facet->tech, win, facet, varnoeval, pl);
+	db_onanobject = oldonobjectstate;
+	return(var);
 }
 
 /*
@@ -1284,10 +1308,10 @@ VARIABLE *tech_filldisplayablefacetvar(NODEPROTO *np, POLYGON *poly, WINDOWPART
  * If "sty" is not -1, the outline is described in trace data on the node.
  */
 void tech_fillportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans,
-	TECH_NODES *nodedata, INTBIG sty)
+	TECH_NODES *nodedata, INTBIG sty, INTBIG lambda)
 {
 	REGISTER INTBIG i, count;
-	REGISTER INTBIG lambda, x, y;
+	REGISTER INTBIG x, y;
 	INTBIG lx, ly, hx, hy;
 	REGISTER TECH_PORTS *portdata;
 	REGISTER VARIABLE *var;
@@ -1317,9 +1341,6 @@ void tech_fillportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans,
 		if (portdata->addr == pp) break;
 	}
 
-	/* get the lambda value */
-	lambda = lambdaofnode(ni);
-
 	/* get the high/low X/Y of the port area */
 	subrange(ni->lowx, ni->highx, portdata->lowxmul, portdata->lowxsum,
 		portdata->highxmul, portdata->highxsum, &lx, &hx, lambda);
@@ -1753,15 +1774,30 @@ VARIABLE *tech_filldisplayableavar(ARCINST *ai, POLYGON *poly, WINDOWPART *win,
 	VARIABLE **varnoeval, POLYLOOP *pl)
 {
 	REGISTER NODEPROTO *facet;
+	REGISTER VARIABLE *var;
+	REGISTER BOOLEAN oldonobjectstate;
+
+	/* track which object we are on */
+	oldonobjectstate = db_onanobject;
+	if (!db_onanobject)
+	{
+		db_onanobject = TRUE;
+		db_onobjectaddr = (INTBIG)ai;
+		db_onobjecttype = VARCINST;
+	}
+	db_lastonobjectaddr = (INTBIG)ai;
+	db_lastonobjecttype = VARCINST;
 
 	facet = ai->parent;
 	pl->numvar = ai->numvar;
 	pl->firstvar = ai->firstvar;
 	if (facet->tech == NOTECHNOLOGY)
 		facet->tech = whattech(facet);
-	return(tech_filldisplayableanyvar(poly, ai->geom->lowx, ai->geom->highx,
+	var = tech_filldisplayableanyvar(poly, ai->geom->lowx, ai->geom->highx,
 		ai->geom->lowy, ai->geom->highy, (INTBIG)ai, VARCINST, facet->tech,
-			win, facet, varnoeval, pl));
+			win, facet, varnoeval, pl);
+	db_onanobject = oldonobjectstate;
+	return(var);
 }
 
 /******************** SUPPORT ********************/
