@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: drcflat.c
@@ -348,7 +349,7 @@ BOOLEAN dr_readshorterrors(INTBIG where)
 				ly = cy-sy/2;   hy = cy+sy/2;
 				layname = layername(dr_maintech, atoi(layer));
 				ttyputmsg(_("Error on %s layer from X(%s...%s) Y(%s...%s)"),
-					layname, latoa(lx), latoa(hx), latoa(ly), latoa(hy));
+					layname, latoa(lx, 0), latoa(hx, 0), latoa(ly, 0), latoa(hy, 0));
 				(void)asktool(us_tool, "show-area", lx, hx, ly, hy, el_curwindowpart->curnodeproto);
 				flushscreen(); /* empty graphics buffer */
 				errors++;
@@ -482,7 +483,7 @@ void dr_flatprint(NODEPROTO *facet, XARRAY mytrans)
 			transmult(temptrans1, mytrans, temptrans2);
 			maketrans(ni, temptrans1);
 			transmult(temptrans1, temptrans2, subtrans);
-			downhierarchy(ni, 0);
+			downhierarchy(ni, ni->proto, 0);
 			dr_flatprint(ni->proto, subtrans);
 			uphierarchy();
 		}
