@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design Systems
  *
  * File: graphdraw.c
@@ -97,7 +98,7 @@ void gra_termgraph(void)
 	if (gra_polysegcount > 0) efree((char *)gra_polysegs);
 }
 
-static INTBIG gra_badcoord(WINDOWFRAME *wf, INTBIG x, INTBIG y, char *where)
+static INTBIG gra_badcoord(WINDOWFRAME *wf, INTBIG x, INTBIG y, char *where )
 {
 	INTBIG height;
 
@@ -112,6 +113,8 @@ static INTBIG gra_badcoord(WINDOWFRAME *wf, INTBIG x, INTBIG y, char *where)
 #if 0		/* for debugging bad coordinates */
 		printf(_("Bad coordinate from %s: (%ld,%ld), window is %ldx%ld\n"), where,
 			x, y, wf->swid, height);
+#else
+		Q_UNUSED( where );
 #endif
 		return(1);
 	}
@@ -431,7 +434,7 @@ static void gra_drawthickpoint(WINDOWFRAME *wf, INTBIG x, INTBIG y, INTBIG mask,
  */
 void gra_drawpolygon(WINDOWPART *win, INTBIG *x, INTBIG *y, INTBIG count, GRAPHICS *desc)
 {
-	REGISTER INTBIG i, j, k, l, ycur, yrev, wrap, lx, hx, ly, hy;
+	REGISTER INTBIG i, j, k, l, ycur=0, yrev, wrap, lx=0, hx=0, ly=0, hy=0;
 	REGISTER char *row;
 	REGISTER INTBIG col, mask, style, pat;
 	REGISTER POLYSEG *a, *active, *edge, *lastedge, *left, *edgelist;
@@ -1032,6 +1035,8 @@ void gra_drawtext(WINDOWPART *win, INTBIG atx, INTBIG aty, INTBIG rotation, char
 			left = atx;           right = atx + hei;
 			bottom = aty - wid;   top = aty;
 			break;
+		default:
+			return;
 	}
 
 	/* set redraw area */
@@ -1376,11 +1381,20 @@ void gra_drawdisc(WINDOWPART *win, INTBIG atx, INTBIG aty, INTBIG radius, GRAPHI
 	bottom = aty + radius + 1;
 	gra_setrect(wf, left, right, top, bottom);
 
+	/* draw a more general disc */
+	gra_curveminx = maxi(win->uselx, 0);
+	gra_curvemaxx = mini(win->usehx, wf->swid);
+	gra_curveminy = maxi(wf->revy - win->usehy, 0);
+	gra_curvemaxy = mini(wf->revy - win->usely, wf->shei);
+
 	if (radius == 1)
 	{
 		/* just fill the area for discs this small */
+		if (left < gra_curveminx) left = gra_curveminx;
+		if (right >= gra_curvemaxx) right = gra_curvemaxx - 1;
 		for(y=top; y<bottom; y++)
 		{
+			if (y < gra_curveminy || y >= gra_curvemaxy) continue;
 			thisrow = wf->rowstart[y];
 			for(x=left; x<right; x++)
 				thisrow[x] = (char)((thisrow[x] & gra_curvemask) | gra_curvecol);
@@ -1388,11 +1402,6 @@ void gra_drawdisc(WINDOWPART *win, INTBIG atx, INTBIG aty, INTBIG radius, GRAPHI
 		return;
 	}
 
-	/* draw a more general disc */
-	gra_curveminx = maxi(win->uselx, 0);
-	gra_curvemaxx = mini(win->usehx, wf->swid);
-	gra_curveminy = maxi(wf->revy - win->usehy, 0);
-	gra_curvemaxy = mini(wf->revy - win->usely, wf->shei);
 	x = 0;   y = radius;
 	d = 3 - 2 * radius;
 	while (x <= y)
