@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design Systems
  *
  * File: graphqt.h
@@ -32,37 +33,43 @@
 #define GRAPHQT_H
 
 #include "global.h"
-#include "usr.h"
 
-#define QT_CLEAN_NAMESPACE
 #include <qapplication.h>
 #include <qbitmap.h>
 #include <qcursor.h>
+#include <qframe.h>
 #include <qimage.h>
 #include <qmainwindow.h>
 #include <qpainter.h>
 #include <qpixmap.h>
 
+#ifdef MACOSX
+/* CHARPIXMAP - use 32-bit QPixmap instead of 1-bit QBitmap for glyph drawing */
+#  define CHARPIXMAP
+#endif
+
 #ifndef Q_ASSERT
-#define Q_ASSERT(x) ASSERT(x)
+#  define Q_ASSERT(x) ASSERT(x)
 #endif
 #ifndef Q_CHECK_PTR
-#define Q_CHECK_PTR(p) CHECK_PTR(p)
+#  define Q_CHECK_PTR(p) CHECK_PTR(p)
 #endif
 
 #define GTRACE          0       /* etrace flag of usual graphics trace */
 #define GTRACE_CONT     0       /* etrace flag of usual graphics trace (continue line) */
 
-#ifdef ONUNIX
-#define QDRAW      /* use Qt drawing */
-#endif
-
+struct Ipopupmenu;
 class QHBox;
 class QLabel;
 class QLineEdit;
-class QMultiLineEdit;
+#if QT_VERSION >= 300
+  class QTextEdit;
+#else
+  class QMultiLineEdit;
+#endif
 class QPaintDevice;
 class EFont;
+class EStatusItem;
 
 class MessagesMainWindow: public QMainWindow
 {
@@ -76,51 +83,19 @@ public slots:
 protected:
     bool eventFilter( QObject *watched, QEvent *e ); 
 public:
+#if QT_VERSION >= 300
+    QTextEdit *scroll;
+#else
     QMultiLineEdit *scroll;
+#endif
     QHBox *input;
     QLabel *prompt;
     QLineEdit *line;
     bool stopped;
 };
 
-class EPainter: public QPainter
-{
-public:
-  EPainter();
-  static void init( QStringList &facelist );
-  static void term();
-  bool begin( QPaintDevice *pd );
-  void screendrawline(INTBIG x1, INTBIG y1, INTBIG x2, INTBIG y2, GRAPHICS *desc, INTBIG texture);
-  void screeninvertline(INTBIG x1, INTBIG y1, INTBIG x2, INTBIG y2);
-  void screendrawbox(INTBIG lowx, INTBIG highx, INTBIG lowy, INTBIG highy, GRAPHICS *desc);
-  void screendrawpolygon(INTBIG *x, INTBIG *y, INTBIG count, GRAPHICS *desc);
-  void screeninvertbox(INTBIG lowx, INTBIG highx, INTBIG lowy, INTBIG highy);
-  void screenmovebox(INTBIG sx, INTBIG sy, INTBIG wid, INTBIG hei, INTBIG dx, INTBIG dy);
-  void drawPixmap( QPoint orig, QPixmap pixmap );
-  void screendrawtext(INTBIG atx, INTBIG aty, char *s, GRAPHICS *desc);
-  void screendrawcircle(INTBIG atx, INTBIG aty, INTBIG radius, GRAPHICS *desc);
-  void screendrawthickcircle(INTBIG atx, INTBIG aty, INTBIG radius, GRAPHICS *desc);
-  void screendrawdisc(INTBIG atx, INTBIG aty, INTBIG radius, GRAPHICS *desc);
-  void screendrawcirclearc(INTBIG centerx, INTBIG centery, INTBIG p1_x, INTBIG p1_y,
-			   INTBIG p2_x, INTBIG p2_y, GRAPHICS *desc);
-  void screendrawthickcirclearc(INTBIG centerx, INTBIG centery, INTBIG p1_x, INTBIG p1_y,
-				INTBIG p2_x, INTBIG p2_y, GRAPHICS *desc);
-  void screendrawgrid(int pointWidth, double x0, double y0, double dx, double dy, int nx, int ny);
-
-  QRect copyRect() { return cr;  };
-  bool dirty() { return cr.isValid(); };
-  void clearDirty() { cr = QRect(); }
-  void updateAll();
-  UINTBIG startTime() { return starttime; }
-private:
-  void setPlanes( ulong planes = ~0);
-  void setBrushPlanes( ulong planes = ~0);
-  void setBrush( GRAPHICS *desc );
-  void updateCopyRect( int x, int y, int w, int h );
-  QBitmap loadStipple( UINTSML raster[] );
-  QRect cr;
-  UINTBIG starttime;
-};
+void gra_initfaces( QStringList &facelist );
+void gra_termfaces();
 
 class GraphicsDraw: public QWidget
 {
@@ -144,11 +119,8 @@ private:
     void recalcSize();
 public:
     WINDOWFRAME *wf;
-    /* offscreen data is in "pixmap" and drawn by "painter if EApplication::qdraw */
-    QPixmap pixmap;
-    EPainter painter;
     int colorvalue[256];
-    /* offscreen data is in "image" and drawn by "graphdraw.c" if not EApplication::qdraw  */
+    /* offscreen data is in "image" and drawn by "graphdraw.c" */
     QImage image;
 };
 
@@ -160,16 +132,16 @@ public:
     ~GraphicsMainWindow();
     void closeEvent( QCloseEvent *e );
 
-    void makepdmenu( POPUPMENU *pm, INTSML value);
-    INTSML pulldownindex(POPUPMENU *pm);
+    void makepdmenu( Ipopupmenu *pm, INTSML value);
+    INTSML pulldownindex( Ipopupmenu *pm );
     void pulldownmenuload();
-    void nativemenurename(POPUPMENU *pm, INTBIG pindex);
+    void nativemenurename( Ipopupmenu *pm, INTBIG pindex);
 private:
     QString describeboundkey(INTSML key, INTBIG special);
 public slots:
     void menuAction( int id );
 public:
-    QLabel *statusItems[100];
+    EStatusItem *statusItems[100];
     WINDOWFRAME *wf;
     GraphicsDraw *draw;
     QPopupMenu **pulldownmenus;		/* list of Windows pulldown menus */
@@ -183,15 +155,17 @@ class EApplication: public QApplication
 public:
     EApplication( int &argc, char **argv);
     ~EApplication();
-    Display* dpy() const;
-    int screen() const;
     MessagesMainWindow *messMainWin;
     static char *localize (QString qstr);
     static char *localizeFilePath (QString filePath, bool addSeparator );
     int translatekey(QKeyEvent *e, INTBIG *special);
-
+    
     QBitmap textbits;
+#ifdef CHARPIXMAP
+    QPixmap charbits;
+#else
     QBitmap charbits;
+#endif
     EFont *curfont;
     QStringList facelist;
     QString defface;
@@ -209,29 +183,32 @@ public:
     QCursor lrcursor;
     QCursor udcursor;
     
-    static bool qdraw;
-
     /* trackcursor */
     INTSML waitforpush;
     BOOLEAN (*whileup)(INTBIG x, INTBIG y);
     void (*whendown)(void);
     BOOLEAN (*eachdown)(INTBIG x, INTBIG y);
     BOOLEAN (*eachchar)(INTBIG x, INTBIG y, INTSML c);
-
+    
     /* modalloop */
     BOOLEAN (*charhandler)(INTSML chr, INTBIG special);
     BOOLEAN (*buttonhandler)(INTBIG x, INTBIG y, INTBIG but);
 
 public slots:
-    void toolTimeSlice(); 
-private:
-    Display      *gra_dpy;				/* the main X display */
+	void toolTimeSlice(); 
 };
 
-inline Display* EApplication::dpy() const
+class EStatusItem: public QFrame
 {
-    return qt_xdisplay();
-}
+    Q_OBJECT
+public:
+    EStatusItem( QWidget *parent, char *name = 0 );
+    void setText( const QString &text );
+protected:
+    void	 drawContents( QPainter *p );
+private:
+    QString stext;
+};
 
 extern EApplication *gra;
 
