@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design Systems
  *
  * File: graphqtdlg.h
@@ -32,14 +33,19 @@
 #define GRAPHQTDLG_H
 
 #include "global.h"
-#include "edialogs.h"
 
-#define QT_CLEAN_NAMESPACE
 #include <qcheckbox.h>
 #include <qdialog.h>
 #include <qlabel.h>
 #include <qlistbox.h>
 
+#if defined(USEQUI) && QT_VERSION >= 300
+#  define QTSTRETCH
+#endif
+#ifdef QTSTRETCH
+#  include <qwidgetfactory.h>
+#endif
+
 #ifndef Q_ASSERT
 #define Q_ASSERT(x) ASSERT(x)
 #endif
@@ -49,71 +55,41 @@
 
 #define MAXLOCKS              3   /* maximum locked pairs of scroll lists */
 
+struct Idialogitem;
 class QSignalMapper;
 class EScrollField;
+class EDialogPrivate;
 
-class EDialog: public QDialog
+class EDialogPrivate: public QDialog
 {
+    friend class EDialog;
     Q_OBJECT
 public:
-    EDialog(QWidget *parent, DIALOG *dialog);
-    ~EDialog();
+#if QT_VERSION >= 300
+    EDialogPrivate( QWidget *parent, const char *name = 0 );
+    void init( EDialog *dia, bool modal );
+#else
+    EDialogPrivate( QWidget *parent, const char *name = 0, bool modal = FALSE );
+    void init( EDialog *dia );
+#endif
     void showMe();
-    static char *localize (QString qstr);
-
-    /* API routines */
-    INTBIG nextHit(void);
-    void setText(INTBIG item, char *msg);
-    char *getText(INTBIG item);
-    void setControl(INTBIG item, INTBIG value);
-    INTBIG getControl(INTBIG item);
-    BOOLEAN validEntry(INTBIG item);
-    void dimItem(INTBIG item);
-    void unDimItem(INTBIG item);
-    void noEditControl(INTBIG item);
-    void editControl(INTBIG item);
-    void opaqueEdit(INTBIG item);
-    void defaultButton(INTBIG item);
-    void setPopup(INTBIG item, INTBIG count, char **names);
-    void setPopupEntry(INTBIG item, INTBIG entry);
-    INTBIG getPopupEntry(INTBIG item);
-    void initTextDialog(INTBIG item, BOOLEAN (*toplist)(char **), char *(*nextinlist)(void),
-			void (*donelist)(void), INTBIG sortpos, INTBIG flags);
-    void loadTextDialog(INTBIG item, BOOLEAN (*toplist)(char **), char *(*nextinlist)(void),
-			void (*donelist)(void), INTBIG sortpos);
-    void stuffLine(INTBIG item, char *line);
-    void selectLine(INTBIG item, INTBIG line);
-    void selectLines(INTBIG item, INTBIG count, INTBIG *lines);
-    INTBIG getCurLine(INTBIG item);
-    INTBIG *getCurLines(INTBIG item);
-    INTBIG getNumLines(INTBIG item);
-    char *getScrollLine(INTBIG item, INTBIG line);
-    void setScrollLine(INTBIG item, INTBIG line, char *msg);
-    void synchVScrolls(INTBIG item1, INTBIG item2, INTBIG item3);
-    void unSynchVScrolls();
-    void itemRect(INTBIG item, RECTAREA *rect);
-    void percent(INTBIG item, INTBIG percent);
-    void redispRoutine(INTBIG item, void (*routine)(RECTAREA*)); 
-    void invertRect(INTBIG item, RECTAREA *r);
-    void frameRect(INTBIG item, RECTAREA *r);
-    void drawLine(INTBIG item, INTBIG fx, INTBIG fy, INTBIG tx, INTBIG ty, INTBIG mode);
-    void fillPoly(INTBIG item, INTBIG *xv, INTBIG *yv, INTBIG count, INTBIG r, INTBIG g, INTBIG b);
-    void allowUserDoubleClick(void);
-    void drawRect(INTBIG item, RECTAREA *rect, INTBIG r, INTBIG g, INTBIG b);
-    void putText(INTBIG item, char *msg, INTBIG x, INTBIG y);
-    void setTextSize(INTBIG size);
-    void getTextInfo(char *msg, INTBIG *wid, INTBIG *hei);
-    void trackCursor(void (*eachdown)(INTBIG x, INTBIG y));
-    void getMouse(INTBIG *x, INTBIG *y);
-
+    static char *localize( QString qstr );
+#ifdef QTSTRETCH
+    static EDialogPrivate *loadUi( QWidget *parent, char *uiFile, bool ext = FALSE );
+    bool checkChildren();
+#endif
 private:
-    DIALOG *itemdesc;
     QWidget *items[100];
     int numlocks;
     QScrollBar *lock1[MAXLOCKS], *lock2[MAXLOCKS], *lock3[MAXLOCKS];
     QSignalMapper *mapper;
     QPoint iniPos;
     QFont userFont;
+public:
+    EDialog *dia;
+	EDialogPrivate *nexttdialog;
+    INTBIG itemHit;
+
 public slots:
     void action(int id);
 protected:
@@ -150,7 +126,8 @@ class EIconField: public QLabel
 {
     Q_OBJECT
 public:
-    EIconField( QWidget *parent, DIALOGITEM *item );
+    EIconField( QWidget *parent, Idialogitem *item );
+    void setBits( uchar *bits ); 
 signals:
     void clicked();
 protected:
@@ -161,7 +138,7 @@ class EUserDrawnField: public QWidget
 {
     Q_OBJECT
 public:
-    EUserDrawnField( QWidget *parent, DIALOGITEM *item );
+    EUserDrawnField( QWidget *parent, Idialogitem *item );
 signals:
     void clicked();
     void doubleClicked();
@@ -172,11 +149,21 @@ protected:
     void mouseReleaseEvent( QMouseEvent *e );
     void paintEvent( QPaintEvent *e );
 public:
-    DIALOGITEM *it;
-    void (*redispRoutine)(RECTAREA*);
+    Idialogitem *it;
+	EDialog *dia;
+    void (*redispRoutine)(RECTAREA *rect, void *dia);
     bool buttonPressed;
     void (*eachdown)(INTBIG x, INTBIG y);
     bool doubleClickAllowed;
 };
 
+#ifdef QTSTRETCH
+class EWidgetFactory: public QWidgetFactory
+{
+public:
+    EWidgetFactory();
+    QWidget* createWidget( const QString& className, QWidget *parent, const char *name) const;    
+};
+#endif
+
 #endif
