@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: conlay.h
@@ -44,6 +45,7 @@ extern "C"
 #define CHANGETYPEREMOVETEMP    8		/* remove temporarily state of arc */
 
 extern CONSTRAINT *cla_constraint;		/* the constraint object for this solver */
+extern INTBIG      cla_changeclock;		/* timestamp for changes */
 
 void cla_layconinit(CONSTRAINT*);
 void cla_layconterm(void);
