@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: database.h
@@ -268,6 +269,14 @@ extern UINTBIG db_changetimestamp;				/* timestamp for changes to database */
 extern UINTBIG db_traversaltimestamp;			/* timestamp for hierarchy traversal */
 extern INTBIG  db_broadcasting;					/* nonzero if broadcasting */
 
+/*************************** LANGUAGE CONTROL ****************************/
+
+extern BOOLEAN db_onanobject;			/* TRUE if code evaluation is "on an object" (getval, etc.) */
+extern INTBIG  db_onobjectaddr;			/* the address of the first object that we are on */
+extern INTBIG  db_onobjecttype;			/* the type of the first object that we are on */
+extern INTBIG  db_lastonobjectaddr;		/* the address of the last object that we were on */
+extern INTBIG  db_lastonobjecttype;		/* the type of the last object that we were on */
+
 /*************************** PROTOTYPES ****************************/
 
 /* database prototypes */
@@ -276,6 +285,7 @@ void         db_addportexpinst(NODEINST*, PORTEXPINST*);
 BOOLEAN      db_addtortnode(UINTBIG, RTNODE*, NODEPROTO*);
 void         db_boundfacet(NODEPROTO*, INTBIG*, INTBIG*, INTBIG*, INTBIG*);
 void         db_buildcellhashtable(LIBRARY*);
+void         db_buildportprotohashtable(NODEPROTO*);
 CHANGE      *db_change(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
 void         db_changeport(PORTPROTO*, NODEINST*, PORTPROTO*);
 void         db_checkallmemoryfree(void);
@@ -304,9 +314,9 @@ void         db_freevar(INTBIG, INTBIG);
 void         db_freevariablememory(void);
 void         db_freevars(VARIABLE**, INTSML*);
 CHANGEBATCH *db_getcurrentbatch(void);
-float        db_getcurrentscale(INTBIG, INTBIG);
+float        db_getcurrentscale(INTBIG, INTBIG, INTBIG);
 void         db_getinternalunitscale(INTBIG*, INTBIG*, INTBIG, INTBIG);
-void         db_gettraversalpath(NODEINST ***nilist, INTBIG *depth);
+void         db_gettraversalpath(NODEPROTO *here, NODEINST ***nilist, INTBIG *depth);
 BOOLEAN      db_getvarptr(INTBIG, INTBIG, VARIABLE***, INTSML**);
 void         db_initclusters(void);
 void         db_initdatabase(void);
@@ -320,6 +330,7 @@ void         db_insertnodeproto(NODEPROTO*);
 void         db_killarcinst(ARCINST*);
 void         db_killnodeinst(NODEINST*);
 void         db_mrgdatainit(void);
+INTBIG       db_namehash(char *name);
 ARCINST     *db_newarcinst(ARCPROTO*, INTBIG, INTBIG, NODEINST*, PORTPROTO*, INTBIG, INTBIG,
 	NODEINST*, PORTPROTO*, INTBIG, INTBIG, NODEPROTO*);
 ARCPROTO    *db_newarcproto(TECHNOLOGY*, char*, INTBIG, INTBIG);
