@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: eio.h
@@ -112,6 +113,9 @@ extern "C"
 #define CIFRESHIGH              01		/* 1: bit set for CIF resolution errors to highlight */
 #define SKILLFLATHIER           02		/* 1: bit set to flatten hierarchy in SKILL */
 #define CDLNOBRACKETS           04		/* 1: bit set to convert brackets in CDL */
+#define BINOUTPRECREADATE      010		/* 1: bit set for binary output to preserve creation dates */
+#define PSAUTOROTATE           020		/* 1: bit set to automatically rotate PS to fit best */
+#define SUEUSE4PORTTRANS       040		/* 1: bit set to make 4-port transistors in Sue input */
 
 #define DEFAULTPSWIDTH         638		/* default PostScript page width (8.5" at 75dpi) */
 #define DEFAULTPSHEIGHT        825		/* default PostScript page height (11.0" at 75dpi) */
@@ -145,6 +149,7 @@ extern INTBIG       io_filetypetlib;	/* Text library disk file descriptor */
 extern INTBIG       io_filetypevhdl;	/* VHDL disk file descriptor */
 extern INTBIG       io_libinputrecursivedepth;	/* for recursing when reading dependent libraries */
 extern INTBIG       io_libinputreadmany;/* nonzero if reading dependent libraries */
+extern void        *io_inputprogressdialog;
 
 /* prototypes for tool interface */
 void io_init(INTBIG*, char*[], TOOL*);
@@ -157,9 +162,10 @@ void io_slice(void);
 INTBIG    *io_getstatebits(void);
 void       io_setstatebits(INTBIG *bits);
 PORTPROTO *io_convertoldportname(char *portname, NODEPROTO *np);
-void       io_fixnewlib(LIBRARY *lib, INTBIG statusitem);
+void       io_fixnewlib(LIBRARY *lib, void *dia);
 NODEPROTO *io_convertoldprimitives(TECHNOLOGY *tech, char *name);
 INTBIG     io_setuptechorder(TECHNOLOGY *tech);
+void       io_queuereadlibraryannouncement(LIBRARY *lib);
 INTBIG     io_nextplotlayer(INTBIG i);
 INTBIG     io_getoutputbloat(char *layer);
 BOOLEAN    io_doreadbinlibrary(LIBRARY *lib, BOOLEAN newprogress);
@@ -187,7 +193,7 @@ BOOLEAN    io_writehpgllibrary(LIBRARY *lib);
 BOOLEAN    io_writellibrary(LIBRARY *lib);
 BOOLEAN    io_writeleflibrary(LIBRARY *lib);
 BOOLEAN    io_writepadslibrary(LIBRARY *lib);
-BOOLEAN    io_writepostscriptlibrary(LIBRARY *lib, INTBIG printit);
+BOOLEAN    io_writepostscriptlibrary(LIBRARY *lib, BOOLEAN printit);
 BOOLEAN    io_writequickdrawlibrary(LIBRARY *lib);
 BOOLEAN    io_writeskilllibrary(LIBRARY *lib);
 BOOLEAN    io_writetextlibrary(LIBRARY *lib);
