@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: network.h
@@ -38,14 +39,18 @@ extern "C"
 #define NETCONCOMMONNAME      2		/* set to connect commonly named nets */
 #define NETDEFBUSBASE1      010		/* set to have default bus base at 1 (instead of 0) */
 #define NETDEFBUSBASEDESC   020		/* set to have default busses numbered descending */
+#define NETIGNORERESISTORS  040		/* set to ignore resistors */
 #define NETDEFAULTABBREVLEN   8		/* default length of cell name abbreviations on nodes */
 
+#define HASHTYPE        INTHUGE		/* the size of hash values */
+
 extern INTBIG      net_ncc_optionskey;			/* key for "NET_ncc_options" */
 extern INTBIG      net_ncc_comptolerancekey;	/* key for "NET_ncc_component_tolerance" */
 extern INTBIG      net_ncc_comptoleranceamtkey;	/* key for "NET_ncc_component_tolerance_amt" */
 extern INTBIG      net_ncc_matchkey;			/* key for "NET_ncc_match" */
 extern INTBIG      net_node_abbrev_key;			/* key for "NET_node_abbreviations" */
 extern INTBIG      net_node_abbrevlen_key;		/* key for "NET_node_abbreviation_length" */
+extern INTBIG      net_ncc_processors_key;		/* key for "NET_ncc_num_processors" */
 extern INTBIG      net_options;					/* cached value for "NET_options" */
 
 /* the meaning of tool:network.NET_ncc_options */
@@ -62,12 +67,14 @@ extern INTBIG      net_options;					/* cached value for "NET_options" */
 #define NCCMERGESERIES            020000		/* set to merge series transistors during NCC */
 #define NCCMERGESERIESOVER        040000		/* set to override series transistors merge for a facet */
 #define NCCHIERARCHICALOVER      0100000		/* set to override hierarchy expansion during NCC */
+#define NCCEXPERIMENTAL          0200000		/* set to use experimental NCC */
+#define NCCGRAPHICPROGRESS       0400000		/* set to show NCC progress graphically */
 
 /*********************** PCOMP MODULES ***********************/
 
 #define NOPCOMP ((PCOMP *)-1)
 
-/* meaning of PNET->state */
+/* meaning of PCOMP->state */
 #define NEGATEDPORT         1			/* if port is negated */
 #define EXPORTEDPORT        2			/* if port is exported */
 
@@ -79,6 +86,7 @@ extern INTBIG      net_options;					/* cached value for "NET_options" */
 #define COMPPARALLELSEEN  020			/* if component has been seen in parallel merge */
 #define COMPTEMPFLAG      040			/* flag used in parallel merging */
 #define COMPLOCALFLAG    0100			/* flag used in NCC */
+#define COMPMATCHED      0200			/* if component has been matched */
 
 typedef struct Ipcomp
 {
@@ -91,8 +99,9 @@ typedef struct Ipcomp
 	INTSML           flags;				/* state bits */
 	INTSML           function;			/* component function */
 	INTSML           wirecount;			/* number of unconnected ports (wires) */
+	INTSML           truewirecount;		/* number of unconnected ports, excluding ignored Pwr&Gnd */
 	INTBIG           timestamp;			/* time stamp for entry into symmetry group */
-	INTHUGE          hashvalue;			/* hash value for NCC */
+	HASHTYPE         hashvalue;			/* hash value for NCC */
 	char            *hashreason;		/* the explanation of the hash value */
 	float            length;			/* length/size of component */
 	float            width;				/* width of component (if FET) */
@@ -115,6 +124,7 @@ extern PCOMP *net_pcompfree;
 #define GLOBALNET      010			/* if net is a global signal */
 #define NETUNIQUEHASH  020			/* if net has unique hash */
 #define NETLOCALFLAG   040			/* flag used in NCC */
+#define NETMATCHED    0100			/* if network has been matched */
 
 typedef struct Ipnet
 {
@@ -122,7 +132,7 @@ typedef struct Ipnet
 	INTSML         realportcount;	/* number of top-level exports on this net */
 	void          *realportlist;	/* top-level export(s) on this net */
 	NETWORK       *network;			/* real network that this references */
-	INTHUGE        hashvalue;		/* hash value for NCC */
+	HASHTYPE       hashvalue;		/* hash value for NCC */
 	char          *hashreason;		/* the explanation of the hash value */
 	INTBIG         timestamp;		/* time stamp for entry into symmetry group */
 	INTBIG         nodecount;		/* number of pnodes on this network */
@@ -185,12 +195,12 @@ void       net_freeallpnet(PNET*);
 void       net_freepcomp(PCOMP*);
 void       net_freepnet(PNET*);
 PCOMP     *net_makepseudo(NODEPROTO*, INTBIG*, INTBIG*, INTBIG*, INTBIG*, PNET**,
-			BOOLEAN, BOOLEAN, BOOLEAN, BOOLEAN);
+			BOOLEAN, BOOLEAN, BOOLEAN, BOOLEAN, BOOLEAN);
 AREAPERIM *net_gathergeometry(NETWORK *net, TRANSISTORINFO **p_gate, TRANSISTORINFO **n_gate,
-			TRANSISTORINFO **p_active, TRANSISTORINFO **n_active);
+			TRANSISTORINFO **p_active, TRANSISTORINFO **n_active, BOOLEAN recurse);
 BOOLEAN    net_equate(BOOLEAN);
-BOOLEAN    net_compare(BOOLEAN preanalyze, NODEPROTO *facet1, NODEPROTO *facet2);
-BOOLEAN    net_analyzefacet(void);
+INTBIG	   net_getequivalentnet(NETWORK *net, NETWORK ***equiv, INTBIG *numequiv, BOOLEAN allowchild);
+BOOLEAN    net_compare(BOOLEAN preanalyze, BOOLEAN interactive, NODEPROTO *facet1, NODEPROTO *facet2);
 INTBIG     net_buswidth(char*);
 INTBIG     net_evalbusname(INTBIG, char*, char***, ARCINST*, NODEPROTO*, INTBIG);
 void       net_freediffmemory(void);
@@ -202,23 +212,37 @@ void       net_dumpnetwork(PCOMP *pclist, PNET *pnlist);
 void       net_removeextraneous(PCOMP **pcomplist, PNET **pnetlist, INTBIG *comp);
 char      *net_describepnet(PNET *pn);
 char      *net_describepcomp(PCOMP *pc);
-INTHUGE    net_getcomphash(PCOMP *pc, INTBIG verbose);
-INTHUGE    net_getnethash(PNET *pn, INTBIG verbose);
+HASHTYPE   net_getcomphash(PCOMP *pc, INTBIG verbose);
+HASHTYPE   net_getnethash(PNET *pn, INTBIG verbose);
+void       net_redoprim(void);
 void       net_initdiff(void);
-void       net_showcomphash(WINDOWPART *win, PCOMP *pc, INTHUGE hashvalue, INTBIG hashindex, INTBIG verbose);
-void       net_shownethash(WINDOWPART *win, PNET *pn, INTHUGE hashvalue, INTBIG hashindex, INTBIG verbose);
+void       net_showcomphash(WINDOWPART *win, PCOMP *pc, HASHTYPE hashvalue, INTBIG hashindex, INTBIG verbose);
+void       net_shownethash(WINDOWPART *win, PNET *pn, HASHTYPE hashvalue, INTBIG hashindex, INTBIG verbose);
 void       net_freenetwork(NETWORK*, NODEPROTO*);
 void       net_conv_to_internal(NODEPROTO *np);
 void       net_removeassociations(void);
 void       net_fillinnetpointers(PCOMP *pcomplist, PNET *pnetlist);
+BOOLEAN    net_nccalreadydone(NODEPROTO *facet1, NODEPROTO *facet2);
 void       net_nccremovematches(NODEPROTO *np);
 INTBIG     net_ncchasmatch(NODEPROTO *np);
+void	   net_nccmatchinfo(NODEPROTO *np, NODEPROTO **facetmatch, UINTBIG *facetdate);
 void       net_parsenccresult(NODEPROTO *np, VARIABLE *var, NODEPROTO **facetmatch,
 			UINTBIG *facetdate);
 VARIABLE  *net_nccfindmatch(NODEPROTO *np, NODEPROTO *onp, UINTBIG *matchdate);
 void       net_setnodewidth(NODEINST *ni);
 INTBIG     net_gathernodenames(NODEPROTO *np, char ***namelist);
 BOOLEAN    net_getfacets(NODEPROTO**, NODEPROTO**);
+void       net_listnccoverrides(BOOLEAN usetemp1);
+BOOLEAN    net_componentequalvalue(float v1, float v2);
+#ifdef FORCESUNTOOLS
+BOOLEAN    net_analyzefacet(void);
+void       net_freeexpdiffmemory(void);
+INTBIG     net_doexpgemini(PCOMP *pcomp1, PNET *pnet1, PCOMP *pcomp2, PNET *pnet2,
+			BOOLEAN checksize, BOOLEAN checkexportnames, BOOLEAN ignorepwrgnd);
+BOOLEAN    net_equateexp(BOOLEAN noise);
+INTBIG     net_expanalyzesymmetrygroups(BOOLEAN reporterrors, BOOLEAN checksize,
+				BOOLEAN checkexportname, BOOLEAN ignorepwrgnd, INTBIG *errorcount);
+#endif
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
 }
