@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: rout.h
@@ -34,20 +35,21 @@ extern "C"
 #endif
 
 /* the meaning of "ROUT_state" */
-#define STITCHMODE             07			/* type of stitching being done */
-#define NOSTITCH                0			/* no stitching */
-#define AUTOSTITCH              1			/* automatic stitching by placement */
-#define MIMICSTITCH             2			/* mimic user stitching */
-#define SELECT                010			/* bit set if each wire should be OKed */
-#define SELSKIP               020			/* bit set if OK's should be skipped */
-#define SELDONE               040			/* bit set if any selection was done */
+#define STITCHMODE              07			/* type of stitching being done */
+#define NOSTITCH                 0			/* no stitching */
+#define AUTOSTITCH               1			/* automatic stitching by placement */
+#define MIMICSTITCH              2			/* mimic user stitching */
+#define SELECT                 010			/* bit set if each wire should be OKed */
+#define SELSKIP                020			/* bit set if OK's should be skipped */
+#define SELDONE                040			/* bit set if any selection was done */
 
 /* the meaning of "ROUT_options" */
-#define MIMICINTERACTIVE      010			/* bit set if mimic runs interactively */
-#define MIMICIGNORENODESIZE   020			/* bit set if mimic ignores node size */
-#define MIMICIGNORENODETYPE   040			/* bit set if mimic ignores node type */
-#define MIMICUNROUTES        0100			/* bit set if mimic can unroute */
-#define MIMICIGNOREPORTS     0200			/* bit set if mimic ignores ports */
+#define MIMICINTERACTIVE       010			/* bit set if mimic runs interactively */
+#define MIMICIGNORENODESIZE    020			/* bit set if mimic ignores node size */
+#define MIMICIGNORENODETYPE    040			/* bit set if mimic ignores node type */
+#define MIMICUNROUTES         0100			/* bit set if mimic can unroute */
+#define MIMICIGNOREPORTS      0200			/* bit set if mimic ignores ports */
+#define MIMICONSIDERARCCOUNT 01000			/* bit set if mimic considers arc count */
 
 extern TOOL     *ro_tool;				/* the Router tool object */
 extern INTBIG    ro_statekey;			/* variable key for "ROUT_state" */
@@ -64,14 +66,26 @@ typedef struct Icheck
 	NODEINST      *entity;				/* node instance being checked */
 	struct Icheck *nextcheck;			/* next in list */
 } RCHECK;
-extern RCHECK *ro_firstrcheck;
+extern RCHECK   *ro_firstrcheck;
+extern POLYLIST *ro_autostitchplist;
 
 /****************************** MIMIC ROUTING ******************************/
 
-extern ARCINST   *ro_createdarc;		/* last created arc for mimicing */
-extern NODEINST  *ro_deletednodes[2];	/* nodes at end of last deleted arc */
-extern PORTPROTO *ro_deletedports[2];	/* ports on nodes at end of last deleted arc */
-extern ARCPROTO  *ro_deletedarc;		/* type of last deleted arc */
+typedef struct
+{
+	INTBIG     numcreatedarcs;			/* number of arcs just created */
+	ARCINST   *createdarcs[3];			/* up to 3 newly created arcs */
+	INTBIG     numcreatednodes;			/* number of nodes just created */
+	NODEINST  *creatednodes[3];			/* up to 3 newly created nodes */
+	INTBIG     numdeletedarcs;			/* number of arcs just created */
+	ARCINST   *deletedarcs[2];			/* up to 2 deleted arcs */
+	INTBIG     numdeletednodes;			/* number of arcs just created */
+	NODEINST  *deletednodes[3];			/* up to 3 deleted nodes */
+} ROUTACTIVITY;
+
+extern ROUTACTIVITY ro_lastactivity;	/* last routing activity */
+extern NODEINST    *ro_deletednodes[2];	/* nodes at end of last deleted arc */
+extern PORTPROTO   *ro_deletedports[2];	/* ports on nodes at end of last deleted arc */
 
 /****************************** MAZE ROUTING ******************************/
 
@@ -89,6 +103,7 @@ void ro_done(void);
 void ro_set(INTBIG, char*[]);
 void ro_slice(void);
 void ro_startbatch(TOOL*, BOOLEAN);
+void ro_endbatch(void);
 void ro_modifynodeinst(NODEINST*,INTBIG,INTBIG,INTBIG,INTBIG,INTBIG,INTBIG);
 void ro_modifyarcinst(ARCINST*,INTBIG,INTBIG,INTBIG,INTBIG,INTBIG,INTBIG);
 void ro_modifyportproto(PORTPROTO*, NODEINST*, PORTPROTO*);
@@ -109,9 +124,6 @@ void    ro_mazeroutecell(void);
 void    ro_mazerouteselected(void);
 void    ro_mimicstitch(BOOLEAN forced);
 BOOLEAN ro_river(NODEPROTO*);
-#if ROUTTOOLSUN
-  void ro_par(void);
-#endif
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
 }
