@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: sim.h
@@ -152,10 +153,6 @@ BOOLEAN sim_window_buscommand(void);
 INTBIG  sim_window_makebus(INTBIG count, INTBIG *traces, char *busname);
 void    sim_window_loaddigtrace(INTBIG, INTBIG, double*, INTSML*);
 void    sim_window_loadanatrace(INTBIG, INTBIG, double*, float*);
-void    sim_window_setanarange(INTBIG, float, float);
-void    sim_window_getanarange(INTBIG, float*, float*);
-void    sim_window_setanaextents(float, float);
-void    sim_window_getanaextents(float*, float*);
 void    sim_window_settraceframe(INTBIG, INTBIG);
 void    sim_window_killtrace(INTBIG);
 void    sim_window_killalltraces(BOOLEAN);
@@ -187,6 +184,11 @@ void    sim_window_updatelayoutwindow(void);
 void    sim_windowconvertengineeringnotation(double, char*);
 void    sim_window_setstate(INTBIG);
 INTBIG  sim_window_getwidevalue(INTBIG **bits);
+void    sim_window_auto_anarange(void);
+void    sim_window_zoom_frame(INTBIG frameno);
+void    sim_window_zoomout_frame(INTBIG frameno);
+void    sim_window_shiftup_frame(INTBIG frameno);
+void    sim_window_shiftdown_frame(INTBIG frameno);
 
 /********************* for ESIM *********************/
 
@@ -234,10 +236,12 @@ extern INTBIG sim_verilog_statekey;		/* key for "SIM_verilog_state" */
 #define SPICERAWFILE     0100			/* set to read rawfile */
 #define SPICEGLOBALPG    0200			/* set to use global power and ground */
 #define SPICEFACETPARAM 01000			/* set to use facet parameters */
+#define SPICEUSELAMBDAS 02000			/* set to use lambdas instead of microns for transistor sizes */
 
 extern INTBIG sim_spice_levelkey;		/* key for "SIM_spice_level" */
 extern INTBIG sim_spice_statekey;		/* key for "SIM_spice_state" */
 extern INTBIG sim_spice_state;			/* value of "SIM_spice_state" */
+extern INTBIG sim_spice_nameuniqueid;	/* key for "SIM_spice_nameuniqueid" */
 extern INTBIG sim_spice_listingfilekey;	/* key for "SIM_listingfile" */
 extern INTBIG sim_spice_runargskey;		/* key for "SIM_spice_runarguments" */
 
