@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: tecschem.h
@@ -122,6 +123,7 @@ INTBIG  sch_allnodepolys(NODEINST*, POLYLIST*, WINDOWPART*, BOOLEAN);
 INTBIG  sch_nodeEpolys(NODEINST*, INTBIG*, WINDOWPART*);
 void    sch_shapeEnodepoly(NODEINST*, INTBIG, POLYGON*);
 INTBIG  sch_allnodeEpolys(NODEINST *ni, POLYLIST *plist, WINDOWPART *win, BOOLEAN onlyreasonable);
+void    sch_nodesizeoffset(NODEINST *ni, INTBIG *lx, INTBIG *ly, INTBIG *hx, INTBIG *hy);
 void    sch_shapeportpoly(NODEINST*, PORTPROTO*, POLYGON*, XARRAY, BOOLEAN);
 INTBIG  sch_arcpolys(ARCINST*, WINDOWPART*);
 void    sch_shapearcpoly(ARCINST*, INTBIG, POLYGON*);
