@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usr.h
@@ -35,7 +36,7 @@ extern "C"
 
 /******************** CONSTANTS ********************/
 
-/* the meaning of NODEINST->userbits: (other bit (037400177) used globally) */
+/* the meaning of NODEINST->userbits: (other bit (0137400177) used globally) */
 #define KILLN                   0200		/* if on, this nodeinst is marked for death */
 #define REWANTN                 0400		/* nodeinst re-drawing is scheduled */
 #define RELOCLN                01000		/* only local nodeinst re-drawing desired */
@@ -110,7 +111,7 @@ extern "C"
 #define EXPORTSFULL                0		/* exports drawn as complete text */
 #define EXPORTSCROSS         0400000		/* exports drawn as crosses */
 #define EXPORTSSHORT        01400000		/* exports drawn as shortened text */
-
+#define MOVENODEWITHEXPORT  02000000		/* move node when export text is moved */
 #define NODATEORVERSION     04000000		/* set to supress date/version in output files */
 #define BEEPAFTERLONGJOB   010000000		/* set to beep after a long job */
 #define FACETCENTERALWAYS  020000000		/* set to place facet-center in new facets */
@@ -126,6 +127,7 @@ extern "C"
 /* the meaning of "JAVA_flags" on the user tool object */
 #define JAVANOCOMPILER             1		/* disable compiler */
 #define JAVANOEVALUATE             2		/* do not evaluate code */
+#define JAVAUSEJOSE                4		/* Use Jose */
 
 /* special keys */
 #define CTRLCKEY                  03		/* the Control-C key */
@@ -251,7 +253,6 @@ extern BOOLEAN      us_optionschanged;		/* true if options changed */
 extern INTBIG       us_layer_letters_key;	/* variable key for "USER_layer_letters" */
 extern INTBIG       us_highlightedkey;		/* variable key for "USER_highlighted" */
 extern INTBIG       us_highlightstackkey;	/* variable key for "USER_highlightstack" */
-extern INTBIG       us_descent_path_key;	/* variable key for "USER_descent_path" */
 extern INTBIG       us_descent_view_key;	/* variable key for "USER_descent_view" */
 extern INTBIG       us_binding_keys_key;	/* variable key for "USER_binding_keys" */
 extern INTBIG       us_binding_buttons_key;	/* variable key for "USER_binding_buttons" */
@@ -292,11 +293,11 @@ extern INTBIG       us_tinyratio;			/* mirror for "USER_tiny_lambda_per_pixel" *
 extern INTBIG       us_separatechar;		/* separating character in port names */
 
 /* file type descriptors */
-extern INTBIG us_filetypecolormap;			/* Color map */
-extern INTBIG us_filetypehelp;				/* Help */
-extern INTBIG us_filetypelog;				/* Log */
-extern INTBIG us_filetypemacro;				/* Macro */
-extern INTBIG us_filetypenews;				/* News */
+extern INTBIG       us_filetypecolormap;	/* Color map */
+extern INTBIG       us_filetypehelp;		/* Help */
+extern INTBIG       us_filetypelog;			/* Log */
+extern INTBIG       us_filetypemacro;		/* Macro */
+extern INTBIG       us_filetypenews;		/* News */
 
 /* internal state */
 extern INTBIG       us_state;				/* miscellaneous state of user interface */
@@ -308,6 +309,7 @@ extern INTBIG       us_alignment_ratio;		/* current alignment in fractional unit
 extern INTBIG       us_edgealignment_ratio;	/* current edge alignment in fractional units */
 extern BOOLEAN      us_dupdistset;			/* true if duplication distance is valid */
 extern NODEINST    *us_dupnode;				/* a node that was duplicated */
+extern LIBRARY     *us_clipboardlib;		/* library with cut/copy/paste facet */
 extern NODEPROTO   *us_clipboardfacet;		/* facet with cut/copy/paste objects */
 extern INTBIG       us_dupx, us_dupy;		/* amount to move duplicated objects */
 extern WINDOWFRAME *us_menuframe;			/* window frame on which menu resides */
@@ -321,6 +323,7 @@ extern INTBIG       us_menulx,us_menuhx;	/* X: low and high of menu area */
 extern INTBIG       us_menuly,us_menuhy;	/* Y: low and high of menu area */
 extern INTBIG       us_lastmeasurex, us_lastmeasurey;	/* last measured distance */
 extern BOOLEAN      us_validmesaure;		/* true if a measure was done */
+extern INTBIG       us_explorerratio;		/* percentage of window used by facet explorer */
 extern FILE        *us_logrecord;			/* logging record file */
 extern FILE        *us_logplay;				/* logging playback file */
 extern FILE        *us_termaudit;			/* messages window auditing file */
@@ -444,7 +447,6 @@ extern POPUPMENU *us_firstpopupmenu;		/* list of existing pop-up menus */
 
 #define MENUBARHEIGHT             25		/* size in pixels */
 
-extern INTBIG      us_menubarsize;			/* actual size (0 if no menu bar) */
 extern INTBIG      us_pulldownmenucount;	/* number of pulldown menus */
 extern INTBIG     *us_pulldownmenupos;		/* position of pulldown menus */
 extern POPUPMENU **us_pulldowns;			/* the current pulldown menus */
@@ -489,9 +491,9 @@ extern STATUSFIELD *us_statusangle;			/* current placement angle */
 extern STATUSFIELD *us_statusarc;			/* current arc prototype */
 extern STATUSFIELD *us_statusfacet;			/* current facet */
 extern STATUSFIELD *us_statusfacetsize;		/* current facet size */
+extern STATUSFIELD *us_statusselectcount;	/* current selection count */
 extern STATUSFIELD *us_statusgridsize;		/* current grid size */
 extern STATUSFIELD *us_statuslambda;		/* current lambda */
-extern STATUSFIELD *us_statusnetwork;		/* current network */
 extern STATUSFIELD *us_statusnode;			/* current node prototype */
 extern STATUSFIELD *us_statustechnology;	/* current technology */
 extern STATUSFIELD *us_statusxpos;			/* current x position */
@@ -554,27 +556,27 @@ extern KEYWORD *us_pathiskey;				/* the current keyword list */
 /******************** PROTOTYPES ********************/
 
 /* prototypes for tool interface */
-void us_init(INTBIG*, char*[], TOOL*);
-void us_done(void);
-void us_set(INTBIG, char*[]);
+void   us_init(INTBIG*, char*[], TOOL*);
+void   us_done(void);
+void   us_set(INTBIG, char*[]);
 INTBIG us_request(char*, va_list);
-void us_examinenodeproto(NODEPROTO*);
-void us_slice(void);
-void us_startbatch(TOOL*, BOOLEAN);
-void us_endbatch(void);
-void us_startobjectchange(INTBIG, INTBIG);
-void us_endobjectchange(INTBIG, INTBIG);
-void us_modifynodeinst(NODEINST*,INTBIG,INTBIG,INTBIG,INTBIG,INTBIG,INTBIG);
-void us_modifynodeproto(NODEPROTO*);
-void us_modifydescript(INTBIG, INTBIG, INTBIG, UINTBIG*);
-void us_newobject(INTBIG, INTBIG);
-void us_killobject(INTBIG, INTBIG);
-void us_newvariable(INTBIG, INTBIG, INTBIG, INTBIG);
-void us_killvariable(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, UINTBIG*);
-void us_modifyvariable(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
-void us_readlibrary(LIBRARY*);
-void us_eraselibrary(LIBRARY*);
-void us_writelibrary(LIBRARY*, BOOLEAN);
+void   us_examinenodeproto(NODEPROTO*);
+void   us_slice(void);
+void   us_startbatch(TOOL*, BOOLEAN);
+void   us_endbatch(void);
+void   us_startobjectchange(INTBIG, INTBIG);
+void   us_endobjectchange(INTBIG, INTBIG);
+void   us_modifynodeinst(NODEINST*,INTBIG,INTBIG,INTBIG,INTBIG,INTBIG,INTBIG);
+void   us_modifynodeproto(NODEPROTO*);
+void   us_modifydescript(INTBIG, INTBIG, INTBIG, UINTBIG*);
+void   us_newobject(INTBIG, INTBIG);
+void   us_killobject(INTBIG, INTBIG);
+void   us_newvariable(INTBIG, INTBIG, INTBIG, INTBIG);
+void   us_killvariable(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, UINTBIG*);
+void   us_modifyvariable(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
+void   us_readlibrary(LIBRARY*);
+void   us_eraselibrary(LIBRARY*);
+void   us_writelibrary(LIBRARY*, BOOLEAN);
 
 /* prototypes for user commands */
 void us_arc(INTBIG, char*[]);
@@ -690,23 +692,26 @@ USERCOM       *us_buildcommand(INTBIG, char*[]);
 void           us_buildquickkeylist(void);
 void           us_buildtexthighpoly(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, POLYGON*);
 void           us_buttonhandler(WINDOWPART*, INTBIG, INTBIG, INTBIG);
-BOOLEAN        us_cantedit(NODEPROTO*, NODEPROTO*, BOOLEAN);
+BOOLEAN        us_cantedit(NODEPROTO*, NODEINST*, BOOLEAN);
 BOOLEAN        us_charhandler(WINDOWPART*, INTSML, INTBIG);
 void           us_chatportproto(NODEINST*, PORTPROTO*);
 void           us_check_facet_date(NODEPROTO*, UINTBIG);
 void           us_checkdatabase(BOOLEAN);
 void           us_checkspiceparts(void);
 void           us_cleanupfacet(NODEPROTO*, BOOLEAN);
+void           us_clearclipboard(void);
 void           us_clearhighlightcount(void);
 void           us_clearwindow(WINDOWPART*);
 char          *us_commandvarname(INTSML);
 void           us_computearcfartextbit(ARCINST *ai);
 void           us_computenodefartextbit(NODEINST *ni);
 UINTBIG        us_computewipestate(NODEINST *ni);
+void           us_continuesessionlogging(void);
 NODEPROTO     *us_convertfacet(NODEPROTO*, TECHNOLOGY*);
 char          *us_convertmactoworld(char *buf);
 char          *us_convertworldtomac(char *buf);
-void           us_copylisttofacet(GEOM**, NODEPROTO*, NODEPROTO*, BOOLEAN, BOOLEAN);
+void           us_copyexplorerwindow(void);
+void           us_copylisttofacet(GEOM**, NODEPROTO*, NODEPROTO*, BOOLEAN, BOOLEAN, BOOLEAN);
 void           us_copyobjects(WINDOWPART*);
 NODEPROTO     *us_copyrecursively(NODEPROTO*, char*, LIBRARY*, VIEW*, BOOLEAN, BOOLEAN, char*, BOOLEAN, BOOLEAN);
 void           us_coverimplant(void);
@@ -721,6 +726,7 @@ void           us_delfacetmessage(NODEPROTO*);
 void           us_delhighlight(HIGHLIGHT*);
 void           us_delnodeprotocenter(NODEPROTO*);
 void           us_deltecedlayerfacet(NODEPROTO *np);
+void           us_deltecednodefacet(NODEPROTO *np);
 BOOLEAN        us_demandxy(INTBIG*, INTBIG*);
 char          *us_describeboundkey(INTSML key, INTBIG special, INTBIG readable);
 void           us_describecontents(NODEPROTO*);
@@ -769,7 +775,10 @@ void           us_execute(USERCOM*, BOOLEAN, BOOLEAN, BOOLEAN);
 BOOLEAN        us_expandaddrtypearray(INTBIG*, INTBIG**, INTBIG**, INTBIG);
 void           us_explorebuttonhandler(WINDOWPART*, INTBIG, INTBIG, INTBIG);
 BOOLEAN        us_explorecharhandler(WINDOWPART*, INTSML, INTBIG);
-void           us_explorepan(WINDOWPART*, INTBIG);
+void           us_explorehpan(WINDOWPART*, INTBIG);
+void           us_explorevpan(WINDOWPART*, INTBIG);
+void           us_explorerreadoutinit(WINDOWPART *w);
+char          *us_explorerreadoutnext(WINDOWPART *w, INTBIG *indent, INTBIG *style, BOOLEAN **donelist, void **en);
 void           us_exploreredisphandler(WINDOWPART*);
 int            us_exportnameindexascending(const void *e1, const void *e2);
 int            us_exportnametypeascending(const void *e1, const void *e2);
@@ -785,6 +794,7 @@ INTBIG         us_fillcomcomp(USERCOM*, COMCOMP*[]);
 INTBIG         us_findboundkey(INTSML key, INTBIG special, char **binding);
 void           us_findlowerport(NODEINST**, PORTPROTO**);
 void           us_findobject(INTBIG, INTBIG, WINDOWPART*, HIGHLIGHT*, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
+VARIABLE      *us_findparametersource(VARIABLE *var, NODEINST *ni);
 void           us_forceeditchanges(void);
 void           us_freebindingparse(COMMANDBINDING *commandbinding);
 void           us_freecomekmemory(void);
@@ -803,6 +813,7 @@ void           us_freestatusmemory(void);
 void           us_freetranslatememory(void);
 void           us_freeusercom(USERCOM*);
 void           us_fullview(NODEPROTO*, INTBIG*, INTBIG*, INTBIG*, INTBIG*);
+void           us_geomhaschanged(GEOM *geom);
 BOOLEAN        us_geominrect(GEOM *geom, INTBIG slx, INTBIG shx, INTBIG sly, INTBIG shy);
 INTBIG        *us_getallhighlighted(void);
 void           us_getarcbounds(ARCINST *ai, INTBIG *nlx, INTBIG *nhx, INTBIG *nly, INTBIG *nhy);
@@ -846,16 +857,17 @@ void           us_graphfacets(NODEPROTO *top);
 void           us_gridset(WINDOWPART*, INTBIG);
 BOOLEAN        us_hasotherwindowpart(INTBIG x, INTBIG y, WINDOWPART *ow);
 void           us_highlighteverywhere(GEOM*, PORTPROTO*, INTBIG, BOOLEAN, INTBIG, BOOLEAN);
+void           us_highlighthaschanged(void);
 void           us_highlightmenu(INTBIG, INTBIG, INTBIG);
 void           us_highlightwindow(WINDOWPART*, BOOLEAN);
 TECHNOLOGY    *us_hightech(HIGHLIGHT *high);
+NODEPROTO     *us_highfacet(HIGHLIGHT *high);
 void           us_hsvtorgb(float, float, float, INTBIG*, INTBIG*, INTBIG*);
 void           us_identifyports(INTBIG, NODEINST**, NODEPROTO*, INTBIG, BOOLEAN);
 void           us_illustratecommandset(void);
 void           us_inheritattributes(NODEINST *ni);
 BOOLEAN        us_initialbinding(void);
 void           us_initlayerletters(void);
-void           us_initpulldownmenu(POPUPMENU*);
 void           us_initqueuedexports(void);
 void           us_initsearchcircuittext(WINDOWPART *win);
 void           us_initstatus(void);
@@ -872,7 +884,7 @@ USERCOM       *us_makecommand(char*);
 ARCINST       *us_makeconnection(NODEINST*, PORTPROTO*, ARCPROTO*, NODEINST*, PORTPROTO*,
 				ARCPROTO*, NODEPROTO*, INTBIG, INTBIG,
 				ARCINST**, ARCINST**, NODEINST**, NODEINST**,
-				INTBIG, BOOLEAN, INTBIG*);
+				INTBIG, BOOLEAN, INTBIG*, BOOLEAN);
 char          *us_makefacetline(NODEPROTO*, INTBIG);
 BOOLEAN        us_makehighlight(char*, HIGHLIGHT*);
 char          *us_makehighlightstring(HIGHLIGHT*);
@@ -925,17 +937,20 @@ void           us_regridfacet(void);
 char          *us_removeampersand(char *name);
 void           us_removeubchange(NODEPROTO*);
 void           us_renameport(PORTPROTO*, char*);
+void           us_renametecedcell(CELL *c, char *oldname);
 void           us_replaceallarcs(GEOM **list, ARCPROTO *ap, BOOLEAN connected, BOOLEAN thisfacet);
 void           us_replacecircuittext(WINDOWPART *win, char *replace);
 void           us_replacelibraryreferences(LIBRARY *oldlib, LIBRARY *newlib);
 NODEINST      *us_replacenodeinst(NODEINST *oldni, NODEPROTO *newnp, BOOLEAN ignoreportnames, BOOLEAN allowdeletedports);
+void           us_reportarcscreated(char *who, INTBIG numarcs1, char *type1, INTBIG numarcs2, char *type2);
 void           us_rgbtohsv(INTBIG, INTBIG, INTBIG, float*, float*, float*);
 void           us_rotatedescript(GEOM*, UINTBIG*);
 void           us_rotatedescriptI(GEOM*, UINTBIG*);
 ARCINST       *us_runarcinst(NODEINST*, PORTPROTO*, INTBIG, INTBIG, NODEINST*, PORTPROTO*, INTBIG, INTBIG, ARCPROTO*, INTBIG);
 BOOLEAN        us_samecontents(NODEPROTO *np1, NODEPROTO *np2, INTBIG explain);
 BOOLEAN        us_samekey(INTSML key1, INTBIG special1, INTSML key2, INTBIG special2);
-BOOLEAN        us_saveoptdlog(void);
+void           us_saveeverything(void);
+BOOLEAN        us_saveoptdlog(BOOLEAN atquit);
 void           us_saveoptions(void);
 void           us_scaletowindow(INTBIG*, INTBIG*, WINDOWPART*);
 void           us_scaletraceinfo(NODEINST*, INTBIG, INTBIG, INTBIG, INTBIG);
@@ -951,11 +966,12 @@ void           us_setarcproto(ARCPROTO*, BOOLEAN);
 void           us_setcolorentry(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, BOOLEAN);
 void           us_setcursorpos(WINDOWFRAME*, INTBIG, INTBIG);
 void           us_setdescriptoffset(UINTBIG*, INTBIG, INTBIG);
-void           us_setfacetname(WINDOWPART*);
-void           us_setfacetsize(WINDOWPART*);
+void           us_setessentialbounds(NODEPROTO *facet);
+void           us_setfacetname(WINDOWPART *w);
+void           us_setfacetsize(WINDOWPART *w);
 void           us_setfind(HIGHLIGHT*, INTBIG, INTBIG, INTBIG, INTBIG);
 void           us_setfloatvariablevalue(GEOM *geom, INTBIG key, VARIABLE *oldvar, float value);
-void           us_setgridsize(WINDOWPART*);
+void           us_setgridsize(WINDOWPART *w);
 void           us_sethighlight(HIGHLIGHT*, INTBIG);
 void           us_setkeybinding(char *binding, INTSML key, INTBIG special, BOOLEAN quietly);
 void           us_setkeyequiv(POPUPMENU*, INTBIG);
@@ -966,20 +982,22 @@ void           us_setmultiplehighlight(char*, BOOLEAN);
 void           us_setnodeangle(WINDOWFRAME*);
 void           us_setnodeproto(NODEPROTO*);
 void           us_setnodeprotocenter(INTBIG, INTBIG, NODEPROTO*);
-NODEPROTO     *us_setscrolltocurrentfacet(INTBIG scrollitem, BOOLEAN curinstance,
-					BOOLEAN forcecurlib, BOOLEAN justcells);
+NODEPROTO     *us_setscrolltocurrentfacet(INTBIG scrollitem, BOOLEAN curinstance, BOOLEAN curfacet,
+					BOOLEAN forcecurlib, BOOLEAN justcells, void *dia);
+void           us_setselectioncount(void);
 void           us_settechname(WINDOWFRAME*);
 void           us_settrace(NODEINST*, INTBIG*, INTBIG);
 INTBIG         us_setunexpand(NODEINST*, INTBIG);
 void           us_setvariablevalue(GEOM *geom, INTBIG key, char *newvalue, INTBIG oldtype, UINTBIG *descript);
 void           us_setwindowsimulation(WINDOWPART*, BOOLEAN);
+void           us_setxarray(INTBIG addr, INTBIG type, char *name, XARRAY array);
 BOOLEAN        us_setxy(INTBIG, INTBIG);
 void           us_shadowarcproto(WINDOWFRAME*, ARCPROTO*);
 void           us_shadownodeproto(WINDOWFRAME*, NODEPROTO*);
 void           us_showallhighlight(void);
 void           us_showcurrentarcproto(void);
 void           us_showcurrentnodeproto(void);
-void           us_shownetworkname(WINDOWFRAME*);
+void           us_showlayercoverage(NODEPROTO *facet);
 void           us_showpoly(POLYGON*, WINDOWPART*);
 NODEPROTO     *us_skeletonize(NODEPROTO *np, char *newname, LIBRARY *newlib, BOOLEAN quiet);
 void           us_slideleft(INTBIG);
@@ -988,7 +1006,7 @@ int            us_sortarcsbyname(const void *e1, const void *e2);
 NODEPROTO    **us_sortlib(LIBRARY*, char*, BOOLEAN);
 int            us_sortnodesbyname(const void *e1, const void *e2);
 int            us_sortpopupmenuascending(const void *e1, const void *e2);
-WINDOWPART    *us_splitcurrentwindow(INTBIG, BOOLEAN, WINDOWPART**);
+WINDOWPART    *us_splitcurrentwindow(INTBIG, BOOLEAN, WINDOWPART**, INTBIG);
 INTBIG         us_spreadaround(NODEINST *ni, INTBIG amount, char *direction);
 void           us_spreadrotateconnection(NODEINST *theni);
 void           us_squarescreen(WINDOWPART*, WINDOWPART*, BOOLEAN, INTBIG*, INTBIG*, INTBIG*, INTBIG*, INTBIG);
@@ -999,12 +1017,14 @@ void           us_switchtofacet(NODEPROTO*, INTBIG, INTBIG, INTBIG, INTBIG, NODE
 void           us_switchtolibrary(LIBRARY*);
 void           us_tecedaddfunstring(void*, INTBIG);
 void           us_teceddeletelayerfacet(NODEPROTO *np);
+void           us_teceddeletenodefacet(NODEPROTO *np);
 void           us_tecedentry(INTBIG, char*[]);
 INTBIG         us_tecedgetoption(NODEINST*);
+void           us_tecedrenamecell(char *oldname, char *newname);
 char          *us_techname(NODEPROTO *np);
 char          *us_tempoptionslibraryname(void);
-void           us_termpulldownmenu(POPUPMENU*);
 void           us_textfacetchanges(WINDOWPART*, INTBIG, char*, char*, INTBIG);
+void           us_toolturnedon(TOOL *tool);
 BOOLEAN        us_topofallthings(char**);
 BOOLEAN        us_topofarcnodes(char**);
 BOOLEAN        us_topofcellfacets(char**);
@@ -1060,7 +1080,7 @@ void           ttysetstatusfield(WINDOWFRAME *frame, STATUSFIELD *field, char *m
 INTBIG         ttynumstatuslines(void);
 
 /* popup and pulldown menus */
-BOOLEAN        nativemenuload(INTBIG, char*[]);
+void           nativemenuload(INTBIG, char*[]);
 void           nativemenurename(POPUPMENU*, INTBIG);
 INTBIG         nativepopupmenu(POPUPMENU **menu, BOOLEAN header, INTBIG left, INTBIG top);
 
