@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: vhdl.h
@@ -351,7 +352,7 @@ typedef struct Idbbody
 typedef struct Idbbodydeclare
 {
 	struct Idbcomponents	*components;	/* components */
-	struct Idbsignals		*signals;		/* signals */
+	struct Idbsignals		*bodysignals;		/* signals */
 } DBBODYDECLARE;
 
 typedef struct Idbcomponents
