@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iobinaryi.c
@@ -41,6 +42,9 @@
 
 #define REPORTINC   2000			/* bytes between graphical updates */
 
+#define LONGJMPABORTED 1			/* for "Aborted" */
+#define LONGJMPEOF     2			/* for "Premature end of file" */
+
 typedef struct
 {
 	BOOLEAN      toolbitsmessed;	/* 0: tool order is correct */
@@ -105,6 +109,7 @@ static BININPUTDATA io_binindata;				/* all data pertaining to reading a file */
 static char        *io_tempstring;				/* for reading temporary strings */
 static INTBIG       io_tempstringlength = 0;	/* length of temporary string */
 static char         io_mainlibdirectory[500];	/* root directory where binary file lives */
+       void        *io_inputprogressdialog;		/* for showing input progress */
 
 /* prototypes for local routines */
 static char       *io_doreadlibrary(LIBRARY*);
@@ -194,7 +199,6 @@ BOOLEAN io_doreadbinlibrary(LIBRARY *lib, BOOLEAN newprogress)
 {
 	REGISTER char *result;
 	char *filename;
-	extern DIALOG us_eprogressdialog;
 	REGISTER void *infstr;
 
 	/* find the library file */
@@ -219,17 +223,18 @@ BOOLEAN io_doreadbinlibrary(LIBRARY *lib, BOOLEAN newprogress)
 		{
 			if (newprogress)
 			{
-				if (DiaInitDialog(&us_eprogressdialog))
+				io_inputprogressdialog = DiaInitProgress("", _("Reading library"));
+				if (io_inputprogressdialog == 0)
 				{
 					xclose(io_binindata.filein);
 					return(TRUE);
 				}
 			}
-			DiaPercent(1, 0);
+			DiaSetProgress(io_inputprogressdialog, 0, io_binindata.filelength);
 			infstr = initinfstr();
 			formatinfstr(infstr, _("Reading library %s"), lib->libname);
-			DiaSetText(3, returninfstr(infstr));
-			DiaSetText(2, _("Initializing..."));
+			DiaSetCaptionProgress(io_inputprogressdialog, returninfstr(infstr));
+			DiaSetTextProgress(io_inputprogressdialog, _("Initializing..."));
 		}
 	} else io_binindata.filelength = 0;
 	io_binindata.reported = 0;
@@ -253,7 +258,7 @@ BOOLEAN io_doreadbinlibrary(LIBRARY *lib, BOOLEAN newprogress)
 	/* clean up */
 	xclose(io_binindata.filein);
 	if (io_verbose < 0 && io_binindata.filelength > 0 && newprogress)
-		DiaDoneDialog();
+		DiaDoneProgress(io_inputprogressdialog);
 	if (io_binindata.swapped_floats != 0)
 		ttyputverbose(M_("Read %ld swapped float variables; may be incorrect"),
 			io_binindata.swapped_floats);
@@ -273,7 +278,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 	REGISTER INTBIG te;
 	REGISTER BOOLEAN imosconv, showerror;
 	REGISTER BOOLEAN *geomtype;
-	REGISTER INTBIG i, thisone, *geommoreup, top, advise,
+	REGISTER INTBIG i, thisone, *geommoreup, top, advise, errorcode,
 		bot, look, nodeindex, arcindex, geomindex, oldunit, convertmosiscmostechnologies;
 	INTBIG num, den, j, count, cou, arcinstpos, nodeinstpos, portprotopos,
 		emajor, eminor, edetail;
@@ -288,15 +293,20 @@ char *io_doreadlibrary(LIBRARY *lib)
 	REGISTER PORTEXPINST **portexpinstlist;
 	REGISTER PORTARCINST **portarcinstlist;
 	REGISTER TOOL *tool;
-	char *version, *errormsg;
+	char *version;
 	unsigned char chmagic[4];
 	REGISTER VIEW *v;
 	REGISTER CELL *c, *lastcell;
 	REGISTER LIBRARY *savelib, *olib;
 	REGISTER void *infstr;
 
-	errormsg = (char *)setjmp(io_binindata.filerror);
-	if (errormsg != 0) return(errormsg);
+	errorcode = setjmp(io_binindata.filerror);
+	if (errorcode != 0)
+	{
+		if (errorcode == LONGJMPABORTED) return(_("Aborted"));
+		if (errorcode == LONGJMPEOF) return(_("Premature end of file"));
+		return(_("Unknown error"));
+	}
 
 	/* read magic number */
 	io_getin(&io_binindata.magic, 4, SIZEOFINTBIG, TRUE);
@@ -632,6 +642,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 			}
 		}
 	}
+
 	/* allocate the nodes, arcs, and ports in each facet */
 	nodeinstpos = arcinstpos = portprotopos = 0;
 	for(i=0; i<io_binindata.nodeprotoindex; i++)
@@ -830,8 +841,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 				io_binindata.portpprotoerror[io_binindata.portpprotoindex] = 0;
 				name = io_gettempstring();
 				if (name == 0) return(_("No memory"));
-				for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-					if (strcmp(pp->protoname, name) == 0) break;
+				pp = getportproto(np, name);
 
 				/* convert special port names */
 				if (pp == NOPORTPROTO)
@@ -1167,7 +1177,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 			{
 				infstr = initinfstr();
 				formatinfstr(infstr, _("Reading %s"), describenodeproto(np));
-				DiaSetText(2, returninfstr(infstr));
+				DiaSetTextProgress(io_inputprogressdialog, returninfstr(infstr));
 			} else ttyputmsg(_("Reading %s"), describenodeproto(np));
 			el_curlib = savelib;
 		}
@@ -1221,8 +1231,8 @@ char *io_doreadlibrary(LIBRARY *lib)
 
 	if (io_verbose < 0 && io_binindata.filelength > 0)
 	{
-		DiaPercent(1, 100);
-		DiaSetText(2, _("Cleaning up..."));
+		DiaSetProgress(io_inputprogressdialog, 999, 1000);
+		DiaSetTextProgress(io_inputprogressdialog, _("Cleaning up..."));
 	}
 
 	/* transform indices to pointers */
@@ -1332,7 +1342,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 
 	/* create those pointers not on the disk file */
 	if (io_verbose < 0 && io_binindata.filelength > 0)
-		io_fixnewlib(lib, 2); else
+		io_fixnewlib(lib, io_inputprogressdialog); else
 			io_fixnewlib(lib, 0);
 
 	/* look for any database changes that did not matter */
@@ -1492,6 +1502,7 @@ BOOLEAN io_readnodeproto(NODEPROTO *np)
 
 	/* read the number of portprotos on this nodeproto */
 	io_getinbig(&portcount);
+	np->numportprotos = portcount;
 
 	/* read the portprotos on this nodeproto */
 	lpt = NOPORTPROTO;
@@ -1614,7 +1625,7 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 	REGISTER LIBRARY *elib;
 	REGISTER BOOLEAN failed;
 	REGISTER char *protoname, *libname, *pt;
-	char *filename, *libfile, *oldline2, *oldline3, *libfilepath, *libfilename,
+	char *filename, *libfile, *oldline2, *libfilepath, *libfilename,
 		*dummycellname, **localportnames;
 	FILE *io;
 	REGISTER PORTPROTO *pp, *lpt;
@@ -1709,8 +1720,7 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 		savebinindata = io_binindata;
 		if (io_verbose < 0 && io_binindata.filelength > 0)
 		{
-			(void)allocstring(&oldline2, DiaGetText(2), el_tempcluster);
-			(void)allocstring(&oldline3, DiaGetText(3), el_tempcluster);
+			(void)allocstring(&oldline2, DiaGetTextProgress(io_inputprogressdialog), el_tempcluster);
 		}
 
 		len = strlen(elib->libfile);
@@ -1728,18 +1738,18 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 		io_libinputrecursivedepth--;
 		if (failed) elib->userbits |= UNWANTEDLIB; else
 		{
-			for(k=0; k<el_maxtools; k++)
-				if (el_tools[k].readlibrary != 0)
-					(*el_tools[k].readlibrary)(elib);
+			/* queue this library for announcement through change control */
+			io_queuereadlibraryannouncement(elib);
 		}
 		io_binindata = savebinindata;
 		if (io_verbose < 0 && io_binindata.filelength > 0)
 		{
-			DiaPercent(1, io_binindata.bytecount*100/io_binindata.filelength);
-			DiaSetText(2, oldline2);
-			DiaSetText(3, oldline3);
+			DiaSetProgress(io_inputprogressdialog, io_binindata.bytecount, io_binindata.filelength);
+			infstr = initinfstr();
+			formatinfstr(infstr, _("Reading library %s"), lib->libname);
+			DiaSetCaptionProgress(io_inputprogressdialog, returninfstr(infstr));
+			DiaSetTextProgress(io_inputprogressdialog, oldline2);
 			efree(oldline2);
-			efree(oldline3);
 		}
 	}
 
@@ -1792,7 +1802,7 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 			np->lowy != lowy || np->highy != highy)
 		{
 			ttyputerr(_("Error: facet %s in library %s has changed size since its use in library %s"),
-				describenodeproto(np), elib->libname, lib->libname);
+				nldescribenodeproto(np), elib->libname, lib->libname);
 			np = NONODEPROTO;
 		}
 	}
@@ -1929,8 +1939,7 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 	{
 		/* read the portproto name */
 		protoname = localportnames[j];
-		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-			if (namesame(protoname, pp->protoname) == 0) break;
+		pp = getportproto(np, protoname);
 		if (pp == NOPORTPROTO)
 		{
 			if (newfacet == 0)
@@ -2348,6 +2357,7 @@ INTBIG io_readvariables(INTBIG addr, INTBIG type)
 					io_binindata.realname[key], us_variabletypename(type));
 			}
 		}
+
 		/* copy this variable into database */
 		if (io_binindata.newnames[key] == 0)
 			io_binindata.newnames[key] = makekey(io_binindata.realname[key]);
@@ -2936,7 +2946,7 @@ TECHNOLOGY *io_gettechlist(INTBIG i)
 		io_binindata.techerror[i] = 0;
 		count = ttygetparam(_("Is this OK? [n] "), &us_noyesp, 2, pars);
 		if (count <= 0 || namesamen(pars[0], "yes", strlen(pars[0])) != 0)
-			longjmp(io_binindata.filerror, (INTBIG)_("Aborted"));
+			longjmp(io_binindata.filerror, LONGJMPABORTED);
 		ttyputerr(_("WARNING: saving this library with a substitute technology may corrupt it!"));
 	}
 	return(io_binindata.techlist[i]);
@@ -2979,12 +2989,12 @@ void io_getin(void *data, INTBIG disksize, INTBIG memorysize, BOOLEAN signextend
 	{
 		/* just peel it off the disk */
 		ret = xfread((char *)data, 1, disksize, io_binindata.filein);
-		if (ret == 0) longjmp(io_binindata.filerror, (INTBIG)_("Premature end of file"));
+		if (ret == 0) longjmp(io_binindata.filerror, LONGJMPEOF);
 	} else
 	{
 		/* not a simple read, use a buffer */
 		ret = xfread((char *)buf, 1, disksize, io_binindata.filein);
-		if (ret == 0) longjmp(io_binindata.filerror, (INTBIG)_("Premature end of file"));
+		if (ret == 0) longjmp(io_binindata.filerror, LONGJMPEOF);
 		if (io_binindata.swap_bytes > 0)
 		{
 			switch (disksize)
@@ -3035,29 +3045,33 @@ void io_getin(void *data, INTBIG disksize, INTBIG memorysize, BOOLEAN signextend
 	{
 		if (io_binindata.bytecount > io_binindata.reported + REPORTINC)
 		{
-			DiaPercent(1, io_binindata.bytecount*100/io_binindata.filelength);
+			DiaSetProgress(io_inputprogressdialog, io_binindata.bytecount, io_binindata.filelength);
 			io_binindata.reported = io_binindata.bytecount;
 		}
 	}
 }
+
 char *io_getstring(CLUSTER *cluster)
 {
 	INTBIG len;
+	REGISTER INTBIG oldswapbytes;
 	char *name;
 
 	io_getinbig(&len);
 	name = (char *)emalloc((len+1), cluster);
 	if (name == 0) return(0);
-	if (io_binindata.swap_bytes != 0 && len != 0) io_binindata.swap_bytes = -1;     /* prevent swapping */
+	oldswapbytes = io_binindata.swap_bytes;
+	if (io_binindata.swap_bytes != 0 && len != 0) io_binindata.swap_bytes = 0;     /* prevent swapping */
 	if (len != 0) io_getin(name, len, len, FALSE);
 	name[len] = 0;
-	if (io_binindata.swap_bytes != 0) io_binindata.swap_bytes = 1;
+	io_binindata.swap_bytes = oldswapbytes;
 	return(name);
 }
 
 char *io_gettempstring(void)
 {
 	INTBIG len;
+	REGISTER INTBIG oldswapbytes;
 
 	io_getinbig(&len);
 	if (len+1 > io_tempstringlength)
@@ -3067,9 +3081,10 @@ char *io_gettempstring(void)
 		io_tempstring = (char *)emalloc(io_tempstringlength, io_tool->cluster);
 		if (io_tempstring == 0) { io_tempstringlength = 0; return(0); }
 	}
-	if (io_binindata.swap_bytes != 0 && len != 0) io_binindata.swap_bytes = -1;
+	oldswapbytes = io_binindata.swap_bytes;
+	if (io_binindata.swap_bytes != 0 && len != 0) io_binindata.swap_bytes = 0;
 	if (len != 0) io_getin(io_tempstring, len, len, FALSE);
 	io_tempstring[len] = 0;
-	if (io_binindata.swap_bytes != 0) io_binindata.swap_bytes = 1;
+	io_binindata.swap_bytes = oldswapbytes;
 	return(io_tempstring);
 }
