@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iobinaryo.c
@@ -33,12 +34,15 @@
 #include "usr.h"
 #include "edialogs.h"
 
+#define FILECOPYBUFSIZE 4096
+
 static BOOLEAN io_outputstage;		/* progress of database (aborting) */
 static char   *io_namesave;			/* for saving the global namespace */
 static char   *io_outfilename = 0;	/* the name of the file being written */
 
 /* prototypes for local routines */
 static void io_cleanup(void);
+static BOOLEAN io_copyfile(char *src, char *dst);
 static void io_writenodeproto(NODEPROTO*, BOOLEAN);
 static void io_writenodeinst(NODEINST*);
 static void io_writearcinst(ARCINST*);
@@ -65,8 +69,8 @@ BOOLEAN io_writebinlibrary(LIBRARY *lib, BOOLEAN nobackup)
 		portprotoindex, portpprotoindex, arcprotoindex, nodeprotoindex, nodeindex,
 		arcindex, geomindex, curnodeproto, cellindex,
 		*curstate, facetshere, filestatus;
-	BOOLEAN err;
-	UINTBIG olddate;
+	BOOLEAN err, createit;
+	time_t olddate;
 	INTBIG year, month, mday, hour, minute, second;
 	INTSML save;
 	REGISTER NODEPROTO *np;
@@ -85,6 +89,7 @@ BOOLEAN io_writebinlibrary(LIBRARY *lib, BOOLEAN nobackup)
 	REGISTER VIEW *v;
 	REGISTER void *infstr;
 
+	/* get the name of the file to write */
 	io_outputstage = FALSE;
 	dotaddr = 0;
 	ext = &lib->libfile[strlen(lib->libfile)-5];
@@ -132,6 +137,7 @@ BOOLEAN io_writebinlibrary(LIBRARY *lib, BOOLEAN nobackup)
 	if (nobackup) curstate[0] &= ~(BINOUTBACKUP|CHECKATWRITE);
 
 	rename = 0;
+	createit = TRUE;
 	filestatus = fileexistence(name);
 	if ((lib->userbits&READFROMDISK) != 0)
 	{
@@ -142,7 +148,7 @@ BOOLEAN io_writebinlibrary(LIBRARY *lib, BOOLEAN nobackup)
 	{
 		msg = 0;
 
-		/* if the file already exists, rename it first */
+		/* if the file already exists, rename it */
 		if (filestatus == 1)
 			rename = (char *)emalloc((strlen(name)+15), el_tempcluster);
 		if (rename != 0)
@@ -153,19 +159,37 @@ BOOLEAN io_writebinlibrary(LIBRARY *lib, BOOLEAN nobackup)
 					(void)strcpy(rename, name);
 					(void)strcat(rename, ".XXXXXX");
 					emaketemp(rename);
-					if (erename(name, rename) != 0)
+					if ((curstate[1]&BINOUTPRECREADATE) != 0)
 					{
-						ttyputerr(_("Could not rename file '%s' to '%s'"), rename, name);
-						efree(rename);
-						rename = 0;
+						if (io_copyfile(name, rename))
+						{
+							efree(rename);
+							rename = 0;
+						}
+						createit = FALSE;
+					} else
+					{
+						if (erename(name, rename) != 0)
+						{
+							ttyputerr(_("Could not rename file '%s' to '%s'"), rename, name);
+							efree(rename);
+							rename = 0;
+						}
 					}
 					break;
 				case BINOUTONEBACK:
 					(void)strcpy(rename, name);
 					(void)strcat(rename, "~");
 					if (fileexistence(rename) == 1) (void)eunlink(rename);
-					if (erename(name, rename) != 0)
-						ttyputerr(_("Could not rename file '%s' to '%s'"), rename, name);
+					if ((curstate[1]&BINOUTPRECREADATE) != 0)
+					{
+						(void)io_copyfile(name, rename);
+						createit = FALSE;
+					} else
+					{
+						if (erename(name, rename) != 0)
+							ttyputerr(_("Could not rename file '%s' to '%s'"), rename, name);
+					}
 					efree(rename);
 					rename = 0;
 					break;
@@ -182,8 +206,15 @@ BOOLEAN io_writebinlibrary(LIBRARY *lib, BOOLEAN nobackup)
 						(void)strcat(rename, ".elib");
 						if (fileexistence(rename) == 0) break;
 					}
-					if (erename(name, rename) != 0)
-						ttyputerr(_("Could not rename file '%s' to '%s'"), rename, name);
+					if ((curstate[1]&BINOUTPRECREADATE) != 0)
+					{
+						(void)io_copyfile(name, rename);
+						createit = FALSE;
+					} else
+					{
+						if (erename(name, rename) != 0)
+							ttyputerr(_("Could not rename file '%s' to '%s'"), rename, name);
+					}
 					efree(rename);
 					rename = 0;
 					break;
@@ -192,10 +223,23 @@ BOOLEAN io_writebinlibrary(LIBRARY *lib, BOOLEAN nobackup)
 	}
 
 	/* create the new library file */
-	io_fileout = xcreate(name, io_filetypeblib, msg, &truename);
+	if (createit)
+	{
+		io_fileout = xcreate(name, io_filetypeblib, msg, &truename);
+	} else
+	{
+		/* remove the ".elib" because "xopen()" adds it */
+		i = strlen(name);
+		name[i-5] = 0;
+		io_fileout = xopen(name, io_filetypeblib|FILETYPEWRITE, 0, &truename);
+		if (io_outfilename == 0) 
+			(void)allocstring(&io_outfilename, truename, io_tool->cluster); else
+				(void)reallocstring(&io_outfilename, truename, io_tool->cluster);
+		name = io_outfilename;
+	}
 	if (io_fileout == NULL)
 	{
-		if (truename != 0) DiaMessageInDialog(_("Cannot write %s"), truename);
+		DiaMessageInDialog(_("Cannot write %s"), name);
 		if (rename != 0) efree(rename);
 		return(TRUE);
 	}
@@ -208,12 +252,8 @@ BOOLEAN io_writebinlibrary(LIBRARY *lib, BOOLEAN nobackup)
 		arg[2] = "library";
 		us_rename(3, arg);
 		if (io_outfilename == 0) 
-		{
-			(void)allocstring(&io_outfilename, truename, io_tool->cluster);
-		} else
-		{
-			(void)reallocstring(&io_outfilename, truename, io_tool->cluster);
-		}
+			(void)allocstring(&io_outfilename, truename, io_tool->cluster); else
+				(void)reallocstring(&io_outfilename, truename, io_tool->cluster);
 		name = io_outfilename;
 	}
 
@@ -1066,6 +1106,47 @@ void io_putoutvar(INTBIG *addr, INTBIG ty)
 
 /******************** I/O ROUTINES ********************/
 
+/*
+ * Routine to copy file "src" to file "dst".  Displays an message and returns TRUE on error.
+ */
+BOOLEAN io_copyfile(char *src, char *dst)
+{
+	FILE *srcf, *dstf;
+	char buf[FILECOPYBUFSIZE], *truesrcname, *truedstname;
+	REGISTER INTBIG total, written;
+
+	srcf = xopen(src, io_filetypeblib, 0, &truesrcname);
+	if (srcf == NULL)
+	{
+		ttyputerr(_("Error: cannot open old library file %s"), src);
+		return(TRUE);
+	}
+	dstf = xcreate(dst, io_filetypeblib, 0, &truedstname);
+	if (dstf == NULL)
+	{
+		xclose(srcf);
+		ttyputerr(_("Error: cannot create new library file %s"), dst);
+		return(TRUE);
+	}
+	for(;;)
+	{
+		total = xfread(buf, 1, FILECOPYBUFSIZE, srcf);
+		if (total <= 0) break;
+		written = xfwrite(buf, 1, total, dstf);
+		if (written != total)
+		{
+			ttyputerr(_("Error: only wrote %ld out of %ld bytes to file %s"),
+				written, total, truedstname);
+			xclose(srcf);
+			xclose(dstf);
+			return(TRUE);
+		}
+	}
+	xclose(srcf);
+	xclose(dstf);
+	return(FALSE);
+}
+
 void io_putoutbig(void *data)
 {
 	io_putout(data, SIZEOFINTBIG);
