@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iocifin.c
@@ -29,8 +30,6 @@
  */
 
 #include "config.h"
-#if IOCIF
-
 #include "global.h"
 #include "database.h"
 #include "eio.h"
@@ -160,7 +159,7 @@ static INTBIG      io_cifnotfoundlimit = 0;	/* size of unknown layer name list *
 /* prototypes for local routines */
 static BOOLEAN    io_interpret(FILE*);
 static BOOLEAN    io_listtonodes(LIBRARY*);
-static NODEPROTO *io_nodes_start(LIBRARY*);
+static NODEPROTO *io_nodes_start(LIBRARY*, void*);
 static BOOLEAN    io_nodes_box(void);
 static BOOLEAN    io_nodes_poly(void);
 static BOOLEAN    io_nodes_call(void);
@@ -265,29 +264,30 @@ BOOLEAN io_interpret(FILE *file)
 
 BOOLEAN io_listtonodes(LIBRARY *lib)
 {
-	extern DIALOG us_progressdialog;
+	REGISTER void *dia;
 
 	if (io_verbose < 0)
 	{
-		if (DiaInitDialog(&us_progressdialog)) return(TRUE);
-		DiaPercent(1, 100);
+		dia = DiaInitProgress(0, 0);
+		if (dia == 0) return(TRUE);
+		DiaSetProgress(dia, 999, 1000);
 	}
 	io_incell = NONODEPROTO;
 	for(io_curlist = io_ciflist; io_curlist != NOCIFLIST; io_curlist = io_curlist->next)
 	{
 		if (stopping(STOPREASONCIF))
 		{
-			if (io_verbose < 0) DiaDoneDialog();
+			if (io_verbose < 0) DiaDoneDialog(dia);
 			return(TRUE);
 		}
 		if (io_incell != NONODEPROTO || io_curlist->identity == C_START)
 			switch (io_curlist->identity)
 		{
 			case C_START:
-				io_incell = io_nodes_start(lib);
+				io_incell = io_nodes_start(lib, dia);
 				if (io_incell == NONODEPROTO)
 				{
-					if (io_verbose < 0) DiaDoneDialog();
+					if (io_verbose < 0) DiaDoneProgress(dia);
 					return(TRUE);
 				}
 				break;
@@ -301,31 +301,31 @@ BOOLEAN io_listtonodes(LIBRARY *lib)
 			case C_BOX:
 				if (io_nodes_box())
 				{
-					if (io_verbose < 0) DiaDoneDialog();
+					if (io_verbose < 0) DiaDoneProgress(dia);
 					return(TRUE);
 				}
 				break;
 			case C_POLY:
 				if (io_nodes_poly())
 				{
-					if (io_verbose < 0) DiaDoneDialog();
+					if (io_verbose < 0) DiaDoneProgress(dia);
 					return(TRUE);
 				}
 				break;
 			case C_CALL:
 				if (io_nodes_call())
 				{
-					if (io_verbose < 0) DiaDoneDialog();
+					if (io_verbose < 0) DiaDoneProgress(dia);
 					return(TRUE);
 				}
 				break;
 		}
 	}
-	if (io_verbose < 0) DiaDoneDialog();
+	if (io_verbose < 0) DiaDoneProgress(dia);
 	return(FALSE);
 }
 
-NODEPROTO *io_nodes_start(LIBRARY *lib)
+NODEPROTO *io_nodes_start(LIBRARY *lib, void *dia)
 {
 	CIFCELL *cifcell;
 	CFSTART *cs;
@@ -353,7 +353,7 @@ NODEPROTO *io_nodes_start(LIBRARY *lib)
 	{
 		infstr = initinfstr();
 		formatinfstr(infstr, _("Reading %s"), io_curnodeprotoname);
-		DiaSetText(2, returninfstr(infstr));
+		DiaSetTextProgress(dia, returninfstr(infstr));
 	} else if (io_verbose > 0) ttyputmsg(_("Reading %s"), io_curnodeprotoname);
 	return(cifcell->addr);
 }
@@ -1091,5 +1091,3 @@ void io_outputdf(void)
 {
 	io_placeciflist(C_END);
 }
-
-#endif  /* IOCIF - at top */
