@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iocifout.c
@@ -29,8 +30,6 @@
  */
 
 #include "config.h"
-#if IOCIF
-
 #include "global.h"
 #include "eio.h"
 #include "egraphics.h"
@@ -76,7 +75,7 @@ static INTBIG     io_ciflayerreqlisttotal = 0;
 static void      *io_cifmerge;			/* for merging geometry */
 
 /* prototypes for local routines */
-static void io_cifwritefacet(NODEPROTO*, BOOLEAN);
+static void io_cifwritefacet(NODEPROTO*, BOOLEAN, void*);
 static void io_outputcifpoly(TECHNOLOGY*, POLYGON*, INTBIG, INTBIG, INTBIG, INTBIG, XARRAY, GEOM*);
 static void io_cifprintf(char *msg, ...);
 static void io_cifreserror(GEOM*, INTBIG, INTBIG, INTBIG, INTBIG, XARRAY);
@@ -134,7 +133,7 @@ BOOLEAN io_writeciflibrary(LIBRARY *lib)
 	REGISTER UINTBIG bytesread, i, j;
 	static UINTBIG crcrow[8] = {0x04C11DB7, 0x09823B6E, 0x130476DC, 0x2608EDB8,
 		0x4C11DB70, 0x9823B6E0, 0x34867077, 0x690CE0EE};
-	extern DIALOG us_progressdialog;
+	REGISTER void *dia;
 
 	/* create the proper disk file for the CIF */
 	if (lib->curnodeproto == NONODEPROTO)
@@ -220,11 +219,11 @@ BOOLEAN io_writeciflibrary(LIBRARY *lib)
 	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
 		for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 			np->temp1 = 0;
-	io_cifwritefacet(lib->curnodeproto, TRUE);
+	io_cifwritefacet(lib->curnodeproto, TRUE, 0);
 	io_cifoutjobsize = io_cifoutfacetnumber;
-	if (DiaInitDialog(&us_progressdialog)) return(TRUE);
-	DiaPercent(1, 0);
-	DiaSetText(2, _("Writing CIF..."));
+	dia = DiaInitProgress(_("Writing CIF..."), 0);
+	if (dia == 0) return(TRUE);
+	DiaSetProgress(dia, 0, io_cifoutjobsize);
 
 	/* write the CIF */
 	io_cifoutfacetnumber = 0;
@@ -233,14 +232,14 @@ BOOLEAN io_writeciflibrary(LIBRARY *lib)
 	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
 		for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 			np->temp1 = 0;
-	io_cifwritefacet(lib->curnodeproto, FALSE);
+	io_cifwritefacet(lib->curnodeproto, FALSE, dia);
 
 	/* clean up */
 	if ((io_cifoutputstate[0]&CIFOUTNOTOPCALL) == 0)
 		io_cifprintf("C %ld;\n", io_cifbase);
 	io_cifprintf("E\n");
 	xclose(io_fileout);
-	DiaDoneDialog();
+	DiaDoneProgress(dia);
 
 #ifndef FULLPOLYMERGE
 	/* if doing polygon output */
@@ -283,7 +282,7 @@ BOOLEAN io_writeciflibrary(LIBRARY *lib)
 	return(FALSE);
 }
 
-void io_cifwritefacet(NODEPROTO *np, BOOLEAN fake)
+void io_cifwritefacet(NODEPROTO *np, BOOLEAN fake, void *dia)
 {
 	REGISTER NODEINST *subni;
 	REGISTER ARCINST *ai;
@@ -310,8 +309,8 @@ void io_cifwritefacet(NODEPROTO *np, BOOLEAN fake)
 		/* show progress */
 		infstr = initinfstr();
 		formatinfstr(infstr, _("Writing %s"), describenodeproto(np));
-		DiaSetText(2, returninfstr(infstr));
-		DiaPercent(1, io_cifoutfacetnumber * 100 / io_cifoutjobsize);
+		DiaSetTextProgress(dia, returninfstr(infstr));
+		DiaSetProgress(dia, io_cifoutfacetnumber, io_cifoutjobsize);
 	}
 
 	/* if there are any sub-facets that have not been written, write them */
@@ -333,7 +332,7 @@ void io_cifwritefacet(NODEPROTO *np, BOOLEAN fake)
 		if (subnt->temp1 != 0) continue;
 
 		/* recurse to the bottom */
-		io_cifwritefacet(subnt, fake);
+		io_cifwritefacet(subnt, fake, dia);
 	}
 
 	np->temp1 = ++io_cifbase;
@@ -1082,7 +1081,7 @@ void io_cif_show_reserror(INTBIG x1, INTBIG y1, INTBIG x2, INTBIG y2)
 	/* show resolution error if it is in the current window */
 	infstr = initinfstr();
 	formatinfstr(infstr, _("Resolution error: edge is (%s,%s) to (%s,%s)"),
-		latoa(x1), latoa(y1), latoa(x2), latoa(y2));
+		latoa(x1, 0), latoa(y1, 0), latoa(x2, 0), latoa(y2, 0));
 	err = logerror(returninfstr(infstr), io_cifoutcurnp, 0);
 	addlinetoerror(err, x1, y1, x2, y2);
 	io_reserr_count++;
@@ -1155,7 +1154,7 @@ static DIALOGITEM io_cifoptionsdialogitems[] =
  /* 13 */ {0, {124,232,140,454}, CHECK, N_("Normalize Coordinates")},
  /* 14 */ {0, {76,248,92,454}, MESSAGE, N_("(time consuming)")}
 };
-static DIALOG io_cifoptionsdialog = {{50,75,307,538}, N_("CIF Options"), 0, 14, io_cifoptionsdialogitems};
+static DIALOG io_cifoptionsdialog = {{50,75,307,538}, N_("CIF Options"), 0, 14, io_cifoptionsdialogitems, 0, 0};
 
 /* special items for the "CIF Options" dialog: */
 #define DCFO_LAYERLIST         3		/* Layer list (scroll list) */
@@ -1178,12 +1177,13 @@ void io_cifoptionsdlog(void)
 	REGISTER VARIABLE *var, *cifvar;
 	static char *whattodisplay[] = {N_("No Resolution Check"),
 		N_("Report Resolution Errors"), N_("Show Resolution Errors")};
-	REGISTER void *infstr;
+	REGISTER void *infstr, *dia;
 
-	if (DiaInitDialog(&io_cifoptionsdialog)) return;
+	dia = DiaInitDialog(&io_cifoptionsdialog);
+	if (dia == 0) return;
 	for(i=0; i<3; i++) newlang[i] = _(whattodisplay[i]);
-	DiaSetPopup(DCFO_ORESOLUTIONMODE, 3, newlang);
-	DiaInitTextDialog(DCFO_LAYERLIST, DiaNullDlogList, DiaNullDlogItem,
+	DiaSetPopup(dia, DCFO_ORESOLUTIONMODE, 3, newlang);
+	DiaInitTextDialog(dia, DCFO_LAYERLIST, DiaNullDlogList, DiaNullDlogItem,
 		DiaNullDlogDone, 0, SCSELMOUSE|SCREPORT);
 	layernames = (char **)emalloc(el_curtech->layercount * (sizeof (char *)),
 		el_tempcluster);
@@ -1199,45 +1199,45 @@ void io_cifoptionsdlog(void)
 		addstringtoinfstr(infstr, " (");
 		addstringtoinfstr(infstr, pt);
 		addstringtoinfstr(infstr, ")");
-		DiaStuffLine(DCFO_LAYERLIST, returninfstr(infstr));
+		DiaStuffLine(dia, DCFO_LAYERLIST, returninfstr(infstr));
 	}
-	DiaSelectLine(DCFO_LAYERLIST, 0);
-	DiaSetText(DCFO_NEWLAYER, layernames[0]);
+	DiaSelectLine(dia, DCFO_LAYERLIST, 0);
+	DiaSetText(dia, DCFO_NEWLAYER, layernames[0]);
 	curstate = io_getstatebits();
 	for(i=0; i<NUMIOSTATEBITWORDS; i++) newstate[i] = curstate[i];
-	if ((curstate[0]&CIFOUTEXACT) != 0) DiaSetControl(DCFO_OMIMICDISPLAY, 1);
-	if ((curstate[0]&CIFOUTMERGE) != 0) DiaSetControl(DCFO_OMERGEBOXES, 1);
-	if ((curstate[0]&CIFINSQUARE) != 0) DiaSetControl(DCFO_ISQUAREWIRES, 1);
-	if ((curstate[0]&CIFOUTNOTOPCALL) == 0) DiaSetControl(DCFO_OINSTANTIATETOP, 1);
-	if ((curstate[0]&CIFOUTNORMALIZE) != 0) DiaSetControl(DCFO_ONORMALIZECOORD, 1);
+	if ((curstate[0]&CIFOUTEXACT) != 0) DiaSetControl(dia, DCFO_OMIMICDISPLAY, 1);
+	if ((curstate[0]&CIFOUTMERGE) != 0) DiaSetControl(dia, DCFO_OMERGEBOXES, 1);
+	if ((curstate[0]&CIFINSQUARE) != 0) DiaSetControl(dia, DCFO_ISQUAREWIRES, 1);
+	if ((curstate[0]&CIFOUTNOTOPCALL) == 0) DiaSetControl(dia, DCFO_OINSTANTIATETOP, 1);
+	if ((curstate[0]&CIFOUTNORMALIZE) != 0) DiaSetControl(dia, DCFO_ONORMALIZECOORD, 1);
 	var = getval((INTBIG)el_curtech, VTECHNOLOGY, VINTEGER, "IO_cif_resolution");
 	if (var == NOVARIABLE) oldresolution = 0; else oldresolution = var->addr;
 	if (oldresolution == 0)
 	{
-		DiaSetText(DCFO_ORESOLUTION, "");
-		DiaSetPopupEntry(DCFO_ORESOLUTIONMODE, 0);
-		DiaDimItem(DCFO_ORESOLUTION_L);
-		DiaDimItem(DCFO_ORESOLUTION);
+		DiaSetText(dia, DCFO_ORESOLUTION, "");
+		DiaSetPopupEntry(dia, DCFO_ORESOLUTIONMODE, 0);
+		DiaDimItem(dia, DCFO_ORESOLUTION_L);
+		DiaDimItem(dia, DCFO_ORESOLUTION);
 	} else
 	{
-		DiaUnDimItem(DCFO_ORESOLUTION_L);
-		DiaUnDimItem(DCFO_ORESOLUTION);
-		DiaSetText(DCFO_ORESOLUTION, latoa(oldresolution));
+		DiaUnDimItem(dia, DCFO_ORESOLUTION_L);
+		DiaUnDimItem(dia, DCFO_ORESOLUTION);
+		DiaSetText(dia, DCFO_ORESOLUTION, latoa(oldresolution, 0));
 		if ((curstate[1]&CIFRESHIGH) != 0)
-			DiaSetPopupEntry(DCFO_ORESOLUTIONMODE, 2); else
-				DiaSetPopupEntry(DCFO_ORESOLUTIONMODE, 1);
+			DiaSetPopupEntry(dia, DCFO_ORESOLUTIONMODE, 2); else
+				DiaSetPopupEntry(dia, DCFO_ORESOLUTIONMODE, 1);
 	}
 
 	/* loop until done */
 	nameschanged = 0;
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == CANCEL) break;
 		if (itemHit == OK)
 		{
-			i = DiaGetPopupEntry(DCFO_ORESOLUTIONMODE);
-			if (i != 0 && atola(DiaGetText(DCFO_ORESOLUTION)) == 0)
+			i = DiaGetPopupEntry(dia, DCFO_ORESOLUTIONMODE);
+			if (i != 0 && atola(DiaGetText(dia, DCFO_ORESOLUTION), 0) == 0)
 			{
 				DiaMessageInDialog(_("Must set nonzero output resolution"));
 				continue;
@@ -1248,36 +1248,36 @@ void io_cifoptionsdlog(void)
 			itemHit == DCFO_ISQUAREWIRES || itemHit == DCFO_OINSTANTIATETOP ||
 			itemHit == DCFO_ONORMALIZECOORD)
 		{
-			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
+			DiaSetControl(dia, itemHit, 1 - DiaGetControl(dia, itemHit));
 			continue;
 		}
 		if (itemHit == DCFO_LAYERLIST)
 		{
-			i = DiaGetCurLine(DCFO_LAYERLIST);
-			DiaSetText(DCFO_NEWLAYER, layernames[i]);
+			i = DiaGetCurLine(dia, DCFO_LAYERLIST);
+			DiaSetText(dia, DCFO_NEWLAYER, layernames[i]);
 			continue;
 		}
 		if (itemHit == DCFO_ORESOLUTIONMODE)
 		{
-			i = DiaGetPopupEntry(DCFO_ORESOLUTIONMODE);
+			i = DiaGetPopupEntry(dia, DCFO_ORESOLUTIONMODE);
 			if (i == 0)
 			{
-				DiaDimItem(DCFO_ORESOLUTION_L);
-				DiaDimItem(DCFO_ORESOLUTION);
-				DiaSetText(DCFO_ORESOLUTION, "");
+				DiaDimItem(dia, DCFO_ORESOLUTION_L);
+				DiaDimItem(dia, DCFO_ORESOLUTION);
+				DiaSetText(dia, DCFO_ORESOLUTION, "");
 			} else
 			{
-				DiaUnDimItem(DCFO_ORESOLUTION_L);
-				DiaUnDimItem(DCFO_ORESOLUTION);
-				if (atola(DiaGetText(DCFO_ORESOLUTION)) == 0)
-					DiaSetText(DCFO_ORESOLUTION, "1");
+				DiaUnDimItem(dia, DCFO_ORESOLUTION_L);
+				DiaUnDimItem(dia, DCFO_ORESOLUTION);
+				if (atola(DiaGetText(dia, DCFO_ORESOLUTION), 0) == 0)
+					DiaSetText(dia, DCFO_ORESOLUTION, "1");
 			}
 			continue;
 		}
 		if (itemHit == DCFO_NEWLAYER)
 		{
-			i = DiaGetCurLine(DCFO_LAYERLIST);
-			pt = DiaGetText(DCFO_NEWLAYER);
+			i = DiaGetCurLine(dia, DCFO_LAYERLIST);
+			pt = DiaGetText(dia, DCFO_NEWLAYER);
 			if (strcmp(pt, layernames[i]) == 0) continue;
 			nameschanged++;
 			(void)reallocstring(&layernames[i], pt, el_tempcluster);
@@ -1286,24 +1286,24 @@ void io_cifoptionsdlog(void)
 			addstringtoinfstr(infstr, " (");
 			addstringtoinfstr(infstr, layernames[i]);
 			addstringtoinfstr(infstr, ")");
-			DiaSetScrollLine(DCFO_LAYERLIST, i, returninfstr(infstr));
+			DiaSetScrollLine(dia, DCFO_LAYERLIST, i, returninfstr(infstr));
 			continue;
 		}
 	}
 
 	if (itemHit != CANCEL)
 	{
-		if (DiaGetControl(DCFO_OMIMICDISPLAY) != 0) newstate[0] |= CIFOUTEXACT; else
+		if (DiaGetControl(dia, DCFO_OMIMICDISPLAY) != 0) newstate[0] |= CIFOUTEXACT; else
 			newstate[0] &= ~CIFOUTEXACT;
-		if (DiaGetControl(DCFO_OMERGEBOXES) != 0) newstate[0] |= CIFOUTMERGE; else
+		if (DiaGetControl(dia, DCFO_OMERGEBOXES) != 0) newstate[0] |= CIFOUTMERGE; else
 			newstate[0] &= ~CIFOUTMERGE;
-		if (DiaGetControl(DCFO_ISQUAREWIRES) != 0) newstate[0] |= CIFINSQUARE; else
+		if (DiaGetControl(dia, DCFO_ISQUAREWIRES) != 0) newstate[0] |= CIFINSQUARE; else
 			newstate[0] &= ~CIFINSQUARE;
-		if (DiaGetControl(DCFO_OINSTANTIATETOP) == 0) newstate[0] |= CIFOUTNOTOPCALL; else
+		if (DiaGetControl(dia, DCFO_OINSTANTIATETOP) == 0) newstate[0] |= CIFOUTNOTOPCALL; else
 			newstate[0] &= ~CIFOUTNOTOPCALL;
-		if (DiaGetControl(DCFO_ONORMALIZECOORD) != 0) newstate[0] |= CIFOUTNORMALIZE; else
+		if (DiaGetControl(dia, DCFO_ONORMALIZECOORD) != 0) newstate[0] |= CIFOUTNORMALIZE; else
 			newstate[0] &= ~CIFOUTNORMALIZE;
-		if (DiaGetPopupEntry(DCFO_ORESOLUTIONMODE) == 2) newstate[1] |= CIFRESHIGH; else
+		if (DiaGetPopupEntry(dia, DCFO_ORESOLUTIONMODE) == 2) newstate[1] |= CIFRESHIGH; else
 			newstate[1] &= ~CIFRESHIGH;
 		for(i=0; i<NUMIOSTATEBITWORDS; i++) if (curstate[i] != newstate[i]) break;
 		if (i < NUMIOSTATEBITWORDS) io_setstatebits(newstate);
@@ -1311,13 +1311,11 @@ void io_cifoptionsdlog(void)
 			setval((INTBIG)el_curtech, VTECHNOLOGY, "IO_cif_layer_names",
 				(INTBIG)layernames, VSTRING|VISARRAY|
 					(el_curtech->layercount<<VLENGTHSH));
-		i = atola(DiaGetText(DCFO_ORESOLUTION));
+		i = atola(DiaGetText(dia, DCFO_ORESOLUTION), 0);
 		if (i != oldresolution)
 			setval((INTBIG)el_curtech, VTECHNOLOGY, "IO_cif_resolution", i, VINTEGER);
 	}
 	for(i=0; i<el_curtech->layercount; i++) efree(layernames[i]);
 	efree((char *)layernames);
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 }
-
-#endif  /* IOCIF - at top */
