@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iocifpars.c
@@ -29,8 +30,6 @@
  */
 
 #include "config.h"
-#if IOCIF
-
 #include "global.h"
 #include "eio.h"
 #include "iocifpars.h"
@@ -1967,6 +1966,7 @@ static char    io_ciflinebuffer[LINELEN];	/* identify line buffer */
 static char   *io_ciflineptr;				/* where we are in line buffer */
 static INTBIG  io_cifcharcount;				/* number of chars in buffer */
 static char    io_cifresetbuffer;			/* flag to reset buffer */
+static void   *io_cifprogressdialog;		/* for showing input progress */
 
 static char io_initinput(void)
 {
@@ -1979,22 +1979,21 @@ static char io_initinput(void)
 static char io_doneinput(void)
 {
 	xclose(io_cifinfile);
-	if (io_verbose < 0 && io_ciffilelength > 0) DiaDoneDialog();
+	if (io_verbose < 0 && io_ciffilelength > 0) DiaDoneProgress(io_cifprogressdialog);
 	return(0);
 }
 
 BOOLEAN io_infromfile(FILE *file)
 {
-	extern DIALOG us_progressdialog;
-
 	io_cifinfile = file;
 	if (io_verbose < 0)
 	{
 		io_ciffilelength = filesize(io_cifinfile);
 		if (io_ciffilelength > 0)
 		{
-			if (DiaInitDialog(&us_progressdialog)) return(TRUE);
-			DiaPercent(1, 0);
+			io_cifprogressdialog = DiaInitProgress(0, 0);
+			if (io_cifprogressdialog == 0) return(TRUE);
+			DiaSetProgress(io_cifprogressdialog, 0, io_ciffilelength);
 		}
 	} else io_ciffilelength = 0;
 	io_cifnextchar = xgetc(io_cifinfile);
@@ -2026,7 +2025,7 @@ static char io_getch(void)
 		io_cifnextchar = xgetc(io_cifinfile);
 		io_cifcharsread++;
 		if (io_verbose < 0 && io_ciffilelength > 0 && (io_cifcharsread%2000) == 0)
-			DiaPercent(1, io_cifcharsread*100/io_ciffilelength);
+			DiaSetProgress(io_cifprogressdialog, io_cifcharsread, io_ciffilelength);
 	}
 	return((char)c);
 }
@@ -3040,5 +3039,3 @@ INTBIG io_parsefile(void)
 	}
 	return(comcount);
 }
-
-#endif  /* IOCIF - at top */
