@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iodefi.c
@@ -35,8 +36,6 @@
  */
 
 #include "config.h"
-#if IODEF
-
 #include "global.h"
 #include "egraphics.h"
 #include "efunction.h"
@@ -61,6 +60,7 @@ static char    io_deffilename[300];
 static INTBIG  io_deflinepos;
 static INTBIG  io_defunits;
 static VIADEF *io_deffirstviadef;
+static void   *io_defprogressdialog;
 
 /* prototypes for local routines */
 static char      *io_defgetkeyword(FILE *f);
@@ -108,7 +108,6 @@ BOOLEAN io_readdeflibrary(LIBRARY *lib)
 	FILE *fp;
 	REGISTER BOOLEAN ret;
 	char *filename;
-	extern DIALOG us_progressdialog;
 
 	/* open the file */
 	fp = xopen(lib->libfile, io_filetypedef, "", &filename);
@@ -121,13 +120,13 @@ BOOLEAN io_readdeflibrary(LIBRARY *lib)
 
 	/* prepare for input */
 	io_deffilesize = filesize(fp);
-	if (DiaInitDialog(&us_progressdialog))
+	io_defprogressdialog = DiaInitProgress(_("Reading DEF file..."), 0);
+	if (io_defprogressdialog == 0)
 	{
 		xclose(fp);
 		return(TRUE);
 	}
-	DiaPercent(1, 0);
-	DiaSetText(2, _("Reading DEF file..."));
+	DiaSetProgress(io_defprogressdialog, 0, io_deffilesize);
 	io_deflineno = 0;
 	io_deflinepos = 0;
 	io_defline[0] = 0;
@@ -138,7 +137,7 @@ BOOLEAN io_readdeflibrary(LIBRARY *lib)
 	ret = io_defreadfile(fp, lib);
 
 	/* clean up */
-	DiaDoneDialog();
+	DiaDoneProgress(io_defprogressdialog);
 	xclose(fp);
 	if (!ret) ttyputmsg(_("DEF file %s is read"), lib->libfile); else
 		ttyputmsg(_("Error reading DEF file %s"), lib->libfile);
@@ -745,8 +744,7 @@ BOOLEAN io_defreadnet(FILE *f, NODEPROTO *facet, BOOLEAN special, INTBIG options
 				/* find the export */
 				key = io_defmustgetkeyword(f, "NET");
 				if (key == 0) return(TRUE);
-				for(pp = facet->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-					if (namesame(key, pp->protoname) == 0) break;
+				pp = getportproto(facet, key);
 				if (pp == NOPORTPROTO)
 				{
 					io_definerror(_("Unknown pin '%s'"), key);
@@ -771,8 +769,7 @@ BOOLEAN io_defreadnet(FILE *f, NODEPROTO *facet, BOOLEAN special, INTBIG options
 				/* get the port name */
 				key = io_defmustgetkeyword(f, "NET");
 				if (key == 0) return(TRUE);
-				for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-					if (namesame(key, pp->protoname) == 0) break;
+				pp = getportproto(ni->proto, key);
 				if (pp == NOPORTPROTO)
 				{
 					io_definerror(_("Unknown port '%s' on component '%s'"),
@@ -1426,7 +1423,7 @@ char *io_defgetkeyword(FILE *f)
 		if ((io_deflineno%50) == 0)
 		{
 			filepos = xtell(f);
-			DiaPercent(1, filepos*100/io_deffilesize);
+			DiaSetProgress(io_defprogressdialog, filepos, io_deffilesize);
 		}
 
 		/* look for the first text on the line */
@@ -1468,7 +1465,7 @@ static DIALOGITEM io_defoptionsdialogitems[] =
  /*  3 */ {0, {32,8,48,230}, CHECK, N_("Place logical interconnect")},
  /*  4 */ {0, {8,8,24,230}, CHECK, N_("Place physical interconnect")}
 };
-static DIALOG io_defoptionsdialog = {{50,75,147,314}, N_("DEF Options"), 0, 4, io_defoptionsdialogitems};
+static DIALOG io_defoptionsdialog = {{50,75,147,314}, N_("DEF Options"), 0, 4, io_defoptionsdialogitems, 0, 0};
 
 /* special items for the "DEF Options" dialog: */
 #define DDFO_PLACELOGICAL    3	/* Place logical interconnect (check) */
@@ -1478,35 +1475,35 @@ void io_defoptionsdlog(void)
 {
 	REGISTER INTBIG itemHit, *curstate, i;
 	INTBIG newstate[NUMIOSTATEBITWORDS];
+	REGISTER void *dia;
 
-	if (DiaInitDialog(&io_defoptionsdialog)) return;
+	dia = DiaInitDialog(&io_defoptionsdialog);
+	if (dia == 0) return;
 	curstate = io_getstatebits();
 	for(i=0; i<NUMIOSTATEBITWORDS; i++) newstate[i] = curstate[i];
-	if ((curstate[0]&DEFNOLOGICAL) == 0) DiaSetControl(DDFO_PLACELOGICAL, 1);
-	if ((curstate[0]&DEFNOPHYSICAL) == 0) DiaSetControl(DDFO_PLACEPHYSICAL, 1);
+	if ((curstate[0]&DEFNOLOGICAL) == 0) DiaSetControl(dia, DDFO_PLACELOGICAL, 1);
+	if ((curstate[0]&DEFNOPHYSICAL) == 0) DiaSetControl(dia, DDFO_PLACEPHYSICAL, 1);
 
 	/* loop until done */
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == OK || itemHit == CANCEL) break;
 		if (itemHit == DDFO_PLACELOGICAL || itemHit == DDFO_PLACEPHYSICAL)
 		{
-			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
+			DiaSetControl(dia, itemHit, 1 - DiaGetControl(dia, itemHit));
 			continue;
 		}
 	}
 
 	if (itemHit != CANCEL)
 	{
-		if (DiaGetControl(DDFO_PLACELOGICAL) == 0) newstate[0] |= DEFNOLOGICAL; else
+		if (DiaGetControl(dia, DDFO_PLACELOGICAL) == 0) newstate[0] |= DEFNOLOGICAL; else
 			newstate[0] &= ~DEFNOLOGICAL;
-		if (DiaGetControl(DDFO_PLACEPHYSICAL) == 0) newstate[0] |= DEFNOPHYSICAL; else
+		if (DiaGetControl(dia, DDFO_PLACEPHYSICAL) == 0) newstate[0] |= DEFNOPHYSICAL; else
 			newstate[0] &= ~DEFNOPHYSICAL;
 		for(i=0; i<NUMIOSTATEBITWORDS; i++) if (curstate[i] != newstate[i]) break;
 		if (i < NUMIOSTATEBITWORDS) io_setstatebits(newstate);
 	}
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 }
-
-#endif  /* IODEF - at top */
