@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iodxf.c
@@ -29,8 +30,6 @@
  */
 
 #include "config.h"
-#if IODXF
-
 #include "global.h"
 #include "egraphics.h"
 #include "efunction.h"
@@ -90,6 +89,7 @@ static INTBIG           *io_dxfvalidlayernumbers;
 static void             *io_dxfignoredlayerstringarray = 0;
 static INTBIG            io_dxflayerkey = 0;
 static INTBIG            io_dxfdispunit;
+static void             *io_dxfprogressdialog;
 
 /* prototypes for local routines */
 static INTBIG     io_dxfgetacceptablelayers(void);
@@ -477,7 +477,6 @@ BOOLEAN io_readdxflibrary(LIBRARY *lib)
 	INTBIG len, i, *curstate;
 	NODEPROTO *np;
 	FORWARDREF *fr, *nextfr;
-	extern DIALOG us_progressdialog;
 	NODEINST *ni;
 	REGISTER void *infstr;
 
@@ -504,12 +503,13 @@ BOOLEAN io_readdxflibrary(LIBRARY *lib)
 	io_dxflength = filesize(io);
 	if (io_verbose < 0 && io_dxflength > 0)
 	{
-		if (DiaInitDialog(&us_progressdialog))
+		io_dxfprogressdialog = DiaInitProgress(0, 0);
+		if (io_dxfprogressdialog == 0)
 		{
 			xclose(io);
 			return(TRUE);
 		}
-		DiaPercent(1, 0);
+		DiaSetProgress(io_dxfprogressdialog, 0,  io_dxflength);
 	}
 
 	/* make the only facet in this library */
@@ -597,8 +597,8 @@ BOOLEAN io_readdxflibrary(LIBRARY *lib)
 	xclose(io);
 	if (io_verbose < 0 && io_dxflength > 0)
 	{
-		DiaPercent(1, 100);
-		DiaDoneDialog();
+		DiaSetProgress(io_dxfprogressdialog, io_dxflength, io_dxflength);
+		DiaDoneProgress(io_dxfprogressdialog);
 	}
 
 	/* insert forward references */
@@ -2101,7 +2101,7 @@ INTBIG io_dxfgetnextline(FILE *io, char *text, INTBIG canBeBlank)
 		if ((io_dxfcurline % 100) == 0)
 		{
 			curLength = xtell(io);
-			DiaPercent(1, curLength * 100L / io_dxflength);
+			DiaSetProgress(io_dxfprogressdialog, curLength, io_dxflength);
 		}
 		if (canBeBlank != 0 || *text != 0) break;
 	}
@@ -2243,7 +2243,7 @@ static DIALOGITEM io_dxfoptionsdialogitems[] =
  /*  8 */ {0, {80,240,96,320}, MESSAGE, N_("DXF Scale:")},
  /*  9 */ {0, {80,324,96,430}, POPUP, ""}
 };
-static DIALOG io_dxfoptionsdialog = {{50,75,228,514}, N_("DXF Options"), 0, 9, io_dxfoptionsdialogitems};
+static DIALOG io_dxfoptionsdialog = {{50,75,228,514}, N_("DXF Options"), 0, 9, io_dxfoptionsdialogitems, 0, 0};
 
 /* special items for the "DXF Options" dialog: */
 #define DDXO_LAYERLIST      3		/* Layer list (scroll) */
@@ -2260,11 +2260,12 @@ void io_dxfoptionsdlog(void)
 	char *translated[7];
 	REGISTER VARIABLE *dxfvar, *var;
 	char *dispunits[] = {N_("Inch"), N_("Centimeter"), N_("Millimeter"), N_("Mil"),
-		N_("Micron"), N_("Centimicron"), N_("Millimicron")};
-	REGISTER void *infstr;
+		N_("Micron"), N_("Centimicron"), N_("Nanometer")};
+	REGISTER void *infstr, *dia;
 
-	if (DiaInitDialog(&io_dxfoptionsdialog)) return;
-	DiaInitTextDialog(DDXO_LAYERLIST, DiaNullDlogList, DiaNullDlogItem,
+	dia = DiaInitDialog(&io_dxfoptionsdialog);
+	if (dia == 0) return;
+	DiaInitTextDialog(dia, DDXO_LAYERLIST, DiaNullDlogList, DiaNullDlogItem,
 		DiaNullDlogDone, 0, SCSELMOUSE|SCREPORT);
 	layernames = (char **)emalloc(el_curtech->layercount * (sizeof (char *)),
 		el_tempcluster);
@@ -2280,37 +2281,37 @@ void io_dxfoptionsdlog(void)
 		addstringtoinfstr(infstr, " (");
 		addstringtoinfstr(infstr, pt);
 		addstringtoinfstr(infstr, ")");
-		DiaStuffLine(DDXO_LAYERLIST, returninfstr(infstr));
+		DiaStuffLine(dia, DDXO_LAYERLIST, returninfstr(infstr));
 	}
-	DiaSelectLine(DDXO_LAYERLIST, 0);
-	DiaSetText(DDXO_NEWLAYER, layernames[0]);
+	DiaSelectLine(dia, DDXO_LAYERLIST, 0);
+	DiaSetText(dia, DDXO_NEWLAYER, layernames[0]);
 	curstate = io_getstatebits();
 	for(i=0; i<NUMIOSTATEBITWORDS; i++) newstate[i] = curstate[i];
-	if ((curstate[0]&DXFFLATTENINPUT) != 0) DiaSetControl(DDXO_FLATTENHIER, 1);
-	if ((curstate[0]&DXFALLLAYERS) != 0) DiaSetControl(DDXO_READALLLAYERS, 1);
+	if ((curstate[0]&DXFFLATTENINPUT) != 0) DiaSetControl(dia, DDXO_FLATTENHIER, 1);
+	if ((curstate[0]&DXFALLLAYERS) != 0) DiaSetControl(dia, DDXO_READALLLAYERS, 1);
 	for(i=0; i<7; i++) translated[i] = _(dispunits[i]);
-	DiaSetPopup(DDXO_DXFUNITS, 7, translated); 
+	DiaSetPopup(dia, DDXO_DXFUNITS, 7, translated); 
 	var = getval((INTBIG)io_tool, VTOOL, VINTEGER, "IO_dxf_units");
 	if (var == NOVARIABLE) units = 2; else
 		units = var->addr;
-	DiaSetPopupEntry(DDXO_DXFUNITS, units);
+	DiaSetPopupEntry(dia, DDXO_DXFUNITS, units);
 
 	/* loop until done */
 	nameschanged = 0;
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == OK || itemHit == CANCEL) break;
 		if (itemHit == DDXO_LAYERLIST)
 		{
-			i = DiaGetCurLine(DDXO_LAYERLIST);
-			DiaSetText(DDXO_NEWLAYER, layernames[i]);
+			i = DiaGetCurLine(dia, DDXO_LAYERLIST);
+			DiaSetText(dia, DDXO_NEWLAYER, layernames[i]);
 			continue;
 		}
 		if (itemHit == DDXO_NEWLAYER)
 		{
-			i = DiaGetCurLine(DDXO_LAYERLIST);
-			pt = DiaGetText(DDXO_NEWLAYER);
+			i = DiaGetCurLine(dia, DDXO_LAYERLIST);
+			pt = DiaGetText(dia, DDXO_NEWLAYER);
 			if (strcmp(pt, layernames[i]) == 0) continue;
 			nameschanged++;
 			(void)reallocstring(&layernames[i], pt, el_tempcluster);
@@ -2319,12 +2320,12 @@ void io_dxfoptionsdlog(void)
 			addstringtoinfstr(infstr, " (");
 			addstringtoinfstr(infstr, layernames[i]);
 			addstringtoinfstr(infstr, ")");
-			DiaSetScrollLine(DDXO_LAYERLIST, i, returninfstr(infstr));
+			DiaSetScrollLine(dia, DDXO_LAYERLIST, i, returninfstr(infstr));
 			continue;
 		}
 		if (itemHit == DDXO_FLATTENHIER || itemHit == DDXO_READALLLAYERS)
 		{
-			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
+			DiaSetControl(dia, itemHit, 1 - DiaGetControl(dia, itemHit));
 			continue;
 		}
 	}
@@ -2335,19 +2336,17 @@ void io_dxfoptionsdlog(void)
 			setval((INTBIG)el_curtech, VTECHNOLOGY, "IO_dxf_layer_names",
 				(INTBIG)layernames, VSTRING|VISARRAY|
 					(el_curtech->layercount<<VLENGTHSH));
-		if (DiaGetControl(DDXO_FLATTENHIER) != 0) newstate[0] |= DXFFLATTENINPUT; else
+		if (DiaGetControl(dia, DDXO_FLATTENHIER) != 0) newstate[0] |= DXFFLATTENINPUT; else
 			newstate[0] &= ~DXFFLATTENINPUT;
-		if (DiaGetControl(DDXO_READALLLAYERS) != 0) newstate[0] |= DXFALLLAYERS; else
+		if (DiaGetControl(dia, DDXO_READALLLAYERS) != 0) newstate[0] |= DXFALLLAYERS; else
 			newstate[0] &= ~DXFALLLAYERS;
 		for(i=0; i<NUMIOSTATEBITWORDS; i++) if (curstate[i] != newstate[i]) break;
 		if (i < NUMIOSTATEBITWORDS) io_setstatebits(newstate);
-		i = DiaGetPopupEntry(DDXO_DXFUNITS);
+		i = DiaGetPopupEntry(dia, DDXO_DXFUNITS);
 		if (i != units)
 			setval((INTBIG)io_tool, VTOOL, "IO_dxf_units", i, VINTEGER);
 	}
 	for(i=0; i<el_curtech->layercount; i++) efree(layernames[i]);
 	efree((char *)layernames);
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 }
-
-#endif  /* IODXF - at top */
