@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: ioeagleo.c
@@ -45,8 +46,6 @@
  */
 
 #include "config.h"
-#if IOEAGLE
-
 #include "global.h"
 #include "eio.h"
 
@@ -374,5 +373,3 @@ void io_eaglemarkfacets(NODEPROTO *curfacet)
 		np->temp1 = 1;
 	}
 }
-
-#endif  /* IOEAGLE - at top */
