@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: ioecado.c
@@ -60,8 +61,6 @@ Net_name : ref_des pad; \
 */
 
 #include "config.h"
-#if IOECAD
-
 #include "global.h"
 #include "eio.h"
 
@@ -163,10 +162,10 @@ BOOLEAN io_writeecadlibrary(LIBRARY *lib)
 		if (lasten == NOECADNET || namesame(lasten->netname, en->netname) != 0)
 		   {
 			if (lasten != NOECADNET)
-            xprintf(io_fileout, ";\n");
+			xprintf(io_fileout, ";\n");
 			xprintf(io_fileout, "%s : ", en->netname);            /* cm - write netname on first line of net */
 		   }
-      else
+		else
 		   {
 			xprintf(io_fileout, "      ");                           /* cm - was net name on each line */
 		   }
@@ -391,5 +390,3 @@ void io_ecadmarkfacets(NODEPROTO *curfacet)
 		np->temp1 = 1;
 	}
 }
-
-#endif  /* IOECAD - at top */
