@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: ioedifo.c
@@ -30,8 +31,6 @@
  */
 
 #include "config.h"
-#if IOEDIF
-
 #include "global.h"
 #include "efunction.h"
 #include "eio.h"
@@ -166,7 +165,7 @@ BOOLEAN io_writeediflibrary(LIBRARY *lib)
 	if ((net_tool->toolstate & TOOLON) == 0)
 	{
 		ttyputerr(_("Network tool must be running...turning it on"));
-		toolturnon(net_tool, FALSE);
+		toolturnon(net_tool);
 		ttyputerr(_("...now reissue the EDIF I/O command"));
 		return(TRUE);
 	}
@@ -1025,8 +1024,8 @@ BOOLEAN io_edifwritefacet(NODEPROTO *np, INTBIG external)
 									(((pp->userbits & STATEBITS) == INPORT) ? _("input") : _("vbias")),
 										pp->protoname, describenodeinst(ni),
 											describenodeproto(ni->parent));
-							else if (onet->signals < pp->network->signals)
-								for (i = onet->signals; i != pp->network->signals; i++)
+							else if (onet->buswidth < pp->network->buswidth)
+								for (i = onet->buswidth; i != pp->network->buswidth; i++)
 									ttyputmsg(_("** WARNING - no connection to %s port %s (signal %ld) on %s in %s"),
 										((pp->userbits & STATEBITS) == INPORT ? _("input") : _("vbias")),
 							 				pp->network->networklist[i]->netname, i,
@@ -1041,7 +1040,7 @@ BOOLEAN io_edifwritefacet(NODEPROTO *np, INTBIG external)
 				for (net = np->firstnetwork; net != NONETWORK; net = net->nextnetwork)
 				{
 					/* skip bus networks altogether (they are done wire by wire) */
-					if (net->signals > 1)
+					if (net->buswidth > 1)
 					{
 						/* handle bus description, note that most nets have single arc
 						description which is handled below */
@@ -1071,7 +1070,7 @@ BOOLEAN io_edifwritefacet(NODEPROTO *np, INTBIG external)
 						EO_open_block(edif_stream, "listOfNets");
 
 						/* now each sub-net name */
-						for (i = 0; i < net->signals; i++)
+						for (i = 0; i < net->buswidth; i++)
 						{
 							EO_open_block(edif_stream, "net");
 
@@ -1277,7 +1276,7 @@ BOOLEAN io_edifwritefacet(NODEPROTO *np, INTBIG external)
 						/* match up exported net with bus connections on this nodeinst */
 						for (pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 						{
-							if ((netcount = pi->conarcinst->network->signals) > 1)
+							if ((netcount = pi->conarcinst->network->buswidth) > 1)
 							{
 								/* first, find the connection point if there is one */
 								for (i = 0; i < netcount; i++)
@@ -1292,9 +1291,9 @@ BOOLEAN io_edifwritefacet(NODEPROTO *np, INTBIG external)
 
 								/* associate by the i-th position in the connection */
 								EO_open_block(edif_stream, "portRef");
-								if (cpp->network->signals > i)
+								if (cpp->network->buswidth > i)
 								{
-									if (cpp->network->signals > 1)
+									if (cpp->network->buswidth > 1)
 										cnet = cpp->network->networklist[i]; else
 											cnet = cpp->network;
 
@@ -1320,7 +1319,7 @@ BOOLEAN io_edifwritefacet(NODEPROTO *np, INTBIG external)
 						/* continue with connected busses */
 						for (pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
 						{
-							if ((netcount = pe->exportproto->network->signals) > 1)
+							if ((netcount = pe->exportproto->network->buswidth) > 1)
 							{
 								/* first, find the connection point if there is one */
 								for (i = 0; i < netcount; i++)
@@ -1332,9 +1331,9 @@ BOOLEAN io_edifwritefacet(NODEPROTO *np, INTBIG external)
 
 								/* associate by the i-th position in the connection */
 								EO_open_block(edif_stream, "portRef");
-								if (cpp->network->signals > i)
+								if (cpp->network->buswidth > i)
 								{
-									if (cpp->network->signals > 1)
+									if (cpp->network->buswidth > 1)
 										EO_put_identifier(edif_stream,
 											io_ediftoken(cpp->network->networklist[i]->netname));
 									else EO_put_identifier(edif_stream,
@@ -1473,7 +1472,7 @@ INTBIG io_edifmarknetports(NODEPROTO *np)
 	for (pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 	{
 		/* mark the nets that were exported with a pointer to their first port */
-		if ((count = pp->network->signals) > 1)
+		if ((count = pp->network->buswidth) > 1)
 			for (i = 0; i < count; i++)
 		{
 			if (pp->network->networklist[i]->temp2 == (INTBIG)NOPORTPROTO)
@@ -2658,7 +2657,7 @@ INTBIG EO_free_stream(EO_STREAM_PTR stream)
 char *EO_get_timestamp(char *buffer)
 {
 	static char get_timestamp_buf[81];
-	UINTBIG t;
+	time_t t;
 	char month[4];
 	unsigned short mon, yr, day, hour, min, sec;
 
@@ -2743,7 +2742,7 @@ static DIALOGITEM io_edifoptionsdialogitems[] =
  /*  4 */ {0, {36,8,52,96}, MESSAGE, N_("Input scale:")},
  /*  5 */ {0, {36,100,52,188}, EDITTEXT, ""}
 };
-static DIALOG io_edifoptionsdialog = {{50,75,151,284}, N_("EDIF Options"), 0, 5, io_edifoptionsdialogitems};
+static DIALOG io_edifoptionsdialog = {{50,75,151,284}, N_("EDIF Options"), 0, 5, io_edifoptionsdialogitems, 0, 0};
 
 /* special items for the "EDIF Options" dialog: */
 #define DEDO_USESCHEMATIC  3		/* Use Schematic View (check) */
@@ -2756,41 +2755,41 @@ void io_edifoptionsdlog(void)
 	REGISTER VARIABLE *var;
 	char line[50];
 	float inputscale, newscale;
+	void *dia;
 
-	if (DiaInitDialog(&io_edifoptionsdialog)) return;
+	dia = DiaInitDialog(&io_edifoptionsdialog);
+	if (dia == 0) return;
 	curstate = io_getstatebits();
 	for(i=0; i<NUMIOSTATEBITWORDS; i++) newstate[i] = curstate[i];
-	if ((curstate[0]&EDIFSCHEMATIC) != 0) DiaSetControl(DEDO_USESCHEMATIC, 1);
+	if ((curstate[0]&EDIFSCHEMATIC) != 0) DiaSetControl(dia, DEDO_USESCHEMATIC, 1);
 	var = getval((INTBIG)io_tool, VTOOL, VFLOAT, "IO_edif_input_scale");
 	if (var == NOVARIABLE) inputscale = 1.0; else
 		inputscale = castfloat(var->addr);
 	sprintf(line, "%g", inputscale);
-	DiaSetText(DEDO_INPUTSCALE, line);
+	DiaSetText(dia, DEDO_INPUTSCALE, line);
 
 	/* loop until done */
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == OK || itemHit == CANCEL) break;
 		if (itemHit == DEDO_USESCHEMATIC)
 		{
-			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
+			DiaSetControl(dia, itemHit, 1 - DiaGetControl(dia, itemHit));
 			continue;
 		}
 	}
 
 	if (itemHit != CANCEL)
 	{
-		if (DiaGetControl(DEDO_USESCHEMATIC) != 0) newstate[0] |= EDIFSCHEMATIC; else
+		if (DiaGetControl(dia, DEDO_USESCHEMATIC) != 0) newstate[0] |= EDIFSCHEMATIC; else
 			newstate[0] &= ~EDIFSCHEMATIC;
 		for(i=0; i<NUMIOSTATEBITWORDS; i++) if (curstate[i] != newstate[i]) break;
 		if (i < NUMIOSTATEBITWORDS) io_setstatebits(newstate);
-		newscale = (float)atof(DiaGetText(DEDO_INPUTSCALE));
+		newscale = (float)atof(DiaGetText(dia, DEDO_INPUTSCALE));
 		if (newscale != inputscale)
 			setval((INTBIG)io_tool, VTOOL, "IO_edif_input_scale", castint(newscale), VFLOAT);
 	}
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 }
 
-#endif  /* IOEDIF - at top */
-
