@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iogdsi.c
@@ -59,8 +60,6 @@
  */
 
 #include "config.h"
-#if IOGDS
-
 #include "global.h"
 #include "egraphics.h"
 #include "efunction.h"
@@ -74,7 +73,6 @@
 #include <math.h>
 
 #define NEWPATH 1		/* path and facet caching optimizations */
-/* #define PLACEFACETCENTERS  1 */	/* uncomment to place facet centers */
 
 /* electric specific globals */
 static LIBRARY        *io_gdslibrary = NOLIBRARY;
@@ -95,6 +93,7 @@ static INTBIG          io_gdslayerwhich;
 static INTBIG          io_gdssrefcount;
 static INTBIG          io_gdsrecomputed;
 static INTBIG         *io_gdscurstate;
+static void           *io_gdsprogressdialog;
 
 /* globals */
 static STREAM          io_gdsstream;
@@ -151,7 +150,21 @@ static char *io_gdsIerror_string[] =
 	N_("Array reference has no parameters"),				/* 27 */
 	N_("Array reference name is missing"),					/* 28 */
 	N_("Property has no value"),							/* 29 */
-	N_("Array orientation oblique")							/* 30 */
+	N_("Array orientation oblique"),						/* 30 */
+	N_("Failed to create AREF"),							/* 31 */
+	N_("Failed to create RECTANGLE"),						/* 32 */
+	N_("Failed to create POLYGON"),							/* 33 */
+	N_("No memory"),										/* 34 */
+	N_("Failed to create POLYGON points"),					/* 35 */
+	N_("Could not create text marker"),						/* 36 */
+	N_("Could not create text string"),						/* 37 */
+	N_("Failed to create structure"),						/* 38 */
+	N_("Failed to create facet center"),					/* 39 */
+	N_("Failed to create NODE"),							/* 40 */
+	N_("Failed to create box"),								/* 41 */
+	N_("Failed to create outline"),							/* 42 */
+	N_("Failed to create SREF proto"),						/* 43 */
+	N_("Failed to create SREF")								/* 44 */
 };
 
 static gsymbol io_gdsIoption_set[] =
@@ -360,7 +373,7 @@ unsigned char io_gdsIgetrawbyte(void)
 		if (io_gdsbytesread/5000 != (io_gdsbytesread-BLOCKSIZE)/5000)
 		{
 			if (io_verbose < 0 && io_gdsfilesize > 0)
-			DiaPercent(1, io_gdsbytesread*100/io_gdsfilesize); else
+			DiaSetProgress(io_gdsprogressdialog, io_gdsbytesread, io_gdsfilesize); else
 				ttyputmsg(_("%ld bytes read (%ld%% done)"), io_gdsbytesread,
 					io_gdsbytesread*100/io_gdsfilesize);
 		}
@@ -607,9 +620,9 @@ gsymbol io_gdsIgetrecord(datatype_symbol *gds_type)
 
 void io_gdsIerror(INTBIG n)
 {
-	ttyputerr(_("Error #%ld: at block number %ld index = %ld\n"), n,
+	ttyputerr(_("Error: %s, block %ld, index %ld"), _(io_gdsIerror_string[n]),
 		io_gdsblockcount, io_gdsbufferindex);
-	longjmp(io_gdsenv, (INTBIG)_(io_gdsIerror_string[n]));
+	longjmp(io_gdsenv, 1);
 }
 
 void io_gdsIgettoken(void)
@@ -722,7 +735,7 @@ void io_gdsIstep(INTBIG nc, INTBIG nr, INTBIG angle, INTBIG trans)
 					pt.px + io_gdssref->highx + ox, pt.py + io_gdssref->lowy + oy,
 						pt.py + io_gdssref->highy + oy, trans, angle,
 							io_gdsstructure)) == NONODEINST)
-								longjmp(io_gdsenv, (INTBIG)_("failed to create AREF"));
+								io_gdsIerror(31);
 				if ((io_gdscurstate[0] & GDSINEXPAND) != 0)
 					io_gdsinstance->userbits |= NEXPAND;
 				/* annotate the array info to this element */
@@ -811,7 +824,7 @@ void io_gdsIshape(INTBIG npts, shape_type perimeter, shape_type oclass)
 				if (newnodeinst(io_gdslayernode, io_gdsvertex[0].px,
 					io_gdsvertex[1].px, io_gdsvertex[0].py, io_gdsvertex[1].py, 0, 0,
 						io_gdsstructure) == NONODEINST)
-							longjmp(io_gdsenv, (INTBIG)_("failed to create RECTANGLE"));
+							io_gdsIerror(32);
 			}
 			break;
 
@@ -835,11 +848,11 @@ void io_gdsIshape(INTBIG npts, shape_type perimeter, shape_type oclass)
 			/* now create the node */
 			if ((ni = newnodeinst(io_gdslayernode, lx, hx, ly, hy, 0, 0,
 				io_gdsstructure)) == NONODEINST)
-					longjmp(io_gdsenv, (INTBIG)_("failed to create POLYGON"));
+					io_gdsIerror(33);
 
 			/* now allocate the trace */
 			pt = trace = emalloc((npts*2*SIZEOFINTBIG), el_tempcluster);
-			if (trace == 0) longjmp(io_gdsenv, (INTBIG)_("no memory"));
+			if (trace == 0) io_gdsIerror(34);
 			cx = (hx + lx) / 2;   cy = (hy + ly) / 2;
 			for (i=0; i<npts; i++)
 			{
@@ -850,7 +863,7 @@ void io_gdsIshape(INTBIG npts, shape_type perimeter, shape_type oclass)
 			/* store the trace information */
 			if (setvalkey((INTBIG)ni, VNODEINST, el_trace_key, (INTBIG)trace,
 				VINTEGER|VISARRAY|((npts*2)<<VLENGTHSH)) == NOVARIABLE)
-					longjmp(io_gdsenv, (INTBIG)_("failed to create POLYGON points"));
+					io_gdsIerror(35);
 
 			/* free the polygon memory */
 			efree((char *)trace);
@@ -878,12 +891,12 @@ void io_gdsItext(char *charstring, INTBIG vjust, INTBIG hjust, INTBIG angle,
 	if ((ni = newnodeinst(io_gdslayernode, io_gdsvertex[0].px, io_gdsvertex[0].px,
 		io_gdsvertex[0].py, io_gdsvertex[0].py, trans, angle,
 			io_gdsstructure)) == NONODEINST)
-				longjmp(io_gdsenv, (INTBIG)_("Could not create text marker"));
+				io_gdsIerror(36);
 
 	/* now add the text */
 	if ((var = setvalkey((INTBIG)ni, VNODEINST, el_node_name_key, (INTBIG)charstring,
 		VSTRING|VDISPLAY)) == NOVARIABLE)
-			longjmp(io_gdsenv, (INTBIG)_("Could not create text string"));
+			io_gdsIerror(37);
 
 	/* set the text size and orientation */
 	size = rounddouble(scale);
@@ -1302,6 +1315,7 @@ void io_gdsIbeginstructure(void)
 {
 	char create_time[IDSTRING+1], mod_time[IDSTRING+1];
 	REGISTER void *infstr;
+	REGISTER NODEINST *ni;
 
 	if (io_gdstoken == GDS_BGNSTR)
 	{
@@ -1320,27 +1334,24 @@ void io_gdsIbeginstructure(void)
 					/* create the proto */
 					io_gdsstructure = us_newnodeproto(io_gdstokenstring, io_gdslibrary);
 					if (io_gdsstructure == NONODEPROTO)
-						longjmp(io_gdsenv, (INTBIG)_("failed to create structure"));
+						io_gdsIerror(38);
 					if (io_verbose < 0 && io_gdsfilesize > 0)
 					{
 						infstr = initinfstr();
 						formatinfstr(infstr, _("Reading %s"), io_gdstokenstring);
-						DiaSetText(2, returninfstr(infstr));
+						DiaSetTextProgress(io_gdsprogressdialog, returninfstr(infstr));
 					} else if (io_verbose != 0) ttyputmsg(_("Reading %s"), io_gdstokenstring);
 
-#ifdef PLACEFACETCENTERS
+					if ((us_useroptions&FACETCENTERALWAYS) != 0)
 					{
-						REGISTER NODEINST *ni;
-
 						/* add the facet center */
 						ni = newnodeinst(gen_facetcenterprim, gen_facetcenterprim->lowx,
 							gen_facetcenterprim->highx, gen_facetcenterprim->lowy,
 							gen_facetcenterprim->highy, 0, 0, io_gdsstructure);
 						if (ni == NONODEINST)
-							longjmp(io_gdsenv, (INTBIG)_("failed to create facet center"));
+							io_gdsIerror(39);
 						ni->userbits |= HARDSELECTN|NVISIBLEINSIDE;
 					}
-#endif
 					if (io_gdslibrary->curnodeproto == NONODEPROTO)
 						io_gdslibrary->curnodeproto = io_gdsstructure;
 				}
@@ -1437,7 +1448,7 @@ void io_gdsIdetermine_node(void)
 		if (newnodeinst(io_gdslayernode, io_gdsvertex[0].px,
 			io_gdsvertex[0].px, io_gdsvertex[0].py, io_gdsvertex[0].py, 0, 0,
 				io_gdsstructure) == NONODEINST)
-					longjmp(io_gdsenv, (INTBIG) _("failed to create NODE"));
+					io_gdsIerror(40);
 	} else
 	{
 		io_gdsIonlyone(io_gdsIrecovery_set, GDS_ENDEL);
@@ -1465,7 +1476,7 @@ void io_gdsIdetermine_box(void)
 			if (newnodeinst(io_gdslayernode, io_gdsvertex[0].px, io_gdsvertex[1].px,
 				io_gdsvertex[0].py, io_gdsvertex[1].py , 0, 0,
 					io_gdsstructure) == NONODEINST)
-						longjmp(io_gdsenv, (INTBIG) _("failed to create box"));
+						io_gdsIerror(41);
 		}
 	} else
 	{
@@ -1614,13 +1625,13 @@ void io_gdsIdetermine_path(void)
 				{
 					if (newnodeinst(io_gdslayernode, lx, hx, ly, hy, 0, 0,
 						io_gdsstructure) == NONODEINST)
-							longjmp(io_gdsenv, (INTBIG)_("failed to create outline"));
+							io_gdsIerror(42);
 				} else
 				{
 					getbbox(poly, &lx, &hx, &ly, &hy);
 					ni = newnodeinst(io_gdslayernode, lx, hx, ly, hy, 0, 0, io_gdsstructure);
 					if (ni == NONODEINST)
-						longjmp(io_gdsenv, (INTBIG)_("failed to create outline"));
+						io_gdsIerror(42);
 
 					/* set the shape into the node */
 					cx = (lx + hx) / 2;
@@ -1646,6 +1657,7 @@ void io_gdsIgetproto(char *name)
 {
 	NODEPROTO *np;
 	REGISTER void *infstr;
+	NODEINST *ni;
 
 	/* scan for this proto */
 	np = io_gdsIfindproto(name);
@@ -1653,27 +1665,24 @@ void io_gdsIgetproto(char *name)
 	{
 		/* FILO order, create this nodeproto */
 		if ((np = us_newnodeproto(io_gdstokenstring, io_gdslibrary)) == NONODEPROTO)
-			longjmp(io_gdsenv, (INTBIG)_("failed to create SREF proto"));
+			io_gdsIerror(43);
 		if (io_verbose < 0 && io_gdsfilesize > 0)
 		{
 			infstr = initinfstr();
 			formatinfstr(infstr, _("Reading %s"), io_gdstokenstring);
-			DiaSetText(2, returninfstr(infstr));
+			DiaSetTextProgress(io_gdsprogressdialog, returninfstr(infstr));
 		} else if (io_verbose != 0) ttyputmsg(_("Reading %s"), io_gdstokenstring);
 
-#ifdef PLACEFACETCENTERS
+		if ((us_useroptions&FACETCENTERALWAYS) != 0)
 		{
-			NODEINST *ni;
-
 			/* add the facet center */
 			ni = newnodeinst(gen_facetcenterprim, gen_facetcenterprim->lowx,
 				gen_facetcenterprim->highx, gen_facetcenterprim->lowy,
 					gen_facetcenterprim->highy, 0, 0, np);
 			if (ni == NONODEINST)
-				longjmp(io_gdsenv, (INTBIG)_("failed to create facet center"));
+				io_gdsIerror(39);
 			ni->userbits |= HARDSELECTN|NVISIBLEINSIDE;
 		}
-#endif
 	}
 
 	/* set the reference node proto */
@@ -1728,7 +1737,7 @@ void io_gdsIdetermine_sref(void)
 			io_gdsvertex[0].py + io_gdssref->lowy + oy,
 			io_gdsvertex[0].py + io_gdssref->highy + oy,
 			trans, angle, io_gdsstructure)) == NONODEINST)
-				longjmp(io_gdsenv, (INTBIG)_("failed to create SREF"));
+				io_gdsIerror(44);
 		if (io_gdscurstate[0]&GDSINEXPAND)
 			io_gdsinstance->userbits |= NEXPAND;
 		io_gdssrefcount++;
@@ -1970,9 +1979,8 @@ BOOLEAN io_readgdslibrary(LIBRARY *lib, INTBIG position)
 	BOOLEAN err;
 	INTBIG i;
 	INTBIG count;
-	char *msg, *filename;
+	char *filename;
 	NODEPROTO *np;
-	extern DIALOG us_progressdialog;
 
 	io_gdslibrary = lib;
 
@@ -1990,21 +1998,21 @@ BOOLEAN io_readgdslibrary(LIBRARY *lib, INTBIG position)
 	io_gdsfilesize = filesize(input);
 	if (io_verbose < 0 && io_gdsfilesize > 0)
 	{
-		if (DiaInitDialog(&us_progressdialog))
+		io_gdsprogressdialog = DiaInitProgress(0, 0);
+		if (io_gdsprogressdialog == 0)
 		{
 			xclose(input);
 			return(TRUE);
 		}
-		DiaPercent(1, 0);
+		DiaSetProgress(io_gdsprogressdialog, 0, io_gdsfilesize);
 	}
 	io_gdsbytesread = 0;
 
 	/* setup the error return location */
-	if ((msg = (char *)setjmp(io_gdsenv)) != NULL)
+	if (setjmp(io_gdsenv) != 0)
 	{
-		ttyputerr(_("Error: %s"), msg);
 		xclose(input);
-		if (io_verbose < 0 && io_gdsfilesize > 0) DiaDoneDialog();
+		if (io_verbose < 0 && io_gdsfilesize > 0) DiaDoneProgress(io_gdsprogressdialog);
 		return(TRUE);
 	}
 
@@ -2053,8 +2061,8 @@ BOOLEAN io_readgdslibrary(LIBRARY *lib, INTBIG position)
 	{
 		if (io_verbose < 0 && io_gdsfilesize > 0)
 		{
-			DiaSetText(2, _("Cleaning up..."));
-			DiaPercent(1, 0);
+			DiaSetTextProgress(io_gdsprogressdialog, _("Cleaning up..."));
+			DiaSetProgress(io_gdsprogressdialog, 0, io_gdssrefcount);
 		}
 
 		/* determine which facets need to be recomputed */
@@ -2069,7 +2077,7 @@ BOOLEAN io_readgdslibrary(LIBRARY *lib, INTBIG position)
 				io_gdsIrecompute(np, count);
 
 	}
-	if (io_verbose < 0 && io_gdsfilesize > 0) DiaDoneDialog();
+	if (io_verbose < 0 && io_gdsfilesize > 0) DiaDoneProgress(io_gdsprogressdialog);
 	return(err);
 }
 
@@ -2120,14 +2128,12 @@ void io_gdsIrecompute(NODEPROTO *np, INTBIG count)
 		if ((io_gdsrecomputed%1000) == 0 && io_verbose < 0 && io_gdsfilesize > 0)
 		{
 			(void)sprintf(numsofar, _("Cleaned up %ld instances..."), io_gdsrecomputed);
-			DiaSetText(2, numsofar);
+			DiaSetTextProgress(io_gdsprogressdialog, numsofar);
 			if (count > 0)
-				DiaPercent(1, (io_gdsrecomputed*100)/count);
+				DiaSetProgress(io_gdsprogressdialog, io_gdsrecomputed, count);
 		}
 	}
 
 	db_boundfacet(np, &np->lowx, &np->highx, &np->lowy, &np->highy);
 	np->temp1 = 1;
 }
-
-#endif  /* IOGDS - at top */
