@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iogdso.c
@@ -29,8 +30,6 @@
  */
 
 #include "config.h"
-#if IOGDS
-
 #include "global.h"
 #include "eio.h"
 #include "egraphics.h"
@@ -40,7 +39,7 @@
 #include "tecart.h"
 #include "edialogs.h"
 
-#define OLDGDS 1
+/* #define OLDGDS 1 */ 
 
 #define GDSVERSION            3
 #define BYTEMASK           0xFF
@@ -113,7 +112,7 @@ static void io_gds_initialize(void);
 static void io_gds_closefile(void);
 static void io_gds_bgnlib(NODEPROTO*);
 static void io_gds_bgnstr(NODEPROTO*);
-static void io_gds_date(UINTBIG*);
+static void io_gds_date(time_t*);
 static void io_gds_header(INTBIG, INTBIG);
 static void io_gds_name(INTBIG, char*, INTBIG);
 static void io_gds_angle(INTBIG);
@@ -557,7 +556,7 @@ void io_outputgdspoly(TECHNOLOGY *tech, POLYGON *poly, INTBIG angle, INTBIG offx
 	static POLYGON *poly2 = NOPOLYGON;
 	REGISTER void *infstr;
 
-	(void)needstaticpolygon(&poly, 4, io_tool->cluster);
+	(void)needstaticpolygon(&poly2, 4, io_tool->cluster);
 
 	/* determine the layer name for this polygon */
 	if (layervar != NOVARIABLE) laynum = (INTBIG)layervar->addr; else
@@ -733,8 +732,8 @@ void io_gds_bgnlib(NODEPROTO *np)
 
 	io_gds_header(HDR_HEADER, GDSVERSION);
 	io_gds_header(HDR_BGNLIB, 0);
-	io_gds_date(&np->creationdate);
-	io_gds_date(&np->revisiondate);
+	io_gds_date((time_t *)&np->creationdate);
+	io_gds_date((time_t *)&np->revisiondate);
 	(void)strcpy(name, np->cell->cellname);
 	io_gds_name(HDR_LIBNAME, io_gds_makename(name), HDR_M_ASCII);
 	io_gds_int2(HDR_N_UNITS);
@@ -744,13 +743,13 @@ void io_gds_bgnlib(NODEPROTO *np)
 
 void io_gds_bgnstr(NODEPROTO *np)
 {
-	UINTBIG cdate, rdate;
+	time_t cdate, rdate;
 	char name[NAMSIZE];
 
 	io_gds_header(HDR_BGNSTR, 0);
-	cdate = np->creationdate;
+	cdate = (time_t)np->creationdate;
 	io_gds_date(&cdate);
-	rdate = np->revisiondate;
+	rdate = (time_t)np->revisiondate;
 	io_gds_date(&rdate);
 	(void)strcpy(name, np->cell->cellname);
 	io_gds_name(HDR_STRNAME, io_gds_makename(name), HDR_M_STRNAME);
@@ -759,7 +758,7 @@ void io_gds_bgnstr(NODEPROTO *np)
 /* Utilities to put the data into the output file */
 
 /* Output date array, converted from a date integer */
-void io_gds_date(UINTBIG *val)
+void io_gds_date(time_t *val)
 {
 	char buffer[28], save;
 	INTSML date[6];
@@ -1097,7 +1096,7 @@ static DIALOGITEM io_gdsoptionsdialogitems[] =
  /* 14 */ {0, {200,396,216,464}, EDITTEXT, ""},
  /* 15 */ {0, {104,240,120,464}, CHECK, N_("Input Ignores Unknown Layers")}
 };
-static DIALOG io_gdsoptionsdialog = {{50,75,307,548}, N_("GDS Options"), 0, 15, io_gdsoptionsdialogitems};
+static DIALOG io_gdsoptionsdialog = {{50,75,307,548}, N_("GDS Options"), 0, 15, io_gdsoptionsdialogitems, 0, 0};
 
 /* special items for the "GDS Options" dialog: */
 #define DGDO_LAYERLIST       3		/* Layer list (scroll list) */
@@ -1116,10 +1115,11 @@ void io_gdsoptionsdlog(void)
 	INTBIG arcres, arcsag, newstate[NUMIOSTATEBITWORDS];
 	char buf[50];
 	REGISTER VARIABLE *gdsvar;
-	REGISTER void *infstr;
+	REGISTER void *infstr, *dia;
 
-	if (DiaInitDialog(&io_gdsoptionsdialog)) return;
-	DiaInitTextDialog(DGDO_LAYERLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0,
+	dia = DiaInitDialog(&io_gdsoptionsdialog);
+	if (dia == 0) return;
+	DiaInitTextDialog(dia, DGDO_LAYERLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0,
 		SCSELMOUSE|SCREPORT);
 	layernumbers = (INTBIG *)emalloc(el_curtech->layercount * SIZEOFINTBIG, el_tempcluster);
 	gdsvar = getval((INTBIG)el_curtech, VTECHNOLOGY, VINTEGER|VISARRAY,
@@ -1133,46 +1133,46 @@ void io_gdsoptionsdlog(void)
 		addstringtoinfstr(infstr, layername(el_curtech, i));
 		sprintf(buf, " (%ld)", val);
 		addstringtoinfstr(infstr, buf);
-		DiaStuffLine(DGDO_LAYERLIST, returninfstr(infstr));
+		DiaStuffLine(dia, DGDO_LAYERLIST, returninfstr(infstr));
 	}
-	DiaSelectLine(DGDO_LAYERLIST, 0);
+	DiaSelectLine(dia, DGDO_LAYERLIST, 0);
 	sprintf(buf, "%ld", layernumbers[0]);
-	DiaSetText(DGDO_NEWLAYER, buf);
+	DiaSetText(dia, DGDO_NEWLAYER, buf);
 	curstate = io_getstatebits();
 	for(i=0; i<NUMIOSTATEBITWORDS; i++) newstate[i] = curstate[i];
-	if ((curstate[0]&GDSINTEXT) != 0) DiaSetControl(DGDO_IINCLUDETEXT, 1);
-	if ((curstate[0]&GDSINEXPAND) != 0) DiaSetControl(DGDO_IEXPANDFACET, 1);
-	if ((curstate[0]&GDSINARRAYS) != 0) DiaSetControl(DGDO_IINSTARRAY, 1);
-	if ((curstate[0]&GDSOUTMERGE) != 0) DiaSetControl(DGDO_OMERGEBOX, 1);
-	if ((curstate[0]&GDSINIGNOREUKN) != 0) DiaSetControl(DGDO_IIGNOREUNKNOWN, 1);
+	if ((curstate[0]&GDSINTEXT) != 0) DiaSetControl(dia, DGDO_IINCLUDETEXT, 1);
+	if ((curstate[0]&GDSINEXPAND) != 0) DiaSetControl(dia, DGDO_IEXPANDFACET, 1);
+	if ((curstate[0]&GDSINARRAYS) != 0) DiaSetControl(dia, DGDO_IINSTARRAY, 1);
+	if ((curstate[0]&GDSOUTMERGE) != 0) DiaSetControl(dia, DGDO_OMERGEBOX, 1);
+	if ((curstate[0]&GDSINIGNOREUKN) != 0) DiaSetControl(dia, DGDO_IIGNOREUNKNOWN, 1);
 	getcontoursegmentparameters(&arcres, &arcsag);
-	DiaSetText(DGDO_OARCANGLE, frtoa(arcres*WHOLE/10));
-	DiaSetText(DGDO_OARCSAG, latoa(arcsag));
+	DiaSetText(dia, DGDO_OARCANGLE, frtoa(arcres*WHOLE/10));
+	DiaSetText(dia, DGDO_OARCSAG, latoa(arcsag, 0));
 
 	/* loop until done */
 	numberschanged = 0;
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == OK || itemHit == CANCEL) break;
 		if (itemHit == DGDO_IINCLUDETEXT || itemHit == DGDO_IEXPANDFACET ||
 			itemHit == DGDO_IINSTARRAY || itemHit == DGDO_OMERGEBOX ||
 			itemHit == DGDO_IIGNOREUNKNOWN)
 		{
-			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
+			DiaSetControl(dia, itemHit, 1 - DiaGetControl(dia, itemHit));
 			continue;
 		}
 		if (itemHit == DGDO_LAYERLIST)
 		{
-			i = DiaGetCurLine(DGDO_LAYERLIST);
+			i = DiaGetCurLine(dia, DGDO_LAYERLIST);
 			sprintf(buf, "%ld", layernumbers[i]);
-			DiaSetText(DGDO_NEWLAYER, buf);
+			DiaSetText(dia, DGDO_NEWLAYER, buf);
 			continue;
 		}
 		if (itemHit == DGDO_NEWLAYER)
 		{
-			i = DiaGetCurLine(DGDO_LAYERLIST);
-			val = atoi(DiaGetText(DGDO_NEWLAYER));
+			i = DiaGetCurLine(dia, DGDO_LAYERLIST);
+			val = atoi(DiaGetText(dia, DGDO_NEWLAYER));
 			if (val == layernumbers[i]) continue;
 			numberschanged++;
 			layernumbers[i] = val;
@@ -1180,22 +1180,22 @@ void io_gdsoptionsdlog(void)
 			addstringtoinfstr(infstr, layername(el_curtech, i));
 			sprintf(buf, " (%ld)", val);
 			addstringtoinfstr(infstr, buf);
-			DiaSetScrollLine(DGDO_LAYERLIST, i, returninfstr(infstr));
+			DiaSetScrollLine(dia, DGDO_LAYERLIST, i, returninfstr(infstr));
 			continue;
 		}
 	}
 
 	if (itemHit != CANCEL)
 	{
-		if (DiaGetControl(DGDO_IINCLUDETEXT) != 0) newstate[0] |= GDSINTEXT; else
+		if (DiaGetControl(dia, DGDO_IINCLUDETEXT) != 0) newstate[0] |= GDSINTEXT; else
 			newstate[0] &= ~GDSINTEXT;
-		if (DiaGetControl(DGDO_IEXPANDFACET) != 0) newstate[0] |= GDSINEXPAND; else
+		if (DiaGetControl(dia, DGDO_IEXPANDFACET) != 0) newstate[0] |= GDSINEXPAND; else
 			newstate[0] &= ~GDSINEXPAND;
-		if (DiaGetControl(DGDO_IINSTARRAY) != 0) newstate[0] |= GDSINARRAYS; else
+		if (DiaGetControl(dia, DGDO_IINSTARRAY) != 0) newstate[0] |= GDSINARRAYS; else
 			newstate[0] &= ~GDSINARRAYS;
-		if (DiaGetControl(DGDO_OMERGEBOX) != 0) newstate[0] |= GDSOUTMERGE; else
+		if (DiaGetControl(dia, DGDO_OMERGEBOX) != 0) newstate[0] |= GDSOUTMERGE; else
 			newstate[0] &= ~GDSOUTMERGE;
-		if (DiaGetControl(DGDO_IIGNOREUNKNOWN) != 0) newstate[0] |= GDSINIGNOREUKN; else
+		if (DiaGetControl(dia, DGDO_IIGNOREUNKNOWN) != 0) newstate[0] |= GDSINIGNOREUKN; else
 			newstate[0] &= ~GDSINIGNOREUKN;
 		for(i=0; i<NUMIOSTATEBITWORDS; i++) if (curstate[i] != newstate[i]) break;
 		if (i < NUMIOSTATEBITWORDS) io_setstatebits(newstate);
@@ -1203,13 +1203,11 @@ void io_gdsoptionsdlog(void)
 			setval((INTBIG)el_curtech, VTECHNOLOGY, "IO_gds_layer_numbers",
 				(INTBIG)layernumbers, VINTEGER|VISARRAY|
 					(el_curtech->layercount<<VLENGTHSH));
-		newres = atofr(DiaGetText(DGDO_OARCANGLE)) * 10 / WHOLE;
-		newsag = atola(DiaGetText(DGDO_OARCSAG));
+		newres = atofr(DiaGetText(dia, DGDO_OARCANGLE)) * 10 / WHOLE;
+		newsag = atola(DiaGetText(dia, DGDO_OARCSAG), 0);
 		if (newres != arcres || newsag != arcsag)
 			setcontoursegmentparameters(newres, newsag);
 	}
 	efree((char *)layernumbers);
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 }
-
-#endif  /* IOGDS - at top */
