@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iohpglout.c
@@ -546,7 +547,7 @@ void io_hptext(INTBIG type, INTBIG xl, INTBIG xh, INTBIG yl, INTBIG yh, INTBIG s
  *		4 - cross-hatched lines at the specified angle
  *
  *	fill distance - The distance between lines used in fill types 3 and 4 above,
- *		measured in half-millimicrons.
+ *		measured in half-nanometers.
  *
  *	fill angle - The angle of the lines used in fill types 3 and 4 above.
  *
