@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iolefi.c
@@ -35,8 +36,6 @@
  */
 
 #include "config.h"
-#if IOLEF
-
 #include "global.h"
 #include "egraphics.h"
 #include "efunction.h"
@@ -71,6 +70,7 @@ static INTBIG  io_leflineno;
 static char    io_lefline[MAXLINE];
 static INTBIG  io_leflinepos;
        VIADEF *io_leffirstviadef = NOVIADEF;
+static void   *io_lefprogressdialog;
 
 /* prototypes for local routines */
 static void       io_leffreepaths(LEFPATH *firstlp);
@@ -104,7 +104,6 @@ BOOLEAN io_readleflibrary(LIBRARY *lib)
 	FILE *fp;
 	REGISTER BOOLEAN ret;
 	char *filename;
-	extern DIALOG us_progressdialog;
 
 	/* open the file */
 	fp = xopen(lib->libfile, io_filetypelef, "", &filename);
@@ -119,13 +118,13 @@ BOOLEAN io_readleflibrary(LIBRARY *lib)
 
 	/* prepare for input */
 	io_leffilesize = filesize(fp);
-	if (DiaInitDialog(&us_progressdialog))
+	io_lefprogressdialog = DiaInitProgress(_("Reading LEF file..."), 0);
+	if (io_lefprogressdialog == 0)
 	{
 		xclose(fp);
 		return(TRUE);
 	}
-	DiaPercent(1, 0);
-	DiaSetText(2, _("Reading LEF file..."));
+	DiaSetProgress(io_lefprogressdialog, 0, io_leffilesize);
 	io_leflineno = 0;
 	io_leflinepos = 0;
 	io_lefline[0] = 0;
@@ -134,7 +133,7 @@ BOOLEAN io_readleflibrary(LIBRARY *lib)
 	ret = io_lefreadfile(fp, lib);
 
 	/* clean up */
-	DiaDoneDialog();
+	DiaDoneProgress(io_lefprogressdialog);
 	xclose(fp);
 	if (!ret) ttyputmsg(_("LEF file %s is read"), lib->libfile); else
 		ttyputmsg(_("Error reading LEF file %s"), lib->libfile);
@@ -1053,8 +1052,7 @@ PORTPROTO *io_lefnewport(NODEPROTO *facet, NODEINST *ni, PORTPROTO *pp, char *th
 	newname = 0;
 	for(i=0; ; i++)
 	{
-		for(ppt = facet->firstportproto; ppt != NOPORTPROTO; ppt = ppt->nextportproto)
-			if (namesame(portname, ppt->protoname) == 0) break;
+		ppt = getportproto(facet, portname);
 		if (ppt == NOPORTPROTO)
 		{
 			ppt = newportproto(facet, ni, pp, portname);
@@ -1204,7 +1202,7 @@ char *io_lefgetkeyword(FILE *f)
 		if ((io_leflineno%50) == 0)
 		{
 			filepos = xtell(f);
-			DiaPercent(1, filepos*100/io_leffilesize);
+			DiaSetProgress(io_lefprogressdialog, filepos, io_leffilesize);
 		}
 
 		/* remove comments from the line */
@@ -1296,5 +1294,3 @@ void io_leffreepaths(LEFPATH *firstlp)
 		io_lefkillpath(lp);
 	}
 }
-
-#endif  /* IOLEF - at top */
