@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iolefo.c
@@ -35,8 +36,6 @@
  */
 
 #include "config.h"
-#if IOLEF
-
 #include "global.h"
 #include "eio.h"
 #include "usr.h"
@@ -342,5 +341,3 @@ char *io_lefoutlayername(TECHNOLOGY *tech, INTBIG layer)
 	}
 	return("");
 }
-
-#endif  /* IOLEF - at top */
