@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iopadso.c
@@ -62,8 +63,6 @@ Net_name : ref_des pad; \
 */
 
 #include "config.h"
-#if IOPADS
-
 #include "global.h"
 #include "eio.h"
 
@@ -389,5 +388,3 @@ void io_padsmarkfacets(NODEPROTO *curfacet)
 		np->temp1 = 1;
 	}
 }
-
-#endif  /* IOPADS - at top */
