@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iopsout.c
@@ -83,31 +84,33 @@ static INTBIG      io_pspolygoncount;		/* number of polygons in job so far */
 static INTBIG      io_pspolygontotal;		/* number of polygons in job */
 static INTBIG      io_psopcount;			/* chunks of code */
 static char        io_psprefix[10];			/* indentation for PostScript */
+static void       *io_psprogressdialog;		/* for showing output progress */
 
 static GRAPHICS io_psblack = {LAYERO, BLACK, SOLIDC, SOLIDC,
 	{0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF}, NOVARIABLE, 0};
 
 /* prototypes for local routines */
-static void   io_psarc(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
-static void   io_pscircle(INTBIG, INTBIG, INTBIG, INTBIG);
-static void   io_psdisc(INTBIG, INTBIG, INTBIG, INTBIG, GRAPHICS*);
-static void   io_psdot(INTBIG, INTBIG);
-static void   io_psdumpcontents(NODEPROTO *np);
-static void   io_psdumpfacets(NODEPROTO *np);
-static char  *io_psfacetname(NODEPROTO *np);
-static void   io_psline(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
-static char   io_pspattern(GRAPHICS*);
-static void   io_pspoly(POLYGON*, WINDOWPART*);
-static void   io_pspolycount(POLYGON*, WINDOWPART*);
-static void   io_pspolygon(INTBIG*, INTBIG*, INTBIG, GRAPHICS*);
-static void   io_psputheader(INTBIG which);
-static void   io_pssendfiletoprinter(char *file);
-static void   io_pstext(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, char*, UINTBIG*);
-static INTBIG io_pstruefontsize(INTBIG font, WINDOWPART *w, TECHNOLOGY *tech);
-static void   io_pswrite(char *s, ...);
-static BOOLEAN io_pswritefacet(NODEPROTO*, char*, INTBIG);
-static void   io_pswritestring(char*);
-static void   io_psxform(INTBIG, INTBIG, INTBIG*, INTBIG*);
+static void    io_psarc(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
+static void    io_pscircle(INTBIG, INTBIG, INTBIG, INTBIG);
+static void    io_psdisc(INTBIG, INTBIG, INTBIG, INTBIG, GRAPHICS*);
+static void    io_psdot(INTBIG, INTBIG);
+static void    io_psdumpcontents(NODEPROTO *np);
+static void    io_psdumpfacets(NODEPROTO *np);
+static char   *io_psfacetname(NODEPROTO *np);
+static void    io_psline(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
+static char    io_pspattern(GRAPHICS*);
+static void    io_pspoly(POLYGON*, WINDOWPART*);
+static void    io_pspolycount(POLYGON*, WINDOWPART*);
+static void    io_pspolygon(INTBIG*, INTBIG*, INTBIG, GRAPHICS*);
+static void    io_psputheader(INTBIG which);
+static void    io_pssendfiletoprinter(char *file);
+static void    io_pstext(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, char*, UINTBIG*);
+static INTBIG  io_pstruefontsize(INTBIG font, WINDOWPART *w, TECHNOLOGY *tech);
+static void    io_pswrite(char *s, ...);
+static void    io_pswriteexplorer(WINDOWPART *win, BOOLEAN printit);
+static BOOLEAN io_pswritefacet(NODEPROTO*, char*, BOOLEAN);
+static void    io_pswritestring(char*);
+static void    io_psxform(INTBIG, INTBIG, INTBIG*, INTBIG*);
 
 /*
  * Routine to free all memory associated with this module.
@@ -119,9 +122,9 @@ void io_freepostscriptmemory(void)
 
 /*
  * Routine to write out a PostScript library.
- * Actually prints the document if "printit" is nonzero.
+ * Actually prints the document if "printit" is TRUE.
  */
-BOOLEAN io_writepostscriptlibrary(LIBRARY *lib, INTBIG printit)
+BOOLEAN io_writepostscriptlibrary(LIBRARY *lib, BOOLEAN printit)
 {
 	char *par[MAXPARS];
 	REGISTER INTBIG i, numsyncs, synchother, filestatus;
@@ -133,7 +136,7 @@ BOOLEAN io_writepostscriptlibrary(LIBRARY *lib, INTBIG printit)
 
 	/* see if there are synchronization links */
 	synchother = 0;
-	if (printit == 0)
+	if (!printit)
 	{
 		for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
 			for(np = olib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
@@ -153,6 +156,11 @@ BOOLEAN io_writepostscriptlibrary(LIBRARY *lib, INTBIG printit)
 	np = lib->curnodeproto;
 	if (np == NONODEPROTO && synchother == 0)
 	{
+		if ((el_curwindowpart->state&WINDOWTYPE) == EXPLORERWINDOW)
+		{
+			io_pswriteexplorer(el_curwindowpart, printit);
+			return(TRUE);
+		}
 		ttyputerr(_("No current facet to plot"));
 		return(TRUE);
 	}
@@ -184,7 +192,7 @@ BOOLEAN io_writepostscriptlibrary(LIBRARY *lib, INTBIG printit)
 				if (filestatus == 1 || filestatus == 3) continue;
 			}
 		}
-		if (io_pswritefacet(np, (char *)var->addr, 0)) return(TRUE);
+		if (io_pswritefacet(np, (char *)var->addr, FALSE)) return(TRUE);
 		numsyncs++;
 
 		/* this is tricky: because the "setvalkey" modifies the facet, the date must be set by hand */
@@ -201,10 +209,153 @@ BOOLEAN io_writepostscriptlibrary(LIBRARY *lib, INTBIG printit)
 }
 
 /*
- * Routine to write the facet "np" to a PostScript file.  If "synchronize" is nonzero,
- * synchronize the facet with that file.  If "printit" is nonzero, queue this for print.
+ * Routine to write the explorer window in "win".  It is automatically printed if "printit" is TRUE.
  */
-BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
+void io_pswriteexplorer(WINDOWPART *win, BOOLEAN printit)
+{
+	char file[100], *truename, *name, *header;
+	REGISTER void *infstr;
+	REGISTER INTBIG topmargin, ypos, page, lastboxxpos, boxsize, dashindent, xpos,
+		fontHeight, pagewid, pagehei, yposbox, leftmargin, i, l, topypos;
+	INTBIG indent, style;
+	REGISTER VARIABLE *var;
+	BOOLEAN *done;
+
+	/* an explorer window: print text */
+	us_explorerreadoutinit(win);
+
+	if (printit)
+	{
+		strcpy(file, "/tmp/ElectricPSOut.XXXXXX");
+		emaketemp(file);
+	} else
+	{
+		strcpy(file, "Explorer.ps");
+	}
+	io_psout = xcreate(file, io_filetypeps, _("PostScript File"), &truename);
+	if (io_psout == NULL)
+	{
+		ttyputerr(_("Cannot write temporary file %s"), file);
+		return;
+	}
+
+	/* print the header */
+	io_pswrite("%%!PS-Adobe-1.0\n");
+	io_pswrite("%%%%Title: Facet Explorer\n");
+	io_pswrite("/Times-Roman findfont 10 scalefont setfont\n");
+	io_pswrite("/Drawline {\n");
+	io_pswrite("    newpath moveto lineto stroke} def\n");
+
+	var = getval((INTBIG)io_tool, VTOOL, VFRACT, "IO_postscript_width");
+	if (var == NOVARIABLE) pagewid = DEFAULTPSWIDTH; else
+		pagewid = muldiv(var->addr, 75, WHOLE);
+	var = getval((INTBIG)io_tool, VTOOL, VFRACT, "IO_postscript_height");
+	if (var == NOVARIABLE) pagehei = DEFAULTPSHEIGHT; else
+		pagehei = muldiv(var->addr, 75, WHOLE);
+	var = getval((INTBIG)io_tool, VTOOL, VFRACT, "IO_postscript_margin");
+	if (var == NOVARIABLE)
+	{
+		leftmargin = DEFAULTPSMARGIN;
+	} else
+	{
+		leftmargin = muldiv(var->addr, 75, WHOLE);
+	}
+
+	fontHeight = 15;
+	topmargin = fontHeight*6;
+	ypos = pagehei - topmargin;
+	page = 1;
+	lastboxxpos = -1;
+	boxsize = fontHeight * 2 / 3;
+	dashindent = boxsize / 5;
+
+	for(;;)
+	{
+		name = us_explorerreadoutnext(win, &indent, &style, &done, 0);
+		if (name == 0) break;
+
+		xpos = leftmargin + indent * fontHeight;
+		yposbox = ypos + (fontHeight-boxsize) / 2;
+
+		/* insert page breaks */
+		if (ypos < topmargin)
+		{
+			ypos = pagehei - topmargin;
+			io_pswrite("showpage\n");
+		}
+
+		/* print the header */
+		if (ypos == pagehei - topmargin)
+		{
+			infstr = initinfstr();
+			formatinfstr(infstr, _("Facet Explorer    Page %ld"), page);
+			header = returninfstr(infstr);
+
+			io_pswrite("%ld %ld moveto ", pagewid/3, yposbox+fontHeight*2);
+			io_pswritestring(header);
+			io_pswrite(" show\n");
+			page++;
+		}
+
+		/* draw the box */
+		io_pswrite("%ld %ld %ld %ld Drawline\n", xpos,         yposbox,         xpos+boxsize, yposbox);
+		io_pswrite("%ld %ld %ld %ld Drawline\n", xpos+boxsize, yposbox,         xpos+boxsize, yposbox+boxsize);
+		io_pswrite("%ld %ld %ld %ld Drawline\n", xpos+boxsize, yposbox+boxsize, xpos,         yposbox+boxsize);
+		io_pswrite("%ld %ld %ld %ld Drawline\n", xpos,         yposbox+boxsize, xpos,         yposbox);
+		if (style >= 2)
+		{
+			/* draw horizontal line in box */
+			io_pswrite("%ld %ld %ld %ld Drawline\n", xpos+dashindent, yposbox+boxsize/2,
+				xpos+boxsize-dashindent, yposbox+boxsize/2);
+			if (style == 2)
+			{
+				/* draw vertical line in box */
+				io_pswrite("%ld %ld %ld %ld Drawline\n", xpos+boxsize/2, yposbox+dashindent,
+					xpos+boxsize/2, yposbox+boxsize-dashindent);
+			}
+		}
+
+		/* draw the connecting lines (dotted) */
+		if (indent > 0)
+		{
+			io_pswrite("[1 2] 0 setdash\n");
+			io_pswrite("%ld %ld %ld %ld Drawline\n", xpos, yposbox+boxsize/2,
+				xpos-fontHeight+boxsize/2, yposbox+boxsize/2);
+			for(i=0; i<indent; i++)
+			{
+				if (done[i]) continue;
+				l = leftmargin+i*fontHeight+boxsize/2;
+				if (l == lastboxxpos)
+					topypos = yposbox+boxsize/2+fontHeight-boxsize/2; else
+						topypos = yposbox+boxsize/2+fontHeight;
+				io_pswrite("%ld %ld %ld %ld Drawline\n", l, yposbox+boxsize/2, l, topypos);
+			}
+			io_pswrite("[] 0 setdash\n");
+		}
+		lastboxxpos = xpos + boxsize/2;
+
+		/* show the text */
+		io_pswrite("%ld %ld moveto ", xpos+boxsize+boxsize/2, yposbox);
+		io_pswritestring(name);
+		io_pswrite(" show\n");
+		ypos -= fontHeight;
+	}
+	io_pswrite("showpage\n");
+	xclose(io_psout);
+	if (printit)
+	{
+		io_pssendfiletoprinter(truename);
+	} else
+	{
+		ttyputmsg(_("%s written"), truename);
+	}
+}
+
+/*
+ * Routine to write the facet "np" to a PostScript file.  If "synchronize" is nonzero,
+ * synchronize the facet with that file.  If "printit" is TRUE, queue this for print.
+ */
+BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, BOOLEAN printit)
 {
 	char file[100], *truename, *ptr;
 	INTBIG lx, hx, ly, hy, gridlx, gridly, gridx, gridy,
@@ -214,19 +365,18 @@ BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	REGISTER BOOLEAN *layarr;
 	REGISTER BOOLEAN plotdates, rotateplot, useplotter, hierpostscript, epsformat;
 	INTBIG bblx, bbhx, bbly, bbhy, t1, t2, unitsx, unitsy;
-	UINTBIG curdate;
+	time_t curdate;
 	REGISTER NODEPROTO *onp;
 	REGISTER LIBRARY *lib;
 	VARIABLE *var;
 	REGISTER VARIABLE *varred, *vargreen, *varblue;
 	static POLYGON *poly = NOPOLYGON;
-	extern DIALOG us_progressdialog;
 	REGISTER void *infstr;
 
 	/* cannot write text-only facets */
 	if ((np->cellview->viewstate&TEXTVIEW) != 0)
 	{
-		if (printit == 0)
+		if (!printit)
 		{
 			ttyputerr(_("Cannot write textual facets"));
 			return(TRUE);
@@ -252,9 +402,9 @@ BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 		{
 			infstr = initinfstr();
 			if (np->creationdate != 0)
-				formatinfstr(infstr, _("Created: %s"), timetostring(np->creationdate));
+				formatinfstr(infstr, _("Created: %s"), timetostring((time_t)np->creationdate));
 			if (np->revisiondate != 0)
-				formatinfstr(infstr, _("   Revised: %s"), timetostring(np->revisiondate));
+				formatinfstr(infstr, _("   Revised: %s"), timetostring((time_t)np->revisiondate));
 			xprintf(io_psout,  returninfstr(infstr));
 		}
 		xprintf(io_psout, "\n\n\n");
@@ -297,9 +447,8 @@ BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	if ((curstate[0]&EPSPSCRIPT) != 0) epsformat = TRUE;
 	if ((curstate[0]&PSHIERARCHICAL) != 0) hierpostscript = TRUE;
 	if ((curstate[0]&PSPLOTTER) != 0) useplotter = TRUE;
-	if ((curstate[0]&PSROTATE) != 0) rotateplot = TRUE;
 	if ((curstate[0]&PLOTDATES) != 0) plotdates = TRUE;
-	if (printit != 0) epsformat = FALSE;
+	if (printit) epsformat = FALSE;
 
 	var = getval((INTBIG)io_tool, VTOOL, VFRACT, "IO_postscript_width");
 	if (var == NOVARIABLE) pagewid = DEFAULTPSWIDTH; else
@@ -337,6 +486,15 @@ BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	/* determine the area of interest */
 	if (io_getareatoprint(np, &lx, &hx, &ly, &hy, FALSE)) return(TRUE);
 
+	if ((curstate[0]&PSROTATE) != 0) rotateplot = TRUE; else
+	{
+		if ((curstate[1]&PSAUTOROTATE) != 0)
+		{
+			if (((pagehei > pagewid || useplotter) && hx-lx > hy-ly) ||
+				(pagewid > pagehei && hy-ly > hx-lx)) rotateplot = TRUE;
+		}
+	}
+
 	/* if plotting, compute height from width */
 	if (useplotter)
 	{
@@ -350,7 +508,7 @@ BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	}
 
 	/* create the PostScript file */
-	if (printit != 0)
+	if (printit)
 	{
 		strcpy(file, "/tmp/ElectricPSOut.XXXXXX");
 		emaketemp(file);
@@ -383,9 +541,9 @@ BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 
 	if (io_verbose < 0)
 	{
-		if (DiaInitDialog(&us_progressdialog)) return(TRUE);
-		DiaPercent(1, 0);
-		DiaSetText(2, _("Preparing PostScript Output..."));
+		io_psprogressdialog = DiaInitProgress(_("Preparing PostScript Output..."), 0);
+		if (io_psprogressdialog == 0) return(TRUE);
+		DiaSetProgress(io_psprogressdialog, 0, 1);
 	}
 
 	/* build pseudowindow for text scaling */
@@ -594,7 +752,7 @@ BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 			(void)asktool(us_tool, "display-to-routine", io_pspolycount);
 			io_pspolygontotal = io_pspolygoncount;
 			if (io_pspolygontotal == 0) io_pspolygontotal = 1;
-			DiaSetText(2, _("Writing PostScript..."));
+			DiaSetTextProgress(io_psprogressdialog, _("Writing PostScript..."));
 		}
 		io_pspolygoncount = 0;
 
@@ -661,7 +819,7 @@ BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 				onp->temp1 = 0;
 
 		if (io_verbose < 0)
-			DiaSetText(2, _("Writing PostScript..."));
+			DiaSetTextProgress(io_psprogressdialog, _("Writing PostScript..."));
 		io_pspolygontotal = 0;
 
 #ifdef DEBUGHIERPS
@@ -790,14 +948,14 @@ BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 		io_pswrite(" %ld Botleftstring\n", CORNERDATESIZE * PSSCALE);
 
 		infstr = initinfstr();
-		formatinfstr(infstr, _("Created: %s"), timetostring(np->creationdate));
+		formatinfstr(infstr, _("Created: %s"), timetostring((time_t)np->creationdate));
 		io_pswrite("%s%ld %ld ", io_psprefix, io_pswindow.uselx,
 			io_pswindow.usely + CORNERDATESIZE * PSSCALE);
 		io_pswritestring(returninfstr(infstr));
 		io_pswrite(" %ld Botleftstring\n", CORNERDATESIZE * PSSCALE);
 
 		infstr = initinfstr();
-		formatinfstr(infstr, _("Revised: %s"), timetostring(np->revisiondate));
+		formatinfstr(infstr, _("Revised: %s"), timetostring((time_t)np->revisiondate));
 		io_pswrite("%s%ld %ld ", io_psprefix, io_pswindow.uselx, io_pswindow.usely);
 		io_pswritestring(returninfstr(infstr));
 		io_pswrite(" %ld Botleftstring\n", CORNERDATESIZE * PSSCALE);
@@ -810,7 +968,7 @@ BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	io_pswrite("%%%%Trailer\n");
 	xclose(io_psout);
 	if (io_verbose < 0)
-		DiaDoneDialog();
+		DiaDoneProgress(io_psprogressdialog);
 
 	/* restore window */
 	el_curwindowpart->uselx = sulx;
@@ -822,11 +980,13 @@ BOOLEAN io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	computewindowscale(el_curwindowpart);
 
 	/* print the PostScript if requested */
-	if (printit != 0)
+	if (printit)
 	{
 		io_pssendfiletoprinter(file);
 	} else
+	{
 		ttyputmsg(_("%s written"), truename);
+	}
 	return(FALSE);
 }
 
@@ -1140,7 +1300,7 @@ void io_pspoly(POLYGON *poly, WINDOWPART *win)
 	if (io_pspolygontotal > 0 && (io_pspolygoncount % 100) == 99)
 	{
 		if (io_verbose < 0)
-			DiaPercent(1, io_pspolygoncount*100/io_pspolygontotal);
+			DiaSetProgress(io_psprogressdialog, io_pspolygoncount, io_pspolygontotal);
 	}
 
 	/* ignore null layers */
