@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: ioquickdraw.c
@@ -98,6 +99,11 @@ BOOLEAN io_writequickdrawlibrary(LIBRARY *lib)
 	np = lib->curnodeproto;
 	if (np == NONODEPROTO)
 	{
+		if ((el_curwindowpart->state&WINDOWTYPE) == EXPLORERWINDOW)
+		{
+			ttyputmsg(_("Cannot convert Explorer window to Quickdraw.  Export PostScript instead"));
+			return(TRUE);
+		}
 		ttyputerr(_("No current facet to plot"));
 		return(TRUE);
 	}
@@ -343,14 +349,14 @@ BOOLEAN io_writequickdrawlibrary(LIBRARY *lib)
 
 		/* plot creation date */
 		infstr = initinfstr();
-		formatinfstr(infstr, _("Created: %s"), timetostring(np->creationdate));
+		formatinfstr(infstr, _("Created: %s"), timetostring((time_t)np->creationdate));
 		poly->string = returninfstr(infstr);
 		poly->yv[0] = np->lowy + (np->highy-np->lowy) / 20;
 		(void)io_plotpoly(poly, el_curwindowpart);
 
 		/* plot revision date */
 		infstr = initinfstr();
-		formatinfstr(infstr, _("Revised: %s"), timetostring(np->revisiondate));
+		formatinfstr(infstr, _("Revised: %s"), timetostring((time_t)np->revisiondate));
 		poly->string = returninfstr(infstr);
 		poly->yv[0] = np->lowy + (np->highy-np->lowy) / 10;
 		(void)io_plotpoly(poly, el_curwindowpart);
