@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iosdfi.c
@@ -29,8 +30,6 @@
  */
 
 #include "config.h"
-#if IOSDF
-
 #include "global.h"
 #include "egraphics.h"
 #include "efunction.h"
@@ -93,7 +92,7 @@ static BOOLEAN    sdfi_addparameter(LISPTREE *tree, INTBIG type, void *value);
 static LISPTREE  *sdfi_alloclisptree(void);
 static void       sdfi_killptree(LISPTREE *lt);
 static BOOLEAN    sdfi_pushkeyword(char *pt);
-static LISPTREE  *sdfi_readfile(FILE *fp);
+static LISPTREE  *sdfi_readfile(FILE *fp, void *dia);
 static BOOLEAN    sdfi_parsetree(LISPTREE *lt);
 static BOOLEAN    sdfi_parsedelayfile(LISPTREE *lt);
 static BOOLEAN    sdfi_parsecell(LISPTREE *lt);
@@ -131,8 +130,8 @@ BOOLEAN io_readsdflibrary(LIBRARY *lib)
 {
 	FILE *fp;
 	char *filename;
-	extern DIALOG us_progressdialog;
 	REGISTER LISPTREE *lt;
+	void *dia;
 
 	/* make sure there is a current facet */
 	sdfi_curfacet = getcurfacet();
@@ -160,17 +159,17 @@ BOOLEAN io_readsdflibrary(LIBRARY *lib)
 
 	/* prepare for input */
 	sdfi_filesize = filesize(fp);
-	if (DiaInitDialog(&us_progressdialog))
+	dia = DiaInitProgress(_("Reading SDF file..."), 0);
+	if (dia == 0)
 	{
 		xclose(fp);
 		return(TRUE);
 	}
-	DiaPercent(1, 0);
-	DiaSetText(2, _("Reading SDF file..."));
+	DiaSetProgress(dia, 0, sdfi_filesize);
 
 	/* read the file */
-	lt = sdfi_readfile(fp);
-	DiaDoneDialog();
+	lt = sdfi_readfile(fp, dia);
+	DiaDoneProgress(dia);
 	xclose(fp);
 	if (lt == 0)
 	{
@@ -234,7 +233,7 @@ BOOLEAN io_readsdflibrary(LIBRARY *lib)
  * Routine to read the FPGA file in "f" and create a LISPTREE structure which is returned.
  * Returns zero on error.
  */
-LISPTREE *sdfi_readfile(FILE *f)
+LISPTREE *sdfi_readfile(FILE *f, void *dia)
 {
 	char line[MAXLINE];
 	REGISTER char save, *pt, *ptend;
@@ -261,7 +260,7 @@ LISPTREE *sdfi_readfile(FILE *f)
 		if ((sdfi_lineno%50) == 0)
 		{
 			filepos = xtell(f);
-			DiaPercent(1, filepos*100/sdfi_filesize);
+			DiaSetProgress(dia, filepos, sdfi_filesize);
 		}
 
 		/* stop now if it is a C++ style '//' comment */
@@ -1343,5 +1342,3 @@ void sdfi_clearglobals(void)
 	if (sdfi_instance != NULL)    efree((char *)sdfi_instance);     sdfi_instance = NULL;
 	if (sdfi_divider != NULL)     efree((char *)sdfi_divider);      sdfi_divider = NULL;
 }
-
-#endif  /* IOSDF - at top */
