@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iosuei.c
@@ -29,8 +30,6 @@
  */
 
 #include "config.h"
-#if IOSUE
-
 #include "global.h"
 #include "edialogs.h"
 #include "egraphics.h"
@@ -65,6 +64,7 @@ typedef struct
 	NODEPROTO   **intproto;
 	INTBIG        netateoutput;
 	INTBIG        rotation;
+	INTBIG        transpose;
 	INTBIG        xoffset;
 	INTBIG        yoffset;
 	INTBIG        detailbits;
@@ -79,6 +79,14 @@ SUEEXTRAWIRE io_suetransistorwires[] =
 	{"g",     0,  H4}
 };
 
+SUEEXTRAWIRE io_suetransistor4wires[] =
+{
+	{"d",    K3,   0},
+	{"s",   -K3,   0},
+	{"b",   -Q0, -H2},
+	{"g",     0,  H4}
+};
+
 SUEEXTRAWIRE io_sueresistorwires[] =
 {
 	{"a",   -K3,   0},
@@ -107,21 +115,40 @@ SUEEXTRAWIRE io_suetwoportwires[] =
 
 SUEEQUIV io_sueequivs[] =
 {
-	/* name         primitive          NEG ANG    X   Y  BITS         WIR EXTRA-WIRES */
-	{"pmos10",     &sch_transistorprim, 0, 900, -K2,  0, TRANPMOS,     3, io_suetransistorwires},
-	{"pmos4",      &sch_transistorprim, 0, 900, -K2,  0, TRANPMOS,     3, io_suetransistorwires},
-	{"pmos",       &sch_transistorprim, 0, 900, -K2,  0, TRANPMOS,     3, io_suetransistorwires},
-	{"nmos10",     &sch_transistorprim, 0, 900, -K2,  0, TRANNMOS,     3, io_suetransistorwires},
-	{"nmos4",      &sch_transistorprim, 0, 900, -K2,  0, TRANNMOS,     3, io_suetransistorwires},
-	{"nmos",       &sch_transistorprim, 0, 900, -K2,  0, TRANNMOS,     3, io_suetransistorwires},
-	{"capacitor",  &sch_capacitorprim,  0,   0,   0,  0, 0,            2, io_suecapacitorwires},
-	{"resistor",   &sch_resistorprim,   0, 900,   0,  0, 0,            2, io_sueresistorwires},
-	{"inductor",   &sch_inductorprim,   0,   0,   0,  0, 0,            0, 0},
-	{"cccs",       &sch_twoportprim,    0,   0,  Q1,-S6, TWOPCCCS,     4, io_suetwoportwires},
-	{"ccvs",       &sch_twoportprim,    0,   0,  Q1,-S6, TWOPCCVS,     4, io_suetwoportwires},
-	{"vcvs",       &sch_twoportprim,    0,   0,  Q1,-S6, TWOPVCVS,     4, io_suetwoportwires},
-	{"vccs",       &sch_twoportprim,    0,   0, -S1,-K5, TWOPVCCS,     0, 0},
-	{0, 0, 0, 0, 0, 0, 0, 0}
+	/* name         primitive           NEG  ANG     X   Y  BITS         WIR EXTRA-WIRES */
+	{"pmos10",     &sch_transistorprim,  0, 900,0, -K2,  0, TRANPMOS,     3, io_suetransistorwires},
+	{"nmos10",     &sch_transistorprim,  0, 900,0, -K2,  0, TRANNMOS,     3, io_suetransistorwires},
+	{"pmos4",      &sch_transistorprim,  0, 900,0, -K2,  0, TRANPMOS,     3, io_suetransistorwires},
+	{"nmos4",      &sch_transistorprim,  0, 900,0, -K2,  0, TRANNMOS,     3, io_suetransistorwires},
+	{"pmos",       &sch_transistorprim,  0, 900,0, -K2,  0, TRANPMOS,     3, io_suetransistorwires},
+	{"nmos",       &sch_transistorprim,  0, 900,0, -K2,  0, TRANNMOS,     3, io_suetransistorwires},
+	{"capacitor",  &sch_capacitorprim,   0,   0,0,   0,  0, 0,            2, io_suecapacitorwires},
+	{"resistor",   &sch_resistorprim,    0, 900,0,   0,  0, 0,            2, io_sueresistorwires},
+	{"inductor",   &sch_inductorprim,    0,   0,0,   0,  0, 0,            0, 0},
+	{"cccs",       &sch_twoportprim,     0,   0,0,  Q1,-S6, TWOPCCCS,     4, io_suetwoportwires},
+	{"ccvs",       &sch_twoportprim,     0,   0,0,  Q1,-S6, TWOPCCVS,     4, io_suetwoportwires},
+	{"vcvs",       &sch_twoportprim,     0,   0,0,  Q1,-S6, TWOPVCVS,     4, io_suetwoportwires},
+	{"vccs",       &sch_twoportprim,     0,   0,0, -S1,-K5, TWOPVCCS,     0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0, 0}
+};
+
+SUEEQUIV io_sueequivs4[] =
+{
+	/* name         primitive           NEG  ANG     X   Y  BITS         WIR EXTRA-WIRES */
+	{"pmos10",     &sch_transistor4prim, 0,   0,1, -K2,  0, TRANPMOS,     4, io_suetransistor4wires},
+	{"nmos10",     &sch_transistor4prim, 0, 900,0, -K2,  0, TRANNMOS,     4, io_suetransistor4wires},
+	{"pmos4",      &sch_transistor4prim, 0,   0,1, -K2,  0, TRANPMOS,     4, io_suetransistor4wires},
+	{"nmos4",      &sch_transistor4prim, 0, 900,0, -K2,  0, TRANNMOS,     4, io_suetransistor4wires},
+	{"pmos",       &sch_transistor4prim, 0,   0,1, -K2,  0, TRANPMOS,     4, io_suetransistor4wires},
+	{"nmos",       &sch_transistor4prim, 0, 900,0, -K2,  0, TRANNMOS,     4, io_suetransistor4wires},
+	{"capacitor",  &sch_capacitorprim,   0,   0,0,   0,  0, 0,            2, io_suecapacitorwires},
+	{"resistor",   &sch_resistorprim,    0, 900,0,   0,  0, 0,            2, io_sueresistorwires},
+	{"inductor",   &sch_inductorprim,    0,   0,0,   0,  0, 0,            0, 0},
+	{"cccs",       &sch_twoportprim,     0,   0,0,  Q1,-S6, TWOPCCCS,     4, io_suetwoportwires},
+	{"ccvs",       &sch_twoportprim,     0,   0,0,  Q1,-S6, TWOPCCVS,     4, io_suetwoportwires},
+	{"vcvs",       &sch_twoportprim,     0,   0,0,  Q1,-S6, TWOPVCVS,     4, io_suetwoportwires},
+	{"vccs",       &sch_twoportprim,     0,   0,0, -S1,-K5, TWOPVCCS,     0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0, 0}
 };
 
 /*************** SUE WIRES ***************/
@@ -176,7 +203,7 @@ static BOOLEAN    io_suefindnode(INTBIG *x, INTBIG *y, INTBIG ox, INTBIG oy, NOD
 static NODEINST  *io_suefindpinnode(INTBIG x, INTBIG y, NODEPROTO *np, PORTPROTO **thepp);
 static void       io_suefreenets(SUENET *firstsn);
 static void       io_suefreewires(SUEWIRE *firstsw);
-static INTBIG     io_suegetnextline(char **keywords, INTBIG curlydepth);
+static INTBIG     io_suegetnextline(char **keywords, INTBIG curlydepth, void *dia);
 static NODEPROTO *io_suegetnodeproto(LIBRARY *lib, char *protoname);
 static void       io_suekillnet(SUENET *sn);
 static void       io_suekillwire(SUEWIRE *sw);
@@ -186,11 +213,13 @@ static PORTPROTO *io_suenewexport(NODEPROTO *facet, NODEINST *ni, PORTPROTO *pp,
 static SUENET    *io_suenewnet(void);
 static SUEWIRE   *io_suenewwire(void);
 static void       io_sueplacenets(SUENET *firstsuenet, NODEPROTO *facet);
+static char      *io_sueparseexpression(char *expression);
 static void       io_sueparseparameters(char **keywords, INTBIG count, INTBIG *x, INTBIG *y, INTBIG lambda,
-					INTBIG *rot, INTBIG *trn, INTBIG *type, char **thename, char **thelabel, char **thetext);
+					INTBIG *rot, INTBIG *trn, INTBIG *type, char **thename, char **thelabel, char **thetext,
+					void *dia);
 static void       io_sueplacewires(SUEWIRE *firstsuewire, SUENET *firstsuenet, NODEPROTO *facet, INTBIG lambda);
-static NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname);
-static NODEPROTO *io_suereadfromdisk(LIBRARY *lib, char *name);
+static NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname, void *dia);
+static NODEPROTO *io_suereadfromdisk(LIBRARY *lib, char *name, void *dia);
 static char      *io_suesearchbusname(ARCINST *ai);
 static PORTPROTO *io_suewiredport(NODEINST *ni, INTBIG *x, INTBIG *y, INTBIG ox, INTBIG oy);
 
@@ -221,9 +250,9 @@ BOOLEAN io_readsuelibrary(LIBRARY *lib)
 {
 	REGISTER INTBIG len, i, filecount;
 	REGISTER NODEPROTO *topfacet;
-	extern DIALOG us_progressdialog;
 	char facetname[300], dirname[300], topdirname[300], truesuefile[300],
 		*filename, **filelist;
+	void *dia;
 
 	/* open the file */
 	io_suefilein = xopen(lib->libfile, io_filetypesue, "", &filename);
@@ -289,7 +318,8 @@ BOOLEAN io_readsuelibrary(LIBRARY *lib)
 
 	/* prepare for input */
 	io_suefilesize = filesize(io_suefilein);
-	if (DiaInitDialog(&us_progressdialog))
+	dia = DiaInitProgress(_("Reading SUE file..."), 0);
+	if (dia == 0)
 	{
 		xclose(io_suefilein);
 		return(TRUE);
@@ -297,15 +327,14 @@ BOOLEAN io_readsuelibrary(LIBRARY *lib)
 	io_suelineno = 0;
 
 	/* read the file */
-	DiaPercent(1, 0);
-	DiaSetText(2, _("Reading SUE file..."));
-	topfacet = io_suereadfile(lib, facetname);
+	DiaSetProgress(dia, 0, io_suefilesize);
+	topfacet = io_suereadfile(lib, facetname, dia);
 	if (topfacet != NONODEPROTO)
 		lib->curnodeproto = topfacet;
 	(void)asktool(net_tool, "total-re-number");
 
 	/* clean up */
-	DiaDoneDialog();
+	DiaDoneProgress(dia);
 	xclose(io_suefilein);
 	ttyputmsg(_("%s read"), truesuefile);
 
@@ -315,17 +344,17 @@ BOOLEAN io_readsuelibrary(LIBRARY *lib)
 /*
  * Routine to read the SUE file in "f"/
  */
-NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
+NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname, void *dia)
 {
 	char *keywords[MAXKEYWORDS], *thename, *thelabel, *thetext, *namestring;
 	char suevarname[200], *pt, *cpt, *startkey, save;
 	char **argnames, **argvalues, **newargnames, **newargvalues;
 	INTBIG outline[MAXICONPOINTS*2], x, y, type, dx, dy, px, py, pinx, piny, xoff, yoff,
-		numargs, namestrlen, rot, trn, rotation;
+		numargs, namestrlen, rot, trn, rotation, transpose, *curstate;
 	REGISTER INTBIG count, i, lx, hx, ly, hy, curly, xshrink, yshrink,
 		varcount, varindex, varoffset, pos, invertoutput, lambda, detailbits,
 		keycount, cx, cy, p1x, p1y, p2x, p2y, start, extent, numextrawires, len,
-		iconx, icony, newaddr, newtype, xpos, ypos, numericlambda;
+		iconx, icony, newaddr, newtype, xpos, ypos, numericlambda, bits;
 	REGISTER BOOLEAN halvesize, placeicon, varissize, isparam;
 	XARRAY trans;
 	double rextent, rstart;
@@ -340,6 +369,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 	REGISTER SUEWIRE *sw, *firstsuewire;
 	REGISTER SUENET *sn, *firstsuenet;
 	REGISTER SUEEXTRAWIRE *extrawires;
+	REGISTER SUEEQUIV *curequivs;
 	REGISTER void *infstr;
 
 	firstsuewire = NOSUEWIRE;
@@ -354,7 +384,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 	for(;;)
 	{
 	   /* get the next line of text */
-		count = io_suegetnextline(keywords, 0);
+		count = io_suegetnextline(keywords, 0, dia);
 		if (count < 0) break;
 		if (count == 0) continue;
 
@@ -418,7 +448,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 
 			/* extract parameters */
 			io_sueparseparameters(&keywords[2], count-2, &x, &y, lambda, &rot, &trn,
-				&type, &thename, &thelabel, &thetext);
+				&type, &thename, &thelabel, &thetext, dia);
 
 			/* save the name string */
 			if (thename != 0)
@@ -464,7 +494,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 			/* first check for special names */
 			proto = NONODEPROTO;
 			invertoutput = 0;
-			rotation = 0;
+			rotation = transpose = 0;
 			xoff = yoff = 0;
 			xshrink = yshrink = 0;
 			detailbits = 0;
@@ -531,22 +561,35 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 			/* now check for internal associations to known primitives */
 			if (proto == NONODEPROTO)
 			{
-				for(i=0; io_sueequivs[i].suename != 0; i++)
-					if (namesame(keywords[1], io_sueequivs[i].suename) == 0) break;
-				if (io_sueequivs[i].suename != 0)
+				curstate = io_getstatebits();
+				if ((curstate[1]&SUEUSE4PORTTRANS) != 0) curequivs = io_sueequivs4; else
+					curequivs = io_sueequivs;
+				for(i=0; curequivs[i].suename != 0; i++)
+					if (namesame(keywords[1], curequivs[i].suename) == 0) break;
+				if (curequivs[i].suename != 0)
 				{
-					proto = *io_sueequivs[i].intproto;
-					invertoutput = io_sueequivs[i].netateoutput;
-					rotation = io_sueequivs[i].rotation;
+					proto = *curequivs[i].intproto;
+					invertoutput = curequivs[i].netateoutput;
+					rotation = curequivs[i].rotation;
+					transpose = curequivs[i].transpose;
 					makeangle(rot, trn, trans);
-					xform(io_sueequivs[i].xoffset, io_sueequivs[i].yoffset, &xoff, &yoff, trans);
+					xform(curequivs[i].xoffset, curequivs[i].yoffset, &xoff, &yoff, trans);
 					xoff = muldiv(xoff, lambda, WHOLE);
 					yoff = muldiv(yoff, lambda, WHOLE);
-					rot += rotation;
-					if (rot >= 3600) rot -= 3600;
-					detailbits = io_sueequivs[i].detailbits;
-					numextrawires = io_sueequivs[i].numextrawires;
-					extrawires = io_sueequivs[i].extrawires;
+					
+					if (transpose != 0)
+					{
+						trn = 1 - trn;
+						rot = rotation - rot;
+						if (rot < 0) rot += 3600;
+					} else
+					{
+						rot += rotation;
+						if (rot >= 3600) rot -= 3600;
+					}
+					detailbits = curequivs[i].detailbits;
+					numextrawires = curequivs[i].numextrawires;
+					extrawires = curequivs[i].extrawires;
 #ifdef SHOWORIGINAL
 					defaultnodesize(proto, &px, &py);
 					lx = x - px/2 + xoff;   hx = lx + px;
@@ -557,7 +600,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 					endobjectchange((INTBIG)ni, VNODEINST);
 					proto = NONODEPROTO;
 					invertoutput = 0;
-					rotation = 0;
+					rotation = transpose = 0;
 					xoff = yoff = 0;
 					detailbits = 0;
 					numextrawires = 0;
@@ -572,7 +615,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 				/* find node or read it from disk */
 				proto = io_suegetnodeproto(lib, keywords[1]);
 				if (proto == NONODEPROTO)
-					proto = io_suereadfromdisk(lib, keywords[1]);
+					proto = io_suereadfromdisk(lib, keywords[1], dia);
 
 				/* set proper offsets for the facet */
 				if (proto != NONODEPROTO)
@@ -609,12 +652,13 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 			if (proto->primindex == 0 && proto->cellview == el_iconview)
 				ni->userbits |= NEXPAND;
 			endobjectchange((INTBIG)ni, VNODEINST);
+			if (facet->tech == gen_tech)
+				facet->tech = whattech(facet);
 
 			/* add any extra wires to the node */
 			for(i=0; i<numextrawires; i++)
 			{
-				for(pp = proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-					if (namesame(pp->protoname, extrawires[i].portname) == 0) break;
+				pp = getportproto(proto, extrawires[i].portname);
 				if (pp == NOPORTPROTO) continue;
 				portposition(ni, pp, &x, &y);
 				makeangle(ni->rotation, ni->transpose, trans);
@@ -632,8 +676,9 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 					if (nni == NONODEINST) continue;
 					npp = nni->proto->firstportproto;
 				}
-				ai = newarcinst(sch_wirearc, 0, us_makearcuserbits(sch_wirearc),
-					ni, pp, x, y, nni, npp, pinx, piny, facet);
+				bits = us_makearcuserbits(sch_wirearc);
+				if (x != pinx && y != piny) bits &= ~FIXANG;
+				ai = newarcinst(sch_wirearc, 0, bits, ni, pp, x, y, nni, npp, pinx, piny, facet);
 				if (ai == NOARCINST)
 				{
 					ttyputerr(_("Error adding extra wires to node %s"), keywords[1]);
@@ -724,31 +769,40 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 					ypos = pos * 4 / lambda;
 					isparam = TRUE;
 				}
-				newtype = VSTRING;
-				if (varissize)
-				{
-					/* see if this size is given in microns */
-					pt = keywords[i+1];
-					if (tolower(pt[strlen(pt)-1]) == 'u')
-					{
-						newaddr = scalefromdispunit((float)atof(pt), DISPUNITMIC) * WHOLE / numericlambda;
-					} else
-					{
-						newaddr = atola(keywords[i+1]) * WHOLE / numericlambda;
-					}
-					if (newaddr != 0) newtype = VFRACT; else
-						newaddr = (INTBIG)keywords[i+1];
-				} else
+				newtype = 0;
+				if (keywords[i][1] == 'W' && keywords[i][2] != 0)
 				{
 					infstr = initinfstr();
-					if (keywords[i][1] == 'W' && keywords[i][2] != 0)
-					{
-						addstringtoinfstr(infstr, &keywords[i][2]);
-						addstringtoinfstr(infstr, ":");
-					}
-					addstringtoinfstr(infstr, keywords[i+1]);
+					addstringtoinfstr(infstr, &keywords[i][2]);
+					addstringtoinfstr(infstr, ":");
+					addstringtoinfstr(infstr, io_sueparseexpression(keywords[i+1]));
 					newaddr = (INTBIG)returninfstr(infstr);
-				}
+					newtype = VSTRING;
+				} else
+				{
+					pt = keywords[i+1];
+					len = strlen(pt) - 1;
+					if (tolower(pt[len]) == 'u')
+					{
+						pt[len] = 0;
+						if (isanumber(pt))
+						{
+							newaddr = scalefromdispunit((float)atof(pt), DISPUNITMIC) * WHOLE / numericlambda;
+							newtype = VFRACT;
+						}
+						pt[len] = 'u';
+					}
+					if (newtype == 0 && isanumber(pt))
+					{
+						newaddr = atola(pt, 0) * WHOLE / numericlambda;
+						newtype = VFRACT;
+					}
+					if (newtype == 0)
+					{
+						newaddr = (INTBIG)io_sueparseexpression(pt);
+						newtype = VSTRING;
+					}
+				} 
 				var = setval((INTBIG)ni, VNODEINST, suevarname, newaddr,
 					newtype|VDISPLAY);
 				if (var != NOVARIABLE)
@@ -785,7 +839,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 		if (namesame(keywords[0], "icon_term") == 0)
 		{
 			io_sueparseparameters(&keywords[1], count-1, &x, &y, lambda, &rot, &trn,
-				&type, &thename, &thelabel, &thetext);
+				&type, &thename, &thelabel, &thetext, dia);
 			strcpy(suevarname, thename);
 
 			proto = sch_buspinprim;
@@ -955,7 +1009,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 		{
 			/* extract parameters */
 			io_sueparseparameters(&keywords[1], count-1, &x, &y, lambda, &rot, &trn,
-				&type, &thename, &thelabel, &thetext);
+				&type, &thename, &thelabel, &thetext, dia);
 			if (thelabel == 0) continue;
 
 			/* substitute parameters */
@@ -994,7 +1048,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 		{
 			/* extract parameters */
 			io_sueparseparameters(&keywords[1], count-1, &x, &y, lambda, &rot, &trn,
-				&type, &thename, &thelabel, &thetext);
+				&type, &thename, &thelabel, &thetext, dia);
 			if (thetext == 0) continue;
 
 			ni = newnodeinst(gen_invispinprim, x, x, y, y, 0, 0, facet);
@@ -1068,6 +1122,37 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 
 }
 
+/*
+ * Routine to examine a SUE expression and add "@" in front of variable names.
+ */
+char *io_sueparseexpression(char *expression)
+{
+	REGISTER void *infstr;
+	REGISTER char *keyword;
+
+	infstr = initinfstr();
+	while (*expression != 0)
+	{
+		keyword = getkeyword(&expression, " \t,+-*/()");
+		if (keyword == NOSTRING) break;
+		if (*keyword != 0)
+		{
+			if (isdigit(keyword[0]))
+			{
+				addstringtoinfstr(infstr, keyword);
+			} else
+			{
+				if (*expression != '(')
+					addtoinfstr(infstr, '@');
+				addstringtoinfstr(infstr, keyword);
+			}
+			if (*expression != 0)
+				addtoinfstr(infstr, *expression++);
+		}
+	}
+	return(returninfstr(infstr));
+}
+
 /*
  * Routine to parse the "count" parameters in "keywords" and fill in the values
  * that are found.  Fills in:
@@ -1079,7 +1164,8 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
  * "-text"    placed into "thetext".
  */
 void io_sueparseparameters(char **keywords, INTBIG count, INTBIG *x, INTBIG *y, INTBIG lambda,
-	INTBIG *rot, INTBIG *trn, INTBIG *type, char **thename, char **thelabel, char **thetext)
+	INTBIG *rot, INTBIG *trn, INTBIG *type, char **thename, char **thelabel, char **thetext,
+	void *dia)
 {
 	char *pt;
 	REGISTER INTBIG textloc, i;
@@ -1142,7 +1228,7 @@ void io_sueparseparameters(char **keywords, INTBIG count, INTBIG *x, INTBIG *y,
 					if (*pt == '}') break;
 					addtoinfstr(infstr, ' ');
 
-					count = io_suegetnextline(keywords, 1);
+					count = io_suegetnextline(keywords, 1, dia);
 					if (count < 0) break;
 					if (count == 0) continue;
 					pt = keywords[0];
@@ -1190,8 +1276,7 @@ PORTPROTO *io_suenewexport(NODEPROTO *facet, NODEINST *ni, PORTPROTO *pp, char *
 	portname = thename;
 	for(i=0; ; i++)
 	{
-		for(ppt = facet->firstportproto; ppt != NOPORTPROTO; ppt = ppt->nextportproto)
-			if (namesame(portname, ppt->protoname) == 0) break;
+		ppt = getportproto(facet, portname);
 		if (ppt == NOPORTPROTO)
 		{
 			ppt = newportproto(facet, ni, pp, portname);
@@ -1870,7 +1955,7 @@ BOOLEAN io_suefindnode(INTBIG *x, INTBIG *y, INTBIG ox, INTBIG oy,
  * Routine to find the SUE file "name" on disk, and read it into library "lib".
  * Returns NONODEPROTO if the file is not found or not read properly.
  */
-NODEPROTO *io_suereadfromdisk(LIBRARY *lib, char *name)
+NODEPROTO *io_suereadfromdisk(LIBRARY *lib, char *name, void *dia)
 {
 	REGISTER INTBIG i, savelineno, filepos, savefilesize;
 	char suevarname[200], subfilename[300], savesuelastline[MAXLINE], saveorigline[MAXLINE],
@@ -1892,19 +1977,19 @@ NODEPROTO *io_suereadfromdisk(LIBRARY *lib, char *name)
 			for(i=0; i<MAXLINE; i++) savecurline[i] = io_suecurline[i];
 			strcpy(saveorigline, io_sueorigline);
 			strcpy(savesuelastline, io_suelastline);
-			strcpy(lastprogressmsg, DiaGetText(2));
+			strcpy(lastprogressmsg, DiaGetTextProgress(dia));
 			strcpy(suevarname, _("Reading "));
 			strcat(suevarname, name);
 			strcat(suevarname, "...");
-			DiaSetText(2, suevarname);
-			DiaPercent(1, 0);
+			DiaSetTextProgress(dia, suevarname);
 
 			strcpy(subfilename, name);
 			savefilesize = io_suefilesize;
 			savelineno = io_suelineno;
 			io_suefilesize = filesize(io_suefilein);
+			DiaSetProgress(dia, 0, io_suefilesize);
 			io_suelineno = 0;
-			(void)io_suereadfile(lib, subfilename);
+			(void)io_suereadfile(lib, subfilename, dia);
 			xclose(io_suefilein);
 			io_suefilein = savefp;
 			io_suefilesize = savefilesize;
@@ -1913,8 +1998,8 @@ NODEPROTO *io_suereadfromdisk(LIBRARY *lib, char *name)
 			strcpy(io_sueorigline, saveorigline);
 			for(i=0; i<MAXLINE; i++) io_suecurline[i] = savecurline[i];
 			filepos = xtell(io_suefilein);
-			DiaSetText(2, lastprogressmsg);
-			DiaPercent(1, filepos*100/io_suefilesize);
+			DiaSetTextProgress(dia, lastprogressmsg);
+			DiaSetProgress(dia, filepos, io_suefilesize);
 
 			/* now try to find the facet in the library */
 			proto = io_suegetnodeproto(lib, subfilename);
@@ -1929,7 +2014,7 @@ NODEPROTO *io_suereadfromdisk(LIBRARY *lib, char *name)
  * it up into space-separated keywords.  Returns the number
  * of keywords (-1 on EOF)
  */
-INTBIG io_suegetnextline(char **keywords, INTBIG curlydepth)
+INTBIG io_suegetnextline(char **keywords, INTBIG curlydepth, void *dia)
 {
 	char *pt;
 	REGISTER INTBIG filepos, keypos, lineno;
@@ -1944,7 +2029,7 @@ INTBIG io_suegetnextline(char **keywords, INTBIG curlydepth)
 			if ((io_suelineno%50) == 0)
 			{
 				filepos = xtell(io_suefilein);
-				DiaPercent(1, filepos*100/io_suefilesize);
+				DiaSetProgress(dia, filepos, io_suefilesize);
 			}
 		}
 		if (lineno == 0)
@@ -2083,5 +2168,3 @@ void io_suefreenets(SUENET *firstsn)
 		io_suekillnet(sn);
 	}
 }
-
-#endif  /* IOSUE - at top */
