@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iotexti.c
@@ -40,6 +41,24 @@
 
 #define REPORTRATE  100		/* number of elements between graphical update */
 
+#define LONGJMPNOMEM         1		/* error: No Memory */
+#define LONGJMPMISCOLON      2		/* error: Missing colon */
+#define LONGJMPUKNKEY        3		/* error: Unknown keyword */
+#define LONGJMPEOF           4		/* error: EOF too soon */
+#define LONGJMPEOFVAR        5		/* error: EOF in variables */
+#define LONGJMPMISCOLVAR     6		/* error: Missing ':' in variable */
+#define LONGJMPMISOSBVAR     7		/* error: Missing '[' in variable */
+#define LONGJMPMISOSBVARARR  8		/* error: Missing '[' in variable array */
+#define LONGJMPSHORTARR      9		/* error: Short array specification */
+#define LONGJMPUKNNOPROTO   10		/* error: Unknown node prototype */
+#define LONGJMPMISVIEWABR   11		/* error: Missing view abbreviation */
+#define LONGJMPBADVIEWABR   12		/* error: View abbreviation bad */
+#define LONGJMPINCVIEWABR   13		/* error: View abbreviation inconsistent */
+#define LONGJMPSECFACETNAME 14		/* error: Second facet name */
+#define LONGJMPUKNTECH      15		/* error: Unknown technology */
+#define LONGJMPINVARCPROTO  16		/* error: Invalid arc prototype */
+#define LONGJMPUKNPORPROTO  17		/* error: Unknown port prototype name */
+
 /* state of input (value of "io_txtindata.textlevel") */
 #define INLIB       1
 #define INFACET     2
@@ -372,7 +391,7 @@ BOOLEAN io_doreadtextlibrary(LIBRARY *lib, BOOLEAN newprogress)
 {
 	struct keyroutine *table;
 	REGISTER char *pp;
-	REGISTER INTBIG i, j, reportcount, lambda;
+	REGISTER INTBIG i, j, reportcount, lambda, errcode;
 	INTBIG num, den;
 	REGISTER INTBIG oldunit;
 	REGISTER NODEPROTO *np;
@@ -380,7 +399,6 @@ BOOLEAN io_doreadtextlibrary(LIBRARY *lib, BOOLEAN newprogress)
 	REGISTER NODEINST *ni;
 	REGISTER ARCINST *ai;
 	REGISTER CELL *c;
-	extern DIALOG us_eprogressdialog;
 	char *filename;
 	REGISTER void *infstr;
 
@@ -408,7 +426,8 @@ BOOLEAN io_doreadtextlibrary(LIBRARY *lib, BOOLEAN newprogress)
 		{
 			if (newprogress)
 			{
-				if (DiaInitDialog(&us_eprogressdialog))
+				io_inputprogressdialog = DiaInitProgress("", _("Reading library"));
+				if (io_inputprogressdialog == 0)
 				{
 					xclose(io_txtindata.textfilein);
 					return(TRUE);
@@ -416,20 +435,41 @@ BOOLEAN io_doreadtextlibrary(LIBRARY *lib, BOOLEAN newprogress)
 			}
 			infstr = initinfstr();
 			formatinfstr(infstr, _("Reading library %s"), lib->libname);
-			DiaSetText(3, returninfstr(infstr));
-			DiaSetText(2, _("Initializing..."));
-			DiaPercent(1, 0);
+			DiaSetCaptionProgress(io_inputprogressdialog, returninfstr(infstr));
+			DiaSetTextProgress(io_inputprogressdialog, _("Initializing..."));
+			DiaSetProgress(io_inputprogressdialog, 0, io_txtindata.filelength);
 		}
 	} else io_txtindata.filelength = 0;
 
 	/* the error recovery point */
-	pp = (char *)setjmp(io_filerror);
-	if (pp != 0)
+	errcode = setjmp(io_filerror);
+	if (errcode != 0)
 	{
+		switch (errcode)
+		{
+			case LONGJMPNOMEM:        pp = _("No Memory");                        break;
+			case LONGJMPMISCOLON:     pp = _("Missing colon");                    break;
+			case LONGJMPUKNKEY:       pp = _("Unknown keyword");                  break;
+			case LONGJMPEOF:          pp = _("EOF too soon");                     break;
+			case LONGJMPEOFVAR:       pp = _("EOF in variables");                 break;
+			case LONGJMPMISCOLVAR:    pp = _("Missing ':' in variable");          break;
+			case LONGJMPMISOSBVAR:    pp = _("Missing '[' in variable");          break;
+			case LONGJMPMISOSBVARARR: pp = _("Missing '[' in variable array");    break;
+			case LONGJMPSHORTARR:     pp = _("Short array specification");        break;
+			case LONGJMPUKNNOPROTO:   pp = _("Unknown node prototype");           break;
+			case LONGJMPMISVIEWABR:   pp = _("Missing view abbreviation");        break;
+			case LONGJMPBADVIEWABR:   pp = _("View abbreviation bad");            break;
+			case LONGJMPINCVIEWABR:   pp = _("View abbreviation inconsistent");   break;
+			case LONGJMPSECFACETNAME: pp = _("Second facet name");                break;
+			case LONGJMPUKNTECH:      pp = _("Unknown technology");               break;
+			case LONGJMPINVARCPROTO:  pp = _("Invalid arc prototype");            break;
+			case LONGJMPUKNPORPROTO:  pp = _("Unknown port prototype name");      break;
+			default:                  pp = _("Unknown");                          break;
+		}
 		ttyputerr(_("Error: %s on line %ld, keyword '%s'"), pp,
 			io_txtindata.linenumber, io_keyword);
 		xclose(io_txtindata.textfilein);
-		if (io_verbose < 0 && io_txtindata.filelength > 0) DiaDoneDialog();
+		if (io_verbose < 0 && io_txtindata.filelength > 0) DiaDoneProgress(io_inputprogressdialog);
 		return(TRUE);
 	}
 
@@ -441,7 +481,7 @@ BOOLEAN io_doreadtextlibrary(LIBRARY *lib, BOOLEAN newprogress)
 	if (j > 4 && namesame(&pp[j-4], ".txt") == 0) j -= 4;
 	for(i=0; i<j; i++) addtoinfstr(infstr, pp[i]);
 	if (reallocstring(&lib->libname, returninfstr(infstr), lib->cluster))
-		longjmp(io_filerror, (INTBIG)_("No memory"));
+		longjmp(io_filerror, LONGJMPNOMEM);
 
 	io_txtindata.curlib = lib;
 	eraselibrary(lib);
@@ -459,14 +499,14 @@ BOOLEAN io_doreadtextlibrary(LIBRARY *lib, BOOLEAN newprogress)
 		/* get keyword from file */
 		if (io_getkeyword()) break;
 		pp = io_keyword;   while (*pp != 0) pp++;
-		if (pp[-1] != ':') longjmp(io_filerror, (INTBIG)_("Missing colon"));
+		if (pp[-1] != ':') longjmp(io_filerror, LONGJMPMISCOLON);
 		pp[-1] = 0;
 
 		if (reportcount++ > REPORTRATE)
 		{
 			reportcount = 0;
 			if (io_verbose < 0 && io_txtindata.filelength > 0)
-				DiaPercent(1, io_txtindata.fileposition*100/io_txtindata.filelength);
+				DiaSetProgress(io_inputprogressdialog, io_txtindata.fileposition, io_txtindata.filelength);
 		}
 
 		/* determine which keyword table to use */
@@ -485,17 +525,17 @@ BOOLEAN io_doreadtextlibrary(LIBRARY *lib, BOOLEAN newprogress)
 		for(i=0; table[i].matchstring != 0; i++)
 			if (strcmp(table[i].matchstring, io_keyword) == 0) break;
 		if (table[i].matchstring == 0)
-			longjmp(io_filerror, (INTBIG)_("Unknown keyword"));
+			longjmp(io_filerror, LONGJMPUKNKEY);
 
 		/* get argument to routine and call it */
-		if (io_getkeyword()) longjmp(io_filerror, (INTBIG)_("EOF too soon"));
+		if (io_getkeyword()) longjmp(io_filerror, LONGJMPEOF);
 		(*(table[i].routine))();
 	}
 	xclose(io_txtindata.textfilein);
 	if (io_verbose < 0 && io_txtindata.filelength > 0)
 	{
-		DiaPercent(1, 100);
-		DiaSetText(2, _("Cleaning up..."));
+		DiaSetProgress(io_inputprogressdialog, 999, 1000);
+		DiaSetTextProgress(io_inputprogressdialog, _("Cleaning up..."));
 	}
 
 	/* set the former version number on the library */
@@ -534,7 +574,7 @@ BOOLEAN io_doreadtextlibrary(LIBRARY *lib, BOOLEAN newprogress)
 	{
 		c->cluster = alloccluster(c->cellname);
 		if (c->cluster == NOCLUSTER)
-			longjmp(io_filerror, (INTBIG)_("No cluster memory"));
+			longjmp(io_filerror, LONGJMPNOMEM);
 	}
 
 	/* warn if the MOSIS CMOS technologies were converted */
@@ -577,7 +617,7 @@ BOOLEAN io_doreadtextlibrary(LIBRARY *lib, BOOLEAN newprogress)
 		io_txtindata.curlib->curnodeproto = io_txtindata.nodeprotolist[io_txtindata.mainfacet]; else
 			io_txtindata.curlib->curnodeproto = NONODEPROTO;
 	if (io_txtindata.facetcount != 0) efree((char *)io_txtindata.nodeprotolist);
-	if (io_verbose < 0 && io_txtindata.filelength > 0 && newprogress) DiaDoneDialog();
+	if (io_verbose < 0 && io_txtindata.filelength > 0 && newprogress) DiaDoneProgress(io_inputprogressdialog);
 
 	/* clean up the library */
 	io_fixnewlib(io_txtindata.curlib, 0);
@@ -677,8 +717,8 @@ PORTPROTO *io_getport(char *line, NODEPROTO *np)
 	REGISTER PORTPROTO *pp;
 	REGISTER INTBIG i;
 
-	for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-		if (namesame(line, pp->protoname) == 0) return(pp);
+	pp = getportproto(np, line);
+	if (pp != NOPORTPROTO) return(pp);
 
 	/* convert special port names */
 	pp = io_convertoldportname(line, np);
@@ -903,9 +943,9 @@ void io_getvar(void)
 	for(i=0; i<count; i++)
 	{
 		/* read the first keyword with the name, type, and descriptor */
-		if (io_getkeyword()) longjmp(io_filerror, (INTBIG)_("EOF in variables"));
+		if (io_getkeyword()) longjmp(io_filerror, LONGJMPEOFVAR);
 		if (io_keyword[strlen(io_keyword)-1] != ':')
-			longjmp(io_filerror, (INTBIG)_("Missing ':' in variable"));
+			longjmp(io_filerror, LONGJMPMISCOLVAR);
 
 		/* get the variable name */
 		infstr = initinfstr();
@@ -929,7 +969,7 @@ void io_getvar(void)
 		} else len = -1;
 
 		/* get type */
-		if (*pt != '[') longjmp(io_filerror, (INTBIG)_("Missing '[' in variable"));
+		if (*pt != '[') longjmp(io_filerror, LONGJMPMISOSBVAR);
 		type = myatoi(&pt[1]);
 
 		/* get the descriptor */
@@ -944,20 +984,20 @@ void io_getvar(void)
 		}
 
 		/* get value */
-		if (io_getkeyword()) longjmp(io_filerror, (INTBIG)_("EOF in variables"));
+		if (io_getkeyword()) longjmp(io_filerror, LONGJMPEOFVAR);
 		if ((type&VISARRAY) != 0)
 		{
 			if (len < 0)
 			{
 				len = (type&VLENGTH) >> VLENGTHSH;
 				store = emalloc((len * SIZEOFINTBIG), el_tempcluster);
-				if (store == 0) longjmp(io_filerror, (INTBIG)_("No memory"));
+				if (store == 0) longjmp(io_filerror, LONGJMPNOMEM);
 				fstore = (float *)store;
 				sstore = (INTSML *)store;
 			} else
 			{
 				store = emalloc(((len+1) * SIZEOFINTBIG), el_tempcluster);
-				if (store == 0) longjmp(io_filerror, (INTBIG)_("No memory"));
+				if (store == 0) longjmp(io_filerror, LONGJMPNOMEM);
 				fstore = (float *)store;
 				sstore = (INTSML *)store;
 				store[len] = -1;
@@ -966,7 +1006,7 @@ void io_getvar(void)
 			}
 			pt = io_keyword;
 			if (*pt++ != '[')
-				longjmp(io_filerror, (INTBIG)_("Missing '[' in variable array"));
+				longjmp(io_filerror, LONGJMPMISOSBVARARR);
 			for(j=0; j<len; j++)
 			{
 				/* string arrays must be handled specially */
@@ -994,7 +1034,7 @@ void io_getvar(void)
 					while (*pt != ',' && *pt != ']' && *pt != 0) pt++;
 				}
 				if (*pt == 0)
-					longjmp(io_filerror, (INTBIG)_("Short array specification"));
+					longjmp(io_filerror, LONGJMPSHORTARR);
 				*pt++ = 0;
 				if ((type&VTYPE) == VGENERAL)
 				{
@@ -1023,7 +1063,7 @@ void io_getvar(void)
 			{
 				nextchangequiet();
 				var = setvalkey(naddr, ntype, key, (INTBIG)store, type);
-				if (var == NOVARIABLE) longjmp(io_filerror, (INTBIG)_("No memory"));
+				if (var == NOVARIABLE) longjmp(io_filerror, LONGJMPNOMEM);
 				TDCOPY(var->textdescript, descript);
 
 				/* handle updating of technology caches */
@@ -1038,7 +1078,7 @@ void io_getvar(void)
 			{
 				nextchangequiet();
 				var = setvalkey(naddr, ntype, key, ret, type);
-				if (var == NOVARIABLE) longjmp(io_filerror, (INTBIG)_("No memory"));
+				if (var == NOVARIABLE) longjmp(io_filerror, LONGJMPNOMEM);
 				TDCOPY(var->textdescript, descript);
 
 				/* handle updating of technology caches */
@@ -1109,9 +1149,9 @@ INTBIG io_decode(char *name, INTBIG type)
 			/* parse primitive nodeproto name */
 			np = getnodeproto(name);
 			if (np == NONODEPROTO)
-				longjmp(io_filerror, (INTBIG)_("Unknown node prototype"));
+				longjmp(io_filerror, LONGJMPUKNNOPROTO);
 			if (np->primindex == 0)
-				longjmp(io_filerror, (INTBIG)_("Unknown node prototype"));
+				longjmp(io_filerror, LONGJMPUKNNOPROTO);
 			return((INTBIG)np);
 		case VPORTARCINST:
 			io_txtindata.portarcinsterror++;   break;
@@ -1168,12 +1208,12 @@ void io_libcc(void)
 	/* allocate a list of node prototypes for this library */
 	io_txtindata.nodeprotolist = (NODEPROTO **)emalloc(((sizeof (NODEPROTO *)) * io_txtindata.facetcount),
 		el_tempcluster);
-	if (io_txtindata.nodeprotolist == 0) longjmp(io_filerror, (INTBIG)_("No memory"));
+	if (io_txtindata.nodeprotolist == 0) longjmp(io_filerror, LONGJMPNOMEM);
 	for(i=0; i<io_txtindata.facetcount; i++)
 	{
 		io_txtindata.nodeprotolist[i] = allocnodeproto(io_txtindata.curlib->cluster);
 		if (io_txtindata.nodeprotolist[i] == NONODEPROTO)
-			longjmp(io_filerror, (INTBIG)_("No memory for facets"));
+			longjmp(io_filerror, LONGJMPNOMEM);
 		io_txtindata.nodeprotolist[i]->cellview = el_unknownview;
 		io_txtindata.nodeprotolist[i]->newestversion = io_txtindata.nodeprotolist[i];
 		io_txtindata.nodeprotolist[i]->primindex = 0;
@@ -1202,16 +1242,16 @@ void io_libvie(void)
 
 	pt = io_keyword;
 	while (*pt != 0 && *pt != '{') pt++;
-	if (*pt != '{') longjmp(io_filerror, (INTBIG)_("Missing view abbreviation"));
+	if (*pt != '{') longjmp(io_filerror, LONGJMPMISVIEWABR);
 	*pt++ = 0;
 	len = strlen(pt);
-	if (pt[len-1] != '}') longjmp(io_filerror, (INTBIG)_("View abbreviation bad"));
+	if (pt[len-1] != '}') longjmp(io_filerror, LONGJMPBADVIEWABR);
 	pt[len-1] = 0;
 	v = getview(io_keyword);
 	if (v == NOVIEW)
 	{
 		v = allocview();
-		if (v == NOVIEW) longjmp(io_filerror, (INTBIG)_("No memory"));
+		if (v == NOVIEW) longjmp(io_filerror, LONGJMPNOMEM);
 		(void)allocstring(&v->viewname, io_keyword, db_cluster);
 		(void)allocstring(&v->sviewname, pt, db_cluster);
 		if (namesamen(io_keyword, "Schematic-Page-", 15) == 0)
@@ -1221,7 +1261,7 @@ void io_libvie(void)
 	} else
 	{
 		if (namesame(v->sviewname, pt) != 0)
-			longjmp(io_filerror, (INTBIG)_("View abbreviation inconsistent"));
+			longjmp(io_filerror, LONGJMPINCVIEWABR);
 	}
 	pt[-1] = '{';   pt[len-1] = '}';
 }
@@ -1251,14 +1291,14 @@ void io_celnam(void)
 	REGISTER void *infstr;
 
 	if (io_txtindata.curnodeproto->cell != NOCELL)
-		longjmp(io_filerror, (INTBIG)_("Second facet name"));
+		longjmp(io_filerror, LONGJMPSECFACETNAME);
 	if (io_verbose != 0)
 	{
 		if (io_verbose < 0 && io_txtindata.filelength > 0)
 		{
 			infstr = initinfstr();
 			formatinfstr(infstr, _("Reading %s"), io_keyword);
-			DiaSetText(2, returninfstr(infstr));
+			DiaSetTextProgress(io_inputprogressdialog, returninfstr(infstr));
 		} else ttyputmsg(_("Reading %s"), io_keyword);
 	}
 	io_txtindata.curnodeproto->cellview = el_unknownview;
@@ -1269,7 +1309,7 @@ void io_celnam(void)
 		*pt = 0;
 		if (allocstring(&cellname, io_keyword,
 			io_txtindata.curlib->cluster))
-				longjmp(io_filerror, (INTBIG)_("No memory for facet name"));
+				longjmp(io_filerror, LONGJMPNOMEM);
 		pt++;
 		pt[strlen(pt)-1] = 0;
 		for(v = el_views; v != NOVIEW; v = v->nextview)
@@ -1279,13 +1319,13 @@ void io_celnam(void)
 	{
 		if (allocstring(&cellname, io_keyword,
 			io_txtindata.curlib->cluster))
-				longjmp(io_filerror, (INTBIG)_("No memory for facet name"));
+				longjmp(io_filerror, LONGJMPNOMEM);
 	}
 	cell = db_findcellname(cellname, io_txtindata.curlib);
 	if (cell == NOCELL)
 	{
 		cell = alloccell(io_txtindata.curlib->cluster);
-		if (cell == NOCELL) longjmp(io_filerror, (INTBIG)_("No memory for facets"));
+		if (cell == NOCELL) longjmp(io_filerror, LONGJMPNOMEM);
 		cell->lib = io_txtindata.curlib;
 		cell->cellname = cellname;
 	} else
@@ -1323,10 +1363,10 @@ void io_celrev(void)
  */
 void io_celext(void)
 {
-	INTBIG k, len, filetype, filelen;
+	INTBIG len, filetype, filelen;
 	REGISTER LIBRARY *elib;
 	REGISTER char *libname, *pt;
-	char *filename, *libfile, *oldline2, *oldline3, *libfilename, *libfilepath;
+	char *filename, *libfile, *oldline2, *libfilename, *libfilepath;
 	FILE *io;
 	REGISTER BOOLEAN failed;
 	REGISTER CELL *cell;
@@ -1403,8 +1443,7 @@ void io_celext(void)
 		savetxtindata = io_txtindata;
 		if (io_verbose < 0 && io_txtindata.filelength > 0)
 		{
-			(void)allocstring(&oldline2, DiaGetText(2), el_tempcluster);
-			(void)allocstring(&oldline3, DiaGetText(3), el_tempcluster);
+			(void)allocstring(&oldline2, DiaGetTextProgress(io_inputprogressdialog), el_tempcluster);
 		}
 
 		len = strlen(libfilename);
@@ -1422,18 +1461,18 @@ void io_celext(void)
 		io_libinputrecursivedepth--;
 		if (failed) elib->userbits |= UNWANTEDLIB; else
 		{
-			for(k=0; k<el_maxtools; k++)
-				if (el_tools[k].readlibrary != 0)
-					(*el_tools[k].readlibrary)(elib);
+			/* queue this library for announcement through change control */
+			io_queuereadlibraryannouncement(elib);
 		}
 		io_txtindata = savetxtindata;
 		if (io_verbose < 0 && io_txtindata.filelength > 0)
 		{
-			DiaPercent(1, io_txtindata.fileposition*100/io_txtindata.filelength);
-			DiaSetText(2, oldline2);
-			DiaSetText(3, oldline3);
+			DiaSetProgress(io_inputprogressdialog, io_txtindata.fileposition, io_txtindata.filelength);
+			infstr = initinfstr();
+			formatinfstr(infstr, _("Reading library %s"), io_txtindata.curlib->libname);
+			DiaSetCaptionProgress(io_inputprogressdialog, returninfstr(infstr));
+			DiaSetTextProgress(io_inputprogressdialog, oldline2);
 			efree(oldline2);
-			efree(oldline3);
 		}
 	}
 
@@ -1506,7 +1545,7 @@ void io_celext(void)
 		if (cell == NOCELL)
 		{
 			cell = alloccell(io_txtindata.curlib->cluster);
-			if (cell == NOCELL) longjmp(io_filerror, (INTBIG)_("No memory for facets"));
+			if (cell == NOCELL) longjmp(io_filerror, LONGJMPNOMEM);
 			cell->lib = io_txtindata.curlib;
 			(void)allocstring(&cell->cellname, cellname, io_txtindata.curlib->cluster);
 		}
@@ -1558,7 +1597,7 @@ void io_tech(void)
 	REGISTER TECHNOLOGY *tech;
 
 	tech = io_gettechnology(io_keyword);
-	if (tech == NOTECHNOLOGY) longjmp(io_filerror, (INTBIG)_("Unknown technology"));
+	if (tech == NOTECHNOLOGY) longjmp(io_filerror, LONGJMPUKNTECH);
 	io_txtindata.curtech = tech;
 }
 
@@ -1606,15 +1645,15 @@ void io_celnoc(void)
 	if (io_txtindata.nodeinstcount == 0) return;
 	io_txtindata.nodelist = (NODEINST **)emalloc(((sizeof (NODEINST *)) * io_txtindata.nodeinstcount),
 		el_tempcluster);
-	if (io_txtindata.nodelist == 0) longjmp(io_filerror, (INTBIG)_("No memory for nodeinsts"));
+	if (io_txtindata.nodelist == 0) longjmp(io_filerror, LONGJMPNOMEM);
 	geomlist = (GEOM **)emalloc(((sizeof (GEOM *)) * io_txtindata.nodeinstcount), el_tempcluster);
-	if (geomlist == 0) longjmp(io_filerror, (INTBIG)_("No memory for nodeinsts"));
+	if (geomlist == 0) longjmp(io_filerror, LONGJMPNOMEM);
 	for(i=0; i<io_txtindata.nodeinstcount; i++)
 	{
 		ni = allocnodeinst(io_txtindata.curlib->cluster);
 		geomlist[i] = allocgeom(io_txtindata.curlib->cluster);
 		if (ni == NONODEINST || geomlist[i] == NOGEOM)
-			longjmp(io_filerror, (INTBIG)_("No memory for nodeinsts"));
+			longjmp(io_filerror, LONGJMPNOMEM);
 		io_txtindata.nodelist[i] = ni;
 		ni->parent = io_txtindata.curnodeproto;
 		ni->firstportarcinst = NOPORTARCINST;
@@ -1644,15 +1683,15 @@ void io_celarc(void)
 	if (io_txtindata.arcinstcount == 0) return;
 	io_txtindata.arclist = (ARCINST **)emalloc(((sizeof (ARCINST *)) * io_txtindata.arcinstcount),
 		el_tempcluster);
-	if (io_txtindata.arclist == 0) longjmp(io_filerror, (INTBIG)_("No memory for arcinsts"));
+	if (io_txtindata.arclist == 0) longjmp(io_filerror, LONGJMPNOMEM);
 	geomlist = (GEOM **)emalloc(((sizeof (GEOM *)) * io_txtindata.arcinstcount), el_tempcluster);
-	if (geomlist == 0) longjmp(io_filerror, (INTBIG)_("No memory for arcinsts"));
+	if (geomlist == 0) longjmp(io_filerror, LONGJMPNOMEM);
 	for(i=0; i<io_txtindata.arcinstcount; i++)
 	{
 		ai = allocarcinst(io_txtindata.curlib->cluster);
 		geomlist[i] = allocgeom(io_txtindata.curlib->cluster);
 		if (ai == NOARCINST || geomlist[i] == NOGEOM)
-			longjmp(io_filerror, (INTBIG)_("No memory for arcinsts"));
+			longjmp(io_filerror, LONGJMPNOMEM);
 		io_txtindata.arclist[i] = ai;
 		ai->parent = io_txtindata.curnodeproto;
 		ai->geom = geomlist[i];
@@ -1675,15 +1714,16 @@ void io_celptc(void)
 	REGISTER INTBIG i;
 
 	io_txtindata.portprotocount = atoi(io_keyword);
+	io_txtindata.curnodeproto->numportprotos = io_txtindata.portprotocount;
 	if (io_txtindata.portprotocount == 0) return;
-	io_txtindata.portprotolist = (PORTPROTO **)emalloc(((sizeof (PORTPROTO *)) * io_txtindata.portprotocount),
-		el_tempcluster);
-	if (io_txtindata.portprotolist == 0) longjmp(io_filerror, (INTBIG)_("No memory for port prototypes"));
+	io_txtindata.portprotolist = (PORTPROTO **)emalloc(((sizeof (PORTPROTO *)) *
+		io_txtindata.portprotocount), el_tempcluster);
+	if (io_txtindata.portprotolist == 0) longjmp(io_filerror, LONGJMPNOMEM);
 	for(i=0; i<io_txtindata.portprotocount; i++)
 	{
 		io_txtindata.portprotolist[i] = allocportproto(io_txtindata.curlib->cluster);
 		if (io_txtindata.portprotolist[i] == NOPORTPROTO)
-			longjmp(io_filerror, (INTBIG)_("No memory for port prototypes"));
+			longjmp(io_filerror, LONGJMPNOMEM);
 		io_txtindata.portprotolist[i]->parent = io_txtindata.curnodeproto;
 	}
 
@@ -1734,7 +1774,7 @@ void io_celdon(void)
 
 	/* create geometry structure in the facet */
 	if (geomstructure(io_txtindata.curnodeproto))
-		longjmp(io_filerror, (INTBIG)_("No memory for geometry structure"));
+		longjmp(io_filerror, LONGJMPNOMEM);
 
 	/* fill geometry module for each nodeinst */
 	for(i=0; i<io_txtindata.nodeinstcount; i++)
@@ -1795,7 +1835,7 @@ void io_nodtyp(void)
 		{
 			*pt = 0;
 			tech = io_gettechnology(line);
-			if (tech == NOTECHNOLOGY) longjmp(io_filerror, (INTBIG)_("Unknown technology"));
+			if (tech == NOTECHNOLOGY) longjmp(io_filerror, LONGJMPUKNTECH);
 			*pt++ = ':';
 			line = pt;
 		} else tech = io_txtindata.curtech;
@@ -1831,7 +1871,7 @@ void io_nodtyp(void)
 		}
 		if (io_txtindata.curnodeinst->proto != NONODEPROTO) return;
 
-		longjmp(io_filerror, (INTBIG)_("Unknown node prototype"));
+		longjmp(io_filerror, LONGJMPUKNNOPROTO);
 	}
 }
 
@@ -1980,7 +2020,7 @@ void io_porarc(void)
 	REGISTER PORTARCINST *pi;
 
 	pi = allocportarcinst(io_txtindata.curlib->cluster);
-	if (pi == NOPORTARCINST) longjmp(io_filerror, (INTBIG)_("No memory"));
+	if (pi == NOPORTARCINST) longjmp(io_filerror, LONGJMPNOMEM);
 	pi->proto = io_txtindata.curportproto;
 	db_addportarcinst(io_txtindata.curnodeinst, pi);
 	pi->conarcinst = io_txtindata.arclist[atoi(io_keyword)];
@@ -1996,7 +2036,7 @@ void io_porexp(void)
 	REGISTER PORTEXPINST *pe;
 
 	pe = allocportexpinst(io_txtindata.curlib->cluster);
-	if (pe == NOPORTEXPINST) longjmp(io_filerror, (INTBIG)_("No memory"));
+	if (pe == NOPORTEXPINST) longjmp(io_filerror, LONGJMPNOMEM);
 	pe->proto = io_txtindata.curportproto;
 	db_addportexpinst(io_txtindata.curnodeinst, pe);
 	pe->exportproto = io_txtindata.portprotolist[atoi(io_keyword)];
@@ -2031,7 +2071,7 @@ void io_arctyp(void)
 	{
 		*pt = 0;
 		tech = io_gettechnology(line);
-		if (tech == NOTECHNOLOGY) longjmp(io_filerror, (INTBIG)_("Unknown technology"));
+		if (tech == NOTECHNOLOGY) longjmp(io_filerror, LONGJMPUKNTECH);
 		*pt++ = ':';
 		line = pt;
 	} else tech = io_txtindata.curtech;
@@ -2071,7 +2111,7 @@ void io_arctyp(void)
 		return;
 	}
 
-	longjmp(io_filerror, (INTBIG)_("Invalid arc prototype"));
+	longjmp(io_filerror, LONGJMPINVARCPROTO);
 }
 
 /*
@@ -2130,7 +2170,7 @@ void io_endpt(void)
 	REGISTER PORTPROTO *pp;
 
 	pp = io_getport(io_keyword,io_txtindata.curarcinst->end[io_txtindata.curarcend].nodeinst->proto);
-	if (pp == NOPORTPROTO) longjmp(io_filerror, (INTBIG)_("Unknown port prototype name"));
+	if (pp == NOPORTPROTO) longjmp(io_filerror, LONGJMPUKNPORPROTO);
 	io_txtindata.curarcinst->end[io_txtindata.curarcend].portarcinst = (PORTARCINST *)pp;
 }
 
@@ -2195,7 +2235,7 @@ void io_newpt(void)
 void io_ptnam(void)
 {
 	if (allocstring(&io_txtindata.curportproto->protoname, io_keyword, io_txtindata.curlib->cluster))
-		longjmp(io_filerror, (INTBIG)_("No memory for portproto name"));
+		longjmp(io_filerror, LONGJMPNOMEM);
 }
 
 /*
@@ -2232,7 +2272,7 @@ void io_ptspt(void)
 	REGISTER PORTPROTO *pp;
 
 	pp = io_getport(io_keyword, io_txtindata.curportproto->subnodeinst->proto);
-	if (pp == NOPORTPROTO) longjmp(io_filerror, (INTBIG)_("Unknown port prototype name"));
+	if (pp == NOPORTPROTO) longjmp(io_filerror, LONGJMPUKNPORPROTO);
 	io_txtindata.curportproto->subportproto = pp;
 }
 
