@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: iovhdl.c
@@ -40,7 +41,7 @@
 static INTBIG io_vhdllength;
 static INTBIG io_vhdlcurline;
 
-static void io_vhdlreadfile(LIBRARY *lib, FILE *io);
+static void io_vhdlreadfile(LIBRARY *lib, FILE *io, void *dia);
 
 /*
  * Routine to read the VHDL file into library "lib".  Returns true on error.
@@ -49,7 +50,7 @@ BOOLEAN io_readvhdllibrary(LIBRARY *lib)
 {
 	REGISTER FILE *io;
 	char *pt;
-	extern DIALOG us_progressdialog;
+	void *dia;
 
 	/* get the VHDL file */
 	if ((io = xopen(lib->libfile, io_filetypevhdl, "", &pt)) == NULL)
@@ -62,26 +63,27 @@ BOOLEAN io_readvhdllibrary(LIBRARY *lib)
 	io_vhdllength = filesize(io);
 	if (io_verbose < 0 && io_vhdllength > 0)
 	{
-		if (DiaInitDialog(&us_progressdialog))
+		dia = DiaInitProgress(0, 0);
+		if (dia == 0)
 		{
 			xclose(io);
 			return(TRUE);
 		}
-		DiaPercent(1, 0);
-	}
+		DiaSetProgress(dia, 0, io_vhdllength);
+	} else dia = 0;
 
-	io_vhdlreadfile(lib, io);
+	io_vhdlreadfile(lib, io, dia);
 
 	xclose(io);
 	if (io_verbose < 0 && io_vhdllength > 0)
 	{
-		DiaPercent(1, 100);
-		DiaDoneDialog();
+		DiaSetProgress(dia, io_vhdllength, io_vhdllength);
+		DiaDoneProgress(dia);
 	}
 	return(FALSE);
 }
 
-void io_vhdlreadfile(LIBRARY *lib, FILE *io)
+void io_vhdlreadfile(LIBRARY *lib, FILE *io, void *dia)
 {
 	char *pt, *start, text[MAXCHARS], facetname[256], endtag[256];
 	REGISTER char save;
@@ -102,7 +104,7 @@ void io_vhdlreadfile(LIBRARY *lib, FILE *io)
 		if ((io_vhdlcurline % 100) == 0)
 		{
 			curLength = xtell(io);
-			DiaPercent(1, curLength * 100L / io_vhdllength);
+			DiaSetProgress(dia, curLength, io_vhdllength);
 		}
 
 		if (storagefacet == NONODEPROTO)
