@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: compact.c
@@ -279,7 +280,7 @@ void com_slice(void)
 	if ((net_tool->toolstate&TOOLON) == 0)
 	{
 		ttyputerr(_("Network tool must be running...turning it on"));
-		toolturnon(net_tool, FALSE);
+		toolturnon(net_tool);
 		return;
 	}
 
@@ -351,7 +352,7 @@ void com_set(INTBIG count, char *par[])
 		if ((net_tool->toolstate&TOOLON) == 0)
 		{
 			ttyputerr(M_("Network tool must be running...turning it on"));
-			toolturnon(net_tool, FALSE);
+			toolturnon(net_tool);
 			ttyputerr(M_("...now reissue the compaction command"));
 			return;
 		}
@@ -981,7 +982,7 @@ BOOLEAN com_lineup_firstrow(LINE *line, LINE *other_line, INTBIG axis, INTBIG lo
 	if (i > 0)
 	{
 		if (com_debug)
-			ttyputmsg(M_("Lining up the first row to low edge %s"), latoa(lowbound));
+			ttyputmsg(M_("Lining up the first row to low edge %s"), latoa(lowbound, 0));
 		if (axis == HORIZONTAL) change = com_move(line, i, 0, change); else
 			change = com_move(line, 0, i, change);
 		com_fixed_nonfixed(line, other_line);
@@ -1082,7 +1083,7 @@ BOOLEAN com_compact(LINE *line, LINE *other_line, INTBIG axis, BOOLEAN change,
 		if (com_debug)
 		{
 			ttyputmsg(M_("Moving object '%s' by %s"), com_describeline(cur_line),
-				latoa(best_motion));
+				latoa(best_motion, 0));
 			if (thisoreason != NOOBJECT && otheroreason != NOOBJECT)
 			{
 				infstr = initinfstr();
@@ -1797,7 +1798,7 @@ static DIALOGITEM com_optionsdialogitems[] =
  /*  3 */ {0, {8,8,24,154}, CHECK, N_("Allow spreading")},
  /*  4 */ {0, {32,8,48,154}, CHECK, N_("Verbose")}
 };
-static DIALOG com_optionsdialog = {{50,75,147,243}, N_("Compaction Options"), 0, 4, com_optionsdialogitems};
+static DIALOG com_optionsdialog = {{50,75,147,243}, N_("Compaction Options"), 0, 4, com_optionsdialogitems, 0, 0};
 
 /* special items for the "Compaction Options" dialog: */
 #define DCMO_CANSPREAD   3		/* Allow spreading (check) */
@@ -1807,35 +1808,37 @@ void comp_optionsdlog(void)
 {
 	REGISTER INTBIG itemHit, spread, origspread;
 	REGISTER VARIABLE *var;
+	REGISTER void *dia;
 
-	if (DiaInitDialog(&com_optionsdialog)) return;
+	dia = DiaInitDialog(&com_optionsdialog);
+	if (dia == 0) return;
 	var = getvalkey((INTBIG)com_tool, VTOOL, VINTEGER, com_spread_key);
 	if (var == NOVARIABLE) origspread = 0; else origspread = var->addr;
-	if (origspread != 0) DiaSetControl(DCMO_CANSPREAD, 1);
-	if (com_debug) DiaSetControl(DCMO_VERBOSE, 1);
+	if (origspread != 0) DiaSetControl(dia, DCMO_CANSPREAD, 1);
+	if (com_debug) DiaSetControl(dia, DCMO_VERBOSE, 1);
 
 	/* loop until done */
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == OK || itemHit == CANCEL) break;
 		if (itemHit == DCMO_CANSPREAD || itemHit == DCMO_VERBOSE)
 		{
-			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
+			DiaSetControl(dia, itemHit, 1 - DiaGetControl(dia, itemHit));
 			continue;
 		}
 	}
 
 	if (itemHit != CANCEL)
 	{
-		if (DiaGetControl(DCMO_CANSPREAD) != 0) spread = 1; else
+		if (DiaGetControl(dia, DCMO_CANSPREAD) != 0) spread = 1; else
 			spread = 0;
 		if (spread != origspread)
 			(void)setvalkey((INTBIG)com_tool, VTOOL, com_spread_key, spread, VINTEGER);
-		if (DiaGetControl(DCMO_VERBOSE) != 0) com_debug = TRUE; else
+		if (DiaGetControl(dia, DCMO_VERBOSE) != 0) com_debug = TRUE; else
 			com_debug = FALSE;
 	}
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 }
 
 #endif  /* COMTOOL - at top */
