@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: erc.c
@@ -41,7 +42,8 @@
 /* the ERC tool table */
 static KEYWORD ercopt[] =
 {
-	{"analyze",        0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"well-check",    0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"antenna-rules", 0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	TERMKEY
 };
 COMCOMP erc_tablep = {ercopt, NOTOPLIST, NONEXTLIST, NOPARAMS,
@@ -68,6 +70,7 @@ static INTBIG      erc_worstpwelledgex, erc_worstpwelledgey;
 static INTBIG      erc_worstnwelldist;
 static INTBIG      erc_worstnwellconx, erc_worstnwellcony;
 static INTBIG      erc_worstnwelledgex, erc_worstnwelledgey;
+static void       *erc_progressdialog;
 
 /* well areas */
 #define NOWELLAREA ((WELLAREA *)-1)
@@ -93,7 +96,6 @@ static WELLAREA *erc_firstwellarea = NOWELLAREA;
        TOOL       *erc_tool;					/* this tool */
 static NODEPROTO  *erc_facet;
 static INTBIG      erc_globalnetnumber;			/* global network number */
-static INTBIG      erc_attachednetnumber;		/* attached network number (above this floats) */
 static INTBIG      erc_optionskey;				/* key for "ERC_options" */
 static INTBIG      erc_options;					/* value of "ERC_options" */
 static INTBIG      erc_jobsize;					/* number of polygons to analyze */
@@ -106,7 +108,7 @@ static WELLCON     *erc_allocwellcon(void);
 static void         erc_analyzefacet(NODEPROTO *facet);
 static void         erc_freeallwellareas(void);
 static void         erc_freeallwellcons(void);
-static void         erc_gatherwells(NODEPROTO *np, XARRAY trans, void *merge);
+static void         erc_gatherwells(NODEPROTO *np, XARRAY trans, void *merge, BOOLEAN gathernet);
 static void         erc_getpolygon(INTBIG layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INTBIG count);
 static void         erc_optionsdlog(void);
 static INTBIG       erc_welllayer(TECHNOLOGY *tech, INTBIG layer);
@@ -118,6 +120,8 @@ static INTBIG       erc_welllayer(TECHNOLOGY *tech, INTBIG layer);
  */
 void erc_init(INTBIG *argc, char *argv[], TOOL *thistool)
 {
+	extern COMCOMP us_debugcp;
+
 	/* only initialize during pass 1 */
 	if (thistool == NOTOOL || thistool == 0) return;
 
@@ -127,6 +131,8 @@ void erc_init(INTBIG *argc, char *argv[], TOOL *thistool)
 	nextchangequiet();
 	setvalkey((INTBIG)erc_tool, VTOOL, erc_optionskey, DEFERCBITS, VINTEGER|VDONTSAVE);
 	DiaDeclareHook("ercopt", &erc_tablep, erc_optionsdlog);
+	DiaDeclareHook("ercantennaopt", &us_debugcp, erc_antoptionsdlog);
+	erc_antennaarcratiokey = makekey("ERC_antenna_arc_ratio");
 }
 
 void erc_done(void)
@@ -134,6 +140,7 @@ void erc_done(void)
 #ifdef DEBUGMEMORY
 	erc_freeallwellcons();
 	erc_freeallwellareas();
+	erc_antterm();
 #endif
 }
 
@@ -154,14 +161,22 @@ void erc_set(INTBIG count, char *par[])
 
 	l = strlen(pp = par[0]);
 
-	if (namesamen(pp, "analyze", l) == 0)
+	if (namesamen(pp, "well-check", l) == 0 && l >= 3)
 	{
-		/* analyze facet */
+		/* check well and substrate layers */
 		np = getcurfacet();
 		erc_analyzefacet(np);
 		return;
 	}
 
+	if (namesamen(pp, "antenna-rules", l) == 0 && l >= 3)
+	{
+		/* do antenna-rules check */
+		np = getcurfacet();
+		erc_antcheckfacet(np);
+		return;
+	}
+
 	ttyputbadusage("telltool erc");
 }
 
@@ -173,9 +188,10 @@ void erc_set(INTBIG count, char *par[])
 void erc_analyzefacet(NODEPROTO *facet)
 {
 	REGISTER INTBIG errors, minsep, dist, layertype, largestnetnum, welltype,
-		desiredcontact, perareabit;
+		desiredcontact, perareabit, lambda;
 	REGISTER NODEPROTO *np;
 	REGISTER TECHNOLOGY *tech;
+	REGISTER LIBRARY *lib;
 	REGISTER BOOLEAN con;
 	REGISTER char *abortseq, *errormsg, *nocontacterror;
 	char errmsg[200];
@@ -186,7 +202,6 @@ void erc_analyzefacet(NODEPROTO *facet)
 	REGISTER void *err;
 	float elapsed;
 	INTBIG edge;
-	extern DIALOG us_progressdialog;
 	REGISTER void *infstr;
 	REGISTER void *polymerge, *submerge;
 
@@ -203,7 +218,7 @@ void erc_analyzefacet(NODEPROTO *facet)
 	{
 		termerrorlogging(TRUE);
 		ttyputerr(_("Network tool must be running...turning it on"));
-		toolturnon(net_tool, FALSE);
+		toolturnon(net_tool);
 		return;
 	}
 
@@ -214,13 +229,13 @@ void erc_analyzefacet(NODEPROTO *facet)
 	if (abortseq != 0)
 		formatinfstr(infstr, _(" (type %s to abort)"), abortseq);
 	ttyputmsg(returninfstr(infstr));
-	if (DiaInitDialog(&us_progressdialog))
+	erc_progressdialog = DiaInitProgress(_("Preparing to analyze..."), 0);
+	if (erc_progressdialog == 0)
 	{
 		termerrorlogging(TRUE);
 		return;
 	}
-	DiaPercent(1, 0);
-	DiaSetText(2, _("Preparing to analyze..."));
+	DiaSetProgress(erc_progressdialog, 0, 1);
 
 	/* begin counting elapsed time */
 	starttimer();
@@ -236,7 +251,6 @@ void erc_analyzefacet(NODEPROTO *facet)
 
 	/* put global network information on the ports of this facet */
 	erc_globalnetnumber = 1;
-	erc_attachednetnumber = 0;
 	for(pp = facet->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		pp->temp1 = 0;
 	for(pp = facet->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
@@ -251,27 +265,27 @@ void erc_analyzefacet(NODEPROTO *facet)
 	}
 
 	/* reset count of primitives and merging information */
-	for(tech = el_technologies; tech != NOTECHNOLOGY; tech = tech->nexttechnology)
-		for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 			np->temp1 = np->temp2 = 0;
 
 	/* run through the work and count the number of polygons */
 	erc_polyscrunched = 0;
 	erc_realrun = FALSE;
-	erc_gatherwells(facet, el_matid, 0);
+	erc_gatherwells(facet, el_matid, 0, FALSE);
 	erc_jobsize = erc_polyscrunched;
 
 	/* now gather all of the geometry into the polygon merging system */
-	DiaSetText(2, _("Gathering geometry..."));
+	DiaSetTextProgress(erc_progressdialog, _("Gathering geometry..."));
 	polymerge = mergenew(erc_tool->cluster);
 	erc_realrun = TRUE;
 	erc_polyscrunched = 0;
-	erc_gatherwells(facet, el_matid, polymerge);
+	erc_gatherwells(facet, el_matid, polymerge, TRUE);
 
 	/* delete intermediate merge information */
-	for(tech = el_technologies; tech != NOTECHNOLOGY; tech = tech->nexttechnology)
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
 	{
-		for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 		{
 			if (np->temp2 == 0) continue;
 			submerge = (void *)np->temp2;
@@ -286,12 +300,12 @@ void erc_analyzefacet(NODEPROTO *facet)
 	erc_jobsize = erc_polyscrunched;
 
 	/* do the analysis */
-	DiaSetText(2, _("Analyzing..."));
+	DiaSetTextProgress(erc_progressdialog, _("Analyzing..."));
 	erc_realrun = TRUE;
 	erc_polyscrunched = 0;
 	mergeextract(polymerge, erc_getpolygon);
 	mergedelete(polymerge);
-	DiaPercent(1, 100);
+	DiaSetProgress(erc_progressdialog, 100, erc_jobsize);
 
 	/* number the well areas according to topology of contacts in them */
 	if (erc_firstwellcon == NOWELLCON) largestnetnum = 0; else
@@ -343,7 +357,7 @@ void erc_analyzefacet(NODEPROTO *facet)
 	}
 
 	/* make sure all of the contacts are on the same net */
-	DiaSetText(2, _("Reporting errors..."));
+	DiaSetTextProgress(erc_progressdialog, _("Reporting errors..."));
 	for(wc = erc_firstwellcon; wc != NOWELLCON; wc = wc->nextwellcon)
 	{
 		if (wc->netnum == 0)
@@ -426,23 +440,24 @@ void erc_analyzefacet(NODEPROTO *facet)
 			{
 				layertype = erc_welllayer(wa->tech, wa->layer);
 				if (layertype == 0) continue;
+				lambda = lambdaoffacet(facet);
 				switch (layertype)
 				{
 					case 1:
 						sprintf(errmsg, _("P-Well areas too close (are %s, should be %s)"),
-							latoa(dist), latoa(minsep));
+							latoa(dist, lambda), latoa(minsep, lambda));
 						break;
 					case 2:
 						sprintf(errmsg, _("N-Well areas too close (are %s, should be %s)"),
-							latoa(dist), latoa(minsep));
+							latoa(dist, lambda), latoa(minsep, lambda));
 						break;
 					case 3:
 						sprintf(errmsg, _("P-Select areas too close (are %s, should be %s)"),
-							latoa(dist), latoa(minsep));
+							latoa(dist, lambda), latoa(minsep, lambda));
 						break;
 					case 4:
 						sprintf(errmsg, _("N-Select areas too close (are %s, should be %s)"),
-							latoa(dist), latoa(minsep));
+							latoa(dist, lambda), latoa(minsep, lambda));
 						break;
 				}
 				err = logerror(errmsg, erc_facet, 0);
@@ -484,14 +499,14 @@ void erc_analyzefacet(NODEPROTO *facet)
 			(void)asktool(us_tool, "show-line", erc_worstpwellconx, erc_worstpwellcony,
 				erc_worstpwelledgex, erc_worstpwelledgey, erc_facet);
 			ttyputmsg(_("Farthest distance from a P-Well contact is %s"),
-				latoa(erc_worstpwelldist));
+				latoa(erc_worstpwelldist, lambda));
 		}
 		if (erc_worstnwelldist > 0)
 		{
 			(void)asktool(us_tool, "show-line", erc_worstnwellconx, erc_worstnwellcony,
 				erc_worstnwelledgex, erc_worstnwelledgey, erc_facet);
 			ttyputmsg(_("Farthest distance from a N-Well contact is %s"),
-				latoa(erc_worstnwelldist));
+				latoa(erc_worstnwelldist, lambda));
 		}
 	}
 
@@ -521,13 +536,13 @@ void erc_analyzefacet(NODEPROTO *facet)
 	}
 
 	/* report the number of networks */
-	DiaDoneDialog();
+	DiaDoneProgress(erc_progressdialog);
 	ttyputmsg(_("%ld networks found"), erc_globalnetnumber);
 
 	/* stop counting time */
 	elapsed = endtimer();
 	if (elapsed > 60.0 && (us_useroptions&BEEPAFTERLONGJOB) != 0)
-		ttybeep(1);
+		ttybeep(SOUNDBEEP, TRUE);
 
 	/* report the number of errors found */
 	errors = numerrors();
@@ -544,9 +559,10 @@ void erc_analyzefacet(NODEPROTO *facet)
 
 /*
  * Recursive helper routine to gather all well areas in facet "np"
- * (transformed by "trans").
+ * (transformed by "trans").  If "merge" is nonzero, merge geometry into it.
+ * If "gathernet" is true, follow networks, looking for special primitives and nets. 
  */
-void erc_gatherwells(NODEPROTO *np, XARRAY trans, void *merge)
+void erc_gatherwells(NODEPROTO *np, XARRAY trans, void *merge, BOOLEAN gathernet)
 {
 	REGISTER NODEINST *ni;
 	REGISTER NODEPROTO *subnp;
@@ -555,41 +571,78 @@ void erc_gatherwells(NODEPROTO *np, XARRAY trans, void *merge)
 	REGISTER PORTEXPINST *pe;
 	REGISTER PORTPROTO *pp, *opp;
 	REGISTER NETWORK *net;
-	REGISTER INTBIG i, tot, nfun;
+	REGISTER GEOM *geom;
+	REGISTER INTBIG i, tot, nfun, updatepct, sea;
 	static INTBIG checkstop = 0;
 	XARRAY localrot, localtran, final, subrot;
 	static POLYGON *poly = NOPOLYGON;
 	REGISTER WELLCON *wc;
+
 	(void)needstaticpolygon(&poly, 4, erc_tool->cluster);
 
 	/* grab global network information from the ports */
 	if (erc_realrun)
 	{
-		for(net = np->firstnetwork; net != NONETWORK; net = net->nextnetwork)
-			net->temp1 = 0;
-		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-			pp->network->temp1 = pp->temp1;
-		for(net = np->firstnetwork; net != NONETWORK; net = net->nextnetwork)
+		if (gathernet)
 		{
-			if (net->temp1 == 0) net->temp1 = erc_globalnetnumber++;
+			for(net = np->firstnetwork; net != NONETWORK; net = net->nextnetwork)
+				net->temp1 = 0;
+			for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+				pp->network->temp1 = pp->temp1;
+			for(net = np->firstnetwork; net != NONETWORK; net = net->nextnetwork)
+			{
+				if (net->temp1 == 0) net->temp1 = erc_globalnetnumber++;
+			}
 		}
-		if (erc_attachednetnumber == 0) erc_attachednetnumber = erc_globalnetnumber;
-		DiaPercent(1, erc_polyscrunched * 100 / erc_jobsize);
 	}
 
-	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+	/* do a spatial search so that polygon merging grows sensibly */
+	updatepct = 0;
+	sea = initsearch(np->lowx, np->highx, np->lowy, np->highy, np);
+	for(;;)
 	{
-		subnp = ni->proto;
-		if (erc_realrun)
+		geom = nextobject(sea);
+		if (geom == NOGEOM) break;
+		checkstop++;
+		if ((checkstop%10) == 0)
 		{
-			makerot(ni, localrot);
-			transmult(localrot, trans, final);
+			if (stopping(STOPREASONERC)) { termsearch(sea);   return; }
 		}
+		if (!geom->entryisnode)
+		{
+			ai = geom->entryaddr.ai;
+			tot = arcpolys(ai, NOWINDOWPART);
+			erc_polyscrunched += tot;
+			if (erc_realrun && merge != 0)
+			{
+				if (erc_jobsize > 0)
+				{
+					if (updatepct++ > 25)
+					{
+						DiaSetProgress(erc_progressdialog, erc_polyscrunched, erc_jobsize);
+						updatepct = 0;
+					}
+				}
+				for(i=0; i<tot; i++)
+				{
+					shapearcpoly(ai, i, poly);
+					if (erc_welllayer(poly->tech, poly->layer) == 0) continue;
+					xformpoly(poly, trans);
+					mergeaddpolygon(merge, poly->layer, poly->tech, poly);
+				}
+			}
+			continue;
+		}
+
+		ni = geom->entryaddr.ni;
+		subnp = ni->proto;
+		makerot(ni, localrot);
+		transmult(localrot, trans, final);
 		if (subnp->primindex == 0)
 		{
-			if (erc_realrun)
+			if (erc_realrun && gathernet)
 			{
-				if (stopping(STOPREASONERC)) return;
+				if (stopping(STOPREASONERC)) { termsearch(sea);   return; }
 
 				/* propagate global network numbers into this instance */
 				for(pp = subnp->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
@@ -606,44 +659,44 @@ void erc_gatherwells(NODEPROTO *np, XARRAY trans, void *merge)
 					if (opp != NOPORTPROTO) pp->temp1 = opp->temp1; else
 						pp->temp1 = erc_globalnetnumber++;
 				}
-
-				maketrans(ni, localtran);
-				transmult(localtran, final, subrot);
 			}
-#if 0		/* recursive polygon merge may be faster, but the merging can't handle the complexity */
-			if (subnp->temp2 == 0)
-			{
-				/* gather the subfacet's merged information */
-				if (erc_realrun)
-				{
-					void *submerge;
+			maketrans(ni, localtran);
+			transmult(localtran, final, subrot);
 
-					submerge = mergenew(erc_tool->cluster);
-					erc_gatherwells(subnp, el_matid, submerge);
+#if 1		/* recursive polygon merge is faster */
+			if (merge != 0)
+			{
+				void *submerge;
+
+				submerge = (void *)subnp->temp2;
+				if (submerge == 0)
+				{
+					/* gather the subfacet's merged information */
+					if (erc_realrun) submerge = mergenew(erc_tool->cluster); else
+						submerge = (void *)1;
+					erc_gatherwells(subnp, el_matid, submerge, FALSE);
 					subnp->temp2 = (INTBIG)submerge;
 				}
+
+				if (erc_realrun)
+					mergeaddmerge(merge, submerge, subrot);
 			}
-			if (erc_realrun)
-				mergeaddmerge(merge, (void *)subnp->temp2, subrot);
+			if (gathernet)
+				erc_gatherwells(subnp, subrot, 0, TRUE);
 #else
-			erc_gatherwells(subnp, subrot, merge);
+			erc_gatherwells(subnp, subrot, merge, TRUE);
 #endif
 		} else
 		{
-			if (erc_realrun)
+			if (erc_realrun && gathernet)
 			{
-				checkstop++;
-				if ((checkstop%10) == 0)
-				{
-					if (stopping(STOPREASONERC)) return;
-				}
 				nfun = nodefunction(ni);
-				ni->proto->temp1++;
+				/* ? */ ni->proto->temp1++;
 
 				if (nfun == NPWELL || nfun == NPSUBSTRATE)
 				{
 					wc = erc_allocwellcon();
-					if (wc == NOWELLCON) return;
+					if (wc == NOWELLCON) { termsearch(sea);   return; }
 					xform((ni->lowx + ni->highx) / 2, (ni->lowy + ni->highy) / 2,
 						&wc->x, &wc->y, final);
 					wc->np = subnp;
@@ -686,8 +739,16 @@ void erc_gatherwells(NODEPROTO *np, XARRAY trans, void *merge)
 
 			tot = nodepolys(ni, 0, NOWINDOWPART);
 			erc_polyscrunched += tot;
-			if (erc_realrun)
+			if (erc_realrun && merge != 0)
 			{
+				if (erc_jobsize > 0)
+				{
+					if (updatepct++ > 25)
+					{
+						DiaSetProgress(erc_progressdialog, erc_polyscrunched, erc_jobsize);
+						updatepct = 0;
+					}
+				}
 				for(i=0; i<tot; i++)
 				{
 					shapenodepoly(ni, i, poly);
@@ -698,27 +759,6 @@ void erc_gatherwells(NODEPROTO *np, XARRAY trans, void *merge)
 			}
 		}
 	}
-
-	for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
-	{
-		checkstop++;
-		if ((checkstop%10) == 0)
-		{
-			if (stopping(STOPREASONERC)) return;
-		}
-		tot = arcpolys(ai, NOWINDOWPART);
-		erc_polyscrunched += tot;
-		if (erc_realrun)
-		{
-			for(i=0; i<tot; i++)
-			{
-				shapearcpoly(ai, i, poly);
-				if (erc_welllayer(poly->tech, poly->layer) == 0) continue;
-				xformpoly(poly, trans);
-				mergeaddpolygon(merge, poly->layer, poly->tech, poly);
-			}
-		}
-	}
 }
 
 /*
@@ -744,7 +784,7 @@ void erc_getpolygon(INTBIG layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INTBIG
 	{
 		if (el_pleasestop != 0) return;
 	}
-	DiaPercent(1, erc_polyscrunched * 100 / erc_jobsize);
+	DiaSetProgress(erc_progressdialog, erc_polyscrunched, erc_jobsize);
 
 	/* save the polygon */
 	wa = erc_allocwellarea();
@@ -768,6 +808,9 @@ void erc_getpolygon(INTBIG layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INTBIG
 	if (welltype == 1) desiredcontact = NPWELL; else
 		desiredcontact = NPSUBSTRATE;
 
+	/* stop now if distance to edge not desired */
+	if ((erc_options&FINDEDGEDIST) == 0) return;
+
 	/* find the worst distance to the edge of the area */
 	for(i=0; i<count*2; i++)
 	{
@@ -914,11 +957,11 @@ WELLAREA *erc_allocwellarea(void)
 
 /******************** OPTIONS DIALOG ********************/
 
-/* ERC: Options Dialog */
+/* Well Check Options Dialog */
 static DIALOGITEM erc_optionsdialogitems[] =
 {
- /*  1 */ {0, {132,396,156,460}, BUTTON, N_("OK")},
- /*  2 */ {0, {132,296,156,360}, BUTTON, N_("Cancel")},
+ /*  1 */ {0, {156,296,180,360}, BUTTON, N_("OK")},
+ /*  2 */ {0, {156,108,180,172}, BUTTON, N_("Cancel")},
  /*  3 */ {0, {32,8,48,228}, RADIO, N_("Must have contact in every area")},
  /*  4 */ {0, {56,8,72,228}, RADIO, N_("Must have at least 1 contact")},
  /*  5 */ {0, {80,8,96,228}, RADIO, N_("Do not check for contacts")},
@@ -928,9 +971,10 @@ static DIALOGITEM erc_optionsdialogitems[] =
  /*  9 */ {0, {80,248,96,480}, RADIO, N_("Do not check for contacts")},
  /* 10 */ {0, {8,284,24,384}, MESSAGE, N_("For N-Well:")},
  /* 11 */ {0, {104,8,120,228}, CHECK, N_("Must connect to Ground")},
- /* 12 */ {0, {104,248,120,480}, CHECK, N_("Must connect to Power")}
+ /* 12 */ {0, {104,248,120,480}, CHECK, N_("Must connect to Power")},
+ /* 13 */ {0, {128,60,144,448}, CHECK, N_("Find farthest distance from contact to edge")}
 };
-static DIALOG erc_optionsdialog = {{50,75,215,564}, N_("ERC Options"), 0, 12, erc_optionsdialogitems};
+static DIALOG erc_optionsdialog = {{50,75,239,564}, N_("Well Check Options"), 0, 13, erc_optionsdialogitems, 0, 0};
 
 /* special items for the "ERC Options" dialog: */
 #define DERO_PWELLEACH   3		/* PWell contact in each area (radio) */
@@ -941,58 +985,63 @@ static DIALOG erc_optionsdialog = {{50,75,215,564}, N_("ERC Options"), 0, 12, er
 #define DERO_NWELLNOT    9		/* NWell contact not needed (radio) */
 #define DERO_PWELLGND   11		/* PWell must connect to ground (check) */
 #define DERO_NWELLVDD   12		/* NWell must connect to power (check) */
+#define DERO_FINDDIST   13		/* Find farthest dist. from contact to edge (check) */
 
 void erc_optionsdlog(void)
 {
 	REGISTER INTBIG itemHit, ercoptions, oldoptions;
 	REGISTER VARIABLE *var;
+	REGISTER void *dia;
 
-	if (DiaInitDialog(&erc_optionsdialog)) return;
+	dia = DiaInitDialog(&erc_optionsdialog);
+	if (dia == 0) return;
 	var = getvalkey((INTBIG)erc_tool, VTOOL, VINTEGER, erc_optionskey);
 	if (var == NOVARIABLE) ercoptions = DEFERCBITS; else
 		ercoptions = var->addr;
 	oldoptions = ercoptions;
 	switch (ercoptions & PWELLCONOPTIONS)
 	{
-		case PWELLCONPERAREA: DiaSetControl(DERO_PWELLEACH, 1);   break;
-		case PWELLCONONCE:    DiaSetControl(DERO_PWELLONCE, 1);   break;
-		case PWELLCONIGNORE:  DiaSetControl(DERO_PWELLNOT, 1);    break;
+		case PWELLCONPERAREA: DiaSetControl(dia, DERO_PWELLEACH, 1);   break;
+		case PWELLCONONCE:    DiaSetControl(dia, DERO_PWELLONCE, 1);   break;
+		case PWELLCONIGNORE:  DiaSetControl(dia, DERO_PWELLNOT, 1);    break;
 	}
 	switch (ercoptions & NWELLCONOPTIONS)
 	{
-		case NWELLCONPERAREA: DiaSetControl(DERO_NWELLEACH, 1);   break;
-		case NWELLCONONCE:    DiaSetControl(DERO_NWELLONCE, 1);   break;
-		case NWELLCONIGNORE:  DiaSetControl(DERO_NWELLNOT, 1);    break;
+		case NWELLCONPERAREA: DiaSetControl(dia, DERO_NWELLEACH, 1);   break;
+		case NWELLCONONCE:    DiaSetControl(dia, DERO_NWELLONCE, 1);   break;
+		case NWELLCONIGNORE:  DiaSetControl(dia, DERO_NWELLNOT, 1);    break;
 	}
-	if ((ercoptions&PWELLONGROUND) != 0) DiaSetControl(DERO_PWELLGND, 1);
-	if ((ercoptions&NWELLONPOWER) != 0) DiaSetControl(DERO_NWELLVDD, 1);
+	if ((ercoptions&PWELLONGROUND) != 0) DiaSetControl(dia, DERO_PWELLGND, 1);
+	if ((ercoptions&NWELLONPOWER) != 0) DiaSetControl(dia, DERO_NWELLVDD, 1);
+	if ((ercoptions&FINDEDGEDIST) != 0) DiaSetControl(dia, DERO_FINDDIST, 1);
 
 	/* loop until done */
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == OK || itemHit == CANCEL) break;
 		if (itemHit == DERO_PWELLEACH || itemHit == DERO_PWELLONCE ||
 			itemHit == DERO_PWELLNOT)
 		{
-			DiaSetControl(DERO_PWELLEACH, 0);
-			DiaSetControl(DERO_PWELLONCE, 0);
-			DiaSetControl(DERO_PWELLNOT, 0);
-			DiaSetControl(itemHit, 1);
+			DiaSetControl(dia, DERO_PWELLEACH, 0);
+			DiaSetControl(dia, DERO_PWELLONCE, 0);
+			DiaSetControl(dia, DERO_PWELLNOT, 0);
+			DiaSetControl(dia, itemHit, 1);
 			continue;
 		}
 		if (itemHit == DERO_NWELLEACH || itemHit == DERO_NWELLONCE ||
 			itemHit == DERO_NWELLNOT)
 		{
-			DiaSetControl(DERO_NWELLEACH, 0);
-			DiaSetControl(DERO_NWELLONCE, 0);
-			DiaSetControl(DERO_NWELLNOT, 0);
-			DiaSetControl(itemHit, 1);
+			DiaSetControl(dia, DERO_NWELLEACH, 0);
+			DiaSetControl(dia, DERO_NWELLONCE, 0);
+			DiaSetControl(dia, DERO_NWELLNOT, 0);
+			DiaSetControl(dia, itemHit, 1);
 			continue;
 		}
-		if (itemHit == DERO_PWELLGND || itemHit == DERO_NWELLVDD)
+		if (itemHit == DERO_PWELLGND || itemHit == DERO_NWELLVDD ||
+			itemHit == DERO_FINDDIST)
 		{
-			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
+			DiaSetControl(dia, itemHit, 1 - DiaGetControl(dia, itemHit));
 			continue;
 		}
 	}
@@ -1001,18 +1050,19 @@ void erc_optionsdlog(void)
 	{
 		/* change state of ERC */
 		ercoptions = 0;
-		if (DiaGetControl(DERO_PWELLEACH) != 0) ercoptions |= PWELLCONPERAREA; else
-			if (DiaGetControl(DERO_PWELLONCE) != 0) ercoptions |= PWELLCONONCE; else
+		if (DiaGetControl(dia, DERO_PWELLEACH) != 0) ercoptions |= PWELLCONPERAREA; else
+			if (DiaGetControl(dia, DERO_PWELLONCE) != 0) ercoptions |= PWELLCONONCE; else
 				ercoptions |= PWELLCONIGNORE;
-		if (DiaGetControl(DERO_NWELLEACH) != 0) ercoptions |= NWELLCONPERAREA; else
-			if (DiaGetControl(DERO_NWELLONCE) != 0) ercoptions |= NWELLCONONCE; else
+		if (DiaGetControl(dia, DERO_NWELLEACH) != 0) ercoptions |= NWELLCONPERAREA; else
+			if (DiaGetControl(dia, DERO_NWELLONCE) != 0) ercoptions |= NWELLCONONCE; else
 				ercoptions |= NWELLCONIGNORE;
-		if (DiaGetControl(DERO_PWELLGND) != 0) ercoptions |= PWELLONGROUND;
-		if (DiaGetControl(DERO_NWELLVDD) != 0) ercoptions |= NWELLONPOWER;
+		if (DiaGetControl(dia, DERO_PWELLGND) != 0) ercoptions |= PWELLONGROUND;
+		if (DiaGetControl(dia, DERO_NWELLVDD) != 0) ercoptions |= NWELLONPOWER;
+		if (DiaGetControl(dia, DERO_FINDDIST) != 0) ercoptions |= FINDEDGEDIST;
 		if (ercoptions != oldoptions)
 			(void)setvalkey((INTBIG)erc_tool, VTOOL, erc_optionskey, ercoptions, VINTEGER);
 	}
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 }
 
 #endif  /* ERCTOOL - at top */
