@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: projecttool.c
@@ -150,7 +151,7 @@ static void          proj_getoldversion(char *facetname);
 static BOOLEAN       proj_getprojinfo(LIBRARY *lib, char *path, char *projfile);
 static BOOLEAN       proj_getusername(BOOLEAN newone, LIBRARY *lib);
 static BOOLEAN       proj_initusers(char **c);
-static BOOLEAN       proj_loadlist(LIBRARY *lib);
+static BOOLEAN       proj_loadlist(LIBRARY *lib, void *dia);
 static BOOLEAN       proj_lockprojfile(char *projectpath, char *projectfile);
 static void          proj_marklocked(NODEPROTO *np, BOOLEAN locked);
 static char         *proj_nextuser(void);
@@ -500,9 +501,9 @@ void proj_newvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG newtype)
 		case VARCINST:   proj_queuecheck(((ARCINST *)addr)->parent);    break;
 		case VPORTPROTO: proj_queuecheck(((PORTPROTO *)addr)->parent);  break;
 		case VNODEPROTO:
-			if (key == us_descent_path_key || key == us_descent_view_key) break;
+			if (key == us_descent_view_key) break;
 			if (key != proj_lockedkey) proj_queuecheck((NODEPROTO *)addr);
-            break;
+			break;
 	}
 }
 
@@ -516,9 +517,9 @@ void proj_killvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG oldaddr, INT
 		case VARCINST:   proj_queuecheck(((ARCINST *)addr)->parent);    break;
 		case VPORTPROTO: proj_queuecheck(((PORTPROTO *)addr)->parent);  break;
 		case VNODEPROTO:
-			if (key == us_descent_path_key || key == us_descent_view_key) break;
+			if (key == us_descent_view_key) break;
 			if (key != proj_lockedkey) proj_queuecheck((NODEPROTO *)addr);
-            break;
+			break;
 	}
 }
 
@@ -932,7 +933,7 @@ static DIALOGITEM proj_oldversdialogitems[] =
  /*  4 */ {0, {28,12,166,312}, SCROLL, ""},
  /*  5 */ {0, {4,140,20,312}, MESSAGE, ""}
 };
-static DIALOG proj_oldversdialog = {{108,75,318,396}, N_("Get Old Version of Facet"), 0, 5, proj_oldversdialogitems};
+static DIALOG proj_oldversdialog = {{108,75,318,396}, N_("Get Old Version of Facet"), 0, 5, proj_oldversdialogitems, 0, 0};
 
 /* special items for the "get old version" dialog: */
 #define DPRV_FACETLIST  4		/* Facet list (scroll) */
@@ -942,11 +943,12 @@ void proj_getoldversion(char *facetname)
 {
 	PROJECTFACET *pf, statpf;
 	REGISTER INTBIG version;
-	REGISTER UINTBIG date;
+	REGISTER time_t date;
 	REGISTER INTBIG itemHit, count, i, len;
 	REGISTER LIBRARY *lib;
 	REGISTER NODEPROTO *np;
 	char line[256], projectpath[256], projectfile[256], **filelist, sep[2], *pt;
+	REGISTER void *dia;
 
 	/* find out which facet is being checked out */
 	np = getnodeproto(facetname);
@@ -985,9 +987,10 @@ void proj_getoldversion(char *facetname)
 	strcat(projectpath, DIRSEPSTR);
 	count = filesindirectory(projectpath, &filelist);
 
-	if (DiaInitDialog(&proj_oldversdialog)) return;
-	DiaSetText(DPRV_FACETNAME, facetname);
-	DiaInitTextDialog(DPRV_FACETLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1,
+	dia = DiaInitDialog(&proj_oldversdialog);
+	if (dia == 0) return;
+	DiaSetText(dia, DPRV_FACETNAME, facetname);
+	DiaInitTextDialog(dia, DPRV_FACETLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1,
 		SCSELMOUSE);
 	for(i=0; i<count; i++)
 	{
@@ -1008,22 +1011,22 @@ void proj_getoldversion(char *facetname)
 		strcat(projectfile, filelist[i]);
 		date = filedate(projectfile);
 		sprintf(line, _("Version %ld, %s"), version, timetostring(date));
-		DiaStuffLine(DPRV_FACETLIST, line);
+		DiaStuffLine(dia, DPRV_FACETLIST, line);
 	}
-	DiaSelectLine(DPRV_FACETLIST, -1);
+	DiaSelectLine(dia, DPRV_FACETLIST, -1);
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == OK || itemHit == CANCEL) break;
 	}
 	version = -1;
-	i = DiaGetCurLine(DPRV_FACETLIST);
+	i = DiaGetCurLine(dia, DPRV_FACETLIST);
 	if (i >= 0)
 	{
-		pt = DiaGetScrollLine(DPRV_FACETLIST, i);
+		pt = DiaGetScrollLine(dia, DPRV_FACETLIST, i);
 		if (strlen(pt) > 8) version = atoi(&pt[8]);
 	}
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 	if (itemHit == CANCEL) return;
 	if (version < 0) return;
 
@@ -1472,7 +1475,7 @@ static DIALOGITEM proj_usersdialogitems[] =
  /*  6 */ {0, {64,184,88,312}, BUTTON, N_("Change Password")},
  /*  7 */ {0, {100,184,124,312}, BUTTON, N_("New User")}
 };
-static DIALOG proj_usersdialog = {{50,75,279,398}, N_("User Name"), 0, 7, proj_usersdialogitems};
+static DIALOG proj_usersdialog = {{50,75,279,398}, N_("User Name"), 0, 7, proj_usersdialogitems, 0, 0};
 
 /* special items for the "User selection" dialog: */
 #define DPRU_USERLIST  3		/* User list (scroll) */
@@ -1492,7 +1495,7 @@ BOOLEAN proj_getusername(BOOLEAN newone, LIBRARY *lib)
 	REGISTER VARIABLE *var;
 	char *truename, userline[256], pwdline[256], projectpath[256], projectfile[256];
 	FILE *io;
-	REGISTER void *infstr;
+	REGISTER void *infstr, *dia;
 
 	/* if name exists and not forcing a new user name, stop now */
 	if (!newone && proj_username[0] != 0) return(FALSE);
@@ -1555,19 +1558,20 @@ BOOLEAN proj_getusername(BOOLEAN newone, LIBRARY *lib)
 
 	/* show the users dialog */
 	writeback = 0;
-	if (DiaInitDialog(&proj_usersdialog)) return(TRUE);
-	DiaInitTextDialog(DPRU_USERLIST, proj_initusers, proj_nextuser, DiaNullDlogDone, 0,
+	dia = DiaInitDialog(&proj_usersdialog);
+	if (dia == 0) return(TRUE);
+	DiaInitTextDialog(dia, DPRU_USERLIST, proj_initusers, proj_nextuser, DiaNullDlogDone, 0,
 		SCSELMOUSE | SCSELKEY | SCDOUBLEQUIT);
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == CANCEL) break;
 		if (itemHit == OK)
 		{
 			/* validate the user name */
-			i = DiaGetCurLine(DPRU_USERLIST);
+			i = DiaGetCurLine(dia, DPRU_USERLIST);
 			if (i < 0) continue;
-			pt = DiaGetScrollLine(DPRU_USERLIST, i);
+			pt = DiaGetScrollLine(dia, DPRU_USERLIST, i);
 			for(pu = proj_users; pu != NOPUSER; pu = pu->nextpuser)
 				if (strcmp(pt, pu->username) == 0) break;
 			if (pu == NOPUSER) continue;
@@ -1585,9 +1589,9 @@ BOOLEAN proj_getusername(BOOLEAN newone, LIBRARY *lib)
 		if (itemHit == DPRU_DELUSER)
 		{
 			/* delete user */
-			i = DiaGetCurLine(DPRU_USERLIST);
+			i = DiaGetCurLine(dia, DPRU_USERLIST);
 			if (i < 0) continue;
-			pt = DiaGetScrollLine(DPRU_USERLIST, i);
+			pt = DiaGetScrollLine(dia, DPRU_USERLIST, i);
 			lastpu = NOPUSER;
 			for(pu = proj_users; pu != NOPUSER; pu = pu->nextpuser)
 			{
@@ -1606,7 +1610,7 @@ BOOLEAN proj_getusername(BOOLEAN newone, LIBRARY *lib)
 			efree(pu->username);
 			efree(pu->userpassword);
 			efree((char *)pu);
-			DiaLoadTextDialog(DPRU_USERLIST, proj_initusers, proj_nextuser,
+			DiaLoadTextDialog(dia, DPRU_USERLIST, proj_initusers, proj_nextuser,
 				DiaNullDlogDone, 0);
 			writeback = 1;
 			continue;
@@ -1614,9 +1618,9 @@ BOOLEAN proj_getusername(BOOLEAN newone, LIBRARY *lib)
 		if (itemHit == DPRU_CHPASS)
 		{
 			/* change password */
-			i = DiaGetCurLine(DPRU_USERLIST);
+			i = DiaGetCurLine(dia, DPRU_USERLIST);
 			if (i < 0) continue;
-			pt = DiaGetScrollLine(DPRU_USERLIST, i);
+			pt = DiaGetScrollLine(dia, DPRU_USERLIST, i);
 			for(pu = proj_users; pu != NOPUSER; pu = pu->nextpuser)
 				if (strcmp(pt, pu->username) == 0) break;
 			if (pu == NOPUSER) continue;
@@ -1658,13 +1662,13 @@ BOOLEAN proj_getusername(BOOLEAN newone, LIBRARY *lib)
 			(void)allocstring(&pu->userpassword, pwdline, proj_tool->cluster);
 			pu->nextpuser = proj_users;
 			proj_users = pu;
-			DiaLoadTextDialog(DPRU_USERLIST, proj_initusers, proj_nextuser,
+			DiaLoadTextDialog(dia, DPRU_USERLIST, proj_initusers, proj_nextuser,
 				DiaNullDlogDone, 0);
 			writeback = 1;
 			continue;
 		}
 	}
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 
 	if (itemHit == CANCEL) return(TRUE);
 
@@ -1710,7 +1714,7 @@ static DIALOGITEM proj_passworddialogitems[] =
  /*  3 */ {0, {12,8,28,205}, MESSAGE, N_("Password for")},
  /*  4 */ {0, {40,48,56,165}, EDITTEXT, ""}
 };
-static DIALOG proj_passworddialog = {{298,75,404,292}, N_("User / Password"), 0, 4, proj_passworddialogitems};
+static DIALOG proj_passworddialog = {{298,75,404,292}, N_("User / Password"), 0, 4, proj_passworddialogitems, 0, 0};
 
 /* special items for the "Password prompt" dialog: */
 #define DPRP_USERNAME  3		/* User name (stat text) */
@@ -1728,34 +1732,36 @@ char *proj_wantpassword(INTBIG mode, char *username)
 {
 	REGISTER INTBIG itemHit;
 	static char typedstuff[256];
+	REGISTER void *dia;
 
-	if (DiaInitDialog(&proj_passworddialog)) return("");
+	dia = DiaInitDialog(&proj_passworddialog);
+	if (dia == 0) return("");
 	switch (mode)
 	{
 		case 0:
-			DiaSetText(DPRP_USERNAME, _("New User Name"));
+			DiaSetText(dia, DPRP_USERNAME, _("New User Name"));
 			break;
 		case 1:
 			sprintf(typedstuff, _("Password for %s"), username);
-			DiaSetText(DPRP_USERNAME, typedstuff);
+			DiaSetText(dia, DPRP_USERNAME, typedstuff);
 			break;
 		case 2:
 			sprintf(typedstuff, _("New Password for %s"), username);
-			DiaSetText(DPRP_USERNAME, typedstuff);
+			DiaSetText(dia, DPRP_USERNAME, typedstuff);
 			break;
 		case 3:
 			sprintf(typedstuff, _("Verify Password for %s"), username);
-			DiaSetText(DPRP_USERNAME, typedstuff);
+			DiaSetText(dia, DPRP_USERNAME, typedstuff);
 			break;
 	}
-	if (mode != 0) DiaOpaqueEdit(DPRP_PASSWORD);
+	if (mode != 0) DiaOpaqueEdit(dia, DPRP_PASSWORD);
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == CANCEL || itemHit == OK) break;
 	}
-	strcpy(typedstuff, DiaGetText(DPRP_PASSWORD));
-	DiaDoneDialog();
+	strcpy(typedstuff, DiaGetText(dia, DPRP_PASSWORD));
+	DiaDoneDialog(dia);
 	if (mode != 0)
 		myencrypt(typedstuff, "BicIsSchediwy");
 	return(typedstuff);
@@ -1913,7 +1919,7 @@ static DIALOGITEM proj_listdialogitems[] =
  /*  7 */ {0, {204,8,228,108}, BUTTON, N_("Check It In")},
  /*  8 */ {0, {204,232,228,296}, BUTTON, N_("Update")}
 };
-static DIALOG proj_listdialog = {{50,75,383,462}, N_("Project Management"), 0, 8, proj_listdialogitems};
+static DIALOG proj_listdialog = {{50,75,383,462}, N_("Project Management"), 0, 8, proj_listdialogitems, 0, 0};
 
 /* special items for the "Project list" dialog: */
 #define DPRL_PROJLIST  2		/* Project list (scroll) */
@@ -1926,7 +1932,7 @@ void proj_showlistdialog(LIBRARY *lib)
 {
 	REGISTER INTBIG itemHit, i, j;
 	REGISTER PROJECTFACET *pf;
-	REGISTER void *infstr;
+	REGISTER void *infstr, *dia;
 
 	if (proj_getusername(FALSE, lib))
 	{
@@ -1936,26 +1942,27 @@ void proj_showlistdialog(LIBRARY *lib)
 	proj_active = TRUE;
 
 	/* show the dialog */
-	if (DiaInitDialog(&proj_listdialog)) return;
-	DiaInitTextDialog(DPRL_PROJLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone,
+	dia = DiaInitDialog(&proj_listdialog);
+	if (dia == 0) return;
+	DiaInitTextDialog(dia, DPRL_PROJLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone,
 		-1, SCSELMOUSE | SCREPORT);
 
 	/* load project information into the scroll area */
-	if (proj_loadlist(lib))
+	if (proj_loadlist(lib, dia))
 	{
-		DiaDoneDialog();
+		DiaDoneDialog(dia);
 		return;
 	}
 
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == OK) break;
 		if (itemHit == DPRL_UPDATE)
 		{
 			/* update project */
 			proj_update(lib);
-			(void)proj_loadlist(lib);
+			(void)proj_loadlist(lib, dia);
 			us_endbatch();
 			continue;
 		}
@@ -1963,7 +1970,7 @@ void proj_showlistdialog(LIBRARY *lib)
 			itemHit == DPRL_PROJLIST)
 		{
 			/* figure out which facet is selected */
-			i = DiaGetCurLine(DPRL_PROJLIST);
+			i = DiaGetCurLine(dia, DPRL_PROJLIST);
 			if (i < 0) continue;
 			for(j=0, pf = proj_firstprojectfacet; pf != NOPROJECTFACET; pf = pf->nextprojectfacet, j++)
 				if (i == j) break;
@@ -1981,7 +1988,7 @@ void proj_showlistdialog(LIBRARY *lib)
 					addtoinfstr(infstr, '}');
 				}
 				proj_checkout(returninfstr(infstr), FALSE);
-				(void)proj_loadlist(lib);
+				(void)proj_loadlist(lib, dia);
 				us_endbatch();
 				continue;
 			}
@@ -1997,28 +2004,28 @@ void proj_showlistdialog(LIBRARY *lib)
 					addtoinfstr(infstr, '}');
 				}
 				proj_checkin(returninfstr(infstr));
-				(void)proj_loadlist(lib);
+				(void)proj_loadlist(lib, dia);
 				continue;
 			}
 			if (itemHit == DPRL_PROJLIST)
 			{
-				if (*pf->comment != 0) DiaSetText(DPRL_COMMENTS, pf->comment);
-				if (*pf->owner == 0) DiaUnDimItem(DPRL_CHECKOUT); else
-					DiaDimItem(DPRL_CHECKOUT);
-				if (strcmp(pf->owner, proj_username) == 0) DiaUnDimItem(DPRL_CHECKIN); else
-					DiaDimItem(DPRL_CHECKIN);
+				if (*pf->comment != 0) DiaSetText(dia, DPRL_COMMENTS, pf->comment);
+				if (*pf->owner == 0) DiaUnDimItem(dia, DPRL_CHECKOUT); else
+					DiaDimItem(dia, DPRL_CHECKOUT);
+				if (strcmp(pf->owner, proj_username) == 0) DiaUnDimItem(dia, DPRL_CHECKIN); else
+					DiaDimItem(dia, DPRL_CHECKIN);
 				continue;
 			}
 		}
 	}
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 }
 
 /*
  * Routine to display the current project information in the list dialog.
  * Returns true on error.
  */
-BOOLEAN proj_loadlist(LIBRARY *lib)
+BOOLEAN proj_loadlist(LIBRARY *lib, void *dia)
 {
 	REGISTER BOOLEAN failed, uptodate;
 	REGISTER INTBIG whichline, thisline;
@@ -2057,7 +2064,7 @@ BOOLEAN proj_loadlist(LIBRARY *lib)
 	curfacet = getcurfacet();
 
 	/* show what is in the project file */
-	DiaLoadTextDialog(DPRL_PROJLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1);
+	DiaLoadTextDialog(dia, DPRL_PROJLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1);
 	whichline = -1;
 	thisline = 0;
 	uptodate = TRUE;
@@ -2108,27 +2115,27 @@ BOOLEAN proj_loadlist(LIBRARY *lib)
 				strcat(line, pf->owner);
 			}
 		}
-		DiaStuffLine(DPRL_PROJLIST, line);
+		DiaStuffLine(dia, DPRL_PROJLIST, line);
 		thisline++;
 	}
-	DiaSelectLine(DPRL_PROJLIST, whichline);
+	DiaSelectLine(dia, DPRL_PROJLIST, whichline);
 
-	DiaDimItem(DPRL_CHECKOUT);
-	DiaDimItem(DPRL_CHECKIN);
+	DiaDimItem(dia, DPRL_CHECKOUT);
+	DiaDimItem(dia, DPRL_CHECKIN);
 	if (whichline >= 0)
 	{
-		if (*curpf->comment != 0) DiaSetText(DPRL_COMMENTS, curpf->comment);
-		if (*curpf->owner == 0) DiaUnDimItem(DPRL_CHECKOUT); else
-			DiaDimItem(DPRL_CHECKOUT);
-		if (strcmp(curpf->owner, proj_username) == 0) DiaUnDimItem(DPRL_CHECKIN); else
-			DiaDimItem(DPRL_CHECKIN);
+		if (*curpf->comment != 0) DiaSetText(dia, DPRL_COMMENTS, curpf->comment);
+		if (*curpf->owner == 0) DiaUnDimItem(dia, DPRL_CHECKOUT); else
+			DiaDimItem(dia, DPRL_CHECKOUT);
+		if (strcmp(curpf->owner, proj_username) == 0) DiaUnDimItem(dia, DPRL_CHECKIN); else
+			DiaDimItem(dia, DPRL_CHECKIN);
 	}
 
-	if (uptodate) DiaDimItem(DPRL_UPDATE); else
+	if (uptodate) DiaDimItem(dia, DPRL_UPDATE); else
 	{
 		ttyputmsg(_("Your library does not contain the most recent additions to the project."));
 		ttyputmsg(_("You should do an 'Update' to make it current."));
-		DiaUnDimItem(DPRL_UPDATE);
+		DiaUnDimItem(dia, DPRL_UPDATE);
 	}
 	return(FALSE);
 }
@@ -2141,7 +2148,7 @@ static DIALOGITEM proj_commentsdialogitems[] =
  /*  3 */ {0, {4,8,20,363}, MESSAGE, N_("Reason for checking out facet")},
  /*  4 */ {0, {28,12,92,358}, EDITTEXT, ""}
 };
-static DIALOG proj_commentsdialog = {{108,75,248,447}, N_("Project Comments"), 0, 4, proj_commentsdialogitems};
+static DIALOG proj_commentsdialog = {{108,75,248,447}, N_("Project Comments"), 0, 4, proj_commentsdialogitems, 0, 0};
 
 /* special items for the "Project list" dialog: */
 #define DPRC_COMMENT_L  3		/* Comment label (stat text) */
@@ -2155,19 +2162,21 @@ BOOLEAN proj_getcomments(PROJECTFACET *pf, char *direction)
 {
 	REGISTER INTBIG itemHit;
 	static char line[256];
+	REGISTER void *dia;
 
-	if (DiaInitDialog(&proj_commentsdialog)) return(TRUE);
+	dia = DiaInitDialog(&proj_commentsdialog);
+	if (dia == 0) return(TRUE);
 	sprintf(line, _("Reason for checking-%s facet %s"), direction, pf->cellname);
-	DiaSetText(DPRC_COMMENT_L, line);
-	DiaSetText(-DPRC_COMMENT, pf->comment);
+	DiaSetText(dia, DPRC_COMMENT_L, line);
+	DiaSetText(dia, -DPRC_COMMENT, pf->comment);
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == CANCEL || itemHit == OK) break;
 	}
 	if (itemHit == OK)
-		(void)reallocstring(&pf->comment, DiaGetText(DPRC_COMMENT), proj_tool->cluster);
-	DiaDoneDialog();
+		(void)reallocstring(&pf->comment, DiaGetText(dia, DPRC_COMMENT), proj_tool->cluster);
+	DiaDoneDialog(dia);
 	if (itemHit == CANCEL) return(TRUE);
 	return(FALSE);
 }
