@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: netextract.c
@@ -45,14 +46,6 @@ static void      add_arc( ARCINST* arc, ARCINST** list );
 static void      via_to_contact( NODEINST**, INTBIG*, TECHNOLOGY*, INTBIG*, ARCINST**, NODEINST**, NODEPROTO* );
 static BOOLEAN   contains_node( NODEINST*, NODEINST* );
 
-#if 0
-static void      remove_arc( ARCINST* arc, ARCINST** list );
-static void      arc_crossings( ARCINST**, NODEINST**, NODEPROTO* );
-static void      t_crossings( ARCINST**, NODEINST**, NODEPROTO* );
-static NODEINST *arc_to_PL( ARCINST*, NODEPROTO* );
-static void      make_arc_geom( ARCINST**, NODEPROTO* );
-#endif
-
 /*
  * Routine to convert facet "facet" from pure-layer nodes to real nodes and arcs.
  */
@@ -622,7 +615,7 @@ void  build_node_arc_list( NODEINST** parselist,
 
 /*********************************** NOT YET USED ***********************************/
 
-#if 0
+#if 0		/* not used yet */
 
 NODEINST* arc_to_PL( ARCINST* arc, NODEPROTO* facet )
 {
