@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: netflat.c
@@ -53,9 +54,10 @@ static INTBIG     net_serieslisttotal = 0;
 static BOOLEAN net_addtoserieslist(PCOMP *pc);
 static PCOMP  *net_allocpcomp(void);
 static PNET   *net_allocpnet(void);
-static PCOMP  *net_buildpseudo(NODEPROTO*, PCOMP*, INTBIG*, PNET**, BOOLEAN);
+static PCOMP  *net_buildpseudo(NODEPROTO*, PCOMP*, INTBIG*, PNET**, BOOLEAN, BOOLEAN);
 static void    net_crawlforseries(PCOMP *pc);
 static INTBIG  net_findotherexporttopology(NODEPROTO *np, char *exportname);
+static INTBIG  net_findotherexportcharacteristic(NODEPROTO *parent, INTBIG bits);
 static PCOMP **net_gatherseries(PCOMP *pc, INTBIG *seriescount);
 static INTBIG  net_getfunction(NODEINST*);
 static float   net_getpartvalue(NODEINST *ni);
@@ -67,6 +69,8 @@ static PNET   *net_newpnet(NETWORK *net, PNET **pnetlist);
 static void    net_setallexporttopology(void);
 static void    net_setthisexporttopology(PORTPROTO *pp, INTBIG *index);
 static int     net_sortpcompbyhash(const void *e1, const void *e2);
+static void    net_addexporttopnet(PNET *pn, PORTPROTO *pp);
+static void    net_gatherglobals(NODEPROTO *facet, PNET **pnetlist);
 
 /*
  * Routine to free all memory associated with this module.
@@ -100,7 +104,7 @@ void net_freeflatmemory(void)
  *   PNET *pnet;
  *   INTBIG components, nets, powernets, groundnets;
  *   pcomp = net_makepseudo(facet, &components, &nets, &powernets, &groundnets,
- *	    &pnet, hierarchical, mergeparallel, mergeseries, checkfacetoverrides);
+ *	    &pnet, hierarchical, mergeparallel, mergeseries, checkfacetoverrides, figuresizes);
  *   .....
  *        do something with the network in "pcomp" and "pnet"
  *   .....
@@ -123,10 +127,11 @@ void net_freeflatmemory(void)
  * If "mergeparallel"       is true, parallel components are merged into one.
  * If "mergeseries"         is true, series transistors are merged into one.
  * If "checkfacetoverrides" is true, individual facets may override "mergeparallel".
+ * If "figuresizes"         is true, extract size information (takes time if programmed).
  */
 PCOMP *net_makepseudo(NODEPROTO *facet, INTBIG *components, INTBIG *nets, INTBIG *powernets,
 	INTBIG *groundnets, PNET **pnetlist, BOOLEAN hierarchical, BOOLEAN mergeparallel,
-	BOOLEAN mergeseries, BOOLEAN checkfacetoverrides)
+	BOOLEAN mergeseries, BOOLEAN checkfacetoverrides, BOOLEAN figuresizes)
 {
 	PCOMP *pcomplist;
 	REGISTER BOOLEAN localmergeparallel, localmergeseries;
@@ -134,7 +139,9 @@ PCOMP *net_makepseudo(NODEPROTO *facet, INTBIG *components, INTBIG *nets, INTBIG
 	REGISTER NETWORK *net, *subnet;
 	REGISTER PNET *pn;
 	REGISTER VARIABLE *var;
-	REGISTER PORTPROTO *pp, *opp, *npp, **newportlist;
+	REGISTER LIBRARY *lib;
+	REGISTER NODEPROTO *np;
+	REGISTER PORTPROTO *pp, *opp;
 
 	/* see if the current facet overrides the options */
 	localmergeparallel = mergeparallel;
@@ -172,34 +179,28 @@ PCOMP *net_makepseudo(NODEPROTO *facet, INTBIG *components, INTBIG *nets, INTBIG
 			if ((pp->userbits&STATEBITS) == GNDPORT) pn->flags |= GROUNDNET;
 		pn->flags |= EXPORTEDNET;
 		net->temp1 = (INTBIG)pn;
-		pn->realportcount = 1;
-		pn->realportlist = pp;
+		net_addexporttopnet(pn, pp);
 		for(opp = pp->nextportproto; opp != NOPORTPROTO; opp = opp->nextportproto)
 		{
 			if (net != opp->network) continue;
-			newportlist = (PORTPROTO **)emalloc((pn->realportcount+1) *
-				(sizeof (PORTPROTO *)), net_tool->cluster);
-			if (newportlist == 0) { *components = -1;   return(NOPCOMP); }
-			for(i=0; i<pn->realportcount; i++)
-			{
-				if (pn->realportcount == 1) npp = (PORTPROTO *)pn->realportlist; else
-					npp = ((PORTPROTO **)pn->realportlist)[i];
-				newportlist[i] = npp;
-			}
-			newportlist[pn->realportcount] = opp;
-			if (pn->realportcount > 1) efree((char *)pn->realportlist);
-			pn->realportlist = newportlist;
-			pn->realportcount++;
+			net_addexporttopnet(pn, opp);
 		}
-		if (net->signals > 1)
+		if (net->buswidth > 1)
 		{
-			for(i=0; i<net->signals; i++)
+			for(i=0; i<net->buswidth; i++)
 			{
 				subnet = net->networklist[i];
-				if (subnet->temp1 != (INTBIG)NOPNET) continue;
+				pn = (PNET *)subnet->temp1;
+				if (pn != NOPNET)
+				{
+					net_addexporttopnet(pn, pp);
+					continue;
+				}
 				pn = net_newpnet(subnet, pnetlist);
 				pn->flags |= EXPORTEDNET;
 				subnet->temp1 = (INTBIG)pn;
+				pn->realportcount = 1;
+				pn->realportlist = pp;
 			}
 		}
 	}
@@ -211,7 +212,7 @@ PCOMP *net_makepseudo(NODEPROTO *facet, INTBIG *components, INTBIG *nets, INTBIG
 	*components = 0;
 	begintraversehierarchy();
 	net_toplevelinst = NONODEINST;
-	pcomplist = net_buildpseudo(facet, NOPCOMP, components, pnetlist, hierarchical);
+	pcomplist = net_buildpseudo(facet, NOPCOMP, components, pnetlist, hierarchical, figuresizes);
 	endtraversehierarchy();
 	if (pcomplist == 0) { *components = -1;   return(NOPCOMP); }
 	if (*components == 0)
@@ -245,12 +246,76 @@ PCOMP *net_makepseudo(NODEPROTO *facet, INTBIG *components, INTBIG *nets, INTBIG
 		if ((pn->flags&POWERNET) != 0) (*powernets)++;
 		if ((pn->flags&GROUNDNET) != 0) (*groundnets)++;
 		if ((pn->flags&(POWERNET|GROUNDNET)) == (POWERNET|GROUNDNET))
-			ttyputmsg(_("WARNING: net %s shorts power and ground"),
-				describenetwork(pn->network));
+		{
+			if (pn->network == NONETWORK) np = NONODEPROTO; else
+				np = pn->network->parent;
+			ttyputmsg(_("WARNING: facet %s, network '%s' shorts power and ground"),
+				describenodeproto(np), describenetwork(pn->network));
+		}
 	}
+
+	/* look to the bottom of the hierarchy and be sure all global signals are listed */
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			np->temp2 = 0;
+	net_gatherglobals(facet, pnetlist);
+
 	return(pcomplist);
 }
 
+/*
+ * Routine to recursively examine the hierarchy and make sure that all globals are mentioned
+ * at the top level.
+ */
+void net_gatherglobals(NODEPROTO *facet, PNET **pnetlist)
+{
+	REGISTER INTBIG i, index;
+	REGISTER NETWORK *net;
+	REGISTER NODEINST *ni;
+	REGISTER NODEPROTO *cnp, *subnp, *pnparent;
+	REGISTER PNET *pn;
+
+	if (facet->temp2 != 0) return;
+	facet->temp2 = 1;
+
+	/* look at all globals in this facet */
+	for(i=0; i<facet->globalnetcount; i++)
+	{
+		net = facet->globalnetworks[i];
+		if (net == NONETWORK) continue;
+		for(pn = *pnetlist; pn != NOPNET; pn = pn->nextpnet)
+		{
+#if 1
+			if (pn->network == NONETWORK) continue;
+#endif
+			index = pn->network->globalnet;
+			if (index < 0) continue;
+			pnparent = pn->network->parent;
+			if (namesame(pnparent->globalnetnames[index], facet->globalnetnames[i]) == 0) break;
+		}
+		if (pn != NOPNET) continue;
+
+		/* didn't find network "net" in the list */
+		pn = net_newpnet(net, pnetlist);
+		net->temp1 = (INTBIG)pn;
+	}
+
+	/* now recurse */
+	for(ni = facet->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+	{
+		subnp = ni->proto;
+		if (subnp->primindex != 0) continue;
+
+		/* don't recurse into a schematic's own icon */
+		if (subnp->cell == facet->cell) continue;
+
+		/* switch from icon to schematic */
+		cnp = contentsview(subnp);
+		if (cnp != NONODEPROTO) subnp = cnp;
+		net_gatherglobals(subnp, pnetlist);
+	}
+}
+
 /*
  * routine to build a linked list of pseudocomponents in facet "facet".  The
  * list is appended to "initiallist" and returned.  The values of "power" and
@@ -259,7 +324,7 @@ PCOMP *net_makepseudo(NODEPROTO *facet, INTBIG *components, INTBIG *nets, INTBIG
  * "compare_hierarchically" is nonzero, net is flattened.  Returns zero on error.
  */
 PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
-	PNET **pnetlist, BOOLEAN compare_hierarchically)
+	PNET **pnetlist, BOOLEAN compare_hierarchically, BOOLEAN figuresizes)
 {
 	REGISTER PCOMP *pcomp;
 	REGISTER PORTPROTO *pp, *opp, *realpp, *temprealpp;
@@ -269,7 +334,7 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 	REGISTER NODEINST *ni;
 	NODEINST **nilist;
 	REGISTER INTBIG fun, i, j, k, l, toplevel, flattenit, lambda,
-		nodewidth, nindex, sigcount;
+		nodewidth, nindex, sigcount, stopcheck;
 	BOOLEAN localcompare_hierarchically;
 	INTBIG width, length, pathcount, *indexlist;
 	REGISTER NODEPROTO *realnp, *anp, *cnp;
@@ -283,9 +348,9 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 	for(pp = facet->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 	{
 		/* check busses for consistent component characteristics */
-		if (pp->network->signals > 1)
+		if (pp->network->buswidth > 1)
 		{
-			for(i=0; i<pp->network->signals; i++)
+			for(i=0; i<pp->network->buswidth; i++)
 			{
 				subnet = pp->network->networklist[i];
 				for(opp = facet->firstportproto; opp != NOPORTPROTO; opp = opp->nextportproto)
@@ -317,9 +382,9 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 		for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 			if (pi->proto == pp) break;
 		if (pi == NOPORTARCINST) continue;
+		net = pi->conarcinst->network;
 
 		/* propagate power and ground */
-		net = pi->conarcinst->network;
 		if ((pe->exportproto->userbits&STATEBITS) == PWRPORT)
 		{
 			pn = (PNET *)net->temp1;
@@ -343,9 +408,9 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 		pn = net_newpnet(net, pnetlist);
 		if (pn == NOPNET) return(0);
 		net->temp1 = (INTBIG)pn;
-		if (net->signals > 1)
+		if (net->buswidth > 1)
 		{
-			for(i=0; i<net->signals; i++)
+			for(i=0; i<net->buswidth; i++)
 			{
 				subnet = net->networklist[i];
 				if (subnet->temp1 != (INTBIG)NOPNET) continue;
@@ -356,8 +421,14 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 	}
 
 	/* search every component in the facet */
+	stopcheck = 0;
 	for(ni = facet->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
+		if (((stopcheck++)%5) == 0)
+		{
+			if (stopping(STOPREASONNCC)) return(0);
+		}
+
 		anp = ni->proto;
 		if (toplevel != 0) net_toplevelinst = ni;
 
@@ -452,29 +523,29 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 					if (foundnet != NONETWORK)
 					{
 						/* propagate export networks to nets inside facet */
-						if (foundnet->signals > 1)
+						if (foundnet->buswidth > 1)
 						{
-							sigcount = foundnet->signals;
-							if (nodewidth > 1 && realpp->network->signals * nodewidth == foundnet->signals)
+							sigcount = foundnet->buswidth;
+							if (nodewidth > 1 && realpp->network->buswidth * nodewidth == foundnet->buswidth)
 							{
 								/* map wide bus to a particular instantiation of an arrayed node */
-								if (realpp->network->signals == 1)
+								if (realpp->network->buswidth == 1)
 								{
 									realpp->network->temp1 = foundnet->networklist[nindex]->temp1;
 								} else
 								{
-									for(i=0; i<realpp->network->signals; i++)
+									for(i=0; i<realpp->network->buswidth; i++)
 										realpp->network->networklist[i]->temp1 =
-											foundnet->networklist[i + nindex*realpp->network->signals]->temp1;
+											foundnet->networklist[i + nindex*realpp->network->buswidth]->temp1;
 								}
 							} else
 							{
-								if (realpp->network->signals != foundnet->signals)
+								if (realpp->network->buswidth != foundnet->buswidth)
 								{
 									ttyputerr(_("***ERROR: port %s on node %s in facet %s is %d wide, but is connected/exported with width %d"),
 										realpp->protoname, describenodeinst(ni), describenodeproto(facet),
-											realpp->network->signals, foundnet->signals);
-									sigcount = mini(sigcount, realpp->network->signals);
+											realpp->network->buswidth, foundnet->buswidth);
+									sigcount = mini(sigcount, realpp->network->buswidth);
 									if (sigcount == 1) sigcount = 0;
 								}
 								realpp->network->temp1 = (INTBIG)foundnet->temp1;
@@ -489,9 +560,9 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 				}
 
 				/* recurse into the facet */
-				downhierarchy(ni, nindex);
+				downhierarchy(ni, realnp, nindex);
 				initiallist = net_buildpseudo(realnp, initiallist, components, pnetlist,
-					compare_hierarchically);
+					compare_hierarchically, figuresizes);
 				uphierarchy();
 				if (initiallist == 0) return(0);
 				continue;
@@ -503,6 +574,13 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 				fun = net_getfunction(ni);
 				if (fun == NPCONNECT || fun == NPART || fun == NPUNKNOWN ||
 					fun == NPCONPOWER || fun == NPCONGROUND) continue;
+
+				/* special case: ignore resistors and capacitors if they are being ignored electrically */
+				if (fun == NPRESIST || fun == NPCAPAC || fun == NPECAPAC)
+				{
+					if (asktech(sch_tech, "ignoring-resistor-topology") != 0)
+						continue;
+				}
 			}
 
 			/* create a pseudo-component */
@@ -512,7 +590,7 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 			initiallist = pcomp;
 			pcomp->function = (INTSML)fun;
 			pcomp->hashreason = 0;
-			gettraversalpath(ni->parent, &nilist, &indexlist, &pathcount);
+			gettraversalpath(ni->parent, &nilist, &indexlist, &pathcount, 0);
 			pcomp->hierpathcount = pathcount;
 			if (pcomp->hierpathcount > 0)
 			{
@@ -555,8 +633,8 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 				}
 
 				/* new port, add in the number of signals */
-				if (pp->network->signals <= 1) pcomp->wirecount++; else
-					pcomp->wirecount += pp->network->signals;
+				if (pp->network->buswidth <= 1) pcomp->wirecount++; else
+					pcomp->wirecount += pp->network->buswidth;
 			}
 
 			/* get parameters for the node */
@@ -570,10 +648,17 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 				case NPTRADMES:  case NPTRA4DMES:
 				case NPTRAEMES:  case NPTRA4EMES:
 					/* transistors that have a length and a width */
-					transistorsize(ni, &length, &width);
-					lambda = facet->cell->lib->lambda[facet->tech->techindex];
-					pcomp->length = (float)muldiv(length, WHOLE, lambda);
-					pcomp->width = (float)muldiv(width, WHOLE, lambda);
+					if (figuresizes)
+					{
+						transistorsize(ni, &length, &width);
+						lambda = lambdaofnode(ni);
+						pcomp->length = (float)muldiv(length, WHOLE, lambda);
+						pcomp->width = (float)muldiv(width, WHOLE, lambda);
+					} else
+					{
+						/* just set fake sizes */
+						pcomp->length = pcomp->width = 2.0f;
+					}
 					pcomp->flags |= COMPHASWIDLEN;
 #ifdef IGNOREFOURPORT
 					pcomp->wirecount = 3;
@@ -583,10 +668,13 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 				case NPTRANPN:    case NPTRA4NPN:
 				case NPTRAPNP:    case NPTRA4PNP:
 					/* transistors that have an area */
-					transistorsize(ni, &length, &width);
-					lambda = facet->cell->lib->lambda[facet->tech->techindex];
-					if (length < 0) pcomp->length = 0.0; else
-						pcomp->length = (float)muldiv(length, WHOLE, lambda);
+					if (figuresizes)
+					{
+						transistorsize(ni, &length, &width);
+						lambda = lambdaofnode(ni);
+						if (length < 0) pcomp->length = 0.0; else
+							pcomp->length = (float)muldiv(length, WHOLE, lambda);
+					} else pcomp->length = 4.0f;
 					pcomp->width = 0.0;
 					pcomp->flags |= COMPHASAREA;
 					break;
@@ -642,7 +730,7 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 						pp = pcomp->portlist[j];
 						pcomp->portindices[j] = (INTSML)pp->network->temp2;
 						if (net_getpnetandstate(ni, pp, NONETWORK, -1, &pcomp->netnumbers[j],
-							&pcomp->state[j], pnetlist, nodewidth, nindex, pp->network->signals)) return(0);
+							&pcomp->state[j], pnetlist, nodewidth, nindex, pp->network->buswidth)) return(0);
 					}
 					break;
 
@@ -665,7 +753,7 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 						pp = pcomp->portlist[j];
 						pcomp->portindices[j] = (INTSML)pp->network->temp2;
 						if (net_getpnetandstate(ni, pp, NONETWORK, -1, &pcomp->netnumbers[j],
-							&pcomp->state[j], pnetlist, nodewidth, nindex, pp->network->signals)) return(0);
+							&pcomp->state[j], pnetlist, nodewidth, nindex, pp->network->buswidth)) return(0);
 					}
 					break;
 
@@ -679,7 +767,7 @@ PCOMP *net_buildpseudo(NODEPROTO *facet, PCOMP *initiallist, INTBIG *components,
 							realpp = equivalentport(anp, pp, cnp);
 							if (realpp == NOPORTPROTO) continue;
 						}
-						sigcount = realpp->network->signals;
+						sigcount = realpp->network->buswidth;
 
 						/* special case for isolated ports */
 						if ((realpp->userbits&PORTISOLATED) != 0)
@@ -771,15 +859,15 @@ BOOLEAN net_getpnetandstate(NODEINST *ni, PORTPROTO *pp, NETWORK *forcenet,
 		net = ai->network;
 		if (index < 0)
 		{
-			if (nodewidth > 1 && net->signals == nodewidth)
+			if (nodewidth > 1 && net->buswidth == nodewidth)
 				*netnumber = (PNET *)net->networklist[nindex]->temp1; else
 					*netnumber = (PNET *)net->temp1;
 		} else
 		{
 			entry = index;
-			if (nodewidth > 1 && net->signals == nodewidth * destsignals)
+			if (nodewidth > 1 && net->buswidth == nodewidth * destsignals)
 				entry = index + nindex*destsignals;
-			if (entry < net->signals)
+			if (entry < net->buswidth)
 				*netnumber = (PNET *)net->networklist[entry]->temp1;
 		}
 		if ((ai->userbits&ISNEGATED) != 0)
@@ -808,15 +896,15 @@ BOOLEAN net_getpnetandstate(NODEINST *ni, PORTPROTO *pp, NETWORK *forcenet,
 		net = pe->exportproto->network;
 		if (index < 0)
 		{
-			if (nodewidth > 1 && net->signals == nodewidth)
+			if (nodewidth > 1 && net->buswidth == nodewidth)
 				*netnumber = (PNET *)net->networklist[nindex]->temp1; else
 					*netnumber = (PNET *)net->temp1;
 		} else
 		{
 			entry = index;
-			if (nodewidth > 1 && net->signals == nodewidth * destsignals)
+			if (nodewidth > 1 && net->buswidth == nodewidth * destsignals)
 				entry = index + nindex*destsignals;
-			if (entry < net->signals)
+			if (entry < net->buswidth)
 				*netnumber = (PNET *)net->networklist[entry]->temp1;
 		}
 		*state |= EXPORTEDPORT;
@@ -1492,6 +1580,35 @@ PNET *net_newpnet(NETWORK *net, PNET **pnetlist)
 	return(pn);
 }
 
+/*
+ * Routine to add portproto "pp" to the list of ports on PNET "pn".
+ */
+void net_addexporttopnet(PNET *pn, PORTPROTO *pp)
+{
+	REGISTER PORTPROTO **newportlist, *npp;
+	REGISTER INTBIG i;
+
+	if (pn->realportcount <= 0)
+	{
+		pn->realportcount = 1;
+		pn->realportlist = pp;
+		return;
+	}
+	newportlist = (PORTPROTO **)emalloc((pn->realportcount+1) *
+		(sizeof (PORTPROTO *)), net_tool->cluster);
+	if (newportlist == 0) return;
+	for(i=0; i<pn->realportcount; i++)
+	{
+		if (pn->realportcount == 1) npp = (PORTPROTO *)pn->realportlist; else
+			npp = ((PORTPROTO **)pn->realportlist)[i];
+		newportlist[i] = npp;
+	}
+	newportlist[pn->realportcount] = pp;
+	if (pn->realportcount > 1) efree((char *)pn->realportlist);
+	pn->realportlist = newportlist;
+	pn->realportcount++;
+}
+
 /*
  * routine to allocate a new pcomp module from the pool (if any) or memory
  */
@@ -1690,15 +1807,11 @@ void net_setallexporttopology(void)
 {
 	REGISTER LIBRARY *lib;
 	REGISTER NODEPROTO *np;
-	REGISTER INTBIG i, j, len;
-	REGISTER char *pt, *name, *oname, **strings;
-	UINTBIG matchdate;
-	NODEPROTO *onp;
-	REGISTER VARIABLE *var, *ovar;
+	REGISTER INTBIG i;
 	REGISTER PORTPROTO *pp;
 	REGISTER CELL *c;
 	REGISTER TECHNOLOGY *tech;
-	REGISTER NETWORK *net, *onet;
+	REGISTER NETWORK *net;
 
 	/* clear prime number indices in the cells */
 	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
@@ -1715,7 +1828,9 @@ void net_setallexporttopology(void)
 		}
 	}
 
+#if 0
 	/* assign network topology information from previous NCC runs */
+	infstr = 0;
 	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
 	{
 		/* assign topology information uniformly in each cell */
@@ -1735,6 +1850,17 @@ void net_setallexporttopology(void)
 				if (ovar == NOVARIABLE) continue;
 				if (onp->revisiondate > matchdate) continue;
 
+				/* report these reuses */
+				if (infstr == 0)
+				{
+					infstr = initinfstr();
+					addstringtoinfstr(infstr, _("NOTE: Using previous NCC match information between"));
+				} else
+				{
+					addstringtoinfstr(infstr, ",");
+				}
+				formatinfstr(infstr, _(" %s and %s"), describenodeproto(np), describenodeproto(onp));
+
 				/* found match: assign network pointers */
 				len = getlength(var);
 				strings = (char **)var->addr;
@@ -1764,6 +1890,8 @@ void net_setallexporttopology(void)
 			}
 		}
 	}
+	if (infstr != 0) ttyputmsg("%s", returninfstr(infstr));
+#endif
 
 	/* assign remaining network topology based on export names */
 	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
@@ -1844,32 +1972,60 @@ void net_setallexporttopology(void)
 void net_setthisexporttopology(PORTPROTO *pp, INTBIG *index)
 {
 	REGISTER NETWORK *net, *subnet;
-	REGISTER INTBIG i;
+	REGISTER PORTPROTO *opp;
+	REGISTER NODEPROTO *np;
+	REGISTER INTBIG i, bits;
 	REGISTER char *name;
 
 	/* assign the unique prime numbers to the export's network */
 	net = pp->network;
+	np = pp->parent;
 	if (net->temp2 == 0)
 	{
-		net->temp2 = net_findotherexporttopology(pp->parent, pp->protoname);
-		if (net->temp2 == 0) net->temp2 = getprime((*index)++);
+		net->temp2 = net_findotherexporttopology(np, pp->protoname);
+		if (net->temp2 == 0)
+		{
+			/* this name not found: see if another name in this facet is connected */
+			for(opp = np->firstportproto; opp != NOPORTPROTO; opp = opp->nextportproto)
+			{
+				if (opp == pp) continue;
+				if (opp->network != pp->network) continue;
+				net->temp2 = net_findotherexporttopology(np, opp->protoname);
+				if (net->temp2 != 0) break;
+			}
+
+			/* if no name found, see if power/ground association can be made */
+			if (net->temp2 == 0)
+			{
+				bits = pp->userbits & STATEBITS;
+				if (bits == PWRPORT || bits == GNDPORT)
+					net->temp2 = net_findotherexportcharacteristic(np, bits);
+			}
+			if (net->temp2 == 0)
+				net->temp2 = getprime((*index)++);
+		}
 	}
-	if (net->signals > 1)
+	if (net->buswidth > 1)
 	{
-		for(i=0; i<net->signals; i++)
+		for(i=0; i<net->buswidth; i++)
 		{
 			subnet = net->networklist[i];
 			if (subnet->temp2 == 0)
 			{
 				if (subnet->namecount > 0) name = subnet->netname; else
 					name = pp->protoname;
-				subnet->temp2 = net_findotherexporttopology(pp->parent, name);
+				subnet->temp2 = net_findotherexporttopology(np, name);
 				if (subnet->temp2 == 0) subnet->temp2 = getprime((*index)++);
 			}
 		}
 	}
 }
 
+/*
+ * Helper routine to look at all facets associated with "parent" (i.e. in the same cell
+ * but with a different view) and find an export with the name "exportname".  Returns
+ * the "temp2" field (unique ID) of that network.  Returns zero if no association can be found.
+ */
 INTBIG net_findotherexporttopology(NODEPROTO *parent, char *exportname)
 {
 	REGISTER NODEPROTO *np;
@@ -1883,11 +2039,8 @@ INTBIG net_findotherexporttopology(NODEPROTO *parent, char *exportname)
 		if (np->cell != parent->cell) continue;
 
 		/* facet from the same cell: look for an equivalent port */
-		for(opp = np->firstportproto; opp != NOPORTPROTO; opp = opp->nextportproto)
-		{
-			if (namesame(exportname, opp->protoname) != 0) continue;
-			if (opp->network->temp2 != 0) return(opp->network->temp2);
-		}
+		opp = getportproto(np, exportname);
+		if (opp != NOPORTPROTO && opp->network->temp2 != 0) return(opp->network->temp2);
 	}
 
 	/* next look for an equivalent network name in another facet */
@@ -1903,3 +2056,48 @@ INTBIG net_findotherexporttopology(NODEPROTO *parent, char *exportname)
 	}
 	return(0);
 }
+
+/*
+ * Helper routine to look at all facets associated with "parent" (i.e. in the same cell
+ * but with a different view) and find a network with the characteristics "bits".  Returns
+ * the "temp2" field (unique ID) of that network.  Returns zero if no association can be found.
+ */
+INTBIG net_findotherexportcharacteristic(NODEPROTO *parent, INTBIG bits)
+{
+	REGISTER NODEPROTO *np;
+	REGISTER PORTPROTO *opp;
+	REGISTER NETWORK *net;
+	REGISTER INTBIG obits;
+
+	/* first look for an equivalent export in another facet */
+	for(np = parent->cell->lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+	{
+		if (np == parent) continue;
+		if (np->cell != parent->cell) continue;
+
+		/* facet from the same cell: look for an equivalent port */
+		for(opp = np->firstportproto; opp != NOPORTPROTO; opp = opp->nextportproto)
+		{
+			obits = opp->userbits & STATEBITS;
+			if (bits != obits) continue;
+			if (opp->network->temp2 != 0) return(opp->network->temp2);
+		}
+	}
+
+	/* next look for an equivalent network name in another facet */
+	for(np = parent->cell->lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+	{
+		if (np == parent) continue;
+		if (np->cell != parent->cell) continue;
+
+		/* facet from the same cell: look for an equivalent network name */
+		for(net = np->firstnetwork; net != NONETWORK; net = net->nextnetwork)
+		{
+			if (net->globalnet < 0) continue;
+			if (net->globalnet >= np->globalnetcount) continue;
+			if (np->globalnetchar[net->globalnet] != bits) continue;
+			if (net->temp2 != 0) return(net->temp2);
+		}
+	}
+	return(0);
+}
