@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: planopt.c
@@ -348,14 +349,14 @@ void pla_SetFlag(void)
 					pla_VddWidth = pla_lam * pla_GetVal();
 				if (pla_VddWidth < 4 * pla_lam) pla_VddWidth = 4 * pla_lam;
 				pla_halfVdd = pla_VddWidth / 2;
-				if (pla_verbose) ttyputmsg(M_("'Vdd Width' set to %s"), latoa(pla_VddWidth));
+				if (pla_verbose) ttyputmsg(M_("'Vdd Width' set to %s"), latoa(pla_VddWidth, 0));
 				break;
 			case GNDWIDTH:
 				if (!pla_ReadOffEqualSign())
 					pla_GndWidth = pla_lam * pla_GetVal();
 				if (pla_GndWidth < 4 * pla_lam) pla_GndWidth = 4 * pla_lam;
 				pla_halfGnd = pla_GndWidth / 2;
-				if (pla_verbose) ttyputmsg(M_("'Gnd Width' set to %s"), latoa(pla_GndWidth));
+				if (pla_verbose) ttyputmsg(M_("'Gnd Width' set to %s"), latoa(pla_GndWidth, 0));
 				break;
 			case VERBOSE:
 			case FLEXIBLE:
@@ -437,8 +438,8 @@ void pla_displayoptions(void)
 	ttyputmsg(M_("       Fixedangle = %s"), pla_onoffmessage(pla_userbits&FIXANG));
 	ttyputmsg(M_("   Buttingcontact = %s"), pla_onoffmessage(pla_buttingcontact));
 	ttyputmsg(M_("   Samesideoutput = %s"), pla_onoffmessage(pla_samesideoutput));
-	ttyputmsg(M_("	     VDD Width = %s"), latoa(pla_VddWidth));
-	ttyputmsg(M_("	     Gnd Width = %s"), latoa(pla_GndWidth));
+	ttyputmsg(M_("	     VDD Width = %s"), latoa(pla_VddWidth, 0));
+	ttyputmsg(M_("	     Gnd Width = %s"), latoa(pla_GndWidth, 0));
 	ttyputmsg(M_("  Name of the PLA = %s"), pla_name);
 }
 
@@ -532,7 +533,7 @@ void pla_setoption(char *option, char *value)
 				{
 					pla_VddWidth = val*pla_lam;
 					pla_halfVdd  = pla_VddWidth / 2;
-					ttyputverbose(M_("VDD width set to %s"), latoa(pla_VddWidth));
+					ttyputverbose(M_("VDD width set to %s"), latoa(pla_VddWidth, 0));
 				}
 				break;
 			case GNDWIDTH:
@@ -540,7 +541,7 @@ void pla_setoption(char *option, char *value)
 				{
 					pla_GndWidth = val*pla_lam;
 					pla_halfGnd  = pla_GndWidth / 2;
-					ttyputverbose(M_("Gnd width set to %s"), latoa(pla_GndWidth));
+					ttyputverbose(M_("Gnd width set to %s"), latoa(pla_GndWidth, 0));
 				}
 				break;
 		}
