@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: planprog1.c
@@ -168,12 +169,12 @@ BOOLEAN pla_Make(void)
 	if (pla_verbose)
 	{
 		ttyputmsg(M_("Facets have been defined with sizes given below:"));
-		ttyputmsg(M_("Pullup  = %sx%s"), latoa(puX), latoa(puY));
-		ttyputmsg(M_("Input   = %sx%s"), latoa(inX), latoa(inY));
-		ttyputmsg(M_("Output  = %sx%s"), latoa(outX), latoa(outY));
-		ttyputmsg(M_("Program = %sx%s"), latoa(progX), latoa(progY));
-		ttyputmsg(M_("Connect = %sx%s"), latoa(connectX), latoa(connectY));
-		ttyputmsg(M_("VDD Width = %s, Gnd Width = %s"), latoa(pla_VddWidth), latoa(pla_GndWidth));
+		ttyputmsg(M_("Pullup  = %sx%s"), latoa(puX, 0), latoa(puY, 0));
+		ttyputmsg(M_("Input   = %sx%s"), latoa(inX, 0), latoa(inY, 0));
+		ttyputmsg(M_("Output  = %sx%s"), latoa(outX, 0), latoa(outY, 0));
+		ttyputmsg(M_("Program = %sx%s"), latoa(progX, 0), latoa(progY, 0));
+		ttyputmsg(M_("Connect = %sx%s"), latoa(connectX, 0), latoa(connectY, 0));
+		ttyputmsg(M_("VDD Width = %s, Gnd Width = %s"), latoa(pla_VddWidth, 0), latoa(pla_GndWidth, 0));
 	}
 
 	/* Create the pla prototype with these dimensions.
@@ -710,9 +711,9 @@ BOOLEAN pla_ProgramPLA(INTBIG inX, INTBIG inY, INTBIG puX, INTBIG puY, INTBIG ou
 	if (pla_verbose)
 	{
 		ttyputmsg(M_("Dimensions of the planes:"));
-		ttyputmsg(M_("  andX = %s,    andY = %s"), latoa(andX), latoa(andY));
-		ttyputmsg(M_("   orX = %s,     orY = %s"), latoa(orX), latoa(andY));
-		ttyputmsg(M_("orLeft = %s, orRight = %s"), latoa(orLeft), latoa(orRight));
+		ttyputmsg(M_("  andX = %s,    andY = %s"), latoa(andX, 0), latoa(andY, 0));
+		ttyputmsg(M_("   orX = %s,     orY = %s"), latoa(orX, 0), latoa(andY, 0));
+		ttyputmsg(M_("orLeft = %s, orRight = %s"), latoa(orLeft, 0), latoa(orRight, 0));
 	}
 
 	/* Place all the input, output and pullup facets */
