@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: routauto.c
@@ -69,7 +70,7 @@ void ro_autostitch(void)
 	if ((net_tool->toolstate&TOOLON) == 0)
 	{
 		ttyputerr(_("Network tool must be running...turning it on"));
-		toolturnon(net_tool, FALSE);
+		toolturnon(net_tool);
 		return;
 	}
 
@@ -153,7 +154,7 @@ void ro_autostitch(void)
 		if (!stopping(STOPREASONROUTING))
 		{
 			ni = r->entity;
-			if ((ni->parent->userbits&NLOCKED) != 0) continue;
+			if ((ni->parent->userbits&NPLOCKED) != 0) continue;
 			count += ro_checkstitching(ni);
 		}
 		ro_freercheck(r);
@@ -194,6 +195,7 @@ INTBIG ro_checkstitching(NODEINST *ni)
 	REGISTER PORTPROTO *pp, *rpp, *bestpp;
 	REGISTER PORTARCINST *pi;
 	REGISTER ARCPROTO *ap;
+	REGISTER POLYGON *polyptr;
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
@@ -350,7 +352,7 @@ INTBIG ro_checkstitching(NODEINST *ni)
 
 			/* look at all polygons on this nodeinst */
 			useportpoly = 0;
-			tot = nodeEpolys(ni, 0, NOWINDOWPART);
+			tot = allnodeEpolys(ni, ro_autostitchplist, NOWINDOWPART, TRUE);
 			if (tot == 0)
 			{
 				useportpoly = 1;
@@ -374,15 +376,16 @@ INTBIG ro_checkstitching(NODEINST *ni)
 					if (bestpp == NOPORTPROTO) continue;
 					rpp = bestpp;
 					shapeportpoly(ni, rpp, poly, FALSE);
+					polyptr = poly;
 				} else
 				{
-					shapeEnodepoly(ni, j, poly);
+					polyptr = ro_autostitchplist->polygons[j];
 
 					/* only want electrically connected polygons */
-					if (poly->portproto == NOPORTPROTO) continue;
+					if (polyptr->portproto == NOPORTPROTO) continue;
 
 					/* if the polygon layer is pseudo, substitute real layer */
-					poly->layer = nonpseudolayer(poly->layer, poly->tech);
+					polyptr->layer = nonpseudolayer(polyptr->layer, polyptr->tech);
 
 					/* get the correct port connected to this polygon */
 					rpp = NOPORTPROTO;
@@ -390,7 +393,7 @@ INTBIG ro_checkstitching(NODEINST *ni)
 					/* search all ports for the closest connected to this layer */
 					bestpp = NOPORTPROTO;
 					for(rpp = ni->proto->firstportproto; rpp != NOPORTPROTO; rpp = rpp->nextportproto)
-						if (rpp->network == poly->portproto->network)
+						if (rpp->network == polyptr->portproto->network)
 					{
 						/* compute best distance to the other node */
 						portposition(ni, rpp, &x, &y);
@@ -403,7 +406,7 @@ INTBIG ro_checkstitching(NODEINST *ni)
 					rpp = bestpp;
 
 					/* transformed the polygon */
-					xformpoly(poly, trans);
+					xformpoly(polyptr, trans);
 				}
 
 				/* stop now if already an arc on this port to other node */
@@ -427,11 +430,11 @@ INTBIG ro_checkstitching(NODEINST *ni)
 					ro_findsmallestlayer(ap);
 					if (useportpoly == 0)
 					{
-						if (!samelayer(ap->tech, ap->temp1, poly->layer)) continue;
+						if (!samelayer(ap->tech, ap->temp1, polyptr->layer)) continue;
 					}
 
 					/* pass it on to the next test */
-					stitched = ro_testpoly(ni, rpp, ap, poly, oni);
+					stitched = ro_testpoly(ni, rpp, ap, polyptr, oni);
 					count += stitched;
 					break;
 				}
@@ -451,11 +454,11 @@ INTBIG ro_checkstitching(NODEINST *ni)
 						ro_findsmallestlayer(ap);
 						if (useportpoly == 0)
 						{
-							if (!samelayer(ap->tech, ap->temp1, poly->layer)) continue;
+							if (!samelayer(ap->tech, ap->temp1, polyptr->layer)) continue;
 						}
 
 						/* pass it on to the next test */
-						stitched = ro_testpoly(ni, rpp, ap, poly, oni);
+						stitched = ro_testpoly(ni, rpp, ap, polyptr, oni);
 						count += stitched;
 						if (stitched != 0) break;
 					}
@@ -700,7 +703,7 @@ INTBIG ro_comparepoly(NODEINST *oni, PORTPROTO *opp, POLYGON *opoly, NODEINST *n
 	x = (ox+x) / 2;   y = (oy+y) / 2;
 
 	/* run the wire */
-	newai = aconnect(ni->geom, pp, oni->geom, opp, ap, x, y, &alt1, &alt2, &con1, &con2, 900, TRUE);
+	newai = aconnect(ni->geom, pp, oni->geom, opp, ap, x, y, &alt1, &alt2, &con1, &con2, 900, TRUE, TRUE);
 	ret = ro_didaccept(newai, alt1, alt2, con1, con2);
 	return(ret);
 }
