@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: routmimic.c
@@ -34,15 +35,16 @@
 #include "global.h"
 #include "rout.h"
 #include "usr.h"
+#include "edialogs.h"
 
 #define NOPOSSIBLEARC ((POSSIBLEARC *)-1)
 #define LIKELYDIFFPORT     1
-#define LIKELYDIFFNODETYPE 2
-#define LIKELYDIFFNODESIZE 4
+#define LIKELYDIFFARCCOUNT 2
+#define LIKELYDIFFNODETYPE 4
+#define LIKELYDIFFNODESIZE 8
 
 typedef struct Ipossiblearc
 {
-	ARCINST             *ai;
 	NODEINST            *ni1, *ni2;
 	PORTPROTO           *pp1, *pp2;
 	INTBIG               x1, y1, x2, y2;
@@ -57,7 +59,13 @@ static POLYGON    **ro_mimicportpolys;
 
 /* prototypes for local routines */
 static void         ro_mimicdelete(ARCPROTO *typ, NODEINST **nodes, PORTPROTO **ports);
-static INTBIG       ro_mimiccreate(ARCINST *arc);
+static INTBIG       ro_mimiccreatedarc(ARCINST *arc);
+static INTBIG       ro_mimiccreatedany(NODEINST *ni1, PORTPROTO *pp1, INTBIG x1, INTBIG y1,
+						NODEINST *ni2, PORTPROTO *pp2, INTBIG x2, INTBIG y2,
+						INTBIG width, INTBIG userbits, ARCPROTO *proto, INTBIG prefx, INTBIG prefy);
+static INTBIG       ro_mimicthis(NODEINST *ni1, PORTPROTO *pp1, INTBIG x1, INTBIG y1,
+						NODEINST *ni2, PORTPROTO *pp2, INTBIG x2, INTBIG y2,
+						INTBIG width, INTBIG userbits, ARCPROTO *proto, INTBIG prefx, INTBIG prefy);
 static POSSIBLEARC *ro_allocpossiblearc(void);
 static void         ro_freepossiblearc(POSSIBLEARC *pa);
 
@@ -88,42 +96,120 @@ void ro_mimicstitch(BOOLEAN forced)
 {
 	REGISTER ARCPROTO *ap;
 	REGISTER ARCINST *ai;
+	REGISTER NODEINST *ni;
 	NODEINST *endnode[2];
 	PORTPROTO *endport[2];
-	REGISTER INTBIG options, i;
+	INTBIG endx[2], endy[2], x0, y0, x1, y1;
+	ARCPROTO *proto;
+	REGISTER NODEPROTO *parent;
+	REGISTER INTBIG options, i, width, userbits, foundends, e, prefx, prefy;
 	float elapsed;
 
-	/* if an arc was deleted, and that is being mimiced, do it */
 	options = ro_getoptions();
-	if (ro_deletedarc != NOARCPROTO && (options&MIMICUNROUTES) != 0)
+
+	/* if a single arc was deleted, and that is being mimiced, do it */
+	if (ro_lastactivity.numdeletedarcs == 1 && (options&MIMICUNROUTES) != 0)
 	{
 		for(i=0; i<2; i++)
 		{
 			endnode[i] = ro_deletednodes[i];
 			endport[i] = ro_deletedports[i];
 		}
-		ap = ro_deletedarc;
-		ro_deletedarc = NOARCPROTO;
+		ap = ro_lastactivity.deletedarcs[0]->proto;
 		ro_mimicdelete(ap, endnode, endport);
 	}
 
-	/* if an arc was just created, mimic that */
-	if (ro_createdarc != NOARCINST)
+	/* if a single arc was just created, mimic that */
+	if (ro_lastactivity.numcreatedarcs == 1)
 	{
-		ai = ro_createdarc;
-		ro_createdarc = NOARCINST;
+		ai = ro_lastactivity.createdarcs[0];
 		starttimer();
-		if (ro_mimiccreate(ai) == 0)
+		if (ro_mimiccreatedarc(ai) == 0)
 		{
 			/* nothing was wired */
 			if (forced) ttyputmsg(_("No wires mimiced"));
 		}
 		elapsed = endtimer();
 		if (elapsed > 60.0 && (us_useroptions&BEEPAFTERLONGJOB) != 0)
-			ttybeep(1);
+			ttybeep(SOUNDBEEP, TRUE);
 		if ((options&MIMICINTERACTIVE) == 0 && elapsed >= 2.0)
 			ttyputmsg(_("Mimicing took %s"), explainduration(elapsed));
 	}
+
+	/* if multiple arcs were just created, find the true end and mimic that */
+	if (ro_lastactivity.numcreatedarcs > 1 && ro_lastactivity.numcreatednodes > 0)
+	{
+		/* find the ends of arcs that do not attach to the intermediate pins */
+		parent = ro_lastactivity.createdarcs[0]->parent;
+		for(ni = parent->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+			ni->temp1 = 0;
+		for(i=0; i<ro_lastactivity.numcreatednodes; i++)
+			ro_lastactivity.creatednodes[i]->temp1 = 0;
+		for(i=0; i<ro_lastactivity.numcreatedarcs; i++)
+		{
+			ro_lastactivity.createdarcs[i]->end[0].nodeinst->temp1++;
+			ro_lastactivity.createdarcs[i]->end[1].nodeinst->temp1++;
+		}
+		foundends = 0;
+		width = 0;
+		for(i=0; i<ro_lastactivity.numcreatedarcs; i++)
+		{
+			for(e=0; e<2; e++)
+			{
+				if (ro_lastactivity.createdarcs[i]->end[e].nodeinst->temp1 != 1) continue;
+				if (foundends < 2)
+				{
+					endnode[foundends] = ro_lastactivity.createdarcs[i]->end[e].nodeinst;
+					endport[foundends] = ro_lastactivity.createdarcs[i]->end[e].portarcinst->proto;
+					endx[foundends] = ro_lastactivity.createdarcs[i]->end[e].xpos;
+					endy[foundends] = ro_lastactivity.createdarcs[i]->end[e].ypos;
+					if (ro_lastactivity.createdarcs[i]->width > width)
+						width = ro_lastactivity.createdarcs[i]->width;
+					userbits = ro_lastactivity.createdarcs[i]->userbits;
+					proto = ro_lastactivity.createdarcs[i]->proto;
+				}
+				foundends++;
+			}
+		}
+
+		/* if exactly two ends are found, mimic that connection */
+		if (foundends == 2)
+		{
+			starttimer();
+			prefx = prefy = 0;
+			if (ro_lastactivity.numcreatednodes == 1)
+			{
+				portposition(endnode[0], endport[0], &x0, &y0);
+				portposition(endnode[1], endport[1], &x1, &y1);
+				prefx = (ro_lastactivity.creatednodes[0]->lowx + ro_lastactivity.creatednodes[0]->highx) / 2 -
+					(x0+x1) / 2;
+				prefy = (ro_lastactivity.creatednodes[0]->lowy + ro_lastactivity.creatednodes[0]->highy) / 2 -
+					(y0+y1) / 2;
+			} else if (ro_lastactivity.numcreatednodes == 2)
+			{
+				portposition(endnode[0], endport[0], &x0, &y0);
+				portposition(endnode[1], endport[1], &x1, &y1);
+				prefx = (ro_lastactivity.creatednodes[0]->lowx + ro_lastactivity.creatednodes[0]->highx +
+					ro_lastactivity.creatednodes[1]->lowx + ro_lastactivity.creatednodes[1]->highx) / 4 -
+						(x0+x1) / 2;
+				prefy = (ro_lastactivity.creatednodes[0]->lowy + ro_lastactivity.creatednodes[0]->highy +
+					ro_lastactivity.creatednodes[1]->lowy + ro_lastactivity.creatednodes[1]->highy) / 4 -
+						(y0+y1) / 2;
+			}
+			if (ro_mimiccreatedany(endnode[0], endport[0], endx[0], endy[0],
+				endnode[1], endport[1], endx[1], endy[1],
+				width, userbits, proto, prefx, prefy) == 0)
+			{
+				/* nothing was wired */
+				if (forced) ttyputmsg(_("No wires mimiced"));
+			}
+			elapsed = endtimer();
+			if (elapsed > 60.0 && (us_useroptions&BEEPAFTERLONGJOB) != 0)
+				ttybeep(SOUNDBEEP, TRUE);
+			if ((options&MIMICINTERACTIVE) == 0 && elapsed >= 2.0)
+				ttyputmsg(_("Mimicing took %s"), explainduration(elapsed));
+		}
+	}
 }
 
 /*
@@ -189,44 +275,109 @@ void ro_mimicdelete(ARCPROTO *typ, NODEINST **nodes, PORTPROTO **ports)
 /*
  * Routine to mimic the creation of arc "arc".  Returns the number of mimics made.
  */
-INTBIG ro_mimiccreate(ARCINST *ai)
+INTBIG ro_mimiccreatedany(NODEINST *ni1, PORTPROTO *pp1, INTBIG x1, INTBIG y1,
+	NODEINST *ni2, PORTPROTO *pp2, INTBIG x2, INTBIG y2,
+	INTBIG width, INTBIG userbits, ARCPROTO *proto, INTBIG prefx, INTBIG prefy)
+{
+	return(ro_mimicthis(ni1, pp1, x1, y1, ni2, pp2, x2, y2, width, userbits, proto, prefx, prefy));
+}
+
+/*
+ * Routine to mimic the creation of arc "arc".  Returns the number of mimics made.
+ */
+INTBIG ro_mimiccreatedarc(ARCINST *ai)
+{
+	REGISTER INTBIG x1, y1, x2, y2;
+
+	x1 = ai->end[0].xpos;   y1 = ai->end[0].ypos;
+	x2 = ai->end[1].xpos;   y2 = ai->end[1].ypos;
+	return(ro_mimicthis(ai->end[0].nodeinst, ai->end[0].portarcinst->proto, x1, y1,
+		ai->end[1].nodeinst, ai->end[1].portarcinst->proto, x2, y2,
+		ai->width, ai->userbits, ai->proto, 0, 0));
+}
+
+/* Prompt: Yes/No with "stop asking" */
+static DIALOGITEM ro_yesnostopdialogitems[] =
+{
+ /*  1 */ {0, {64,156,88,228}, BUTTON, N_("Yes")},
+ /*  2 */ {0, {64,68,88,140}, BUTTON, N_("No")},
+ /*  3 */ {0, {6,15,54,279}, MESSAGE, ""},
+ /*  4 */ {0, {96,156,120,276}, BUTTON, N_("Yes, then stop")},
+ /*  5 */ {0, {96,20,120,140}, BUTTON, N_("No, and stop")}
+};
+static DIALOG ro_yesnostopdialog = {{50,75,179,363}, N_("Warning"), 0, 5, ro_yesnostopdialogitems, 0, 0};
+
+#define DYNS_YES     1		/* "Yes" (button) */
+#define DYNS_NO      2		/* "No" (button) */
+#define DYNS_MESSAGE 3		/* routing message (stat text) */
+#define DYNS_YESSTOP 4		/* "Yes, then stop" (button) */
+#define DYNS_NOSTOP  5		/* "No, and stop" (button) */
+
+INTBIG ro_mimicthis(NODEINST *oni1, PORTPROTO *opp1, INTBIG ox1, INTBIG oy1,
+	NODEINST *oni2, PORTPROTO *opp2, INTBIG ox2, INTBIG oy2,
+	INTBIG owidth, INTBIG ouserbits, ARCPROTO *oproto, INTBIG prefx, INTBIG prefy)
 {
 	REGISTER NODEPROTO *facet, *proto0, *proto1;
 	REGISTER NODEINST *ni, *oni, *node0, *node1;
+	NODEINST *endni[2], *nicon1, *nicon2;
 	REGISTER PORTPROTO *port0, *port1, *pp, *opp;
+	PORTPROTO *endpp[2];
 	REGISTER PORTARCINST *pi;
 	REGISTER ARCINST *newai, *oai;
+	ARCINST *alt1, *alt2;
 	REGISTER INTBIG dist, angle, options, count, i, wantx1, wanty1, end, end0offx, end0offy,
-		situation, total, node0wid, node0hei, node1wid, node1hei, wid, w, hei,
-		portpos, oportpos, stopcheck, ifignoreports, ifignorenodetype, ifignorenodesize;
-	INTBIG x0, y0, x1, y1, x0c, y0c, desiredangle, existingangle, thisend, lx, hx, ly, hy;
+		situation, total, node0wid, node0hei, node1wid, node1hei, wid, hei, con1, con2,
+		portpos, oportpos, stopcheck, ifignoreports, ifignorenodetype, ifignorenodesize,
+		ifignorearccount, itemHit;
+	INTBIG x0, y0, x1, y1, x0c, y0c, desiredangle, existingangle, thisend, lx, hx, ly, hy,
+		endx[2], endy[2];
 	static POLYGON *poly = NOPOLYGON;
 	REGISTER POLYGON *thispoly, **newportpolys;
 	REGISTER POSSIBLEARC *firstpossiblearc, *pa;
-	extern COMCOMP us_yesnop;
-	char *pars[5], question[200];
-#define NUMSITUATIONS 6
+	char question[200];
+#define NUMSITUATIONS 16
 	static INTBIG situations[] = {
-		0,														/* everything matches */
-		LIKELYDIFFNODESIZE,										/* different node size */
-		LIKELYDIFFPORT,											/* different ports */
-		LIKELYDIFFPORT|LIKELYDIFFNODESIZE,						/* different port and node size */
-		LIKELYDIFFNODETYPE|LIKELYDIFFPORT,						/* different ports and node type */
-		LIKELYDIFFNODETYPE|LIKELYDIFFPORT|LIKELYDIFFNODESIZE	/* different ports, node type and size */
+		0,
+		LIKELYDIFFNODESIZE,
+		                   LIKELYDIFFARCCOUNT,
+		                                      LIKELYDIFFPORT,
+		                                                     LIKELYDIFFNODETYPE,
+		LIKELYDIFFNODESIZE|LIKELYDIFFARCCOUNT,
+		LIKELYDIFFNODESIZE|                   LIKELYDIFFPORT,
+		LIKELYDIFFNODESIZE|                                  LIKELYDIFFNODETYPE,
+		                   LIKELYDIFFARCCOUNT|LIKELYDIFFPORT,
+		                   LIKELYDIFFARCCOUNT|               LIKELYDIFFNODETYPE,
+		                                      LIKELYDIFFPORT|LIKELYDIFFNODETYPE,
+
+		LIKELYDIFFNODESIZE|LIKELYDIFFARCCOUNT|LIKELYDIFFPORT,
+		LIKELYDIFFNODESIZE|LIKELYDIFFARCCOUNT|               LIKELYDIFFNODETYPE,
+		LIKELYDIFFNODESIZE|                   LIKELYDIFFPORT|LIKELYDIFFNODETYPE,
+		                   LIKELYDIFFARCCOUNT|LIKELYDIFFPORT|LIKELYDIFFNODETYPE,
+		LIKELYDIFFNODESIZE|LIKELYDIFFARCCOUNT|LIKELYDIFFPORT|LIKELYDIFFNODETYPE
 	};
-	REGISTER void *infstr;
+	REGISTER void *infstr, *dia;
 
 	ttyputmsg(_("Mimicing last arc..."));
 
 	/* get polygon */
 	(void)needstaticpolygon(&poly, 5, ro_tool->cluster);
 
-	facet = ai->parent;
+	facet = oni1->parent;
+	endni[0] = oni1;   endni[1] = oni2;
+	endpp[0] = opp1;   endpp[1] = opp2;
+	endx[0] = ox1;     endx[1] = ox2;
+	endy[0] = oy1;     endy[1] = oy2;
 	options = ro_getoptions();
 	count = 0;
 	stopcheck = 0;
 	firstpossiblearc = NOPOSSIBLEARC;
 
+	/* count the number of other arcs on the ends */
+	con1 = 0;
+	for(pi = oni1->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst) con1++;
+	for(pi = oni2->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst) con1++;
+	con1 -= 2;
+
 	/* precompute information about every port in the facet */
 	total = 0;
 	for(ni = facet->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
@@ -235,7 +386,7 @@ INTBIG ro_mimiccreate(ARCINST *ai)
 		{
 			pp->temp1 = 0;
 			for(i=0; pp->connects[i] != NOARCPROTO; i++)
-				if (pp->connects[i] == ai->proto) break;
+				if (pp->connects[i] == oproto) break;
 			if (pp->connects[i] == NOARCPROTO) continue;
 			pp->temp1 = 1;
 			total++;
@@ -269,14 +420,14 @@ INTBIG ro_mimiccreate(ARCINST *ai)
 	/* search from both ends */
 	for(end=0; end<2; end++)
 	{
-		node0 = ai->end[end].nodeinst;
-		node1 = ai->end[1-end].nodeinst;
+		node0 = endni[end];
+		node1 = endni[1-end];
 		proto0 = node0->proto;
 		proto1 = node1->proto;
-		port0 = ai->end[end].portarcinst->proto;
-		port1 = ai->end[1-end].portarcinst->proto;
-		x0 = ai->end[end].xpos;     y0 = ai->end[end].ypos;
-		x1 = ai->end[1-end].xpos;   y1 = ai->end[1-end].ypos;
+		port0 = endpp[end];
+		port1 = endpp[1-end];
+		x0 = endx[end];     y0 = endy[end];
+		x1 = endx[1-end];   y1 = endy[1-end];
 		dist = computedistance(x0, y0, x1, y1);
 		if (dist != 0)
 			angle = figureangle(x0, y0, x1, y1);
@@ -409,6 +560,10 @@ INTBIG ro_mimiccreate(ARCINST *ai)
 						situation = 0;
 						if (pp != port0 || opp != port1)
 							situation |= LIKELYDIFFPORT;
+						con2 = 0;
+						for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst) con2++;
+						for(pi = oni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst) con2++;
+						if (con1 != con2) situation |= LIKELYDIFFARCCOUNT;
 						if (ni->proto != proto0 || oni->proto != proto1)
 							situation |= LIKELYDIFFNODETYPE;
 						nodesizeoffset(ni, &lx, &ly, &hx, &hy);
@@ -444,7 +599,6 @@ INTBIG ro_mimiccreate(ARCINST *ai)
 							pa->nextpossiblearc = firstpossiblearc;
 							firstpossiblearc = pa;
 						}
-						pa->ai = ai;
 						pa->ni1 = ni;      pa->pp1 = pp;
 						pa->x1 = x0;       pa->y1 = y0;
 						pa->ni2 = oni;     pa->pp2 = opp;
@@ -456,8 +610,11 @@ INTBIG ro_mimiccreate(ARCINST *ai)
 		}
 	}
 
+	/* turn off network tool before mimicing to speed up reevaluation */
+	toolturnoff(net_tool, FALSE);
+
 	/* now create the mimiced arcs */
-	ifignoreports = ifignorenodetype = ifignorenodesize = 0;
+	ifignoreports = ifignorearccount = ifignorenodetype = ifignorenodesize = 0;
 	for(i=0; i<NUMSITUATIONS; i++)
 	{
 		/* see if this situation is possible */
@@ -475,6 +632,11 @@ INTBIG ro_mimiccreate(ARCINST *ai)
 				ifignoreports += total;
 				continue;
 			}
+			if ((options&MIMICONSIDERARCCOUNT) != 0 && (situations[i]&LIKELYDIFFARCCOUNT) != 0)
+			{
+				ifignorearccount += total;
+				continue;
+			}
 			if ((options&MIMICIGNORENODETYPE) == 0 && (situations[i]&LIKELYDIFFNODETYPE) != 0)
 			{
 				ifignorenodetype += total;
@@ -504,33 +666,50 @@ INTBIG ro_mimiccreate(ARCINST *ai)
 				}
 			}
 			sprintf(question, _("Create %ld %s shown here?"), total, makeplural(_("wire"), total));
-			total = ttygetparam(question, &us_yesnop, 5, pars);
+			dia = DiaInitDialog(&ro_yesnostopdialog);
+			if (dia == 0)
+			{
+				toolturnon(net_tool);
+				return(0);
+			}
+			DiaSetText(dia, DYNS_MESSAGE, question);
+			for(;;)
+			{
+				itemHit = DiaNextHit(dia);
+				if (itemHit == DYNS_YES || itemHit == DYNS_YESSTOP ||
+					itemHit == DYNS_NO || itemHit == DYNS_NOSTOP) break;
+			}
+			DiaDoneDialog(dia);
 			(void)asktool(us_tool, "up-stack");
-			if (total > 0 && namesamen(pars[0], "no", strlen(pars[0])) == 0) continue;
+			if (itemHit == DYNS_NO) continue;
+			if (itemHit == DYNS_NOSTOP) break;
 		}
 
 		/* make the wires */
 		for(pa = firstpossiblearc; pa != NOPOSSIBLEARC; pa = pa->nextpossiblearc)
 		{
 			if (pa->situation != situations[i]) continue;
-			wid = defaultarcwidth(pa->ai->proto);
-			w = us_widestarcinst(pa->ai->proto, pa->ni1, pa->pp1);
-			if (w > wid) wid = w;
-			w = us_widestarcinst(pa->ai->proto, pa->ni2, pa->pp2);
-			if (w > wid) wid = w;
-			newai = newarcinst(pa->ai->proto, wid, pa->ai->userbits,
-				pa->ni1, pa->pp1, pa->x1, pa->y1, pa->ni2, pa->pp2, pa->x2, pa->y2, facet);
+			portposition(pa->ni1, pa->pp1, &x0, &y0);
+			portposition(pa->ni2, pa->pp2, &x1, &y1);
+			newai = aconnect(pa->ni1->geom, pa->pp1, pa->ni2->geom, pa->pp2,
+				oproto, (x0+x1)/2+prefx, (y0+y1)/2+prefy, &alt1, &alt2, &nicon1, &nicon2, 900, FALSE, FALSE);
 			if (newai == NOARCINST)
 			{
 				ttyputerr(_("Problem creating arc"));
+				toolturnon(net_tool);
 				return(count);
 			}
-			endobjectchange((INTBIG)newai, VARCINST);
 			count++;
 		}
 		(void)asktool(us_tool, "flush-changes");
+
+		/* stop now if requested */
+		if ((options&MIMICINTERACTIVE) != 0 && itemHit == DYNS_YESSTOP) break;
 	}
 
+	/* turn network tool back on */
+	toolturnon(net_tool);
+
 	/* free memory for mimicing */
 	while (firstpossiblearc != NOPOSSIBLEARC)
 	{
@@ -539,12 +718,16 @@ INTBIG ro_mimiccreate(ARCINST *ai)
 		ro_freepossiblearc(pa);
 	}
 	if (count != 0)
-		ttyputmsg(_("MIMIC ROUTING: added %ld %s"), count, makeplural(_("wire"), count)); else
+	{
+		us_reportarcscreated(_("MIMIC ROUTING"), count, 0, 0, 0);
+	} else
 	{
 		infstr = initinfstr();
 		addstringtoinfstr(infstr, _("No wires added"));
 		if (ifignoreports != 0)
 			formatinfstr(infstr, _(", would add %ld wires if 'ports must match' were off"), ifignoreports);
+		if (ifignorearccount != 0)
+			formatinfstr(infstr, _(", would add %ld wires if 'number of existing arcs must match' were off"), ifignorearccount);
 		if (ifignorenodetype != 0)
 			formatinfstr(infstr, _(", would add %ld wires if 'node types must match' were off"), ifignorenodetype);
 		if (ifignorenodesize != 0)
