@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: routriver.c
@@ -1084,8 +1085,8 @@ BOOLEAN ro_sorted_rivrot(ARCPROTO *layerdesc, RDESC *listr, INTBIG width,
 	if (fixedheight > 0 && height > fixedheight)
 	{
 		/* can't make it in fixed height */
-		ttyputerr(_("River router: Unable to route in %s wide channel, need %s"), latoa(fixedheight),
-			latoa(height));
+		ttyputerr(_("River router: Unable to route in %s wide channel, need %s"), latoa(fixedheight, 0),
+			latoa(height, 0));
 		return(FALSE);
 	}
 	ro_calculate_bb(&ro_rrinfo.llx, &ro_rrinfo.lly, &ro_rrinfo.urx, &ro_rrinfo.ury,
@@ -1187,9 +1188,11 @@ void ro_pseudomake(RDESC *rd, NODEPROTO *facet)
 void ro_makepseudogeometry(NODEPROTO *facet)
 {
 	RDESC *q;
+	INTBIG lambda;
 
-	ttyputmsg(_("Routing bounds %s <= X <= %s   %s <= Y <= %s"), latoa(ro_rrinfo.llx),
-		latoa(ro_rrinfo.urx), latoa(ro_rrinfo.lly), latoa(ro_rrinfo.ury));
+	lambda = lambdaoffacet(facet);
+	ttyputmsg(_("Routing bounds %s <= X <= %s   %s <= Y <= %s"), latoa(ro_rrinfo.llx, lambda),
+		latoa(ro_rrinfo.urx, lambda), latoa(ro_rrinfo.lly, lambda), latoa(ro_rrinfo.ury, lambda));
 
 	/* remove highlighting */
 	(void)asktool(us_tool, "clear");
