@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: sc1electric.c
@@ -1778,7 +1779,7 @@ static DIALOGITEM sc_optionsdialogitems[] =
  /* 35 */ {0, {280,8,296,199}, MESSAGE, N_("N-Well offset from top:")},
  /* 36 */ {0, {280,204,296,278}, EDITTEXT, "2.5"}
 };
-static DIALOG sc_optionsdialog = {{50,75,583,472}, N_("Silicon Compiler Options"), 0, 36, sc_optionsdialogitems};
+static DIALOG sc_optionsdialog = {{50,75,583,472}, N_("Silicon Compiler Options"), 0, 36, sc_optionsdialogitems, 0, 0};
 
 /* special items for the "Silicon Compiler Options" dialog: */
 #define DSCO_VERTARCS      3		/* Vertical arc list (popup) */
@@ -1807,6 +1808,7 @@ void sc_optionsdlog(void)
 		mainpwrrail;
 	REGISTER ARCPROTO *ap, *aphoriz, *apvert;
 	char **arcnames, numstring[20], *transnames[2];
+	REGISTER void *dia;
 	static char *powerarcnames[2] = {N_("Horizontal Arc"), N_("Vertical Arc")};
 
 	/* get parameters */
@@ -1847,81 +1849,82 @@ void sc_optionsdlog(void)
 	}
 
 	/* display the silicon compiler options dialog box */
-	if (DiaInitDialog(&sc_optionsdialog)) return;
+	dia = DiaInitDialog(&sc_optionsdialog);
+	if (dia == 0) return;
 	for(i=0; i<2; i++) transnames[i] = _(powerarcnames[i]);
-	DiaSetPopup(DSCO_MPOWERARC, 2, transnames);
-	DiaSetPopup(DSCO_VERTARCS, numarcnames, arcnames);
-	DiaSetPopup(DSCO_HORIZARCS, numarcnames, arcnames);
-	DiaSetPopupEntry(DSCO_VERTARCS, vertindex);
-	DiaSetPopupEntry(DSCO_HORIZARCS, horizindex);
-	DiaSetText(DSCO_HORIZWIDTH, latoa(l1width));
-	DiaSetText(DSCO_VERTWIDTH, latoa(l2width));
-	DiaSetText(DSCO_POWERWIDTH, latoa(pwrwidth));
-	DiaSetText(DSCO_MPOWERWIDTH, latoa(mainpwrwidth));
-	DiaSetText(DSCO_PWELLHEIGHT, latoa(pwellsize));
-	DiaSetText(DSCO_PWELLOFFSET, latoa(pwelloffset));
-	DiaSetText(DSCO_VIASIZE, latoa(viasize));
-	DiaSetText(DSCO_MINSPACING, latoa(minspacing));
-	DiaSetText(DSCO_FEEDTHRU, latoa(feedthrusize));
-	DiaSetText(DSCO_PORTDIST, latoa(portxmindist));
-	DiaSetText(DSCO_ACTIVEDIST, latoa(activedist));
+	DiaSetPopup(dia, DSCO_MPOWERARC, 2, transnames);
+	DiaSetPopup(dia, DSCO_VERTARCS, numarcnames, arcnames);
+	DiaSetPopup(dia, DSCO_HORIZARCS, numarcnames, arcnames);
+	DiaSetPopupEntry(dia, DSCO_VERTARCS, vertindex);
+	DiaSetPopupEntry(dia, DSCO_HORIZARCS, horizindex);
+	DiaSetText(dia, DSCO_HORIZWIDTH, latoa(l1width, 0));
+	DiaSetText(dia, DSCO_VERTWIDTH, latoa(l2width, 0));
+	DiaSetText(dia, DSCO_POWERWIDTH, latoa(pwrwidth, 0));
+	DiaSetText(dia, DSCO_MPOWERWIDTH, latoa(mainpwrwidth, 0));
+	DiaSetText(dia, DSCO_PWELLHEIGHT, latoa(pwellsize, 0));
+	DiaSetText(dia, DSCO_PWELLOFFSET, latoa(pwelloffset, 0));
+	DiaSetText(dia, DSCO_VIASIZE, latoa(viasize, 0));
+	DiaSetText(dia, DSCO_MINSPACING, latoa(minspacing, 0));
+	DiaSetText(dia, DSCO_FEEDTHRU, latoa(feedthrusize, 0));
+	DiaSetText(dia, DSCO_PORTDIST, latoa(portxmindist, 0));
+	DiaSetText(dia, DSCO_ACTIVEDIST, latoa(activedist, 0));
 	sprintf(numstring, "%ld", numrows);
-	DiaSetText(DSCO_NUMROWS, numstring);
-	DiaSetPopupEntry(DSCO_MPOWERARC, mainpwrrail);
-	DiaSetText(DSCO_NWELLHEIGHT, latoa(nwellsize));
-	DiaSetText(DSCO_NWELLOFFSET, latoa(nwelloffset));
+	DiaSetText(dia, DSCO_NUMROWS, numstring);
+	DiaSetPopupEntry(dia, DSCO_MPOWERARC, mainpwrrail);
+	DiaSetText(dia, DSCO_NWELLHEIGHT, latoa(nwellsize, 0));
+	DiaSetText(dia, DSCO_NWELLOFFSET, latoa(nwelloffset, 0));
 
 	/* loop until done */
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == OK || itemHit == CANCEL) break;
 	}
 
 	if (itemHit != CANCEL)
 	{
-		i = DiaGetPopupEntry(DSCO_VERTARCS);
+		i = DiaGetPopupEntry(dia, DSCO_VERTARCS);
 		if (i == 0) ap = NOARCPROTO; else
 			ap = getarcproto(arcnames[i]);
 		if (ap != apvert) ScSetParameter(SC_PARAM_MAKE_VERT_ARC, (INTBIG)ap);
-		i = DiaGetPopupEntry(DSCO_HORIZARCS);
+		i = DiaGetPopupEntry(dia, DSCO_HORIZARCS);
 		if (i == 0) ap = NOARCPROTO; else
 			ap = getarcproto(arcnames[i]);
 		if (ap != aphoriz) ScSetParameter(SC_PARAM_MAKE_HORIZ_ARC, (INTBIG)ap);
 
-		i = atola(DiaGetText(DSCO_HORIZWIDTH));
+		i = atola(DiaGetText(dia, DSCO_HORIZWIDTH), 0);
 		if (i != l1width) ScSetParameter(SC_PARAM_MAKE_L1_WIDTH, i);
-		i = atola(DiaGetText(DSCO_VERTWIDTH));
+		i = atola(DiaGetText(dia, DSCO_VERTWIDTH), 0);
 		if (i != l2width) ScSetParameter(SC_PARAM_MAKE_L2_WIDTH, i);
-		i = atola(DiaGetText(DSCO_POWERWIDTH));
+		i = atola(DiaGetText(dia, DSCO_POWERWIDTH), 0);
 		if (i != pwrwidth) ScSetParameter(SC_PARAM_MAKE_PWR_WIDTH, i);
-		i = atola(DiaGetText(DSCO_MPOWERWIDTH));
+		i = atola(DiaGetText(dia, DSCO_MPOWERWIDTH), 0);
 		if (i != mainpwrwidth) ScSetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH, i);
-		i = atola(DiaGetText(DSCO_PWELLHEIGHT));
+		i = atola(DiaGetText(dia, DSCO_PWELLHEIGHT), 0);
 		if (i != pwellsize) ScSetParameter(SC_PARAM_MAKE_PWELL_SIZE, i);
-		i = atola(DiaGetText(DSCO_PWELLOFFSET));
+		i = atola(DiaGetText(dia, DSCO_PWELLOFFSET), 0);
 		if (i != pwelloffset) ScSetParameter(SC_PARAM_MAKE_PWELL_OFFSET, i);
-		i = atola(DiaGetText(DSCO_VIASIZE));
+		i = atola(DiaGetText(dia, DSCO_VIASIZE), 0);
 		if (i != viasize) ScSetParameter(SC_PARAM_MAKE_VIA_SIZE, i);
-		i = atola(DiaGetText(DSCO_MINSPACING));
+		i = atola(DiaGetText(dia, DSCO_MINSPACING), 0);
 		if (i != minspacing) ScSetParameter(SC_PARAM_MAKE_MIN_SPACING, i);
-		i = atola(DiaGetText(DSCO_FEEDTHRU));
+		i = atola(DiaGetText(dia, DSCO_FEEDTHRU), 0);
 		if (i != feedthrusize) ScSetParameter(SC_PARAM_ROUTE_FEEDTHRU_SIZE, i);
-		i = atola(DiaGetText(DSCO_PORTDIST));
+		i = atola(DiaGetText(dia, DSCO_PORTDIST), 0);
 		if (i != portxmindist) ScSetParameter(SC_PARAM_ROUTE_PORT_X_MIN_DIST, i);
-		i = atola(DiaGetText(DSCO_ACTIVEDIST));
+		i = atola(DiaGetText(dia, DSCO_ACTIVEDIST), 0);
 		if (i != activedist) ScSetParameter(SC_PARAM_ROUTE_ACTIVE_DIST, i);
-		i = atoi(DiaGetText(DSCO_NUMROWS));
+		i = atoi(DiaGetText(dia, DSCO_NUMROWS));
 		if (i != numrows) ScSetParameter(SC_PARAM_PLACE_NUM_ROWS, i);
-		i = DiaGetPopupEntry(DSCO_MPOWERARC);
+		i = DiaGetPopupEntry(dia, DSCO_MPOWERARC);
 		if (i != mainpwrrail) ScSetParameter(SC_PARAM_MAKE_MAIN_PWR_RAIL, i);
-		i = atola(DiaGetText(DSCO_NWELLHEIGHT));
+		i = atola(DiaGetText(dia, DSCO_NWELLHEIGHT), 0);
 		if (i != nwellsize) ScSetParameter(SC_PARAM_MAKE_NWELL_SIZE, i);
-		i = atola(DiaGetText(DSCO_NWELLOFFSET));
+		i = atola(DiaGetText(dia, DSCO_NWELLOFFSET), 0);
 		if (i != nwelloffset) ScSetParameter(SC_PARAM_MAKE_NWELL_OFFSET, i);
 	}
 	efree((char *)arcnames);
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 }
 
 #endif  /* SCTOOL - at top */
