@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: sc1sim.c
@@ -302,7 +303,7 @@ int Sc_simwrite(int count, char *pars[])
 	char	*strtime, outfile[60];
 	FILE	*fp;
 	SCCELL	*cell;
-	UINTBIG	tsec;
+	time_t	tsec;
 	int		err;
 	SCPORT	*port;
 
