@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: simalsgraph.c
@@ -194,15 +195,15 @@ BOOLEAN simals_charhandlerwave(WINDOWPART *w, INTSML chr, INTBIG special)
 
 		/* handle weak and strong prefixes */
 		case 'w':
-			if (sim_als_wantweak || sim_als_wantstrong) ttybeep(0);
+			if (sim_als_wantweak || sim_als_wantstrong) ttybeep(SOUNDBEEP, FALSE);
 			sim_als_wantweak = TRUE;
 			return(FALSE);
 		case 's':
-			if (sim_als_wantweak || sim_als_wantstrong) ttybeep(0);
+			if (sim_als_wantweak || sim_als_wantstrong) ttybeep(SOUNDBEEP, FALSE);
 			sim_als_wantstrong = TRUE;
 			return(FALSE);
 		case 'f':
-			if (sim_als_wantfull) ttybeep(0);
+			if (sim_als_wantfull) ttybeep(SOUNDBEEP, FALSE);
 			sim_als_wantfull = TRUE;
 			return(FALSE);
 
@@ -454,11 +455,11 @@ BOOLEAN simals_charhandlerschem(WINDOWPART *w, INTSML chr, INTBIG special)
 			if ((sim_window_state&ADVANCETIME) != 0) sim_window_setmaincursor(endtime);
 			return(FALSE);
 		case 'w':
-			if (sim_als_wantweak || sim_als_wantstrong) ttybeep(0);
+			if (sim_als_wantweak || sim_als_wantstrong) ttybeep(SOUNDBEEP, FALSE);
 			sim_als_wantweak = TRUE;
 			return(FALSE);
 		case 's':
-			if (sim_als_wantweak || sim_als_wantstrong) ttybeep(0);
+			if (sim_als_wantweak || sim_als_wantstrong) ttybeep(SOUNDBEEP, FALSE);
 			sim_als_wantstrong = TRUE;
 			return(FALSE);
 		case '0':
