@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: simfasthenry.c
@@ -83,7 +84,8 @@ void sim_writefasthenrynetlist(NODEPROTO *np)
 	char name[100], *truename, *args[30], txtpoles[20], *path;
 	FILE *io;
 	float startfreq, endfreq;
-	INTBIG options, runsperdecade, numpoles, seglenlimit, thickness, widsubdiv, heisubdiv;
+	INTBIG options, runsperdecade, numpoles, seglenlimit, thickness, widsubdiv, heisubdiv,
+		lambda;
 	REGISTER INTBIG argc, i;
 	REGISTER void *infstr;
 
@@ -91,7 +93,7 @@ void sim_writefasthenrynetlist(NODEPROTO *np)
 	if ((net_tool->toolstate&TOOLON) == 0)
 	{
 		ttyputerr(_("Network tool must be running...turning it on"));
-		toolturnon(net_tool, FALSE);
+		toolturnon(net_tool);
 		ttyputerr(_("...now reissue the simulation command"));
 		return;
 	}
@@ -114,9 +116,9 @@ void sim_writefasthenrynetlist(NODEPROTO *np)
 	if ((us_useroptions&NODATEORVERSION) == 0)
 	{
 		if (np->creationdate != 0)
-			xprintf(io, "* Facet created on %s\n", timetostring(np->creationdate));
+			xprintf(io, "* Facet created on %s\n", timetostring((time_t)np->creationdate));
 		if (np->revisiondate != 0)
-			xprintf(io, "* Facet last modified on %s\n", timetostring(np->revisiondate));
+			xprintf(io, "* Facet last modified on %s\n", timetostring((time_t)np->revisiondate));
 		xprintf(io, "* Netlist written on %s\n", timetostring(getcurrenttime()));
 		xprintf(io, "* Written by Electric VLSI Design System, version %s\n", el_version);
 	} else
@@ -128,8 +130,9 @@ void sim_writefasthenrynetlist(NODEPROTO *np)
 	xprintf(io, ".units um\n");
 
 	/* write default width and height subdivisions */
+	lambda = lambdaoffacet(np);
 	xprintf(io, "\n* Default number of subdivisions\n");
-	xprintf(io, ".Default nwinc=%ld nhinc=%ld h=%s\n", widsubdiv, heisubdiv, latoa(thickness));
+	xprintf(io, ".Default nwinc=%ld nhinc=%ld h=%s\n", widsubdiv, heisubdiv, latoa(thickness, lambda));
 
 	/* reset flags for facets that have been written */
 	if (sim_writefhfacet(np, io))
@@ -492,7 +495,7 @@ static DIALOGITEM sim_fasthenrydialogitems[] =
  /* 23 */ {0, {176,140,192,344}, POPUP, ""},
  /* 24 */ {0, {156,176,172,320}, MESSAGE, N_("After writing deck:")}
 };
-static DIALOG sim_fasthenrydialog = {{75,75,276,564}, N_("FastHenry Options"), 0, 24, sim_fasthenrydialogitems};
+static DIALOG sim_fasthenrydialog = {{75,75,276,564}, N_("FastHenry Options"), 0, 24, sim_fasthenrydialogitems, 0, 0};
 
 /* special items for the "FastHenry options" dialog: */
 #define DFHO_FREQSTART      4		/* frequency start (edit text) */
@@ -523,113 +526,116 @@ void sim_fasthenrydlog(void)
 	float startfreq, endfreq;
 	INTBIG options, runsperdecade, numpoles, seglenlimit, thickness,
 		widsubdiv, heisubdiv;
+	REGISTER void *dia;
 
 	/* get all parameters */
 	sim_fasthenrygetoptions(&options, &startfreq, &endfreq, &runsperdecade, &numpoles,
 		&seglenlimit, &thickness, &widsubdiv, &heisubdiv);
 
 	/* Display the FastHenry options dialog box */
-	if (DiaInitDialog(&sim_fasthenrydialog)) return;
+	dia = DiaInitDialog(&sim_fasthenrydialog);
+	if (dia == 0) return;
 
 	/* set popup */
 	for(i=0; i<3; i++) newlang[i] = _(exechoices[i]);
-	DiaSetPopup(DFHO_AFTERWRITE, 3, newlang);
+	DiaSetPopup(dia, DFHO_AFTERWRITE, 3, newlang);
 	canexecute = graphicshas(CANRUNPROCESS);
 	if (canexecute)
 	{
+		DiaUnDimItem(dia, DFHO_AFTERWRITE);
 		if ((options&FHEXECUTETYPE) == FHEXECUTERUNFH)
 		{
-			DiaSetPopupEntry(DFHO_AFTERWRITE, 1);
+			DiaSetPopupEntry(dia, DFHO_AFTERWRITE, 1);
 		} else if ((options&FHEXECUTETYPE) == FHEXECUTERUNFHMUL)
 		{
-			DiaSetPopupEntry(DFHO_AFTERWRITE, 2);
+			DiaSetPopupEntry(dia, DFHO_AFTERWRITE, 2);
 		}
 	} else
 	{
-		DiaDimItem(DFHO_AFTERWRITE);
+		DiaDimItem(dia, DFHO_AFTERWRITE);
 	}
 
 	/* set checkboxes */
-	if ((options&FHUSESINGLEFREQ) != 0) DiaSetControl(DFHO_USESINGLEFREQ, 1);
-	if ((options&FHMAKEMULTIPOLECKT) != 0) DiaSetControl(DFHO_MULTIPOLE, 1);
-	if ((options&FHMAKEPOSTSCRIPTVIEW) != 0) DiaSetControl(DFHO_POSTSCRIPT, 1);
-	if ((options&FHMAKESPICESUBCKT) != 0) DiaSetControl(DFHO_MAKESPICE, 1);
+	if ((options&FHUSESINGLEFREQ) != 0) DiaSetControl(dia, DFHO_USESINGLEFREQ, 1);
+	if ((options&FHMAKEMULTIPOLECKT) != 0) DiaSetControl(dia, DFHO_MULTIPOLE, 1);
+	if ((options&FHMAKEPOSTSCRIPTVIEW) != 0) DiaSetControl(dia, DFHO_POSTSCRIPT, 1);
+	if ((options&FHMAKESPICESUBCKT) != 0) DiaSetControl(dia, DFHO_MAKESPICE, 1);
 
 	/* load default frequency range */
-	sprintf(line, "%g", startfreq);   DiaSetText(DFHO_FREQSTART, line);
-	sprintf(line, "%g", endfreq);   DiaSetText(DFHO_FREQEND, line);
-	sprintf(line, "%ld", runsperdecade);   DiaSetText(DFHO_RUNSPERDEC, line);
+	sprintf(line, "%g", startfreq);   DiaSetText(dia, DFHO_FREQSTART, line);
+	sprintf(line, "%g", endfreq);   DiaSetText(dia, DFHO_FREQEND, line);
+	sprintf(line, "%ld", runsperdecade);   DiaSetText(dia, DFHO_RUNSPERDEC, line);
 
 	/* load segment limits */
-	DiaSetText(DFHO_MAXSEGLEN, latoa(seglenlimit));
-	DiaSetText(DFHO_DEFTHICK, latoa(thickness));
-	sprintf(line, "%ld", widsubdiv);   DiaSetText(DFHO_DEFWIDSUB, line);
-	sprintf(line, "%ld", heisubdiv);   DiaSetText(DFHO_DEFHEISUB, line);
+	DiaSetText(dia, DFHO_MAXSEGLEN, latoa(seglenlimit, 0));
+	DiaSetText(dia, DFHO_DEFTHICK, latoa(thickness, 0));
+	sprintf(line, "%ld", widsubdiv);   DiaSetText(dia, DFHO_DEFWIDSUB, line);
+	sprintf(line, "%ld", heisubdiv);   DiaSetText(dia, DFHO_DEFHEISUB, line);
 
 	/* load other numeric options */
-	sprintf(line, "%ld", numpoles);   DiaSetText(DFHO_NUMPOLES, line);
+	sprintf(line, "%ld", numpoles);   DiaSetText(dia, DFHO_NUMPOLES, line);
 
 	if ((options&FHMAKEMULTIPOLECKT) != 0)
 	{
-		DiaUnDimItem(DFHO_NUMPOLES_L);
-		DiaEditControl(DFHO_NUMPOLES);
+		DiaUnDimItem(dia, DFHO_NUMPOLES_L);
+		DiaEditControl(dia, DFHO_NUMPOLES);
 	} else
 	{
-		DiaDimItem(DFHO_NUMPOLES_L);
-		DiaNoEditControl(DFHO_NUMPOLES);
+		DiaDimItem(dia, DFHO_NUMPOLES_L);
+		DiaNoEditControl(dia, DFHO_NUMPOLES);
 	}
 	if ((options&FHUSESINGLEFREQ) != 0)
 	{
-		DiaDimItem(DFHO_FREQEND_L);
-		DiaNoEditControl(DFHO_FREQEND);
-		DiaDimItem(DFHO_RUNSPERDEC_L);
-		DiaNoEditControl(DFHO_RUNSPERDEC);
+		DiaDimItem(dia, DFHO_FREQEND_L);
+		DiaNoEditControl(dia, DFHO_FREQEND);
+		DiaDimItem(dia, DFHO_RUNSPERDEC_L);
+		DiaNoEditControl(dia, DFHO_RUNSPERDEC);
 	} else
 	{
-		DiaUnDimItem(DFHO_FREQEND_L);
-		DiaEditControl(DFHO_FREQEND);
-		DiaUnDimItem(DFHO_RUNSPERDEC_L);
-		DiaEditControl(DFHO_RUNSPERDEC);
+		DiaUnDimItem(dia, DFHO_FREQEND_L);
+		DiaEditControl(dia, DFHO_FREQEND);
+		DiaUnDimItem(dia, DFHO_RUNSPERDEC_L);
+		DiaEditControl(dia, DFHO_RUNSPERDEC);
 	}
-	DiaDimItem(DFHO_POSTSCRIPT);
-	DiaNoEditControl(DFHO_MAXSEGLEN);
-	DiaDimItem(DFHO_MAKESPICE);
+	DiaDimItem(dia, DFHO_POSTSCRIPT);
+	DiaNoEditControl(dia, DFHO_MAXSEGLEN);
+	DiaDimItem(dia, DFHO_MAKESPICE);
 
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == OK || itemHit == CANCEL) break;
 		if (itemHit == DFHO_USESINGLEFREQ || itemHit == DFHO_MULTIPOLE ||
 			itemHit == DFHO_POSTSCRIPT || itemHit == DFHO_MAKESPICE)
 		{
-			value = 1 - DiaGetControl(itemHit);
-			DiaSetControl(itemHit, value);
+			value = 1 - DiaGetControl(dia, itemHit);
+			DiaSetControl(dia, itemHit, value);
 			if (itemHit == DFHO_USESINGLEFREQ)
 			{
 				if (value != 0)
 				{
-					DiaDimItem(DFHO_FREQEND_L);
-					DiaNoEditControl(DFHO_FREQEND);
-					DiaDimItem(DFHO_RUNSPERDEC_L);
-					DiaNoEditControl(DFHO_RUNSPERDEC);
+					DiaDimItem(dia, DFHO_FREQEND_L);
+					DiaNoEditControl(dia, DFHO_FREQEND);
+					DiaDimItem(dia, DFHO_RUNSPERDEC_L);
+					DiaNoEditControl(dia, DFHO_RUNSPERDEC);
 				} else
 				{
-					DiaUnDimItem(DFHO_FREQEND_L);
-					DiaEditControl(DFHO_FREQEND);
-					DiaUnDimItem(DFHO_RUNSPERDEC_L);
-					DiaEditControl(DFHO_RUNSPERDEC);
+					DiaUnDimItem(dia, DFHO_FREQEND_L);
+					DiaEditControl(dia, DFHO_FREQEND);
+					DiaUnDimItem(dia, DFHO_RUNSPERDEC_L);
+					DiaEditControl(dia, DFHO_RUNSPERDEC);
 				}
 			}
 			if (itemHit == DFHO_MULTIPOLE)
 			{
 				if (value != 0)
 				{
-					DiaUnDimItem(DFHO_NUMPOLES_L);
-					DiaEditControl(DFHO_NUMPOLES);
+					DiaUnDimItem(dia, DFHO_NUMPOLES_L);
+					DiaEditControl(dia, DFHO_NUMPOLES);
 				} else
 				{
-					DiaDimItem(DFHO_NUMPOLES_L);
-					DiaNoEditControl(DFHO_NUMPOLES);
+					DiaDimItem(dia, DFHO_NUMPOLES_L);
+					DiaNoEditControl(dia, DFHO_NUMPOLES);
 				}
 			}
 			continue;
@@ -640,43 +646,43 @@ void sim_fasthenrydlog(void)
 	{
 		/* save options */
 		value = 0;
-		if (DiaGetControl(DFHO_USESINGLEFREQ) != 0) value |= FHUSESINGLEFREQ;
-		if (DiaGetControl(DFHO_MULTIPOLE) != 0) value |= FHMAKEMULTIPOLECKT;
-		if (DiaGetControl(DFHO_POSTSCRIPT) != 0) value |= FHMAKEPOSTSCRIPTVIEW;
-		if (DiaGetControl(DFHO_MAKESPICE) != 0) value |= FHMAKESPICESUBCKT;
-		i = DiaGetPopupEntry(DFHO_AFTERWRITE);
+		if (DiaGetControl(dia, DFHO_USESINGLEFREQ) != 0) value |= FHUSESINGLEFREQ;
+		if (DiaGetControl(dia, DFHO_MULTIPOLE) != 0) value |= FHMAKEMULTIPOLECKT;
+		if (DiaGetControl(dia, DFHO_POSTSCRIPT) != 0) value |= FHMAKEPOSTSCRIPTVIEW;
+		if (DiaGetControl(dia, DFHO_MAKESPICE) != 0) value |= FHMAKESPICESUBCKT;
+		i = DiaGetPopupEntry(dia, DFHO_AFTERWRITE);
 		if (i == 1) value |= FHEXECUTERUNFH; else
 			if (i == 2) value |= FHEXECUTERUNFHMUL;
 		if (value != options)
 			(void)setvalkey((INTBIG)sim_tool, VTOOL, sim_fasthenrystatekey, value, VINTEGER);
 
 		/* save other limits */
-		fvalue = (float)atof(DiaGetText(DFHO_FREQSTART));
+		fvalue = (float)atof(DiaGetText(dia, DFHO_FREQSTART));
 		if (fvalue != startfreq)
 			(void)setvalkey((INTBIG)sim_tool, VTOOL, sim_fasthenryfreqstartkey, castint(fvalue), VFLOAT);
-		fvalue = (float)atof(DiaGetText(DFHO_FREQEND));
+		fvalue = (float)atof(DiaGetText(dia, DFHO_FREQEND));
 		if (fvalue != endfreq)
 			(void)setvalkey((INTBIG)sim_tool, VTOOL, sim_fasthenryfreqendkey, castint(fvalue), VFLOAT);
-		value = atoi(DiaGetText(DFHO_RUNSPERDEC));
+		value = atoi(DiaGetText(dia, DFHO_RUNSPERDEC));
 		if (value != runsperdecade)
 			(void)setvalkey((INTBIG)sim_tool, VTOOL, sim_fasthenryrunsperdecadekey, value, VINTEGER);
-		value = atola(DiaGetText(DFHO_MAXSEGLEN));
+		value = atola(DiaGetText(dia, DFHO_MAXSEGLEN), 0);
 		if (value != seglenlimit)
 			(void)setvalkey((INTBIG)sim_tool, VTOOL, sim_fasthenryseglimitkey, value, VINTEGER);
-		value = atola(DiaGetText(DFHO_DEFTHICK));
+		value = atola(DiaGetText(dia, DFHO_DEFTHICK), 0);
 		if (value != thickness)
 			(void)setvalkey((INTBIG)sim_tool, VTOOL, sim_fasthenrythicknesskey, value, VINTEGER);
-		value = atoi(DiaGetText(DFHO_DEFWIDSUB));
+		value = atoi(DiaGetText(dia, DFHO_DEFWIDSUB));
 		if (value != widsubdiv)
 			(void)setvalkey((INTBIG)sim_tool, VTOOL, sim_fasthenrywidthsubdivkey, value, VINTEGER);
-		value = atoi(DiaGetText(DFHO_DEFHEISUB));
+		value = atoi(DiaGetText(dia, DFHO_DEFHEISUB));
 		if (value != heisubdiv)
 			(void)setvalkey((INTBIG)sim_tool, VTOOL, sim_fasthenryheightsubdivkey, value, VINTEGER);
-		value = atoi(DiaGetText(DFHO_NUMPOLES));
+		value = atoi(DiaGetText(dia, DFHO_NUMPOLES));
 		if (value != numpoles)
 			(void)setvalkey((INTBIG)sim_tool, VTOOL, sim_fasthenrynumpoleskey, value, VINTEGER);
 	}
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 }
 
 /* Simulation: FastHenry Arc */
@@ -715,7 +721,7 @@ static DIALOGITEM sim_fasthenryarcdialogitems[] =
  /* 31 */ {0, {304,88,320,168}, MESSAGE, N_("Default Z:")},
  /* 32 */ {0, {304,172,320,268}, MESSAGE, ""}
 };
-static DIALOG sim_fasthenryarcdialog = {{75,75,404,401}, N_("FastHenry Arc Properties"), 0, 32, sim_fasthenryarcdialogitems};
+static DIALOG sim_fasthenryarcdialog = {{75,75,404,401}, N_("FastHenry Arc Properties"), 0, 32, sim_fasthenryarcdialogitems, 0, 0};
 
 /* special items for the "FastHenry arc" dialog: */
 #define DFHA_ARCTHICK_L      3		/* arc thickness label (stat text) */
@@ -741,7 +747,7 @@ static DIALOG sim_fasthenryarcdialog = {{75,75,404,401}, N_("FastHenry Arc Prope
 
 void sim_fasthenryarcdlog(void)
 {
-	REGISTER INTBIG itemHit, value, groupnamesize, i, val;
+	REGISTER INTBIG itemHit, value, groupnamesize, i, val, lambda;
 	REGISTER ARCINST *ai, *oai;
 	REGISTER VARIABLE *var;
 	char line[60], *groupname, *pt, **groupnames, **newgroupnames, *newname;
@@ -749,6 +755,7 @@ void sim_fasthenryarcdlog(void)
 	INTBIG options, runsperdecade, numpoles, seglenlimit, thickness, defthickness,
 		widsubdiv, defwidsubdiv, heisubdiv, defheisubdiv, z_head, z_tail, defz;
 	BOOLEAN zhover, ztover;
+	REGISTER void *dia;
 
 	/* get currently selected arc */
 	ai = (ARCINST *)asktool(us_tool, "get-arc");
@@ -757,6 +764,7 @@ void sim_fasthenryarcdlog(void)
 		ttyputerr(_("Select an arc first"));
 		return;
 	}
+	lambda = lambdaofarc(ai);
 
 	/* get all parameters */
 	sim_fasthenrygetoptions(&options, &startfreq, &endfreq, &runsperdecade, &numpoles,
@@ -765,7 +773,8 @@ void sim_fasthenryarcdlog(void)
 		&z_head, &z_tail, &zhover, &ztover, &defz);
 
 	/* display the FastHenry arc dialog box */
-	if (DiaInitDialog(&sim_fasthenryarcdialog)) return;
+	dia = DiaInitDialog(&sim_fasthenryarcdialog);
+	if (dia == 0) return;
 
 	/* make a list of all group names */
 	groupnamesize = 0;
@@ -799,120 +808,120 @@ void sim_fasthenryarcdlog(void)
 		(void)allocstring(&groupnames[0], _("Group 1"), sim_tool->cluster);
 		groupnamesize++;
 	}
-	DiaSetPopup(DFHA_GROUPNAME, groupnamesize, groupnames);
+	DiaSetPopup(dia, DFHA_GROUPNAME, groupnamesize, groupnames);
 
 	/* select group name */
 	if (groupname != 0)
 	{
 		for(i=0; i<groupnamesize; i++)
 			if (namesame(groupname, groupnames[i]) == 0) break;
-		if (i < groupnamesize) DiaSetPopupEntry(DFHA_GROUPNAME, i);
+		if (i < groupnamesize) DiaSetPopupEntry(dia, DFHA_GROUPNAME, i);
 	}
 
 	/* show arc parameters */
-	sprintf(line, _("Thickness (%s):"), latoa(defthickness));
-	DiaSetText(DFHA_ARCTHICK_L, line);
+	sprintf(line, _("Thickness (%s):"), latoa(defthickness, lambda));
+	DiaSetText(dia, DFHA_ARCTHICK_L, line);
 	if (thickness >= 0)
-		DiaSetText(DFHA_ARCTHICK, latoa(thickness));
-	DiaSetText(DFHA_ARCWIDTH, latoa(ai->width));
+		DiaSetText(dia, DFHA_ARCTHICK, latoa(thickness, lambda));
+	DiaSetText(dia, DFHA_ARCWIDTH, latoa(ai->width, lambda));
 
 	/* show subdivision overrides */
 	sprintf(line, _("Width subdivisions (%ld):"), defwidsubdiv);
-	DiaSetText(DFHA_WIDTHSUBDIV_L, line);
+	DiaSetText(dia, DFHA_WIDTHSUBDIV_L, line);
 	if (widsubdiv > 0)
 	{
 		sprintf(line, "%ld", widsubdiv);
-		DiaSetText(DFHA_WIDTHSUBDIV, line);
+		DiaSetText(dia, DFHA_WIDTHSUBDIV, line);
 	}
 	sprintf(line, _("Height subdivisions (%ld):"), defheisubdiv);
-	DiaSetText(DFHA_HEIGHTSUBDIV_L, line);
+	DiaSetText(dia, DFHA_HEIGHTSUBDIV_L, line);
 	if (heisubdiv > 0)
 	{
 		sprintf(line, "%ld", heisubdiv);
-		DiaSetText(DFHA_HEIGHTSUBDIV, line);
+		DiaSetText(dia, DFHA_HEIGHTSUBDIV, line);
 	}
 
 	/* show end coordinates */
-	DiaSetText(DFHA_DEFZ, latoa(defz));
-	DiaSetText(DFHA_ARCHEADX, latoa(ai->end[0].xpos));
-	DiaSetText(DFHA_ARCHEADY, latoa(ai->end[0].ypos));
-	if (zhover) DiaSetText(DFHA_ARCHEADZ, latoa(z_head));
-	DiaSetText(DFHA_ARCTAILX, latoa(ai->end[1].xpos));
-	DiaSetText(DFHA_ARCTAILY, latoa(ai->end[1].ypos));
-	if (ztover) DiaSetText(DFHA_ARCTAILZ, latoa(z_tail));
+	DiaSetText(dia, DFHA_DEFZ, latoa(defz, lambda));
+	DiaSetText(dia, DFHA_ARCHEADX, latoa(ai->end[0].xpos, lambda));
+	DiaSetText(dia, DFHA_ARCHEADY, latoa(ai->end[0].ypos, lambda));
+	if (zhover) DiaSetText(dia, DFHA_ARCHEADZ, latoa(z_head, lambda));
+	DiaSetText(dia, DFHA_ARCTAILX, latoa(ai->end[1].xpos, lambda));
+	DiaSetText(dia, DFHA_ARCTAILY, latoa(ai->end[1].ypos, lambda));
+	if (ztover) DiaSetText(dia, DFHA_ARCTAILZ, latoa(z_tail, lambda));
 
 	if (groupname != 0)
 	{
-		DiaUnDimItem(DFHA_ARCTHICK_L);
-		DiaEditControl(DFHA_ARCTHICK);
-		DiaUnDimItem(DFHA_WIDTHSUBDIV_L);
-		DiaEditControl(DFHA_WIDTHSUBDIV);
-		DiaUnDimItem(DFHA_HEIGHTSUBDIV_L);
-		DiaEditControl(DFHA_HEIGHTSUBDIV);
-		DiaUnDimItem(DFHA_ARCHEADZ_L);
-		DiaEditControl(DFHA_ARCHEADZ);
-		DiaUnDimItem(DFHA_ARCTAILZ_L);
-		DiaEditControl(DFHA_ARCTAILZ);
-		DiaUnDimItem(DFHA_GROUPNAME_L);
-		DiaUnDimItem(DFHA_GROUPNAME);
-		DiaUnDimItem(DFHA_NEWGROUP);
-		DiaSetControl(DFHA_INCLUDE, 1);
+		DiaUnDimItem(dia, DFHA_ARCTHICK_L);
+		DiaEditControl(dia, DFHA_ARCTHICK);
+		DiaUnDimItem(dia, DFHA_WIDTHSUBDIV_L);
+		DiaEditControl(dia, DFHA_WIDTHSUBDIV);
+		DiaUnDimItem(dia, DFHA_HEIGHTSUBDIV_L);
+		DiaEditControl(dia, DFHA_HEIGHTSUBDIV);
+		DiaUnDimItem(dia, DFHA_ARCHEADZ_L);
+		DiaEditControl(dia, DFHA_ARCHEADZ);
+		DiaUnDimItem(dia, DFHA_ARCTAILZ_L);
+		DiaEditControl(dia, DFHA_ARCTAILZ);
+		DiaUnDimItem(dia, DFHA_GROUPNAME_L);
+		DiaUnDimItem(dia, DFHA_GROUPNAME);
+		DiaUnDimItem(dia, DFHA_NEWGROUP);
+		DiaSetControl(dia, DFHA_INCLUDE, 1);
 	} else
 	{
-		DiaDimItem(DFHA_ARCTHICK_L);
-		DiaNoEditControl(DFHA_ARCTHICK);
-		DiaDimItem(DFHA_WIDTHSUBDIV_L);
-		DiaNoEditControl(DFHA_WIDTHSUBDIV);
-		DiaDimItem(DFHA_HEIGHTSUBDIV_L);
-		DiaNoEditControl(DFHA_HEIGHTSUBDIV);
-		DiaDimItem(DFHA_ARCHEADZ_L);
-		DiaNoEditControl(DFHA_ARCHEADZ);
-		DiaDimItem(DFHA_ARCTAILZ_L);
-		DiaNoEditControl(DFHA_ARCTAILZ);
-		DiaDimItem(DFHA_GROUPNAME_L);
-		DiaDimItem(DFHA_GROUPNAME);
-		DiaDimItem(DFHA_NEWGROUP);
+		DiaDimItem(dia, DFHA_ARCTHICK_L);
+		DiaNoEditControl(dia, DFHA_ARCTHICK);
+		DiaDimItem(dia, DFHA_WIDTHSUBDIV_L);
+		DiaNoEditControl(dia, DFHA_WIDTHSUBDIV);
+		DiaDimItem(dia, DFHA_HEIGHTSUBDIV_L);
+		DiaNoEditControl(dia, DFHA_HEIGHTSUBDIV);
+		DiaDimItem(dia, DFHA_ARCHEADZ_L);
+		DiaNoEditControl(dia, DFHA_ARCHEADZ);
+		DiaDimItem(dia, DFHA_ARCTAILZ_L);
+		DiaNoEditControl(dia, DFHA_ARCTAILZ);
+		DiaDimItem(dia, DFHA_GROUPNAME_L);
+		DiaDimItem(dia, DFHA_GROUPNAME);
+		DiaDimItem(dia, DFHA_NEWGROUP);
 	}
 
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == OK || itemHit == CANCEL) break;
 		if (itemHit == DFHA_INCLUDE)
 		{
-			value = 1 - DiaGetControl(itemHit);
-			DiaSetControl(itemHit, value);
+			value = 1 - DiaGetControl(dia, itemHit);
+			DiaSetControl(dia, itemHit, value);
 			if (value != 0)
 			{
-				DiaUnDimItem(DFHA_ARCTHICK_L);
-				DiaEditControl(DFHA_ARCTHICK);
-				DiaUnDimItem(DFHA_WIDTHSUBDIV_L);
-				DiaEditControl(DFHA_WIDTHSUBDIV);
-				DiaUnDimItem(DFHA_HEIGHTSUBDIV_L);
-				DiaEditControl(DFHA_HEIGHTSUBDIV);
-				DiaUnDimItem(DFHA_ARCHEADZ_L);
-				DiaEditControl(DFHA_ARCHEADZ);
-				DiaUnDimItem(DFHA_ARCTAILZ_L);
-				DiaEditControl(DFHA_ARCTAILZ);
-				DiaUnDimItem(DFHA_GROUPNAME_L);
-				DiaUnDimItem(DFHA_GROUPNAME);
-				DiaUnDimItem(DFHA_NEWGROUP);
-				DiaSetControl(DFHA_INCLUDE, 1);
+				DiaUnDimItem(dia, DFHA_ARCTHICK_L);
+				DiaEditControl(dia, DFHA_ARCTHICK);
+				DiaUnDimItem(dia, DFHA_WIDTHSUBDIV_L);
+				DiaEditControl(dia, DFHA_WIDTHSUBDIV);
+				DiaUnDimItem(dia, DFHA_HEIGHTSUBDIV_L);
+				DiaEditControl(dia, DFHA_HEIGHTSUBDIV);
+				DiaUnDimItem(dia, DFHA_ARCHEADZ_L);
+				DiaEditControl(dia, DFHA_ARCHEADZ);
+				DiaUnDimItem(dia, DFHA_ARCTAILZ_L);
+				DiaEditControl(dia, DFHA_ARCTAILZ);
+				DiaUnDimItem(dia, DFHA_GROUPNAME_L);
+				DiaUnDimItem(dia, DFHA_GROUPNAME);
+				DiaUnDimItem(dia, DFHA_NEWGROUP);
+				DiaSetControl(dia, DFHA_INCLUDE, 1);
 			} else
 			{
-				DiaDimItem(DFHA_ARCTHICK_L);
-				DiaNoEditControl(DFHA_ARCTHICK);
-				DiaDimItem(DFHA_WIDTHSUBDIV_L);
-				DiaNoEditControl(DFHA_WIDTHSUBDIV);
-				DiaDimItem(DFHA_HEIGHTSUBDIV_L);
-				DiaNoEditControl(DFHA_HEIGHTSUBDIV);
-				DiaDimItem(DFHA_ARCHEADZ_L);
-				DiaNoEditControl(DFHA_ARCHEADZ);
-				DiaDimItem(DFHA_ARCTAILZ_L);
-				DiaNoEditControl(DFHA_ARCTAILZ);
-				DiaDimItem(DFHA_GROUPNAME_L);
-				DiaDimItem(DFHA_GROUPNAME);
-				DiaDimItem(DFHA_NEWGROUP);
+				DiaDimItem(dia, DFHA_ARCTHICK_L);
+				DiaNoEditControl(dia, DFHA_ARCTHICK);
+				DiaDimItem(dia, DFHA_WIDTHSUBDIV_L);
+				DiaNoEditControl(dia, DFHA_WIDTHSUBDIV);
+				DiaDimItem(dia, DFHA_HEIGHTSUBDIV_L);
+				DiaNoEditControl(dia, DFHA_HEIGHTSUBDIV);
+				DiaDimItem(dia, DFHA_ARCHEADZ_L);
+				DiaNoEditControl(dia, DFHA_ARCHEADZ);
+				DiaDimItem(dia, DFHA_ARCTAILZ_L);
+				DiaNoEditControl(dia, DFHA_ARCTAILZ);
+				DiaDimItem(dia, DFHA_GROUPNAME_L);
+				DiaDimItem(dia, DFHA_GROUPNAME);
+				DiaDimItem(dia, DFHA_NEWGROUP);
 			}
 			continue;
 		}
@@ -932,8 +941,8 @@ void sim_fasthenryarcdlog(void)
 			if (groupnamesize > 0) efree((char *)groupnames);
 			groupnames = newgroupnames;
 			groupnamesize++;
-			DiaSetPopup(DFHA_GROUPNAME, groupnamesize, groupnames);
-			DiaSetPopupEntry(DFHA_GROUPNAME, groupnamesize-1);
+			DiaSetPopup(dia, DFHA_GROUPNAME, groupnamesize, groupnames);
+			DiaSetPopupEntry(dia, DFHA_GROUPNAME, groupnamesize-1);
 			continue;
 		}
 	}
@@ -942,10 +951,10 @@ void sim_fasthenryarcdlog(void)
 	{
 		startobjectchange((INTBIG)ai, VARCINST);
 		var = getvalkey((INTBIG)ai, VARCINST, VSTRING, sim_fasthenrygroupnamekey);
-		if (DiaGetControl(DFHA_INCLUDE) != 0)
+		if (DiaGetControl(dia, DFHA_INCLUDE) != 0)
 		{
 			/* active in FastHenry */
-			i = DiaGetPopupEntry(DFHA_GROUPNAME);
+			i = DiaGetPopupEntry(dia, DFHA_GROUPNAME);
 			groupname = groupnames[i];
 			if (var == NOVARIABLE || strcmp(groupname, (char *)var->addr) != 0)
 				setvalkey((INTBIG)ai, VARCINST, sim_fasthenrygroupnamekey,
@@ -957,11 +966,11 @@ void sim_fasthenryarcdlog(void)
 		}
 
 		/* save thickness */
-		pt = DiaGetText(DFHA_ARCTHICK);
+		pt = DiaGetText(dia, DFHA_ARCTHICK);
 		var = getvalkey((INTBIG)ai, VARCINST, VINTEGER, sim_fasthenrythicknesskey);
 		if (*pt != 0)
 		{
-			val = atola(pt);
+			val = atola(pt, 0);
 			if (var == NOVARIABLE || val != var->addr)
 				setvalkey((INTBIG)ai, VARCINST, sim_fasthenrythicknesskey, val, VINTEGER);
 		} else
@@ -971,7 +980,7 @@ void sim_fasthenryarcdlog(void)
 		}
 
 		/* save width and height subdivisions */
-		pt = DiaGetText(DFHA_WIDTHSUBDIV);
+		pt = DiaGetText(dia, DFHA_WIDTHSUBDIV);
 		var = getvalkey((INTBIG)ai, VARCINST, VINTEGER, sim_fasthenrywidthsubdivkey);
 		if (*pt != 0)
 		{
@@ -983,7 +992,7 @@ void sim_fasthenryarcdlog(void)
 			if (var != NOVARIABLE)
 				(void)delvalkey((INTBIG)ai, VARCINST, sim_fasthenrywidthsubdivkey);
 		}
-		pt = DiaGetText(DFHA_HEIGHTSUBDIV);
+		pt = DiaGetText(dia, DFHA_HEIGHTSUBDIV);
 		var = getvalkey((INTBIG)ai, VARCINST, VINTEGER, sim_fasthenryheightsubdivkey);
 		if (*pt != 0)
 		{
@@ -997,11 +1006,11 @@ void sim_fasthenryarcdlog(void)
 		}
 
 		/* save z overrides */
-		pt = DiaGetText(DFHA_ARCHEADZ);
+		pt = DiaGetText(dia, DFHA_ARCHEADZ);
 		var = getvalkey((INTBIG)ai, VARCINST, VINTEGER, sim_fasthenryzheadkey);
 		if (*pt != 0)
 		{
-			val = atola(pt);
+			val = atola(pt, 0);
 			if (var == NOVARIABLE || val != var->addr)
 				setvalkey((INTBIG)ai, VARCINST, sim_fasthenryzheadkey, val, VINTEGER);
 		} else
@@ -1009,11 +1018,11 @@ void sim_fasthenryarcdlog(void)
 			if (var != NOVARIABLE)
 				(void)delvalkey((INTBIG)ai, VARCINST, sim_fasthenryzheadkey);
 		}
-		pt = DiaGetText(DFHA_ARCTAILZ);
+		pt = DiaGetText(dia, DFHA_ARCTAILZ);
 		var = getvalkey((INTBIG)ai, VARCINST, VINTEGER, sim_fasthenryztailkey);
 		if (*pt != 0)
 		{
-			val = atola(pt);
+			val = atola(pt, 0);
 			if (var == NOVARIABLE || val != var->addr)
 				setvalkey((INTBIG)ai, VARCINST, sim_fasthenryztailkey, val, VINTEGER);
 		} else
@@ -1023,7 +1032,7 @@ void sim_fasthenryarcdlog(void)
 		}
 		endobjectchange((INTBIG)ai, VARCINST);
 	}
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 
 	/* free group list memory */
 	for(i=0; i<groupnamesize; i++)
