@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: simirsim.c
@@ -47,6 +48,12 @@ static IRSIMTRANSISTOR *sim_irsimfacetlisthead;
 	   IRSIMNETWORK    *sim_irsimnets;
        INTBIG           sim_irsimnetnumber = 0;
 static INTBIG           sim_irsimnettotal = 0;
+#if SIMFSDBWRITE != 0
+static INTBIG           sim_irsimtopnetnumber;
+	   IRSIMALIAS      *sim_irsimaliases;
+       INTBIG           sim_irsimaliasnumber = 0;
+static INTBIG           sim_irsimaliastotal = 0;
+#endif
        INTBIG           sim_irsim_statekey;		/* variable key for "SIM_irsim_state" */
 static INTBIG           sim_irsim_state;		/* cached value of "SIM_irsim_state" variable */
 static INTBIG           sim_irsimlambda;
@@ -74,6 +81,7 @@ static void             sim_irsimgetareaperimeter(INTBIG *x, INTBIG *y, INTBIG c
 							float *a, INTBIG *p);
 static INTBIG           sim_irsimgetglobal(NETWORK *net);
 static INTBIG           sim_irsimnewnetnumber(char *name);
+static void             sim_irsimnewalias(INTBIG netnum, char *alias);
 static INTBIG           sim_irsimrecurse(NODEPROTO *facet, XARRAY trans);
 
 /*
@@ -119,7 +127,7 @@ void sim_writeirsim(NODEPROTO *facet)
 	if ((net_tool->toolstate&TOOLON) == 0)
 	{
 		ttyputerr(_("Network tool must be running...turning it on"));
-		toolturnon(net_tool, FALSE);
+		toolturnon(net_tool);
 		ttyputerr(_("...now reissue the simulation command"));
 		return;
 	}
@@ -176,6 +184,9 @@ void sim_irsimgeneratedeck(NODEPROTO *facet, FILE *f)
 	/* initialize the list of networks */
 	sim_irsimfreenetworkinfo();
 	sim_irsimnetnumber = 0;
+#if SIMFSDBWRITE != 0
+	sim_irsimaliasnumber = 0;
+#endif
 
 	/* gather global signal names */
 	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
@@ -205,9 +216,9 @@ void sim_irsimgeneratedeck(NODEPROTO *facet, FILE *f)
 	for(pp = facet->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 	{
 		if (pp->network->temp1 != -1) continue;
-		if (pp->network->signals > 1)
+		if (pp->network->buswidth > 1)
 		{
-			for(i=0; i<pp->network->signals; i++)
+			for(i=0; i<pp->network->buswidth; i++)
 			{
 				net = pp->network->networklist[i];
 				if (net->globalnet >= 0) net->temp1 = sim_irsimgetglobal(net); else
@@ -229,6 +240,9 @@ void sim_irsimgeneratedeck(NODEPROTO *facet, FILE *f)
 			}
 		}
 	}
+#if SIMFSDBWRITE != 0
+	sim_irsimtopnetnumber = sim_irsimnetnumber;
+#endif
 	begintraversehierarchy();
 	if ((sim_irsim_state&IRSIMPARASITICS) == IRSIMPARAFULL)
 		sim_irsimmerge = mergenew(sim_tool->cluster);
@@ -258,9 +272,9 @@ void sim_irsimgeneratedeck(NODEPROTO *facet, FILE *f)
 		if ((us_useroptions&NODATEORVERSION) == 0)
 		{
 			if (facet->creationdate)
-				xprintf(f, "| Created on %s\n", timetostring(facet->creationdate));
+				xprintf(f, "| Created on %s\n", timetostring((time_t)facet->creationdate));
 			if (facet->revisiondate)
-				xprintf(f, "| Last revised on %s\n", timetostring(facet->revisiondate));
+				xprintf(f, "| Last revised on %s\n", timetostring((time_t)facet->revisiondate));
 			(void)sprintf(numberstring, "%s", timetostring(getcurrenttime()));
 			xprintf(f, "| Written on %s by Electric VLSI Design System, version %s\n",
 				numberstring, el_version);
@@ -276,8 +290,8 @@ void sim_irsimgeneratedeck(NODEPROTO *facet, FILE *f)
 			xprintf(f, " %s", sim_irsimnets[it->gate].name);
 			xprintf(f, " %s", sim_irsimnets[it->source].name);
 			xprintf(f, " %s", sim_irsimnets[it->drain].name);
-			xprintf(f, " %s %s", latoa(it->length), latoa(it->width));
-			xprintf(f, " %s %s", latoa(it->xpos), latoa(it->ypos));
+			xprintf(f, " %s %s", latoa(it->length, sim_irsimlambda), latoa(it->width, sim_irsimlambda));
+			xprintf(f, " %s %s", latoa(it->xpos, sim_irsimlambda), latoa(it->ypos, sim_irsimlambda));
 			if (it->transistortype == NPTRANMOS) xprintf(f, " g=S_gnd"); else
 				xprintf(f, " g=S_vdd");
 			sa = (INTBIG)(it->sarea / sim_irsimlambda);
@@ -291,8 +305,8 @@ void sim_irsimgeneratedeck(NODEPROTO *facet, FILE *f)
 			da = (da + sim_irsimlambda/2) / sim_irsimlambda * sim_irsimlambda;
 			dp = (it->dperimeter + sim_irsimlambda/2) / sim_irsimlambda * sim_irsimlambda;
 			if (dp == 0) dp = it->width + 12*sim_irsimlambda;
-			xprintf(f, " s=A_%s,P_%s", latoa(sa), latoa(sp));
-			xprintf(f, " d=A_%s,P_%s\n", latoa(da), latoa(dp));
+			xprintf(f, " s=A_%s,P_%s", latoa(sa, sim_irsimlambda), latoa(sp, sim_irsimlambda));
+			xprintf(f, " d=A_%s,P_%s\n", latoa(da, sim_irsimlambda), latoa(dp, sim_irsimlambda));
 		}
 	}
 
@@ -342,8 +356,15 @@ INTBIG sim_irsimrecurse(NODEPROTO *facet, XARRAY trans)
 	/* create unique numbers for internal nets */
 	for(net = facet->firstnetwork; net != NONETWORK; net = net->nextnetwork)
 	{
-		if (net->temp1 != -1) continue;
-		if (net->signals > 1) continue;
+		if (net->buswidth > 1)  continue;
+		if (net->temp1 != -1)
+		{
+#if SIMFSDBWRITE != 0
+			if (net->namecount > 0 && net->globalnet < 0 && net->temp1 >= sim_irsimtopnetnumber)
+				sim_irsimnewalias(net->temp1, sim_irsimbuildnetname(net->netname));
+#endif
+			continue;
+		}
 		if (net->globalnet >= 0) net->temp1 = sim_irsimgetglobal(net); else
 		{
 			if (net->namecount > 0)
@@ -415,29 +436,29 @@ INTBIG sim_irsimrecurse(NODEPROTO *facet, XARRAY trans)
 						if (pe != NOPORTEXPINST) net = pe->exportproto->network;
 					}
 					if (net == NONETWORK) continue;
-					if (net->signals > 1)
+					if (net->buswidth > 1)
 					{
-						sigcount = net->signals;
-						if (nodewidth > 1 && cpp->network->signals * nodewidth == net->signals)
+						sigcount = net->buswidth;
+						if (nodewidth > 1 && cpp->network->buswidth * nodewidth == net->buswidth)
 						{
 							/* map wide bus to a particular instantiation of an arrayed node */
-							if (cpp->network->signals == 1)
+							if (cpp->network->buswidth == 1)
 							{
 								cpp->network->temp1 = net->networklist[nindex]->temp1;
 							} else
 							{
-								for(i=0; i<cpp->network->signals; i++)
+								for(i=0; i<cpp->network->buswidth; i++)
 									cpp->network->networklist[i]->temp1 =
-										net->networklist[i + nindex*cpp->network->signals]->temp1;
+										net->networklist[i + nindex*cpp->network->buswidth]->temp1;
 							}
 						} else
 						{
-							if (cpp->network->signals != net->signals)
+							if (cpp->network->buswidth != net->buswidth)
 							{
 								ttyputerr(_("***ERROR: port %s on node %s in facet %s is %d wide, but is connected/exported with width %d"),
 									pp->protoname, describenodeinst(ni), describenodeproto(np),
-										cpp->network->signals, net->signals);
-								sigcount = mini(sigcount, cpp->network->signals);
+										cpp->network->buswidth, net->buswidth);
+								sigcount = mini(sigcount, cpp->network->buswidth);
 								if (sigcount == 1) sigcount = 0;
 							}
 							cpp->network->temp1 = net->temp1;
@@ -453,9 +474,9 @@ INTBIG sim_irsimrecurse(NODEPROTO *facet, XARRAY trans)
 				for(pp = cnp->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 				{
 					if (pp->network->temp1 != -1) continue;
-					if (pp->network->signals > 1)
+					if (pp->network->buswidth > 1)
 					{
-						for(i=0; i<pp->network->signals; i++)
+						for(i=0; i<pp->network->buswidth; i++)
 						{
 							net = pp->network->networklist[i];
 							if (net->temp1 != -1) continue;
@@ -482,7 +503,7 @@ INTBIG sim_irsimrecurse(NODEPROTO *facet, XARRAY trans)
 				}
 				maketrans(ni, singletrans);
 				transmult(singletrans, localtrans, subrot);
-				downhierarchy(ni, nindex);
+				downhierarchy(ni, cnp, nindex);
 				if (sim_irsimrecurse(cnp, subrot) != 0) backannotate = 1;
 				uphierarchy();
 			}
@@ -879,7 +900,7 @@ char *sim_irsimbuildnetname(char *name)
 	REGISTER VARIABLE *var;
 	REGISTER void *infstr;
 
-	gettraversalpath(el_curlib->curnodeproto, &nilist, &indexlist, &depth);
+	gettraversalpath(el_curlib->curnodeproto, &nilist, &indexlist, &depth, 0);
 	infstr = initinfstr();
 	for(i=0; i<depth; i++)
 	{
@@ -982,9 +1003,45 @@ INTBIG sim_irsimnewnetnumber(char *name)
 	}
 	i = sim_irsimnetnumber++;
 	(void)allocstring(&sim_irsimnets[i].name, name, sim_tool->cluster);
+#if SIMFSDBWRITE != 0
+	sim_irsimnewalias(i, name);
+#endif
 	return(i);
 }
 
+#if SIMFSDBWRITE != 0
+/*
+ * Routine to create an alias to IRSIM network object netnum
+ */
+void sim_irsimnewalias(INTBIG netnum, char *alias)
+{
+	REGISTER INTBIG newtotal, i;
+	IRSIMALIAS *newaliases;
+
+	if (sim_irsimaliasnumber >= sim_irsimaliastotal)
+	{
+		newtotal = sim_irsimaliasnumber * 2;
+		if (newtotal <= 0) newtotal = 10;
+		newaliases = (IRSIMALIAS *)emalloc(newtotal * (sizeof (IRSIMALIAS)), sim_tool->cluster);
+		if (newaliases == 0) return;
+		for(i=0; i<sim_irsimaliasnumber; i++)
+		{
+			newaliases[i].name = sim_irsimaliases[i].name;
+			newaliases[i].alias = sim_irsimaliases[i].alias;
+		}
+		if (sim_irsimaliastotal > 0) efree((char *)sim_irsimaliases);
+		sim_irsimaliases = newaliases;
+		sim_irsimaliastotal = newtotal;
+	}
+	i = sim_irsimaliasnumber++;
+	sim_irsimaliases[i].name = sim_irsimnets[netnum].name;
+	(void)allocstring(&sim_irsimaliases[i].alias, alias, sim_tool->cluster);
+#if 0
+	printf("irsimalias <%s> of <%s> %d\n", alias, sim_irsimnets[netnum].name, netnum);
+#endif
+}
+#endif
+
 INTBIG sim_irsimgetglobal(NETWORK *net)
 {
 	REGISTER INTBIG i;
