@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: simmossim.c
@@ -69,7 +70,7 @@ void sim_writemossim(NODEPROTO *np)
 	if ((net_tool->toolstate&TOOLON) == 0)
 	{
 		ttyputerr(_("Network tool must be running...turning it on"));
-		toolturnon(net_tool, FALSE);
+		toolturnon(net_tool);
 		ttyputerr(_("...now reissue the simulation command"));
 		return;
 	}
