@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: simsilos.c
@@ -104,7 +105,7 @@ void sim_writesilnetlist(NODEPROTO *np)
 	if ((net_tool->toolstate&TOOLON) == 0)
 	{
 		ttyputerr(_("Network tool must be running...turning it on"));
-		toolturnon(net_tool, FALSE);
+		toolturnon(net_tool);
 		ttyputerr(_("...now reissue the simulation command"));
 		return;
 	}
@@ -128,7 +129,7 @@ void sim_writesilnetlist(NODEPROTO *np)
 	{
 		if (np->creationdate)
 		{
-			(void)sprintf(numberstring,"$ FACET CREATED ON %s", timetostring(np->creationdate));
+			(void)sprintf(numberstring,"$ FACET CREATED ON %s", timetostring((time_t)np->creationdate));
 			sim_sendtosil(numberstring);
 			sim_sendtosil("\n");
 		}
@@ -140,7 +141,7 @@ void sim_writesilnetlist(NODEPROTO *np)
 		}
 		if (np->revisiondate)
 		{
-			(void)sprintf(numberstring,"$ LAST REVISED %s", timetostring(np->revisiondate));
+			(void)sprintf(numberstring,"$ LAST REVISED %s", timetostring((time_t)np->revisiondate));
 			sim_sendtosil(numberstring);
 			sim_sendtosil("\n");
 		}
@@ -431,6 +432,7 @@ void sim_writesilinstances(NODEPROTO *np, BOOLEAN top)
 				for(pp = cnp->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 				{
 					ipp = equivalentport(cnp, pp, ni->proto);
+					if (ipp == NOPORTPROTO) continue;
 					sim_sendtosil(sim_silportname(top, ni, ipp, np));
 				}
 				break;
@@ -765,7 +767,7 @@ void sim_sendtosil(char *str)
 	if (*str == '\n') count = 0;
 	if ((count + (i = strlen(str))) > MAXSTR)
 	{
-		if(*str == '$')
+		if (*str == '$')
 		{
 			xprintf(sim_silfile, "\n");
 			count = 0;
@@ -1011,11 +1013,8 @@ char *sim_sil_isaport(NODEPROTO *np, char *portname)
 	REGISTER PORTPROTO *pp;
 
 	if (strchr(portname, '[') == NULL) return(portname);		/* no references anyhow */
-
-	for (pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-	{
-		if (namesame(pp->protoname, portname) == 0) return(sim_convertsub(portname));
-	}
+	pp = getportproto(np, portname);
+	if (pp != NOPORTPROTO) return(sim_convertsub(portname));
 	return(portname);
 }
 
