@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: simsim.c
@@ -67,7 +68,7 @@ void sim_writesim(NODEPROTO *simnt, INTBIG format)
 	if ((net_tool->toolstate&TOOLON) == 0)
 	{
 		ttyputerr(_("Network tool must be running...turning it on"));
-		toolturnon(net_tool, FALSE);
+		toolturnon(net_tool);
 		ttyputerr(_("...now reissue the simulation command"));
 		return;
 	}
@@ -110,10 +111,10 @@ void sim_writesim(NODEPROTO *simnt, INTBIG format)
 	if ((us_useroptions&NODATEORVERSION) == 0)
 	{
 		if (simnt->creationdate != 0)
-			xprintf(f, "| Facet created %s\n", timetostring(simnt->creationdate));
+			xprintf(f, "| Facet created %s\n", timetostring((time_t)simnt->creationdate));
 		xprintf(f, "| Version %ld", simnt->version);
 		if (simnt->revisiondate != 0)
-			xprintf(f, " last revised %s\n", timetostring(simnt->revisiondate));
+			xprintf(f, " last revised %s\n", timetostring((time_t)simnt->revisiondate));
 	}
 	if (format == COSMOS)
 	{
@@ -396,8 +397,7 @@ void sim_simpointout(char *nonum, INTBIG format)
 	/* figure out the desired nodeinst number */
 	if (*nonum >= '0' && *nonum <= '9') want = atoi(nonum); else
 	{
-		for(pp = sim_simnt->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-			if (namesame(nonum, pp->protoname) == 0) break;
+		pp = getportproto(sim_simnt, nonum);
 		if (pp == NOPORTPROTO)
 		{
 			ttyputerr(_("Cannot find a port called '%s'"), nonum);
