@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: simspicerun.c
@@ -39,6 +40,8 @@
 #include <math.h>
 #include <errno.h>
 
+#define SP_BUF_SZ 1024
+
 /******************** SPICE NUMBER OUTPUT QUEUE ********************/
 
 #define NONUMBERS  ((NUMBERS *)-1)
@@ -78,7 +81,6 @@ static float   *sim_spice_val;
 static INTBIG   sim_spice_signals = 0;				/* entries in "signames/sigtypes" */
 static INTBIG   sim_spice_iter;
 static INTBIG   sim_spice_limit = 0;				/* entries in "time/val" */
-static float    sim_spice_lowy, sim_spice_highy;	/* range of an entry */
 static INTBIG   sim_spice_filepos;
 static INTBIG   sim_spice_filelen;
 static BOOLEAN  sim_spice_tr0binary;				/* true if tr0 file is binary */
@@ -97,7 +99,7 @@ static BOOLEAN  sim_spice_topofsignals(char**);
 static char    *sim_spice_nextsignals(void);
 static BOOLEAN  sim_spice_plotvalues(INTBIG);
 static BOOLEAN  sim_spice_ensurespace(INTBIG);
-static INTBIG   sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode);
+static INTBIG   sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode, void *dia);
 static INTBIG   sim_spice_findsignalname(char *name);
 static BOOLEAN  sim_spice_charhandlerschem(WINDOWPART *w, INTSML chr, INTBIG special);
 static BOOLEAN  sim_spice_charhandlerwave(WINDOWPART *w, INTSML chr, INTBIG special);
@@ -109,6 +111,7 @@ static PA0LINE *sim_allocpa0line(void);
 static void     sim_freepa0line(PA0LINE *pl);
 static void     sim_spice_removesignal(void);
 static void     sim_spice_addarguments(INTBIG index, char *deckname, char *facetname);
+static INTBIG   sim_spice_get_double_fromsimulator2(double *d) ;
 
 /*
  * Routine to free all memory allocated by this module.
@@ -209,9 +212,8 @@ void sim_spice_execute(char *infile, char *outfile, NODEPROTO *facet)
 	REGISTER FILE *outputfile;
 	REGISTER PA0LINE *pl;
 	VARIABLE *var;
-	extern DIALOG us_progressdialog;
 	char *host, *path, *name, *truename, *pa0file, *pa0truefile, *pt;
-	REGISTER void *infstr;
+	REGISTER void *infstr, *dia;
 
 	/* determine execution/parsing mode */
 	var = getvalkey((INTBIG)sim_tool, VTOOL, VINTEGER, sim_dontrunkey);
@@ -233,9 +235,11 @@ void sim_spice_execute(char *infile, char *outfile, NODEPROTO *facet)
 		simstate = sim_window_isactive(&np);
 		if ((simstate&SIMWINDOWWAVEFORM) != 0)
 		{
+#ifndef ACCUMALATE_TRACES
 			ttyputerr(_("Only one simulation window can run at a time"));
 			ttyputmsg(_("Terminate the current one before reading SPICE output"));
 			return;
+#endif
 		}
 		if ((simstate&SIMWINDOWSCHEMATIC) != 0)
 			sim_window_stopsimulation();
@@ -395,8 +399,9 @@ void sim_spice_execute(char *infile, char *outfile, NODEPROTO *facet)
 	/* show progress if possible */
 	if (sim_spice_filelen > 0)
 	{
-		if (DiaInitDialog(&us_progressdialog)) return;
-		DiaPercent(1, 0);
+		dia = DiaInitProgress(0, 0);
+		if (dia == 0) return;
+		DiaSetProgress(dia, 0, sim_spice_filelen);
 	}
 	sim_spice_filepos = 0;
 
@@ -413,17 +418,21 @@ void sim_spice_execute(char *infile, char *outfile, NODEPROTO *facet)
 		important = -1;
 	} else
 	{
-		important = sim_spice_parseoutput(*infile != 0, outputfile, parsemode);
+		important = sim_spice_parseoutput(*infile != 0, outputfile, parsemode, dia);
 	}
 
 	/* clean up */
-	if (sim_spice_filelen > 0) DiaDoneDialog();
+	if (sim_spice_filelen > 0) DiaDoneProgress(dia);
 	(void)sim_spice_terminate(outputfile, outfile, infile);
 	if (stopping(STOPREASONSPICE)) return;
 
 	/* plot the list of numbers */
 	if (important < 0) return;
+#ifdef SHOW_IMPORTANT
 	if (sim_spice_plotvalues(important)) ttyputerr(_("Problem making the plot"));
+#else
+	if (sim_spice_plotvalues(0)) ttyputerr(_("Problem making the plot"));
+#endif
 }
 
 /*
@@ -467,16 +476,18 @@ void sim_spice_addarguments(INTBIG index, char *deckname, char *facetname)
  * to "outputfile" if it is valid).
  * Returns the number of important traces in the data (-1 on error).
  */
-INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
+INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode, void *dia)
 {
 	char line[MAXLINE+5], *ptr, *start;
-	REGISTER INTBIG i, j, k, l, important, nodcnt, numnoi, rowcount, retval;
+	REGISTER INTBIG i, j, k, l, important, nodcnt, numnoi, rowcount, retval,
+		multiplier;
 	BOOLEAN past_end, first, datamode, knows;
 	float numbers[20];
 	REGISTER PA0LINE *pl;
 	double lastt, time, value;
 	NUMBERS *num, *numend;
 	REGISTER void *infstr;
+	double d;
 
 	/* parse SPICE rawfile output */
 	if ((sim_spice_state & SPICERAWFILE) != 0)
@@ -490,7 +501,7 @@ INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
 			if (stopping(STOPREASONSPICE)) return(-1);
 			if (sim_spice_getlinefromsimulator(line)) break;
 			if (sim_spice_filelen > 0)
-				DiaPercent(1, sim_spice_filepos*100/sim_spice_filelen);
+				DiaSetProgress(dia, sim_spice_filepos, sim_spice_filelen);
 			if (first)
 			{
 				/* check the first line for HSPICE format possibility */
@@ -555,8 +566,6 @@ INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
 						ttyputerr(_("Error: end of file during signal names"));
 						return(-1);
 					}
-					if (sim_spice_filelen > 0)
-						DiaPercent(1, sim_spice_filepos*100/sim_spice_filelen);
 					ptr = line;
 					while (*ptr == ' ' || *ptr == '\t') ptr++;
 					if (myatoi(ptr) != i)
@@ -600,6 +609,9 @@ INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
 					num->nextnumbers = NONUMBERS;
 					numend = num;
 
+					if (sim_spice_filelen > 0)
+						DiaSetProgress(dia, sim_spice_filepos, sim_spice_filelen);
+
 					for(i=0; i<=sim_spice_signals; i++)
 					{
 						if (stopping(STOPREASONSPICE)) return(-1);
@@ -609,8 +621,6 @@ INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
 								j, rowcount);
 							return(-1);
 						}
-						if (sim_spice_filelen > 0)
-							DiaPercent(1, sim_spice_filepos*100/sim_spice_filelen);
 						ptr = line;
 						if (i == 0)
 						{
@@ -625,6 +635,44 @@ INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
 					}
 				}
 			}
+			if (namesame(line, "Binary") == 0)
+			{
+				if (sim_spice_signals < 0)
+				{
+					ttyputerr(_("Missing variable count in file"));
+					return(-1);
+				}
+				if (rowcount < 0)
+				{
+					ttyputerr(_("Missing point count in file"));
+					return(-1);
+				}
+				for(j=0; j<rowcount; j++)
+				{
+					num = sim_spice_allocnumbers(sim_spice_signals);
+					if (num == NONUMBERS) return(-1);
+					if (numend == NONUMBERS) sim_spice_numbers = num; else
+						numend->nextnumbers = num;
+					num->nextnumbers = NONUMBERS;
+					numend = num;
+
+					if (sim_spice_filelen > 0 )
+						DiaSetProgress(dia, sim_spice_filepos, sim_spice_filelen);
+
+					for(i=0; i<=sim_spice_signals; i++)
+					{
+						if (stopping(STOPREASONSPICE)) return(-1);
+						if (sim_spice_get_double_fromsimulator2(&d) == EOF)
+						{
+							ttyputerr(_("Error: end of file during data points (read %ld out of %ld)"),
+								j, rowcount);
+							return(-1);
+						}
+						if (i == 0) num->time = (float)d; else
+							num->list[i-1] = (float)d;
+					}
+				}
+			}
 		}
 
 		if (sim_spice_signals > 0)
@@ -646,7 +694,7 @@ INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
 			if (stopping(STOPREASONSPICE)) break;
 			if (sim_spice_getlinefromsimulator(line)) break;
 			if (sim_spice_filelen > 0)
-				DiaPercent(1, sim_spice_filepos*100/sim_spice_filelen);
+				DiaSetProgress(dia, sim_spice_filepos, sim_spice_filelen);
 			if (first)
 			{
 				/* check the first line for HSPICE format possibility */
@@ -744,7 +792,7 @@ INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
 			if (stopping(STOPREASONSPICE)) break;
 			if (sim_spice_getlinefromsimulator(line)) break;
 			if (sim_spice_filelen > 0)
-				DiaPercent(1, sim_spice_filepos*100/sim_spice_filelen);
+				DiaSetProgress(dia, sim_spice_filepos, sim_spice_filelen);
 
 			if (first)
 			{
@@ -845,7 +893,16 @@ INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
 		numnoi = atoi(line);
 		for(j=0; j<4; j++) line[j] = (char)sim_spice_getfromsimulator();
 		/* cndcnt = atoi(line); */
+
+		/*
+		 * Although this isn't documented anywhere, it appears that the 4th
+		 * value on the line is a multiplier for the first, which allows
+		 * there to be more than 10000 nodes.
+		 */
 		for(j=0; j<4; j++) line[j] = (char)sim_spice_getfromsimulator();
+		multiplier = atoi(line);
+		if (multiplier != 0) nodcnt += multiplier * 10000;
+
 		sim_spice_signals = numnoi + nodcnt - 1;
 
 		/* get version number (known to work with 9007, 9601) */
@@ -880,7 +937,7 @@ INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
 			k = sim_spice_getfromsimulator();
 			if (!sim_spice_tr0binary && k == '\n') j--;
 		}
-		if (sim_spice_filelen > 0) DiaPercent(1, sim_spice_filepos*100/sim_spice_filelen);
+		if (sim_spice_filelen > 0) DiaSetProgress(dia, sim_spice_filepos, sim_spice_filelen);
 
 		/* get the type of each signal */
 		important = numnoi;
@@ -914,6 +971,9 @@ INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
 				if (l == '\n') continue;
 				if (l == ' ') break;
 				line[j++] = (char)l;
+#if 1
+				if (j >= MAXLINE) break;
+#endif
 			}
 			line[j] = 0;
 			l = (j+15) / 16 * 16 - 1;
@@ -976,7 +1036,7 @@ INTBIG sim_spice_parseoutput(INTBIG running, FILE *outputfile, INTBIG parsemode)
 		for(;;)
 		{
 			if (stopping(STOPREASONSPICE)) break;
-			if (sim_spice_filelen > 0) DiaPercent(1, sim_spice_filepos*100/sim_spice_filelen);
+			if (sim_spice_filelen > 0) DiaSetProgress(dia, sim_spice_filepos, sim_spice_filelen);
 
 			/* get the first number, see if it terminates */
 			retval = sim_spice_gethspicefloat(&time);
@@ -1160,6 +1220,22 @@ BOOLEAN sim_spice_readbtr0block(BOOLEAN firstbyteread)
 	return(FALSE);
 }
 
+INTBIG sim_spice_get_double_fromsimulator2(double *d)
+{
+	INTBIG i;
+	if (sim_spice_streamfromsim != 0)
+	{
+		i = xfread((char *)d, sizeof(double), 1, sim_spice_streamfromsim);
+		if (i != 1) return(EOF);
+	} else
+	{
+		i = eread(sim_fromsim[0], (char *)d, (INTBIG)sizeof(double));
+		if (i != (INTBIG)sizeof(double)) return(EOF);
+	}
+	sim_spice_filepos+=(int)sizeof(double);
+	return(i);
+}
+
 /*
  * Routine to get the next character from the simulator (file or pipe).
  * Returns EOF at end of file.
@@ -1168,6 +1244,11 @@ INTSML sim_spice_getfromsimulator(void)
 {
 	INTBIG i;
 	char val;
+#ifdef USE_BUFFER
+	static char buf[SP_BUF_SZ];
+	static char *bufp = buf;
+	static int n=0;
+#endif
 
 	if (sim_spice_streamfromsim != 0)
 	{
@@ -1201,10 +1282,35 @@ INTSML sim_spice_getfromsimulator(void)
 				return(val&0xFF);
 			}
 		}
+#ifdef USE_BUFFER
+		if (n==0) {
+			n = xfread(buf, 1, SP_BUF_SZ, sim_spice_streamfromsim);
+			bufp=buf;
+		}
+
+		if (--n >=0) {
+			val = *bufp++;
+			i = 1;
+		} else i = 0;
+#else
 		i = xfread(&val, 1, 1, sim_spice_streamfromsim);
+#endif
+
 	} else
 	{
+#ifdef USE_BUFFER
+		if (n==0) {
+			n = eread(sim_fromsim[0], buf, SP_BUF_SZ);
+			bufp=buf;
+		}
+
+		if (--n >=0) {
+			val = *bufp++;
+			i = 1;
+		} else i = 0;
+#else
 		i = eread(sim_fromsim[0], &val, 1);
+#endif
 	}
 	if (i != 1) return(EOF);
 	sim_spice_filepos++;
@@ -1260,10 +1366,9 @@ static COMCOMP sim_spice_picktrace = {NOKEYWORD, sim_spice_topofsignals,
  */
 BOOLEAN sim_spice_plotvalues(INTBIG important)
 {
-	REGISTER INTBIG i, numtotal, ysteps, j, k, tr, position, numtraces;
+	REGISTER INTBIG i, numtotal, j, k, tr, position, numtraces;
 	INTBIG oldsigcount;
 	double min, max;
-	float r, mul, tl,th, lowy, highy;
 	char *pars[3], **oldsignames, *pt, *start;
 	REGISTER NODEPROTO *plotnp;
 	REGISTER VARIABLE *var;
@@ -1308,52 +1413,12 @@ BOOLEAN sim_spice_plotvalues(INTBIG important)
 	/* get memory for the values */
 	if (sim_spice_ensurespace(numtotal)) return(TRUE);
 
-	/* get Y range of values */
-	lowy = highy = (float)sim_spice_numbers->time;
 	j = 0;
 	for(num = sim_spice_numbers; num != NONUMBERS; num = num->nextnumbers)
 	{
 		sim_spice_time[j++] = num->time;
-		for(i=0; i<sim_spice_signals; i++)
-		{
-			if (num->list[i] < lowy) lowy = num->list[i];
-			if (num->list[i] > highy) highy = num->list[i];
-		}
 	}
 
-	/* spread the Y values if they are all equal */
-	if (lowy == highy)
-	{
-		highy = lowy * 2.0f;   lowy = 0.0;
-		if (highy == 0.0) highy = 1.0;
-	}
-
-	/* determine range of values, multiply units by "mul" to get magnitude */
-	r = highy - lowy;
-	mul = 1.0;
-	while (r > 10.0) { r /= 10.0;   mul *= 10.0; }
-	while (r < 1.0)  { r *= 10.0;   mul /= 10.0; }
-
-	/* range selected, step lower bound to a half-step below */
-	ysteps = (INTBIG)(r-1);
-	j = (INTBIG)((lowy+highy) / 2.0 / mul);
-	do
-	{
-		ysteps++;
-		tl = j*mul - ysteps * mul / 2.0f;
-	} while (tl > lowy);
-
-	/* adjust upper bound to a half step above */
-	th = tl + ysteps*mul;
-	while (th < highy)
-	{
-		ysteps++;
-		th = tl + ysteps*mul;
-	}
-
-	/* set the globals with the range of the simulation window */
-	sim_spice_lowy = tl;   sim_spice_highy = th;
-
 	/* find former signal order */
 	oldsigcount = 0;
 	numtraces = 1;
@@ -1387,9 +1452,6 @@ BOOLEAN sim_spice_plotvalues(INTBIG important)
 	if (sim_window_create(numtraces, plotnp, sim_spice_charhandlerwave,
 		sim_spice_charhandlerschem, SPICE)) return(TRUE);
 
-	/* create the trace nodes */
-	sim_window_setanaextents(sim_spice_lowy, sim_spice_highy);
-
 	/* add in saved signals */
 	for(j=0; j<oldsigcount; j++)
 	{
@@ -1416,7 +1478,6 @@ BOOLEAN sim_spice_plotvalues(INTBIG important)
 			for(num = sim_spice_numbers; num != NONUMBERS; num = num->nextnumbers)
 				sim_spice_val[k++] = num->list[i];
 			sim_window_loadanatrace(tr, numtotal, sim_spice_time, sim_spice_val);
-			sim_window_setanarange(tr, sim_spice_lowy, sim_spice_highy);
 			break;
 		}
 	}
@@ -1429,9 +1490,10 @@ BOOLEAN sim_spice_plotvalues(INTBIG important)
 		for(num = sim_spice_numbers; num != NONUMBERS; num = num->nextnumbers)
 			sim_spice_val[k++] = num->list[i];
 		sim_window_loadanatrace(tr, numtotal, sim_spice_time, sim_spice_val);
-		sim_window_setanarange(tr, sim_spice_lowy, sim_spice_highy);
 	}
 
+	sim_window_auto_anarange();
+
 	/* clean up */
 	min = sim_spice_time[0];   max = sim_spice_time[numtotal-1];
 	if (min >= max)
@@ -1512,8 +1574,7 @@ BOOLEAN sim_spice_charhandlerschem(WINDOWPART *w, INTSML chr, INTBIG special)
 BOOLEAN sim_spice_charhandlerwave(WINDOWPART *w, INTSML chr, INTBIG special)
 {
 	char *par[3];
-	float v, flow, fhigh;
-	REGISTER INTBIG tr, otr, i, j, thispos, frame, oframe;
+	REGISTER INTBIG tr, i, j, thispos, frame;
 	REGISTER NUMBERS *num;
 
 	/* special characters are not handled here */
@@ -1534,93 +1595,36 @@ BOOLEAN sim_spice_charhandlerwave(WINDOWPART *w, INTSML chr, INTBIG special)
 			ttyputinstruction(" a", 6, _("add signal to simulation window in a new frame"));
 			ttyputinstruction(" o", 6, _("overlay signal, added to current frame"));
 			ttyputinstruction(" r", 6, _("remove selected signal from simulation window"));
+			ttyputinstruction(" e", 6, _("remove all traces"));
 			ttyputinstruction(" p", 6, _("preserve snapshot of simulation window in database"));
 			return(FALSE);
 		case '9':		/* show entire vertical range */
 			tr = sim_window_gethighlighttrace();
-			sim_window_getanaextents(&flow, &fhigh);
-			sim_window_inittraceloop();
-			frame = sim_window_gettraceframe(tr);
-			for(;;)
-			{
-				otr = sim_window_nexttraceloop();
-				if (otr == 0) break;
-				oframe = sim_window_gettraceframe(otr);
-				if (tr != 0 && frame != oframe) continue;
-				sim_window_setanarange(otr, flow, fhigh);
-			}
+			sim_window_auto_anarange() ;
 			sim_window_redraw();
 			return(FALSE);
 		case '7':		/* zoom vertical range in */
 			tr = sim_window_gethighlighttrace();
-			sim_window_inittraceloop();
 			frame = sim_window_gettraceframe(tr);
-			for(;;)
-			{
-				otr = sim_window_nexttraceloop();
-				if (otr == 0) break;
-				oframe = sim_window_gettraceframe(otr);
-				if (tr != 0 && frame != oframe) continue;
-				sim_window_getanarange(otr, &flow, &fhigh);
-				v = (fhigh-flow) / 4;
-				flow = flow + v;
-				fhigh = fhigh - v;
-				sim_window_setanarange(otr, flow, fhigh);
-			}
+			sim_window_zoom_frame(frame);
 			sim_window_redraw();
 			return(FALSE);
 		case '0':		/* zoom vertical range out */
 			tr = sim_window_gethighlighttrace();
-			sim_window_inittraceloop();
 			frame = sim_window_gettraceframe(tr);
-			for(;;)
-			{
-				otr = sim_window_nexttraceloop();
-				if (otr == 0) break;
-				oframe = sim_window_gettraceframe(otr);
-				if (tr != 0 && frame != oframe) continue;
-				sim_window_getanarange(otr, &flow, &fhigh);
-				v = (fhigh-flow) / 2;
-				flow = flow - v;
-				fhigh = fhigh + v;
-				sim_window_setanarange(otr, flow, fhigh);
-			}
+			sim_window_zoomout_frame(frame);
 			sim_window_redraw();
 			return(FALSE);
 		case '8':		/* shift vertical range up */
 			tr = sim_window_gethighlighttrace();
-			sim_window_inittraceloop();
 			frame = sim_window_gettraceframe(tr);
-			for(;;)
-			{
-				otr = sim_window_nexttraceloop();
-				if (otr == 0) break;
-				oframe = sim_window_gettraceframe(otr);
-				if (tr != 0 && frame != oframe) continue;
-				sim_window_getanarange(otr, &flow, &fhigh);
-				v = (fhigh-flow) / 4;
-				flow = flow - v;
-				fhigh = fhigh - v;
-				sim_window_setanarange(otr, flow, fhigh);
-			}
+			sim_window_shiftup_frame(frame);
 			sim_window_redraw();
 			return(FALSE);
 		case '2':		/* shift vertical range down */
 			tr = sim_window_gethighlighttrace();
-			sim_window_inittraceloop();
 			frame = sim_window_gettraceframe(tr);
-			for(;;)
-			{
-				otr = sim_window_nexttraceloop();
-				if (otr == 0) break;
-				oframe = sim_window_gettraceframe(otr);
-				if (tr != 0 && frame != oframe) continue;
-				sim_window_getanarange(otr, &flow, &fhigh);
-				v = (fhigh-flow) / 4;
-				flow = flow + v;
-				fhigh = fhigh + v;
-				sim_window_setanarange(otr, flow, fhigh);
-			}
+			sim_window_shiftdown_frame(frame);
 			sim_window_redraw();
 			return(FALSE);
 		case 'p':		/* preserve image in {simulation_snapshot} view */
@@ -1635,12 +1639,13 @@ BOOLEAN sim_spice_charhandlerwave(WINDOWPART *w, INTSML chr, INTBIG special)
 			/* create a new trace in the last slot */
 			tr = sim_window_newtrace(-1, sim_spice_signames[i], 0);
 			j = 0;
-			for(num = sim_spice_numbers; num != NONUMBERS; num = num->nextnumbers)
+			for(num = sim_spice_numbers; num != NONUMBERS; num = num->nextnumbers) {
 				sim_spice_val[j++] = num->list[i];
+			}
 			sim_window_loadanatrace(tr, j, sim_spice_time, sim_spice_val);
-			sim_window_setanarange(tr, sim_spice_lowy, sim_spice_highy);
 			sim_window_cleartracehighlight();
 			sim_window_addhighlighttrace(tr);
+			sim_window_auto_anarange();
 			sim_window_redraw();
 			return(FALSE);
 		case 'o':		/* overlap trace */
@@ -1655,18 +1660,23 @@ BOOLEAN sim_spice_charhandlerwave(WINDOWPART *w, INTSML chr, INTBIG special)
 			/* create a new trace in this slot */
 			tr = sim_window_newtrace(thispos, sim_spice_signames[i], 0);
 			j = 0;
-			for(num = sim_spice_numbers; num != NONUMBERS; num = num->nextnumbers)
+			for(num = sim_spice_numbers; num != NONUMBERS; num = num->nextnumbers) {
 				sim_spice_val[j++] = num->list[i];
+			}
 			sim_window_loadanatrace(tr, j, sim_spice_time, sim_spice_val);
-			sim_window_setanarange(tr, sim_spice_lowy, sim_spice_highy);
 			sim_window_cleartracehighlight();
 			sim_window_addhighlighttrace(tr);
+			sim_window_auto_anarange();
 			sim_window_redraw();
 			return(FALSE);
 		case 'r':		/* remove trace */
 		case DELETEKEY:
 			sim_spice_removesignal();
 			return(FALSE);
+		case 'e':
+			sim_window_killalltraces(FALSE);
+			sim_window_redraw();
+			return(FALSE);
 	}
 	return(us_charhandler(w, chr, special));
 }
@@ -1774,9 +1784,11 @@ void sim_spice_addhighlightednet(BOOLEAN overlay)
 	tr = sim_window_newtrace(frameno, sim_spice_signames[i], 0);
 	j = 0;
 	for(num = sim_spice_numbers; num != NONUMBERS; num = num->nextnumbers)
+	{
 		sim_spice_val[j++] = num->list[i];
+	}
 	sim_window_loadanatrace(tr, j, sim_spice_time, sim_spice_val);
-	sim_window_setanarange(tr, sim_spice_lowy, sim_spice_highy);
+	sim_window_auto_anarange();
 	sim_window_redraw();
 	sim_window_cleartracehighlight();
 	sim_window_addhighlighttrace(tr);
@@ -1840,6 +1852,7 @@ char *sim_spice_signalname(NETWORK *net)
 	REGISTER PORTEXPINST *pe;
 	REGISTER NODEINST *ni;
 	REGISTER void *infstr;
+	INTBIG index;
 
 	/* get simulation facet, quit if not simulating */
 	if (sim_window_isactive(&np) == 0) return(0);
@@ -1851,7 +1864,7 @@ char *sim_spice_signalname(NETWORK *net)
 		if (net->parent == np) break;
 
 		/* find the instance that is the proper parent of this facet */
-		ni = descentparent(net->parent);
+		ni = descentparent(net->parent, &index);
 		if (ni == NONODEINST) break;
 
 		/* see if the network is exported from this level */
@@ -1888,7 +1901,7 @@ char *sim_spice_signalname(NETWORK *net)
 		if (netpar->cell == np->cell) break;
 
 		/* find the instance that is the proper parent of this facet */
-		ni = descentparent(netpar);
+		ni = descentparent(netpar, &index);
 		if (ni == NONODEINST) break;
 
 		varname = getvalkey((INTBIG)ni, VNODEINST, VSTRING, el_node_name_key);
