@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: tecart.c
@@ -671,7 +672,7 @@ void art_shapeportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans,
 	pindex = ni->proto->primindex;
 	if (pindex == NPIN || pindex == NARROW)
 	{
-		tech_fillportpoly(ni, pp, poly, trans, art_nodeprotos[pindex-1], CLOSED);
+		tech_fillportpoly(ni, pp, poly, trans, art_nodeprotos[pindex-1], CLOSED, lambdaofnode(ni));
 		return;
 	}
 
