@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: tecbipolar.c
@@ -29,8 +30,6 @@
  */
 
 #include "config.h"
-#if TECBIPOLAR
-
 #include "global.h"
 #include "egraphics.h"
 #include "tech.h"
@@ -856,5 +855,3 @@ BOOLEAN bipolar_initprocess(TECHNOLOGY *tech, INTBIG pass)
 {
 	return(FALSE);
 }
-
-#endif  /* TECBIPOLAR - at top */
