@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: teccmos.c
@@ -30,8 +31,6 @@
  */
 
 #include "config.h"
-#if TECCMOS
-
 #include "global.h"
 #include "egraphics.h"
 #include "tech.h"
@@ -719,5 +718,3 @@ BOOLEAN cmos_initprocess(TECHNOLOGY *tech, INTBIG pass)
 {
 	return(FALSE);
 }
-
-#endif  /* TECCMOS - at top */
