@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: tecefido.c
@@ -29,8 +30,6 @@
  */
 
 #include "config.h"
-#if TECEFIDO
-
 #include "global.h"
 #include "egraphics.h"
 #include "tech.h"
@@ -440,5 +439,3 @@ BOOLEAN efido_initprocess(TECHNOLOGY *tech, INTBIG pass)
 {
 	return(FALSE);
 }
-
-#endif  /* TECEFIDO - at top */
