@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: tecfpga.c
@@ -29,8 +30,6 @@
  */
 
 #include "config.h"
-#if TECFPGA
-
 #include "global.h"
 #include "egraphics.h"
 #include "tech.h"
@@ -193,7 +192,7 @@ static BOOLEAN    fpga_makeprimpip(NODEPROTO *np, LISPTREE *lt, FPGANODE *fn, FP
 static BOOLEAN    fpga_makeprimport(NODEPROTO *np, LISPTREE *lt, FPGAPORT *fp);
 static NODEPROTO *fpga_placeprimitives(LISPTREE *lt);
 static BOOLEAN    fpga_pushkeyword(char *pt);
-static LISPTREE  *fpga_readfile(FILE *f);
+static LISPTREE  *fpga_readfile(FILE *f, void *dia);
 static void       fpga_reevaluatepips(NODEINST *ni, FPGANODE *fn);
 static BOOLEAN    fpga_repeateractive(NODEINST *ni);
 static void       fpga_setpips(char *name);
@@ -436,9 +435,8 @@ void fpga_setmode(INTBIG count, char *par[])
 	FILE *f;
 	REGISTER INTBIG l, total;
 	REGISTER LISPTREE *lt;
-	extern DIALOG us_progressdialog;
 	static INTBIG filetypefpga = -1;
-	REGISTER void *infstr;
+	REGISTER void *infstr, *dia;
 
 	if (count == 0)
 	{
@@ -552,17 +550,17 @@ void fpga_setmode(INTBIG count, char *par[])
 
 		/* prepare for input */
 		fpga_filesize = filesize(f);
-		if (DiaInitDialog(&us_progressdialog))
+		dia = DiaInitProgress(_("Reading FPGA architecture file..."), 0);
+		if (dia == 0)
 		{
 			xclose(f);
 			return;
 		}
-		DiaPercent(1, 0);
-		DiaSetText(2, _("Reading FPGA architecture file..."));
+		DiaSetProgress(dia, 0, fpga_filesize);
 
 		/* read the file */
-		lt = fpga_readfile(f);
-		DiaDoneDialog();
+		lt = fpga_readfile(f, dia);
+		DiaDoneProgress(dia);
 		xclose(f);
 		if (lt == 0)
 		{
@@ -662,7 +660,7 @@ INTBIG fpga_intnodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win, POLY
 						fn->piplist[i]->pipactive |= ACTIVESAVE;
 
 				/* propagate inactive segments to others that may be active */
-				gettraversalpath(ni->parent, &nilist, &indexlist, &depth);
+				gettraversalpath(ni->parent, &nilist, &indexlist, &depth, 1);
 				if (depth > 0)
 				{
 					oni = nilist[depth-1];
@@ -684,7 +682,7 @@ INTBIG fpga_intnodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win, POLY
 						}
 						if (j < fn->portcount) fn->netlist[i]->segactive |= ACTIVESAVE;
 					}
-					downhierarchy(oni, 0);
+					downhierarchy(oni, oni->proto, 0);
 				}
 
 				/* add up the active segments */
@@ -900,7 +898,7 @@ void fpga_shapeportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans
 	if (pindex <= NODEPROTOCOUNT)
 	{
 		/* static primitive */
-		tech_fillportpoly(ni, pp, poly, trans, fpga_nodeprotos[pindex-1], CLOSED);
+		tech_fillportpoly(ni, pp, poly, trans, fpga_nodeprotos[pindex-1], CLOSED, lambdaofnode(ni));
 		return;
 	}
 
@@ -1016,7 +1014,7 @@ BOOLEAN fpga_arcendactive(ARCINST *ai, INTBIG j)
 	if (pindex == 0)
 	{
 		/* follow down into facet */
-		downhierarchy(ni, 0);
+		downhierarchy(ni, ni->proto, 0);
 		subni = pp->subnodeinst;
 		for(pi = subni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 		{
@@ -1033,7 +1031,7 @@ BOOLEAN fpga_arcendactive(ARCINST *ai, INTBIG j)
 		if (ni->proto->tech == fpga_tech && pindex > NODEPROTOCOUNT)
 		{
 			fn = fpga_nodes[pindex - NODEPROTOCOUNT - 1];
-			downhierarchy(ni, 0);
+			downhierarchy(ni, ni->proto, 0);
 			fpga_reevaluatepips(ni, fn);
 			uphierarchy();
 			if (fn->netcount != 0)
@@ -1060,7 +1058,7 @@ BOOLEAN fpga_arcendactive(ARCINST *ai, INTBIG j)
 			if (fpga_arcendactive(oai, newend)) return(TRUE);
 		}
 
-		gettraversalpath(ni->parent, &nilist, &indexlist, &depth);
+		gettraversalpath(ni->parent, &nilist, &indexlist, &depth, 1);
 		if (depth > 0)
 		{
 			oni = nilist[depth-1];
@@ -1077,7 +1075,7 @@ BOOLEAN fpga_arcendactive(ARCINST *ai, INTBIG j)
 					if (fpga_arcendactive(oai, newend)) break;
 				}
 
-				downhierarchy(oni, 0);
+				downhierarchy(oni, oni->proto, 0);
 				if (pi != NOPORTARCINST) return(TRUE);
 			}
 		}
@@ -1098,7 +1096,7 @@ BOOLEAN fpga_arcactive(ARCINST *ai)
 	if (ai->end[0].portarcinst == NOPORTARCINST) return(FALSE);
 
 	/* see if there is a cache on the arc */
-	gettraversalpath(ai->parent, &nilist, &indexlist, &depth);
+	gettraversalpath(ai->parent, &nilist, &indexlist, &depth, 0);
 	var = getvalkey((INTBIG)ai, VARCINST, VCHAR|VISARRAY, fpga_arcactivecachekey);
 	if (var != NOVARIABLE)
 	{
@@ -1166,7 +1164,7 @@ void fpga_reevaluatepips(NODEINST *ni, FPGANODE *fn)
 	if (fn->netcount == 0 && fn->pipcount == 0) return;
 
 	/* see if there is a cache on the node */
-	gettraversalpath(ni->parent, &nilist, &indexlist, &depth);
+	gettraversalpath(ni->parent, &nilist, &indexlist, &depth, 0);
 	var = getvalkey((INTBIG)ni, VNODEINST, VCHAR|VISARRAY, fpga_nodepipcachekey);
 	if (var != NOVARIABLE)
 	{
@@ -1329,7 +1327,7 @@ void fpga_findvariableobjects(NODEINST *ni, INTBIG varkey, void (*setit)(char*))
 	char tempbuf[100];
 
 	/* search hierarchical path */
-	gettraversalpath(ni->parent, &localnilist, &indexlist, &localdepth);
+	gettraversalpath(ni->parent, &localnilist, &indexlist, &localdepth, 0);
 	depth = 0;
 	for(i = localdepth - 1; i >= 0; i--)
 	{
@@ -1425,7 +1423,7 @@ void fpga_describenetseg(NODEINST *ni, FPGANET *fnet, INTBIG whichseg, POLYGON *
  * Routine to read the FPGA file in "f" and create a LISPTREE structure which is returned.
  * Returns zero on error.
  */
-LISPTREE *fpga_readfile(FILE *f)
+LISPTREE *fpga_readfile(FILE *f, void *dia)
 {
 	char line[MAXLINE];
 	REGISTER char save, *pt, *ptend;
@@ -1452,7 +1450,7 @@ LISPTREE *fpga_readfile(FILE *f)
 		if ((fpga_lineno%50) == 0)
 		{
 			filepos = xtell(f);
-			DiaPercent(1, filepos*100/fpga_filesize);
+			DiaSetProgress(dia, filepos, fpga_filesize);
 		}
 
 		/* stop now if it is a comment */
@@ -2085,8 +2083,8 @@ BOOLEAN fpga_makeprimport(NODEPROTO *np, LISPTREE *lt, FPGAPORT *fp)
 		ttyputerr(_("Port position must be two atoms (line %ld)"), ltposition->lineno);
 		return(TRUE);
 	}
-	for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-		if (namesame(pp->protoname, (char *)ltname->paramvalue[0]) == 0)
+	pp = getportproto(np, (char *)ltname->paramvalue[0]);
+	if (pp != NOPORTPROTO)
 	{
 		ttyputerr(_("Duplicate port name (line %ld)"), lt->lineno);
 		return(TRUE);
@@ -2105,7 +2103,7 @@ BOOLEAN fpga_makeprimport(NODEPROTO *np, LISPTREE *lt, FPGAPORT *fp)
 	newpp->network = (NETWORK *)emalloc(sizeof (NETWORK), fpga_tech->cluster);
 	if (newpp->network == 0) return(TRUE);
 	newpp->network->namecount = 0;
-	newpp->network->signals = 1;
+	newpp->network->buswidth = 1;
 	newpp->network->nextnetwork  = NONETWORK;
 	newpp->network->prevnetwork = NONETWORK;
 	newpp->network->networklist = (NETWORK **)NONETWORK;
@@ -3032,10 +3030,7 @@ BOOLEAN fpga_makeblocknet(NODEPROTO *facet, LISPTREE *lt)
 					}
 
 					/* find port */
-					for(pp = facet->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-					{
-						if (namesame(pp->protoname, (char *)scanlt->paramvalue[pos+1]) == 0) break;
-					}
+					pp = getportproto(facet, (char *)scanlt->paramvalue[pos+1]);
 					if (pp == NOPORTPROTO)
 					{
 						ttyputerr(_("Cannot find port '%s' in block net segment (line %ld)"),
@@ -3067,5 +3062,3 @@ BOOLEAN fpga_makeblocknet(NODEPROTO *facet, LISPTREE *lt)
 	}
 	return(FALSE);
 }
-
-#endif  /* TECFPGA - at top */
