@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: tecgem.c
@@ -29,8 +30,6 @@
  */
 
 #include "config.h"
-#if TECGEM
-
 #include "global.h"
 #include "egraphics.h"
 #include "tech.h"
@@ -605,5 +604,3 @@ void gem_addzigzagbody(POLYGON *poly, INTBIG angle, INTBIG x1, INTBIG y1,
 	poly->xv[c] = x2;   poly->yv[c] = y2;   c++;
 	poly->count = c;
 }
-
-#endif  /* TECGEM - at top */
