@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: tecgen.c
@@ -46,7 +47,7 @@ static GENPOLYLOOP gen_oneprocpolyloop;
 
 TECHNOLOGY *gen_tech;
 NODEPROTO  *gen_univpinprim, *gen_invispinprim, *gen_unroutedpinprim, *gen_facetcenterprim,
-	*gen_portprim, *gen_drcprim;
+	*gen_portprim, *gen_drcprim, *gen_essentialprim;
 ARCPROTO   *gen_universalarc, *gen_invisiblearc, *gen_unroutedarc;
 ARCPROTO  **gen_upconn = 0;
 
@@ -127,21 +128,23 @@ static INTBIG gen_pc_riu[] = {-1, GENAUNROUTED, GENAINVIS, GENAUNIV, -1};
 
 /******************** NODES ********************/
 
-#define NODEPROTOCOUNT     6
+#define NODEPROTOCOUNT     7
 #define NUNIV              1		/* universal pin */
 #define NINVIS             2		/* invisible pin */
 #define NUNROUTED          3		/* unrouted pin */
 #define NCENTER            4		/* facet center */
 #define NPORT              5		/* port declaration (for tech edit) */
 #define NDRC               6		/* drc ignore mask (for ECAD's DRC) */
+#define NESSENTIAL         7		/* essential bounds marker */
 
 static INTBIG gen_fullbox[]    = {LEFTEDGE, BOTEDGE, RIGHTEDGE, TOPEDGE};
 static INTBIG gen_disccenter[] = {CENTER,   CENTER,  RIGHTEDGE, CENTER};
 static INTBIG gen_in2box[]     = {LEFTIN2,  BOTIN2,  RIGHTIN2,  TOPIN2};
 static INTBIG gen_center_lc[]  = {LEFTEDGE, BOTEDGE, LEFTEDGE,  BOTEDGE};
 static INTBIG gen_portabox[]   = {CENTER,   CENTER,  CENTER,    CENTER,
-								 CENTER,   CENTER};
-
+								  CENTER,   CENTER};
+static INTBIG gen_essentiall[] = {CENTERL1, CENTER,  CENTER,    CENTER,
+								  CENTER,   CENTERD1};
 /* Universal pin */
 static TECH_PORTS gen_u_p[] = {						/* ports */
 	{(INTBIG *)0, "univ", NOPORTPROTO, (180<<PORTARANGESH),
@@ -228,11 +231,25 @@ static TECH_NODES gen_d = {
 	(NPNODE<<NFUNCTIONSH)|HOLDSTRACE,				/* userbits */
 	POLYGONAL,0,0,0,0,0,0,0,0};						/* characteristics */
 
+/* Essential Bounds marker */
+static TECH_PORTS gen_e_p[] = {{					/* ports */
+	gen_pc_iu, "center", NOPORTPROTO, (180<<PORTARANGESH),
+		LEFTEDGE, BOTEDGE, LEFTEDGE, BOTEDGE}};
+static TECH_POLYGON gen_e_l[] = {					/* layers */
+	{LGLYPH, 0, 3, OPENED, POINTS, gen_essentiall}};
+static TECH_NODES gen_e = {
+	"Essential-Bounds",NESSENTIAL,NONODEPROTO,		/* name */
+	K0,K0,											/* size */
+	1,gen_e_p,										/* ports */
+	1,gen_e_l,										/* layers */
+	(NPART<<NFUNCTIONSH),							/* userbits */
+	0,0,0,0,0,0,0,0,0};								/* characteristics */
+
 TECH_NODES *gen_nodeprotos[NODEPROTOCOUNT+1] = {&gen_u, &gen_i,
-	&gen_r, &gen_c, &gen_p, &gen_d, ((TECH_NODES *)-1)};
+	&gen_r, &gen_c, &gen_p, &gen_d, &gen_e, ((TECH_NODES *)-1)};
 
 static INTBIG gen_node_widoff[NODEPROTOCOUNT*4] = {0,0,0,0, 0,0,0,0, 0,0,0,0,
-	0,0,0,0, K2,K2,K2,K2, 0,0,0,0};
+	0,0,0,0, K2,K2,K2,K2, 0,0,0,0, 0,0,0,0};
 
 /******************** VARIABLE AGGREGATION ********************/
 
@@ -277,6 +294,7 @@ BOOLEAN gen_initprocess(TECHNOLOGY *tech, INTBIG pass)
 			gen_facetcenterprim = getnodeproto("Generic:Facet-Center");
 			gen_portprim = getnodeproto("Generic:Port");
 			gen_drcprim = getnodeproto("Generic:DRC-Node");
+			gen_essentialprim = getnodeproto("Generic:Essential-Bounds");
 
 			gen_universalarc = getarcproto("Generic:Universal");
 			gen_invisiblearc = getarcproto("Generic:Invisible");
@@ -563,5 +581,5 @@ void gen_shapeportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans,
 	REGISTER INTBIG pindex;
 
 	pindex = ni->proto->primindex;
-	tech_fillportpoly(ni, pp, poly, trans, gen_nodeprotos[pindex-1], OPENED);
+	tech_fillportpoly(ni, pp, poly, trans, gen_nodeprotos[pindex-1], OPENED, lambdaofnode(ni));
 }
