@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: tecmocmos.c
@@ -30,8 +31,6 @@
  */
 
 #include "config.h"
-#if TECMOCMOSOLD
-
 #include "global.h"
 #include "egraphics.h"
 #include "tech.h"
@@ -1965,5 +1964,3 @@ INTBIG mocmosold_allarcpolys(ARCINST *ai, POLYLIST *plist, WINDOWPART *win)
 	}
 	return(tot);
 }
-
-#endif  /* TECMOCOSOLD - at top */
