@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: tecnmos.c
@@ -29,8 +30,6 @@
  */
 
 #include "config.h"
-#if TECNMOS
-
 #include "global.h"
 #include "egraphics.h"
 #include "tech.h"
@@ -1075,5 +1074,3 @@ BOOLEAN nmos_initprocess(TECHNOLOGY *tech, INTBIG pass)
 {
 	return(FALSE);
 }
-
-#endif  /* TECNMOS - at top */
