@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: tecpcb.c
@@ -29,8 +30,6 @@
  */
 
 #include "config.h"
-#if TECPCB
-
 #include "global.h"
 #include "egraphics.h"
 #include "tech.h"
@@ -1073,5 +1072,3 @@ BOOLEAN pcb_initprocess(TECHNOLOGY *tech, INTBIG pass)
 {
 	return(FALSE);
 }
-
-#endif  /* TECPCB - at top */
