@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: tecrcmos.c
@@ -30,8 +31,6 @@
  */
 
 #include "config.h"
-#if TECRCMOS
-
 #include "global.h"
 #include "egraphics.h"
 #include "tech.h"
@@ -1060,5 +1059,3 @@ INTBIG rcmos_allarcpolys(ARCINST *ai, POLYLIST *plist, WINDOWPART *win)
 	}
 	return(tot);
 }
-
-#endif  /* TECRCMOS - at top */
