@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: tecschem.c
@@ -50,8 +51,9 @@
 #include "tecschem.h"
 #include "efunction.h"
 #include "usr.h"
+#include "network.h"
 
-/* #define SCALABLEGATES 1 */   /* uncomment for experimental scalable gate code */
+#define SCALABLEGATES 1   /* uncomment for experimental scalable gate code */
 
 /* twentieths of a unit fractions */
 #define D1	(WHOLE/20)		/* 0.05 */
@@ -260,11 +262,6 @@ static PORTPROTO *sch_anddiffports, *sch_ordiffports, *sch_xordiffports;
 INTBIG            sch_wirepinsizex;		/* X size if wire-pin primitives */
 INTBIG            sch_wirepinsizey;		/* Y size if wire-pin primitives */
 
-#ifdef SCALABLEGATES
-void   sch_fillpoly(POLYGON *poly, TECH_POLYGON *lay, NODEINST *ni, INTBIG lambda,
-		INTBIG sty);
-INTBIG sch_getrange(INTBIG low, INTBIG high, INTBIG mul,INTBIG sum, INTBIG lambda, float scale);
-#endif
 INTBIG sch_intnodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win, POLYLOOP *pl, SCHPOLYLOOP *schpl);
 void   sch_intshapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl, SCHPOLYLOOP *schpl);
 INTBIG sch_intarcpolys(ARCINST *ai, WINDOWPART *win, POLYLOOP *pl, SCHPOLYLOOP *schpl);
@@ -1192,7 +1189,7 @@ static TECH_NODES sch_globalsig = {
 
 TECH_NODES *sch_nodeprotos[NODEPROTOCOUNT+1] = {
 	&sch_wirepin,  &sch_buspin,   &sch_wirecon,					/* pins */
-	&sch_buf,      &sch_and,      &sch_or,       &sch_xor,		/* gates */
+	&sch_buf,      &sch_and,      &sch_or,       &sch_xor,		/* gates: buffer, and, or, xor */
 	&sch_ff,       &sch_mux,									/* flipflop, mux */
 	&sch_bbox,     &sch_switch,									/* box/switch */
 	&sch_offpage,												/* offpage */
@@ -1207,7 +1204,7 @@ TECH_NODES *sch_nodeprotos[NODEPROTOCOUNT+1] = {
 
 static INTBIG sch_node_widoff[NODEPROTOCOUNT*4] = {
 	  0, 0, 0, 0,    0, 0, 0, 0,    0, 0, 0, 0,					/* pins */
-	  0,K1, 0, 0,    0,H0, 0, 0,   K1,H0, 0, 0,    0,H0, 0, 0,	/* gates */
+	  0,K1, 0, 0,    0,H0, 0, 0,   K1,H0, 0, 0,    0,H0, 0, 0,	/* gates: buffer, and, or, xor */
 	  0, 0, 0, 0,    0, 0, 0, 0,								/* flipflop, mux */
 	  0, 0, 0, 0,    0, 0, 0, 0,								/* box/switch */
 	  0, 0, 0, 0,												/* offpage */
@@ -1257,56 +1254,70 @@ TECH_VARIABLES sch_variables[] =
 
 BOOLEAN sch_initprocess(TECHNOLOGY *tech, INTBIG pass)
 {
-	/* initialize the technology variable */
-	if (pass == 0) sch_tech = tech; else
-		if (pass == 1)
+	switch (pass)
 	{
-		sch_wirepinprim = getnodeproto("schematic:Wire_Pin");
-		sch_buspinprim = getnodeproto("schematic:Bus_Pin");
-		sch_wireconprim = getnodeproto("schematic:Wire_Con");
-		sch_bufprim = getnodeproto("schematic:Buffer");
-		sch_andprim = getnodeproto("schematic:And");
-		sch_orprim = getnodeproto("schematic:Or");
-		sch_xorprim = getnodeproto("schematic:Xor");
-		sch_ffprim = getnodeproto("schematic:Flip-Flop");
-		sch_muxprim = getnodeproto("schematic:Mux");
-		sch_bboxprim = getnodeproto("schematic:Bbox");
-		sch_switchprim = getnodeproto("schematic:Switch");
-		sch_offpageprim = getnodeproto("schematic:Off-Page");
-		sch_pwrprim = getnodeproto("schematic:Power");
-		sch_gndprim = getnodeproto("schematic:Ground");
-		sch_sourceprim = getnodeproto("schematic:Source");
-		sch_transistorprim = getnodeproto("schematic:Transistor");
-		sch_resistorprim = getnodeproto("schematic:Resistor");
-		sch_capacitorprim = getnodeproto("schematic:Capacitor");
-		sch_diodeprim = getnodeproto("schematic:Diode");
-		sch_inductorprim = getnodeproto("schematic:Inductor");
-		sch_meterprim = getnodeproto("schematic:Meter");
-		sch_wellprim = getnodeproto("schematic:Well");
-		sch_substrateprim = getnodeproto("schematic:Substrate");
-		sch_twoportprim = getnodeproto("schematic:Two-Port");
-		sch_transistor4prim = getnodeproto("schematic:4-Port-Transistor");
-		sch_globalprim = getnodeproto("schematic:Global-Signal");
+		case 0:
+			/* initialize the technology variable */
+			sch_tech = tech;
+			break;
 
-		sch_wirearc = getarcproto("schematic:wire");
-		sch_busarc = getarcproto("schematic:bus");
+		case 1:
+			/* cache pointers to the primitives */
+			sch_wirepinprim = getnodeproto("schematic:Wire_Pin");
+			sch_buspinprim = getnodeproto("schematic:Bus_Pin");
+			sch_wireconprim = getnodeproto("schematic:Wire_Con");
+			sch_bufprim = getnodeproto("schematic:Buffer");
+			sch_andprim = getnodeproto("schematic:And");
+			sch_orprim = getnodeproto("schematic:Or");
+			sch_xorprim = getnodeproto("schematic:Xor");
+			sch_ffprim = getnodeproto("schematic:Flip-Flop");
+			sch_muxprim = getnodeproto("schematic:Mux");
+			sch_bboxprim = getnodeproto("schematic:Bbox");
+			sch_switchprim = getnodeproto("schematic:Switch");
+			sch_offpageprim = getnodeproto("schematic:Off-Page");
+			sch_pwrprim = getnodeproto("schematic:Power");
+			sch_gndprim = getnodeproto("schematic:Ground");
+			sch_sourceprim = getnodeproto("schematic:Source");
+			sch_transistorprim = getnodeproto("schematic:Transistor");
+			sch_resistorprim = getnodeproto("schematic:Resistor");
+			sch_capacitorprim = getnodeproto("schematic:Capacitor");
+			sch_diodeprim = getnodeproto("schematic:Diode");
+			sch_inductorprim = getnodeproto("schematic:Inductor");
+			sch_meterprim = getnodeproto("schematic:Meter");
+			sch_wellprim = getnodeproto("schematic:Well");
+			sch_substrateprim = getnodeproto("schematic:Substrate");
+			sch_twoportprim = getnodeproto("schematic:Two-Port");
+			sch_transistor4prim = getnodeproto("schematic:4-Port-Transistor");
+			sch_globalprim = getnodeproto("schematic:Global-Signal");
 
-		sch_meterkey = makekey("SCHEM_meter_type");
-		sch_diodekey = makekey("SCHEM_diode");
-		sch_capacitancekey = makekey("SCHEM_capacitance");
-		sch_resistancekey = makekey("SCHEM_resistance");
-		sch_inductancekey = makekey("SCHEM_inductance");
-		sch_functionkey = makekey("SCHEM_function");
-		sch_spicemodelkey = makekey("SIM_spice_model");
-		sch_globalnamekey = makekey("SCHEM_global_name");
+			sch_wirearc = getarcproto("schematic:wire");
+			sch_busarc = getarcproto("schematic:bus");
 
-		/* differential ports are enabled */
-		sch_anddiffports = sch_ordiffports = sch_xordiffports = NOPORTPROTO;
+			sch_meterkey = makekey("SCHEM_meter_type");
+			sch_diodekey = makekey("SCHEM_diode");
+			sch_capacitancekey = makekey("SCHEM_capacitance");
+			sch_resistancekey = makekey("SCHEM_resistance");
+			sch_inductancekey = makekey("SCHEM_inductance");
+			sch_functionkey = makekey("SCHEM_function");
+			sch_spicemodelkey = makekey("SIM_spice_model");
+			sch_globalnamekey = makekey("SCHEM_global_name");
 
-		/* translate strings on schematic nodes */
-		sch_g_fftextclr[16] = _("CLR");
-		sch_wirepinsizex = sch_wirepinprim->highx - sch_wirepinprim->lowx;
-		sch_wirepinsizey = sch_wirepinprim->highy - sch_wirepinprim->lowy;
+			/* differential ports are enabled */
+			sch_anddiffports = sch_ordiffports = sch_xordiffports = NOPORTPROTO;
+
+			/* translate strings on schematic nodes */
+			sch_g_fftextclr[16] = _("CLR");
+			sch_wirepinsizex = sch_wirepinprim->highx - sch_wirepinprim->lowx;
+			sch_wirepinsizey = sch_wirepinprim->highy - sch_wirepinprim->lowy;
+			break;
+
+		case 2:
+			/* set the default transistor placement to be rotated */
+			nextchangequiet();
+			setvalkey((INTBIG)sch_transistorprim, VNODEPROTO, us_placement_angle_key, 900, VINTEGER|VDONTSAVE);
+			nextchangequiet();
+			setvalkey((INTBIG)sch_transistor4prim, VNODEPROTO, us_placement_angle_key, 900, VINTEGER|VDONTSAVE);
+			break;
 	}
 	return(FALSE);
 }
@@ -1382,6 +1393,29 @@ void sch_setmode(INTBIG count, char *par[])
 
 INTBIG sch_request(char *command, va_list ap)
 {
+	REGISTER PORTPROTO *pp;
+
+	if (namesame(command, "ignoring-resistor-topology") == 0)
+	{
+		pp = sch_resistorprim->firstportproto->nextportproto;
+		if ((pp->userbits&PORTNET) == 0) return(1);
+		return(0);
+	}
+	if (namesame(command, "ignore-resistor-topology") == 0)
+	{
+		pp = sch_resistorprim->firstportproto->nextportproto;
+		pp->userbits = (pp->userbits & ~PORTNET);
+		net_redoprim();
+		return(0);
+	}
+	if (namesame(command, "include-resistor-topology") == 0)
+	{
+		pp = sch_resistorprim->firstportproto->nextportproto;
+		pp->userbits = (pp->userbits & ~PORTNET) | (1 << PORTNETSH);
+		net_redoprim();
+		return(0);
+	}
+
 	if (namesame(command, "get-bubble-size") == 0)
 	{
 		return(sch_bubblediameter);
@@ -1433,7 +1467,7 @@ INTBIG sch_intnodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win, POLYL
 			/* if the next level up the hierarchy is visible, consider arcs connected there */
 			if (win != NOWINDOWPART && ni->firstportexpinst != NOPORTEXPINST)
 			{
-				db_gettraversalpath(&nilist, &depth);
+				db_gettraversalpath(ni->parent, &nilist, &depth);
 				if (depth == 1)
 				{
 					upni = nilist[0];
@@ -1665,7 +1699,7 @@ void sch_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 
 void sch_intshapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl, SCHPOLYLOOP *schpl)
 {
-	REGISTER INTBIG lambda;
+	REGISTER INTBIG lambda, width, height;
 	REGISTER VARIABLE *var;
 	REGISTER TECH_PORTS *tp;
 	REGISTER TECH_POLYGON *lay;
@@ -1677,12 +1711,34 @@ void sch_intshapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl,
 		return;
 	}
 
+	/* get the unit size (lambda) */
+	switch (ni->proto->primindex)
+	{
+#ifdef SCALABLEGATES
+		case NAND:
+			width = ni->highx - ni->lowx;
+			height = ni->highy - ni->lowy;
+			lambda = width / 8;
+			if (height < lambda * 6) lambda = height / 6;
+			break;
+		case NOR:
+		case NXOR:
+			width = ni->highx - ni->lowx;
+			height = ni->highy - ni->lowy;
+			lambda = width / 10;
+			if (height < lambda * 6) lambda = height / 6;
+			break;
+#endif
+		default:
+			lambda = lambdaofnode(ni);
+			break;
+	}
+
 	if (box >= schpl->extrasteinerpoint)
 	{
 		/* handle extra steiner points */
 		tp = schpl->extrasteinerport[box - schpl->extrasteinerpoint];
 		if (poly->limit < 2) (void)extendpolygon(poly, 2);
-		lambda = lambdaofnode(ni);
 		poly->xv[0] = (getrange(ni->lowx, ni->highx, tp->lowxmul, tp->lowxsum, lambda) +
 			getrange(ni->lowx, ni->highx, tp->highxmul, tp->highxsum, lambda)) / 2;
 		poly->yv[0] = (getrange(ni->lowy, ni->highy, tp->lowymul, tp->lowysum, lambda) +
@@ -1759,18 +1815,7 @@ void sch_intshapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly, POLYLOOP *pl,
 			break;
 	}
 
-	lambda = lambdaofnode(ni);
-#ifdef SCALABLEGATES
-	if (ni->proto->primindex == NAND || ni->proto->primindex == NOR ||
-		ni->proto->primindex == NXOR)
-	{
-		sch_fillpoly(poly, lay, ni, lambda, FILLED);
-	} else
-#endif
-	{
-		tech_fillpoly(poly, lay, ni, lambda, FILLED);
-	}
-
+	tech_fillpoly(poly, lay, ni, lambda, FILLED);
 	TDCLEAR(poly->textdescript);
 	TDSETSIZE(poly->textdescript, lay->portnum);
 	if (ni->parent == NONODEPROTO) poly->tech = sch_tech; else
@@ -1817,120 +1862,81 @@ INTBIG sch_allnodeEpolys(NODEINST *ni, POLYLIST *plist, WINDOWPART *win, BOOLEAN
 	return(0);
 }
 
-#ifdef SCALABLEGATES
-/* scalable version of polygon filling for AND, OR, and XOR gates */
-void sch_fillpoly(POLYGON *poly, TECH_POLYGON *lay, NODEINST *ni, INTBIG lambda,
-	INTBIG sty)
+void sch_nodesizeoffset(NODEINST *ni, INTBIG *lx, INTBIG *ly, INTBIG *hx, INTBIG *hy)
 {
-	REGISTER INTBIG i, lastpoint;
-	REGISTER INTBIG *pt, xm, xs, ym, ys;
-	INTBIG minlx, minhx, minly, minhy;
-	float scale;
+	REGISTER INTBIG index, width, height, unitsize;
 
-	/* one thing is constant */
-	poly->layer = lay->layernum;
-
-	/* normal description from the technology tables */
-	switch (lay->representation)
+	index = ni->proto->primindex;
+	switch (index)
 	{
-		case BOX:
-		lastpoint = 8;
-		if (lay->style == FILLEDRECT || lay->style == CLOSEDRECT)
-		{
-			if (poly->limit < 2) (void)extendpolygon(poly, 2);
-			subrange(ni->lowx, ni->highx, lay->points[0], lay->points[1],
-				lay->points[4], lay->points[5], &poly->xv[0], &poly->xv[1], lambda);
-			subrange(ni->lowy, ni->highy, lay->points[2], lay->points[3],
-				lay->points[6], lay->points[7], &poly->yv[0], &poly->yv[1], lambda);
-			if (lay->representation == MINBOX)
-			{
-				/* make sure the box is large enough */
-				lastpoint = 16;
-				subrange(ni->lowx, ni->highx, lay->points[8], lay->points[9],
-					lay->points[12], lay->points[13], &minlx, &minhx, lambda);
-				subrange(ni->lowy, ni->highy, lay->points[10], lay->points[11],
-					lay->points[14], lay->points[15], &minly, &minhy, lambda);
-				if (poly->xv[0] > minlx) poly->xv[0] = minlx;
-				if (poly->xv[1] < minhx) poly->xv[1] = minhx;
-				if (poly->yv[0] > minly) poly->yv[0] = minly;
-				if (poly->yv[1] < minhy) poly->yv[1] = minhy;
-			}
-		} else
-		{
-			if (poly->limit < 4) (void)extendpolygon(poly, 4);
-			subrange(ni->lowx, ni->highx, lay->points[0], lay->points[1],
-				lay->points[4], lay->points[5], &poly->xv[0], &poly->xv[2], lambda);
-			subrange(ni->lowy, ni->highy, lay->points[2], lay->points[3],
-				lay->points[6], lay->points[7], &poly->yv[0], &poly->yv[1], lambda);
-			if (lay->representation == MINBOX)
-			{
-				/* make sure the box is large enough */
-				lastpoint = 16;
-				subrange(ni->lowx, ni->highx, lay->points[8], lay->points[9],
-					lay->points[10], lay->points[13], &minlx, &minhx, lambda);
-				subrange(ni->lowy, ni->highy, lay->points[10], lay->points[11],
-					lay->points[14], lay->points[15], &minly, &minhy, lambda);
-				if (poly->xv[0] > minlx) poly->xv[0] = minlx;
-				if (poly->xv[2] < minhx) poly->xv[2] = minhx;
-				if (poly->yv[0] > minly) poly->yv[0] = minly;
-				if (poly->yv[1] < minhy) poly->yv[1] = minhy;
-			}
-			poly->xv[1] = poly->xv[0];   poly->xv[3] = poly->xv[2];
-			poly->yv[3] = poly->yv[0];   poly->yv[2] = poly->yv[1];
-		}
-		break;
-
-		case POINTS:
-		pt = &lay->points[0];
-		scale = (float)(ni->highx - ni->lowx);
-		scale /= (ni->proto->highx - ni->proto->lowx);
-		if (poly->limit < lay->count) (void)extendpolygon(poly, lay->count);
-		for(i=0; i<lay->count; i++)
-		{
-			xm = *pt++;   xs = *pt++;   ym = *pt++;   ys = *pt++;
-			poly->xv[i] = sch_getrange(ni->lowx,ni->highx, xm,xs, lambda, scale);
-			poly->yv[i] = sch_getrange(ni->lowy,ni->highy, ym,ys, lambda, 1.0);
-		}
-		lastpoint = lay->count*4;
-		break;
-	}
-	poly->count = lay->count;
-	poly->style = lay->style;
-	if (lay->style >= TEXTCENT && lay->style <= TEXTBOX)
-		poly->string = (char *)lay->points[lastpoint];
-}
-
-/*
- * routine to perform a range calculation (similar to "subrange") but on
- * only one value rather than two.  The extent of the range is from "low"
- * to "high" and the value of lambda is "lambda".  The routine returns
- * the value (low+high)/2 + (high-low)*mul/WHOLE + sum*lambda/WHOLE.
- */
-INTBIG sch_getrange(INTBIG low, INTBIG high, INTBIG mul,INTBIG sum, INTBIG lambda, float scale)
-{
-	REGISTER float center, range, offset;
-
-	center = (float)(low + high);   center /= 2.0;
-	range = (float)(high - low); 
-	offset = range*mul/WHOLE;
-	offset += sum*lambda/WHOLE;
-	offset *= scale;
-	return((INTBIG)(center + offset));
-}
+#ifdef SCALABLEGATES
+		case NAND:
+			width = ni->highx - ni->lowx;
+			height = ni->highy - ni->lowy;
+			unitsize = width / 8;
+			if (height < unitsize * 6) unitsize = height / 6;
+			*lx = 0;
+			*hx = unitsize/2;
+			*ly = *hy = 0;
+			break;
+		case NOR:
+			width = ni->highx - ni->lowx;
+			height = ni->highy - ni->lowy;
+			unitsize = width / 10;
+			if (height < unitsize * 6) unitsize = height / 6;
+			*lx = unitsize;
+			*hx = unitsize/2;
+			*ly = *hy = 0;
+			break;
+		case NXOR:
+			width = ni->highx - ni->lowx;
+			height = ni->highy - ni->lowy;
+			unitsize = width / 10;
+			if (height < unitsize * 6) unitsize = height / 6;
+			*lx = 0;
+			*hx = unitsize/2;
+			*ly = *hy = 0;
+			break;
 #endif
+		default:
+			tech_nodeprotosizeoffset(ni->proto, lx, ly, hx, hy, lambdaofnode(ni));
+			break;
+	}
+}
 
 void sch_shapeportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans,
 	BOOLEAN purpose)
 {
-	REGISTER INTBIG pindex, i, e, total, besti;
-	REGISTER INTBIG xposition, yposition, x, y, lambda, wantx, wanty, bestdist,
-		bestx, besty, dist;
+	REGISTER INTBIG pindex, i, e, total, besti, xposition, yposition, x, y, lambda,
+		wantx, wanty, bestdist, bestx, besty, dist, width, height;
 	REGISTER PORTARCINST *pi;
 	REGISTER ARCINST *ai;
 	REGISTER WINDOWPART *w;
 
 	pindex = ni->proto->primindex;
 
+	switch (ni->proto->primindex)
+	{
+#ifdef SCALABLEGATES
+		case NAND:
+			width = ni->highx - ni->lowx;
+			height = ni->highy - ni->lowy;
+			lambda = width / 8;
+			if (height < lambda * 6) lambda = height / 6;
+			break;
+		case NOR:
+		case NXOR:
+			width = ni->highx - ni->lowx;
+			height = ni->highy - ni->lowy;
+			lambda = width / 10;
+			if (height < lambda * 6) lambda = height / 6;
+			break;
+#endif
+		default:
+			lambda = lambdaofnode(ni);
+			break;
+	}
+
 	/* special case for extendible primitives */
 	if (purpose && sch_nodeprotos[pindex-1]->portlist[0].addr == pp)
 	{
@@ -1938,7 +1944,6 @@ void sch_shapeportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans,
 		wantx = poly->xv[0];   wanty = poly->yv[0];
 		poly->count = 1;
 		poly->style = FILLED;
-		lambda = lambdaofnode(ni);
 		bestdist = MAXINTBIG;
 
 		/* schematic gates must keep connections discrete and separate */
@@ -2036,7 +2041,7 @@ void sch_shapeportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans,
 			return;
 		}
 	}
-	tech_fillportpoly(ni, pp, poly, trans, sch_nodeprotos[pindex-1], CLOSED);
+	tech_fillportpoly(ni, pp, poly, trans, sch_nodeprotos[pindex-1], CLOSED, lambda);
 }
 
 INTBIG sch_arcpolys(ARCINST *ai, WINDOWPART *win)
