@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: tectable.c
@@ -37,53 +38,41 @@ extern TECH_ARCS *art_arcprotos[];
 extern TECH_NODES *art_nodeprotos[];
 extern TECH_VARIABLES art_variables[];
 
-#if TECBICMOS
-  extern GRAPHICS *bicmos_layers[];
-  extern TECH_ARCS *bicmos_arcprotos[];
-  extern TECH_NODES *bicmos_nodeprotos[];
-  extern TECH_VARIABLES bicmos_variables[];
-  BOOLEAN bicmos_initprocess(TECHNOLOGY*, INTBIG);
-#endif
+extern GRAPHICS *bicmos_layers[];
+extern TECH_ARCS *bicmos_arcprotos[];
+extern TECH_NODES *bicmos_nodeprotos[];
+extern TECH_VARIABLES bicmos_variables[];
+BOOLEAN bicmos_initprocess(TECHNOLOGY*, INTBIG);
 
-#if TECBIPOLAR
-  extern GRAPHICS *bipolar_layers[];
-  extern TECH_ARCS *bipolar_arcprotos[];
-  extern TECH_NODES *bipolar_nodeprotos[];
-  extern TECH_VARIABLES bipolar_variables[];
-  BOOLEAN bipolar_initprocess(TECHNOLOGY*, INTBIG);
-#endif
+extern GRAPHICS *bipolar_layers[];
+extern TECH_ARCS *bipolar_arcprotos[];
+extern TECH_NODES *bipolar_nodeprotos[];
+extern TECH_VARIABLES bipolar_variables[];
+BOOLEAN bipolar_initprocess(TECHNOLOGY*, INTBIG);
 
-#if TECCMOS
-  extern GRAPHICS *cmos_layers[];
-  extern TECH_ARCS *cmos_arcprotos[];
-  extern TECH_NODES *cmos_nodeprotos[];
-  extern TECH_VARIABLES cmos_variables[];
-  BOOLEAN cmos_initprocess(TECHNOLOGY*, INTBIG);
-#endif
+extern GRAPHICS *cmos_layers[];
+extern TECH_ARCS *cmos_arcprotos[];
+extern TECH_NODES *cmos_nodeprotos[];
+extern TECH_VARIABLES cmos_variables[];
+BOOLEAN cmos_initprocess(TECHNOLOGY*, INTBIG);
 
-#if TECCMOSDODN
-# include "teccmosdodn.h"
-  extern GRAPHICS *dodcmosn_layers[];
-  extern TECH_ARCS *dodcmosn_arcprotos[];
-  extern TECH_NODES *dodcmosn_nodeprotos[];
-  extern TECH_VARIABLES dodcmosn_variables[];
-#endif
+#include "teccmosdodn.h"
+extern GRAPHICS *dodcmosn_layers[];
+extern TECH_ARCS *dodcmosn_arcprotos[];
+extern TECH_NODES *dodcmosn_nodeprotos[];
+extern TECH_VARIABLES dodcmosn_variables[];
 
-#if TECEFIDO
-  extern GRAPHICS *efido_layers[];
-  extern TECH_ARCS *efido_arcprotos[];
-  extern TECH_NODES *efido_nodeprotos[];
-  extern TECH_VARIABLES efido_variables[];
-  BOOLEAN efido_initprocess(TECHNOLOGY*, INTBIG);
-#endif
+extern GRAPHICS *efido_layers[];
+extern TECH_ARCS *efido_arcprotos[];
+extern TECH_NODES *efido_nodeprotos[];
+extern TECH_VARIABLES efido_variables[];
+BOOLEAN efido_initprocess(TECHNOLOGY*, INTBIG);
 
-#if TECGEM
-# include "tecgem.h"
-  extern GRAPHICS *gem_layers[];
-  extern TECH_ARCS *gem_arcprotos[];
-  extern TECH_NODES *gem_nodeprotos[];
-  extern TECH_VARIABLES gem_variables[];
-#endif
+#include "tecgem.h"
+extern GRAPHICS *gem_layers[];
+extern TECH_ARCS *gem_arcprotos[];
+extern TECH_NODES *gem_nodeprotos[];
+extern TECH_VARIABLES gem_variables[];
 
 #include "tecgen.h"
 extern GRAPHICS *gen_layers[];
@@ -91,64 +80,44 @@ extern TECH_ARCS *gen_arcprotos[];
 extern TECH_NODES *gen_nodeprotos[];
 extern TECH_VARIABLES gen_variables[];
 
-#if TECMOCMOS
-# include "tecmocmos.h"
-  extern COMCOMP mocmos_parse;
-  extern GRAPHICS *mocmos_layers[];
-  extern TECH_ARCS *mocmos_arcprotos[];
-  extern TECH_NODES *mocmos_nodeprotos[];
-  extern TECH_VARIABLES mocmos_variables[];
-#endif
+#include "tecmocmos.h"
+extern COMCOMP mocmos_parse;
+extern GRAPHICS *mocmos_layers[];
+extern TECH_ARCS *mocmos_arcprotos[];
+extern TECH_NODES *mocmos_nodeprotos[];
+extern TECH_VARIABLES mocmos_variables[];
 
-#if TECMOCMOSOLD
-# include "tecmocmosold.h"
-  extern COMCOMP mocmosold_parse;
-  extern GRAPHICS *mocmosold_layers[];
-  extern TECH_ARCS *mocmosold_arcprotos[];
-  extern TECH_NODES *mocmosold_nodeprotos[];
-  extern TECH_VARIABLES mocmosold_variables[];
-#endif
+#include "tecmocmosold.h"
+extern COMCOMP mocmosold_parse;
+extern GRAPHICS *mocmosold_layers[];
+extern TECH_ARCS *mocmosold_arcprotos[];
+extern TECH_NODES *mocmosold_nodeprotos[];
+extern TECH_VARIABLES mocmosold_variables[];
 
-#if TECMOCMOSSUB
-# include "tecmocmossub.h"
-  extern COMCOMP mocmossub_parse;
-  extern GRAPHICS *mocmossub_layers[];
-  extern TECH_ARCS *mocmossub_arcprotos[];
-  extern TECH_NODES *mocmossub_nodeprotos[];
-  extern TECH_VARIABLES mocmossub_variables[];
-#endif
+#include "tecmocmossub.h"
+extern COMCOMP mocmossub_parse;
+extern GRAPHICS *mocmossub_layers[];
+extern TECH_ARCS *mocmossub_arcprotos[];
+extern TECH_NODES *mocmossub_nodeprotos[];
+extern TECH_VARIABLES mocmossub_variables[];
 
-#if TECMOCMOS2
-  extern GRAPHICS *mocmos2_layers[];
-  extern TECH_ARCS *mocmos2_arcprotos[];
-  extern TECH_NODES *mocmos2_nodeprotos[];
-  extern TECH_VARIABLES mocmos2_variables[];
-  BOOLEAN mocmos2_initprocess(TECHNOLOGY*, INTBIG);
-#endif
+extern GRAPHICS *nmos_layers[];
+extern TECH_ARCS *nmos_arcprotos[];
+extern TECH_NODES *nmos_nodeprotos[];
+extern TECH_VARIABLES nmos_variables[];
+BOOLEAN nmos_initprocess(TECHNOLOGY*, INTBIG);
 
-#if TECNMOS
-  extern GRAPHICS *nmos_layers[];
-  extern TECH_ARCS *nmos_arcprotos[];
-  extern TECH_NODES *nmos_nodeprotos[];
-  extern TECH_VARIABLES nmos_variables[];
-  BOOLEAN nmos_initprocess(TECHNOLOGY*, INTBIG);
-#endif
+extern GRAPHICS *pcb_layers[];
+extern TECH_ARCS *pcb_arcprotos[];
+extern TECH_NODES *pcb_nodeprotos[];
+extern TECH_VARIABLES pcb_variables[];
+BOOLEAN pcb_initprocess(TECHNOLOGY*, INTBIG);
 
-#if TECPCB
-  extern GRAPHICS *pcb_layers[];
-  extern TECH_ARCS *pcb_arcprotos[];
-  extern TECH_NODES *pcb_nodeprotos[];
-  extern TECH_VARIABLES pcb_variables[];
-  BOOLEAN pcb_initprocess(TECHNOLOGY*, INTBIG);
-#endif
-
-#if TECRCMOS
-# include "tecrcmos.h"
-  extern GRAPHICS *rcmos_layers[];
-  extern TECH_ARCS *rcmos_arcprotos[];
-  extern TECH_NODES *rcmos_nodeprotos[];
-  extern TECH_VARIABLES rcmos_variables[];
-#endif
+#include "tecrcmos.h"
+extern GRAPHICS *rcmos_layers[];
+extern TECH_ARCS *rcmos_arcprotos[];
+extern TECH_NODES *rcmos_nodeprotos[];
+extern TECH_VARIABLES rcmos_variables[];
 
 #include "tecschem.h"
 extern COMCOMP sch_parse;
@@ -157,14 +126,12 @@ extern TECH_ARCS *sch_arcprotos[];
 extern TECH_NODES *sch_nodeprotos[];
 extern TECH_VARIABLES sch_variables[];
 
-#if TECFPGA
-# include "tecfpga.h"
-  extern COMCOMP fpga_parse;
-  extern GRAPHICS *fpga_layers[];
-  extern TECH_ARCS *fpga_arcprotos[];
-  extern TECH_NODES *fpga_nodeprotos[];
-  extern TECH_VARIABLES fpga_variables[];
-#endif
+#include "tecfpga.h"
+extern COMCOMP fpga_parse;
+extern GRAPHICS *fpga_layers[];
+extern TECH_ARCS *fpga_arcprotos[];
+extern TECH_NODES *fpga_nodeprotos[];
+extern TECH_VARIABLES fpga_variables[];
 
 /*
  * the first entry in this table MUST BE THE GENERIC TECHNOLOGY!
@@ -181,7 +148,6 @@ TECHNOLOGY el_technologylist[] =
 	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONSTANDARD|STATICTECHNOLOGY, 0, 0},								/* miscellaneous */
 
-#if TECNMOS
 	/* NMOS */
 	{"nmos", 0, 4000, NONODEPROTO,NOARCPROTO,NOVARIABLE,0,NOCOMCOMP,NOCLUSTER,		/* info */
 	N_("n-channel MOS (from Mead & Conway)"),										/* description */
@@ -191,9 +157,7 @@ TECHNOLOGY el_technologylist[] =
 	0,																				/* port routine */
 	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
-#endif
 
-#if TECCMOS
 	/* CMOS */
 	{"cmos", 0, 4000, NONODEPROTO,NOARCPROTO,NOVARIABLE,0,NOCOMCOMP,NOCLUSTER,		/* info */
 	N_("Complementary MOS (old, N-Well, from Griswold)"),							/* description */
@@ -203,9 +167,7 @@ TECHNOLOGY el_technologylist[] =
 	0,																				/* port routine */
 	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
-#endif
 
-#if TECMOCMOS
 	/* MOSIS CMOS */
 	{"mocmos", 0, 400,NONODEPROTO,NOARCPROTO,NOVARIABLE,0,&mocmos_parse,NOCLUSTER,	/* info */
 	N_("Complementary MOS (from MOSIS, 2-6 metals [4], 1-2 polys [2], flex rules [submicron])"),	/* description */
@@ -215,9 +177,7 @@ TECHNOLOGY el_technologylist[] =
 	0,																				/* port routine */
 	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
-#endif
 
-#if TECMOCMOSOLD
 	/* Old MOSIS CMOS */
 	{"mocmosold", 0, 2000,NONODEPROTO,NOARCPROTO,NOVARIABLE,0,&mocmosold_parse,NOCLUSTER,	/* info */
 	N_("Complementary MOS (old, from MOSIS, P-Well, double metal)"),				/* description */
@@ -227,9 +187,7 @@ TECHNOLOGY el_technologylist[] =
 	0,																				/* port routine */
 	mocmosold_arcpolys, mocmosold_shapearcpoly, mocmosold_allarcpolys, 0,			/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
-#endif
 
-#if TECMOCMOSSUB
 	/* MOSIS CMOS Submicron */
 	{"mocmossub", 0, 400,NONODEPROTO,NOARCPROTO,NOVARIABLE,0,&mocmossub_parse,NOCLUSTER,/* info */
 	N_("Complementary MOS (old, from MOSIS, Submicron, 2-6 metals [4], double poly)"),	/* description */
@@ -239,21 +197,7 @@ TECHNOLOGY el_technologylist[] =
 	0,																				/* port routine */
 	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
-#endif
 
-#if TECMOCMOS2
-	/* MOSIS CMOS, 2 micron (obsolete) */
-	{"mocmos2", 0, 2000, NONODEPROTO,NOARCPROTO,NOVARIABLE,0,NOCOMCOMP,NOCLUSTER,	/* info */
-	N_("Complementary MOS (MOSIS, Orbit, P-Well, double metal, double poly)"),		/* description */
-	0, mocmos2_layers, 0, mocmos2_arcprotos, 0, mocmos2_nodeprotos, mocmos2_variables,	/* tables */
-	mocmos2_initprocess, 0, 0, 0,													/* control routines */
-	0, 0, 0, 0, 0, 0, 0,															/* node routines */
-	0,																				/* port routine */
-	0, 0, 0, 0,																		/* arc routines */
-	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
-#endif
-
-#if TECBICMOS
 	/* MOSIS BiCMOS */
 	{"bicmos", 0, 2000, NONODEPROTO,NOARCPROTO,NOVARIABLE,0,NOCOMCOMP,NOCLUSTER,	/* info */
 	N_("Bipolar/CMOS (from MOSIS, N-Well, SCE Rules)"),								/* description */
@@ -263,9 +207,7 @@ TECHNOLOGY el_technologylist[] =
 	0,																				/* port routine */
 	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
-#endif
 
-#if TECRCMOS
 	/* Round MOSIS CMOS */
 	{"rcmos", 0, 2000, NONODEPROTO,NOARCPROTO,NOVARIABLE,0,NOCOMCOMP,NOCLUSTER,		/* info */
 	N_("Complementary MOS (round, from MOSIS, P-Well, double metal)"),				/* description */
@@ -275,9 +217,7 @@ TECHNOLOGY el_technologylist[] =
 	0,																				/* port routine */
 	rcmos_arcpolys, rcmos_shapearcpoly, rcmos_allarcpolys, 0,						/* arc routines */
 	NOTECHNOLOGY, NONSTANDARD|NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},				/* miscellaneous */
-#endif
 
-#if TECCMOSDODN
 	/* DOD CMOS (from Gary Lambert) */
 	{"cmosdodn", 0, 2000, NONODEPROTO,NOARCPROTO,NOVARIABLE,0,NOCOMCOMP,NOCLUSTER,	/* info */
 	N_("Complementary MOS (from DOD, P-Well, double metal"),						/* description */
@@ -287,9 +227,7 @@ TECHNOLOGY el_technologylist[] =
 	0,																				/* port routine */
 	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
-#endif
 
-#if TECBIPOLAR
 	/* bipolar */
 	{"bipolar", 0, 4000, NONODEPROTO,NOARCPROTO,NOVARIABLE,0,NOCOMCOMP,NOCLUSTER,	/* info */
 	N_("Bipolar (self-aligned, single poly)"),										/* description */
@@ -299,19 +237,17 @@ TECHNOLOGY el_technologylist[] =
 	0,																				/* port routine */
 	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
-#endif
 
 	/* Schematic capture */
 	{"schematic", 0, 4000, NONODEPROTO,NOARCPROTO,NOVARIABLE,0,&sch_parse,NOCLUSTER,/* info */
 	N_("Schematic Capture"),														/* description */
 	0, sch_layers, 0, sch_arcprotos, 0, sch_nodeprotos, sch_variables,				/* tables */
 	sch_initprocess, sch_termprocess, sch_setmode, sch_request,						/* control routines */
-	sch_nodepolys, sch_nodeEpolys, sch_shapenodepoly, sch_shapeEnodepoly, sch_allnodepolys, sch_allnodeEpolys, 0,	/* node routines */
+	sch_nodepolys, sch_nodeEpolys, sch_shapenodepoly, sch_shapeEnodepoly, sch_allnodepolys, sch_allnodeEpolys, sch_nodesizeoffset,	/* node routines */
 	sch_shapeportpoly,																/* port routine */
 	sch_arcpolys, sch_shapearcpoly, sch_allarcpolys, 0,								/* arc routines */
 	NOTECHNOLOGY, NONSTANDARD|STATICTECHNOLOGY, 0, 0},								/* miscellaneous */
 
-#if TECFPGA
 	/* FPGA */
 	{"fpga", 0, 2000, NONODEPROTO,NOARCPROTO,NOVARIABLE,0,&fpga_parse,NOCLUSTER,	/* info */
 	N_("FPGA Building-Blocks"),														/* description */
@@ -321,9 +257,7 @@ TECHNOLOGY el_technologylist[] =
 	fpga_shapeportpoly,																/* port routine */
 	fpga_arcpolys, fpga_shapearcpoly, fpga_allarcpolys, 0,							/* arc routines */
 	NOTECHNOLOGY, NONSTANDARD|STATICTECHNOLOGY|NOPRIMTECHNOLOGY, 0, 0},				/* miscellaneous */
-#endif
 
-#if TECPCB
 	/* Printed Circuit Board */
 	{"pcb", 0, 2540000, NONODEPROTO,NOARCPROTO,NOVARIABLE,0,NOCOMCOMP,NOCLUSTER,	/* info */
 	N_("Printed Circuit Board (eight-layer)"),										/* description */
@@ -333,7 +267,6 @@ TECHNOLOGY el_technologylist[] =
 	0,																				/* port routine */
 	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
-#endif
 
 	/* Artwork */
 	{"artwork", 0, 4000, NONODEPROTO,NOARCPROTO,NOVARIABLE,0,&art_parse,NOCLUSTER,	/* info */
@@ -345,7 +278,6 @@ TECHNOLOGY el_technologylist[] =
 	art_arcpolys, art_shapearcpoly, art_allarcpolys, 0,								/* arc routines */
 	NOTECHNOLOGY, NONSTANDARD|NONELECTRICAL|NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},	/* miscellaneous */
 
-#if TECGEM
 	/* Gem Planning */
 	{"gem", 0, 2000, NONODEPROTO,NOARCPROTO,NOVARIABLE,0,NOCOMCOMP,NOCLUSTER,		/* info */
 	N_("Temporal Specification Facility (from Lansky)"),							/* description */
@@ -355,9 +287,7 @@ TECHNOLOGY el_technologylist[] =
 	0,																				/* port routine */
 	0, gem_shapearcpoly, gem_allarcpolys, 0,										/* arc routines */
 	NOTECHNOLOGY, NONSTANDARD|NONELECTRICAL|NODIRECTIONALARCS|NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},/* miscellaneous */
-#endif
 
-#if TECEFIDO
 	/* Digital Filter */
 	{"efido", 0, 20000, NONODEPROTO,NOARCPROTO,NOVARIABLE,0,NOCOMCOMP,NOCLUSTER,	/* info */
 	N_("Digital Filter Facility (from Kroeker)"),									/* description */
@@ -367,13 +297,12 @@ TECHNOLOGY el_technologylist[] =
 	0,																				/* port routine */
 	0, 0, 0, 0,																		/* arc routines */
 	NOTECHNOLOGY, NONEGATEDARCS|STATICTECHNOLOGY, 0, 0},							/* miscellaneous */
-#endif
 
 	/* termination */
 	{NULL, 0, 0, NULL, NULL, NULL, 0, NULL, NULL,									/* info */
 	NULL,																			/* description */
 	0, NULL, 0, NULL, 0, NULL, NULL,												/* tables */
-    0, 0, 0, 0,																		/* control routines */
+	0, 0, 0, 0,																		/* control routines */
 	0, 0, 0, 0, 0, 0, 0,															/* node routines */
 	0,																				/* port routine */
 	0, 0, 0, 0,																		/* arc routines */
