@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usrarc.c
@@ -37,19 +38,21 @@
 
 /* prototypes for local routines */
 static TECHNOLOGY *us_gettech(NODEINST*, PORTPROTO*);
-static ARCINST *us_directarcinst(NODEINST*, PORTPROTO*, ARCPROTO*, NODEINST*, PORTPROTO*, ARCPROTO*,
-	NODEPROTO*, INTBIG, INTBIG, INTBIG, INTBIG, NODEINST**, INTBIG, INTBIG*);
-static ARCINST *us_onebend(NODEINST*, PORTPROTO*, ARCPROTO*, NODEINST*, PORTPROTO*, ARCPROTO*,
-	NODEPROTO*, INTBIG, INTBIG, INTBIG, INTBIG, ARCINST**, NODEINST**, INTBIG*);
-static ARCINST *us_twobend(NODEINST*, PORTPROTO*, ARCPROTO*, NODEINST*, PORTPROTO*, ARCPROTO*,
-	NODEPROTO*, INTBIG, INTBIG, INTBIG, INTBIG, ARCINST**, ARCINST**, NODEINST**, NODEINST**, INTBIG*);
-static INTBIG us_portdistance(NODEINST*, PORTPROTO*, INTBIG, INTBIG, INTBIG*, INTBIG*, INTBIG, BOOLEAN);
-static BOOLEAN us_fitportangle(NODEINST*, PORTPROTO*, INTBIG, INTBIG);
-static INTBIG us_figurearcwidth(ARCPROTO *typ, INTBIG wid, NODEINST *ni1, PORTPROTO *pp1,
-	INTBIG ox1, INTBIG oy1, NODEINST *ni2, PORTPROTO *pp2, INTBIG ox2, INTBIG oy2);
-static NODEPROTO *us_findconnectingpin(ARCPROTO *ap, GEOM *geom, PORTPROTO *pp);
-static void us_getproperconsize(NODEPROTO *con, ARCPROTO *fromap, INTBIG fwid,
-	ARCPROTO *toap, INTBIG twid, INTBIG *pxs, INTBIG *pys);
+static ARCINST    *us_directarcinst(NODEINST*, PORTPROTO*, ARCPROTO*, NODEINST*, PORTPROTO*, ARCPROTO*,
+					NODEPROTO*, INTBIG, INTBIG, INTBIG, INTBIG, NODEINST**, INTBIG, INTBIG*, BOOLEAN);
+static ARCINST    *us_onebend(NODEINST*, PORTPROTO*, ARCPROTO*, NODEINST*, PORTPROTO*, ARCPROTO*,
+					NODEPROTO*, INTBIG, INTBIG, INTBIG, INTBIG, ARCINST**, NODEINST**, INTBIG*, BOOLEAN);
+static ARCINST    *us_twobend(NODEINST*, PORTPROTO*, ARCPROTO*, NODEINST*, PORTPROTO*, ARCPROTO*,
+					NODEPROTO*, INTBIG, INTBIG, INTBIG, INTBIG, ARCINST**, ARCINST**, NODEINST**, NODEINST**,
+					INTBIG*, BOOLEAN);
+static INTBIG      us_portdistance(NODEINST*, PORTPROTO*, INTBIG, INTBIG, INTBIG*, INTBIG*, INTBIG, BOOLEAN);
+static BOOLEAN     us_fitportangle(NODEINST*, PORTPROTO*, INTBIG, INTBIG);
+static INTBIG      us_figurearcwidth(ARCPROTO *typ, INTBIG wid, NODEINST *ni1, PORTPROTO *pp1,
+					INTBIG ox1, INTBIG oy1, NODEINST *ni2, PORTPROTO *pp2, INTBIG ox2, INTBIG oy2);
+static NODEPROTO  *us_findconnectingpin(ARCPROTO *ap, GEOM *geom, PORTPROTO *pp);
+static void        us_getproperconsize(NODEPROTO *con, ARCPROTO *fromap, INTBIG fwid,
+					ARCPROTO *toap, INTBIG twid, INTBIG *pxs, INTBIG *pys);
+static BOOLEAN     us_breakarcinsertnode(ARCINST *ai, NODEINST *ni, ARCINST **ai1, ARCINST **ai2);
 
 /*
  * routine to run an arcinst from "fromgeom" to "togeom".  The prefered
@@ -64,16 +67,17 @@ static void us_getproperconsize(NODEPROTO *con, ARCPROTO *fromap, INTBIG fwid,
  * addresses are returned in "alt1" and "alt2", if contacts or pins are created,
  * their addresses are returned in "con1" and "con2").  Otherwise, it issues an
  * error and returns NOARCINST.  The required angle increment for the
- * arcs is "ang" tenth-degrees (900 for manhattan geometry).
+ * arcs is "ang" tenth-degrees (900 for manhattan geometry).  The number of
+ * arcs created is reported if "report" is TRUE.
  */
 ARCINST *aconnect(GEOM *fromgeom, PORTPROTO *frompp, GEOM *togeom, PORTPROTO *topp,
 	ARCPROTO *typ, INTBIG prefx, INTBIG prefy,
 	ARCINST **alt1, ARCINST **alt2, NODEINST **con1, NODEINST **con2,
-	INTBIG ang, BOOLEAN nozigzag)
+	INTBIG ang, BOOLEAN nozigzag, BOOLEAN report)
 {
 	REGISTER NODEINST *fromnodeinst, *tonodeinst, *ni;
 	REGISTER NODEPROTO *con;
-	REGISTER ARCINST *ai, *newai;
+	REGISTER ARCINST *ai, *newai, *ai1, *ai2;
 	REGISTER PORTPROTO *fpt, *tpt, *pp;
 	REGISTER ARCPROTO *ap, *fap, *tap;
 	REGISTER TECHNOLOGY *tech, *tech1, *tech2;
@@ -127,19 +131,46 @@ ARCINST *aconnect(GEOM *fromgeom, PORTPROTO *frompp, GEOM *togeom, PORTPROTO *to
 		}
 	}
 
-	/* get actual nodes for each end of connection */
-	fromnodeinst = us_getnodeonarcinst(&fromgeom, &frompp, togeom, topp, prefx, prefy, 0);
-	if (fromnodeinst == NONODEINST)
+	/* if two arcs are selected and they cross, make only 1 contact */
+	fromnodeinst = NONODEINST;
+	if (!fromgeom->entryisnode && !togeom->entryisnode)
 	{
-		us_abortcommand(_("Cannot create splitting pin"));
-		return(NOARCINST);
+		ai1 = fromgeom->entryaddr.ai;
+		ai2 = togeom->entryaddr.ai;
+		if (segintersect(ai1->end[0].xpos, ai1->end[0].ypos, ai1->end[1].xpos, ai1->end[1].ypos,
+			ai2->end[0].xpos, ai2->end[0].ypos, ai2->end[1].xpos, ai2->end[1].ypos, &x, &y))
+		{
+			/* they intersect */
+			fromnodeinst = us_getnodeonarcinst(&fromgeom, &frompp, togeom, topp, x, y, 0);
+			if (fromnodeinst == NONODEINST)
+			{
+				us_abortcommand(_("Cannot create splitting pin"));
+				return(NOARCINST);
+			}
+			if (us_breakarcinsertnode(ai2, fromnodeinst, alt1, alt2)) return(NOARCINST);
+			ai = *alt2;
+			*alt2 = NOARCINST;
+			*con1 = fromnodeinst;
+			return(ai);
+		}
 	}
 
-	tonodeinst = us_getnodeonarcinst(&togeom, &topp, fromgeom, frompp, prefx, prefy, 0);
-	if (tonodeinst == NONODEINST)
+	/* get actual nodes for each end of connection */
+	if (fromnodeinst == NONODEINST)
 	{
-		us_abortcommand(_("Cannot create splitting pin"));
-		return(NOARCINST);
+		fromnodeinst = us_getnodeonarcinst(&fromgeom, &frompp, togeom, topp, prefx, prefy, 0);
+		if (fromnodeinst == NONODEINST)
+		{
+			us_abortcommand(_("Cannot create splitting pin"));
+			return(NOARCINST);
+		}
+
+		tonodeinst = us_getnodeonarcinst(&togeom, &topp, fromgeom, frompp, prefx, prefy, 0);
+		if (tonodeinst == NONODEINST)
+		{
+			us_abortcommand(_("Cannot create splitting pin"));
+			return(NOARCINST);
+		}
 	}
 
 	/* default to single port on one-port nodeinsts */
@@ -250,7 +281,7 @@ ARCINST *aconnect(GEOM *fromgeom, PORTPROTO *frompp, GEOM *togeom, PORTPROTO *to
 	if (usecontact != 0)
 	{
 		newai = us_makeconnection(fromnodeinst, frompp, fap, tonodeinst, topp, tap,
-			con, prefx, prefy, alt1, alt2, con1, con2, ang, nozigzag, 0);
+			con, prefx, prefy, alt1, alt2, con1, con2, ang, nozigzag, 0, report);
 		if (newai != NOARCINST) return(newai);
 	}
 
@@ -258,7 +289,7 @@ ARCINST *aconnect(GEOM *fromgeom, PORTPROTO *frompp, GEOM *togeom, PORTPROTO *to
 	if ((typ->userbits&CANCONNECT) != 0)
 	{
 		newai = us_makeconnection(fromnodeinst, frompp, typ, tonodeinst, topp, typ,
-			getpinproto(typ), prefx, prefy, alt1, alt2, con1, con2, ang, nozigzag, 0);
+			getpinproto(typ), prefx, prefy, alt1, alt2, con1, con2, ang, nozigzag, 0, report);
 		if (newai != NOARCINST) return(newai);
 	}
 
@@ -268,7 +299,7 @@ ARCINST *aconnect(GEOM *fromgeom, PORTPROTO *frompp, GEOM *togeom, PORTPROTO *to
 		if ((ap->userbits&CANCONNECT) == 0) continue;
 		if (ap == typ) continue;
 		newai = us_makeconnection(fromnodeinst, frompp, ap, tonodeinst, topp, ap,
-			getpinproto(ap), prefx, prefy, alt1, alt2, con1, con2, ang, nozigzag, 0);
+			getpinproto(ap), prefx, prefy, alt1, alt2, con1, con2, ang, nozigzag, 0, report);
 		if (newai != NOARCINST) return(newai);
 	}
 
@@ -284,7 +315,7 @@ ARCINST *aconnect(GEOM *fromgeom, PORTPROTO *frompp, GEOM *togeom, PORTPROTO *to
 			if ((ap->userbits&CANCONNECT) == 0) continue;
 			if (ap == typ) continue;
 			newai = us_makeconnection(fromnodeinst, frompp, ap, tonodeinst, topp, ap,
-				getpinproto(ap), prefx, prefy, alt1, alt2, con1, con2, ang, nozigzag, 0);
+				getpinproto(ap), prefx, prefy, alt1, alt2, con1, con2, ang, nozigzag, 0, report);
 			if (newai != NOARCINST) return(newai);
 		}
 	} else
@@ -296,7 +327,7 @@ ARCINST *aconnect(GEOM *fromgeom, PORTPROTO *frompp, GEOM *togeom, PORTPROTO *to
 		{
 			if ((ap->userbits&CANCONNECT) == 0) continue;
 			newai = us_makeconnection(fromnodeinst, frompp, ap, tonodeinst, topp, ap,
-				getpinproto(ap), prefx, prefy, alt1, alt2, con1, con2, ang, nozigzag, 0);
+				getpinproto(ap), prefx, prefy, alt1, alt2, con1, con2, ang, nozigzag, 0, report);
 			if (newai != NOARCINST) return(newai);
 		}
 	}
@@ -330,12 +361,13 @@ TECHNOLOGY *us_gettech(NODEINST *ni, PORTPROTO *pp)
  * returned in "con1" and "con2". The preferred arcinst runs closest to (prefx, prefy).
  * The most direct route will be traveled, but an additional nodeinst or two
  * may have to be created.  The angle increment for the arcs is "ang".
- * If "nozigzag" is nonzero, disallow 3-arc connections (only 1 or 2).
+ * If "nozigzag" is nonzero, disallow 3-arc connections (only 1 or 2).  The number of
+ * arcs created is reported if "report" is TRUE.
  */
 ARCINST *us_makeconnection(NODEINST *fromnodeinst, PORTPROTO *fromportproto, ARCPROTO *fromarcproto,
 	NODEINST *tonodeinst, PORTPROTO *toportproto, ARCPROTO *toarcproto, NODEPROTO *con, INTBIG prefx,
 	INTBIG prefy, ARCINST **alt1, ARCINST **alt2, NODEINST **con1, NODEINST **con2,
-	INTBIG ang, BOOLEAN nozigzag, INTBIG *fakecoords)
+	INTBIG ang, BOOLEAN nozigzag, INTBIG *fakecoords, BOOLEAN report)
 {
 	REGISTER PORTPROTO *fpt, *tpt;
 	REGISTER INTBIG bestdist, dist, fwid, twid, w;
@@ -385,7 +417,7 @@ ARCINST *us_makeconnection(NODEINST *fromnodeinst, PORTPROTO *fromportproto, ARC
 	*con1 = *con2 = NONODEINST;
 	newai = us_directarcinst(fromnodeinst, fromportproto, fromarcproto,
 		tonodeinst, toportproto, toarcproto, con,
-			prefx, prefy, fwid, twid, con1, ang, fakecoords);
+			prefx, prefy, fwid, twid, con1, ang, fakecoords, report);
 	if (newai != NOARCINST)
 	{
 		*alt1 = *alt2 = NOARCINST;
@@ -423,14 +455,14 @@ ARCINST *us_makeconnection(NODEINST *fromnodeinst, PORTPROTO *fromportproto, ARC
 		{
 			newai = us_twobend(fromnodeinst, fromportproto, fromarcproto, tonodeinst,
 				toportproto, toarcproto, con, prefx, prefy, fwid, twid,
-				alt1, alt2, con1, con2, fakecoords);
+				alt1, alt2, con1, con2, fakecoords, report);
 			if (newai != NOARCINST) return(newai);
 		}
 	}
 
 	/* next check for a one-bend connection */
 	newai = us_onebend(fromnodeinst, fromportproto, fromarcproto, tonodeinst, toportproto,
-		toarcproto, con, prefx, prefy, fwid, twid, alt1, con1, fakecoords);
+		toarcproto, con, prefx, prefy, fwid, twid, alt1, con1, fakecoords, report);
 	if (newai != NOARCINST)
 	{
 		*alt2 = NOARCINST;
@@ -439,7 +471,7 @@ ARCINST *us_makeconnection(NODEINST *fromnodeinst, PORTPROTO *fromportproto, ARC
 
 	/* finally check for any zig-zag connection */
 	newai = us_twobend(fromnodeinst, fromportproto, fromarcproto, tonodeinst, toportproto,
-		toarcproto, con, prefx, prefy, fwid, twid, alt1, alt2, con1, con2, fakecoords);
+		toarcproto, con, prefx, prefy, fwid, twid, alt1, alt2, con1, con2, fakecoords, report);
 	if (newai != NOARCINST) return(newai);
 
 	/* give up */
@@ -459,7 +491,8 @@ ARCINST *us_makeconnection(NODEINST *fromnodeinst, PORTPROTO *fromportproto, ARC
  */
 ARCINST *us_directarcinst(NODEINST *fromnodeinst, PORTPROTO *fromppt, ARCPROTO *fromarcproto,
 	NODEINST *tonodeinst, PORTPROTO *toppt, ARCPROTO *toarcproto, NODEPROTO *con,
-	INTBIG prefx, INTBIG prefy, INTBIG fwid, INTBIG twid, NODEINST **con1, INTBIG ang, INTBIG *fakecoords)
+	INTBIG prefx, INTBIG prefy, INTBIG fwid, INTBIG twid, NODEINST **con1, INTBIG ang,
+	INTBIG *fakecoords, BOOLEAN report)
 {
 	REGISTER PORTPROTO *fpt, *tpt, *fstart, *tstart;
 	REGISTER ARCINST *ai;
@@ -772,13 +805,15 @@ ARCINST *us_directarcinst(NODEINST *fromnodeinst, PORTPROTO *fromppt, ARCPROTO *
 				ai = us_runarcinst(ni, con->firstportproto, bestxi, bestyi, 
 					tonodeinst, toportproto, besttx, bestty, toarcproto, twid);
 				if (ai == NOARCINST) return(NOARCINST);
-				ttyputmsg(_("Created 1 %s and 1 %s arc"), fromarcproto->protoname, toarcproto->protoname);
+				if (report)
+					us_reportarcscreated(0, 1, fromarcproto->protoname, 1, toarcproto->protoname);
 			} else
 			{
 				ai = us_runarcinst(fromnodeinst, fromportproto, bestfx, bestfy, 
 					tonodeinst, toportproto, besttx, bestty, fromarcproto, wid);
 				if (ai == NOARCINST) return(NOARCINST);
-				ttyputmsg(_("Created 1 %s arc"), fromarcproto->protoname);
+				if (report)
+					us_reportarcscreated(0, 1, fromarcproto->protoname, 0, "");
 			}
 			return(ai);
 		} else
@@ -795,6 +830,50 @@ ARCINST *us_directarcinst(NODEINST *fromnodeinst, PORTPROTO *fromppt, ARCPROTO *
 	return(NOARCINST);
 }
 
+/*
+ * Routine to report that activity "who" created "numarcs1" arcs of type "type1" and "numarcs2"
+ * arcs of type "type2".  If "who" is zero, ignore that part of the message.  If "numarcs2" is zero,
+ * ignore that part of the message.  If "type1" is zero, ignore that part of the message.
+ * Also plays a sound appropriate to the number of arcs created.
+ */
+void us_reportarcscreated(char *who, INTBIG numarcs1, char *type1, INTBIG numarcs2, char *type2)
+{
+	REGISTER void *infstr;
+	REGISTER INTBIG totalcreated;
+
+	infstr = initinfstr();
+	if (who != 0) formatinfstr(infstr, "%s: ", who);
+	if (numarcs2 > 0)
+	{
+		formatinfstr(infstr, _("Created %ld %s and %ld %s %s"), numarcs1, type1, numarcs2, type2,
+			makeplural("arc", numarcs2));
+	} else
+	{
+		if (type1 == 0)
+		{
+			formatinfstr(infstr, _("Created %ld %s"), numarcs1, makeplural("arc", numarcs1));
+		} else
+		{
+			formatinfstr(infstr, _("Created %ld %s %s"), numarcs1, type1, makeplural("arc", numarcs1));
+		}
+	}
+	ttyputmsg("%s", returninfstr(infstr));
+	totalcreated = numarcs1 + numarcs2;
+	if (totalcreated == 1)
+	{
+		ttybeep(SOUNDCLICK, TRUE);
+	} else if (totalcreated == 2)
+	{
+		ttybeep(SOUNDCLICK, TRUE);
+		ttybeep(SOUNDCLICK, TRUE);
+	} else if (totalcreated >= 3)
+	{
+		ttybeep(SOUNDCLICK, TRUE);
+		ttybeep(SOUNDCLICK, TRUE);
+		ttybeep(SOUNDCLICK, TRUE);
+	}
+}
+
 /*
  * routine to check for a one-bend connection running from nodeinst
  * "fromnodeinst" to nodeinst "tonodeinst" on arcproto "typ" and create two
@@ -810,7 +889,8 @@ ARCINST *us_directarcinst(NODEINST *fromnodeinst, PORTPROTO *fromppt, ARCPROTO *
  */
 ARCINST *us_onebend(NODEINST *fromnodeinst, PORTPROTO *fromppt, ARCPROTO *fromarcproto,
 	NODEINST *tonodeinst, PORTPROTO *toppt, ARCPROTO *toarcproto, NODEPROTO *con,
-	INTBIG prefx, INTBIG prefy, INTBIG fwid, INTBIG twid, ARCINST **alt, NODEINST **con1, INTBIG *fakecoords)
+	INTBIG prefx, INTBIG prefy, INTBIG fwid, INTBIG twid, ARCINST **alt, NODEINST **con1,
+	INTBIG *fakecoords, BOOLEAN report)
 {
 	REGISTER NODEINST *ni;
 	REGISTER PORTPROTO *fpt, *tpt;
@@ -988,8 +1068,16 @@ ARCINST *us_onebend(NODEINST *fromnodeinst, PORTPROTO *fromppt, ARCPROTO *fromar
 			ai2 = us_runarcinst(ni, fpt, bestxi, bestyi, tonodeinst, toportproto,
 				besttx, bestty, toarcproto, twid);
 			if (ai2 == NOARCINST) return(NOARCINST);
-			if (fromarcproto == toarcproto) ttyputmsg(_("Created 2 %s arcs"), fromarcproto->protoname); else
-				ttyputmsg(_("Created 1 %s and 1 %s arc"), fromarcproto->protoname, toarcproto->protoname);
+			if (report)
+			{
+				if (fromarcproto == toarcproto)
+				{
+					us_reportarcscreated(0, 2, fromarcproto->protoname, 0, "");
+				} else
+				{
+					us_reportarcscreated(0, 1, fromarcproto->protoname, 1, toarcproto->protoname);
+				}
+			}
 			if (abs(bestfx-bestxi) + abs(bestfy-bestyi) >
 				abs(bestxi-besttx) + abs(bestyi-bestty))
 			{
@@ -1035,7 +1123,7 @@ ARCINST *us_twobend(NODEINST *fromnodeinst, PORTPROTO *fromppt, ARCPROTO *fromar
 	NODEINST *tonodeinst, PORTPROTO *toppt, ARCPROTO *toarcproto, NODEPROTO *con,
 	INTBIG prefx, INTBIG prefy, INTBIG fwid, INTBIG twid,
 	ARCINST **alt1, ARCINST **alt2, NODEINST **con1, NODEINST **con2,
-	INTBIG *fakecoords)
+	INTBIG *fakecoords, BOOLEAN report)
 {
 	REGISTER NODEINST *ni1, *ni2;
 	REGISTER NODEPROTO *fpin;
@@ -1181,13 +1269,15 @@ ARCINST *us_twobend(NODEINST *fromnodeinst, PORTPROTO *fromppt, ARCPROTO *fromar
 		*alt2 = us_runarcinst(ni2,con->firstportproto, xi2,yi2, tonodeinst,
 			toportproto, besttx, bestty, toarcproto, twid);
 		if (*alt2 == NOARCINST) return(NOARCINST);
-		if (fromarcproto == toarcproto)
+		if (report)
 		{
-			ttyputmsg(_("Created 3 %s arcs"), fromarcproto->protoname);
-		} else
-		{
-			ttyputmsg(_("Created 2 %s arcs and 1 %s arc"), fromarcproto->protoname,
-				toarcproto->protoname);
+			if (fromarcproto == toarcproto)
+			{
+				us_reportarcscreated(0, 3, fromarcproto->protoname, 0, "");
+			} else
+			{
+				us_reportarcscreated(0, 2, fromarcproto->protoname, 1, toarcproto->protoname);
+			}
 		}
 		return(ai);
 	}
@@ -1397,15 +1487,15 @@ INTBIG us_figurearcwidth(ARCPROTO *typ, INTBIG wid, NODEINST *ni1, PORTPROTO *pp
 NODEINST *us_getnodeonarcinst(GEOM **ipos, PORTPROTO **ipp, GEOM *othergeom,
 	PORTPROTO *otherport, INTBIG prefx, INTBIG prefy, INTBIG fake)
 {
-	REGISTER ARCINST *ai, *oar, *ar1, *ar2, *mainarc;
+	REGISTER ARCINST *ai, *oar;
+	ARCINST *ai1, *ai2;
 	REGISTER ARCPROTO *ap;
-	REGISTER NODEINST *ni, *fno, *tno;
-	REGISTER PORTPROTO *fpt, *tpt, *pt1;
-	REGISTER NODEPROTO *np, *pnt;
+	REGISTER NODEINST *ni;
+	REGISTER NODEPROTO *np;
 	REGISTER GEOM *geom;
 	static NODEINST node;
-	REGISTER INTBIG w, wid, owid, bits1, bits2, lx, hx, ly, hy, delta;
-	INTBIG fendx, fendy, tendx, tendy, pxs, pys;
+	REGISTER INTBIG w, wid, owid, lx, hx, ly, hy, delta;
+	INTBIG pxs, pys;
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get the actual geometry modules */
@@ -1445,7 +1535,7 @@ NODEINST *us_getnodeonarcinst(GEOM **ipos, PORTPROTO **ipp, GEOM *othergeom,
 		{
 			/* if two arcs are perpendicular, find point of intersection */
 			oar = othergeom->entryaddr.ai;
-			if (oar->end[0].ypos == oar->end[1].ypos) prefy=oar->end[0].ypos;
+			if (oar->end[0].ypos == oar->end[1].ypos) prefy = oar->end[0].ypos;
 		}
 	} else if (ai->end[0].ypos == ai->end[1].ypos)
 	{
@@ -1454,13 +1544,13 @@ NODEINST *us_getnodeonarcinst(GEOM **ipos, PORTPROTO **ipp, GEOM *othergeom,
 		{
 			/* if two arcs are perpendicular, find point of intersection */
 			oar = othergeom->entryaddr.ai;
-			if (oar->end[0].xpos == oar->end[1].xpos) prefx=oar->end[0].xpos;
+			if (oar->end[0].xpos == oar->end[1].xpos) prefx = oar->end[0].xpos;
 		}
 	}
 
 	/* adjust to closest point */
-	(void)closestpointtosegment(ai->end[0].xpos,ai->end[0].ypos,
-		ai->end[1].xpos,ai->end[1].ypos, &prefx, &prefy);
+	(void)closestpointtosegment(ai->end[0].xpos, ai->end[0].ypos,
+		ai->end[1].xpos, ai->end[1].ypos, &prefx, &prefy);
 	if (prefx == ai->end[0].xpos && prefy == ai->end[0].ypos)
 	{
 		*ipp = ai->end[0].portarcinst->proto;
@@ -1474,29 +1564,13 @@ NODEINST *us_getnodeonarcinst(GEOM **ipos, PORTPROTO **ipp, GEOM *othergeom,
 		return(ai->end[1].nodeinst);
 	}
 
-	/* break is at (prefx, prefy): save information about the arcinst */
-	fno = ai->end[0].nodeinst;    fpt = ai->end[0].portarcinst->proto;
-	tno = ai->end[1].nodeinst;    tpt = ai->end[1].portarcinst->proto;
-	fendx = ai->end[0].xpos;      fendy = ai->end[0].ypos;
-	tendx = ai->end[1].xpos;      tendy = ai->end[1].ypos;
-	ap = ai->proto;   wid = ai->width;  pnt = ai->parent;
-	bits1 = bits2 = ai->userbits;
-	if ((bits1&ISNEGATED) != 0)
-	{
-		if ((bits1&REVERSEEND) == 0) bits2 &= ~ISNEGATED; else
-			bits1 &= ~ISNEGATED;
-	}
-	if (figureangle(fendx,fendy, prefx,prefy) != figureangle(prefx,prefy, tendx,tendy))
-	{
-		bits1 &= ~FIXANG;
-		bits2 &= ~FIXANG;
-	}
-
 	/* create the splitting pin */
+	ap = ai->proto;
 	np = us_findconnectingpin(ap, othergeom, otherport);
 	if (np == NONODEPROTO) return(NONODEINST);
 
 	/* determine width of other arc */
+	wid = ai->width;
 	us_getproperconsize(np, ap, wid, ap, wid, &pxs, &pys);
 
 	/* make contact asymmetric if other arc is wider */
@@ -1525,7 +1599,6 @@ NODEINST *us_getnodeonarcinst(GEOM **ipos, PORTPROTO **ipp, GEOM *othergeom,
 	}
 
 	/* create the contact */
-	pt1 = np->firstportproto;
 	lx = prefx - pxs/2;   hx = lx + pxs;
 	ly = prefy - pys/2;   hy = ly + pys;
 	if (fake != 0)
@@ -1536,7 +1609,7 @@ NODEINST *us_getnodeonarcinst(GEOM **ipos, PORTPROTO **ipp, GEOM *othergeom,
 		ni->proto = np;
 	} else
 	{
-		ni = newnodeinst(np, lx,hx, ly,hy, 0, 0, pnt);
+		ni = newnodeinst(np, lx, hx, ly, hy, 0, 0, ai->parent);
 		if (ni == NONODEINST)
 		{
 			ttyputerr(_("Cannot create splitting pin"));
@@ -1545,38 +1618,80 @@ NODEINST *us_getnodeonarcinst(GEOM **ipos, PORTPROTO **ipp, GEOM *othergeom,
 		endobjectchange((INTBIG)ni, VNODEINST);
 
 		/* create the two new arcinsts */
-		ar1 = newarcinst(ap, wid, bits1, fno, fpt, fendx, fendy, ni, pt1, prefx, prefy, pnt);
-		ar2 = newarcinst(ap, wid, bits2, ni, pt1, prefx, prefy, tno, tpt, tendx, tendy, pnt);
-		if (ar1 == NOARCINST || ar2 == NOARCINST)
-		{
-			ttyputerr(_("Error creating the split arc parts"));
-			return(ni);
-		}
-
-		/* figure out on which half of the arc to place former arc information */
-		mainarc = ar1;
-		if (fno->proto->primindex == 0 && tno->proto->primindex != 0)
-			mainarc = ar2; else
-		{
-			if (computedistance(fendx,fendy, prefx,prefy) <
-				computedistance(tendx,tendy, prefx,prefy)) mainarc = ar2;
-		}
-
-		(void)copyvars((INTBIG)ai, VARCINST, (INTBIG)mainarc, VARCINST, FALSE);
-		endobjectchange((INTBIG)ar1, VARCINST);
-		endobjectchange((INTBIG)ar2, VARCINST);
-
-		/* delete the old arcinst */
-		startobjectchange((INTBIG)ai, VARCINST);
-		if (killarcinst(ai)) ttyputerr(_("Error deleting original arc"));
+		(void)us_breakarcinsertnode(ai, ni, &ai1, &ai2);
 	}
 
 	/* return pointers to the splitting pin */
 	*ipos = ni->geom;
-	*ipp = pt1;
+	*ipp = ni->proto->firstportproto;
 	return(ni);
 }
 
+/*
+ * Routine to break arc "ai" at node "ni".  Sets the two new halves in "ai1" and "ai2".
+ * Returns TRUE on error.
+ */
+BOOLEAN us_breakarcinsertnode(ARCINST *ai, NODEINST *ni, ARCINST **ai1, ARCINST **ai2)
+{
+	REGISTER NODEINST *fno, *tno;
+	REGISTER ARCINST *mainarc;
+	REGISTER PORTPROTO *fpt, *tpt, *pp;
+	REGISTER NODEPROTO *parent;
+	REGISTER INTBIG fendx, fendy, tendx, tendy, bits1, bits2, wid;
+	INTBIG x, y;
+	REGISTER ARCPROTO *ap;
+
+	fno = ai->end[0].nodeinst;    fpt = ai->end[0].portarcinst->proto;
+	tno = ai->end[1].nodeinst;    tpt = ai->end[1].portarcinst->proto;
+	fendx = ai->end[0].xpos;      fendy = ai->end[0].ypos;
+	tendx = ai->end[1].xpos;      tendy = ai->end[1].ypos;
+	ap = ai->proto;   wid = ai->width;  parent = ai->parent;
+	pp = ni->proto->firstportproto;
+	portposition(ni, pp, &x, &y);
+	bits1 = bits2 = ai->userbits;
+	if ((bits1&ISNEGATED) != 0)
+	{
+		if ((bits1&REVERSEEND) == 0) bits2 &= ~ISNEGATED; else
+			bits1 &= ~ISNEGATED;
+	}
+	if (figureangle(fendx, fendy, x, y) != figureangle(x, y, tendx, tendy))
+	{
+		bits1 &= ~FIXANG;
+		bits2 &= ~FIXANG;
+	}
+
+	/* create the two new arcinsts */
+	*ai1 = newarcinst(ap, wid, bits1, fno, fpt, fendx, fendy, ni, pp, x, y, parent);
+	*ai2 = newarcinst(ap, wid, bits2, ni, pp, x, y, tno, tpt, tendx, tendy, parent);
+	if (*ai1 == NOARCINST || *ai2 == NOARCINST)
+	{
+		ttyputerr(_("Error creating the split arc parts"));
+		return(TRUE);
+	}
+
+	/* figure out on which half of the arc to place former arc information */
+	mainarc = *ai1;
+	if (fno->proto->primindex == 0 && tno->proto->primindex != 0)
+		mainarc = *ai2; else
+	{
+		if (computedistance(fendx, fendy, x, y) <
+			computedistance(tendx, tendy, x, y)) mainarc = *ai2;
+	}
+
+	(void)copyvars((INTBIG)ai, VARCINST, (INTBIG)mainarc, VARCINST, FALSE);
+	endobjectchange((INTBIG)*ai1, VARCINST);
+	endobjectchange((INTBIG)*ai2, VARCINST);
+
+	/* delete the old arcinst */
+	startobjectchange((INTBIG)ai, VARCINST);
+	if (killarcinst(ai))
+	{
+		ttyputerr(_("Error deleting original arc"));
+		return(TRUE);
+	}
+	return(FALSE);
+}
+
 /*
  * Routine to find the connecting node that can join arcs of type "ap" with
  * an object "geom" (which may be a node with port "pp").  Returns NONODEPROTO
