@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usrcheck.c
@@ -47,6 +48,7 @@ typedef struct
 	char   *name;
 } VARLIST;
 
+/* these variables should not exist */
 VARLIST us_deprecatedvariables[] =
 {
 	{VNODEPROTO, 0,                 "NET_last_good_ncc"},
@@ -390,7 +392,7 @@ void us_checkdatabase(BOOLEAN verbose)
 		}
 
 		/* check that cell is in cell hash table */
-		if (lib->cellhashtablesize > 0 && db_findcellname (c->cellname, c->lib) != c)
+		if (lib->cellhashtablesize > 0 && db_findcellname(c->cellname, c->lib) != c)
 		{
 			ttyputmsg(_("Library %c, cell %s: not in cell hash"), lib->libname, c->cellname);
 			db_buildcellhashtable(lib);
@@ -450,7 +452,7 @@ void us_checkdatabase(BOOLEAN verbose)
 		}
 		if (co != lib->numcells)
 		{
-			ttyputmsg(_("Library %s: numcell reported=%ld actual=%ld"),
+			ttyputmsg(_("Library %s: found %ld cells but records %ld"),
 				lib->libname, co, lib->numcells);
 			db_buildcellhashtable(lib);
 			warnings++;
@@ -561,8 +563,9 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 	REGISTER VIEW *v;
 	REGISTER INTBIG found, foundtot, foundtrue, facetcenters, neterrors, len, accumulatesize,
 		totlx, tothx, totly, tothy, truelx, truehx, truely, truehy, i, j, *arr, problems,
-		sx, sy, cx, cy;
-	REGISTER UINTBIG wiped, oldbits, curdate;
+		sx, sy, cx, cy, lambda, co;
+	REGISTER UINTBIG wiped, oldbits;
+	REGISTER time_t curdate;
 	INTBIG lx, hx, ly, hy, position[2], growx, growy, shiftx, shifty;
 	BOOLEAN canscalefonts;
 	XARRAY trans;
@@ -668,18 +671,18 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 		}
 
 		/* check the dates */
-		if (np->revisiondate > curdate)
+		if ((time_t)np->revisiondate > curdate)
 		{
 			ttyputmsg(_("Facet %s: invalid revision date (%s)"), describenodeproto(np),
-				timetostring(np->revisiondate));
+				timetostring((time_t)np->revisiondate));
 			np->revisiondate = curdate;
 			(*warnings)++;
 			curlib->userbits |= LIBCHANGEDMAJOR;
 		}
-		if (np->creationdate > curdate)
+		if ((time_t)np->creationdate > curdate)
 		{
 			ttyputmsg(_("Facet %s: invalid creation date (%s)"), describenodeproto(np),
-				timetostring(np->creationdate));
+				timetostring((time_t)np->creationdate));
 			np->creationdate = curdate;
 			(*warnings)++;
 			curlib->userbits |= LIBCHANGEDMAJOR;
@@ -708,6 +711,7 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 			if (found != 0)
 			{
 				ttyputmsg(_("...renamed to '%s'"), pp->protoname);
+				db_buildportprotohashtable(np);
 				(*warnings)++;
 				curlib->userbits |= LIBCHANGEDMAJOR;
 				break;
@@ -924,6 +928,16 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 				}
 			}
 
+			/* check that port is in port hash table */
+			if (np->portprotohashtablesize > 0 && getportproto(np, pp->protoname) != pp)
+			{
+				ttyputmsg(_("Facet %s, export %s: not in portproto hash"),
+					describenodeproto(np), pp->protoname);
+				db_buildportprotohashtable(np);
+				(*warnings)++;
+				curlib->userbits |= LIBCHANGEDMAJOR;
+			}
+
 			/* check variables in the port */
 			if (us_checkvariables((INTBIG)pp, VPORTPROTO, &pp->numvar, pp->firstvar, curlib))
 			{
@@ -932,6 +946,32 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 			}
 		}
 
+		/* check portproto list */
+		co = 0;
+		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto) co++;
+		if (co != np->numportprotos)
+		{
+			ttyputmsg(_("Facet %s: found %ld portprotos but records %ld"),
+				describenodeproto(np), co, np->numportprotos);
+			db_buildportprotohashtable(np);
+			(*warnings)++;
+			curlib->userbits |= LIBCHANGEDMAJOR;
+		}
+		if (np->portprotohashtablesize > 0)
+		{
+			co = 0;
+			for (i=0; i<np->portprotohashtablesize; i++)
+				if (np->portprotohashtable[i] != NOPORTPROTO) co++;
+			if (co != np->numportprotos)
+			{
+				ttyputmsg(_("Facet %s: numcell=%ld in hash=%ld"),
+					describenodeproto(np), np->numportprotos, co);
+				db_buildportprotohashtable(np);
+				(*warnings)++;
+				curlib->userbits |= LIBCHANGEDMAJOR;
+			}
+		}
+
 		/* if this is an icon facet, check port equivalences to the layout */
 		if (np->cellview == el_iconview)
 		{
@@ -997,6 +1037,7 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 		foundtot = foundtrue = 0;
 		totlx = tothx = totly = tothy = 0;
 		truelx = truehx = truely = truehy = 0;
+		lambda = lambdaoffacet(np);
 
 		if (verbose) ttyputmsg(_("Checking facet %s"), describenodeproto(np));
 
@@ -1053,9 +1094,9 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 					ni->highy-ni->lowy != ni->proto->highy-ni->proto->lowy)
 				{
 					ttyputmsg(_("Facet %s, node %s: node size was %sx%s, should be %sx%s"),
-						describenodeproto(np), describenodeinst(ni), latoa(ni->highx-ni->lowx),
-							latoa(ni->highy-ni->lowy), latoa(ni->proto->highx-ni->proto->lowx),
-								latoa(ni->proto->highy-ni->proto->lowy));
+						describenodeproto(np), describenodeinst(ni), latoa(ni->highx-ni->lowx, lambda),
+							latoa(ni->highy-ni->lowy, lambda), latoa(ni->proto->highx-ni->proto->lowx, lambda),
+								latoa(ni->proto->highy-ni->proto->lowy, lambda));
 					ni->lowx += ((ni->highx-ni->lowx) - (ni->proto->highx-ni->proto->lowx))/2;
 					ni->highx = ni->lowx + ni->proto->highx-ni->proto->lowx;
 					ni->lowy += ((ni->highy-ni->lowy) - (ni->proto->highy-ni->proto->lowy))/2;
@@ -1378,9 +1419,10 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 				ly != ni->geom->lowy || hy != ni->geom->highy)
 			{
 				ttyputmsg(_("Facet %s, node %s: geometry size bad (was %s<=X<=%s, %s<=Y<=%s, is %s<=X<=%s, %s<=Y<=%s)"),
-					describenodeproto(np), describenodeinst(ni), latoa(ni->geom->lowx),
-						latoa(ni->geom->highx), latoa(ni->geom->lowy), latoa(ni->geom->highy),
-							latoa(lx), latoa(hx), latoa(ly), latoa(hy));
+					describenodeproto(np), describenodeinst(ni), latoa(ni->geom->lowx, lambda),
+						latoa(ni->geom->highx, lambda), latoa(ni->geom->lowy, lambda),
+						latoa(ni->geom->highy, lambda), latoa(lx, lambda), latoa(hx, lambda),
+						latoa(ly, lambda), latoa(hy, lambda));
 				updategeom(ni->geom, np);
 				(*warnings)++;
 				curlib->userbits |= LIBCHANGEDMAJOR;
@@ -1715,7 +1757,7 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 			if (i != ai->length)
 			{
 				ttyputmsg(_("Facet %s, arc %s: bad length (was %s)"), describenodeproto(np),
-					describearcinst(ai), latoa(ai->length));
+					describearcinst(ai), latoa(ai->length, lambda));
 				ai->length = i;
 				(*warnings)++;
 				curlib->userbits |= LIBCHANGEDMAJOR;
@@ -1828,8 +1870,9 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 		if (np->lowx != totlx || np->highx != tothx || np->lowy != totly || np->highy != tothy)
 		{
 			ttyputmsg(_("Facet %s: bounds are %s<=X<=%s %s<=Y<=%s, should be %s<=X<=%s %s<=Y<=%s"),
-				describenodeproto(np), latoa(np->lowx), latoa(np->highx), latoa(np->lowy),
-					latoa(np->highy), latoa(totlx), latoa(tothx), latoa(totly), latoa(tothy));
+				describenodeproto(np), latoa(np->lowx, lambda), latoa(np->highx, lambda),
+				latoa(np->lowy, lambda), latoa(np->highy, lambda), latoa(totlx, lambda),
+				latoa(tothx, lambda), latoa(totly, lambda), latoa(tothy, lambda));
 			for(ni = np->firstinst; ni != NONODEINST; ni = ni->nextinst)
 			{
 				growx = ((tothx - np->highx) + (totlx - np->lowx)) / 2;
@@ -1882,9 +1925,9 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 			np->rtree->lowy != truely || np->rtree->highy != truehy)
 		{
 			ttyputmsg(_("Facet %s: Incorrect R-tree size (was %s<=X<=%s, %s<=Y<=%s, is %s<=X<=%s, %s<=Y<=%s)"),
-				describenodeproto(np), latoa(np->rtree->lowx), latoa(np->rtree->highx),
-					latoa(np->rtree->lowy), latoa(np->rtree->highy), latoa(truelx), latoa(truehx),
-						latoa(truely), latoa(truehy));
+				describenodeproto(np), latoa(np->rtree->lowx, lambda), latoa(np->rtree->highx, lambda),
+				latoa(np->rtree->lowy, lambda), latoa(np->rtree->highy, lambda), latoa(truelx, lambda),
+				latoa(truehx, lambda), latoa(truely, lambda), latoa(truehy, lambda));
 			np->rtree->lowx = truelx;
 			np->rtree->highx = truehx;
 			np->rtree->lowy = truely;
@@ -1949,7 +1992,7 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 				}
 			} else
 			{
-				if (ai->network->signals > 1 &&
+				if (ai->network->buswidth > 1 &&
 					((ai->proto->userbits&AFUNCTION)>>AFUNCTIONSH) != APBUS)
 				{
 					ttyputmsg(_("Facet %s, arc %s: not a bus but network %s is, NOT REPAIRED"), describenodeproto(np),
@@ -1969,19 +2012,19 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 					if (ni->proto->primindex == 0)
 					{
 						pi = ai->end[i].portarcinst;
-						len = pi->proto->network->signals;
+						len = pi->proto->network->buswidth;
 						if (ni->arraysize > 1) len *= ni->arraysize;
-						if (ai->network->signals != len)
+						if (ai->network->buswidth != len)
 						{
 							if (ni->arraysize > 1)
 							{
 								ttyputmsg(_("Facet %s, arc %s: is %d wide but port (on %ld-wide node) is %ld"),
-									describenodeproto(np), describearcinst(ai), ai->network->signals,
+									describenodeproto(np), describearcinst(ai), ai->network->buswidth,
 										ni->arraysize, len);
 							} else
 							{
 								ttyputmsg(_("Facet %s, arc %s: is %d wide but port is %ld"),
-									describenodeproto(np), describearcinst(ai), ai->network->signals,
+									describenodeproto(np), describearcinst(ai), ai->network->buswidth,
 										len);
 							}
 							neterrors++;
@@ -2047,7 +2090,7 @@ void us_checklibrary(LIBRARY *curlib, BOOLEAN verbose, INTBIG *warnings, INTBIG
 			net->temp1 = 0;
 		for(net = np->firstnetwork; net != NONETWORK; net = net->nextnetwork)
 		{
-			if (net->signals > 1) for(i=0; i<net->signals; i++)
+			if (net->buswidth > 1) for(i=0; i<net->buswidth; i++)
 				net->networklist[i]->temp1++;
 		}
 		for(net = np->firstnetwork; net != NONETWORK; net = net->nextnetwork)
@@ -2107,6 +2150,7 @@ BOOLEAN us_checkvariables(INTBIG addr, INTBIG type, INTSML *numvar, VARIABLE *fi
 	REGISTER NODEPROTO *np, *vnp;
 	REGISTER char *varname, *matchname;
 	REGISTER TECHNOLOGY *tech;
+	REGISTER VARIABLE *var;
 
 	if (*numvar < 0)
 	{
@@ -2122,8 +2166,9 @@ BOOLEAN us_checkvariables(INTBIG addr, INTBIG type, INTSML *numvar, VARIABLE *fi
 	errors = 0;
 	for(i = 0; i < *numvar; i++)
 	{
-		ty = firstvar[i].type;
-		varname = (char *)firstvar[i].key;
+		var = &firstvar[i];
+		ty = var->type;
+		varname = (char *)var->key;
 
 		/* see if this variable has been deprecated */
 		for(j=0; us_deprecatedvariables[j].type != 0; j++)
@@ -2199,6 +2244,30 @@ BOOLEAN us_checkvariables(INTBIG addr, INTBIG type, INTSML *numvar, VARIABLE *fi
 			continue;
 		}
 
+		/* check parameters on facets have the "inheritable" bit set */
+		if (type == VNODEPROTO && TDGETISPARAM(var->textdescript) != 0)
+		{
+			/* make sure the "inheritable" bit is set */
+			if (TDGETINHERIT(var->textdescript) == 0)
+			{
+				ttyputmsg(_("%s: variable '%s' is parameter but not inheritable"),
+					us_describeobject(addr, type), varname);
+				TDSETINHERIT(var->textdescript, VTINHERIT);
+				errors++;
+				continue;
+			}
+
+			/* make sure it is visible */
+			if ((var->type&VDISPLAY) == 0)
+			{
+				ttyputmsg(_("%s: variable '%s' is parameter but not visible"),
+					us_describeobject(addr, type), varname);
+				var->type |= VDISPLAY;
+				errors++;
+				continue;
+			}
+		}
+
 		/* check validity of certain pointer variables */
 		if ((ty&(VTYPE|VISARRAY)) == VNODEPROTO)
 		{
@@ -2415,9 +2484,21 @@ char *us_describeobject(INTBIG addr, INTBIG type)
  */
 BOOLEAN us_checktextdescript(INTBIG addr, INTBIG type, UINTBIG *descript)
 {
-	INTBIG face, maxfaces;
+	INTBIG face, maxfaces, disppart;
 	char **facelist;
 
+	if ((type&VTYPE) != VNODEPROTO)
+	{
+		disppart = TDGETDISPPART(descript);
+		if(disppart == VTDISPLAYNAMEVALINH || disppart == VTDISPLAYNAMEVALINHALL)
+		{
+			ttyputmsg(_("%s: textdescript has inherited display, changing to name-value-display"),
+				  us_describeobject(addr, type));
+			TDSETDISPPART(descript, VTDISPLAYNAMEVALUE);
+			return(TRUE);
+		}
+	}
+
 	if (!graphicshas(CANCHOOSEFACES)) return(FALSE);
 	face = TDGETFACE(descript);
 	maxfaces = screengetfacelist(&facelist, FALSE);
