@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usrcomab.c
@@ -31,11 +32,12 @@
 #include "global.h"
 #include "egraphics.h"
 #include "usr.h"
+#include "drc.h"
 #include "usrtrack.h"
 
 void us_arc(INTBIG count, char *par[])
 {
-	REGISTER INTBIG negate, toggle, r, j, i, l, interactive;
+	REGISTER INTBIG negate, toggle, r, j, i, l, interactive, lambda, wid;
 	INTBIG ix, iy, xcur, ycur, dx[2], dy[2];
 	REGISTER ARCINST *ai;
 	REGISTER NODEPROTO *np;
@@ -100,7 +102,7 @@ void us_arc(INTBIG count, char *par[])
 
 	/* disallow arraying if lock is on */
 	np = geomparent(list[0]);
-	if (us_cantedit(np, NONODEPROTO, TRUE)) return;
+	if (us_cantedit(np, NONODEINST, TRUE)) return;
 
 	/* save and erase highlighting */
 	us_pushhighlight();
@@ -316,6 +318,9 @@ void us_arc(INTBIG count, char *par[])
 			{
 				shapeportpoly(ai->end[j].nodeinst, ai->end[j].portarcinst->proto,
 					poly, FALSE);
+				wid = ai->width - arcwidthoffset(ai);
+				reduceportpoly(poly, ai->end[j].nodeinst, ai->end[j].portarcinst->proto, wid,
+					(ai->userbits&AANGLE)>>AANGLESH);
 				ix = ai->end[1-j].xpos;   iy = ai->end[1-j].ypos;
 				closestpoint(poly, &ix, &iy);
 				dx[j] = ix - ai->end[j].xpos;
@@ -324,7 +329,13 @@ void us_arc(INTBIG count, char *par[])
 			if (dx[0] != 0 || dy[0] != 0 || dx[1] != 0 || dy[1] != 0)
 			{
 				startobjectchange((INTBIG)ai, VARCINST);
-				(void)modifyarcinst(ai, 0, dx[0], dy[0], dx[1], dy[1]);
+				if (modifyarcinst(ai, 0, dx[0], dy[0], dx[1], dy[1]))
+				{
+					lambda = lambdaofarc(ai);
+					ttyputerr(_("Problem shortening arc %s: cannot move tail by (%s,%s) and head by (%s,%s)"),
+						describearcinst(ai), latoa(dx[0], lambda), latoa(dy[0], lambda),
+							latoa(dx[1], lambda), latoa(dy[1], lambda));
+				}
 				endobjectchange((INTBIG)ai, VARCINST);
 				l++;
 			}
@@ -394,16 +405,16 @@ void us_arc(INTBIG count, char *par[])
 void us_array(INTBIG count, char *par[])
 {
 	REGISTER BOOLEAN flipx, flipy, first, staggerx, staggery, centerx, centery,
-		noname, diagonal;
+		noname, diagonal, onlydrcvalid, *validity;
 	REGISTER INTSML ro, tr;
 	REGISTER INTBIG x, y, xpos, ypos, nodecount, arccount, i, l, lx, hx, ly, hy, cx, cy,
 		cx0, cy0, cx1, cy1, xoff0, xoff1, yoff0, yoff1, xoff, yoff, keepbits, xindex,
-		xsize, ysize, yindex, originalx, originaly;
+		xsize, ysize, yindex, originalx, originaly, lambda, checknodecount;
 	REGISTER GEOM **list, *geom;
 	INTBIG xoverlap, yoverlap, plx, ply, phx, phy;
 	REGISTER char *pt1, *pt2;
 	char totalname[200], *objname;
-	REGISTER NODEINST *ni, *newno, **nodelist, *ni0, *ni1;
+	REGISTER NODEINST *ni, *newno, **nodelist, *ni0, *ni1, **nodestocheck;
 	REGISTER ARCINST *ai, **arclist, *newai;
 	REGISTER NODEPROTO *np;
 	REGISTER PORTEXPINST *pe;
@@ -424,21 +435,22 @@ void us_array(INTBIG count, char *par[])
 		return;
 	}
 	np = geomparent(list[0]);
+	lambda = lambdaoffacet(np);
 
 	/* disallow arraying if lock is on */
 	for(i=0; list[i] != NOGEOM; i++)
 	{
 		if (!list[i]->entryisnode)
 		{
-			if (us_cantedit(np, NONODEPROTO, TRUE)) return;
+			if (us_cantedit(np, NONODEINST, TRUE)) return;
 		} else
 		{
-			if (us_cantedit(np, list[i]->entryaddr.ni->proto, TRUE)) return;
+			if (us_cantedit(np, list[i]->entryaddr.ni, TRUE)) return;
 		}
 	}
 
 	/* see if the array is to be unnamed (last argument is "no-names") */
-	noname = diagonal = FALSE;
+	noname = diagonal = onlydrcvalid = FALSE;
 	while (count > 0)
 	{
 		l = strlen(pt1 = par[count-1]);
@@ -450,13 +462,17 @@ void us_array(INTBIG count, char *par[])
 		{
 			diagonal = TRUE;
 			count--;
+		} else if (namesamen(pt1, "only-drc-valid", l) == 0)
+		{
+			onlydrcvalid = TRUE;
+			count--;
 		} else break;
 	}
 
 	/* get the X extent of the array */
 	if (count == 0)
 	{
-		ttyputusage("array XREP[f][s][c] [YREP[f][s][c] [XOVRLP [YOVRLP [no-names][diagonal]]]]");
+		ttyputusage("array XREP[f][s][c] [YREP[f][s][c] [XOVRLP [YOVRLP [no-names][diagonal][only-drc-valid]]]]");
 		return;
 	}
 	pt1 = par[0];
@@ -583,8 +599,8 @@ void us_array(INTBIG count, char *par[])
 	/* get the overlap (if any) */
 	if (count >= 3)
 	{
-		xoverlap -= atola(par[2]);
-		if (count >= 4) yoverlap -= atola(par[3]);
+		xoverlap -= atola(par[2], lambda);
+		if (count >= 4) yoverlap -= atola(par[3], lambda);
 	}
 
 	/* check to ensure the array doesn't overflow the address space */
@@ -635,6 +651,18 @@ void us_array(INTBIG count, char *par[])
 		}
 	}
 
+	/* if only arraying where DRC clean, make an array of newly created nodes */
+	if (onlydrcvalid)
+	{
+		nodestocheck = (NODEINST **)emalloc(xsize * ysize * (sizeof (NODEINST *)), us_tool->cluster);
+		if (nodestocheck == 0) return;
+		validity = (BOOLEAN *)emalloc(xsize * ysize * (sizeof (BOOLEAN)), us_tool->cluster);
+		if (validity == 0) return;
+		checknodecount = 0;
+		if (nodecount == 1)
+			nodestocheck[checknodecount++] = nodelist[0];
+	}
+
 	/* save highlighting */
 	us_pushhighlight();
 	us_clearhighlightcount();
@@ -737,6 +765,8 @@ void us_array(INTBIG count, char *par[])
 
 			endobjectchange((INTBIG)newno, VNODEINST);
 			ni->temp1 = (UINTBIG)newno;
+			if (onlydrcvalid && i == 0)
+				nodestocheck[checknodecount++] = newno;
 		}
 
 		/* create any queued exports */
@@ -858,6 +888,23 @@ void us_array(INTBIG count, char *par[])
 		}
 	}
 
+	/* if only arraying where DRC valid, check them now and delete what is not valid */
+	if (onlydrcvalid)
+	{
+		(void)asktool(dr_tool, "check-instances", (INTBIG)np, checknodecount, (INTBIG)nodestocheck,
+			(INTBIG)validity);
+		for(i=1; i<checknodecount; i++)
+		{
+			if (!validity[i])
+			{
+				/* delete the node */
+				killnodeinst(nodestocheck[i]);
+			}
+		}
+		efree((char *)nodestocheck);
+		efree((char *)validity);
+	}
+
 	/* restore highlighting */
 	us_pophighlight(FALSE);
 	if (nodecount > 0) efree((char *)nodelist);
