@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usrcomcd.c
@@ -30,6 +31,7 @@
 
 #include "global.h"
 #include "egraphics.h"
+#include "edialogs.h"
 #include "usr.h"
 #include "usrtrack.h"
 #include "database.h"
@@ -833,11 +835,11 @@ void us_create(INTBIG count, char *par[])
 	REGISTER NODEINST *ni, *newno, *ni2;
 	REGISTER PORTPROTO *pp, *ppt, *fpt, *pp1, *pp2;
 	PORTPROTO *fromport, *toport;
-	REGISTER ARCPROTO *ap;
+	REGISTER ARCPROTO *ap, *wantap;
 	REGISTER ARCINST *ai, *newai;
 	REGISTER TECHNOLOGY *tech;
 	NODEINST *ani[2];
-	REGISTER INTBIG ang, pangle, l, len, bestdist, bestdist1, bestdist2, wid, truewid, i, j, k,
+	REGISTER INTBIG ang, pangle, l, bestdist, bestdist1, bestdist2, wid, truewid, i, j, k,
 		bits1, bits2, dist, alignment, edgealignment;
 	BOOLEAN remainhighlighted, ishor, isver, join, angledcreate,
 		waitfordown, centeredprimitives, getcontents;
@@ -849,7 +851,7 @@ void us_create(INTBIG count, char *par[])
 	ARCINST *alt1, *alt2;
 	NODEINST *con1, *con2;
 	char *oldarcname, *newpar[2];
-	HIGHLIGHT newhigh;
+	HIGHLIGHT newhigh, *high;
 	REGISTER GEOM *foundgeom, *geom, **list;
 	GEOM *fromgeom, *togeom;
 
@@ -943,7 +945,7 @@ void us_create(INTBIG count, char *par[])
 			curfacet = us_needfacet();
 
 			/* disallow creating if lock is on */
-			if (us_cantedit(curfacet, np, TRUE)) return;
+			if (us_cantedit(curfacet, NONODEINST, TRUE)) return;
 
 			/* create and display the nodeinst */
 			corneroffset(NONODEINST, np, pangle%3600, pangle/3600, &cx, &cy,
@@ -958,7 +960,8 @@ void us_create(INTBIG count, char *par[])
 				return;
 			}
 			if ((np->userbits&WANTNEXPAND) != 0) newno->userbits |= NEXPAND;
-			if (np == gen_facetcenterprim) newno->userbits |= HARDSELECTN|NVISIBLEINSIDE;
+			if (np == gen_facetcenterprim) newno->userbits |= HARDSELECTN|NVISIBLEINSIDE; else
+				if (np == gen_essentialprim) newno->userbits |= HARDSELECTN;
 
 			/* copy "inheritable" attributes */
 			us_inheritattributes(newno);
@@ -985,14 +988,14 @@ void us_create(INTBIG count, char *par[])
 				return;
 			}
 
-			if (us_cantedit(curfacet, NONODEPROTO, TRUE)) return;
+			if (us_cantedit(curfacet, NONODEINST, TRUE)) return;
 
 			/* run an arc from "fromgeom" to "togeom" */
 			if (remainhighlighted) us_pushhighlight();
 			us_clearhighlightcount();
 			if (!angledcreate) ang = 900;
 			ai = aconnect(fromgeom, fromport, togeom, toport, us_curarcproto,
-				xcur, ycur, &alt1, &alt2, &con1, &con2, ang, FALSE);
+				xcur, ycur, &alt1, &alt2, &con1, &con2, ang, FALSE, TRUE);
 			if (remainhighlighted) us_pophighlight(FALSE);
 			if (ai == NOARCINST) return;
 
@@ -1027,12 +1030,15 @@ void us_create(INTBIG count, char *par[])
 			us_abortcommand(_("Must start new arcs from one node or arc"));
 			return;
 		}
+		splitpp = NOPORTPROTO;
 		if (list[0]->entryisnode)
 		{
 			ni = list[0]->entryaddr.ni;
+			high = us_getonehighlight();
+			if (high != NOHIGHLIGHT && (high->status&HIGHTYPE) == HIGHFROM)
+				splitpp = high->fromport;
 		} else
 		{
-			splitpp = NOPORTPROTO;
 			ni = us_getnodeonarcinst(&list[0], &splitpp, list[0], NOPORTPROTO, xcur, ycur, 0);
 			if (ni == NONODEINST)
 			{
@@ -1048,25 +1054,14 @@ void us_create(INTBIG count, char *par[])
 			us_abortcommand(_("Cursor must be in the same facet as the highlighted node"));
 			return;
 		}
-		if (us_cantedit(curfacet, NONODEPROTO, TRUE)) return;
+		if (us_cantedit(curfacet, NONODEINST, TRUE)) return;
 
 		/* find the closest port to the cursor */
-		var = getvalkey((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, us_highlightedkey);
-		if (var != NOVARIABLE)
-		{
-			len = getlength(var);
-			for(i=0; i<len; i++)
-			{
-				(void)us_makehighlight(((char **)var->addr)[i], &newhigh);
-				if (newhigh.status != HIGHFROM) continue;
-				if (newhigh.fromgeom == list[0]) break;
-			}
-		}
 		bestdist1 = bestdist2 = MAXINTBIG;
 		ppt = fpt = NOPORTPROTO;
 		for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		{
-			if (newhigh.fromport != NOPORTPROTO && newhigh.fromport != pp) continue;
+			if (splitpp != NOPORTPROTO && pp != splitpp) continue;
 			shapeportpoly(ni, pp, poly, FALSE);
 			k = polydistance(poly, xcur, ycur);
 
@@ -1142,7 +1137,29 @@ void us_create(INTBIG count, char *par[])
 				}
 			}
 		} else ppt = fpt;
-		newhigh.fromport = ppt;
+		splitpp = ppt;
+
+		/* make sure the selected arc connects to this port */
+		wantap = us_curarcproto;
+		for(i=0; splitpp->connects[i] != NOARCPROTO; i++)
+			if (splitpp->connects[i] == us_curarcproto) break;
+		if (splitpp->connects[i] == NOARCPROTO)
+		{
+			/* cannot: see if anything in this technology can connect */
+			for(i=0; splitpp->connects[i] != NOARCPROTO; i++)
+				if (splitpp->connects[i]->tech == us_curarcproto->tech) break;
+			if (splitpp->connects[i] != NOARCPROTO) wantap = splitpp->connects[i]; else
+			{
+				/* cannot: see if anything nongeneric can connect */
+				for(i=0; splitpp->connects[i] != NOARCPROTO; i++)
+					if (splitpp->connects[i]->tech != gen_tech) break;
+				if (splitpp->connects[i] != NOARCPROTO) wantap = splitpp->connects[i]; else
+				{
+					/* cannot: select the first possible arcproto */
+					wantap = splitpp->connects[0];
+				}
+			}
+		}
 
 		/* turn off highlighting */
 		us_clearhighlightcount();
@@ -1152,7 +1169,7 @@ void us_create(INTBIG count, char *par[])
 		shapeportpoly(ni, ppt, poly, 1);
 
 		/* determine pin for making the connection */
-		np = getpinproto(us_curarcproto);
+		np = getpinproto(wantap);
 		if (np == NONODEPROTO) return;
 
 		/* get placement angle */
@@ -1217,8 +1234,8 @@ void us_create(INTBIG count, char *par[])
 					if (us_figuredrawpath(fromgeom, ppt, togeom, toport, &xcur, &ycur))
 						return;
 					us_clearhighlightcount();
-					ai = aconnect(fromgeom, ppt, togeom, toport, us_curarcproto,
-						xcur, ycur, &alt1, &alt2, &con1, &con2, ang, FALSE);
+					ai = aconnect(fromgeom, ppt, togeom, toport, wantap,
+						xcur, ycur, &alt1, &alt2, &con1, &con2, ang, FALSE, TRUE);
 					if (ai == NOARCINST) return;
 
 					/* if an arc was highlighted and it got broken, fix the highlight */
@@ -1281,7 +1298,7 @@ void us_create(INTBIG count, char *par[])
 						{
 							for (i=0; pp->connects[i] != NOARCPROTO; i++)
 							{
-								if (pp->connects[i] == us_curarcproto)
+								if (pp->connects[i] == wantap)
 								{
 									/* can connect */
 									bestdist2 = dist;
@@ -1296,11 +1313,11 @@ void us_create(INTBIG count, char *par[])
 						for(i=0; fpt->connects[i] != NOARCPROTO; i++)
 						{
 							if (fpt->connects[i]->tech == gen_tech) continue;
-							for(j=0; newhigh.fromport->connects[j] != NOARCPROTO; j++)
+							for(j=0; splitpp->connects[j] != NOARCPROTO; j++)
 							{
-								if (fpt->connects[i] == newhigh.fromport->connects[j]) break;
+								if (fpt->connects[i] == splitpp->connects[j]) break;
 							}
-							if (newhigh.fromport->connects[j] != NOARCPROTO) break;
+							if (splitpp->connects[j] != NOARCPROTO) break;
 						}
 						if (fpt->connects[i] != NOARCPROTO)
 						{
@@ -1310,8 +1327,8 @@ void us_create(INTBIG count, char *par[])
 					}
 					if (ppt != NOPORTPROTO) fpt = ppt;
 				}
-				(void)aconnect(newhigh.fromgeom, newhigh.fromport, foundgeom,
-					fpt, us_curarcproto, xcur, ycur, &alt1, &alt2, &con1, &con2, ang, TRUE);
+				(void)aconnect(ni->geom, splitpp, foundgeom,
+					fpt, wantap, xcur, ycur, &alt1, &alt2, &con1, &con2, ang, TRUE, TRUE);
 
 				/* if an arc will be highlighted and it broke, fix the highlight */
 				if (!foundgeom->entryisnode &&
@@ -1329,10 +1346,10 @@ void us_create(INTBIG count, char *par[])
 		}
 
 		/* determine arc width */
-		truewid = maxi(defaultarcwidth(us_curarcproto), us_widestarcinst(us_curarcproto, ni, ppt));
-		i = us_stretchtonodes(us_curarcproto, truewid, ni, ppt, xcur, ycur);
+		truewid = maxi(defaultarcwidth(wantap), us_widestarcinst(wantap, ni, ppt));
+		i = us_stretchtonodes(wantap, truewid, ni, ppt, xcur, ycur);
 		if (i > truewid) truewid = i;
-		wid = truewid - arcprotowidthoffset(us_curarcproto);
+		wid = truewid - arcprotowidthoffset(wantap);
 
 		/* see if arc edge alignment is appropriate */
 		if (us_edgealignment_ratio != 0 && (x == nx || y == ny))
@@ -1390,8 +1407,8 @@ void us_create(INTBIG count, char *par[])
 		if (((ppt->userbits&PORTARANGE) >> PORTARANGESH) == 180)
 		{
 			ai = us_runarcinst(ni, ppt, x, y, newno,
-				newno->proto->firstportproto, nx, ny, us_curarcproto, truewid);
-			if (ai != NOARCINST) ttyputmsg(_("Created 1 %s arc"), us_curarcproto->protoname);
+				newno->proto->firstportproto, nx, ny, wantap, truewid);
+			if (ai != NOARCINST) us_reportarcscreated(0, 1, wantap->protoname, 0, "");
 		} else
 		{
 			k = us_bottomrecurse(ni, ppt) % 3600;
@@ -1399,7 +1416,7 @@ void us_create(INTBIG count, char *par[])
 			xcur = x + mult(cosine(k), ni->highx-ni->lowx);
 			ycur = y + mult(sine(k), ni->highy-ni->lowy);
 			(void)aconnect(ni->geom, ppt, newno->geom, newno->proto->firstportproto,
-				us_curarcproto, xcur, ycur, &alt1, &alt2, &con1, &con2, ang, TRUE);
+				wantap, xcur, ycur, &alt1, &alt2, &con1, &con2, ang, TRUE, TRUE);
 		}
 
 		newhigh.status = HIGHFROM;
@@ -1486,14 +1503,14 @@ void us_create(INTBIG count, char *par[])
 			ttyputusage("create to X Y");
 			return;
 		}
-		nx = atola(par[1]);
-		ny = atola(par[2]);
+		nx = atola(par[1], 0);
+		ny = atola(par[2], 0);
 
 		/* disallow copying if lock is on */
 		np = us_nodetocreate(TRUE, curfacet);
 		if (np == NONODEPROTO) return;
 
-		if (us_cantedit(curfacet, np, TRUE)) return;
+		if (us_cantedit(curfacet, NONODEINST, TRUE)) return;
 
 		/* get placement angle */
 		pangle = us_getplacementangle(np);
@@ -1510,7 +1527,8 @@ void us_create(INTBIG count, char *par[])
 			return;
 		}
 		if ((np->userbits&WANTNEXPAND) != 0) newno->userbits |= NEXPAND;
-		if (np == gen_facetcenterprim) newno->userbits |= HARDSELECTN|NVISIBLEINSIDE;
+		if (np == gen_facetcenterprim) newno->userbits |= HARDSELECTN|NVISIBLEINSIDE; else
+			if (np == gen_essentialprim) newno->userbits |= HARDSELECTN;
 		endobjectchange((INTBIG)newno, VNODEINST);
 
 		us_clearhighlightcount();
@@ -1538,7 +1556,7 @@ void us_create(INTBIG count, char *par[])
 		if (np == NONODEPROTO) return;
 
 		/* disallow creating if lock is on */
-		if (us_cantedit(curfacet, np, TRUE)) return;
+		if (us_cantedit(curfacet, NONODEINST, TRUE)) return;
 
 		/* turn off highlighting */
 		us_clearhighlightcount();
@@ -1712,7 +1730,7 @@ void us_create(INTBIG count, char *par[])
 		}
 		ppt = np->firstportproto;
 
-		if (us_cantedit(curfacet, np, TRUE)) return;
+		if (us_cantedit(curfacet, NONODEINST, TRUE)) return;
 
 		/* turn off highlighting */
 		us_clearhighlightcount();
@@ -1887,6 +1905,66 @@ void us_debugwritepolygon(INTBIG layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y,
 }
 #endif
 
+/*#define TESTMODELESS 1*/
+
+#ifdef TESTMODELESS
+#include "edialogs.h"
+/* test */
+static DIALOGITEM us_testdialogitems[] =
+{
+ /*  1 */ {0, {172,164,196,244}, BUTTON, N_("Done")},
+ /*  2 */ {0, {28,8,44,88}, BUTTON, N_("Button")},
+ /*  3 */ {0, {28,100,44,216}, CHECK, N_("Check Box")},
+ /*  4 */ {0, {56,8,72,124}, RADIO, N_("Radio 1")},
+ /*  5 */ {0, {76,8,92,124}, RADIO, N_("Radio 2")},
+ /*  6 */ {0, {96,8,112,124}, RADIO, N_("Radio 3")},
+ /*  7 */ {0, {132,8,232,132}, SCROLL, ""},
+ /*  8 */ {0, {4,4,20,220}, MESSAGE, N_("A Modeless Dialog")},
+ /*  9 */ {0, {56,140,72,280}, EDITTEXT, ""},
+ /* 10 */ {0, {120,8,121,288}, DIVIDELINE, ""},
+ /* 11 */ {0, {93,140,109,280}, POPUP, ""}
+};
+static DIALOG us_testdialog = {{75,75,316,372}, "Modeless Dialog", 0, 11, us_testdialogitems, 0, 0};
+void *us_testmodelessdia;
+void us_modelessdialoghitroutine(void *dia, INTBIG item);
+void us_modelessdialoghitroutine(void *dia, INTBIG item)
+{
+	if (item == 1)
+	{
+		DiaDoneDialog(us_testmodelessdia);
+		return;
+	}
+	if (item == 2)
+	{
+		ttyputmsg("Button clicked");
+		return;
+	}
+	if (item == 3)
+	{
+		DiaSetControl(us_testmodelessdia, item, 1-DiaGetControl(us_testmodelessdia, item));
+		return;
+	}
+	if (item == 4 || item == 5 || item == 6)
+	{
+		DiaSetControl(us_testmodelessdia, 4, 0);
+		DiaSetControl(us_testmodelessdia, 5, 0);
+		DiaSetControl(us_testmodelessdia, 6, 0);
+		DiaSetControl(us_testmodelessdia, item, 1);
+		return;
+	}
+	if (item == 9)
+	{
+		ttyputmsg("Edit now is '%s'", DiaGetText(us_testmodelessdia, 9));
+		return;
+	}
+	if (item == 11)
+	{
+		ttyputmsg("Selected popup item %d", DiaGetPopupEntry(us_testmodelessdia, 11));
+		return;
+	}
+}
+#endif
+
 void us_debug(INTBIG count, char *par[])
 {
 	REGISTER NODEPROTO *np;
@@ -1898,144 +1976,76 @@ void us_debug(INTBIG count, char *par[])
 	REGISTER INTBIG i, j, l, let1, let2, let3, filestatus;
 	char line[200], *pt, *libname, *libfile;
 	extern BOOLEAN db_printerrors;
-	HIGHLIGHT newhigh;
 
 	if (count == 0) return;
 	l = strlen(par[0]);
 
 	/********** THESE ARE SPECIAL CASES **********/
-	if (namesamen(par[0], "gainsley", l) == 0)
+#ifdef TESTMODELESS
+	if (namesamen(par[0], "modeless", l) == 0)
 	{
-		for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
-		{
-			for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
-			{
-				for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
-					if (ai->proto == gen_universalarc) break;
-				if (ai != NOARCINST)
-					ttyputmsg("Found Universal arcs in facet %s", describenodeproto(np));
-			}
-		}
-		return;
-	}
-
-#if 0		/* Test JAVA hack */
-	if (namesamen(par[0], "java", l) == 0)
-	{
-		GEOM **list;
-		INTBIG methodreturntype;
-		char *str;
-		char *java_init(void);
-		char *java_query(char *str, INTBIG *methodreturntype);
-
-		list = us_gethighlighted(WANTNODEINST, 0, 0);
-		if (list[0] == NOGEOM)
-		{
-			ttyputerr("Select a node first");
-			return;
-		}
-		for(i=0; i<1000; i++)
-		{
-			java_init();
-			str = java_query("P(\"SP\")>0.5?6*P(\"SP\"):3", &methodreturntype);
-			if (str == 0)
-			{
-				ttyputmsg("Query returned zero on iteration %ld", i);
-				break;
-			};
-		}
-		ttyputmsg("ran the query");
-		return;
-	}
-#endif
-
-#if 0	/* reverse length and width on schematic transistors */
-	if (namesamen(par[0], "ivan", l) == 0)
-	{
-		VARIABLE *varl, *varw;
-		INTBIG type, addr, ltype, wtype;
-		char lenstr[200], widstr[200];
-		UINTBIG ldescript[TEXTDESCRIPTSIZE], wdescript[TEXTDESCRIPTSIZE];
-
-		for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
-		{
-			for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
-			{
-				for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
-				{
-					if (ni->proto != sch_transistorprim) continue;
-					varl = getvalkey((INTBIG)ni, VNODEINST, -1, el_attrkey_length);
-					varw = getvalkey((INTBIG)ni, VNODEINST, -1, el_attrkey_width);
-					if (varl != NOVARIABLE)
-					{
-						strcpy(lenstr, describesimplevariable(varl));
-						ltype = varl->type;
-						TDCOPY(ldescript, varl->textdescript);
-					}
-					if (varw != NOVARIABLE)
-					{
-						strcpy(widstr, describesimplevariable(varw));
-						wtype = varw->type;
-						TDCOPY(wdescript, varw->textdescript);
-					}
-					if (varl != NOVARIABLE)
-					{
-						getsimpletype(lenstr, &type, &addr, 0);
-						type |= (ltype & ~VTYPE);
-						var = setvalkey((INTBIG)ni, VNODEINST, el_attrkey_width, addr, type);
-						if (var != NOVARIABLE)
-							TDCOPY(var->textdescript, ldescript);
-					}
-					if (varw != NOVARIABLE)
-					{
-						getsimpletype(widstr, &type, &addr, 0);
-						type |= (wtype & ~VTYPE);
-						var = setvalkey((INTBIG)ni, VNODEINST, el_attrkey_length, addr, type);
-						if (var != NOVARIABLE)
-							TDCOPY(var->textdescript, wdescript);
-					}
-				}
-			}
-		}
-		ttyputmsg("Lengths and Widths swapped: redisplay to see them");
+		char *popups[3];
+		us_testmodelessdia = DiaInitDialogModeless(&us_testdialog, us_modelessdialoghitroutine);
+		if (us_testmodelessdia == 0) return;
+		DiaInitTextDialog(us_testmodelessdia, 7, DiaNullDlogList, DiaNullDlogItem,
+			DiaNullDlogDone, -1, SCSELMOUSE|SCREPORT);
+		DiaStuffLine(us_testmodelessdia, 7, "First Line");
+		DiaStuffLine(us_testmodelessdia, 7, "Second Line");
+		DiaStuffLine(us_testmodelessdia, 7, "Third Line");
+		DiaStuffLine(us_testmodelessdia, 7, "Fourth Line");
+		DiaStuffLine(us_testmodelessdia, 7, "Fifth Line");
+		DiaStuffLine(us_testmodelessdia, 7, "Sixth Line");
+		DiaStuffLine(us_testmodelessdia, 7, "Seventh Line");
+		DiaStuffLine(us_testmodelessdia, 7, "Eighth Line");
+		DiaStuffLine(us_testmodelessdia, 7, "Ninth Line");
+		DiaStuffLine(us_testmodelessdia, 7, "Tenth Line");
+		DiaSelectLine(us_testmodelessdia, 7, -1);
+		DiaSetText(us_testmodelessdia, 9, "Initial edit");
+		popups[0] = "First popup";
+		popups[1] = "Second popup";
+		popups[2] = "Third popup";
+		DiaSetPopup(us_testmodelessdia, 11, 3, popups);
+		DiaSetControl(us_testmodelessdia, 4, 1);
 		return;
 	}
 #endif
 
 #if 0		/* place 1 million parts */
+#  define MILLIONSIZE   1000			/* square this to get number of nodes */
 	if (namesamen(par[0], "million", l) == 0)
 	{
-#include "edialogs.h"
 		NODEPROTO *item;
-		extern TOOL *dr_tool;
+		extern TOOL *dr_tool, *ro_tool;
 		REGISTER ARCPROTO *apwire;
 		REGISTER PORTPROTO *ppy, *ppa;
 		float duration;
 		NODEINST *nilast;
+		REGISTER void *dia;
 		INTBIG lx, hx, ly, hy, wid, hei, maxy, maxx, nilastx, nilasty, nix, niy;
-		extern DIALOG us_progressdialog;
 
 		toolturnoff(dr_tool, FALSE);
+		toolturnoff(ro_tool, FALSE);
 		toolturnoff(us_tool, FALSE);
 		apwire = getarcproto("schematic:wire");
-		item = getnodeproto("schematic:Buffer");
+		item = getnodeproto("schematic:Off-Page");
 		if (item == NONODEPROTO) return;
 		ppy = getportproto(item, "y");
 		ppa = getportproto(item, "a");
 		wid = item->highx - item->lowx;
 		hei = item->highy - item->lowy;
 		np = newnodeproto("million", el_curlib);
-		(void)DiaInitDialog(&us_progressdialog);
-		maxy = 1000;    maxx = 1000;
+		dia = DiaInitProgress("Building...");
+		if (dia == 0) return;
+		maxy = MILLIONSIZE;    maxx = MILLIONSIZE;
 		starttimer();
 		for(i=0; i<maxy; i++)
 		{
-			DiaPercent(1, i*100/maxy);
+			DiaSetProgress(dia, i, maxy);
 			nilast = NONODEINST;
 			for(j=0; j<maxx; j++)
 			{
-				lx = wid*j;   hx = lx + wid;
-				ly = hei*i;   hy = ly + hei;
+				lx = wid*j*2;   hx = lx + wid;
+				ly = hei*i*2;   hy = ly + hei;
 				ni = newnodeinst(item, lx, hx, ly, hy, 0, 0, np);
 				if (nilast != NONODEINST)
 				{
@@ -2049,39 +2059,13 @@ void us_debug(INTBIG count, char *par[])
 		}
 		duration = endtimer();
 		ttyputmsg("Took %s", explainduration(duration));
-		DiaDoneDialog();
-		toolturnon(us_tool, FALSE);
+		DiaDoneProgress(dia);
+		toolturnon(us_tool);
+		toolturnon(ro_tool);
 		return;
 	}
 #endif
 
-	if (namesamen(par[0], "convert-dashes", l) == 0)
-	{
-		i = 0;
-		for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
-		{
-			for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
-			{
-				for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-				{
-					for(pt = pp->protoname; *pt != 0; pt++)
-						if (*pt == '-') break;
-					if (*pt != 0)
-					{
-						i++;
-						strcpy(line, pp->protoname);
-						for(pt = line; *pt != 0; pt++)
-							if (*pt == '-') *pt = '_';
-						pt = us_uniqueportname(line, np);
-						(void)reallocstring(&pp->protoname, pt, us_tool->cluster);
-					}
-				}
-			}
-		}
-		if (i == 0) ttyputmsg("No conversion necessary"); else
-			ttyputmsg("Converted %ld port names", i);
-		return;
-	}
 #ifdef DEBUGMERGE
 	if (namesamen(par[0], "merge", l) == 0)
 	{
@@ -2144,6 +2128,29 @@ void us_debug(INTBIG count, char *par[])
 	}
 #endif
 
+#if 0		/* don't need this: done automatically on library input */
+	if (namesamen(par[0], "make-parameters-visible", l) == 0)
+	{
+		j = 0;
+		for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		{
+			/* look at all parameters on the instance */
+			for(i=0; i<np->numvar; i++)
+			{
+				var = &np->firstvar[i];
+				if (TDGETISPARAM(var->textdescript) == 0) continue;
+				if (TDGETINTERIOR(var->textdescript) == 0) continue;
+				TDSETINTERIOR(var->textdescript, 0);
+				j++;
+			}
+		}
+		ttyputmsg("Cleared 'visible only inside facet' setting on %d parameters in library %s",
+			j, el_curlib->libname);
+		if (j > 0) el_curlib->userbits |= LIBCHANGEDMINOR;
+		return;
+	}
+#endif
+
 	if (namesamen(par[0], "prepare-tsmc-io", l) == 0 && l >= 2)
 	{
 		INTBIG total;
@@ -2311,63 +2318,6 @@ void us_debug(INTBIG count, char *par[])
 		return;
 	}
 
-	if (namesamen(par[0], "out-of-bounds", l) == 0)
-	{
-		np = us_needfacet();
-		if (np == NONODEPROTO) return;
-		i = 0;
-		for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
-		{
-			if (ni->geom->lowx > -MAXINTBIG/2 && ni->geom->highx < MAXINTBIG/2 &&
-				ni->geom->lowy > -MAXINTBIG/2 && ni->geom->highy < MAXINTBIG/2) continue;
-			if (i == 0) us_clearhighlightcount();
-			newhigh.status = HIGHFROM;
-			newhigh.facet = np;
-			newhigh.fromgeom = ni->geom;
-			newhigh.fromport = NOPORTPROTO;
-			newhigh.frompoint = 0;
-			us_addhighlight(&newhigh);
-			i++;
-		}
-		for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
-		{
-			if (ai->geom->lowx > -MAXINTBIG/2 && ai->geom->highx < MAXINTBIG/2 &&
-				ai->geom->lowy > -MAXINTBIG/2 && ai->geom->highy < MAXINTBIG/2) continue;
-			newhigh.status = HIGHFROM;
-			newhigh.facet = np;
-			newhigh.fromgeom = ai->geom;
-			newhigh.fromport = NOPORTPROTO;
-			newhigh.frompoint = 0;
-			us_addhighlight(&newhigh);
-			i++;
-		}
-		if (i == 0) ttyputmsg("No objects are out of bounds"); else
-			ttyputmsg("%ld objects are out of bounds", i);
-		return;
-	}
-
-	if (namesamen(par[0], "kill-nodenames", l) == 0)
-	{
-		i = 0;
-		for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
-		{
-			for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
-			{
-				for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
-				{
-					var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, el_node_name_key);
-					if (var == NOVARIABLE) continue;
-					if ((var->type&VDISPLAY) != 0) continue;
-					(void)delvalkey((INTBIG)ni, VNODEINST, el_node_name_key);
-					i++;
-				}
-			}
-		}
-		if (i == 0) ttyputmsg("No node names removed"); else
-			ttyputmsg("Removed %ld node names", i);
-		return;
-	}
-
 	/********** THESE ARE STANDARD **********/
 
 	if (namesamen(par[0], "arena", l) == 0 && l >= 1)
@@ -2459,7 +2409,7 @@ void us_debug(INTBIG count, char *par[])
 
 	if (namesamen(par[0], "options-changed", l) == 0 && l >= 1)
 	{
-		explainoptionchanges(-1);
+		explainoptionchanges(-1, 0);
 		return;
 	}
 
@@ -2553,8 +2503,9 @@ void us_defarc(INTBIG count, char *par[])
 		if ((style&WANTNOEXTEND) != 0) addstringtoinfstr(infstr, _("not ends-extended")); else
 			addstringtoinfstr(infstr, _("ends-extended"));
 		if (ap != NOARCPROTO)
-			formatinfstr(infstr, _(", width %s, angle %ld"), latoa(defaultarcwidth(ap)-arcprotowidthoffset(ap)),
-				(ap->userbits&AANGLEINC) >> AANGLEINCSH);
+			formatinfstr(infstr, _(", width %s, angle %ld"),
+				latoa(defaultarcwidth(ap)-arcprotowidthoffset(ap), 0),
+					(ap->userbits&AANGLEINC) >> AANGLEINCSH);
 		ttyputmsg("%s", returninfstr(infstr));
 		return;
 	}
@@ -2735,7 +2686,7 @@ void us_defarc(INTBIG count, char *par[])
 			}
 		}
 		pt = par[2];
-		i = atola(pt);
+		i = atola(pt, 0);
 		if (i&1) i++;
 		if (i < 0)
 		{
@@ -2745,7 +2696,7 @@ void us_defarc(INTBIG count, char *par[])
 		wid = arcprotowidthoffset(ap) + i;
 		wid = wid * WHOLE / el_curlib->lambda[ap->tech->techindex];
 		(void)setvalkey((INTBIG)ap, VARCPROTO, el_arc_width_default_key, wid, VINTEGER);
-		ttyputverbose(M_("%s arcs default to %s wide"), aname, latoa(i));
+		ttyputverbose(M_("%s arcs default to %s wide"), aname, latoa(i, 0));
 		return;
 	}
 
@@ -2994,7 +2945,7 @@ void us_defnode(INTBIG count, char *par[])
 			/* set alterability of facets */
 			if (!negated)
 			{
-				if ((np->userbits&NLOCKED) == 0)
+				if ((np->userbits&NPLOCKED) == 0)
 				{
 					ttyputverbose(M_("Contents of facet %s are already able to be freely altered"),
 						describenodeproto(np));
@@ -3004,10 +2955,10 @@ void us_defnode(INTBIG count, char *par[])
 						describenodeproto(np));
 				}
 				(void)setval((INTBIG)np, VNODEPROTO, "userbits",
-					np->userbits & ~NLOCKED, VINTEGER);
+					np->userbits & ~NPLOCKED, VINTEGER);
 			} else
 			{
-				if ((np->userbits&NLOCKED) != 0)
+				if ((np->userbits&NPLOCKED) != 0)
 				{
 					ttyputverbose(M_("Contents of facet %s are already locked"),
 						describenodeproto(np));
@@ -3017,7 +2968,7 @@ void us_defnode(INTBIG count, char *par[])
 						describenodeproto(np));
 				}
 				(void)setval((INTBIG)np, VNODEPROTO, "userbits",
-					np->userbits | NLOCKED, VINTEGER);
+					np->userbits | NPLOCKED, VINTEGER);
 			}
 		} else
 		{
@@ -3037,7 +2988,7 @@ void us_defnode(INTBIG count, char *par[])
 		/* set alterability of facet instances */
 		if (negated)
 		{
-			if ((np->userbits&NILOCKED) == 0)
+			if ((np->userbits&NPILOCKED) == 0)
 			{
 				ttyputverbose(M_("Instances in facet %s are already able to be freely altered"),
 					describenodeproto(np));
@@ -3047,10 +2998,10 @@ void us_defnode(INTBIG count, char *par[])
 					describenodeproto(np));
 			}
 			(void)setval((INTBIG)np, VNODEPROTO, "userbits",
-				np->userbits & ~NILOCKED, VINTEGER);
+				np->userbits & ~NPILOCKED, VINTEGER);
 		} else
 		{
-			if ((np->userbits&NILOCKED) != 0)
+			if ((np->userbits&NPILOCKED) != 0)
 			{
 				ttyputverbose(M_("Instances in facet %s are already locked"),
 					describenodeproto(np));
@@ -3060,7 +3011,7 @@ void us_defnode(INTBIG count, char *par[])
 					describenodeproto(np));
 			}
 			(void)setval((INTBIG)np, VNODEPROTO, "userbits",
-				np->userbits | NILOCKED, VINTEGER);
+				np->userbits | NPILOCKED, VINTEGER);
 		}
 		return;
 	}
@@ -3140,7 +3091,7 @@ void us_defnode(INTBIG count, char *par[])
 			}
 		}
 		pp = par[2];
-		i = atola(pp);
+		i = atola(pp, 0);
 		if (i&1) i++;
 		if (count <= 3)
 		{
@@ -3152,7 +3103,7 @@ void us_defnode(INTBIG count, char *par[])
 			}
 		}
 		pp = par[3];
-		j = atola(pp);
+		j = atola(pp, 0);
 		if (j&1) j++;
 		if (i < 0 || j < 0)
 		{
@@ -3167,7 +3118,7 @@ void us_defnode(INTBIG count, char *par[])
 		(void)setvalkey((INTBIG)np, VNODEPROTO, el_node_size_default_key,
 			(INTBIG)nodesize, VINTEGER|VISARRAY|(2<<VLENGTHSH));
 		ttyputverbose(M_("%s nodes will be created %sx%s in size"), describenodeproto(np),
-			latoa(i), latoa(j));
+			latoa(i, 0), latoa(j, 0));
 		return;
 	}
 
@@ -3226,7 +3177,7 @@ void us_duplicate(INTBIG count, char *par[])
 
 	if ((us_useroptions&NOMOVEAFTERDUP) != 0) interactiveplace = FALSE; else
 		interactiveplace = TRUE;
-	us_copylisttofacet(list, np, np, TRUE, interactiveplace);
+	us_copylisttofacet(list, np, np, TRUE, interactiveplace, TRUE);
 }
 
 #if 0		/* test code for John Wood */
