@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usrcomek.c
@@ -89,7 +90,7 @@ void us_editfacet(INTBIG count, char *par[])
 	REGISTER WINDOWPART *win;
 	REGISTER char *pt;
 	NODEINST *hini;
-	XARRAY rotarray, transarray, xfarray, prevarray;
+	XARRAY rotarray, transarray, xfarray, newarray;
 	PORTPROTO *hipp;
 	REGISTER void *infstr;
 
@@ -108,6 +109,11 @@ void us_editfacet(INTBIG count, char *par[])
 		{
 			if (namesamen(par[0], "in-place", len) == 0)
 			{
+				if (el_curwindowpart->inplacedepth >= MAXINPLACEDEPTH)
+				{
+					ttyputerr(_("Can only go down %ld levels in-place"), MAXINPLACEDEPTH);
+					return;
+				}
 				inplace = 1;
 				count--;
 			}
@@ -206,8 +212,7 @@ void us_editfacet(INTBIG count, char *par[])
 		push = 1;
 		onp = contentsview(np);
 		if (onp == NONODEPROTO || intoicon != 0) onp = np;
-		(void)setvalkey((INTBIG)onp, VNODEPROTO, us_descent_path_key, (INTBIG)stacknodeinst,
-			VNODEINST|VDONTSAVE);
+		sethierarchicalparent(onp, stacknodeinst);
 
 		/* save the current view */
 		infstr = initinfstr();
@@ -303,22 +308,34 @@ void us_editfacet(INTBIG count, char *par[])
 		makerotI(stacknodeinst, rotarray);
 		maketransI(stacknodeinst, transarray);
 		transmult(rotarray, transarray, xfarray);
-		transcpy(el_curwindowpart->intofacet, prevarray);
-		transmult(prevarray, xfarray, el_curwindowpart->intofacet);
+		transmult(el_curwindowpart->intofacet, xfarray, newarray);
+		us_setxarray((INTBIG)el_curwindowpart, VWINDOWPART, "intofacet", newarray);
 
 		makerot(stacknodeinst, rotarray);
 		maketrans(stacknodeinst, transarray);
 		transmult(transarray, rotarray, xfarray);
-		transcpy(el_curwindowpart->outoffacet, prevarray);
-		transmult(xfarray, prevarray, el_curwindowpart->outoffacet);
-		el_curwindowpart->state |= INPLACEEDIT;
+		transmult(xfarray, el_curwindowpart->outoffacet, newarray);
+		us_setxarray((INTBIG)el_curwindowpart, VWINDOWPART, "outoffacet", newarray);
+		if ((el_curwindowpart->state&INPLACEEDIT) == 0)
+		{
+			(void)setval((INTBIG)el_curwindowpart, VWINDOWPART, "inplacedepth", 0, VINTEGER);
+			(void)setval((INTBIG)el_curwindowpart, VWINDOWPART, "topnodeproto",
+				(INTBIG)el_curwindowpart->curnodeproto, VNODEPROTO);
+		}
+		(void)setind((INTBIG)el_curwindowpart, VWINDOWPART, "inplacestack",
+			el_curwindowpart->inplacedepth, (INTBIG)stacknodeinst);
+		(void)setval((INTBIG)el_curwindowpart, VWINDOWPART, "inplacedepth",
+			el_curwindowpart->inplacedepth+1, VINTEGER);
+		(void)setval((INTBIG)el_curwindowpart, VWINDOWPART, "state",
+			el_curwindowpart->state | INPLACEEDIT, VINTEGER);
 	} else
 	{
 		if (el_curwindowpart != NOWINDOWPART)
 		{
-			transid(el_curwindowpart->intofacet);
-			transid(el_curwindowpart->outoffacet);
-			el_curwindowpart->state &= ~INPLACEEDIT;
+			us_setxarray((INTBIG)el_curwindowpart, VWINDOWPART, "intofacet", el_matid);
+			us_setxarray((INTBIG)el_curwindowpart, VWINDOWPART, "outoffacet", el_matid);
+			(void)setval((INTBIG)el_curwindowpart, VWINDOWPART, "state",
+				el_curwindowpart->state & ~INPLACEEDIT, VINTEGER);
 		}
 	}
 	us_switchtofacet(np, lx, hx, ly, hy, hini, hipp, newframe, push);
@@ -435,7 +452,7 @@ void us_erase(INTBIG count, char *par[])
 		if (list[i]->entryisnode)
 		{
 			ni = list[i]->entryaddr.ni;
-			if (us_cantedit(np, ni->proto, TRUE)) continue;
+			if (us_cantedit(np, ni, TRUE)) continue;
 			foundnode = TRUE;
 		}
 		list[j++] = list[i];
@@ -450,7 +467,7 @@ void us_erase(INTBIG count, char *par[])
 	if (!foundnode)
 	{
 		/* disallow erasing if lock is on */
-		if (us_cantedit(np, NONODEPROTO, TRUE)) return;
+		if (us_cantedit(np, NONODEINST, TRUE)) return;
 	}
 
 	/* unhighlight */
@@ -480,7 +497,7 @@ void us_erase(INTBIG count, char *par[])
 		np = high.facet;
 		if (np != NONODEPROTO)
 		{
-			if (us_cantedit(np, NONODEPROTO, TRUE)) continue;
+			if (us_cantedit(np, NONODEINST, TRUE)) continue;
 		}
 
 		/* do not deal with text on an object if the object is already in the list */
@@ -2038,7 +2055,8 @@ void us_grid(INTBIG count, char *par[])
 	if (count == 0)
 	{
 		if (us_needwindow()) return;
-		if ((el_curwindowpart->state&WINDOWTYPE) != DISPWINDOW)
+		if ((el_curwindowpart->state&WINDOWTYPE) != DISPWINDOW &&
+		    (el_curwindowpart->state&WINDOWTYPE) != WAVEFORMWINDOW)
 		{
 			us_abortcommand(_("Cannot show grid in this type of window"));
 			return;
@@ -2098,11 +2116,11 @@ void us_grid(INTBIG count, char *par[])
 			ttyputusage("grid size X [Y]");
 			return;
 		}
-		i = atola(par[1]);
+		i = atola(par[1], 0);
 		if (i&1) i++;
 		if (count >= 3)
 		{
-			j = atola(par[2]);
+			j = atola(par[2], 0);
 			if (j&1) j++;
 		} else j = i;
 		if (i <= 0 || j <= 0)
@@ -2149,7 +2167,7 @@ void us_help(INTBIG count, char *par[])
 	INTBIG day, month, year, hour, minute, second;
 	FILE *in;
 	char line[256], *filename, *hd;
-	UINTBIG clock;
+	time_t clock;
 	REGISTER void *infstr;
 
 	if (count > 0) len = strlen(pp = par[0]);
@@ -2639,7 +2657,7 @@ void us_killfacet(INTBIG count, char *par[])
 	}
 
 	/* disallow killing if lock is on */
-	if (us_cantedit(np, NONODEPROTO, TRUE)) return;
+	if (us_cantedit(np, NONODEINST, TRUE)) return;
 
 	/* if there are still instances of the facet, mention them */
 	if (np->firstinst != NONODEINST)
