@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usrcomln.c
@@ -68,11 +69,11 @@ void us_lambda(INTBIG count, char *par[])
 				if (namesamen(pp, "lambda", l) == 0) newunit = DISPUNITLAMBDA; else
 				if (namesamen(pp, "inch", l) == 0) newunit = DISPUNITINCH; else
 				if (namesamen(pp, "centimeter", l) == 0 && l >= 7) newunit = DISPUNITCM; else
-				if (namesamen(pp, "millimeter", l) == 0 && l >= 7) newunit = DISPUNITMM; else
+				if (namesamen(pp, "millimeter", l) == 0 && l >= 4) newunit = DISPUNITMM; else
 				if (namesamen(pp, "mil", l) == 0 && l >= 3) newunit = DISPUNITMIL; else
 				if (namesamen(pp, "micron", l) == 0 && l >= 3) newunit = DISPUNITMIC; else
 				if (namesamen(pp, "centimicron", l) == 0 && l >= 7) newunit = DISPUNITCMIC; else
-				if (namesamen(pp, "millimicron", l) == 0 && l >= 7) newunit = DISPUNITMMIC; else
+				if (namesamen(pp, "nanometer", l) == 0) newunit = DISPUNITMMIC; else
 				{
 					us_abortcommand(_("Unknown display unit: %s"), pp);
 					return;
@@ -104,7 +105,7 @@ void us_lambda(INTBIG count, char *par[])
 					ttyputverbose(M_("Distance expressed in centimicrons"));
 					break;
 				case DISPUNITMMIC:
-					ttyputverbose(M_("Distance expressed in millimicrons"));
+					ttyputverbose(M_("Distance expressed in nanometers"));
 					break;
 			}
 			return;
@@ -119,7 +120,7 @@ void us_lambda(INTBIG count, char *par[])
 				if (namesamen(pp, "half-decimicron", l) == 0 && l >= 6)
 				{
 					changeinternalunits(NOLIBRARY, el_units, INTUNITHDMIC);
-				} else if (namesamen(pp, "half-millimicron", l) == 0 && l >= 6)
+				} else if (namesamen(pp, "half-nanometer", l) == 0 && l >= 6)
 				{
 					changeinternalunits(NOLIBRARY, el_units, INTUNITHMMIC);
 				} else
@@ -204,14 +205,13 @@ void us_lambda(INTBIG count, char *par[])
 	{
 		if (tech->temp1 == 0) continue;
 		ttyputverbose(M_("Lambda size for %s is %s"), tech->techname,
-			latoa(el_curlib->lambda[tech->techindex]));
+			latoa(el_curlib->lambda[tech->techindex], 0));
 	}
 }
 
 void us_library(INTBIG count, char *par[])
 {
-	REGISTER LIBRARY *lib, *olib, *lastlib, *mergelib, **libsave, *deplib,
-		*oldlib;
+	REGISTER LIBRARY *lib, *olib, *lastlib, *mergelib, *oldlib;
 	REGISTER INTBIG i, l, makecurrent, multifile, found, total, stripopts,
 		retval, arg;
 	REGISTER char *pp, *pt, *style, *libf, *oldlibfile, save;
@@ -250,6 +250,18 @@ void us_library(INTBIG count, char *par[])
 		return;
 	}
 
+	if (namesamen(pp, "touch", l) == 0 && l >= 1)
+	{
+		/* mark all libraries as "changed" */
+		for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		{
+			if ((lib->userbits&HIDDENLIBRARY) != 0) continue;
+			lib->userbits |= LIBCHANGEDMAJOR | LIBCHANGEDMINOR;
+		}
+		ttyputmsg(_("All libraries now need to be saved"));
+		return;
+	}
+
 	if (namesamen(pp, "purge", l) == 0 && l >= 1)
 	{
 		/* delete all facets that are not the most recent and are unused */
@@ -355,7 +367,7 @@ void us_library(INTBIG count, char *par[])
 			if (w->curnodeproto->cell->lib == lib) us_clearwindow(w);
 		}
 
-		ttyputmsg("Library %s deleted", lib->libname);
+		ttyputmsg("Library %s closed", lib->libname);
 
 		/* kill the library */
 		killlibrary(lib);
@@ -547,10 +559,13 @@ void us_library(INTBIG count, char *par[])
 					us_shadownodeproto(NOWINDOWFRAME, el_curtech->firstnodeproto);
 				}
 				np = el_curlib->curnodeproto;
-				if (el_curwindowpart->curnodeproto == NONODEPROTO) curw = el_curwindowpart; else
+				if (el_curwindowpart->curnodeproto == NONODEPROTO &&
+					(el_curwindowpart->state&WINDOWTYPE) != EXPLORERWINDOW)
+						curw = el_curwindowpart; else
 				{
 					for(curw = el_topwindowpart; curw != NOWINDOWPART; curw = curw->nextwindowpart)
-						if (curw->curnodeproto == NONODEPROTO) break;
+						if (curw->curnodeproto == NONODEPROTO &&
+							(curw->state&WINDOWTYPE) != EXPLORERWINDOW) break;
 				}
 				if (np != NONODEPROTO)
 				{
@@ -772,67 +787,10 @@ void us_library(INTBIG count, char *par[])
 
 	if (namesamen(pp, "save", l) == 0 && l >= 2)
 	{
-		/* see which libraries need to be saved */
-		total = 0;
-		for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
-		{
-			if ((lib->userbits&HIDDENLIBRARY) != 0) continue;
-			if ((lib->userbits&(LIBCHANGEDMAJOR | LIBCHANGEDMINOR)) == 0) continue;
-			total++;
-		}
-		if (total == 0) return;
-		libsave = (LIBRARY **)emalloc(total * (sizeof (LIBRARY *)), el_tempcluster);
-		if (libsave == 0) return;
+		us_saveeverything();
 
-		/* make a list of the libraries to save */
-		total = 0;
-		for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
-		{
-			if ((lib->userbits&HIDDENLIBRARY) != 0) continue;
-			if ((lib->userbits&(LIBCHANGEDMAJOR | LIBCHANGEDMINOR)) == 0) continue;
-			libsave[total++] = lib;
-		}
-
-		/* be sure the most referenced libraries are saved first */
-		for(i=0; i<total; i++) libsave[i]->temp1 = i;
-		for(i=0; i<total; i++)
-		{
-			lib = libsave[i];
-			for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
-			{
-				for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
-				{
-					if (ni->proto->primindex != 0) continue;
-					deplib = ni->proto->cell->lib;
-					if (deplib == lib) continue;
-					if (deplib->temp1 <= lib->temp1) continue;
-					l = deplib->temp1;   deplib->temp1 = lib->temp1;   lib->temp1 = l;
-				}
-			}
-		}
-		esort(libsave, total, sizeof (LIBRARY *), us_librarytemp1ascending);
-
-		/* now save every library that needs it */
-		for(i=0; i<total; i++)
-		{
-			lib = libsave[i];
-
-			/* save the library in binary format */
-			makeoptionstemporary(lib);
-			retval = asktool(io_tool, "write", (INTBIG)lib, (INTBIG)"binary");
-			restoreoptionstate(lib);
-			if (retval != 0) break;
-		}
-
-		/* clean up */
-		efree((char *)libsave);
-
-		/* rewind session logging if it is on */
-		if (us_logrecord != NULL)
-		{
-			logfinishrecord();
-			logstartrecord();
-		}
+		/* manage reset of session log (if all information is saved) */
+		us_continuesessionlogging();
 		return;
 	}
 
@@ -887,13 +845,6 @@ void us_library(INTBIG count, char *par[])
 		if (stripopts != 0) restoreoptionstate(lib);
 		if (retval != 0) return;
 
-		/* rewind session recording if writing in permanent format */
-		if ((*style == 'b' || *style == 't') && us_logrecord != NULL)
-		{
-			logfinishrecord();
-			logstartrecord();
-		}
-
 		/* offer to save any referenced libraries */
 		if (namesame(style, "binary") == 0)
 		{
@@ -929,6 +880,13 @@ void us_library(INTBIG count, char *par[])
 				restoreoptionstate(olib);
 			}
 		}
+
+		/* rewind session recording if writing in permanent format */
+		if (*style == 'b' || *style == 't')
+		{
+			/* manage reset of session log (if all information is saved) */
+			us_continuesessionlogging();
+		}
 		return;
 	}
 
@@ -1066,9 +1024,7 @@ void us_menu(INTBIG count, char *par[])
 	REGISTER INTBIG l, s, i, total, changed, position, x, y, varkey,
 		j, len, arctot, pintot, comptot, puretot, fun, coms;
 	REGISTER BOOLEAN autoset, verbose;
-	INTBIG wid, hei;
 	BOOLEAN butstate;
-	UINTBIG descript[TEXTDESCRIPTSIZE];
 	REGISTER char *pp, **temp;
 	char number[10], *pt, *implicit[6];
 	POPUPMENU *pm, *cpm, *newpm;
@@ -1491,40 +1447,8 @@ void us_menu(INTBIG count, char *par[])
 			us_validatemenu(pm);
 		}
 
-		/* see if platform has native menu handling */
-		if (nativemenuload(count-1, &par[1]))
-		{
-			/* no native menus: do them by hand */
-			if (count > 1)
-			{
-				if (us_needwindow()) return;
-
-				/* turn on the menu bar with these popups */
-				us_menubarsize = MENUBARHEIGHT;
-
-				/* load the information */
-				TDCLEAR(descript);
-				TDSETSIZE(descript, TXTMENU);
-				screensettextinfo(el_curwindowpart, NOTECHNOLOGY, descript);
-				x = 0;
-				for(i=1; i<count; i++)
-				{
-					pm = us_pulldowns[i-1];
-					screengettextsize(el_curwindowpart, us_stripampersand(pm->header), &wid, &hei);
-					x += wid + 12;
-					us_pulldownmenupos[i-1] = x;
-				}
-
-				/* display the pulldown menu */
-				us_drawmenu(0, NOWINDOWFRAME);
-			} else
-			{
-				/* turn off the menu bar */
-				if (us_menubarsize == 0) return;
-				us_menubarsize = 0;
-				us_drawmenu(0, NOWINDOWFRAME);
-			}
-		}
+		/* always use native menu handling */
+		nativemenuload(count-1, &par[1]);
 		return;
 	}
 
@@ -1655,11 +1579,11 @@ void us_mirror(INTBIG count, char *par[])
 		if (geom->entryisnode)
 		{
 			ni = geom->entryaddr.ni;
-			if (us_cantedit(np, ni->proto, TRUE)) continue;
+			if (us_cantedit(np, ni, TRUE)) continue;
 			ni->temp1 = 1;
 		} else
 		{
-			if (us_cantedit(np, NONODEPROTO, TRUE)) return;
+			if (us_cantedit(np, NONODEINST, TRUE)) return;
 			ai = geom->entryaddr.ai;
 			ai->end[0].nodeinst->temp1 = 1;
 			ai->end[1].nodeinst->temp1 = 1;
@@ -1933,7 +1857,7 @@ static DIALOGITEM us_movebydialogitems[] =
  /*  5 */ {0, {32,48,48,142}, EDITTEXT, ""},
  /*  6 */ {0, {32,20,48,42}, MESSAGE, N_("dY:")}
 };
-static DIALOG us_movebydialog = {{50,75,147,244}, N_("Move By Amount"), 0, 6, us_movebydialogitems};
+static DIALOG us_movebydialog = {{50,75,147,244}, N_("Move By Amount"), 0, 6, us_movebydialogitems, 0, 0};
 
 /* special items for the "move by" dialog: */
 #define DMVB_XBY       3		/* X change (edit text) */
@@ -1958,6 +1882,7 @@ void us_move(INTBIG count, char *par[])
 	char **textlist;
 	static POLYGON *poly = NOPOLYGON;
 	REGISTER VARIABLE *var;
+	REGISTER void *dia;
 
 	/* get polygon */
 	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
@@ -1986,7 +1911,7 @@ void us_move(INTBIG count, char *par[])
 	for(total=0, ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
 		if (ni->temp1 == 0) continue;
-		if (us_cantedit(np, ni->proto, TRUE)) return;
+		if (us_cantedit(np, ni, TRUE)) return;
 		total++;
 	}
 	if (total == 0 && numtexts == 0) return;
@@ -2132,8 +2057,8 @@ void us_move(INTBIG count, char *par[])
 			efree((char *)nodelist);
 			return;
 		}
-		xcur = atola(par[1]);
-		ycur = atola(par[2]);
+		xcur = atola(par[1], 0);
+		ycur = atola(par[2], 0);
 
 		/* make the move if it is valid */
 		if (xcur == bestlx && ycur == bestly) ttyputverbose(M_("Null motion")); else
@@ -2155,19 +2080,20 @@ void us_move(INTBIG count, char *par[])
 		if (count == 2 && namesamen(par[1], "dialog", strlen(par[1])) == 0)
 		{
 			/* get coordinates from dialog */
-			if (DiaInitDialog(&us_movebydialog)) return;
-			DiaSetText(DMVB_XBY, frtoa(movebyx));
-			DiaSetText(DMVB_YBY, frtoa(movebyy));
+			dia = DiaInitDialog(&us_movebydialog);
+			if (dia == 0) return;
+			DiaSetText(dia, DMVB_XBY, frtoa(movebyx));
+			DiaSetText(dia, DMVB_YBY, frtoa(movebyy));
 			for(;;)
 			{
-				itemHit = DiaNextHit();
+				itemHit = DiaNextHit(dia);
 				if (itemHit == CANCEL || itemHit == OK) break;
 			}
-			movebyx = atofr(DiaGetText(DMVB_XBY));
-			movebyy = atofr(DiaGetText(DMVB_YBY));
-			xcur = atola(DiaGetText(DMVB_XBY));
-			ycur = atola(DiaGetText(DMVB_YBY));
-			DiaDoneDialog();
+			movebyx = atofr(DiaGetText(dia, DMVB_XBY));
+			movebyy = atofr(DiaGetText(dia, DMVB_YBY));
+			xcur = atola(DiaGetText(dia, DMVB_XBY), 0);
+			ycur = atola(DiaGetText(dia, DMVB_YBY), 0);
+			DiaDoneDialog(dia);
 			if (itemHit == CANCEL) return;
 		} else
 		{
@@ -2178,8 +2104,8 @@ void us_move(INTBIG count, char *par[])
 				efree((char *)nodelist);
 				return;
 			}
-			xcur = atola(par[1]);
-			ycur = atola(par[2]);
+			xcur = atola(par[1], 0);
+			ycur = atola(par[2], 0);
 		}
 
 		/* make the move if it is valid */
@@ -2188,7 +2114,7 @@ void us_move(INTBIG count, char *par[])
 			us_pushhighlight();
 			us_clearhighlightcount();
 			us_manymove(list, nodelist, total, xcur, ycur);
-			us_moveselectedtext(numtexts, textlist, list, xcur-bestlx, ycur-bestly);
+			us_moveselectedtext(numtexts, textlist, list, xcur, ycur);
 			us_pophighlight(TRUE);
 		}
 		efree((char *)nodelist);
@@ -2254,7 +2180,7 @@ void us_move(INTBIG count, char *par[])
 		{
 			l = el_units;
 			el_units = (el_units & ~DISPLAYUNITS) | DISPUNITLAMBDA;
-			amt = atola(par[1]);
+			amt = atola(par[1], 0);
 			el_units = l;
 		}
 		dy += amt;
@@ -2264,7 +2190,7 @@ void us_move(INTBIG count, char *par[])
 		{
 			l = el_units;
 			el_units = (el_units & ~DISPLAYUNITS) | DISPUNITLAMBDA;
-			amt = atola(par[1]);
+			amt = atola(par[1], 0);
 			el_units = l;
 		}
 		dy -= amt;
@@ -2274,7 +2200,7 @@ void us_move(INTBIG count, char *par[])
 		{
 			l = el_units;
 			el_units = (el_units & ~DISPLAYUNITS) | DISPUNITLAMBDA;
-			amt = atola(par[1]);
+			amt = atola(par[1], 0);
 			el_units = l;
 		}
 		dx -= amt;
@@ -2284,7 +2210,7 @@ void us_move(INTBIG count, char *par[])
 		{
 			l = el_units;
 			el_units = (el_units & ~DISPLAYUNITS) | DISPUNITLAMBDA;
-			amt = atola(par[1]);
+			amt = atola(par[1], 0);
 			el_units = l;
 		}
 		dx += amt;
@@ -2463,7 +2389,7 @@ void us_node(INTBIG count, char *par[])
 	}
 
 	/* everything after this point is structural and must check for locks */
-	if (us_cantedit(np, NONODEPROTO, TRUE)) return;
+	if (us_cantedit(np, NONODEINST, TRUE)) return;
 
 	if (namesamen(pt, "cover-implant", l) == 0 && l >= 1)
 	{
@@ -2513,8 +2439,8 @@ void us_node(INTBIG count, char *par[])
 				ttyputusage("node trace construct-annulus INSIDE OUTSIDE [RESOLUTION] [DEGREES]");
 				return;
 			}
-			inside = atola(par[2]);
-			outside = atola(par[3]);
+			inside = atola(par[2], 0);
+			outside = atola(par[3], 0);
 			segs = 16;
 			if (count >= 5) segs = myatoi(par[4]);
 			if (segs < 4) segs = 4;
@@ -2587,8 +2513,8 @@ void us_node(INTBIG count, char *par[])
 				whichpoint = high->frompoint;
 				if (count > i+2 && isanumber(par[i+1]) && isanumber(par[i+2]))
 				{
-					xcur = atola(par[i+1]);
-					ycur = atola(par[i+2]);
+					xcur = atola(par[i+1], 0);
+					ycur = atola(par[i+2], 0);
 					i += 2;
 					pointgiven++;
 				}
@@ -2601,8 +2527,8 @@ void us_node(INTBIG count, char *par[])
 				whichpoint = high->frompoint;
 				if (count > i+2 && isanumber(par[i+1]) && isanumber(par[i+2]))
 				{
-					xcur = atola(par[i+1]);
-					ycur = atola(par[i+2]);
+					xcur = atola(par[i+1], 0);
+					ycur = atola(par[i+2], 0);
 					i += 2;
 					pointgiven++;
 				}
