@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usrcomoq.c
@@ -46,24 +47,17 @@ void us_offtool(INTBIG count, char *par[])
 
 void us_ontool(INTBIG count, char *par[])
 {
-	REGISTER INTBIG catchup;
-
-	if (count == 2 && namesamen(par[1], "no-catch-up", strlen(par[1])) == 0)
-	{
-		count--;
-		catchup = 3;
-	} else catchup = 2;
-	us_settool(count, par, catchup);
+	us_settool(count, par, 2);
 }
 
 void us_outhier(INTBIG count, char *par[])
 {
-	REGISTER INTBIG len, found, total, i, haveview, vlx, vhx, vly, vhy, value;
-	INTBIG lx, hx, ly, hy;
+	REGISTER INTBIG len, total, i, haveview, vlx, vhx, vly, vhy, value;
+	INTBIG lx, hx, ly, hy, index;
 	REGISTER NODEPROTO *curnp, *hinp, *np;
 	REGISTER char *pt;
-	REGISTER BOOLEAN newwindow;
-	REGISTER NODEINST *ni, *hini;
+	REGISTER BOOLEAN newwindow, found;
+	REGISTER NODEINST *ni, *hini, *iconinst;
 	REGISTER PORTPROTO *pp, *curpp, *hipp;
 	REGISTER PORTEXPINST *pe;
 	REGISTER VARIABLE *var;
@@ -137,7 +131,7 @@ void us_outhier(INTBIG count, char *par[])
 	{
 		hinp = NONODEPROTO;
 
-		hini = descentparent(curnp);
+		hini = descentparent(curnp, &index);
 		if (hini != NONODEINST)
 		{
 			hinp = hini->parent;
@@ -188,13 +182,18 @@ void us_outhier(INTBIG count, char *par[])
 			if (np == NONODEPROTO) np = curnp;
 
 			/* see if there is any instance of that facet */
-			found = 0;
+			found = FALSE;
 			hinp = NONODEPROTO;
+			iconinst = NONODEINST;
 			for(ni = np->firstinst; ni != NONODEINST; ni = ni->nextinst)
 			{
-				if (ni->parent->cell == np->cell) continue;
+				if (ni->parent->cell == np->cell)
+				{
+					iconinst = ni;
+					continue;
+				}
 				if ((ni->parent->cell->lib->userbits&HIDDENLIBRARY) != 0) continue;
-				found = 1;
+				found = TRUE;
 				if (hinp != NONODEPROTO)
 				{
 					if (ni->parent == hinp) continue;
@@ -204,7 +203,13 @@ void us_outhier(INTBIG count, char *par[])
 				hinp = ni->parent;
 				hini = ni;
 			}
-			if (found == 0)
+			if (!found && iconinst != NONODEINST)
+			{
+				hini = iconinst;
+				hinp = iconinst->parent;
+				found = TRUE;
+			}
+			if (!found)
 			{
 				if (i == 0)
 				{
@@ -257,11 +262,19 @@ void us_outhier(INTBIG count, char *par[])
 	/* display the final location */
 	if (haveview != 0)
 	{
-		transcpy(into, el_curwindowpart->intofacet);
-		transcpy(outof, el_curwindowpart->outoffacet);
+		us_setxarray((INTBIG)el_curwindowpart, VWINDOWPART, "intofacet", into);
+		us_setxarray((INTBIG)el_curwindowpart, VWINDOWPART, "outoffacet", outof);
 		if (outof[2][0] == 0 && outof[2][1] == 0 && outof[2][2] == 0)
-			el_curwindowpart->state &= ~INPLACEEDIT; else
-				el_curwindowpart->state |= INPLACEEDIT;
+		{
+			(void)setval((INTBIG)el_curwindowpart, VWINDOWPART, "state",
+				el_curwindowpart->state & ~INPLACEEDIT, VINTEGER);
+		} else
+		{
+			(void)setval((INTBIG)el_curwindowpart, VWINDOWPART, "state",
+				el_curwindowpart->state | INPLACEEDIT, VINTEGER);
+			(void)setval((INTBIG)el_curwindowpart, VWINDOWPART, "inplacedepth",
+				el_curwindowpart->inplacedepth-1, VINTEGER);
+		}
 	}
 	us_switchtofacet(hinp, lx, hx, ly, hy, hini, hipp, newwindow, 1);
 }
@@ -581,7 +594,7 @@ void us_port(INTBIG count, char *par[])
 		}
 
 		/* disallow port action if lock is on */
-		if (us_cantedit(np, NONODEPROTO, TRUE)) return;
+		if (us_cantedit(np, NONODEINST, TRUE)) return;
 
 		reducehighlighting = 1;
 		if (count > 1)
@@ -644,7 +657,7 @@ void us_port(INTBIG count, char *par[])
 		if (np == NONODEPROTO) return;
 
 		/* disallow port action if lock is on */
-		if (us_cantedit(np, NONODEPROTO, TRUE)) return;
+		if (us_cantedit(np, NONODEINST, TRUE)) return;
 
 		/* save highlighting */
 		us_pushhighlight();
@@ -719,7 +732,7 @@ void us_port(INTBIG count, char *par[])
 		}
 
 		/* disallow port action if lock is on */
-		if (us_cantedit(np, NONODEPROTO, TRUE)) return;
+		if (us_cantedit(np, NONODEINST, TRUE)) return;
 
 		/* get list of nodes in the selected area */
 		g = us_gethighlighted(WANTNODEINST, 0, 0);
@@ -805,7 +818,7 @@ void us_port(INTBIG count, char *par[])
 		if (np == NONODEPROTO) return;
 
 		/* disallow port action if lock is on */
-		if (us_cantedit(np, NONODEPROTO, TRUE)) return;
+		if (us_cantedit(np, NONODEINST, TRUE)) return;
 
 		/* save highlighting */
 		us_pushhighlight();
@@ -968,7 +981,7 @@ void us_port(INTBIG count, char *par[])
 		}
 
 		/* disallow port action if lock is on */
-		if (us_cantedit(ni->parent, NONODEPROTO, TRUE)) return;
+		if (us_cantedit(ni->parent, NONODEINST, TRUE)) return;
 
 		/* get the specification details */
 		pp = us_portdetails(ppt, count-2, &par[2], ni, &bits, &mask, TRUE, "");
@@ -996,7 +1009,7 @@ void us_port(INTBIG count, char *par[])
 		if (ni == NONODEINST) return;
 
 		/* disallow port action if lock is on */
-		if (us_cantedit(ni->parent, NONODEPROTO, TRUE)) return;
+		if (us_cantedit(ni->parent, NONODEINST, TRUE)) return;
 
 		if (count <= 1)
 		{
@@ -1061,7 +1074,7 @@ void us_port(INTBIG count, char *par[])
 		}
 
 		/* disallow port action if lock is on */
-		if (us_cantedit(ni->parent, NONODEPROTO, TRUE)) return;
+		if (us_cantedit(ni->parent, NONODEINST, TRUE)) return;
 
 		/* get the specification details */
 		pp = us_portdetails(ppt, count-1, &par[1], ni, &bits, &mask, TRUE, "");
