@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usrcomtz.c
@@ -47,7 +48,7 @@ static struct
 	{"facet",      1,  &us_statusfacet},
 	{"grid",       1,  &us_statusgridsize},
 	{"lambda",     1,  &us_statuslambda},
-	{"network",    2,  &us_statusnetwork},
+	{"numselected",2,  &us_statusselectcount},
 	{"node",       2,  &us_statusnode},
 	{"package",    3,  &us_statuspackage},
 	{"part",       3,  &us_statuspart},
@@ -367,7 +368,7 @@ void us_window(INTBIG count, char *par[])
 		{
 			case DISPWINDOW:
 				if (us_needfacet() == NONODEPROTO) return;
-				if (pp[strlen(pp)-1] == 'l') dist = atola(pp); else
+				if (pp[strlen(pp)-1] == 'l') dist = atola(pp, 0); else
 					dist = muldiv(atofr(pp), (el_curwindowpart->screenhy - el_curwindowpart->screenly), WHOLE);
 				us_slideup(-dist);
 				return;
@@ -386,7 +387,7 @@ void us_window(INTBIG count, char *par[])
 				return;
 #endif
 			case EXPLORERWINDOW:
-				us_explorepan(el_curwindowpart, -1);
+				us_explorevpan(el_curwindowpart, -1);
 				return;
 		}
 		us_abortcommand(_("Cannot pan this kind of windows"));
@@ -416,8 +417,6 @@ void us_window(INTBIG count, char *par[])
 
 	if (namesamen(pp, "explore", l) == 0)
 	{
-		extern INTBIG us_explorefirstline;
-
 		/* see if the explorer window is already there */
 		for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
 			if ((w->state&WINDOWTYPE) == EXPLORERWINDOW) break;
@@ -442,7 +441,7 @@ void us_window(INTBIG count, char *par[])
 		}
 		if (strcmp(w->location, "entire") == 0)
 		{
-			w = us_splitcurrentwindow(2, FALSE, 0);
+			w = us_splitcurrentwindow(2, FALSE, 0, us_explorerratio);
 			if (w == NOWINDOWPART) return;
 		}
 
@@ -450,7 +449,6 @@ void us_window(INTBIG count, char *par[])
 		for(oldw = el_topwindowpart; oldw != NOWINDOWPART; oldw = oldw->nextwindowpart)
 			if (oldw != w && oldw->frame == w->frame) break;
 
-		us_explorefirstline = 0;
 		us_createexplorerstruct();
 		startobjectchange((INTBIG)w, VWINDOWPART);
 		(void)setval((INTBIG)w, VWINDOWPART, "buttonhandler", (INTBIG)us_explorebuttonhandler,
@@ -578,7 +576,7 @@ void us_window(INTBIG count, char *par[])
 		np = us_needfacet();
 		if (np == NONODEPROTO) return;
 		lambda = lambdaoffacet(np);
-		if (count >= 2) dist = atola(par[1]); else
+		if (count >= 2) dist = atola(par[1], 0); else
 			dist = 2 * lambda;
 		if (dist == 0)
 		{
@@ -586,9 +584,11 @@ void us_window(INTBIG count, char *par[])
 			return;
 		}
 
+#if 0
 		/* save and erase highlighting */
 		us_pushhighlight();
 		us_clearhighlightcount();
+#endif
 
 		diffx = muldiv(el_curwindowpart->screenhx - el_curwindowpart->screenlx,
 			lambda, dist);
@@ -609,7 +609,13 @@ void us_window(INTBIG count, char *par[])
 		endobjectchange((INTBIG)el_curwindowpart, VWINDOWPART);
 
 		/* restore highlighting */
+#if 0
 		us_pophighlight(FALSE);
+#else
+		us_endchanges(NOWINDOWPART);
+		us_state |= HIGHLIGHTSET;
+		us_showallhighlight();
+#endif
 		return;
 	}
 
@@ -637,7 +643,7 @@ void us_window(INTBIG count, char *par[])
 		{
 			case DISPWINDOW:
 				if (us_needfacet() == NONODEPROTO) return;
-				if (pp[strlen(pp)-1] == 'l') dist = atola(pp); else
+				if (pp[strlen(pp)-1] == 'l') dist = atola(pp, 0); else
 					dist = muldiv(atofr(pp), (el_curwindowpart->screenhx - el_curwindowpart->screenlx), WHOLE);
 				us_slideleft(dist);
 				return;
@@ -896,7 +902,7 @@ void us_window(INTBIG count, char *par[])
 		np = us_needfacet();
 		if (np == NONODEPROTO) return;
 		lambda = lambdaoffacet(np);
-		if (count >= 2) dist = atola(par[1]); else
+		if (count >= 2) dist = atola(par[1], 0); else
 			dist = 2 * lambda;
 		if (dist == 0)
 		{
@@ -904,9 +910,11 @@ void us_window(INTBIG count, char *par[])
 			return;
 		}
 
+#if 0
 		/* save and erase highlighting */
 		us_pushhighlight();
 		us_clearhighlightcount();
+#endif
 
 		lx = el_curwindowpart->screenlx;   hx = el_curwindowpart->screenhx;
 		ly = el_curwindowpart->screenly;   hy = el_curwindowpart->screenhy;
@@ -925,7 +933,13 @@ void us_window(INTBIG count, char *par[])
 		endobjectchange((INTBIG)el_curwindowpart, VWINDOWPART);
 
 		/* restore highlighting */
+#if 0
 		us_pophighlight(FALSE);
+#else
+		us_endchanges(NOWINDOWPART);
+		us_state |= HIGHLIGHTSET;
+		us_showallhighlight();
+#endif
 		return;
 	}
 
@@ -1017,7 +1031,7 @@ void us_window(INTBIG count, char *par[])
 		{
 			case DISPWINDOW:
 				if (us_needfacet() == NONODEPROTO) return;
-				if (pp[strlen(pp)-1] == 'l') dist = atola(pp); else
+				if (pp[strlen(pp)-1] == 'l') dist = atola(pp, 0); else
 					dist = muldiv(atofr(pp), (el_curwindowpart->screenhx - el_curwindowpart->screenlx), WHOLE);
 				us_slideleft(-dist);
 				return;
@@ -1081,7 +1095,7 @@ void us_window(INTBIG count, char *par[])
 		}
 
 		/* split the window */
-		(void)us_splitcurrentwindow(splitkey, TRUE, 0);
+		(void)us_splitcurrentwindow(splitkey, TRUE, 0, 50);
 		return;
 	}
 
@@ -1307,7 +1321,7 @@ void us_window(INTBIG count, char *par[])
 		{
 			case DISPWINDOW:
 				if (us_needfacet() == NONODEPROTO) return;
-				if (pp[strlen(pp)-1] == 'l') dist = atola(pp); else
+				if (pp[strlen(pp)-1] == 'l') dist = atola(pp, 0); else
 					dist = muldiv(atofr(pp), (el_curwindowpart->screenhy - el_curwindowpart->screenly), WHOLE);
 				us_slideup(dist);
 				return;
@@ -1326,7 +1340,7 @@ void us_window(INTBIG count, char *par[])
 				return;
 #endif
 			case EXPLORERWINDOW:
-				us_explorepan(el_curwindowpart, 1);
+				us_explorevpan(el_curwindowpart, 1);
 				return;
 		}
 		us_abortcommand(_("Cannot pan this kind of windows"));
@@ -1567,7 +1581,7 @@ void us_yanknode(INTBIG count, char *par[])
 		if (topno->proto->primindex != 0) continue;
 
 		/* disallow yanking if lock is on */
-		if (us_cantedit(np, topno->proto, TRUE)) return;
+		if (us_cantedit(np, topno, TRUE)) return;
 
 		/* turn off highlighting for the first facet */
 		if (!found) us_clearhighlightcount();
