@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usrctech.c
@@ -426,9 +427,7 @@ PORTPROTO *us_tranconvpp(NODEINST *ni, PORTPROTO *mospp)
 			schempp = schemni->proto->firstportproto;
 			break;
 		case TRAN_FACET:
-			for(schempp = schemni->proto->firstportproto; schempp != NOPORTPROTO;
-				schempp = schempp->nextportproto)
-					if (namesame(schempp->protoname, mospp->protoname) == 0) break;
+			schempp = getportproto(schemni->proto, mospp->protoname);
 			break;
 		default: /* transistor */
 			for(port = 1, pp = ni->proto->firstportproto; pp != NOPORTPROTO;
@@ -692,9 +691,9 @@ BOOLEAN us_tran_makelayoutparts(NODEPROTO *oldfacet, NODEPROTO *newfacet,
 		{
 			ttyputmsg(_("Facet %s: can't run arc from node %s port %s at (%s,%s)"),
 				describenodeproto(newfacet), describenodeinst(end1),
-					schempp1->protoname, latoa(x1), latoa(y1));
+					schempp1->protoname, latoa(x1, 0), latoa(y1, 0));
 			ttyputmsg(_("   to node %s port %s at (%s,%s)"), describenodeinst(end2),
-				schempp2->protoname, latoa(x2), latoa(y2));
+				schempp2->protoname, latoa(x2, 0), latoa(y2, 0));
 		}
 	}
 
@@ -821,8 +820,8 @@ PORTPROTO *us_convport(NODEINST *ni, NODEINST *newni, PORTPROTO *oldpp)
 	if (newni->proto->primindex == 0)
 	{
 		/* facets can associate by comparing names */
-		for(pp = newni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-			if (strcmp(pp->protoname, oldpp->protoname) == 0) return(pp);
+		pp = getportproto(newni->proto, oldpp->protoname);
+		if (pp != NOPORTPROTO) return(pp);
 	}
 
 	/* if functions are different, handle some special cases */
@@ -1181,7 +1180,7 @@ void us_printtechnology(TECHNOLOGY *tech)
 			area = (INTBIG)(fabs(areapoly(poly)) / 4000.0);
 			saveunit = el_units & DISPLAYUNITS;
 			el_units = (el_units & ~DISPLAYUNITS) | DISPUNITMIC;
-			(void)allocstring(&layersizes[tot], latoa(area), el_tempcluster);
+			(void)allocstring(&layersizes[tot], latoa(area, 0), el_tempcluster);
 			el_units = (el_units & ~DISPLAYUNITS) | saveunit;
 			if (k > 0)
 			{
@@ -1294,7 +1293,7 @@ void us_printtechnology(TECHNOLOGY *tech)
 			getbbox(poly, &lx, &hx, &ly, &hy);
 			saveunit = el_units & DISPLAYUNITS;
 			el_units = (el_units & ~DISPLAYUNITS) | DISPUNITMIC;
-			(void)sprintf(thefield, "%s x %s", latoa(hx-lx), latoa(hy-ly));
+			(void)sprintf(thefield, "%s x %s", latoa(hx-lx, 0), latoa(hy-ly, 0));
 			(void)allocstring(&layersizes[tot], thefield, el_tempcluster);
 			el_units = (el_units & ~DISPLAYUNITS) | saveunit;
 			if (k > 0)
@@ -1311,7 +1310,7 @@ void us_printtechnology(TECHNOLOGY *tech)
 			getbbox(poly, &lx, &hx, &ly, &hy);
 			saveunit = el_curtech->userbits & DISPLAYUNITS;
 			el_units = (el_units & ~DISPLAYUNITS) | DISPUNITMIC;
-			(void)sprintf(thefield, "%s x %s", latoa(hx-lx), latoa(hy-ly));
+			(void)sprintf(thefield, "%s x %s", latoa(hx-lx, 0), latoa(hy-ly, 0));
 			(void)allocstring(&portsizes[base], thefield, el_tempcluster);
 			el_units = (el_units & ~DISPLAYUNITS) | saveunit;
 			if (((pp->userbits&PORTARANGE) >> PORTARANGESH) == 180) (void)strcpy(thefield, ""); else
