@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usrdisp.c
@@ -80,17 +81,31 @@ static GRAPHICS us_graybox = {LAYERO, GRAY, PATTERNED, PATTERNED,
 					0x3030}, /*   XX      XX     */
 					NOVARIABLE, 0};
 
+/* for drawing dimmed background (when editing in-place) */
+static GRAPHICS us_dimbox = {LAYERH, HIGHLIT, PATTERNED, PATTERNED,
+					{0x0404, /*      X       X   */
+					0x0000,  /*                  */
+					0x4040,  /*  X       X       */
+					0x0000,  /*                  */
+					0x0404,  /*      X       X   */
+					0x0000,  /*                  */
+					0x4040,  /*  X       X       */
+					0x0000}, /*                  */
+					NOVARIABLE, 0};
+
 #define MAXGRID         75
 #define MINGRID          4	/* the minimum grid has 4 pixels spacing */
 #define TXTMAXFACETSIZE 36	/* maximum point size of facet text */
 
 /* prototypes for local routines */
-static void us_showemptywindow(WINDOWPART*);
-static void us_graphicsarcs(PORTPROTO*, INTBIG*, INTBIG*);
-static void us_combinelayers(ARCINST*, INTBIG*, INTBIG*);
+static void   us_showemptywindow(WINDOWPART*);
+static void   us_graphicsarcs(PORTPROTO*, INTBIG*, INTBIG*);
+static void   us_combinelayers(ARCINST*, INTBIG*, INTBIG*);
 static INTBIG us_drawall(INTBIG, INTBIG, INTBIG, INTBIG, NODEPROTO*, XARRAY, INTBIG, BOOLEAN);
 static INTBIG us_drawarcinstpeek(ARCINST*, XARRAY, INTBIG);
 static INTBIG us_drawnodeinstpeek(NODEINST*, XARRAY, INTBIG);
+static void   us_drawfacetcontents(NODEPROTO *facet, WINDOWPART *w, BOOLEAN now);
+static int    us_sortshownports(const void *e1, const void *e2);
 
 /******************** WINDOW CONTROL ********************/
 
@@ -985,12 +1000,9 @@ void us_drawhorizontalsliderthumb(WINDOWPART *w, INTBIG ly, INTBIG lx, INTBIG hx
 void us_redisplaynow(WINDOWPART *w, BOOLEAN now)
 {
 	REGISTER NODEPROTO *facet;
-	REGISTER NODEINST *ni;
-	REGISTER ARCINST *ai;
 	REGISTER VARIABLE *var;
 	REGISTER TECHNOLOGY *tech;
-	REGISTER INTBIG i, j, ret, lambda;
-	REGISTER INTBIG alignment;
+	REGISTER INTBIG i, j, lambda, alignment;
 	INTBIG cenx, ceny, dummy;
 	static POLYGON *poly = NOPOLYGON;
 
@@ -1006,8 +1018,32 @@ void us_redisplaynow(WINDOWPART *w, BOOLEAN now)
 		return;
 	}
 
-	/* initialize hierarchy traversal */
-	begintraversehierarchy();
+	/* show the background if editing in place */
+	if ((w->state&INPLACEEDIT) != 0 && w->topnodeproto != NONODEPROTO)
+	{
+		/* indicate that the in-place surround is being drawn */
+		w->state = (w->state & ~INPLACEEDIT) | SURROUNDINPLACEEDIT;
+
+		/* draw the surround */
+		us_drawfacetcontents(w->topnodeproto, w, TRUE);
+
+		/* dim the entire facet */
+		(void)needstaticpolygon(&poly, 4, us_tool->cluster);
+		maketruerectpoly(w->screenlx, w->screenhx, w->screenly, w->screenhy, poly);
+		poly->desc = &us_dimbox;
+		poly->style = FILLEDRECT;
+		(*us_displayroutine)(poly, w);
+
+		/* clear the area of the facet */
+		makerectpoly(facet->lowx, facet->highx, facet->lowy, facet->highy, poly);
+		poly->style = FILLED;
+		xformpoly(poly, w->outoffacet);
+		poly->desc = &us_hbox;   us_hbox.col = 0;
+		(*us_displayroutine)(poly, w);
+
+		/* go back to drawing in-place */
+		w->state = (w->state & ~SURROUNDINPLACEEDIT) | INPLACEEDIT;
+	}
 
 	/* draw the frame if there is one */
 	if ((w->state&WINDOWTYPE) != DISP3DWINDOW)
@@ -1025,34 +1061,7 @@ void us_redisplaynow(WINDOWPART *w, BOOLEAN now)
 		}
 	}
 
-	for(ni = facet->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
-	{
-		if (now)
-		{
-			if (us_drawfacet(ni, LAYERA, el_matid, 3, w) < 0) break;
-		} else
-		{
-			ret = us_drawfacet(ni, LAYERA, el_matid, 1, w);
-			if (ret < 0) break;
-			if (ret == 2) us_queueopaque(ni->geom, FALSE);
-		}
-	}
-
-	for(ai = facet->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
-	{
-		if (now)
-		{
-			if (us_drawarcinst(ai, LAYERA, el_matid, 3, w) < 0) break;
-		} else
-		{
-			ret = us_drawarcinst(ai, LAYERA, el_matid, 1, w);
-			if (ret < 0) break;
-			if (ret == 2) us_queueopaque(ai->geom, FALSE);
-		}
-	}
-
-	/* stop hierarchy traversal */
-	endtraversehierarchy();
+	us_drawfacetcontents(facet, w, now);
 
 	/* show this facet and export variables */
 	us_drawnodeprotovariables(facet, el_matid, w, FALSE);
@@ -1105,6 +1114,48 @@ void us_redisplaynow(WINDOWPART *w, BOOLEAN now)
 	if ((w->state&WINDOWTYPE) == DISP3DWINDOW) us_3denddrawing();
 }
 
+/*
+ * Routine to draw the nodes and arcs in facet "facet".  Draw it in 1 pass if "now" is TRUE.
+ */
+void us_drawfacetcontents(NODEPROTO *facet, WINDOWPART *w, BOOLEAN now)
+{
+	REGISTER NODEINST *ni;
+	REGISTER ARCINST *ai;
+	REGISTER INTBIG ret;
+
+	/* initialize hierarchy traversal */
+	begintraversehierarchy();
+
+	for(ni = facet->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+	{
+		if (now)
+		{
+			if (us_drawfacet(ni, LAYERA, el_matid, 3, w) < 0) break;
+		} else
+		{
+			ret = us_drawfacet(ni, LAYERA, el_matid, 1, w);
+			if (ret < 0) break;
+			if (ret == 2) us_queueopaque(ni->geom, FALSE);
+		}
+	}
+
+	for(ai = facet->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
+	{
+		if (now)
+		{
+			if (us_drawarcinst(ai, LAYERA, el_matid, 3, w) < 0) break;
+		} else
+		{
+			ret = us_drawarcinst(ai, LAYERA, el_matid, 1, w);
+			if (ret < 0) break;
+			if (ret == 2) us_queueopaque(ai->geom, FALSE);
+		}
+	}
+
+	/* stop hierarchy traversal */
+	endtraversehierarchy();
+}
+
 /* erase the screen area in window frame "mw" (all windows if "mw" is zero) */
 void us_erasescreen(WINDOWFRAME *mw)
 {
@@ -1174,6 +1225,12 @@ void us_gridset(WINDOWPART *w, INTBIG state)
 		return;
 	}
 
+	if ((w->state&WINDOWTYPE) == WAVEFORMWINDOW)
+	{
+		(void)setval((INTBIG)w, VWINDOWPART, "state", (w->state & ~GRIDTOOSMALL) | GRIDON, VINTEGER);
+		return;
+	}
+
 	np = w->curnodeproto;
 	if (np == NONODEPROTO)
 	{
@@ -1204,6 +1261,13 @@ void us_gridset(WINDOWPART *w, INTBIG state)
 
 /******************** PORT SELECTION ******************/
 
+typedef struct
+{
+	INTBIG     portx, porty;
+	PORTPROTO *portpp;
+	INTBIG     portangle;
+} SHOWNPORTS;
+
 /*
  * routine to identify the port locations by number in a nodeinst.  If "count" is
  * nonzero, identify "count" nodes in the list "nilist".  Otherwise identify the
@@ -1212,19 +1276,20 @@ void us_gridset(WINDOWPART *w, INTBIG state)
  */
 void us_identifyports(INTBIG count, NODEINST **nilist, NODEPROTO *np, INTBIG on, BOOLEAN usenumbers)
 {
-	REGISTER PORTPROTO *pp, **portpp, *bestpp, *subpp;
+	REGISTER PORTPROTO *pp, *subpp;
 	REGISTER NODEPROTO *parent;
-	REGISTER NODEINST *subni, *ni, **portni;
-	REGISTER INTBIG digitindentx, digitindenty, total, *portx, *porty,
-		*portused, sidecount, i, j, dist, bestdist, bestindex, ignored;
-	INTBIG x, y, xout, yout;
+	REGISTER NODEINST *subni, *ni;
+	REGISTER INTBIG digitindentx, digitindenty, total, *portx, *porty, numperside,
+		leftsidecount, topsidecount, rightsidecount, botsidecount, bestoff,
+		i, j, dist, bestdist, ignored, halfsizex, halfsizey;
+	INTBIG x, y, xout, yout, tsx, tsy, sidex[4], sidey[4];
 	char line[80];
 	REGISTER WINDOWPART *w;
-	static POLYGON *poly = NOPOLYGON, *ppoly = NOPOLYGON;
+	REGISTER SHOWNPORTS *portlist;
+	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
 	(void)needstaticpolygon(&poly, 4, us_tool->cluster);
-	(void)needstaticpolygon(&ppoly, 4, us_tool->cluster);
 
 	/* determine the window to use */
 	if (count == 0) parent = np; else parent = nilist[0]->parent;
@@ -1287,38 +1352,43 @@ void us_identifyports(INTBIG count, NODEINST **nilist, NODEPROTO *np, INTBIG on,
 	if (portx == 0) return;
 	porty = (INTBIG *)emalloc(total * SIZEOFINTBIG, el_tempcluster);
 	if (porty == 0) return;
-	portused = (INTBIG *)emalloc(total * SIZEOFINTBIG, el_tempcluster);
-	if (portused == 0) return;
-	portpp = (PORTPROTO **)emalloc(total * (sizeof (PORTPROTO *)), el_tempcluster);
-	if (portpp == 0) return;
-	portni = (NODEINST **)emalloc(total * (sizeof (NODEINST *)), el_tempcluster);
-	if (portni == 0) return;
+	portlist = (SHOWNPORTS *)emalloc(total * (sizeof (SHOWNPORTS)), el_tempcluster);
+	if (portlist == 0) return;
+	numperside = (total + 3) / 4;
+	leftsidecount = topsidecount = rightsidecount = botsidecount = numperside;
+	if (leftsidecount + topsidecount + rightsidecount + botsidecount > total)
+		botsidecount--;
+	if (leftsidecount + topsidecount + rightsidecount + botsidecount > total)
+		topsidecount--;
+	if (leftsidecount + topsidecount + rightsidecount + botsidecount > total)
+		rightsidecount--;
+	j = 0;
+	for(i=0; i<leftsidecount; i++)
+	{
+		portx[j] = w->screenlx + digitindentx;
+		porty[j] = (w->screenhy - w->screenly) / (leftsidecount+1) * (i+1) + w->screenly;
+		j++;
+	}
+	for(i=0; i<topsidecount; i++)
+	{
+		portx[j] = (w->screenhx - w->screenlx) / (topsidecount+1) * (i+1) + w->screenlx;
+		porty[j] = w->screenhy - digitindenty;
+		j++;
+	}
+	for(i=0; i<rightsidecount; i++)
+	{
+		portx[j] = w->screenhx - digitindentx;
+		porty[j] = w->screenhy - (w->screenhy - w->screenly) / (rightsidecount+1) * (i+1);
+		j++;
+	}
+	for(i=0; i<botsidecount; i++)
+	{
+		portx[j] = w->screenhx - (w->screenhx - w->screenlx) / (botsidecount+1) * (i+1);
+		porty[j] = w->screenly + digitindenty;
+		j++;
+	}
 	for(i=0; i<total; i++)
 	{
-		portused[i] = 0;
-		switch (i%4)
-		{
-			case 0:
-				sidecount = (total+3) / 4;
-				portx[i] = w->screenlx + digitindentx;
-				porty[i] = (w->screenhy - w->screenly) / (sidecount+1) * (i/4+1) + w->screenly;
-				break;
-			case 1:
-				sidecount = (total+2) / 4;
-				portx[i] = (w->screenhx - w->screenlx) / (sidecount+1) * (i/4+1) + w->screenlx;
-				porty[i] = w->screenly + digitindenty;
-				break;
-			case 2:
-				sidecount = (total+1) / 4;
-				portx[i] = w->screenhx - digitindentx;
-				porty[i] = (w->screenhy - w->screenly) / (sidecount+1) * (i/4+1) + w->screenly;
-				break;
-			case 3:
-				sidecount = total / 4;
-				portx[i] = (w->screenhx - w->screenlx) / (sidecount+1) * (i/4+1) + w->screenlx;
-				porty[i] = w->screenhy - digitindenty;
-				break;
-		}
 		if ((w->state&INPLACEEDIT) != 0)
 			xform(portx[i], porty[i], &portx[i], &porty[i], w->intofacet);
 	}
@@ -1326,6 +1396,7 @@ void us_identifyports(INTBIG count, NODEINST **nilist, NODEPROTO *np, INTBIG on,
 	/* associate ports with display locations */
 	if (count == 0)
 	{
+		i = 0;
 		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		{
 			subni = pp->subnodeinst;
@@ -1337,25 +1408,14 @@ void us_identifyports(INTBIG count, NODEINST **nilist, NODEPROTO *np, INTBIG on,
 			if (xout < w->screenlx || xout > w->screenhx ||
 				yout < w->screenly || yout > w->screenhy) continue;
 
-			/* find the closest unused port location */
-			bestdist = (w->screenhx - w->screenlx) + (w->screenhy - w->screenly);
-			bestpp = NOPORTPROTO;
-			for(i=0; i<total; i++)
-			{
-				if (portused[i] != 0) continue;
-				dist = computedistance(x, y, portx[i], porty[i]);
-				if (dist > bestdist && bestpp != NOPORTPROTO) continue;
-				bestdist = dist;
-				bestindex = i;
-				bestpp = pp;
-			}
-
-			portused[bestindex] = 1;
-			portpp[bestindex] = bestpp;
-			portni[bestindex] = NONODEINST;
+			portlist[i].portx = x;
+			portlist[i].porty = y;
+			portlist[i].portpp = pp;
+			i++;
 		}
 	} else
 	{
+		i = 0;
 		for(j=0; j<count; j++)
 		{
 			ni = nilist[j];
@@ -1368,42 +1428,56 @@ void us_identifyports(INTBIG count, NODEINST **nilist, NODEPROTO *np, INTBIG on,
 				if (xout < w->screenlx || xout > w->screenhx ||
 					yout < w->screenly || yout > w->screenhy) continue;
 
-				/* find the closest unused port location */
-				bestdist = (w->screenhx - w->screenlx) + (w->screenhy - w->screenly);
-				bestpp = NOPORTPROTO;
-				for(i=0; i<total; i++)
-				{
-					if (portused[i] != 0) continue;
-					dist = computedistance(x, y, portx[i], porty[i]);
-					if (dist > bestdist && bestpp != NOPORTPROTO) continue;
-					bestdist = dist;
-					bestindex = i;
-					bestpp = pp;
-				}
-				portused[bestindex] = 1;
-				portpp[bestindex] = bestpp;
-				portni[bestindex] = ni;
+				portlist[i].portx = x;
+				portlist[i].porty = y;
+				portlist[i].portpp = pp;
+				i++;
 			}
 		}
 	}
 
+	/* build a sorted list of ports around the center */
+	x = y = 0;
+	for(i=0; i<total; i++)
+	{
+		x += portlist[i].portx;
+		y += portlist[i].porty;
+	}
+	x /= total;   y /= total;
+	for(i=0; i<total; i++)
+	{
+		if (x == portlist[i].portx && y == portlist[i].porty)
+			portlist[i].portangle = 0; else
+				portlist[i].portangle = -figureangle(x, y, portlist[i].portx, portlist[i].porty);
+	}
+	esort(portlist, total, sizeof(SHOWNPORTS), us_sortshownports);
+
+	/* figure out the best rotation offset */
+	bestdist = 0;
+	for(i=0; i<total; i++)
+	{
+		dist = 0;
+		for(j=0; j<total; j++)
+			dist += computedistance(portx[j],porty[j],
+				portlist[(j+i)%total].portx,portlist[(j+i)%total].porty);
+		if (dist < bestdist || i == 0)
+		{
+			bestoff = i;
+			bestdist = dist;
+		}
+	}
+
 	/* show the ports */
 	for(i=0; i<total; i++)
 	{
-		if (portni[i] == NONODEINST)
-		{
-			subni = portpp[i]->subnodeinst;
-			subpp = portpp[i]->subportproto;
-		} else
-		{
-			subni = portni[i];
-			subpp = portpp[i];
-		}
 		if (usenumbers)
 		{
-			(void)sprintf(line, "%ld", portused[i]);
+			(void)sprintf(line, "%ld", i);
 			poly->string = line;
-		} else poly->string = portpp[i]->protoname;
+		} else
+		{
+			poly->string = portlist[(bestoff+i)%total].portpp->protoname;
+		}
 
 		/* draw the port index number */
 		poly->xv[0] = portx[i];   poly->yv[0] = porty[i];   poly->count = 1;
@@ -1411,37 +1485,56 @@ void us_identifyports(INTBIG count, NODEINST **nilist, NODEPROTO *np, INTBIG on,
 		poly->style = TEXTCENT;
 		TDCLEAR(poly->textdescript);
 		TDSETSIZE(poly->textdescript, TXTSETPOINTS(20));
-		poly->tech = subni->proto->tech;
+		poly->tech = el_curtech;
 		us_hbox.col = HIGHLIT&on;
 		us_showpoly(poly, w);
 		(void)strcpy(line, "");
 
 		/* draw the line from the port to the index number */
-		portposition(subni, subpp, &x, &y);
-		poly->xv[0] = portx[i];   poly->yv[0] = porty[i];
-		poly->xv[1] = x;   poly->yv[1] = y;
+		poly->xv[0] = portlist[(bestoff+i)%total].portx;
+		poly->yv[0] = portlist[(bestoff+i)%total].porty;
+		screensettextinfo(w, el_curtech, poly->textdescript);
+		screengettextsize(w, poly->string, &tsx, &tsy);
+		halfsizex = (INTBIG)((tsx+2) / w->scalex / 2.0f + 0.5f);
+		halfsizey = (INTBIG)((tsy+2) / w->scaley / 2.0f + 0.5f);
+		sidex[0] = portx[i]-halfsizex;   sidey[0] = porty[i];
+		sidex[1] = portx[i]+halfsizex;   sidey[1] = porty[i];
+		sidex[2] = portx[i];             sidey[2] = porty[i]-halfsizey;
+		sidex[3] = portx[i];             sidey[3] = porty[i]+halfsizey;
+		for(j=0; j<4; j++)
+		{
+			dist = computedistance(poly->xv[0], poly->yv[0], sidex[j], sidey[j]);
+			if (j != 0 && dist >= bestdist) continue;
+			bestdist = dist;
+			poly->xv[1] = sidex[j];   poly->yv[1] = sidey[j];
+		}
 		poly->count = 2;
 		poly->desc = &us_arbit;
 		poly->style = VECTORS;
 		us_arbit.bits = LAYERH;   us_arbit.col = HIGHLIT&on;
 		us_showpoly(poly, w);
-
-		/* draw the port outline */
-		ppoly->style = CLOSED;
-		ppoly->desc = &us_arbit;
-		us_showpoly(ppoly, w);
 	}
 	flushscreen();
 	efree((char *)portx);
 	efree((char *)porty);
-	efree((char *)portused);
-	efree((char *)portpp);
-	efree((char *)portni);
+	efree((char *)portlist);
 	if (ignored > 0)
 		ttyputmsg(_("Could not display %ld %s (outside of the window)"), ignored,
 			makeplural(_("port"), ignored));
 }
 
+/*
+ * Helper routine for "esort" that makes shown ports go in proper circular order.
+ */
+int us_sortshownports(const void *e1, const void *e2)
+{
+	REGISTER SHOWNPORTS *s1, *s2;
+
+	s1 = (SHOWNPORTS *)e1;
+	s2 = (SHOWNPORTS *)e2;
+	return(s1->portangle - s2->portangle);
+}
+
 /******************** NODE/ARC DISPLAY ********************/
 
 /*
@@ -1617,11 +1710,36 @@ INTBIG us_drawnodeinst(NODEINST *ni, INTBIG on, XARRAY prevtrans, INTBIG layers,
 			us_getnodebounds(ni, &bx, &ux, &by, &uy);
 			maketruerectpoly(bx, ux, by, uy, poly);
 			xformpoly(poly, prevtrans);
+
+			/* because "us_getnodebounds()" returns rotated area, "prevtrans" over-rotates */
+			if (ni->rotation != 0 || ni->transpose != 0)
+			{
+				makerotI(ni, localtran);
+				xformpoly(poly, localtran);
+			}
 			poly->style = FILLEDRECT;
 			(*us_displayroutine)(poly, w);
 			return(0);
 		}
 
+		/* if drawing in-place surround, do not draw the facet being edited */
+		if ((w->state&SURROUNDINPLACEEDIT) != 0)
+		{
+			NODEINST **nilist;
+			INTBIG *indexlist, depth, inplacedepth;
+			gettraversalpath(ni->parent, &nilist, &indexlist, &depth, 0);
+			inplacedepth = w->inplacedepth-1;
+			if (ni == w->inplacestack[inplacedepth])
+			{
+				if (depth >= inplacedepth)
+				{
+					for(i=0; i<inplacedepth; i++)
+						if (w->inplacestack[i] != nilist[depth-inplacedepth+i]) break;
+					if (i >= inplacedepth) return(0);
+				}
+			}
+		}
+
 		/* transform into the nodeinst for display of its guts */
 		maketrans(ni, localtran);
 		transmult(localtran, prevtrans, subrot);
@@ -1809,7 +1927,7 @@ INTBIG us_drawnodeinst(NODEINST *ni, INTBIG on, XARRAY prevtrans, INTBIG layers,
 			}
 
 			/* descend hierarchy to this node */
-			downhierarchy(ni, 0);
+			downhierarchy(ni, ni->proto, 0);
 
 			/* search through facet */
 			for(ino = np->firstnodeinst; ino != NONODEINST; ino = ino->nextnodeinst)
@@ -1830,16 +1948,19 @@ INTBIG us_drawnodeinst(NODEINST *ni, INTBIG on, XARRAY prevtrans, INTBIG layers,
 	}
 
 	/* write export names if appropriate */
-	if (ni->firstportexpinst != NOPORTEXPINST && ni->parent == w->curnodeproto)
+	if ((us_useroptions&HIDETXTEXPORT) == 0)
 	{
-		if ((layers&2) == 0) moretodo |= 2; else
+		if (ni->firstportexpinst != NOPORTEXPINST && ni->parent == w->curnodeproto)
 		{
-			for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
+			if ((layers&2) == 0) moretodo |= 2; else
 			{
-				us_writeprotoname(pe->exportproto, on, prevtrans, LAYERO,
-					el_colfacettxt&on, w, 0, 0, 0, 0,
-						(us_useroptions&EXPORTLABELS)>>EXPORTLABELSSH);
-				us_drawportprotovariables(pe->exportproto, on, prevtrans, w, FALSE);
+				for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
+				{
+					us_writeprotoname(pe->exportproto, on, prevtrans, LAYERO,
+						el_colfacettxt&on, w, 0, 0, 0, 0,
+							(us_useroptions&EXPORTLABELS)>>EXPORTLABELSSH);
+					us_drawportprotovariables(pe->exportproto, on, prevtrans, w, FALSE);
+				}
 			}
 		}
 	}
@@ -2335,8 +2456,8 @@ INTBIG us_drawall(INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy, NODEPROTO *np, XAR
 			} else
 			{
 				/* node is complex: recurse */
-				downhierarchy(ni, 0);
 				subnt = ni->proto;
+				downhierarchy(ni, subnt, 0);
 				maketransI(ni, localtrans);
 				if (ni->rotation == 0 && ni->transpose == 0)
 					tbound = (XARRAY *)localtrans; else
