@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usreditemacs.c
@@ -117,7 +118,7 @@ static void us_editemacsredraweditor(WINDOWPART*);
 static BOOLEAN us_editemacsimplementchar(WINDOWPART*, INTBIG, BOOLEAN, BOOLEAN);
 static BOOLEAN us_editemacsbackupchar(WINDOWPART*, BOOLEAN);
 static BOOLEAN us_editemacsadvancechar(WINDOWPART*, BOOLEAN);
-static void us_editemacsdeletechar(WINDOWPART*);
+static void us_editemacsdeletechar(WINDOWPART*, BOOLEAN fromuser);
 static void us_editemacsshiftscreenup(WINDOWPART*);
 static void us_editemacsshiftscreendown(WINDOWPART*);
 static void us_editemacsensuretextshown(WINDOWPART*, INTBIG);
@@ -137,6 +138,7 @@ static void us_editemacsinvertbox(WINDOWPART*, INTBIG, INTBIG, INTBIG, INTBIG);
 static void us_editemacstext(WINDOWPART*, char*, INTBIG, INTBIG);
 static void us_editemacsdosearch(WINDOWPART *win, char *str, BOOLEAN reverse,
 			BOOLEAN fromtop, BOOLEAN casesensitive, BOOLEAN fromuser);
+static char us_editemacsnextvalidcharacter(EDITOR *e);
 
 /******************** INTERFACE FOR TEXT WINDOW ********************/
 
@@ -651,13 +653,13 @@ BOOLEAN us_editemacsgotchar(WINDOWPART *win, INTSML i, INTBIG special)
 	return(FALSE);
 }
 
-void us_editemacscut(WINDOWPART *w) { ttybeep(1); }
+void us_editemacscut(WINDOWPART *w) { ttybeep(SOUNDBEEP, TRUE); }
 
-void us_editemacscopy(WINDOWPART *w) { ttybeep(1); }
+void us_editemacscopy(WINDOWPART *w) { ttybeep(SOUNDBEEP, TRUE); }
 
-void us_editemacspaste(WINDOWPART *w) { ttybeep(1); }
+void us_editemacspaste(WINDOWPART *w) { ttybeep(SOUNDBEEP, TRUE); }
 
-void us_editemacsundo(WINDOWPART *w) { ttybeep(1); }
+void us_editemacsundo(WINDOWPART *w) { ttybeep(SOUNDBEEP, TRUE); }
 
 /*
  * routine to search and/or replace text.  If "replace" is nonzero, this is
@@ -717,8 +719,8 @@ void us_editemacspan(WINDOWPART *win, INTBIG dx, INTBIG dy)
 BOOLEAN us_editemacsimplementchar(WINDOWPART *win, INTBIG i, BOOLEAN m, BOOLEAN fromuser)
 {
 	REGISTER EDITOR *e;
-	INTBIG j, k, savecur, len;
-	char s[2], *pt, *nextline;
+	INTBIG j, k, savecur, len, nextlineno, nextcharno;
+	char s[2], *pt, *nextline, ch;
 	REGISTER void *infstr;
 
 	e = win->editor;
@@ -888,13 +890,8 @@ BOOLEAN us_editemacsimplementchar(WINDOWPART *win, INTBIG i, BOOLEAN m, BOOLEAN
 	/* delete next character on a line (cannot delete line break) */
 	if (i == CTLD && !m)
 	{
-		if (e->textarray[e->curline][e->curchar] == 0)
-		{
-			ttyputerr(_("Can only delete characters within a line.  Use ^K to remove lines."));
-			return(FALSE);
-		}
 		if (fromuser) us_editemacsworkingoncurline(e);
-		us_editemacsdeletechar(win);
+		us_editemacsdeletechar(win, fromuser);
 		return(FALSE);
 	}
 
@@ -904,34 +901,34 @@ BOOLEAN us_editemacsimplementchar(WINDOWPART *win, INTBIG i, BOOLEAN m, BOOLEAN
 		if (fromuser) us_editemacsworkingoncurline(e);
 
 		/* first delete all nonalphanumeric characters */
-		while (!isalnum(e->textarray[e->curline][e->curchar]) &&
-			e->textarray[e->curline][e->curchar] != 0)
-				us_editemacsdeletechar(win);
-
-		/* if stopped at end-of-line, print error */
-		if (e->textarray[e->curline][e->curchar] == 0)
+		for(;;)
 		{
-			ttyputerr(_("Can only delete characters within a line.  Use ^K to remove lines."));
-			return(FALSE);
+			ch = us_editemacsnextvalidcharacter(e);
+			if (ch == 0) break;
+			if (isalnum(ch)) break;
+			us_editemacsdeletechar(win, fromuser);
 		}
 
-		/* now delete alphanumeric characters */
-		while (isalnum(e->textarray[e->curline][e->curchar]))
-			us_editemacsdeletechar(win);
+		for(;;)
+		{
+			us_editemacsdeletechar(win, fromuser);
+			if (!isalnum(e->textarray[e->curline][e->curchar])) break;
+		}
 		return(FALSE);
 	}
 
 	/* delete previous character on a line (cannot delete line break) */
 	if ((i == CTLH || i == DELETEKEY) && !m)
 	{
-		if (e->curchar <= 0)
-		{
-			ttyputerr(_("Can only delete characters within a line.  Use ^K to remove lines."));
-			return(FALSE);
-		}
 		if (fromuser) us_editemacsworkingoncurline(e);
 		e->curchar--;
-		us_editemacsdeletechar(win);
+		if (e->curchar < 0)
+		{
+			if (e->curline <= 0) { e->curchar++;   return(FALSE); }
+			e->curline--;
+			e->curchar = strlen(e->textarray[e->curline]);
+		}
+		us_editemacsdeletechar(win, fromuser);
 		return(FALSE);
 	}
 
@@ -943,26 +940,32 @@ BOOLEAN us_editemacsimplementchar(WINDOWPART *win, INTBIG i, BOOLEAN m, BOOLEAN
 		/* first backwards delete all nonalphanumeric characters */
 		for(;;)
 		{
-			if (e->curchar <= 0) break;
-			if (isalnum(e->textarray[e->curline][e->curchar-1])) break;
 			e->curchar--;
-			us_editemacsdeletechar(win);
-		}
-
-		/* if stopped at start-of-line, print error */
-		if (e->curchar <= 0)
-		{
-			ttyputerr(_("Can only delete characters in a single line"));
-			return(FALSE);
+			if (e->curchar < 0)
+			{
+				if (e->curline <= 0) { e->curchar++;   return(FALSE); }
+				e->curline--;
+				e->curchar = strlen(e->textarray[e->curline]);
+			}
+			if (isalnum(e->textarray[e->curline][e->curchar])) break;
+			us_editemacsdeletechar(win, fromuser);
 		}
 
 		/* now backwards delete alphanumeric characters */
 		for(;;)
 		{
-			e->curchar--;
-			us_editemacsdeletechar(win);
-			if (e->curchar <= 0) break;
-			if (!isalnum(e->textarray[e->curline][e->curchar-1])) break;
+			us_editemacsdeletechar(win, fromuser);
+			nextcharno = e->curchar - 1;
+			nextlineno = e->curline;
+			if (nextcharno < 0)
+			{
+				if (nextlineno <= 0) return(FALSE);
+				nextlineno--;
+				nextcharno = strlen(e->textarray[nextlineno]);
+			}
+			if (!isalnum(e->textarray[nextlineno][nextcharno])) break;
+			e->curchar = nextcharno;
+			e->curline = nextlineno;
 		}
 		return(FALSE);
 	}
@@ -995,6 +998,9 @@ BOOLEAN us_editemacsimplementchar(WINDOWPART *win, INTBIG i, BOOLEAN m, BOOLEAN
 					nextline = e->textarray[e->curline+1];
 				if (strcmp(e->textarray[e->curline], nextline) == 0) continue;
 				if (fromuser) us_editemacsworkingoncurline(e);
+				len = strlen(nextline);
+				while (len > e->maxchars[e->curline])
+					us_editemacsaddmorechars(e, e->curline);
 				(void)strcpy(e->textarray[e->curline], nextline);
 				if (fromuser) us_editemacsshipchanges(win);
 			}
@@ -1504,15 +1510,68 @@ BOOLEAN us_editemacsadvancechar(WINDOWPART *win, BOOLEAN fromuser)
 /*
  * routine to delete the current character
  */
-void us_editemacsdeletechar(WINDOWPART *win)
+void us_editemacsdeletechar(WINDOWPART *win, BOOLEAN fromuser)
 {
-	REGISTER INTBIG j;
+	REGISTER INTBIG j, curl, len;
+	REGISTER char *nextline;
 	char s[2];
 	REGISTER EDITOR *e;
 
 	e = win->editor;
 	if (e == NOEDITOR) return;
 
+	/* at end of line: delete an entire line */
+	if (e->textarray[e->curline][e->curchar] == 0)
+	{
+		/* only continue if there is a valid next line */
+		if (e->curline < e->maxlines-1)
+		{
+			/* start by appending the next line to this */
+			if (fromuser) us_editemacsworkingoncurline(e);
+			nextline = e->textarray[e->curline+1];
+			len = strlen(e->textarray[e->curline]) + strlen(nextline);
+			while (len > e->maxchars[e->curline])
+				us_editemacsaddmorechars(e, e->curline);
+			(void)strcat(e->textarray[e->curline], nextline);
+			us_editemacsshipchanges(win);
+			us_editemacsredrawline(win, e->curline);
+
+			/* now report deletion of following line */
+			if (win->changehandler != 0)
+				(*win->changehandler)(win, DELETETEXTLINE, "", e->textarray[e->curline+1], e->curline+1);
+			us_editemacsworkingoncurline(e);
+
+			/* shift up lines in memory */
+			for(curl = e->curline+1; curl < e->maxlines; curl++)
+			{
+				if (curl == e->maxlines-1) nextline = ""; else
+					nextline = e->textarray[curl+1];
+				if (strcmp(e->textarray[curl], nextline) == 0) continue;
+				len = strlen(nextline);
+				while (len > e->maxchars[curl])
+					us_editemacsaddmorechars(e, curl);
+				(void)strcpy(e->textarray[curl], nextline);
+			}
+
+			/* delete line on the screen */
+			curl = e->curline + 1;
+			if (curl-e->firstline < e->screenlines-1 &&
+				curl >= e->firstline && (e->state&EGRAPHICSOFF) == 0)
+			{
+				us_editemacsmovebox(win, 0, curl-e->firstline, e->swid,
+					e->shei-(curl-e->firstline+1)*us_thei, 0,
+						curl-e->firstline+1);
+				if (e->screenlines < e->maxlines)
+				{
+					us_editemacsclearbox(win, 0, e->screenlines-1, e->swid, us_thei);
+					us_editemacsredrawline(win, e->screenlines-1);
+				}
+			}
+		}
+		return;
+	}
+
+	/* just delete the character in the line */
 	for(j=e->curchar; j<e->maxchars[e->curline]; j++)
 		e->textarray[e->curline][j] = e->textarray[e->curline][j+1];
 	if (e->curline-e->firstline < e->screenlines && e->curline >= e->firstline &&
@@ -1742,6 +1801,22 @@ void us_editemacssetheader(WINDOWPART *win, char *header)
 		e->offx+e->swid+1, e->revy-e->shei-1, &us_menufigs, 0);
 }
 
+/*
+ * Routine to return the next valid character (ignoring end-of-lines).
+ */
+char us_editemacsnextvalidcharacter(EDITOR *e)
+{
+	INTBIG line, chr;
+
+	line = e->curline;   chr = e->curchar;
+	while (line < e->maxlines && e->textarray[line][chr] == 0)
+	{
+		line++;
+		chr = 0;
+	}
+	return(e->textarray[line][chr]);
+}
+
 /*
  * routine to invert the character cursor, turning it on or off
  */
