@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usreditpac.c
@@ -544,8 +545,7 @@ void us_editpacreadtextfile(WINDOWPART *win, char *file)
 	REGISTER FILE *f;
 	UINTBIG descript[TEXTDESCRIPTSIZE];
 	char *filename;
-	extern DIALOG us_progressdialog;
-	REGISTER void *infstr;
+	REGISTER void *infstr, *dia;
 
 	e = win->editor;
 	if (e == NOEDITOR) return;
@@ -560,15 +560,15 @@ void us_editpacreadtextfile(WINDOWPART *win, char *file)
 	filelength = filesize(f);
 	if (filelength > 0)
 	{
-		if (DiaInitDialog(&us_progressdialog))
+		infstr = initinfstr();
+		formatinfstr(infstr, _("Reading %s..."), file);
+		dia = DiaInitProgress(returninfstr(infstr), 0);
+		if (dia == 0)
 		{
 			xclose(f);
 			return;
 		}
-		DiaPercent(1, 0);
-		infstr = initinfstr();
-		formatinfstr(infstr, _("Reading %s..."), file);
-		DiaSetText(2, returninfstr(infstr));
+		DiaSetProgress(dia, 1, filelength);
 	} else
 		ttyputmsg(_("Reading %s"), file);
 
@@ -606,7 +606,7 @@ void us_editpacreadtextfile(WINDOWPART *win, char *file)
 		}
 		if (filelength > 0 && (charsread%100) == 0)
 		{
-			DiaPercent(1, charsread*100/filelength);
+			DiaSetProgress(dia, charsread, filelength);
 		}
 	}
 	xclose(f);
@@ -614,11 +614,11 @@ void us_editpacreadtextfile(WINDOWPART *win, char *file)
 	/* announce the changes (go backwards so the array expands only once) */
 	if (filelength > 0)
 	{
-		DiaPercent(1, 100);
-		DiaSetText(2, _("Cleaning up..."));
+		DiaSetProgress(dia, 999, 1000);
+		DiaSetTextProgress(dia, _("Cleaning up..."));
 	}
 	(*win->changehandler)(win, REPLACEALLTEXT, "", (char *)e->textarray, e->linecount);
-	if (filelength > 0) DiaDoneDialog();
+	if (filelength > 0) DiaDoneProgress(dia);
 
 	/* restore the display */
 	e->state &= ~EGRAPHICSOFF;
@@ -917,7 +917,7 @@ BOOLEAN us_editpacsinglesearch(WINDOWPART *win, char *str, char *replace, INTBIG
 		/* if it came all the way around, stop */
 		if (chr == startchr && lne == startlne)
 		{
-			ttybeep(1);
+			ttybeep(SOUNDBEEP, TRUE);
 			return(FALSE);
 		}
 	}
@@ -1523,7 +1523,7 @@ BOOLEAN us_editpacimplementchar(WINDOWPART *win, INTSML i, INTBIG special, BOOLE
 		/* disallow line insertion, deletion or any action on the last line */
 		if (e->curline != e->endline || i == '\n' || i == '\r' || e->curline == e->linecount-1)
 		{
-			ttybeep(1);
+			ttybeep(SOUNDBEEP, TRUE);
 			return(FALSE);
 		}
 	}
