@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usredtecc.c
@@ -208,6 +209,7 @@ static INTBIG us_teceditparsefun(char*);
 static BOOLEAN us_tecedgetdrc(char *str, BOOLEAN *connected, BOOLEAN *wide, BOOLEAN *multi,
 			INTBIG *widrule, BOOLEAN *edge, INTBIG *amt, INTBIG *layer1, INTBIG *layer2,
 			char **rule, INTBIG maxlayers, char **layernames);
+static void us_tecedrenamesequence(char *varname, char *oldname, char *newname);
 
 /*
  * the entry routine for all technology editing
@@ -511,7 +513,7 @@ void us_tecedentry(INTBIG count, char *par[])
 }
 
 /*
- * routine for editing the DRC tables
+ * Routine for editing the DRC tables.
  */
 void us_teceditdrc(void)
 {
@@ -540,7 +542,7 @@ void us_teceditdrc(void)
 	us_teceditgetdrcarrays(var, rules);
 
 	/* edit the design-rule arrays */
-	changed = dr_rulesdialog(NOTECHNOLOGY, rules);
+	changed = dr_rulesdlog(NOTECHNOLOGY, rules);
 
 	/* if changes were made, convert the arrays back into a text-list */
 	if (changed != 0)
@@ -552,6 +554,174 @@ void us_teceditdrc(void)
 	dr_freerules(rules);
 }
 
+/*
+ * Routine to update tables to reflect that cell "oldname" is now called "newname".
+ * If "newname" is not valid, any rule that refers to "oldname" is removed.
+ * 
+ */
+void us_tecedrenamecell(char *oldname, char *newname)
+{
+	REGISTER VARIABLE *var;
+	REGISTER INTBIG i, len;
+	REGISTER BOOLEAN valid;
+	INTBIG count;
+	REGISTER char *origstr, *firstkeyword, *keyword;
+	char *str, **strings;
+	REGISTER void *infstr, *sa;
+
+	/* if this is a layer, rename the layer sequence array */
+	if (namesamen(oldname, "layer-", 6) == 0 && namesamen(newname, "layer-", 6) == 0)
+	{
+		us_tecedrenamesequence("EDTEC_layersequence", &oldname[6], &newname[6]);
+	}
+
+	/* if this is an arc, rename the arc sequence array */
+	if (namesamen(oldname, "arc-", 4) == 0 && namesamen(newname, "arc-", 4) == 0)
+	{
+		us_tecedrenamesequence("EDTEC_arcsequence", &oldname[4], &newname[4]);
+	}
+
+	/* if this is a node, rename the node sequence array */
+	if (namesamen(oldname, "node-", 5) == 0 && namesamen(newname, "node-", 5) == 0)
+	{
+		us_tecedrenamesequence("EDTEC_nodesequence", &oldname[5], &newname[5]);
+	}
+
+	/* see if there are design rules in the current library */
+	var = getval((INTBIG)el_curlib, VLIBRARY, VSTRING|VISARRAY, "EDTEC_DRC");
+	if (var == NOVARIABLE) return;
+
+	/* examine the rules and convert the name */
+	len = getlength(var);
+	sa = newstringarray(us_tool->cluster);
+	for(i=0; i<len; i++)
+	{
+		/* parse the DRC rule */
+		str = ((char **)var->addr)[i];
+		origstr = str;
+		firstkeyword = getkeyword(&str, " ");
+		if (firstkeyword == NOSTRING) return;
+
+		/* pass wide wire limitation through */
+		if (*firstkeyword == 'l')
+		{
+			addtostringarray(sa, origstr);
+			continue;
+		}
+
+		/* rename nodes in the minimum node size rule */
+		if (*firstkeyword == 'n')
+		{
+			if (namesamen(oldname, "node-", 5) == 0 &&
+				namesame(&oldname[5], &firstkeyword[1]) == 0)
+			{
+				/* substitute the new name */
+				if (namesamen(newname, "node-", 5) == 0)
+				{
+					infstr = initinfstr();
+					addstringtoinfstr(infstr, "n");
+					addstringtoinfstr(infstr, &newname[5]);
+					addstringtoinfstr(infstr, str);
+					addtostringarray(sa, returninfstr(infstr));
+				}
+				continue;
+			}
+			addtostringarray(sa, origstr);
+			continue;
+		}
+
+		/* rename layers in the minimum layer size rule */
+		if (*firstkeyword == 's')
+		{
+			valid = TRUE;
+			infstr = initinfstr();
+			formatinfstr(infstr, "%s ", firstkeyword);
+			keyword = getkeyword(&str, " ");
+			if (keyword == NOSTRING) return;
+			if (namesamen(oldname, "layer-", 6) == 0 &&
+				namesame(&oldname[6], keyword) == 0)
+			{
+				if (namesamen(newname, "layer-", 6) != 0) valid = FALSE; else
+					addstringtoinfstr(infstr, &newname[6]);
+			} else
+				addstringtoinfstr(infstr, keyword);
+			addstringtoinfstr(infstr, str);
+			str = returninfstr(infstr);
+			if (valid) addtostringarray(sa, str);
+			continue;
+		}
+
+		/* layer width rule: substitute layer names */
+		infstr = initinfstr();
+		formatinfstr(infstr, "%s ", firstkeyword);
+		valid = TRUE;
+
+		/* get the first layer name and convert it */
+		keyword = getkeyword(&str, " ");
+		if (keyword == NOSTRING) return;
+		if (namesamen(oldname, "layer-", 6) == 0 &&
+			namesame(&oldname[6], keyword) == 0)
+		{
+			/* substitute the new name */
+			if (namesamen(newname, "layer-", 6) != 0) valid = FALSE; else
+				addstringtoinfstr(infstr, &newname[6]);
+		} else
+			addstringtoinfstr(infstr, keyword);
+		addtoinfstr(infstr, ' ');
+
+		/* get the second layer name and convert it */
+		keyword = getkeyword(&str, " ");
+		if (keyword == NOSTRING) return;
+		if (namesamen(oldname, "layer-", 6) == 0 &&
+			namesame(&oldname[6], keyword) == 0)
+		{
+			/* substitute the new name */
+			if (namesamen(newname, "layer-", 6) != 0) valid = FALSE; else
+				addstringtoinfstr(infstr, &newname[6]);
+		} else
+			addstringtoinfstr(infstr, keyword);
+
+		addstringtoinfstr(infstr, str);
+		str = returninfstr(infstr);
+		if (valid) addtostringarray(sa, str);
+	}
+	strings = getstringarray(sa, &count);
+	setval((INTBIG)el_curlib, VLIBRARY, "EDTEC_DRC", (INTBIG)strings,
+		VSTRING|VISARRAY|(count<<VLENGTHSH));
+	killstringarray(sa);
+}
+
+/*
+ * Routine to rename the layer/arc/node sequence arrays to account for a name change.
+ * The sequence array is in variable "varname", and the item has changed from "oldname" to
+ * "newname".
+ */
+void us_tecedrenamesequence(char *varname, char *oldname, char *newname)
+{
+	REGISTER VARIABLE *var;
+	char **strings;
+	REGISTER INTBIG i, len;
+	INTBIG count;
+	void *sa;
+
+	var = getval((INTBIG)el_curlib, VLIBRARY, VSTRING|VISARRAY, varname);
+	if (var == NOVARIABLE) return;
+
+	strings = (char **)var->addr;
+	len = getlength(var);
+	sa = newstringarray(us_tool->cluster);
+	for(i=0; i<len; i++)
+	{
+		if (namesame(strings[i], oldname) == 0)
+			addtostringarray(sa, newname); else
+				addtostringarray(sa, strings[i]);
+	}
+	strings = getstringarray(sa, &count);
+	setval((INTBIG)el_curlib, VLIBRARY, varname, (INTBIG)strings,
+		VSTRING|VISARRAY|(count<<VLENGTHSH));
+	killstringarray(sa);
+}
+
 void us_tecedgetlayernamelist(void)
 {
 	REGISTER INTBIG i;
@@ -2731,11 +2901,8 @@ void us_teceddeletelayerfacet(NODEPROTO *np)
 	REGISTER VARIABLE *var;
 	REGISTER NODEPROTO *onp;
 	REGISTER NODEINST *ni;
-	REGISTER BOOLEAN warned, isnode, found;
-	REGISTER char *layername, *str, **strings;
-	REGISTER void *sa;
-	REGISTER INTBIG len, layerlen, i, j, l;
-	INTBIG count;
+	REGISTER BOOLEAN warned, isnode;
+	REGISTER char *layername;
 	static INTBIG edtec_layer_key = 0;
 	REGISTER void *infstr;
 
@@ -2770,39 +2937,17 @@ void us_teceddeletelayerfacet(NODEPROTO *np)
 		ttyputmsg("%s", returninfstr(infstr));
 
 	/* see if this layer is mentioned in the design rules */
-	var = getval((INTBIG)el_curlib, VLIBRARY, VSTRING|VISARRAY, "EDTEC_DRC");
-	if (var != NOVARIABLE)
-	{
-		layerlen = strlen(layername);
-		found = FALSE;
-		l = getlength(var);
-		sa = newstringarray(us_tool->cluster);
-		for(i=0; i<l; i++)
-		{
-			/* parse the DRC rule */
-			str = ((char **)var->addr)[i];
-			len = strlen(str);
-			for(j=1; j <= len-layerlen; j++)
-			{
-				if (namesamen(layername, &str[j], layerlen) != 0) continue;
-				if (str[j-1] != ' ') continue;
-				if (str[j+layerlen] != 0 && str[j+layerlen] != ' ') continue;
-				found = TRUE;
-				break;
-			}
-			if (j > len-layerlen)
-				addtostringarray(sa, str);
-		}
-		if (found)
-		{
-			strings = getstringarray(sa, &count);
-			(void)setval((INTBIG)el_curlib, VLIBRARY, "EDTEC_DRC", (INTBIG)strings,
-				VSTRING|VISARRAY|(count<<VLENGTHSH));
-			ttyputmsg(_("Note: deleted references to layer %s in the design rules"),
-				layername);
-		}
-		killstringarray(sa);
-	}
+	us_tecedrenamecell(np->cell->cellname, "");
+}
+
+/*
+ * Routine called when facet "np" has been deleted (and it may be a node facet because its name
+ * started with "node-").
+ */
+void us_teceddeletenodefacet(NODEPROTO *np)
+{
+	/* see if this node is mentioned in the design rules */
+	us_tecedrenamecell(np->cell->cellname, "");
 }
 
 /******************** SUPPORT FOR "usredtecp.c" ROUTINES ********************/
