@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usredtecg.c
@@ -50,13 +51,13 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 {
 	REGISTER LIBRARY *lib;
 	REGISTER char *lay, *dxf, **sequence, **varnames, *fname;
-	REGISTER INTBIG i, j, k, e, xs, ys, oldlam, *newmap, *mapptr,
+	REGISTER INTBIG i, j, k, e, xs, ys, oldlam, *newmap, *mapptr, *minnodesize,
 		tcon, func, gds, nodexpos, bits, layertotal, arctotal, nodetotal,
-		height3d, thick3d, lambda, min2x, min2y;
+		height3d, thick3d, lambda, min2x, min2y, nlx, nhx, nly, nhy;
 	REGISTER BOOLEAN serp, square, wipes, lockable;
 	REGISTER BOOLEAN first;
 	INTBIG lx, hx, ly, hy, xpos[4], ypos[4], xsc[4], ysc[4], lxo, hxo, lyo, hyo,
-		lxp, hxp, lyp, hyp;
+		lxp, hxp, lyp, hyp, blx, bhx, bly, bhy;
 	REGISTER NODEPROTO *np, **nplist, *pnp, **aplist;
 	float spires, spicap, spiecap;
 	REGISTER VARIABLE *var, *var2, *var3, *var5, *var6, *var7, *var8,
@@ -80,7 +81,7 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 	if ((net_tool->toolstate&TOOLON) == 0)
 	{
 		ttyputerr(_("Network tool must be running...turning it on"));
-		toolturnon(net_tool, FALSE);
+		toolturnon(net_tool);
 		ttyputerr(_("...now reissue the technology editing command"));
 		return(NOLIBRARY);
 	}
@@ -294,6 +295,11 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 	/* create the node nodes */
 	ttyputmsg(_("Creating the nodes..."));
 	nodetotal = 0;
+	for(pnp = tech->firstnodeproto; pnp != NONODEPROTO; pnp = pnp->nextnodeproto)
+		if ((pnp->userbits&NNOTUSED) == 0) nodetotal++;
+	minnodesize = (INTBIG *)emalloc(nodetotal*2*SIZEOFINTBIG, el_tempcluster);
+	if (minnodesize == 0) return(NOLIBRARY);
+	nodetotal = 0;
 	for(pnp = tech->firstnodeproto; pnp != NONODEPROTO; pnp = pnp->nextnodeproto)
 	{
 		pnp->temp1 = 0;
@@ -362,6 +368,23 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 					poly->yv[k] = poly->yv[k] * lambda / oldlam;
 				}
 
+				/* accumulate total size of main example */
+				if (e == 0)
+				{
+					getbbox(poly, &blx, &bhx, &bly, &bhy);
+					if (i == 0)
+					{
+						nlx = blx;   nhx = bhx;
+						nly = bly;   nhy = bhy;
+					} else
+					{
+						if (blx < nlx) nlx = blx;
+						if (bhx > nhx) nhx = bhx;
+						if (bly < nly) nly = bly;
+						if (bhy > nhy) nhy = bhy;
+					}
+				}
+
 				/* create the node facet on the first valid layer */
 				if (first)
 				{
@@ -393,7 +416,6 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 					if ((pnp->userbits&LOCKEDPRIM) != 0) lockable = TRUE; else
 						lockable = FALSE;
 					us_tecedmakenode(np, func, serp, square, wipes, lockable);
-					nodetotal++;
 					pnp->temp1 = (INTBIG)np;
 				}
 
@@ -513,6 +535,9 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 				}
 			}
 		}
+		minnodesize[nodetotal*2] = (nhx - nlx) * WHOLE / lambda;
+		minnodesize[nodetotal*2+1] = (nhy - nly) * WHOLE / lambda;
+		nodetotal++;
 	}
 
 	/* save the node sequence */
@@ -634,6 +659,15 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 			{
 				rules->minnodesize[i*2] = ((INTBIG *)var->addr)[j*2];
 				rules->minnodesize[i*2+1] = ((INTBIG *)var->addr)[j*2+1];
+
+				/* if rule is valid, make sure it is no larger than actual size */
+				if (rules->minnodesize[i*2] > 0 && rules->minnodesize[i*2+1] > 0)
+				{
+					if (rules->minnodesize[i*2] > minnodesize[i*2])
+						rules->minnodesize[i*2] = minnodesize[i*2];
+					if (rules->minnodesize[i*2+1] > minnodesize[i*2+1])
+						rules->minnodesize[i*2+1] = minnodesize[i*2+1];
+				}
 				i++;
 			}
 			j++;
@@ -660,6 +694,7 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 	/* clean up */
 	ttyputmsg(_("Done."));
 	efree((char *)nplist);
+	efree((char *)minnodesize);
 	lib->lambda[tech->techindex] = oldlam;
 	return(lib);
 }
