@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usrhigh.c
@@ -53,7 +54,6 @@ static INTBIG   us_getallhighlistsize = 0;
 static void       us_makecurrentobject(void);
 static void       us_highlightverbose(GEOM*, PORTPROTO*, INTBIG, NODEPROTO*, POLYGON*);
 static BOOLEAN    us_geommatchestype(GEOM *geom, INTBIG type);
-static NODEPROTO *us_highfacet(HIGHLIGHT *high);
 
 /*
  * Routine to free all memory associated with this module.
@@ -462,7 +462,8 @@ INTBIG us_selectarea(NODEPROTO *np, INTBIG slx, INTBIG shx, INTBIG sly, INTBIG s
 			if ((var->type&VDISPLAY) == 0) continue;
 			newhightext.fromgeom = NOGEOM;
 			newhightext.fromport = NOPORTPROTO;
-			newhightext.fromvar = var;
+			newhightext.fromvarnoeval = var;
+			newhightext.fromvar = evalvar(newhightext.fromvarnoeval);
 			us_gethightextcenter(&newhightext, &xc, &yc, &style);
 			us_gethightextsize(&newhightext, &xw, &yw, el_curwindowpart);
 			if (xc+xw/2 < slx || xc-xw/2 > shx ||
@@ -487,7 +488,8 @@ INTBIG us_selectarea(NODEPROTO *np, INTBIG slx, INTBIG shx, INTBIG sly, INTBIG s
 			if ((var->type&VDISPLAY) == 0) continue;
 			newhightext.fromgeom = ni->geom;
 			newhightext.fromport = NOPORTPROTO;
-			newhightext.fromvar = var;
+			newhightext.fromvarnoeval = var;
+			newhightext.fromvar = evalvar(newhightext.fromvarnoeval);
 			us_gethightextcenter(&newhightext, &xc, &yc, &style);
 			us_gethightextsize(&newhightext, &xw, &yw, el_curwindowpart);
 			if (xc+xw/2 < slx || xc-xw/2 > shx ||
@@ -507,7 +509,8 @@ INTBIG us_selectarea(NODEPROTO *np, INTBIG slx, INTBIG shx, INTBIG sly, INTBIG s
 				if ((var->type&VDISPLAY) == 0) continue;
 				newhightext.fromgeom = ai->geom;
 				newhightext.fromport = NOPORTPROTO;
-				newhightext.fromvar = var;
+				newhightext.fromvarnoeval = var;
+				newhightext.fromvar = evalvar(newhightext.fromvarnoeval);
 				us_gethightextcenter(&newhightext, &xc, &yc, &style);
 				us_gethightextsize(&newhightext, &xw, &yw, el_curwindowpart);
 				if (xc+xw/2 < slx || xc-xw/2 > shx ||
@@ -560,7 +563,8 @@ INTBIG us_selectarea(NODEPROTO *np, INTBIG slx, INTBIG shx, INTBIG sly, INTBIG s
 					{
 						newhightext.fromgeom = geom;
 						newhightext.fromport = NOPORTPROTO;
-						newhightext.fromvar = var;
+						newhightext.fromvarnoeval = var;
+						newhightext.fromvar = evalvar(newhightext.fromvarnoeval);
 						us_gethightextcenter(&newhightext, &xc, &yc, &style);
 						us_gethightextsize(&newhightext, &xw, &yw, el_curwindowpart);
 						if (xc+xw/2 < slx || xc-xw/2 > shx ||
@@ -579,7 +583,7 @@ INTBIG us_selectarea(NODEPROTO *np, INTBIG slx, INTBIG shx, INTBIG sly, INTBIG s
 				{
 					newhightext.fromgeom = geom;
 					newhightext.fromport = pe->exportproto;
-					newhightext.fromvar = NOVARIABLE;
+					newhightext.fromvarnoeval = newhightext.fromvar = NOVARIABLE;
 					us_gethightextcenter(&newhightext, &xc, &yc, &style);
 					us_gethightextsize(&newhightext, &xw, &yw, el_curwindowpart);
 					if (xc+xw/2 < slx || xc-xw/2 > shx ||
@@ -630,7 +634,8 @@ INTBIG us_selectarea(NODEPROTO *np, INTBIG slx, INTBIG shx, INTBIG sly, INTBIG s
 						if ((var->type&VDISPLAY) == 0) continue;
 						newhightext.fromgeom = geom;
 						newhightext.fromport = NOPORTPROTO;
-						newhightext.fromvar = var;
+						newhightext.fromvarnoeval = var;
+						newhightext.fromvar = evalvar(newhightext.fromvarnoeval);
 						us_gethightextcenter(&newhightext, &xc, &yc, &style);
 						us_gethightextsize(&newhightext, &xw, &yw, el_curwindowpart);
 						if (xc+xw/2 < slx || xc-xw/2 > shx ||
@@ -681,6 +686,7 @@ INTBIG us_selectarea(NODEPROTO *np, INTBIG slx, INTBIG shx, INTBIG sly, INTBIG s
 		(void)allocstring(&(*highlist)[addtotal], us_makehighlightstring(&newhigh), el_tempcluster);
 		addtotal++;
 	}
+	if (addtotal == 0) efree((char *)*highlist);
 	return(addtotal);
 }
 
@@ -844,7 +850,6 @@ GEOM **us_gethighlighted(INTBIG type, INTBIG *textcount, char ***textinfo)
 	REGISTER VARIABLE *var;
 	REGISTER INTBIG total, len, i, j, k, newcount, numtext, lx, hx, ly, hy,
 		textonnode, search;
-	INTBIG oaddr, otype;
 	REGISTER GEOM *geom;
 	REGISTER NODEPROTO *np;
 	REGISTER NODEINST *ni;
@@ -1056,22 +1061,6 @@ GEOM **us_gethighlighted(INTBIG type, INTBIG *textcount, char ***textinfo)
 		for(i=0; i<numtext; i++)
 		{
 			(void)us_makehighlight(us_selectedtextlist[i], &high);
-			if (us_nodemoveswithtext(&high))
-			{
-				us_gethighaddrtype(&high, &oaddr, &otype);
-
-				/* undraw the text */
-				if (otype == VPORTPROTO)
-				{
-					ni = ((PORTPROTO *)oaddr)->subnodeinst;
-					ni->temp1 = 0;
-				} else if (otype == VNODEINST)
-				{
-					ni = (NODEINST *)oaddr;
-					ni->temp1 = 0;
-				}
-				continue;
-			}
 
 			/* make sure we aren't moving text and object it resides on */
 			textonnode = 0;
@@ -2670,8 +2659,8 @@ void us_gethighaddrtype(HIGHLIGHT *high, INTBIG *addr, INTBIG *type)
 
 /*
  * Routine to return true if highlight "high" is text on a node that must move together.
- * This only happens for "nonlayout text" (a variable on an invisible node) or
- * for exports on an invisible pin.
+ * This only happens for "nonlayout text" (a variable on an invisible node), for exports
+ * on an invisible pin, or for any export when it is requested that the node move too.
  */
 BOOLEAN us_nodemoveswithtext(HIGHLIGHT *high)
 {
@@ -2682,13 +2671,17 @@ BOOLEAN us_nodemoveswithtext(HIGHLIGHT *high)
 	if (!high->fromgeom->entryisnode) return(FALSE);
 	ni = high->fromgeom->entryaddr.ni;
 	if (ni == NONODEINST) return(FALSE);
-	if (ni->proto != gen_invispinprim) return(FALSE);
-	if (high->fromvar == NOVARIABLE)
+	if (high->fromvar != NOVARIABLE)
 	{
-		if (high->fromport != NOPORTPROTO) return(TRUE);
+		/* moving variable text */
+		if (ni->proto != gen_invispinprim) return(FALSE);
+		if (high->fromport == NOPORTPROTO) return(TRUE);
 	} else
 	{
-		if (high->fromport == NOPORTPROTO) return(TRUE);
+		/* moving export text */
+		if ((us_useroptions&MOVENODEWITHEXPORT) == 0 && ni->proto != gen_invispinprim)
+			return(FALSE);
+		if (high->fromport != NOPORTPROTO) return(TRUE);
 	}
 	return(FALSE);
 }
