@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usrmenu.c
@@ -49,7 +50,7 @@ static GRAPHICS us_shadowdesc = {LAYERA, MENGLY, PATTERNED, PATTERNED,
 GRAPHICS us_menutext = {LAYERA, MENTXT, SOLIDC, SOLIDC,
 	{0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF}, NOVARIABLE, 0};
 
-#define MENUARCLENGTH   5	/* length of arcs in menu entries */
+#define MENUARCLENGTH   8	/* length of arcs in menu entries */
 #define MENUSHADOW      2	/* size of shadow around popup menus */
 
 /* prototypes for local routines */
@@ -70,13 +71,9 @@ static void us_pminvertline(void);
 void us_drawmenu(INTBIG scaletofit, WINDOWFRAME *mw)
 {
 	REGISTER INTBIG x, y;
-	REGISTER INTBIG i;
 	BOOLEAN placemenu;
-	INTBIG swid, shei;
 	REGISTER WINDOWPART *w;
 	REGISTER VARIABLE *var;
-	static POLYGON *poly = NOPOLYGON;
-	WINDOWPART ww;
 
 	/* re-draw windows */
 	if (scaletofit != 0) placemenu = TRUE; else
@@ -92,48 +89,6 @@ void us_drawmenu(INTBIG scaletofit, WINDOWFRAME *mw)
 	}
 	if (el_curwindowpart != NOWINDOWPART) us_highlightwindow(el_curwindowpart, TRUE);
 
-	/* draw menu bar if it exists */
-	if (us_menubarsize != 0 && el_curwindowpart != NOWINDOWPART)
-	{
-		/* get polygon */
-		(void)needstaticpolygon(&poly, 4, us_tool->cluster);
-
-		/* get size of current window */
-		getwindowframesize(el_curwindowpart->frame, &swid, &shei);
-
-		/* set drawing bounds to address the world */
-		ww.screenlx = ww.screenly = ww.uselx = ww.usely = 0;
-		ww.screenhx = ww.usehx = swid-1;   ww.screenhy = ww.usehy = shei-1;
-		ww.frame = el_curwindowpart->frame;
-		ww.state = DISPWINDOW;
-		computewindowscale(&ww);
-
-		/* draw the horizontal line separating the menu bar */
-		poly->xv[0] = 0;       poly->yv[0] = shei-us_menubarsize;
-		poly->xv[1] = swid;    poly->yv[1] = shei-us_menubarsize;
-		poly->count = 2;
-		poly->style = OPENED;
-		poly->desc = &us_box;
-		us_box.col = el_colmengly;
-		us_showpoly(poly, &ww);
-
-		/* draw the menu entries */
-		x = 0;
-		poly->desc = &us_menutext;
-		us_menutext.col = el_colmentxt;
-		poly->style = TEXTBOX;
-		TDCLEAR(poly->textdescript);
-		TDSETSIZE(poly->textdescript, TXTMENU);
-		poly->tech = el_curtech;
-		for(i=0; i<us_pulldownmenucount; i++)
-		{
-			maketruerectpoly(x, us_pulldownmenupos[i], shei-us_menubarsize, shei, poly);
-			poly->string = us_stripampersand(us_pulldowns[i]->header);
-			us_showpoly(poly, &ww);
-			x = us_pulldownmenupos[i];
-		}
-	}
-
 	/* draw menus if requested */
 	if ((us_tool->toolstate&MENUON) != 0)
 	{
@@ -166,7 +121,7 @@ void us_drawmenu(INTBIG scaletofit, WINDOWFRAME *mw)
 void us_drawmenuentry(INTBIG x, INTBIG y, char *str)
 {
 	REGISTER INTBIG i, scaleu, scaled, x1, y1, lowx, lowy, highx, highy, xl,
-		yl, xh, yh, ofun, fun, lambda, bits;
+		yl, xh, yh, ofun, fun, lambda, bits, wid, twid;
 	REGISTER NODEINST *ni;
 	NODEINST node;
 	REGISTER ARCINST *ai;
@@ -315,9 +270,11 @@ void us_drawmenuentry(INTBIG x, INTBIG y, char *str)
 			ai->proto = commandbinding.arcglyph;
 			ai->length = MENUARCLENGTH * lambda;
 			ai->width = defaultarcwidth(commandbinding.arcglyph);
-			ai->end[0].xpos = -MENUARCLENGTH*lambda/2;
+			if (ai->width > ai->length)
+				ai->length = ai->width * 2;
+			ai->end[0].xpos = -ai->length/2;
 			ai->end[0].ypos = ai->end[1].ypos = 0;
-			ai->end[1].xpos = MENUARCLENGTH*lambda/2;
+			ai->end[1].xpos = ai->length/2;
 			lowx = -(ai->width+ai->length)/2;  highx = -lowx;
 			lowy = -ai->width / 2;             highy = -lowy;
 
@@ -360,10 +317,17 @@ void us_drawmenuentry(INTBIG x, INTBIG y, char *str)
 		} else if (commandbinding.arcglyph != NOARCPROTO)
 		{
 			/* make all arcs be the same scale */
+			twid = defaultarcwidth(commandbinding.arcglyph);
 			for(tap = tech->firstarcproto; tap != NOARCPROTO; tap = tap->nextarcproto)
-				scaled = maxi(scaled, MENUARCLENGTH*lambda + defaultarcwidth(tap));
+			{
+				wid = defaultarcwidth(tap);
+				if (twid <= MENUARCLENGTH*lambda && wid <= MENUARCLENGTH*lambda)
+					scaled = maxi(scaled, MENUARCLENGTH*lambda + wid);
+				if (twid > MENUARCLENGTH*lambda && wid > MENUARCLENGTH*lambda)
+					scaled = maxi(scaled, wid*3);
+			}
 
-			/* and make them the same scale as the pins */
+			/* and make them at least the same scale as the pins */
 			for(tnp = tech->firstnodeproto; tnp != NONODEPROTO; tnp = tnp->nextnodeproto)
 			{
 				fun = us_groupfunction(tnp);
@@ -831,7 +795,7 @@ POPUPMENUITEM *us_popupmenu(POPUPMENU **cmenu, BOOLEAN *waitfordown, BOOLEAN hea
 
 		/* if the menu fits, quit */
 		if (us_pmcur.sizex+MENUSHADOW <= swid &&
-			us_pmcur.sizey+MENUSHADOW <= shei-us_menubarsize) break;
+			us_pmcur.sizey+MENUSHADOW <= shei) break;
 	}
 
 	/* no text size will fit */
@@ -857,8 +821,8 @@ POPUPMENUITEM *us_popupmenu(POPUPMENU **cmenu, BOOLEAN *waitfordown, BOOLEAN hea
 	}
 	if (us_pmcur.posx+us_pmcur.sizex+MENUSHADOW >= swid)
 		us_pmcur.posx = swid - us_pmcur.sizex-MENUSHADOW-1;
-	if (us_pmcur.posy+us_pmcur.sizey > shei-us_menubarsize)
-		us_pmcur.posy = shei - us_menubarsize - us_pmcur.sizey;
+	if (us_pmcur.posy+us_pmcur.sizey > shei)
+		us_pmcur.posy = shei - us_pmcur.sizey;
 
 	/* save the display under the menu */
 	save_pix = screensavebox(&us_pmcur.popupwindow, us_pmcur.posx,
@@ -1143,7 +1107,7 @@ BOOLEAN us_pmeachdown(INTBIG x, INTBIG y)
 	switch (us_pmcur.quitline)
 	{
 		case 1:			/* quit if off left side */
-			if (y < shei-us_menubarsize)
+			if (y < shei)
 			{
 				if (x >= us_pmcur.posx) break;
 				if (y <= maxy-us_pmcur.yshift+2 &&
@@ -1152,7 +1116,7 @@ BOOLEAN us_pmeachdown(INTBIG x, INTBIG y)
 			us_pmcur.inarea = -1;
 			return(TRUE);
 		case 2:			/* quit if off right side */
-			if (y < shei-us_menubarsize)
+			if (y < shei)
 			{
 				if (x <= us_pmcur.posx+us_pmcur.sizex) break;
 				if (y <= maxy-us_pmcur.yshift+2 &&
@@ -1285,74 +1249,6 @@ void us_setkeyequiv(POPUPMENU *pm, INTBIG i)
 	mi->attribute[len] = lastchar;
 }
 
-/*
- * routine to initialize popup menu "pm" for pulldown menu use by marking
- * sub-menu calls with ">>" and ignoring key equivalents
- */
-void us_initpulldownmenu(POPUPMENU *pm)
-{
-	REGISTER INTBIG i, slash;
-	REGISTER POPUPMENUITEM *mi;
-	REGISTER char *pt;
-	char keybuf[10], *acceleratorstring, *acceleratorprefix;
-
-	getacceleratorstrings(&acceleratorstring, &acceleratorprefix);
-	for(i=0; i<pm->total; i++)
-	{
-		mi = &pm->list[i];
-		mi->value = 0;
-		if (mi->response == NOUSERCOM) continue;
-		if (mi->response->active < 0) continue;
-		if (mi->response->menu != NOPOPUPMENU)
-		{
-			mi->value = ">>";
-			mi->maxlen = -1;
-			us_initpulldownmenu(mi->response->menu);
-			continue;
-		}
-		for(pt = mi->attribute; *pt != 0 && *pt != '/' && *pt != '\\'; pt++) ;
-		if (*pt == 0) continue;
-		slash = *pt;
-		*pt++ = 0;
-		keybuf[0] = 0;
-		if (slash == '/') strcat(keybuf, acceleratorprefix);
-		strcat(keybuf, pt);
-		(void)allocstring(&mi->value, keybuf, us_tool->cluster);
-		mi->maxlen = -1;
-	}
-}
-
-/*
- * routine to terminate popup menu "pm" for pulldown menu use by unmarking
- * sub-menu calls and key equivalents
- */
-void us_termpulldownmenu(POPUPMENU *pm)
-{
-	REGISTER INTBIG i, len;
-	REGISTER POPUPMENUITEM *mi;
-	REGISTER char *pt;
-	char *acceleratorstring, *acceleratorprefix;
-
-	getacceleratorstrings(&acceleratorstring, &acceleratorprefix);
-	len = strlen(acceleratorprefix);
-	for(i=0; i<pm->total; i++)
-	{
-		mi = &pm->list[i];
-		if (mi->response == NOUSERCOM) continue;
-		if (mi->response->active < 0) continue;
-		if (mi->response->menu != NOPOPUPMENU)
-		{
-			us_termpulldownmenu(mi->response->menu);
-			continue;
-		}
-		if (mi->value == 0) continue;
-		pt = mi->attribute;
-		if (namesamen(mi->value, acceleratorprefix, len) == 0) pt[strlen(pt)] = '/'; else
-			pt[strlen(pt)] = '\\';
-		efree(mi->value);
-	}
-}
-
 /*
  * Routine to examine popup menu "pm" and ensure that no two letters have the same
  * "&"-invocation key.
