@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usrparse.c
@@ -69,7 +70,7 @@ static USERCOM *us_fleshcommand(USERCOM*);
 static void     us_dopopupmenu(POPUPMENU*);
 static void     us_domacro(char*, INTBIG, char*[]);
 static BOOLEAN  us_expandvarlist(INTBIG **varlist, INTBIG *varlimit);
-static BOOLEAN  us_pahandlechar(INTBIG);
+static BOOLEAN  us_pahandlechar(void*, INTBIG);
 static void     us_pabackup(INTBIG to_curchar);
 static void     us_expandttybuffers(INTBIG);
 
@@ -1631,7 +1632,7 @@ static DIALOGITEM us_listdialogitems[] =
  /*  3 */ {0, {56,8,200,206}, SCROLL, ""},
  /*  4 */ {0, {8,8,54,275}, MESSAGE, ""}
 };
-DIALOG us_listdialog = {{50,75,259,369}, "", 0, 4, us_listdialogitems};
+DIALOG us_listdialog = {{50,75,259,369}, "", 0, 4, us_listdialogitems, 0, 0};
 
 /* special items for the "ttygetline" dialog: */
 #define DLST_LIST     3		/* line (scroll) */
@@ -1653,14 +1654,14 @@ INTBIG ttygetparam(char *prompt, COMCOMP *parameter, INTBIG keycount, char *para
 		us_colorreadp, us_colorwritep, us_varvep, us_nodetp, us_showop, us_librarywp,
 		us_windowrnamep, us_technologyup, us_technologyedlp, us_librarydp, us_helpp,
 		us_technologyctdp, us_textdsp, us_librarywriteformatp, us_colorhighp,
-		us_showdp, us_renameop, us_renamenp, us_findnamep,
+		us_showdp, us_renameop, us_renamenp, us_findnamep, us_librarynp,
 		us_nodetcaip, us_bindgkeyp, us_sizep, us_sizewp, us_defnodep, us_showup,
 		us_librarykp, us_findobjap, us_window3dp, us_gridp, us_technologytp, us_textfp,
 		us_renamecp, us_findnnamep, us_findexportp, us_noyesalwaysp, us_varop,
 		us_copyfacetdp, us_findnodep, us_technologycnnp, us_showep, us_purelayerp,
 		us_textsp, us_viewdp, us_interpretcp, us_findarcp, us_varvmp, us_showrp,
-		us_varbdp, us_portcp, us_showfp;
-	REGISTER void *infstr;
+		us_varbdp, us_portcp, us_showfp, us_varbs1p, us_findintp, us_viewc1p;
+	REGISTER void *infstr, *dia;
 
 	/* is this necessary?  if not, us_pathiskey does not need to be global */
 	us_pathiskey = parameter->ifmatch;
@@ -1681,16 +1682,19 @@ INTBIG ttygetparam(char *prompt, COMCOMP *parameter, INTBIG keycount, char *para
 	if (parameter == &us_varvmp)              return(us_attrenumdlog());
 	if (parameter == &us_varop)               return(us_attributesdlog());
 	if (parameter == &us_varbdp)              return(us_attrparamdlog());
+	if (parameter == &us_varbs1p)             return(us_attrreportdlog());
 	if (parameter == &us_copyfacetp)          return(us_copyfacetdlog(prompt));
 	if (parameter == &us_defarcsp)            return(us_defarcdlog());
 	if (parameter == &us_defnodexsp)          return(us_defnodedlog());
 	if (parameter == &us_textdsp)             return(us_deftextdlog(prompt));
 	if (parameter == &us_technologyedlp)      return(us_dependentlibdlog());
 	if (parameter == &us_editfacetp)          return(us_editfacetdlog(prompt));
+	if (parameter == &us_librarynp)           return(us_examineoptionsdlog());
 	if (parameter == &us_defnodep)            return(us_facetdlog());
 	if (parameter == &us_showfp)              return(us_facetlist());
-	if (parameter == &us_showdp)              return(us_facetselect(prompt, paramstart, TRUE));
-	if (parameter == &us_showup)              return(us_facetselect(_("Facet to delete"), paramstart, FALSE));
+	if (parameter == &us_showdp)              return(us_facetselect(prompt, paramstart, 0));
+	if (parameter == &us_viewc1p)             return(us_facetselect(prompt, paramstart, 1));
+	if (parameter == &us_showup)              return(us_facetselect(_("Facet to delete"), paramstart, 2));
 	if (parameter == &us_textfp)              return(us_findtextdlog());
 	if (parameter == &us_viewfp)              return(us_frameoptionsdlog());
 	if (parameter == &us_showrp)              return(us_generaloptionsdlog());
@@ -1721,6 +1725,7 @@ INTBIG ttygetparam(char *prompt, COMCOMP *parameter, INTBIG keycount, char *para
 	if (parameter == &us_renamenp)            return(us_renamedlog(VTECHNOLOGY));
 	if (parameter == &us_renamecp)            return(us_renamedlog(VPORTPROTO));
 	if (parameter == &us_findnamep)           return(us_renamedlog(VCELL));
+	if (parameter == &us_findintp)            return(us_renamedlog(VNETWORK));
 	if (parameter == &us_replacep)            return(us_replacedlog());
 	if (parameter == &us_findobjap)           return(us_selectoptdlog());
 	if (parameter == &us_findnnamep)          return(us_selectobjectdlog(0));
@@ -1803,18 +1808,19 @@ INTBIG ttygetparam(char *prompt, COMCOMP *parameter, INTBIG keycount, char *para
 	}
 
 	/* the general case: display the dialog box */
-	if (DiaInitDialog(&us_listdialog)) return(0);
-	DiaInitTextDialog(DLST_LIST, parameter->toplist, parameter->nextcomcomp, DiaNullDlogDone,
+	dia = DiaInitDialog(&us_listdialog);
+	if (dia == 0) return(0);
+	DiaInitTextDialog(dia, DLST_LIST, parameter->toplist, parameter->nextcomcomp, DiaNullDlogDone,
 		0, SCSELMOUSE|SCSELKEY|SCDOUBLEQUIT|SCHORIZBAR);
-	DiaSetText(DLST_PROMPT, prompt);
+	DiaSetText(dia, DLST_PROMPT, prompt);
 
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == OK || itemHit == CANCEL) break;
 	}
-	paramstart[0] = us_putintoinfstr(DiaGetScrollLine(DLST_LIST, DiaGetCurLine(DLST_LIST)));
-	DiaDoneDialog();
+	paramstart[0] = us_putintoinfstr(DiaGetScrollLine(dia, DLST_LIST, DiaGetCurLine(dia, DLST_LIST)));
+	DiaDoneDialog(dia);
 	if (itemHit == CANCEL) return(0);
 	return(1);
 }
@@ -1831,11 +1837,11 @@ static DIALOGITEM us_ttyfulldialogitems[] =
  /*  5 */ {0, {88,8,212,294}, SCROLL, ""},
  /*  6 */ {0, {72,56,88,270}, MESSAGE, N_("Type '?' for a list of options")}
 };
-static DIALOG us_ttyfulldialog = {{50,75,271,493}, N_("Command Completion"), 0, 6, us_ttyfulldialogitems};
+static DIALOG us_ttyfulldialog = {{50,75,271,493}, N_("Command Completion"), 0, 6, us_ttyfulldialogitems, 0, 0};
 
 /* special items for the "ttygetline" dialog: */
 #define DGTF_LINE     3		/* line (edit text) */
-#define DGTF_PROMPT   4		/* prompt (edit text) */
+#define DGTF_PROMPT   4		/* prompt (stat text) */
 #define DGTF_OPTIONS  5		/* options (scroll) */
 
 /*
@@ -1848,7 +1854,7 @@ INTBIG ttygetfullparam(char *prompt, COMCOMP *parameter, INTBIG keycount,
 {
 	INTBIG i, itemHit;
 	char *line;
-	REGISTER void *infstr;
+	REGISTER void *infstr, *dia;
 
 	/* initialize for command completion */
 	us_expandttybuffers(80);
@@ -1861,8 +1867,9 @@ INTBIG ttygetfullparam(char *prompt, COMCOMP *parameter, INTBIG keycount,
 	us_painquote = FALSE;
 
 	/* display the dialog box */
-	if (DiaInitDialog(&us_ttyfulldialog)) return(-1);
-	DiaInitTextDialog(DGTF_OPTIONS, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1, 0);
+	dia = DiaInitDialog(&us_ttyfulldialog);
+	if (dia == 0) return(-1);
+	DiaInitTextDialog(dia, DGTF_OPTIONS, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1, 0);
 
 	/* load header message */
 	infstr = initinfstr();
@@ -1872,20 +1879,20 @@ INTBIG ttygetfullparam(char *prompt, COMCOMP *parameter, INTBIG keycount,
 		if (parameter->def != 0)
 			formatinfstr(infstr, " (%s)", parameter->def);
 	} else addstringtoinfstr(infstr, prompt);
-	DiaSetText(DGTF_PROMPT, returninfstr(infstr));
+	DiaSetText(dia, DGTF_PROMPT, returninfstr(infstr));
 
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == OK || itemHit == CANCEL) break;
 
 		if (itemHit == DGTF_LINE)
 		{
-			line = DiaGetText(DGTF_LINE);
+			line = DiaGetText(dia, DGTF_LINE);
 			for(i=0; line[i] == us_pattyline[i] && line[i] != 0; i++);
 			if (us_pattyline[i] != 0) us_pabackup(i);
-			for(; line[i] != 0; i++) us_pahandlechar(line[i]);
-			if (strcmp(us_pattyline, line) != 0) DiaSetText(DGTF_LINE, us_pattyline);
+			for(; line[i] != 0; i++) us_pahandlechar(dia, line[i]);
+			if (strcmp(us_pattyline, line) != 0) DiaSetText(dia, DGTF_LINE, us_pattyline);
 		}
 	}
 
@@ -1898,14 +1905,14 @@ INTBIG ttygetfullparam(char *prompt, COMCOMP *parameter, INTBIG keycount,
 		if(us_paparamcount>keycount) us_paparamcount = keycount;
 		for(i=0; i<us_paparamcount; i++) paramstart[i] = us_paparamstart[i];
 	}
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 	return(us_paparamcount);
 }
 
 /*
  * Routine to handle next char
  */
-static BOOLEAN us_pahandlechar(INTBIG c)
+static BOOLEAN us_pahandlechar(void *dia, INTBIG c)
 {
 	REGISTER INTBIG i, j, k, nofill, noshoall, isbreak, nmatch;
 	REGISTER BOOLEAN topval;
@@ -1943,7 +1950,7 @@ static BOOLEAN us_pahandlechar(INTBIG c)
 	if (c == '?' && !us_painquote)
 	{
 		/* initialize scrolled list of choices */
-		DiaLoadTextDialog(5, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1);
+		DiaLoadTextDialog(dia, 5, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1);
 
 		compare = initialline;
 		topval = (*toplist)(&compare);
@@ -1958,7 +1965,7 @@ static BOOLEAN us_pahandlechar(INTBIG c)
 				addtoinfstr(infstr, ')');
 			}
 			if (topval) addtoinfstr(infstr, ':');
-			DiaStuffLine(5, returninfstr(infstr));
+			DiaStuffLine(dia, 5, returninfstr(infstr));
 		}
 
 		if (*compare != 0 || noshoall == 0)
@@ -1970,18 +1977,18 @@ static BOOLEAN us_pahandlechar(INTBIG c)
 				if (j == -2) j = strlen(pt);
 				if (j > i) i = j;
 			}
-			if (i == 0 && *compare != 0) ttybeep(1); else
+			if (i == 0 && *compare != 0) ttybeep(SOUNDBEEP, TRUE); else
 			{
 				compare = initialline;
 				for((void)(*toplist)(&compare); (pt = (*nextinlist)()); )
 				{
 					j = stringmatch(pt, compare);
 					if (j == -1) continue;
-					if (j == i || j == -2) DiaStuffLine(5, pt);
+					if (j == i || j == -2) DiaStuffLine(dia, 5, pt);
 				}
 			}
 		}
-		DiaSelectLine(5, -1);
+		DiaSelectLine(dia, 5, -1);
 		return(FALSE);
 	}
 
@@ -2041,7 +2048,7 @@ static BOOLEAN us_pahandlechar(INTBIG c)
 			}
 
 			/* print an error bell if there is ambiguity */
-			if (nmatch != 1 && nofill == 0 && c != ESCKEY) ttybeep(1);
+			if (nmatch != 1 && nofill == 0 && c != ESCKEY) ttybeep(SOUNDBEEP, TRUE);
 		};
 
 		/* if an escape was typed, stop after filling in the keyword */
