@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usrstatus.c
@@ -34,6 +35,7 @@
 #include "drc.h"
 #include "network.h"
 #include "efunction.h"
+#include "edialogs.h"
 #include "tecgen.h"
 #include "tecschem.h"
 #include "usrdiacom.h"
@@ -43,6 +45,10 @@ static INTBIG *us_layer_letter_data = 0;	/* cache for "us_layerletters" */
 /* working memory for "us_uniqueportname()" */
 static char *us_uniqueretstr = NOSTRING;
 
+/* working memory for "us_shadowarcproto()" */
+static char *us_lastnetworkname;
+static INTBIG us_lastnetworknametotal = 0;
+
 /* structure for "usrbits" information parsing */
 struct explanation
 {
@@ -73,9 +79,9 @@ void us_freestatusmemory(void)
 	if (us_statusarc != 0) ttyfreestatusfield(us_statusarc);
 	if (us_statusfacet != 0) ttyfreestatusfield(us_statusfacet);
 	if (us_statusfacetsize != 0) ttyfreestatusfield(us_statusfacetsize);
+	if (us_statusselectcount != 0) ttyfreestatusfield(us_statusselectcount);
 	if (us_statusgridsize != 0) ttyfreestatusfield(us_statusgridsize);
 	if (us_statuslambda != 0) ttyfreestatusfield(us_statuslambda);
-	if (us_statusnetwork != 0) ttyfreestatusfield(us_statusnetwork);
 	if (us_statusnode != 0) ttyfreestatusfield(us_statusnode);
 	if (us_statustechnology != 0) ttyfreestatusfield(us_statustechnology);
 	if (us_statusxpos != 0) ttyfreestatusfield(us_statusxpos);
@@ -90,6 +96,7 @@ void us_freestatusmemory(void)
 	for(i=0; i<lines; i++) ttyfreestatusfield(us_clearstatuslines[i]);
 
 	if (us_uniqueretstr != NOSTRING) efree(us_uniqueretstr);
+	if (us_lastnetworknametotal > 0) efree((char *)us_lastnetworkname);
 }
 
 /******************** STATUS DISPLAY MANIPULATION ********************/
@@ -105,24 +112,25 @@ void us_initstatus(void)
 		first = FALSE;
 		lines = ttynumstatuslines();
 		for(i=0; i<lines; i++) us_clearstatuslines[i] = ttydeclarestatusfield(i+1, 0, 100, "");
-		us_statusfacet = us_statusnode = us_statusarc = us_statusnetwork = 0;
+		us_statusfacet = us_statusnode = us_statusarc = 0;
 		us_statuslambda = us_statustechnology = us_statusxpos = us_statusypos = 0;
 		us_statusangle = us_statusalign = us_statusfacetsize = us_statusgridsize = 0;
 		us_statusproject = us_statusroot = us_statuspart = us_statuspackage = 0;
-		us_statusselection = 0;
-		us_statusfacet      = ttydeclarestatusfield(0,  0,   0, "");
-		us_statusnode       = ttydeclarestatusfield(1,  0,  22, _("NODE: "));
-		us_statusarc        = ttydeclarestatusfield(1, 22,  44, _("ARC: "));
-		us_statusfacetsize  = ttydeclarestatusfield(1, 44,  65, _("SIZE: "));
-		us_statuslambda     = ttydeclarestatusfield(1, 65,  80, _("LAMBDA: "));
-		us_statustechnology = ttydeclarestatusfield(1, 80, 100, _("TECHNOLOGY: "));
-		us_statusxpos       = ttydeclarestatusfield(1, 65,  80, _("X: "));
-		us_statusypos       = ttydeclarestatusfield(1, 80, 100, _("Y: "));
+		us_statusselection = us_statusselectcount = 0;
+		us_statusfacet       = ttydeclarestatusfield(0,  0,   0, "");
+		us_statusnode        = ttydeclarestatusfield(1,  0,  20, _("NODE: "));
+		us_statusarc         = ttydeclarestatusfield(1, 20,  40, _("ARC: "));
+		us_statusselectcount = ttydeclarestatusfield(1, 40,  52, _("SELECTED: "));
+		us_statusfacetsize   = ttydeclarestatusfield(1, 52,  69, _("SIZE: "));
+		us_statuslambda      = ttydeclarestatusfield(1, 69,  82, _("LAMBDA: "));
+		us_statustechnology  = ttydeclarestatusfield(1, 82, 100, _("TECHNOLOGY: "));
+		us_statusxpos        = ttydeclarestatusfield(1, 69,  82, _("X: "));
+		us_statusypos        = ttydeclarestatusfield(1, 82, 100, _("Y: "));
 		if (lines > 1)
 		{
-			us_statusangle     = ttydeclarestatusfield(2,  0,  25, _("ANGLE: "));
-			us_statusalign     = ttydeclarestatusfield(2, 25,  50, _("ALIGN: "));
-			us_statusgridsize  = ttydeclarestatusfield(2, 75, 100, _("GRID: "));
+			us_statusangle    = ttydeclarestatusfield(2,  0,  25, _("ANGLE: "));
+			us_statusalign    = ttydeclarestatusfield(2, 25,  50, _("ALIGN: "));
+			us_statusgridsize = ttydeclarestatusfield(2, 75, 100, _("GRID: "));
 		}
 	}
 }
@@ -145,6 +153,9 @@ STATUSFIELD *ttydeclarestatusfield(INTBIG line, INTBIG startper, INTBIG endper,
 	sf->startper = startper;
 	sf->endper = endper;
 	(void)allocstring(&sf->label, label, us_tool->cluster);
+#ifdef USEQT
+	ttysetstatusfield(NOWINDOWFRAME, sf, "", FALSE);
+#endif
 	return(sf);
 }
 
@@ -204,13 +215,13 @@ void us_setcursorpos(WINDOWFRAME *frame, INTBIG x, INTBIG y)
 
 	if (lastx != x)
 	{
-		ttysetstatusfield(frame, us_statusxpos, latoa(x), FALSE);
+		ttysetstatusfield(frame, us_statusxpos, latoa(x, 0), FALSE);
 		lastx = x;
 	}
 
 	if (lasty != y)
 	{
-		ttysetstatusfield(frame, us_statusypos, latoa(y), FALSE);
+		ttysetstatusfield(frame, us_statusypos, latoa(y, 0), FALSE);
 		lasty = y;
 	}
 }
@@ -244,78 +255,74 @@ void us_shadownodeproto(WINDOWFRAME *frame, NODEPROTO *np)
 	REGISTER NODEINST *ni;
 	REGISTER void *infstr;
 
-	if (us_curnodeproto == np) return;
-
 	/* code cannot be called by multiple procesors: uses globals */
 	NOT_REENTRANT;
 
 	us_curnodeproto = np;
 
 	if (np == NONODEPROTO) pt = ""; else
-	{
 		pt = describenodeproto(np);
 
-		/* see if this type of node is selected */
-		var = getvalkey((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, us_highlightedkey);
-		if (var != NOVARIABLE)
+	/* see if the selection further qualifies the node description */
+	var = getvalkey((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, us_highlightedkey);
+	if (var != NOVARIABLE)
+	{
+		len = getlength(var);
+		if (len == 1)
 		{
-			len = getlength(var);
-			if (len == 1)
+			if (!us_makehighlight(((char **)var->addr)[0], &high))
 			{
-				if (!us_makehighlight(((char **)var->addr)[0], &high))
+				if ((high.status&HIGHTEXT) != 0)
 				{
-					if ((high.status&HIGHTEXT) != 0)
+					infstr = initinfstr();
+					var = high.fromvar;
+					if (var == NOVARIABLE)
 					{
-						infstr = initinfstr();
-						var = high.fromvar;
-						if (var == NOVARIABLE)
+						if (high.fromport == NOPORTPROTO)
 						{
-							if (high.fromport == NOPORTPROTO)
-							{
-								formatinfstr(infstr, _("%s (facet name)"),
-									describenodeinst(high.fromgeom->entryaddr.ni));
-							} else
-							{
-								formatinfstr(infstr, _("%s (export)"), high.fromport->protoname);
-							}
+							formatinfstr(infstr, _("%s (facet name)"),
+								describenodeinst(high.fromgeom->entryaddr.ni));
 						} else
 						{
-							if (high.fromgeom == NOGEOM)
+							formatinfstr(infstr, _("%s (export)"), high.fromport->protoname);
+						}
+					} else
+					{
+						if (high.fromgeom == NOGEOM)
+						{
+							formatinfstr(infstr, _("%s on facet"),
+								us_trueattributename(var));
+						} else if (high.fromport != NOPORTPROTO)
+						{
+							formatinfstr(infstr, _("%s on export"),
+								us_trueattributename(var));
+						} else
+						{
+							if (high.fromgeom->entryisnode)
 							{
-								formatinfstr(infstr, _("%s on instance"),
-									us_trueattributename(var));
-							} else if (high.fromport != NOPORTPROTO)
-							{
-								formatinfstr(infstr, _("%s on export"),
-									us_trueattributename(var));
-							} else
-							{
-								if (high.fromgeom->entryisnode)
+								ni = high.fromgeom->entryaddr.ni;
+								if (ni->proto == gen_invispinprim)
 								{
-									ni = high.fromgeom->entryaddr.ni;
-									if (ni->proto == gen_invispinprim)
-									{
-										addstringtoinfstr(infstr, us_invisiblepintextname(var));
-									} else
-									{
-										formatinfstr(infstr, _("%s on node"), us_trueattributename(var));
-									}
+									addstringtoinfstr(infstr, us_invisiblepintextname(var));
 								} else
 								{
-									formatinfstr(infstr, _("%s on arc"), us_trueattributename(var));
+									formatinfstr(infstr, _("%s on node"), us_trueattributename(var));
 								}
+							} else
+							{
+								formatinfstr(infstr, _("%s on arc"), us_trueattributename(var));
 							}
 						}
-						pt = returninfstr(infstr);
 					}
-					if ((high.status&HIGHFROM) != 0 && high.fromgeom->entryisnode)
+					pt = returninfstr(infstr);
+				}
+				if ((high.status&HIGHFROM) != 0 && high.fromgeom->entryisnode)
+				{
+					ni = high.fromgeom->entryaddr.ni;
+					if (ni->proto == np)
 					{
-						ni = high.fromgeom->entryaddr.ni;
-						if (ni->proto == np)
-						{
-							/* one node selected that is the same as the current proto: qualify proto */
-							pt = us_describenodeinsttype(np, ni->userbits&NTECHBITS);
-						}
+						/* one node selected that is the same as the current proto: qualify proto */
+						pt = us_describenodeinsttype(np, ni->userbits&NTECHBITS);
 					}
 				}
 			}
@@ -365,19 +372,50 @@ void us_showcurrentnodeproto(void)
 void us_shadowarcproto(WINDOWFRAME *frame, ARCPROTO *ap)
 {
 	REGISTER ARCPROTO *oldap;
+	REGISTER NETWORK *net;
+	REGISTER char *pt;
+	REGISTER INTBIG len;
+	REGISTER void *infstr;
 
-	if (us_curarcproto == ap) return;
+	/* first show the arc/network in the status bar */
+	net = net_gethighlightednet(FALSE, FALSE);
+	if (net == NONETWORK || (net->globalnet < 0 && net->namecount <= 0)) pt = ""; else
+		pt = describenetwork(net);
+	if (ap != us_curarcproto || us_lastnetworkname == 0 || namesame(us_lastnetworkname, pt) != 0)
+	{
+		len = strlen(pt) + 1;
+		if (len > us_lastnetworknametotal)
+		{
+			if (us_lastnetworknametotal > 0) efree((char *)us_lastnetworkname);
+			us_lastnetworknametotal = 0;
+			us_lastnetworkname = (char *)emalloc(len, us_tool->cluster);
+			if (us_lastnetworkname == 0) return;
+			us_lastnetworknametotal = len;
+		}
+		strcpy(us_lastnetworkname, pt);
+		pt = describearcproto(ap);
+		if (*us_lastnetworkname != 0)
+		{
+			infstr = initinfstr();
+			formatinfstr(infstr, "%s [%s]", pt, us_lastnetworkname);
+			pt = returninfstr(infstr);
+		}
 
-	/* undraw the menu entry with the last arc proto in it */
-	oldap = us_curarcproto;
-	us_curarcproto = ap;
+		ttysetstatusfield(frame, us_statusarc, pt, TRUE);
+	}
 
-	/* adjust the component menu entries */
-	if (oldap != NOARCPROTO) us_drawarcmenu(oldap);
-	if (us_curarcproto != NOARCPROTO) us_drawarcmenu(us_curarcproto);
+	/* next handle menu highlighting */
+	if (us_curarcproto != ap)
+	{
+		/* undraw the menu entry with the last arc proto in it */
+		oldap = us_curarcproto;
+		us_curarcproto = ap;
 
-	ttysetstatusfield(frame, us_statusarc, describearcproto(ap), TRUE);
-	us_showcurrentarcproto();
+		/* adjust the component menu entries */
+		if (oldap != NOARCPROTO) us_drawarcmenu(oldap);
+		if (us_curarcproto != NOARCPROTO) us_drawarcmenu(us_curarcproto);
+		us_showcurrentarcproto();
+	}
 }
 
 /*
@@ -443,18 +481,6 @@ void us_showcurrentarcproto(void)
 	}
 }
 
-/* show the current network name */
-void us_shownetworkname(WINDOWFRAME *frame)
-{
-	REGISTER NETWORK *net;
-	REGISTER char *hold;
-
-	net = net_gethighlightednet(FALSE, FALSE);
-	if (net == NONETWORK) hold = ""; else
-		hold = describenetwork(net);
-	ttysetstatusfield(frame, us_statusnetwork, hold, FALSE);
-}
-
 /* write the default placement angle from "tool:user.USER_placement_angle" */
 void us_setnodeangle(WINDOWFRAME *frame)
 {
@@ -526,6 +552,28 @@ void us_setfacetname(WINDOWPART *w)
 	ttysetstatusfield(w->frame, us_statusfacet, returninfstr(infstr), TRUE);
 }
 
+/*
+ * Routine to update the number of selected objects.
+ */
+void us_setselectioncount(void)
+{
+	char valstring[50];
+	REGISTER VARIABLE *var;
+	REGISTER WINDOWPART *w;
+
+	valstring[0] = 0;
+	var = getvalkey((INTBIG)us_tool, VTOOL, VSTRING|VISARRAY, us_highlightedkey);
+	if (var != NOVARIABLE)
+		sprintf(valstring, "%ld", getlength(var));
+
+	for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
+	{
+		if ((w->state&WINDOWTYPE) == DISPWINDOW)
+			ttysetstatusfield(w->frame, us_statusselectcount, valstring, FALSE); else
+				ttysetstatusfield(w->frame, us_statusselectcount, "", FALSE);
+	}
+}
+
 /*
  * write the size of the facet in window "w" from the size of "w->curnodeproto"
  */
@@ -533,7 +581,7 @@ void us_setfacetsize(WINDOWPART *w)
 {
 	REGISTER NODEPROTO *np;
 	char valstring[50];
-	REGISTER INTBIG lines;
+	REGISTER INTBIG lines, lambda;
 
 	np = w->curnodeproto;
 	valstring[0] = 0;
@@ -548,9 +596,10 @@ void us_setfacetsize(WINDOWPART *w)
 				break;
 			case DISPWINDOW:
 			case DISP3DWINDOW:
-				(void)strcpy(valstring, latoa(np->highx - np->lowx));
+				lambda = lambdaoffacet(np);
+				(void)strcpy(valstring, latoa(np->highx - np->lowx, lambda));
 				(void)strcat(valstring, "x");
-				(void)strcat(valstring, latoa(np->highy - np->lowy));
+				(void)strcat(valstring, latoa(np->highy - np->lowy, lambda));
 				break;
 		}
 	}
@@ -598,7 +647,6 @@ void us_redostatus(WINDOWFRAME *frame)
 	us_shadownodeproto(NOWINDOWFRAME, np);
 	us_shadowarcproto(NOWINDOWFRAME, ap);
 
-#ifdef USEQT
 	/* write the window information on the window lines */
 	for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
 	{
@@ -607,30 +655,15 @@ void us_redostatus(WINDOWFRAME *frame)
 			us_setfacetsize(w);
 		us_setgridsize(w);
 	}
+	if (el_curwindowpart != NOWINDOWPART)
+		us_setfacetsize(el_curwindowpart);
 
 	/* other fields */
-	us_shownetworkname(frame);
 	us_setnodeangle(frame);
 	us_setalignment(frame);
 	us_setlambda(frame);
 	us_settechname(frame);
-#else
-	/* other fields */
-	us_shownetworkname(frame);
-	us_setnodeangle(frame);
-	us_settechname(frame);
-	us_setalignment(frame);
-	us_setlambda(frame);
-
-	/* write the window information on the window lines */
-	for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
-	{
-		us_setfacetname(w);
-		if (graphicshas(CANSTATUSPERFRAME) || w->frame == frame)
-			us_setfacetsize(w);
-		us_setgridsize(w);
-	}
-#endif
+	us_setselectioncount();
 }
 
 /******************** OUTPUT PREPARATION ********************/
@@ -1176,7 +1209,7 @@ void us_chatportproto(NODEINST *ni, PORTPROTO *pp)
 	REGISTER PORTARCINST *pi;
 	REGISTER PORTEXPINST *pe;
 	REGISTER ARCINST *ai;
-	REGISTER INTBIG i;
+	REGISTER INTBIG i, lambda;
 	INTBIG lx, hx, ly, hy;
 	REGISTER char *activity;
 	char locx[40], locy[40];
@@ -1191,6 +1224,7 @@ void us_chatportproto(NODEINST *ni, PORTPROTO *pp)
 	pp->temp1++;
 
 	/* talk about the port prototype */
+	lambda = lambdaofnode(ni);
 	infstr = initinfstr();
 	activity = describeportbits(pp);
 	if (*activity != 0)
@@ -1216,19 +1250,19 @@ void us_chatportproto(NODEINST *ni, PORTPROTO *pp)
 			if (poly->xv[i] != poly->xv[i-1] || poly->yv[i] != poly->yv[i-1]) break;
 		if (i >= poly->count)
 		{
-			ttyputmsg(_("  Located at (%s, %s)"), latoa(poly->xv[0]), latoa(poly->yv[0]));
+			ttyputmsg(_("  Located at (%s, %s)"), latoa(poly->xv[0], lambda), latoa(poly->yv[0], lambda));
 		} else if (isbox(poly, &lx,&hx, &ly,&hy))
 		{
-			if (lx == hx) (void)sprintf(locx, _("at %s"), latoa(lx)); else
-				(void)sprintf(locx, _("from %s to %s"), latoa(lx), latoa(hx));
-			if (ly == hy) (void)sprintf(locy, _("at %s"), latoa(ly)); else
-				(void)sprintf(locy, _("from %s to %s"), latoa(ly), latoa(hy));
+			if (lx == hx) (void)sprintf(locx, _("at %s"), latoa(lx, lambda)); else
+				(void)sprintf(locx, _("from %s to %s"), latoa(lx, lambda), latoa(hx, lambda));
+			if (ly == hy) (void)sprintf(locy, _("at %s"), latoa(ly, lambda)); else
+				(void)sprintf(locy, _("from %s to %s"), latoa(ly, lambda), latoa(hy, lambda));
 			ttyputmsg(_("  Located %s in X and %s in Y"), locx, locy);
 		} else
 		{
 			infstr = initinfstr();
 			for(i=0; i<poly->count; i++)
-				formatinfstr(infstr, " (%s, %s)", latoa(poly->xv[i]), latoa(poly->yv[i]));
+				formatinfstr(infstr, " (%s, %s)", latoa(poly->xv[i], lambda), latoa(poly->yv[i], lambda));
 			ttyputmsg(_("  Located inside polygon%s"), returninfstr(infstr));
 		}
 	} else ttyputmsg(_("  Located at subnode %s subport %s"),
@@ -1240,8 +1274,8 @@ void us_chatportproto(NODEINST *ni, PORTPROTO *pp)
 		if (pi->proto != pp) continue;
 		ai = pi->conarcinst;
 		if (ai->end[0].portarcinst == pi) i = 0; else i = 1;
-		ttyputmsg(_("  Connected at (%s,%s) to %s"), latoa(ai->end[i].xpos),
-			latoa(ai->end[i].ypos), describearcinst(ai));
+		ttyputmsg(_("  Connected at (%s,%s) to %s"), latoa(ai->end[i].xpos, lambda),
+			latoa(ai->end[i].ypos, lambda), describearcinst(ai));
 	}
 
 	/* talk about any exports of this prototype */
@@ -1390,8 +1424,6 @@ char *us_bettervariablename(char *name)
 char *us_variableunits(VARIABLE *var)
 {
 	REGISTER INTBIG units;
-	extern char *us_capacitancenames[], *us_resistancenames[], *us_inductancenames[],
-		*us_currentnames[], *us_voltagenames[], *us_timenames[];
 
 	/* specialized fields that adjust with units */
 	units = TDGETUNITS(var->textdescript);
@@ -1499,7 +1531,7 @@ void us_printcolorvalue(INTBIG ind)
 char *us_makefacetline(NODEPROTO *np, INTBIG maxlen)
 {
 	char line[40];
-	REGISTER INTBIG l, i, gooddrc, len, total;
+	REGISTER INTBIG l, i, gooddrc, len, total, lambda;
 	INTBIG year, month, mday, hour, minute, second;
 	REGISTER char *pt;
 	REGISTER VARIABLE *var;
@@ -1531,7 +1563,7 @@ char *us_makefacetline(NODEPROTO *np, INTBIG maxlen)
 	{
 		if (maxlen < 0)
 		{
-			parsetime(np->creationdate, &year, &month, &mday, &hour, &minute, &second);
+			parsetime((time_t)np->creationdate, &year, &month, &mday, &hour, &minute, &second);
 			sprintf(line, "%4ld-%02ld-%02ld %02ld:%02ld:%02ld", year, month, mday, hour,
 				minute, second);
 			addstringtoinfstr(infstr, line);
@@ -1561,7 +1593,7 @@ char *us_makefacetline(NODEPROTO *np, INTBIG maxlen)
 	{
 		if (maxlen < 0)
 		{
-			parsetime(np->revisiondate, &year, &month, &mday, &hour, &minute, &second);
+			parsetime((time_t)np->revisiondate, &year, &month, &mday, &hour, &minute, &second);
 			sprintf(line, "%4ld-%02ld-%02ld %02ld:%02ld:%02ld", year, month, mday, hour,
 				minute, second);
 			addstringtoinfstr(infstr, line);
@@ -1593,14 +1625,15 @@ char *us_makefacetline(NODEPROTO *np, INTBIG maxlen)
 		addstringtoinfstr(infstr, line);
 	} else
 	{
-		strcpy(line, latoa(np->highx-np->lowx));
+		lambda = lambdaoffacet(np);
+		strcpy(line, latoa(np->highx-np->lowx, lambda));
 		if (maxlen >= 0)
 		{
 			for(i = strlen(line); i<8; i++) addtoinfstr(infstr, ' ');
 		}
 		addstringtoinfstr(infstr, line);
 		addtoinfstr(infstr, 'x');
-		strcpy(line, latoa(np->highy-np->lowy));
+		strcpy(line, latoa(np->highy-np->lowy, lambda));
 		addstringtoinfstr(infstr, line);
 		if (maxlen >= 0)
 		{
@@ -1619,11 +1652,11 @@ char *us_makefacetline(NODEPROTO *np, INTBIG maxlen)
 	/* show other factors about the facet */
 	if (maxlen < 0) addtoinfstr(infstr, '\t'); else
 		addstringtoinfstr(infstr, "   ");
-	if ((np->userbits&NLOCKED) != 0) addstringtoinfstr(infstr, "L"); else
+	if ((np->userbits&NPLOCKED) != 0) addstringtoinfstr(infstr, "L"); else
 		addstringtoinfstr(infstr, " ");
 	if (maxlen < 0) addtoinfstr(infstr, '\t'); else
 		addstringtoinfstr(infstr, " ");
-	if ((np->userbits&NILOCKED) != 0) addstringtoinfstr(infstr, "I"); else
+	if ((np->userbits&NPILOCKED) != 0) addstringtoinfstr(infstr, "I"); else
 		addstringtoinfstr(infstr, " ");
 	if (maxlen < 0) addtoinfstr(infstr, '\t'); else
 		addstringtoinfstr(infstr, " ");
@@ -1714,7 +1747,7 @@ BOOLEAN us_preventloss(LIBRARY *lib, INTBIG action, BOOLEAN cancancel)
 	{
 		if (optionshavechanged())
 		{
-			if (us_saveoptdlog())
+			if (us_saveoptdlog(TRUE))
 			{
 				ttyputverbose(M_("Keep working"));
 				return(TRUE);
@@ -1731,22 +1764,22 @@ BOOLEAN us_preventloss(LIBRARY *lib, INTBIG action, BOOLEAN cancancel)
 void us_saveoptions(void)
 {
 	REGISTER INTBIG retval;
-	REGISTER LIBRARY *l;
+	REGISTER LIBRARY *lib;
 	REGISTER char *libname, *libfile;
 
 	libname = us_tempoptionslibraryname();
 	libfile = optionsfilepath();
-	l = newlibrary(libname, libfile);
-	if (l == NOLIBRARY)
+	lib = newlibrary(libname, libfile);
+	if (lib == NOLIBRARY)
 	{
 		ttyputerr(_("Cannot create options library %s"), libfile);
 		return;
 	}
-	l->userbits |= READFROMDISK | HIDDENLIBRARY;
-	retval = asktool(io_tool, "write", (INTBIG)l, (INTBIG)"nobackupbinary");
-	killlibrary(l);
-	if (retval != 0)
-		ttyputerr(_("Could not save options library"));
+	lib->userbits |= READFROMDISK | HIDDENLIBRARY;
+	retval = asktool(io_tool, "write", (INTBIG)lib, (INTBIG)"nobackupbinary");
+	if (retval != 0) ttyputerr(_("Could not save options library")); else
+		ttyputmsg(_("Options have been saved to %s"), lib->libfile);
+	killlibrary(lib);
 	us_optionschanged = FALSE;
 }
 
@@ -1769,6 +1802,162 @@ char *us_tempoptionslibraryname(void)
 	return(libname);
 }
 
+/*
+ * Routine to ensure that everything gets saved.
+ */
+void us_saveeverything(void)
+{
+	REGISTER INTBIG total, i, l, retval;
+	REGISTER LIBRARY *lib, *deplib, **libsave;
+	REGISTER NODEPROTO *np;
+	REGISTER NODEINST *ni;
+
+	/* see which libraries need to be saved */
+	total = 0;
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+	{
+		if ((lib->userbits&HIDDENLIBRARY) != 0) continue;
+		if ((lib->userbits&(LIBCHANGEDMAJOR | LIBCHANGEDMINOR)) == 0) continue;
+		total++;
+	}
+	if (total == 0) return;
+	libsave = (LIBRARY **)emalloc(total * (sizeof (LIBRARY *)), el_tempcluster);
+	if (libsave == 0) return;
+
+	/* make a list of the libraries to save */
+	total = 0;
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+	{
+		if ((lib->userbits&HIDDENLIBRARY) != 0) continue;
+		if ((lib->userbits&(LIBCHANGEDMAJOR | LIBCHANGEDMINOR)) == 0) continue;
+		libsave[total++] = lib;
+	}
+
+	/* be sure the most referenced libraries are saved first */
+	for(i=0; i<total; i++) libsave[i]->temp1 = i;
+	for(i=0; i<total; i++)
+	{
+		lib = libsave[i];
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		{
+			for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+			{
+				if (ni->proto->primindex != 0) continue;
+				deplib = ni->proto->cell->lib;
+				if (deplib == lib) continue;
+				if (deplib->temp1 <= lib->temp1) continue;
+				l = deplib->temp1;   deplib->temp1 = lib->temp1;   lib->temp1 = l;
+			}
+		}
+	}
+	esort(libsave, total, sizeof (LIBRARY *), us_librarytemp1ascending);
+
+	/* now save every library that needs it */
+	for(i=0; i<total; i++)
+	{
+		lib = libsave[i];
+
+		/* save the library in binary format */
+		makeoptionstemporary(lib);
+		retval = asktool(io_tool, "write", (INTBIG)lib, (INTBIG)"binary");
+		restoreoptionstate(lib);
+		if (retval != 0) break;
+	}
+
+	/* clean up */
+	efree((char *)libsave);
+
+	/* save options if any have changed */
+	if (us_optionschanged)
+	{
+		if (optionshavechanged())
+			(void)us_saveoptdlog(FALSE);
+	}
+}
+
+/* Prompt: Session Logging */
+static DIALOGITEM us_seslogdialogitems[] =
+{
+ /*  1 */ {0, {204,100,228,268}, BUTTON, N_("Save All Information")},
+ /*  2 */ {0, {240,100,264,268}, BUTTON, N_("Disable Session Logging")},
+ /*  3 */ {0, {4,4,20,284}, MESSAGE, N_("Warning: not all information has been saved.")},
+ /*  4 */ {0, {36,4,52,356}, MESSAGE, N_("Unless all libraries and options are saved together")},
+ /*  5 */ {0, {56,4,72,356}, MESSAGE, N_("It is not possible to reconstruct the session after a crash.")},
+ /*  6 */ {0, {88,4,104,356}, MESSAGE, N_("The following information has not been saved:")},
+ /*  7 */ {0, {108,4,192,356}, SCROLL, ""}
+};
+static DIALOG us_seslogdialog = {{75,75,348,441}, N_("Session Logging Warning"), 0, 7, us_seslogdialogitems, 0, 0};
+
+#define DSEL_SAVEALL    1		/* save all information (button) */
+#define DSEL_DISLOG     2		/* disable session logging (button) */
+#define DSEL_UNSAVED    7		/* what is unsaved (scroll) */
+
+void us_continuesessionlogging(void)
+{
+	REGISTER BOOLEAN unsaved;
+	REGISTER void *dia, *infstr;
+	REGISTER INTBIG itemHit;
+	REGISTER LIBRARY *lib;
+
+	/* if session logging is already off, never mind */
+	if (us_logrecord == NULL) return;
+
+	/* see if there are any unsaved libraries */
+	unsaved = FALSE;
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+	{
+		if ((lib->userbits&HIDDENLIBRARY) != 0) continue;
+		if ((lib->userbits&(LIBCHANGEDMAJOR | LIBCHANGEDMINOR)) != 0)
+			unsaved = TRUE;
+	}
+	if (us_optionschanged && optionshavechanged())
+		unsaved = TRUE;
+
+	/* if something is unsaved, prompt the user */
+	if (unsaved)
+	{
+		dia = DiaInitDialog(&us_seslogdialog);
+		if (dia == 0) return;
+		DiaInitTextDialog(dia, DSEL_UNSAVED, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1, 0);
+		for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		{
+			if ((lib->userbits&HIDDENLIBRARY) != 0) continue;
+			if ((lib->userbits&(LIBCHANGEDMAJOR | LIBCHANGEDMINOR)) == 0) continue;
+			infstr = initinfstr();
+			formatinfstr(infstr, "Library %s", lib->libname);
+			DiaStuffLine(dia, DSEL_UNSAVED, returninfstr(infstr));
+		}
+		if (us_optionschanged && optionshavechanged())
+			DiaStuffLine(dia, DSEL_UNSAVED, _("Options"));
+		DiaSelectLine(dia, DSEL_UNSAVED, -1);
+		for(;;)
+		{
+			itemHit = DiaNextHit(dia);
+			if (itemHit == DSEL_SAVEALL || itemHit == DSEL_DISLOG) break;
+		}
+		DiaDoneDialog(dia);
+		if (itemHit == DSEL_DISLOG)
+		{
+			/* disable session logging */
+			logfinishrecord();
+			return;
+		}
+
+		/* save everything */
+		us_saveeverything();
+	}
+
+	/* erase everything in the clipboard */
+	us_clearclipboard();
+
+	/* close all modeless dialogs */
+	DiaCloseAllModeless();
+
+	/* truncate the session log */
+	logfinishrecord();
+	logstartrecord();
+}
+
 /*
  * routine to determine the color map entry that corresponds to the string
  * "pp".  If the string is a number then return that number.  If the string
@@ -1867,12 +2056,10 @@ BOOLEAN us_isuniquename(char *name, NODEPROTO *facet, INTBIG type, void *exclude
 	switch (type)
 	{
 		case VPORTPROTO:
-			for(pp = facet->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-			{
-				if (exclude != 0 && (PORTPROTO *)exclude == pp) continue;
-				if (namesame(name, pp->protoname) == 0) return(FALSE);
-			}
-			break;
+			pp = getportproto(facet, name);
+			if (pp == NOPORTPROTO) break;
+			if (exclude != 0 && (PORTPROTO *)exclude == pp) break;
+			return(FALSE);
 		case VNODEINST:
 			for(ni = facet->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 			{
@@ -2034,7 +2221,7 @@ char *us_layerletters(TECHNOLOGY *tech, INTBIG layer)
 /*
  * routine to change an tool state.  The name of the tool is in "pt" (if "pt" is
  * null then an tool name is prompted).  The state of the tool is set to "state"
- * (0 for off, 1 for permanently off, 2 for on, 3 for on without catchup).
+ * (0 for off, 1 for permanently off, 2 for on).
  */
 void us_settool(INTBIG count, char *par[], INTBIG state)
 {
@@ -2079,22 +2266,18 @@ void us_settool(INTBIG count, char *par[], INTBIG state)
 		ttyputverbose(M_("%s already on"), pt);
 		return;
 	}
-	if (state <= 1) ttyputverbose(M_("%s turned off"), tool->toolname); else
-	{
-		if ((tool->toolstate&TOOLINCREMENTAL) == 0 || state == 3)
-			ttyputverbose(M_("%s turned on"), tool->toolname); else
-				ttyputmsg(_("%s turned on, catching up on changes..."), tool->toolname);
-	}
 
 	if (state <= 1)
 	{
+		ttyputverbose(M_("%s turned off"), tool->toolname);
 		if (state != 0) toolstate = TRUE; else
 			toolstate = FALSE;
 		toolturnoff(tool, toolstate);
 	} else
 	{
-		if (state == 2) toolturnon(tool, FALSE); else toolturnon(tool, TRUE);
-		if ((tool->toolstate&TOOLINCREMENTAL) != 0 && state != 3)
-			ttyputmsg(_("...%s caught up"), tool->toolname);
+		if ((tool->toolstate&TOOLINCREMENTAL) == 0)
+			ttyputverbose(M_("%s turned on"), tool->toolname); else
+				ttyputmsg(_("%s turned on, catching up on changes"), tool->toolname);
+		toolturnon(tool);
 	}
 }
