@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: dbterminal.c
@@ -107,7 +108,7 @@ void ttyputerr(char *msg, ...)
 	va_list ap;
 
 	var_start(ap, msg);
-	ttybeep(0);
+	ttybeep(SOUNDBEEP, FALSE);
 	us_ttyprint(TRUE, msg, ap);
 	va_end(ap);
 }
@@ -208,7 +209,7 @@ static DIALOGITEM us_ttyinputdialogitems[] =
  /*  3 */ {0, {58,38,74,246}, EDITTEXT, ""},
  /*  4 */ {0, {3,9,51,281}, MESSAGE, ""}
 };
-static DIALOG us_ttyinputdialog = {{50,75,188,371}, "", 0, 4, us_ttyinputdialogitems};
+static DIALOG us_ttyinputdialog = {{50,75,188,371}, "", 0, 4, us_ttyinputdialogitems, 0, 0};
 
 /* special items for the "ttygetline" dialog: */
 #define DGTL_LINE     3		/* line (edit text) */
@@ -222,6 +223,7 @@ char *ttygetline(char *prompt)
 {
 	INTBIG itemHit;
 	char *line, *pt, *defaultval, localprompt[256];
+	REGISTER void *dia;
 
 	/* parse default value */
 	strcpy(localprompt, prompt);
@@ -236,23 +238,24 @@ char *ttygetline(char *prompt)
 	}
 
 	/* display the dialog box */
-	if (DiaInitDialog(&us_ttyinputdialog)) return(0);
-	DiaSetText(DGTL_PROMPT, localprompt);
-	DiaSetText(-DGTL_LINE, defaultval);
+	dia = DiaInitDialog(&us_ttyinputdialog);
+	if (dia == 0) return(0);
+	DiaSetText(dia, DGTL_PROMPT, localprompt);
+	DiaSetText(dia, -DGTL_LINE, defaultval);
 
 	/* loop until done */
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == CANCEL) break;
 		if (itemHit == OK) break;
 	}
 
 	if (itemHit != CANCEL)
-		line = us_putintoinfstr(DiaGetText(DGTL_LINE));
+		line = us_putintoinfstr(DiaGetText(dia, DGTL_LINE));
 
 	/* terminate the dialog */
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 	if (itemHit == CANCEL) return(0);
 	return(line);
 }
@@ -281,19 +284,22 @@ void ttyclearmessages(void)
  */
 void etrace(INTBIG mode, char *s, ...)
 {
-#ifdef ETRACE
 	va_list ap;
 	char line[500];
 
-	if (us_tracefile == NULL) return;
+	if (us_tracefile == NULL)
+	{
+		us_tracefile = fopen("trace.txt", "w");
+		if (us_tracefile == NULL) return;
+	}
 
 	/* build the error message */
 	var_start(ap, s);
 	evsnprintf(line, 500, s, ap);
 	va_end(ap);
 
-	fprintf (us_tracefile, line);
-#endif
+	fprintf(us_tracefile, line);
+	fflush(us_tracefile);
 }
 
 /***************************** INTERNAL SUPPORT *****************************/
