@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usrtrack.c
@@ -56,6 +57,7 @@ static INTBIG us_dragx, us_dragy, us_dragox, us_dragoy, us_dragoffx,
 	us_dragcorner, us_dragspecial;
 static BOOLEAN     us_dragjoinfactor;
 static BOOLEAN     us_multidragshowinvpin, us_multidragshowoffset;
+static INTBIG      us_multidragmostzeros;
 static UINTBIG     us_dragdescript[TEXTDESCRIPTSIZE];
 static INTBIG      us_arrowamount;				/* for slider tracking */
 static INTBIG      us_arrowlx, us_arrowhx, us_arrowly, us_arrowhy;
@@ -969,6 +971,7 @@ void us_finddoibegin(BOOLEAN complement)
 		if (i < len)
 		{
 			/* re-find the closest port when one node is selected */
+			(void)us_makehighlight(((char **)highvar->addr)[i], &newhigh);
 			if (len == 1 && (newhigh.status&HIGHFROM) != 0 &&
 				newhigh.fromgeom->entryisnode)
 			{
@@ -1098,7 +1101,7 @@ void us_finddoibegin(BOOLEAN complement)
 	if (us_dragstayonhigh == 0) return;
 
 	/* do no motion if stillness requested */
-	if (us_dragstill != 0 || (us_dragnodeproto->userbits&NLOCKED) != 0)
+	if (us_dragstill != 0 || (us_dragnodeproto->userbits&NPLOCKED) != 0)
 	{
 		us_dragstate = -1;
 		return;
@@ -1124,7 +1127,7 @@ void us_finddoibegin(BOOLEAN complement)
 	for(ni = us_dragnodeproto->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
 		if (ni->temp1 == 0) continue;
-		if (us_cantedit(us_dragnodeproto, ni->proto, FALSE))
+		if (us_cantedit(us_dragnodeproto, ni, FALSE))
 		{
 			ni->temp1 = 0;
 			continue;
@@ -1140,8 +1143,8 @@ void us_finddoibegin(BOOLEAN complement)
 		if (!us_dragobjectlist[i]->entryisnode)
 		{
 			ai = us_dragobjectlist[i]->entryaddr.ai;
-			if (us_cantedit(us_dragnodeproto, ai->end[0].nodeinst->proto, FALSE) ||
-				us_cantedit(us_dragnodeproto, ai->end[1].nodeinst->proto, FALSE)) continue;
+			if (us_cantedit(us_dragnodeproto, ai->end[0].nodeinst, FALSE) ||
+				us_cantedit(us_dragnodeproto, ai->end[1].nodeinst, FALSE)) continue;
 		}
 		us_dragobjectlist[j++] = us_dragobjectlist[i];
 	}
@@ -1182,8 +1185,7 @@ BOOLEAN us_findidown(INTBIG x, INTBIG y)
 		case 0:
 			/* pan the screen if the edge was hit */
 			us_panatscreenedge(&x, &y);
-
-			now = ticktime();
+			now = eventtime();
 			if (now < us_initialtime + us_doubleclicktime) ret = 0; else
 				ret = us_multidragdown(x, y);
 			break;
@@ -1648,7 +1650,7 @@ BOOLEAN us_createadown(INTBIG x, INTBIG y)
 			ai1 = us_makeconnection(us_dragfromgeom->entryaddr.ni, us_dragfromport,
 				gen_universalarc, foundgeom->entryaddr.ni, foundpp, gen_universalarc,
 				gen_univpinprim, unalx, unaly, &ai2, &ai3, &con1, &con2,
-				us_dragangle, FALSE, fakecoords);
+				us_dragangle, FALSE, fakecoords, TRUE);
 			if (ai1 != NOARCINST) count = 4;
 			if (ai2 != NOARCINST) count = 6;
 			if (ai3 != NOARCINST) count = 8;
@@ -1860,6 +1862,7 @@ void us_multidragbegin(void)
 		}
 	}
 
+	us_multidragmostzeros = -1;
 	us_multidragoffx = us_dragx-us_dragox;
 	us_multidragoffy = us_dragy-us_dragoy;
 	us_multidragdraw(HIGHLIT, us_multidragoffx, us_multidragoffy);
@@ -1927,9 +1930,9 @@ void us_multidragup(void)
 
 void us_multidragdraw(INTBIG col, INTBIG dx, INTBIG dy)
 {
-	REGISTER INTBIG i, thisend, portcount, endfixed, wid;
+	REGISTER INTBIG i, thisend, portcount, endfixed, wid, xzeros, yzeros, savestate;
 	INTBIG lx, ly, hx, hy, xc, yc, xw, yw, j;
-	char coords[100];
+	char coords[100], xcoord[50], ycoord[50], *pt;
 	XARRAY trans;
 	REGISTER NODEINST *ni;
 	REGISTER ARCINST *ai;
@@ -2124,7 +2127,26 @@ void us_multidragdraw(INTBIG col, INTBIG dx, INTBIG dy)
 	/* show the distance moved */
 	if (us_multidragshowoffset)
 	{
-		sprintf(coords, "(%s,%s)", latoa(dx), latoa(dy));
+		strcpy(xcoord, latoa(dx, 0));
+		for(pt = xcoord; *pt != 0; pt++) if (*pt == '.') break;
+		if (*pt == 0) xzeros = -1; else xzeros = strlen(pt) - 1;
+		strcpy(ycoord, latoa(dy, 0));
+		for(pt = ycoord; *pt != 0; pt++) if (*pt == '.') break;
+		if (*pt == 0) yzeros = -1; else yzeros = strlen(pt) - 1;
+		if (xzeros > us_multidragmostzeros) us_multidragmostzeros = xzeros;
+		if (yzeros > us_multidragmostzeros) us_multidragmostzeros = yzeros;
+		if (xzeros < us_multidragmostzeros)
+		{
+			if (xzeros < 0) { strcat(xcoord, "."); xzeros++; }
+			for(i=xzeros; i<us_multidragmostzeros; i++) strcat(xcoord, "0");
+		}
+		if (yzeros < us_multidragmostzeros)
+		{
+			if (yzeros < 0) { strcat(ycoord, "."); yzeros++; }
+			for(i=yzeros; i<us_multidragmostzeros; i++) strcat(ycoord, "0");
+		}
+
+		sprintf(coords, "(%s,%s)", xcoord, ycoord);
 		TDCLEAR(us_dragpoly->textdescript);
 		TDSETSIZE(us_dragpoly->textdescript, TXTSETPOINTS(20));
 		us_dragpoly->style = TEXTCENT;
@@ -2132,7 +2154,10 @@ void us_multidragdraw(INTBIG col, INTBIG dx, INTBIG dy)
 		us_dragpoly->xv[0] = (us_dragwindow->screenlx + us_dragwindow->screenhx) / 2;
 		us_dragpoly->yv[0] = (us_dragwindow->screenly + us_dragwindow->screenhy) / 2;
 		us_dragpoly->string = coords;
+		savestate = us_dragwindow->state;
+		us_dragwindow->state &= ~INPLACEEDIT;
 		us_showpoly(us_dragpoly, us_dragwindow);
+		us_dragwindow->state = savestate;
 	}
 }
 
@@ -2197,11 +2222,11 @@ BOOLEAN us_distancedown(INTBIG x, INTBIG y)
 	us_dragpoly->tech = el_curtech;
 	dist = computedistance(us_firstmeasurex, us_firstmeasurey, us_dragx, us_dragy);
 	(void)strcpy(us_dragmessage, _("Distance: "));
-	(void)strcat(us_dragmessage, latoa(dist));
+	(void)strcat(us_dragmessage, latoa(dist, 0));
 	(void)strcat(us_dragmessage, " (dX=");
-	(void)strcat(us_dragmessage, latoa(us_dragx - us_firstmeasurex));
+	(void)strcat(us_dragmessage, latoa(us_dragx - us_firstmeasurex, 0));
 	(void)strcat(us_dragmessage, " dY=");
-	(void)strcat(us_dragmessage, latoa(us_dragy - us_firstmeasurey));
+	(void)strcat(us_dragmessage, latoa(us_dragy - us_firstmeasurey, 0));
 	(void)strcat(us_dragmessage, ")");
 	us_dragpoly->string = us_dragmessage;
 	us_drawdistance();
@@ -2649,6 +2674,13 @@ void us_vpartdividerdone(void)
 				}
 			}
 		}
+
+		/* remember the explorer percentage */
+		for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
+		{
+			if ((w->state&WINDOWTYPE) != EXPLORERWINDOW) continue;
+			us_explorerratio = w->hratio;
+		}
 		us_beginchanges();
 		us_drawmenu(1, us_trackww.frame);
 		us_endchanges(NOWINDOWPART);
