@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usrtranslate.c
@@ -72,7 +73,7 @@ static DIALOGITEM us_transdialogitems[] =
  /* 32 */ {0, {312,156,328,300}, CHECK, N_("Find in translation")},
  /* 33 */ {0, {337,328,361,408}, BUTTON, N_("Validate")}
 };
-static DIALOG us_transdialog = {{75,75,444,613}, N_("Translate Electric Strings"), 0, 33, us_transdialogitems};
+static DIALOG us_transdialog = {{75,75,444,613}, N_("Translate Electric Strings"), 0, 33, us_transdialogitems, 0, 0};
 
 #define DTRN_NEXT       1		/* next string (button) */
 #define DTRN_FOREIGN    3		/* translated string (edit text) */
@@ -141,10 +142,11 @@ static char     *us_translation;
 static INTBIG    us_translationsize = 0;
 
 /* prototypes for local routines */
-static BOOLEAN      us_transloadmessages(char *file, INTBIG *entrycount, INTBIG *untranslatedcount, INTBIG *fuzzycount);
+static BOOLEAN      us_transloadmessages(char *file, INTBIG *entrycount, INTBIG *untranslatedcount,
+						INTBIG *fuzzycount, void *dia);
 static BOOLEAN      us_transgathermessage(char *line);
 static STRINGENTRY *us_transfirst(INTBIG choose);
-static void         us_transhowentry(STRINGENTRY *se);
+static void         us_transhowentry(STRINGENTRY *se, void *dia);
 static void         us_transave(char *language, INTBIG platform);
 static char        *us_transgetfiles(char *language, INTBIG platform);
 static char        *us_transquoteit(char *message);
@@ -173,6 +175,7 @@ void us_translationdlog(void)
 	BOOLEAN dirty, findenglish, findtranslation;
 	char *language, num[20], *pt, *sept, *par[6], **filelist, intlpath[300], **langlist;
 	REGISTER STRINGENTRY *se, *curse;
+	REGISTER void *dia;
 
 	if (us_filetypetrans == 0)
 		us_filetypetrans = setupfiletype("", "*.*", MACFSTAG('TEXT'), FALSE, "transfile", _("Translation File"));
@@ -198,13 +201,14 @@ void us_translationdlog(void)
 	}
 
 	/* show the dialog */
-	if (DiaInitDialog(&us_transdialog)) return;
-	DiaInitTextDialog(DTRN_COMMENTS, DiaNullDlogList, DiaNullDlogItem,
+	dia = DiaInitDialog(&us_transdialog);
+	if (dia == 0) return;
+	DiaInitTextDialog(dia, DTRN_COMMENTS, DiaNullDlogList, DiaNullDlogItem,
 		DiaNullDlogDone, -1, 0);
-	DiaSetControl(DTRN_FINDENG, 1);
+	DiaSetControl(dia, DTRN_FINDENG, 1);
 
 	/* setup the different languages */
-	DiaSetPopup(DTRN_LANGUAGE, langcount, langlist);
+	DiaSetPopup(dia, DTRN_LANGUAGE, langcount, langlist);
 	language = langlist[0];
 
 	/* setup the different types of messages to preview */
@@ -214,36 +218,36 @@ void us_translationdlog(void)
 	par[CHOOSENLFUZZY]   = _("No longer fuzzy");
 	par[CHOOSEFIND]      = _("Matching");
 	par[CHOOSEALL]       = _("All");
-	DiaSetPopup(DTRN_NEXTTYPE, 6, par);
+	DiaSetPopup(dia, DTRN_NEXTTYPE, 6, par);
 	choose = CHOOSEUNTRANS;
-	DiaSetPopupEntry(DTRN_NEXTTYPE, choose);
+	DiaSetPopupEntry(dia, DTRN_NEXTTYPE, choose);
 
 	/* setup the different translation files */
 	par[0] = _("Macintosh");
 	par[1] = _("Windows,UNIX");
-	DiaSetPopup(DTRN_PLATFORM, 2, par);	
+	DiaSetPopup(dia, DTRN_PLATFORM, 2, par);	
 #ifdef MACOS
 	platform = 0;
 #else
 	platform = 1;
 #endif
-	DiaSetPopupEntry(DTRN_PLATFORM, platform);
+	DiaSetPopupEntry(dia, DTRN_PLATFORM, platform);
 
 	/* load the selected set of messages */
 	pt = us_transgetfiles(language, platform);
-	if (us_transloadmessages(pt, &total, &untranslatedcount, &fuzzycount))
+	if (us_transloadmessages(pt, &total, &untranslatedcount, &fuzzycount, dia))
 	{
-		DiaDoneDialog();
+		DiaDoneDialog(dia);
 		return;
 	}
 	curse = us_transfirst(choose);
-	us_transhowentry(curse);
+	us_transhowentry(curse, dia);
 
 	/* run the dialog */
 	dirty = FALSE;
 	for(;;)
 	{
-		itemno = DiaNextHit();
+		itemno = DiaNextHit(dia);
 		if (itemno == DTRN_DONE)
 		{
 			if (dirty)
@@ -261,26 +265,26 @@ void us_translationdlog(void)
 		}
 		if (itemno == DTRN_FINDENG || itemno == DTRN_FINDTRN)
 		{
-			DiaSetControl(itemno, 1 - DiaGetControl(itemno));
+			DiaSetControl(dia, itemno, 1 - DiaGetControl(dia, itemno));
 			continue;
 		}
 		if (itemno == DTRN_FINDBUT)
 		{
-			DiaSetPopupEntry(DTRN_NEXTTYPE, CHOOSEFIND);
+			DiaSetPopupEntry(dia, DTRN_NEXTTYPE, CHOOSEFIND);
 			itemno = DTRN_NEXTTYPE;
 			/* fall into next test */
 		}
 		if (itemno == DTRN_NEXTTYPE)
 		{
-			choose = DiaGetPopupEntry(DTRN_NEXTTYPE);
+			choose = DiaGetPopupEntry(dia, DTRN_NEXTTYPE);
 			if (choose == CHOOSEFIND)
 			{
 				/* do the search and mark strings that match */
-				if (DiaGetControl(DTRN_FINDENG) != 0) findenglish = TRUE; else
+				if (DiaGetControl(dia, DTRN_FINDENG) != 0) findenglish = TRUE; else
 					findenglish = FALSE;
-				if (DiaGetControl(DTRN_FINDTRN) != 0) findtranslation = TRUE; else
+				if (DiaGetControl(dia, DTRN_FINDTRN) != 0) findtranslation = TRUE; else
 					findtranslation = FALSE;
-				pt = DiaGetText(DTRN_FINDSTR);
+				pt = DiaGetText(dia, DTRN_FINDSTR);
 				ptlen = strlen(pt);
 				foundcount = 0;
 				for(se = us_transfirststringentry; se != NOSTRINGENTRY; se = se->nextstringentry)
@@ -315,15 +319,15 @@ void us_translationdlog(void)
 					}
 				}
 				sprintf(num, "%ld", foundcount);
-				DiaSetText(DTRN_FOUNDSTR, num);
-			} else DiaSetText(DTRN_FOUNDSTR, "");
+				DiaSetText(dia, DTRN_FOUNDSTR, num);
+			} else DiaSetText(dia, DTRN_FOUNDSTR, "");
 			curse = us_transfirst(choose);
-			us_transhowentry(curse);
+			us_transhowentry(curse, dia);
 			continue;
 		}
 		if (itemno == DTRN_PLATFORM)
 		{
-			i = DiaGetPopupEntry(DTRN_PLATFORM);
+			i = DiaGetPopupEntry(dia, DTRN_PLATFORM);
 			if (i == platform) continue;
 			if (dirty)
 			{
@@ -333,18 +337,18 @@ void us_translationdlog(void)
 			}
 
 			pt = us_transgetfiles(language, i);
-			if (us_transloadmessages(pt, &total, &untranslatedcount, &fuzzycount))
+			if (us_transloadmessages(pt, &total, &untranslatedcount, &fuzzycount, dia))
 				continue;
 
 			platform = i;
 			curse = us_transfirst(choose);
-			us_transhowentry(curse);
+			us_transhowentry(curse, dia);
 			dirty = FALSE;
 			continue;
 		}
 		if (itemno == DTRN_LANGUAGE)
 		{
-			i = DiaGetPopupEntry(DTRN_LANGUAGE);
+			i = DiaGetPopupEntry(dia, DTRN_LANGUAGE);
 			if (strcmp(langlist[i], language) == 0) continue;
 			if (dirty)
 			{
@@ -354,12 +358,12 @@ void us_translationdlog(void)
 			}
 
 			pt = us_transgetfiles(langlist[i], platform);
-			if (us_transloadmessages(pt, &total, &untranslatedcount, &fuzzycount))
+			if (us_transloadmessages(pt, &total, &untranslatedcount, &fuzzycount, dia))
 				continue;
 
 			language = langlist[i];
 			curse = us_transfirst(choose);
-			us_transhowentry(curse);
+			us_transhowentry(curse, dia);
 			dirty = FALSE;
 			continue;
 		}
@@ -399,7 +403,7 @@ void us_translationdlog(void)
 				}
 				curse = se;
 			}
-			us_transhowentry(curse);
+			us_transhowentry(curse, dia);
 			continue;
 		}
 		if (itemno == DTRN_PREV)
@@ -412,20 +416,20 @@ void us_translationdlog(void)
 				}
 				curse = se;
 			}
-			us_transhowentry(curse);
+			us_transhowentry(curse, dia);
 			continue;
 		}
 		if (itemno == DTRN_FIRST)
 		{
 			curse = us_transfirst(choose);
-			us_transhowentry(curse);
+			us_transhowentry(curse, dia);
 			continue;
 		}
 		if (itemno == DTRN_ISFUZZY)
 		{
 			if ((curse->flags&FUZZY) == 0) continue;
-			i = 1 - DiaGetControl(DTRN_ISFUZZY);
-			DiaSetControl(DTRN_ISFUZZY, i);
+			i = 1 - DiaGetControl(dia, DTRN_ISFUZZY);
+			DiaSetControl(dia, DTRN_ISFUZZY, i);
 			if (i == 0)
 			{
 				curse->flags |= WASFUZZY;
@@ -436,14 +440,14 @@ void us_translationdlog(void)
 				fuzzycount++;
 			}
 			sprintf(num, "%ld", fuzzycount);
-			DiaSetText(DTRN_FUZSTR, num);
+			DiaSetText(dia, DTRN_FUZSTR, num);
 			dirty = TRUE;
 			continue;
 		}
 		if (itemno == DTRN_FOREIGN)
 		{
 			if (curse == NOSTRINGENTRY) continue;
-			pt = DiaGetText(DTRN_FOREIGN);
+			pt = DiaGetText(dia, DTRN_FOREIGN);
 			if (curse->translated == 0)
 			{
 				if (*pt == 0) continue;
@@ -456,14 +460,14 @@ void us_translationdlog(void)
 			{
 				untranslatedcount--;
 				sprintf(num, "%ld", untranslatedcount);
-				DiaSetText(DTRN_UNTSTR, num);
+				DiaSetText(dia, DTRN_UNTSTR, num);
 			}
 			if ((curse->flags & FUZZY) != 0)
 			{
 				fuzzycount--;
 				sprintf(num, "%ld", fuzzycount);
-				DiaSetText(DTRN_FUZSTR, num);
-				DiaSetControl(DTRN_ISFUZZY, 0);
+				DiaSetText(dia, DTRN_FUZSTR, num);
+				DiaSetControl(dia, DTRN_ISFUZZY, 0);
 				if ((curse->flags&FUZZY) != 0) curse->flags |= WASFUZZY;
 			}
 			if (curse->translated != 0) efree((char *)curse->translated);
@@ -472,27 +476,27 @@ void us_translationdlog(void)
 			continue;
 		}
 	}
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 }
 
-void us_transhowentry(STRINGENTRY *se)
+void us_transhowentry(STRINGENTRY *se, void *dia)
 {
 	char line[100];
 	REGISTER INTBIG whichone, i;
 	BOOLEAN matching;
 	REGISTER STRINGENTRY *ose;
 
-	DiaLoadTextDialog(DTRN_COMMENTS, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1);
-	DiaSetControl(DTRN_ISFUZZY, 0);
+	DiaLoadTextDialog(dia, DTRN_COMMENTS, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1);
+	DiaSetControl(dia, DTRN_ISFUZZY, 0);
 	if (se == NOSTRINGENTRY)
 	{
-		DiaSetText(DTRN_ENGLISH, "");
-		DiaSetText(DTRN_FOREIGN, "");
-		DiaSetText(DTRN_MSGID, _("No strings left"));
+		DiaSetText(dia, DTRN_ENGLISH, "");
+		DiaSetText(dia, DTRN_FOREIGN, "");
+		DiaSetText(dia, DTRN_MSGID, _("No strings left"));
 		return;
 	}
 	whichone = 0;
-	if (DiaGetPopupEntry(DTRN_NEXTTYPE) == CHOOSEFIND) matching = TRUE; else
+	if (DiaGetPopupEntry(dia, DTRN_NEXTTYPE) == CHOOSEFIND) matching = TRUE; else
 		matching = FALSE;
 	for(ose = us_transfirststringentry; ose != NOSTRINGENTRY; ose = ose->nextstringentry)
 	{
@@ -533,15 +537,15 @@ void us_transhowentry(STRINGENTRY *se)
 			sprintf(line, _("String %ld (already translated)"), se->index);
 		}
 	}
-	DiaSetText(DTRN_MSGID, line);
-	if (se->english == 0) DiaSetText(DTRN_ENGLISH, ""); else
-		DiaSetText(DTRN_ENGLISH, se->english);
-	if (se->translated == 0) DiaSetText(-DTRN_FOREIGN, ""); else
-		DiaSetText(-DTRN_FOREIGN, se->translated);
-	if ((se->flags&(FUZZY|WASFUZZY)) == FUZZY) DiaSetControl(DTRN_ISFUZZY, 1);
+	DiaSetText(dia, DTRN_MSGID, line);
+	if (se->english == 0) DiaSetText(dia, DTRN_ENGLISH, ""); else
+		DiaSetText(dia, DTRN_ENGLISH, se->english);
+	if (se->translated == 0) DiaSetText(dia, -DTRN_FOREIGN, ""); else
+		DiaSetText(dia, -DTRN_FOREIGN, se->translated);
+	if ((se->flags&(FUZZY|WASFUZZY)) == FUZZY) DiaSetControl(dia, DTRN_ISFUZZY, 1);
 	for(i=0; i<se->headerlinecount; i++)
-		DiaStuffLine(DTRN_COMMENTS, se->headerlines[i]);
-	DiaSelectLine(DTRN_COMMENTS, -1);
+		DiaStuffLine(dia, DTRN_COMMENTS, se->headerlines[i]);
+	DiaSelectLine(dia, DTRN_COMMENTS, -1);
 }
 
 STRINGENTRY *us_transfirst(INTBIG choose)
@@ -595,7 +599,8 @@ BOOLEAN us_transmeetscriteria(STRINGENTRY *se, INTBIG choose)
 	return(FALSE);
 }
 
-BOOLEAN us_transloadmessages(char *file, INTBIG *entrycount, INTBIG *untranslatedcount, INTBIG *fuzzycount)
+BOOLEAN us_transloadmessages(char *file, INTBIG *entrycount, INTBIG *untranslatedcount,
+	INTBIG *fuzzycount, void *dia)
 {
 	REGISTER FILE *io;
 	REGISTER INTBIG i, j, eof, inenglish, endchr, haveline, index, lineno;
@@ -781,11 +786,11 @@ BOOLEAN us_transloadmessages(char *file, INTBIG *entrycount, INTBIG *untranslate
 		}
 	}
 	sprintf(line, "%ld", *entrycount);
-	DiaSetText(DTRN_TOTSTR, line);
+	DiaSetText(dia, DTRN_TOTSTR, line);
 	sprintf(line, "%ld", *untranslatedcount);
-	DiaSetText(DTRN_UNTSTR, line);
+	DiaSetText(dia, DTRN_UNTSTR, line);
 	sprintf(line, "%ld", *fuzzycount);
-	DiaSetText(DTRN_FUZSTR, line);
+	DiaSetText(dia, DTRN_FUZSTR, line);
 	return(FALSE);
 }
 
@@ -823,7 +828,7 @@ void us_transave(char *language, INTBIG platform)
 	REGISTER STRINGENTRY *se;
 	REGISTER INTBIG i;
 	INTBIG year, month, mday, hour, minute, second;
-	REGISTER UINTBIG olddate;
+	REGISTER time_t olddate;
 	char filename[300], rename[300], datesuffix[100], *truename;
 	FILE *io;
 	REGISTER void *infstr;
