@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: usrwindow.c
@@ -570,6 +571,7 @@ void us_endchanges(WINDOWPART *which)
 			}
 		}
 	}
+
 	us_firstopaque = NOREDRAW;
 	us_lastopaque = NOREDRAW;
 
@@ -886,22 +888,19 @@ void us_queuevicinity(GEOM *object, NODEPROTO *parnt, XARRAY trans, INTBIG lx,
 	REGISTER GEOM *look;
 	REGISTER NODEINST *ni;
 	REGISTER ARCINST *ai;
-	INTBIG bx, ux, by, uy;
-	XARRAY localtran, xf, subrot;
+	INTBIG bx, ux, by, uy, olx, ohx, oly, ohy;
 	REGISTER BOOLEAN local;
-	REGISTER INTBIG i, search, former;
+	REGISTER INTBIG i, search, slop;
 
 	/* compute full extent of the object, including text */
 	if (object != NOGEOM)
 	{
 		if (object->entryisnode)
 		{
-			ni = object->entryaddr.ni;
-			us_getnodebounds(ni, &lx, &hx, &ly, &hy);
+			us_getnodebounds(object->entryaddr.ni, &lx, &hx, &ly, &hy);
 		} else
 		{
-			ai = object->entryaddr.ai;
-			us_getarcbounds(ai, &lx, &hx, &ly, &hy);
+			us_getarcbounds(object->entryaddr.ai, &lx, &hx, &ly, &hy);
 		}
 	}
 
@@ -909,7 +908,8 @@ void us_queuevicinity(GEOM *object, NODEPROTO *parnt, XARRAY trans, INTBIG lx,
 	xform(hx, hy, &ux, &uy, trans);
 	if (bx > ux) { i = bx;  bx = ux;  ux = i; }
 	if (by > uy) { i = by;  by = uy;  uy = i; }
-	search = initsearch(bx, ux, by, uy, parnt);
+	slop = FARTEXTLIMIT * lambdaoffacet(parnt);
+	search = initsearch(bx-slop, ux+slop, by-slop, uy+slop, parnt);
 	if (object == NOGEOM) local = TRUE; else local = FALSE;
 	for(;;)
 	{
@@ -918,13 +918,21 @@ void us_queuevicinity(GEOM *object, NODEPROTO *parnt, XARRAY trans, INTBIG lx,
 		/* don't re-draw the current object whose vicinity is being checked */
 		if (look == object) continue;
 
+#if 0
 		/* get the former inside factor (if it is a nodeinst) */
 		if (look->entryisnode) former = look->entryaddr.ni->userbits;
+#endif
+
+		/* see if the object really intersects the redraw area */
+		if (look->entryisnode)
+			us_getnodebounds(look->entryaddr.ni, &olx, &ohx, &oly, &ohy); else
+				us_getarcbounds(look->entryaddr.ai, &olx, &ohx, &oly, &ohy);
+		if (olx >= ux || ohx <= bx || oly >= uy || ohy <= by) continue;
 
 		/* redraw all if object is going off, redraw opaque if going on */
 		if (on == LAYERN) us_queueredraw(look, local); else
 			us_queueopaque(look, local);
-
+#if 0
 		/* that's all if the object is an arcinst */
 		if (!look->entryisnode) continue;
 		ni = look->entryaddr.ni;
@@ -951,6 +959,25 @@ void us_queuevicinity(GEOM *object, NODEPROTO *parnt, XARRAY trans, INTBIG lx,
 
 		/* now flag this nodeinst so that it doesn't get completely redrawn */
 		ni->userbits = (ni->userbits & ~REWANTN) | RELOCLN;
+#endif
+	}
+
+	/* now queue things with "far text" */
+	for(ni = parnt->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+	{
+		if ((ni->userbits&NHASFARTEXT) == 0) continue;
+		us_getnodebounds(ni, &olx, &ohx, &oly, &ohy);
+		if (olx >= ux || ohx <= bx || oly >= uy || ohy <= by) continue;
+		if (on == LAYERN) us_queueredraw(ni->geom, local); else
+			us_queueopaque(ni->geom, local);
+	}
+	for(ai = parnt->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
+	{
+		if ((ai->userbits&AHASFARTEXT) == 0) continue;
+		us_getarcbounds(ai, &olx, &ohx, &oly, &ohy);
+		if (olx >= ux || ohx <= bx || oly >= uy || ohy <= by) continue;
+		if (on == LAYERN) us_queueredraw(ai->geom, local); else
+			us_queueopaque(ai->geom, local);
 	}
 }
 
