@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: vhdl.c
@@ -218,7 +219,7 @@ static INTBIG    vhdl_endinput(void);
 static INTBIG    vhdl_endoutput(void);
 static void      vhdl_freescannermemory(void);
 static BOOLEAN   vhdl_generatevhdl(NODEPROTO *np);
-static UINTBIG   vhdl_getfacetdate(NODEPROTO *np);
+static time_t    vhdl_getfacetdate(NODEPROTO *np);
 static INTBIG    vhdl_getnextline(char*);
 static INTBIG    vhdl_identfirsthash(char*);
 static INTBIG    vhdl_identsecondhash(char*, INTBIG);
@@ -759,7 +760,7 @@ INTBIG vhdl_startinput(NODEPROTO *np, INTBIG filetype, BOOLEAN diskflag, char **
 		addstringtoinfstr(infstr, np->cell->cellname);
 		*intended = returninfstr(infstr);
 		if ((vhdl_fp = xopen(np->cell->cellname, filetype, "", &filename)) == 0) return(1);
-		*revisiondate = filedate(filename);
+		*revisiondate = (UINTBIG)filedate(filename);
 	} else
 	{
 		/* find proper view of this cell */
@@ -963,12 +964,12 @@ void vhdl_print(char *fstring, ...)
 /*
  * Routine to determine the revision date of facet "np", including any subfacets.
  */
-UINTBIG vhdl_getfacetdate(NODEPROTO *np)
+time_t vhdl_getfacetdate(NODEPROTO *np)
 {
-	REGISTER UINTBIG mostrecent, subfacetdate;
+	REGISTER time_t mostrecent, subfacetdate;
 	REGISTER NODEINST *ni;
 
-	mostrecent = np->revisiondate;
+	mostrecent = (time_t)np->revisiondate;
 	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
 		if (ni->proto->primindex != 0) continue;
@@ -995,7 +996,7 @@ void vhdl_convertfacet(NODEPROTO *np, BOOLEAN force)
 	REGISTER INTBIG i, len;
 	REGISTER BOOLEAN backannotate;
 	BOOLEAN vhdlondisk;
-	REGISTER UINTBIG dateoflayout;
+	REGISTER time_t dateoflayout;
 	REGISTER VARIABLE *var;
 	FILE *f;
 	char *intended, *filename, *desired, *firstline;
@@ -1042,11 +1043,11 @@ void vhdl_convertfacet(NODEPROTO *np, BOOLEAN force)
 					if (strncmp(firstline, desired, len) == 0)
 					{
 						if (strcmp(&desired[len], describenodeproto(np)) != 0)
-							dateoflayout = npvhdl->revisiondate + 1;
+							dateoflayout = (time_t)(npvhdl->revisiondate + 1);
 					}
 				}
 
-				if (npvhdl->revisiondate >= dateoflayout) return;
+				if ((time_t)npvhdl->revisiondate >= dateoflayout) return;
 			}
 		}
 	}
@@ -1282,9 +1283,9 @@ BOOLEAN vhdl_generatevhdl(NODEPROTO *np)
 	for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 	{
 		pp->network->temp1 = 1;
-		if (pp->network->signals > 1)
+		if (pp->network->buswidth > 1)
 		{
-			for(i=0; i<pp->network->signals; i++)
+			for(i=0; i<pp->network->buswidth; i++)
 				pp->network->networklist[i]->temp1 = 1;
 		}
 	}
@@ -1295,9 +1296,9 @@ BOOLEAN vhdl_generatevhdl(NODEPROTO *np)
 
 		/* disallow if part of a bus export or already written */
 		if (net->temp1 != 0) continue;
-		for(i=0; i<net->signals; i++)
+		for(i=0; i<net->buswidth; i++)
 		{
-			if (net->signals <= 1)
+			if (net->buswidth <= 1)
 			{
 				if (net->namecount > 0) pt = net->netname; else
 					pt = describenetwork(net);
@@ -1455,7 +1456,7 @@ BOOLEAN vhdl_generatevhdl(NODEPROTO *np)
 			if (onp != NONODEPROTO) continue;
 		}
 
-		downhierarchy(ni, 0);
+		downhierarchy(ni, cnp, 0);
 		if (vhdl_generatevhdl(cnp)) backannotate = TRUE;
 		uphierarchy();
 	}
@@ -1603,9 +1604,9 @@ void vhdl_addrealports(void *infstr, NODEINST *ni, INTBIG special)
 					}
 
 					net = ai->network;
-					if (net->signals > 1)
+					if (net->buswidth > 1)
 					{
-						for(i=0; i<net->signals; i++)
+						for(i=0; i<net->buswidth; i++)
 						{
 							if (i != 0) addstringtoinfstr(infstr, ", ");
 							subnet = net->networklist[i];
@@ -3062,7 +3063,7 @@ static DIALOGITEM vhdl_optionsdialogitems[] =
  /* 10 */ {0, {8,8,24,183}, MESSAGE, N_("Schematics primitives:")},
  /* 11 */ {0, {292,8,293,319}, DIVIDELINE, ""}
 };
-static DIALOG vhdl_optionsdialog = {{50,75,419,403}, N_("VHDL Options"), 0, 11, vhdl_optionsdialogitems};
+static DIALOG vhdl_optionsdialog = {{50,75,419,403}, N_("VHDL Options"), 0, 11, vhdl_optionsdialogitems, 0, 0};
 
 /* special items for the "VHDL Options" dialog: */
 #define DVHO_VHDLINFACET   3		/* VHDL stored in facet (check) */
@@ -3080,9 +3081,10 @@ void vhdl_optionsdlog(void)
 	REGISTER NODEPROTO *np, **nplist;
 	REGISTER INTBIG i, len, technameschanged;
 	BOOLEAN vhdlondisk, netlistondisk, ondisk;
-	REGISTER void *infstr;
+	REGISTER void *infstr, *dia;
 
-	if (DiaInitDialog(&vhdl_optionsdialog)) return;
+	dia = DiaInitDialog(&vhdl_optionsdialog);
+	if (dia == 0) return;
 	var = getval((INTBIG)vhdl_tool, VTOOL, VINTEGER, "VHDL_vhdl_on_disk");
 	vhdlondisk = FALSE;
 	if (var != NOVARIABLE && var->addr != 0) vhdlondisk = TRUE;
@@ -3092,7 +3094,7 @@ void vhdl_optionsdlog(void)
 
 	/* setup for generated names in the technology */
 	tech = sch_tech;
-	DiaInitTextDialog(DVHO_PRIMLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0,
+	DiaInitTextDialog(dia, DVHO_PRIMLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0,
 		SCSELMOUSE|SCREPORT);
 	var = getval((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, "TECH_vhdl_names");
 	if (var == NOVARIABLE)
@@ -3128,51 +3130,51 @@ void vhdl_optionsdlog(void)
 			addstringtoinfstr(infstr, varnames[i]);
 			addstringtoinfstr(infstr, ")");
 		}
-		DiaStuffLine(DVHO_PRIMLIST, returninfstr(infstr));
+		DiaStuffLine(dia, DVHO_PRIMLIST, returninfstr(infstr));
 	}
-	DiaSelectLine(DVHO_PRIMLIST, 0);
+	DiaSelectLine(dia, DVHO_PRIMLIST, 0);
 
-	if (!vhdlondisk) DiaSetControl(DVHO_VHDLINFACET, 1);
-	if (!netlistondisk) DiaSetControl(DVHO_NETINFACET, 1);
+	if (!vhdlondisk) DiaSetControl(dia, DVHO_VHDLINFACET, 1);
+	if (!netlistondisk) DiaSetControl(dia, DVHO_NETINFACET, 1);
 
 	/* loop until done */
 	technameschanged = 0;
 	for(;;)
 	{
-		itemHit = DiaNextHit();
+		itemHit = DiaNextHit(dia);
 		if (itemHit == OK || itemHit == CANCEL) break;
 		if (itemHit == DVHO_VHDLINFACET || itemHit == DVHO_NETINFACET)
 		{
-			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
+			DiaSetControl(dia, itemHit, 1 - DiaGetControl(dia, itemHit));
 			continue;
 		}
 		if (itemHit == DVHO_PRIMLIST)
 		{
-			DiaSetText(DVHO_UNNEGATEDNAME, "");
-			DiaSetText(DVHO_NEGATEDNAME, "");
-			i = DiaGetCurLine(DVHO_PRIMLIST);
+			DiaSetText(dia, DVHO_UNNEGATEDNAME, "");
+			DiaSetText(dia, DVHO_NEGATEDNAME, "");
+			i = DiaGetCurLine(dia, DVHO_PRIMLIST);
 			pt = varnames[i];
 			for(opt = pt; *opt != 0; opt++) if (*opt == '/') break;
 			if (*opt == '/')
 			{
 				*opt = 0;
-				DiaSetText(DVHO_UNNEGATEDNAME, pt);
-				DiaSetText(DVHO_NEGATEDNAME, &opt[1]);
+				DiaSetText(dia, DVHO_UNNEGATEDNAME, pt);
+				DiaSetText(dia, DVHO_NEGATEDNAME, &opt[1]);
 				*opt = '/';
 			} else
 			{
-				DiaSetText(DVHO_UNNEGATEDNAME, pt);
-				DiaSetText(DVHO_NEGATEDNAME, pt);
+				DiaSetText(dia, DVHO_UNNEGATEDNAME, pt);
+				DiaSetText(dia, DVHO_NEGATEDNAME, pt);
 			}
 			continue;
 		}
 		if (itemHit == DVHO_UNNEGATEDNAME || itemHit == DVHO_NEGATEDNAME)
 		{
-			i = DiaGetCurLine(DVHO_PRIMLIST);
+			i = DiaGetCurLine(dia, DVHO_PRIMLIST);
 			infstr = initinfstr();
-			addstringtoinfstr(infstr, DiaGetText(DVHO_UNNEGATEDNAME));
+			addstringtoinfstr(infstr, DiaGetText(dia, DVHO_UNNEGATEDNAME));
 			addtoinfstr(infstr, '/');
-			addstringtoinfstr(infstr, DiaGetText(DVHO_NEGATEDNAME));
+			addstringtoinfstr(infstr, DiaGetText(dia, DVHO_NEGATEDNAME));
 			pt = returninfstr(infstr);
 			if (strcmp(varnames[i], pt) == 0) continue;
 
@@ -3187,24 +3189,24 @@ void vhdl_optionsdlog(void)
 				addstringtoinfstr(infstr, varnames[i]);
 				addstringtoinfstr(infstr, ")");
 			}
-			DiaSetScrollLine(DVHO_PRIMLIST, i, returninfstr(infstr));
+			DiaSetScrollLine(dia, DVHO_PRIMLIST, i, returninfstr(infstr));
 			continue;
 		}
 	}
 
 	if (itemHit != CANCEL)
 	{
-		if (DiaGetControl(DVHO_VHDLINFACET) != 0) ondisk = FALSE; else ondisk = TRUE;
+		if (DiaGetControl(dia, DVHO_VHDLINFACET) != 0) ondisk = FALSE; else ondisk = TRUE;
 		if (ondisk != vhdlondisk)
 			setval((INTBIG)vhdl_tool, VTOOL, "VHDL_vhdl_on_disk", ondisk, VINTEGER);
-		if (DiaGetControl(DVHO_NETINFACET) != 0) ondisk = FALSE; else ondisk = TRUE;
+		if (DiaGetControl(dia, DVHO_NETINFACET) != 0) ondisk = FALSE; else ondisk = TRUE;
 		if (ondisk != netlistondisk)
 			setval((INTBIG)vhdl_tool, VTOOL, "VHDL_netlist_on_disk", ondisk, VINTEGER);
 		if (technameschanged != 0)
 			setval((INTBIG)tech, VTECHNOLOGY, "TECH_vhdl_names", (INTBIG)varnames,
 				VSTRING|VISARRAY|(len<<VLENGTHSH));
 	}
-	DiaDoneDialog();
+	DiaDoneDialog(dia);
 	for(i=0; i<len; i++) efree(varnames[i]);
 	efree((char *)varnames);
 	efree((char *)nplist);
