@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: vhdlparser.c
@@ -239,6 +240,10 @@ void vhdl_reporterrormsg(TOKENLIST *tlist, char *err_msg)
 			case TOKEN_CHAR:
 				sprintf(stemp, "%c", (char)((INTBIG)tstart->pointer));
 				strcat(buffer, stemp);
+			case TOKEN_DECIMAL:
+				sprintf(stemp, "%s", tstart->pointer);
+				strcat(buffer, stemp);
+				break;
 			default:
 				if (tstart->pointer != 0)
 				{
@@ -1310,7 +1315,7 @@ Description:
 
 		discrete_range ::= subtype_indication | range
 ------------------------------------------------------------------------
-CAlling Sequence:  range = vhdl_parsediscrete_range();
+Calling Sequence:  range = vhdl_parsediscrete_range();
 
 Name		Type			Description
 ----		----			-----------
