@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: vhdlquisc.c
@@ -304,7 +305,7 @@ void vhdl_genquisc_interface(DBINTERFACE *interfacef)
 	/* add local signals */
 	if (interfacef->bodies && interfacef->bodies->declare)
 	{
-		for (signal = interfacef->bodies->declare->signals; signal; signal = signal->next)
+		for (signal = interfacef->bodies->declare->bodysignals; signal; signal = signal->next)
 		{
 			if (signal->type == NULL || signal->type->type == DBTYPE_SINGLE)
 			{
