@@ -1,4 +1,5 @@
-/*
+/* -*- tab-width: 4 -*-
+ *
  * Electric(tm) VLSI Design System
  *
  * File: vhdlsemantic.c
@@ -140,10 +141,10 @@ void vhdl_freesemantic(void)
 				}
 				efree((char *)comp);
 			}
-			while (bodydecl->signals != 0)
+			while (bodydecl->bodysignals != 0)
 			{
-				signals = bodydecl->signals;
-				bodydecl->signals = bodydecl->signals->next;
+				signals = bodydecl->bodysignals;
+				bodydecl->bodysignals = bodydecl->bodysignals->next;
 				efree((char *)signals);
 			}
 			efree((char *)bodydecl);
@@ -660,7 +661,7 @@ DBBODYDECLARE *vhdl_sembody_declare(BODYDECLARE *declare)
 	if (declare == 0) return(dbdeclare);
 	dbdeclare = (DBBODYDECLARE *)emalloc((INTBIG)sizeof(DBBODYDECLARE), vhdl_tool->cluster);
 	dbdeclare->components = endcomponent = 0;
-	dbdeclare->signals = endsignal = 0;
+	dbdeclare->bodysignals = endsignal = 0;
 
 	for (; declare != 0; declare = declare->next)
 	{
@@ -672,7 +673,7 @@ DBBODYDECLARE *vhdl_sembody_declare(BODYDECLARE *declare)
 				{
 					if (endsignal == 0)
 					{
-						dbdeclare->signals = endsignal = newsignals;
+						dbdeclare->bodysignals = endsignal = newsignals;
 					} else
 					{
 						endsignal->next = newsignals;
